@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,22 +18,26 @@
 #define incl_HPHP_CLASS_SCOPE_H_
 
 #include "hphp/compiler/analysis/block_scope.h"
+#include "hphp/compiler/analysis/exceptions.h"
+#include "hphp/compiler/analysis/function_container.h"
+#include "hphp/compiler/expression/user_attribute.h"
+#include "hphp/compiler/json.h"
+#include "hphp/compiler/option.h"
+#include "hphp/compiler/statement/class_statement.h"
+#include "hphp/compiler/statement/method_statement.h"
+#include "hphp/compiler/statement/trait_alias_statement.h"
+#include "hphp/compiler/statement/trait_prec_statement.h"
+
+#include "hphp/util/functional.h"
+#include "hphp/util/hash-map-typedefs.h"
+#include "hphp/util/text-util.h"
+
 #include <list>
 #include <map>
 #include <set>
+#include <unordered_map>
 #include <utility>
 #include <vector>
-#include "hphp/compiler/analysis/function_container.h"
-#include "hphp/compiler/statement/class_statement.h"
-#include "hphp/compiler/statement/method_statement.h"
-#include "hphp/compiler/statement/trait_prec_statement.h"
-#include "hphp/compiler/statement/trait_alias_statement.h"
-#include "hphp/compiler/expression/user_attribute.h"
-#include "hphp/compiler/json.h"
-#include "hphp/util/functional.h"
-#include "hphp/util/hash-map-typedefs.h"
-#include "hphp/compiler/option.h"
-#include "hphp/compiler/analysis/exceptions.h"
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -59,12 +63,14 @@ class ClassScope : public BlockScope, public FunctionContainer,
                    public JSON::DocTarget::ISerializable {
 
 public:
-  enum KindOf {
-    KindOfObjectClass,
-    KindOfAbstractClass,
-    KindOfFinalClass,
-    KindOfInterface,
-    KindOfTrait
+  enum class KindOf : int {
+    ObjectClass,
+    AbstractClass,
+    FinalClass,
+    UtilClass,
+    Enum,
+    Interface,
+    Trait,
   };
 
 #define DECLARE_MAGIC(prefix, prev)                                     \
@@ -102,12 +108,9 @@ public:
     Final = 32
   };
 
-  enum JumpTableName {
-    JumpTableCallInfo
-  };
-
 public:
-  ClassScope(KindOf kindOf, const std::string &name,
+  ClassScope(FileScopeRawPtr fs,
+             KindOf kindOf, const std::string &originalName,
              const std::string &parent,
              const std::vector<std::string> &bases,
              const std::string &docComment, StatementPtr stmt,
@@ -117,10 +120,14 @@ public:
    * Special constructor for extension classes.
    */
   ClassScope(AnalysisResultPtr ar,
-             const std::string &name, const std::string &parent,
-             const std::vector<std::string> &bases,
-             const FunctionScopePtrVec &methods);
+             const std::string& originalName, const std::string& parent,
+             const std::vector<std::string>& bases,
+             const std::vector<FunctionScopePtr>& methods);
 
+  bool isNamed(const char* n) const;
+  bool isNamed(const std::string& n) const {
+    return isNamed(n.c_str());
+  }
   bool classNameCtor() const {
     return getAttribute(ClassNameConstructor);
   }
@@ -130,23 +137,20 @@ public:
   void checkDerivation(AnalysisResultPtr ar, hphp_string_iset &seen);
   const std::string &getOriginalParent() const { return m_parent; }
 
-  /**
-   * Returns topmost parent class that has the method.
-   */
-  ClassScopePtr getRootParent(AnalysisResultConstPtr ar,
-                              const std::string &methodName = "");
-  void getRootParents(AnalysisResultConstPtr ar, const std::string &methodName,
-                      ClassScopePtrVec &roots, ClassScopePtr curClass);
-
   /**
    * Whether this is a user-defined class.
    */
   bool isUserClass() const { return !getAttribute(System);}
   bool isExtensionClass() const { return getAttribute(Extension); }
-  bool isDynamic() const { return m_dynamic; }
   bool isBaseClass() const { return m_bases.empty(); }
   bool isBuiltin() const { return !getStmt(); }
 
+  /**
+   * Helpers for parsing class functions and variables.
+   */
+  ModifierExpressionPtr setModifiers(ModifierExpressionPtr modifiers);
+  ModifierExpressionPtr getModifiers() { return m_modifiers;}
+
   /**
    * Whether this class name was declared twice or more.
    */
@@ -154,14 +158,11 @@ public:
   bool isRedeclaring() const { return m_redeclaring >= 0;}
   int getRedeclaringId() { return m_redeclaring; }
 
-  void setStaticDynamic(AnalysisResultConstPtr ar);
-  void setDynamic(AnalysisResultConstPtr ar, const std::string &name);
-
   void addReferer(BlockScopePtr ref, int useKinds);
 
   /* For class_exists */
   void setVolatile();
-  bool isVolatile() const { return m_volatile;}
+  bool isVolatile() const { return m_volatile; }
   bool isPersistent() const { return m_persistent; }
   void setPersistent(bool p) { m_persistent = p; }
 
@@ -171,10 +172,6 @@ public:
     return m_derivesFromRedeclaring;
   }
 
-  bool derivedByDynamic() const {
-    return m_derivedByDynamic;
-  }
-
   /**
    * Get/set attributes.
    */
@@ -190,18 +187,7 @@ public:
     ClassScopePtr parent = getParentScope(ar);
     return parent && !parent->isRedeclaring() && parent->hasAttribute(attr, ar);
   }
-  void setKnownBase(int i) { assert(i < 32); m_knownBases |= 1u << i; }
-  bool hasUnknownBases() const {
-    int n = m_bases.size();
-    if (!n) return false;
-    if (n >= 32) n = 0;
-    return m_knownBases != (((1u << n) - 1) & 0xffffffff);
-  }
-  bool hasKnownBase(int i) const {
-    return m_knownBases & (1u << (i < 32 ? i : 31));
-  }
-
-  const FunctionScopePtrVec &getFunctionsVec() const {
+  const std::vector<FunctionScopePtr>& getFunctionsVec() const {
     return m_functionsVec;
   }
 
@@ -257,29 +243,9 @@ public:
   Symbol *findProperty(ClassScopePtr &cls, const std::string &name,
                        AnalysisResultConstPtr ar);
 
-  /**
-   * Caller is assumed to hold a lock on this scope
-   */
-  TypePtr checkProperty(BlockScopeRawPtr context,
-                        Symbol *sym, TypePtr type,
-                        bool coerce, AnalysisResultConstPtr ar);
-
-  /**
-   * Caller is *NOT* assumed to hold any locks. Context is
-   */
-  TypePtr checkConst(BlockScopeRawPtr context,
-                     const std::string &name, TypePtr type,
-                     bool coerce, AnalysisResultConstPtr ar,
-                     ConstructPtr construct,
-                     const std::vector<std::string> &bases,
-                     BlockScope *&defScope);
-
   /**
    * Collect parent class names.
    */
-  void getAllParents(AnalysisResultConstPtr ar,
-                     std::vector<std::string> &names);
-
   void getInterfaces(AnalysisResultConstPtr ar,
                      std::vector<std::string> &names,
                      bool recursive = true) const;
@@ -307,6 +273,10 @@ public:
     }
   }
 
+  int32_t getNumDeclMethods() const {
+    return m_numDeclMethods;
+  }
+
   const boost::container::flat_set<std::string>& getClassRequiredExtends()
     const {
     return m_requiredExtends;
@@ -338,16 +308,20 @@ public:
   void serialize(JSON::CodeError::OutputStream &out) const;
   void serialize(JSON::DocTarget::OutputStream &out) const;
 
-  bool isInterface() const { return m_kindOf == KindOfInterface; }
-  bool isFinal() const { return m_kindOf == KindOfFinalClass ||
-                                m_kindOf == KindOfTrait; }
-  bool isAbstract() const { return m_kindOf == KindOfAbstractClass ||
-                                   m_kindOf == KindOfTrait; }
-  bool isTrait() const { return m_kindOf == KindOfTrait; }
+  bool isInterface() const { return m_kindOf == KindOf::Interface; }
+  bool isFinal() const { return m_kindOf == KindOf::FinalClass ||
+                                m_kindOf == KindOf::Trait ||
+                                m_kindOf == KindOf::UtilClass ||
+                                m_kindOf == KindOf::Enum; }
+  bool isAbstract() const { return m_kindOf == KindOf::AbstractClass ||
+                                   m_kindOf == KindOf::Trait ||
+                                   m_kindOf == KindOf::UtilClass; }
+  bool isTrait() const { return m_kindOf == KindOf::Trait; }
+  bool isEnum() const { return m_kindOf == KindOf::Enum; }
+  bool isStaticUtil() const { return m_kindOf == KindOf::UtilClass; }
   bool hasProperty(const std::string &name) const;
   bool hasConst(const std::string &name) const;
 
-  static bool NeedStaticArray(ClassScopePtr cls, FunctionScopePtr func);
   void inheritedMagicMethods(ClassScopePtr super);
   void derivedMagicMethods(ClassScopePtr super);
   /* true if it might, false if it doesnt */
@@ -366,27 +340,9 @@ public:
    * Override function container
    */
   bool addFunction(AnalysisResultConstPtr ar,
+                   FileScopeRawPtr fileScope,
                    FunctionScopePtr funcScope);
 
-  void setNeedsCppCtor(bool needsCppCtor) {
-    m_needsCppCtor = needsCppCtor;
-  }
-
-  bool needsCppCtor() const {
-    return m_needsCppCtor;
-  }
-
-  void setNeedsInitMethod(bool needsInit) {
-    m_needsInit = needsInit;
-  }
-
-  bool needsInitMethod() const {
-    return m_needsInit;
-  }
-
-  bool canSkipCreateMethod(AnalysisResultConstPtr ar) const;
-  bool checkHasPropTable(AnalysisResultConstPtr ar);
-
   const StringData* getFatalMessage() const {
     return m_fatal_error_msg;
   }
@@ -397,44 +353,172 @@ public:
     assert(m_fatal_error_msg != nullptr);
   }
 
+private:
+  void informClosuresAboutScopeClone(ConstructPtr root,
+                                     FunctionScopePtr outerScope,
+                                     AnalysisResultPtr ar);
+
+
+  /////////////////////////////////////////////////////////////////////////////
+  // Trait flattening.
+
+public:
+  struct TraitMethod {
+    TraitMethod(ClassScopePtr trait_,
+                MethodStatementPtr method_,
+                ModifierExpressionPtr modifiers_,
+                StatementPtr ruleStmt_)
+      : trait(trait_)
+      , method(method_)
+      , originalName(method_->getOriginalName())
+      , modifiers(modifiers_ ? modifiers_ : method_->getModifiers())
+      , ruleStmt(ruleStmt_)
+    {}
+
+    TraitMethod(ClassScopePtr trait_,
+                MethodStatementPtr method_,
+                ModifierExpressionPtr modifiers_,
+                StatementPtr ruleStmt_,
+                const std::string& originalName_)
+      : trait(trait_)
+      , method(method_)
+      , originalName(originalName_)
+      , modifiers(modifiers_ ? modifiers_ : method_->getModifiers())
+      , ruleStmt(ruleStmt_)
+    {}
+
+    using class_type = ClassScopePtr;
+    using method_type = MethodStatementPtr;
+    using modifiers_type = ModifierExpressionPtr;
+
+    const ClassScopePtr      trait;
+    const MethodStatementPtr method;
+    const std::string        originalName;
+    ModifierExpressionPtr    modifiers;
+    const StatementPtr       ruleStmt; // for methods imported via aliasing
+  };
+
+private:
+  struct TMIOps {
+    using prec_type  = TraitPrecStatementPtr;
+    using alias_type = TraitAliasStatementPtr;
+
+    static bool strEmpty(const std::string& str)    { return str.empty(); }
+    static std::string clsName(ClassScopePtr cls)   {
+      return cls->getOriginalName();
+    }
+
+    static bool isTrait(ClassScopePtr cls)          { return cls->isTrait(); }
+    static bool isAbstract(ModifierExpressionPtr m) { return m->isAbstract(); }
+
+    static bool exclude(const std::string&) { return false; }
+
+    static TraitMethod traitMethod(ClassScopePtr traitCls,
+                                   MethodStatementPtr methStmt,
+                                   alias_type stmt) {
+      return TraitMethod(traitCls, methStmt, stmt->getModifiers(),
+                         stmt, stmt->getNewMethodName());
+    }
+
+    static std::string precMethodName(prec_type stmt) {
+      return stmt->getMethodName();
+    }
+    static std::string precSelectedTraitName(prec_type stmt) {
+      return stmt->getTraitName();
+    }
+    static hphp_string_iset precOtherTraitNames(prec_type stmt) {
+      hphp_string_iset otherTraitNames;
+      stmt->getOtherTraitNames(otherTraitNames);
+      return otherTraitNames;
+    }
+
+    static std::string aliasTraitName(alias_type stmt) {
+      return stmt->getTraitName();
+    }
+    static std::string aliasOrigMethodName(alias_type stmt) {
+      return stmt->getMethodName();
+    }
+    static std::string aliasNewMethodName(alias_type stmt) {
+      return stmt->getNewMethodName();
+    }
+    static ModifierExpressionPtr aliasModifiers(alias_type stmt) {
+      return stmt->getModifiers();
+    }
+
+    static void addTraitAlias(ClassScope* cs, alias_type stmt,
+                              ClassScopePtr traitCls);
+
+    static ClassScopePtr findSingleTraitWithMethod(ClassScope* cs,
+                                              const std::string& origMethName);
+    static ClassScopePtr findTraitClass(ClassScope* cs,
+                                        const std::string& traitName);
+    static MethodStatementPtr findTraitMethod(ClassScope* cs,
+                                              ClassScopePtr traitCls,
+                                              const std::string& origMethName);
+
+    static void errorUnknownMethod(prec_type stmt) {
+      Compiler::Error(Compiler::UnknownObjectMethod, stmt);
+    }
+    static void errorUnknownMethod(alias_type stmt,
+                                   const std::string& methName) {
+      stmt->analysisTimeFatal(Compiler::UnknownTraitMethod,
+                              Strings::TRAITS_UNKNOWN_TRAIT_METHOD,
+                              methName.c_str());
+    }
+    template <class Stmt>
+    static void errorUnknownTrait(Stmt stmt,
+                                  const std::string& traitName) {
+      stmt->analysisTimeFatal(Compiler::UnknownTrait,
+                              Strings::TRAITS_UNKNOWN_TRAIT,
+                              traitName.c_str());
+    }
+    static void errorDuplicateMethod(const ClassScope* cs,
+                                     const std::string& methName) {
+      cs->getStmt()->analysisTimeFatal(
+        Compiler::MethodInMultipleTraits,
+        Strings::METHOD_IN_MULTIPLE_TRAITS,
+        methName.c_str()
+      );
+    }
+  };
+
+  friend class TMIOps;
+
+public:
+  using TMIData = TraitMethodImportData<TraitMethod, TMIOps,
+                                        std::string,
+                                        string_hashi, string_eqstri>;
+
+private:
+  MethodStatementPtr importTraitMethod(const TraitMethod& traitMethod,
+                                       AnalysisResultPtr ar,
+                                       std::string methName);
+  void applyTraitRules(TMIData& tmid);
+
+  bool hasMethod(const std::string &methodName) const;
+  bool usesTrait(const std::string &traitName) const;
+
+  void importTraitProperties(AnalysisResultPtr ar);
+  void importClassRequirements(AnalysisResultPtr ar, ClassScopePtr trait);
+
+
+  /////////////////////////////////////////////////////////////////////////////
+  // Data members.
+
 private:
   // need to maintain declaration order for ClassInfo map
-  FunctionScopePtrVec m_functionsVec;
+  std::vector<FunctionScopePtr> m_functionsVec;
 
   std::string m_parent;
   mutable std::vector<std::string> m_bases;
   UserAttributeMap m_userAttributes;
+  ModifierExpressionPtr m_modifiers;
 
   std::vector<std::string> m_usedTraitNames;
   boost::container::flat_set<std::string> m_requiredExtends;
   boost::container::flat_set<std::string> m_requiredImplements;
   // m_traitAliases is used to support ReflectionClass::getTraitAliases
-  std::vector<std::pair<std::string, std::string> > m_traitAliases;
-
-  struct TraitMethod {
-    const ClassScopePtr      m_trait;
-    const MethodStatementPtr m_method;
-    const std::string        m_originalName;
-    ModifierExpressionPtr    m_modifiers;
-    const StatementPtr       m_ruleStmt; // for methods imported via aliasing
-    TraitMethod(ClassScopePtr trait, MethodStatementPtr method,
-                ModifierExpressionPtr modifiers, StatementPtr ruleStmt) :
-        m_trait(trait), m_method(method),
-        m_originalName(method->getOriginalName()), m_modifiers(modifiers),
-        m_ruleStmt(ruleStmt) {
-    }
-    TraitMethod(ClassScopePtr trait, MethodStatementPtr method,
-                ModifierExpressionPtr modifiers, StatementPtr ruleStmt,
-                const std::string &originalName) :
-        m_trait(trait), m_method(method), m_originalName(originalName),
-        m_modifiers(modifiers), m_ruleStmt(ruleStmt) {
-    }
-  };
-
-  typedef std::list<TraitMethod> TraitMethodList;
-  typedef std::map<std::string, TraitMethodList> MethodToTraitListMap;
-  MethodToTraitListMap m_importMethToTraitMap;
-  typedef std::map<std::string, MethodStatementPtr> ImportedMethodMap;
+  std::vector<std::pair<std::string, std::string>> m_traitAliases;
 
   mutable int m_attribute;
   int m_redeclaring; // multiple definition of the same class
@@ -445,63 +529,12 @@ private:
     BEING_FLATTENED,
     FLATTENED
   } m_traitStatus;
-  unsigned m_dynamic:1;
   unsigned m_volatile:1; // for class_exists
   unsigned m_persistent:1;
-  unsigned m_derivedByDynamic:1;
-  unsigned m_needsCppCtor:1;
-  unsigned m_needsInit:1;
-  // m_knownBases has a bit for each base class saying whether
-  // its known to exist at the point of definition of this class.
-  // for classes with more than 31 bases, bit 31 is set iff
-  // bases 32 through n are all known.
-  unsigned m_knownBases;
+  int32_t m_numDeclMethods{-1};
 
   // holds the fact that accessing this class declaration is a fatal error
   const StringData* m_fatal_error_msg = nullptr;
-
-  void addImportTraitMethod(const TraitMethod &traitMethod,
-                            const std::string &methName);
-  void informClosuresAboutScopeClone(ConstructPtr root,
-                                     FunctionScopePtr outerScope,
-                                     AnalysisResultPtr ar);
-
-  void setImportTraitMethodModifiers(const std::string &methName,
-                                     ClassScopePtr traitCls,
-                                     ModifierExpressionPtr modifiers);
-
-  MethodStatementPtr importTraitMethod(const TraitMethod&  traitMethod,
-                                       AnalysisResultPtr   ar,
-                                       std::string         methName,
-                                       const ImportedMethodMap &
-                                       importedTraitMethods);
-
-  void importTraitProperties(AnalysisResultPtr ar);
-
-  void findTraitMethodsToImport(AnalysisResultPtr ar, ClassScopePtr trait);
-
-  void importClassRequirements(AnalysisResultPtr ar, ClassScopePtr trait);
-
-  MethodStatementPtr findTraitMethod(AnalysisResultPtr ar,
-                                     ClassScopePtr trait,
-                                     const std::string &methodName,
-                                     std::set<ClassScopePtr> &visitedTraits);
-
-  void applyTraitRules(AnalysisResultPtr ar);
-
-  void applyTraitPrecRule(TraitPrecStatementPtr stmt);
-
-  void applyTraitAliasRule(AnalysisResultPtr ar, TraitAliasStatementPtr stmt);
-
-  ClassScopePtr findSingleTraitWithMethod(AnalysisResultPtr ar,
-                                          const std::string &methodName) const;
-
-  void removeSpareTraitAbstractMethods(AnalysisResultPtr ar);
-
-  bool usesTrait(const std::string &traitName) const;
-
-  bool hasMethod(const std::string &methodName) const;
-
 };
 
 ///////////////////////////////////////////////////////////////////////////////
