@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -21,15 +21,23 @@
 #include "hphp/compiler/statement/statement.h"
 #include "hphp/compiler/statement/use_trait_statement.h"
 #include "hphp/compiler/statement/class_require_statement.h"
+#include "hphp/compiler/statement/class_statement.h"
 #include "hphp/compiler/statement/trait_prec_statement.h"
 #include "hphp/compiler/statement/trait_alias_statement.h"
 #include "hphp/compiler/statement/typedef_statement.h"
+#include "hphp/compiler/expression/binary_op_expression.h"
+#include "hphp/compiler/expression/object_property_expression.h"
+#include "hphp/parser/parser.h"
 
+#include "hphp/runtime/base/header-kind.h"
 #include "hphp/runtime/vm/func.h"
 #include "hphp/runtime/vm/func-emitter.h"
 #include "hphp/runtime/vm/unit.h"
+#include "hphp/runtime/vm/unit-emitter.h"
 #include "hphp/util/hash.h"
 
+#include <folly/Optional.h>
+
 #include <deque>
 #include <utility>
 #include <map>
@@ -60,15 +68,19 @@ namespace Compiler {
 class Label;
 class EmitterVisitor;
 
+using OptLocation = folly::Optional<Location::Range>;
+
 class Emitter {
 public:
   Emitter(ConstructPtr node, UnitEmitter& ue, EmitterVisitor& ev)
-    : m_node(node), m_ue(ue), m_ev(ev) {}
+      : m_node(node), m_ue(ue), m_ev(ev) {}
   UnitEmitter& getUnitEmitter() { return m_ue; }
   ConstructPtr getNode() { return m_node; }
   EmitterVisitor& getEmitterVisitor() { return m_ev; }
-  void setTempLocation(LocationPtr loc) { m_tempLoc = loc; }
-  LocationPtr getTempLocation() { return m_tempLoc; }
+  void setTempLocation(const OptLocation& r) {
+    m_tempLoc = r;
+  }
+  const OptLocation& getTempLocation() { return m_tempLoc; }
   void incStat(int counter, int value) {
     if (RuntimeOption::EnableEmitterStats) {
       IncStat(counter, value);
@@ -139,7 +151,7 @@ private:
   ConstructPtr m_node;
   UnitEmitter& m_ue;
   EmitterVisitor& m_ev;
-  LocationPtr m_tempLoc;
+  OptLocation m_tempLoc;
 };
 
 struct SymbolicStack {
@@ -153,12 +165,6 @@ struct SymbolicStack {
     CLS_PARENT
   };
 
-  enum MetaType {
-    META_NONE,
-    META_LITSTR,
-    META_DATA_TYPE
-  };
-
 private:
   /**
    * Symbolic stack (m_symStack)
@@ -178,24 +184,14 @@ private:
   struct SymEntry {
     explicit SymEntry(char s = 0)
       : sym(s)
-      , metaType(META_NONE)
-      , notRef(false)
-      , notNull(false)
-      , dtPredicted(false)
+      , name(nullptr)
       , className(nullptr)
       , intval(-1)
       , unnamedLocalStart(InvalidAbsoluteOffset)
       , clsBaseType(CLS_INVALID)
     {}
     char sym;
-    MetaType metaType;
-    bool notRef:1;
-    bool notNull:1;
-    bool dtPredicted:1;
-    union {
-      const StringData* name;   // META_LITSTR
-      DataType dt;              // META_DATA_TYPE
-    }   metaData;
+    const StringData* name;
     const StringData* className;
     int64_t intval; // used for L and I symbolic flavors
 
@@ -209,6 +205,8 @@ private:
     // early.  How this works depends on the type of class base---see
     // emitResolveClsBase for details.
     ClassBaseType clsBaseType;
+
+    std::string pretty() const;
   };
   std::vector<SymEntry> m_symStack;
 
@@ -237,11 +235,7 @@ public:
   void setInt(int64_t v);
   void setString(const StringData* s);
   void setKnownCls(const StringData* s, bool nonNull);
-  void setNotRef();
-  bool getNotRef() const;
-  void setKnownType(DataType dt, bool predicted = false);
   void cleanTopMeta();
-  DataType getKnownType(int index = -1, bool noRef = true) const;
   void setClsBaseType(ClassBaseType);
   void setUnnamedLocal(int index, int localId, Offset startOffset);
   void pop();
@@ -252,7 +246,8 @@ public:
   bool isCls(int index) const;
   bool isTypePredicted(int index = -1 /* stack top */) const;
   void set(int index, char sym);
-  unsigned size() const;
+  size_t size() const;
+  size_t actualSize() const;
   bool empty() const;
   void clear();
 
@@ -440,6 +435,7 @@ class EmitterVisitor {
   friend class FuncFinisher;
 public:
   typedef std::vector<int> IndexChain;
+  typedef std::pair<ExpressionPtr, IndexChain> IndexPair;
   typedef Emitter::IterPair IterPair;
   typedef std::vector<IterPair> IterVec;
 
@@ -447,7 +443,6 @@ public:
   ~EmitterVisitor();
 
   bool visit(ConstructPtr c);
-  bool visitImpl(ConstructPtr c);
   void visitKids(ConstructPtr c);
   void visit(FileScopePtr file);
   void assignLocalVariableIds(FunctionScopePtr fs);
@@ -457,9 +452,9 @@ public:
 
   void listAssignmentVisitLHS(Emitter& e, ExpressionPtr exp,
                               IndexChain& indexChain,
-                              std::vector<IndexChain*>& chainList);
+                              std::vector<IndexPair>& chainList);
   void listAssignmentAssignElements(Emitter& e,
-                                    std::vector<IndexChain*>& indexChains,
+                                    std::vector<IndexPair>& indexChains,
                                     std::function<void()> emitSrc);
 
   void visitIfCondition(ExpressionPtr cond, Emitter& e, Label& tru, Label& fals,
@@ -504,15 +499,13 @@ public:
     return m_retLocal;
   }
 
-  class IncludeTimeFatalException : public Exception {
-  public:
+  struct IncludeTimeFatalException : Exception {
     ConstructPtr m_node;
     bool m_parseFatal;
     IncludeTimeFatalException(ConstructPtr node, const char* fmt, ...)
         : Exception(), m_node(node), m_parseFatal(false) {
       va_list ap; va_start(ap, fmt); format(fmt, ap); va_end(ap);
     }
-    virtual ~IncludeTimeFatalException() throw() {}
     EXCEPTION_COMMON_IMPL(IncludeTimeFatalException);
     void setParseFatal(bool b = true) { m_parseFatal = b; }
   };
@@ -628,7 +621,8 @@ private:
   };
 
 private:
-  static const size_t kMinStringSwitchCases = 8;
+  static constexpr size_t kMinStringSwitchCases = 8;
+
   UnitEmitter& m_ue;
   FuncEmitter* m_curFunc;
   FileScopePtr m_file;
@@ -657,8 +651,9 @@ private:
   std::deque<FaultRegion*> m_faultRegions;
   std::deque<FPIRegion*> m_fpiRegions;
   std::vector<Array> m_staticArrays;
+  std::vector<folly::Optional<CollectionType>> m_staticColType;
   std::set<std::string,stdltistr> m_hoistables;
-  LocationPtr m_tempLoc;
+  OptLocation m_tempLoc;
   std::unordered_set<std::string> m_staticEmitted;
 
   // The stack of all Regions that this EmitterVisitor is currently inside
@@ -675,21 +670,34 @@ public:
   void unexpectedStackSym(char sym, const char* where) const;
 
   int scanStackForLocation(int iLast);
+
   void buildVectorImm(std::vector<unsigned char>& vectorImm,
                       int iFirst, int iLast, bool allowW,
                       Emitter& e);
+  /*
+   * Emit bytecodes for the base and intermediate dims, returning the number of
+   * eval stack slots containing member keys that should be consumed by the
+   * final operation.
+   */
+  size_t emitMOp(int iFirst, int& iLast, bool allowW, bool rhsVal,
+                 Emitter& e, MOpFlags flags);
+  void emitQueryMOp(int iFirst, int iLast, Emitter& e, QueryMOp op);
+
   enum class PassByRefKind {
     AllowCell,
     WarnOnCell,
     ErrorOnCell,
   };
   PassByRefKind getPassByRefKind(ExpressionPtr exp);
+  void emitCall(Emitter& e, FunctionCallPtr func,
+                ExpressionListPtr params, Offset fpiStart);
   void emitAGet(Emitter& e);
   void emitCGetL2(Emitter& e);
   void emitCGetL3(Emitter& e);
   void emitPushL(Emitter& e);
   void emitCGet(Emitter& e);
-  void emitVGet(Emitter& e);
+  void emitCGetQuiet(Emitter& e);
+  bool emitVGet(Emitter& e, bool skipCells = false);
   void emitIsset(Emitter& e);
   void emitIsType(Emitter& e, IsTypeOp op);
   void emitEmpty(Emitter& e);
@@ -706,24 +714,30 @@ public:
   void emitConvertSecondToCell(Emitter& e);
   void emitConvertToVar(Emitter& e);
   void emitFPass(Emitter& e, int paramID, PassByRefKind passByRefKind);
-  void emitVirtualLocal(int localId, DataType dt = KindOfUnknown);
+  void emitVirtualLocal(int localId);
   template<class Expr> void emitVirtualClassBase(Emitter&, Expr* node);
   void emitResolveClsBase(Emitter& e, int pos);
   void emitClsIfSPropBase(Emitter& e);
   Id emitVisitAndSetUnnamedL(Emitter& e, ExpressionPtr exp);
   Id emitSetUnnamedL(Emitter& e);
   void emitPushAndFreeUnnamedL(Emitter& e, Id tempLocal, Offset start);
-  DataType analyzeSwitch(SwitchStatementPtr s, SwitchState& state);
+  MaybeDataType analyzeSwitch(SwitchStatementPtr s, SwitchState& state);
   void emitIntegerSwitch(Emitter& e, SwitchStatementPtr s,
                          std::vector<Label>& caseLabels, Label& done,
                          const SwitchState& state);
   void emitStringSwitch(Emitter& e, SwitchStatementPtr s,
                         std::vector<Label>& caseLabels, Label& done,
                         const SwitchState& state);
-
+  void emitArrayInit(Emitter& e, ExpressionListPtr el,
+                     folly::Optional<CollectionType> ct = folly::none);
+  void emitPairInit(Emitter&e, ExpressionListPtr el);
+  void emitVectorInit(Emitter&e, CollectionType ct, ExpressionListPtr el);
+  void emitMapInit(Emitter&e, CollectionType ct, ExpressionListPtr el);
+  void emitSetInit(Emitter&e, CollectionType ct, ExpressionListPtr el);
+  void emitCollectionInit(Emitter& e, BinaryOpExpressionPtr exp);
   void markElem(Emitter& e);
   void markNewElem(Emitter& e);
-  void markProp(Emitter& e);
+  void markProp(Emitter& e, PropAccessType propAccessType);
   void markSProp(Emitter& e);
   void markName(Emitter& e);
   void markNameSecond(Emitter& e);
@@ -743,6 +757,10 @@ public:
                           FuncEmitter *fe,
                           bool &allowOverride);
   void bindNativeFunc(MethodStatementPtr meth, FuncEmitter *fe);
+  int32_t emitNativeOpCodeImpl(MethodStatementPtr meth,
+                               const char* funcName,
+                               const char* className,
+                               FuncEmitter* fe);
   void emitMethodMetadata(MethodStatementPtr meth,
                           ClosureUseVarVec* useVars,
                           bool top);
@@ -750,10 +768,15 @@ public:
                              ExpressionListPtr params,
                              bool coerce_params = false);
   void emitMethodPrologue(Emitter& e, MethodStatementPtr meth);
+  void emitDeprecationWarning(Emitter& e, MethodStatementPtr meth);
   void emitMethod(MethodStatementPtr meth);
-  void emitConstMethodCallNoParams(Emitter& e, string name);
-  void emitCreateStaticWaitHandle(Emitter& e, std::string cls,
-                                  std::function<void()> emitParam);
+  void emitMemoizeProp(Emitter& e, MethodStatementPtr meth, Id localID,
+                       const std::vector<Id>& paramIDs, uint32_t numParams);
+  void addMemoizeProp(MethodStatementPtr meth);
+  void emitMemoizeMethod(MethodStatementPtr meth, const StringData* methName);
+  void emitConstMethodCallNoParams(Emitter& e, const std::string& name);
+  bool emitInlineGenva(Emitter& e, const ExpressionPtr);
+  bool emitHHInvariant(Emitter& e, SimpleFunctionCallPtr);
   void emitMethodDVInitializers(Emitter& e,
                                 MethodStatementPtr& meth,
                                 Label& topOfBody);
@@ -781,12 +804,15 @@ public:
   void emitFuncCall(Emitter& e, FunctionCallPtr node,
                     const char* nameOverride = nullptr,
                     ExpressionListPtr paramsOverride = nullptr);
-  void emitFuncCallArg(Emitter& e, ExpressionPtr exp, int paramId);
-  void emitBuiltinCallArg(Emitter& e, ExpressionPtr exp, int paramId,
-                         bool byRef);
-  void emitBuiltinDefaultArg(Emitter& e, Variant& v, DataType t, int paramId);
+  void emitFuncCallArg(Emitter& e, ExpressionPtr exp, int paramId,
+                       bool isUnpack);
+  bool emitBuiltinCallArg(Emitter& e, ExpressionPtr exp, int paramId,
+                          bool byRef, bool mustBeRef);
+  void emitLambdaCaptureArg(Emitter& e, ExpressionPtr exp);
+  void emitBuiltinDefaultArg(Emitter& e, Variant& v,
+                             MaybeDataType t, int paramId);
   void emitClass(Emitter& e, ClassScopePtr cNode, bool topLevel);
-  void emitTypedef(Emitter& e, TypedefStatementPtr);
+  Id emitTypedef(Emitter& e, TypedefStatementPtr);
   void emitForeachListAssignment(Emitter& e,
                                  ListAssignmentPtr la,
                                  std::function<void()> emitSrc);
@@ -805,11 +831,13 @@ public:
   // These methods handle the return, break, continue, and goto operations.
   // These methods are aware of try/finally blocks and foreach blocks and
   // will free iterators and jump to finally epilogues as appropriate.
-  void emitReturn(Emitter& e, char sym, bool hasConstraint, StatementPtr s);
+  void emitReturn(Emitter& e, char sym, StatementPtr s);
   void emitBreak(Emitter& e, int depth, StatementPtr s);
   void emitContinue(Emitter& e, int depth, StatementPtr s);
   void emitGoto(Emitter& e, StringData* name, StatementPtr s);
 
+  void emitYieldFrom(Emitter& e, ExpressionPtr exp);
+
   // Helper methods for emitting IterFree instructions
   void emitIterFree(Emitter& e, IterVec& iters);
   void emitIterFreeForReturn(Emitter& e);
@@ -829,6 +857,10 @@ public:
   void emitGotoTrampoline(Emitter& e, Region* entry,
                           std::vector<Label*>& cases, StringData* name);
 
+  // Returns true if VerifyRetType should be emitted before Ret for
+  // the current function.
+  bool shouldEmitVerifyRetType();
+
   Funclet* addFunclet(Thunklet* body);
   Funclet* addFunclet(StatementPtr stmt,
                       Thunklet* body);
@@ -864,10 +896,10 @@ public:
   void newFPIRegion(Offset start, Offset end, Offset fpOff);
   void copyOverCatchAndFaultRegions(FuncEmitter* fe);
   void copyOverFPIRegions(FuncEmitter* fe);
-  void saveMaxStackCells(FuncEmitter* fe);
-  void finishFunc(Emitter& e, FuncEmitter* fe);
-
-  void initScalar(TypedValue& tvVal, ExpressionPtr val);
+  void saveMaxStackCells(FuncEmitter* fe, int32_t stackPad);
+  void finishFunc(Emitter& e, FuncEmitter* fe, int32_t stackPad);
+  void initScalar(TypedValue& tvVal, ExpressionPtr val,
+                  folly::Optional<CollectionType> ct = folly::none);
   bool requiresDeepInit(ExpressionPtr initExpr) const;
 
   void emitClassTraitPrecRule(PreClassEmitter* pce, TraitPrecStatementPtr rule);
@@ -901,12 +933,10 @@ public:
                                 StringData* name, bool alloc);
 };
 
-void emitAllHHBC(AnalysisResultPtr ar);
+void emitAllHHBC(AnalysisResultPtr&& ar);
 
 
 extern "C" {
-  StringData* hphp_compiler_serialize_code_model_for(String code,
-                                                     String prefix);
   Unit* hphp_compiler_parse(const char* code, int codeLen, const MD5& md5,
                             const char* filename);
   Unit* hphp_build_native_func_unit(const HhbcExtFuncInfo* builtinFuncs,
