@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,17 +17,14 @@
 #include "hphp/compiler/compiler.h"
 #include "hphp/compiler/package.h"
 #include "hphp/compiler/analysis/analysis_result.h"
-#include "hphp/compiler/analysis/alias_manager.h"
 #include "hphp/compiler/analysis/code_error.h"
 #include "hphp/compiler/analysis/emitter.h"
-#include "hphp/compiler/analysis/type.h"
 #include "hphp/compiler/analysis/symbol_table.h"
 #include "hphp/compiler/option.h"
 #include "hphp/compiler/parser/parser.h"
 #include "hphp/compiler/builtin_symbols.h"
 #include "hphp/compiler/json.h"
 #include "hphp/util/logger.h"
-#include "hphp/util/db-conn.h"
 #include "hphp/util/exception.h"
 #include "hphp/util/process.h"
 #include "hphp/util/text-util.h"
@@ -38,23 +35,27 @@
 #include "hphp/runtime/base/file-util.h"
 #include "hphp/runtime/base/program-functions.h"
 #include "hphp/runtime/base/externals.h"
-#include "hphp/runtime/base/thread-init-fini.h"
 #include "hphp/runtime/base/config.h"
+#include "hphp/runtime/base/ini-setting.h"
 #include "hphp/runtime/vm/repo.h"
-#include "hphp/system/constants.h"
 #include "hphp/system/systemlib.h"
 #include "hphp/util/repo-schema.h"
 
 #include "hphp/hhvm/process-init.h"
 
 #include <sys/types.h>
+#ifndef _MSC_VER
 #include <sys/wait.h>
 #include <dlfcn.h>
+#endif
 
+#include <boost/algorithm/string/replace.hpp>
 #include <boost/program_options/options_description.hpp>
 #include <boost/program_options/positional_options.hpp>
 #include <boost/program_options/variables_map.hpp>
 #include <boost/program_options/parsers.hpp>
+#include <boost/filesystem.hpp>
+
 #include <exception>
 
 using namespace boost::program_options;
@@ -64,46 +65,44 @@ namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
 
 struct CompilerOptions {
-  string target;
-  string format;
-  string outputDir;
-  string syncDir;
-  vector<string> config;
-  string configDir;
-  vector<string> confStrings;
-  string inputDir;
-  vector<string> inputs;
-  string inputList;
-  vector<string> includePaths;
-  vector<string> modules;
-  vector<string> excludeDirs;
-  vector<string> excludeFiles;
-  vector<string> excludePatterns;
-  vector<string> excludeStaticDirs;
-  vector<string> excludeStaticFiles;
-  vector<string> excludeStaticPatterns;
-  vector<string> fmodules;
-  vector<string> ffiles;
-  vector<string> cfiles;
-  vector<string> cmodules;
+  std::string target;
+  std::string format;
+  std::string outputDir;
+  std::string syncDir;
+  std::vector<std::string> config;
+  std::string configDir;
+  std::vector<std::string> confStrings;
+  std::vector<std::string> iniStrings;
+  std::string inputDir;
+  std::vector<std::string> inputs;
+  std::string inputList;
+  std::vector<std::string> includePaths;
+  std::vector<std::string> modules;
+  std::vector<std::string> excludeDirs;
+  std::vector<std::string> excludeFiles;
+  std::vector<std::string> excludePatterns;
+  std::vector<std::string> excludeStaticDirs;
+  std::vector<std::string> excludeStaticFiles;
+  std::vector<std::string> excludeStaticPatterns;
+  std::vector<std::string> fmodules;
+  std::vector<std::string> ffiles;
+  std::vector<std::string> cfiles;
+  std::vector<std::string> cmodules;
   bool parseOnDemand;
-  string program;
-  string programArgs;
-  string branch;
+  std::string program;
+  std::string programArgs;
+  std::string branch;
   int revision;
   bool genStats;
   bool keepTempDir;
-  string dbStats;
   bool noTypeInference;
   int logLevel;
   bool force;
   int optimizeLevel;
-  string filecache;
+  std::string filecache;
   bool dump;
-  string docjson;
   bool coredump;
   bool nofork;
-  string optimizations;
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -137,25 +136,20 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv);
 void createOutputDirectory(CompilerOptions &po);
 int process(const CompilerOptions &po);
 int lintTarget(const CompilerOptions &po);
-int analyzeTarget(const CompilerOptions &po, AnalysisResultPtr ar);
 int phpTarget(const CompilerOptions &po, AnalysisResultPtr ar);
 void hhbcTargetInit(const CompilerOptions &po, AnalysisResultPtr ar);
-int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr ar,
+int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr&& ar,
                AsyncFileCacheSaver &fcThread);
-int runTargetCheck(const CompilerOptions &po, AnalysisResultPtr ar,
+int runTargetCheck(const CompilerOptions &po, AnalysisResultPtr&& ar,
                    AsyncFileCacheSaver &fcThread);
 int runTarget(const CompilerOptions &po);
+void pcre_init();
 
 ///////////////////////////////////////////////////////////////////////////////
 
-extern "C" void compiler_hook_initialize();
-
 int compiler_main(int argc, char **argv) {
   try {
     CompilerOptions po;
-#ifdef FACEBOOK
-    compiler_hook_initialize();
-#endif
 
     int ret = prepareOptions(po, argc, argv);
     if (ret == 1) return 0; // --help
@@ -164,6 +158,7 @@ int compiler_main(int argc, char **argv) {
     Timer totalTimer(Timer::WallTime, "running hphp");
     createOutputDirectory(po);
     if (ret == 0) {
+#ifndef _MSC_VER
       if (!po.nofork && !Process::IsUnderGDB()) {
         int pid = fork();
         if (pid == 0) {
@@ -172,7 +167,9 @@ int compiler_main(int argc, char **argv) {
         }
         wait(&ret);
         ret = WIFEXITED(ret) ? WEXITSTATUS(ret) : -1;
-      } else {
+      } else
+#endif
+      {
         ret = process(po);
       }
     }
@@ -210,67 +207,67 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
   desc.add_options()
     ("help", "display this message")
     ("version", "display version number")
-    ("target,t", value<string>(&po.target)->default_value("run"),
+    ("target,t", value<std::string>(&po.target)->default_value("run"),
      "lint | "
-     "analyze | "
      "php | "
      "hhbc | "
      "filecache | "
      "run (default)")
-    ("format,f", value<string>(&po.format),
+    ("format,f", value<std::string>(&po.format),
      "lint: (none); \n"
-     "analyze: (none); \n"
-     "php: trimmed (default) | inlined | pickled | typeinfo |"
+     "php: trimmed (default) | inlined | pickled |"
      " <any combination of them by any separator>; \n"
      "hhbc: binary (default) | text; \n"
      "run: cluster (default) | file")
-    ("input-dir", value<string>(&po.inputDir), "input directory")
-    ("program", value<string>(&po.program)->default_value("program"),
+    ("input-dir", value<std::string>(&po.inputDir), "input directory")
+    ("program", value<std::string>(&po.program)->default_value("program"),
      "final program name to use")
-    ("args", value<string>(&po.programArgs), "program arguments")
-    ("inputs,i", value<vector<string> >(&po.inputs), "input file names")
-    ("input-list", value<string>(&po.inputList),
+    ("args", value<std::string>(&po.programArgs), "program arguments")
+    ("inputs,i", value<std::vector<std::string>>(&po.inputs), "input file names")
+    ("input-list", value<std::string>(&po.inputList),
      "file containing list of file names, one per line")
     ("include-path",
-     value<vector<string> >(&po.includePaths)->composing(),
+     value<std::vector<std::string>>(&po.includePaths)->composing(),
      "a list of full paths to search for files being included in includes "
      "or requires but cannot be found assuming relative paths")
-    ("module", value<vector<string> >(&po.modules)->composing(),
+    ("module", value<std::vector<std::string>>(&po.modules)->composing(),
      "directories containing all input files")
-    ("exclude-dir", value<vector<string> >(&po.excludeDirs)->composing(),
+    ("exclude-dir",
+     value<std::vector<std::string>>(&po.excludeDirs)->composing(),
      "directories to exclude from the input")
-    ("fmodule", value<vector<string> >(&po.fmodules)->composing(),
+    ("fmodule", value<std::vector<std::string>>(&po.fmodules)->composing(),
      "same with module, except no exclusion checking is performed, so these "
      "modules are forced to be included")
-    ("ffile", value<vector<string> >(&po.ffiles)->composing(),
+    ("ffile", value<std::vector<std::string>>(&po.ffiles)->composing(),
      "extra PHP files forced to include without exclusion checking")
-    ("exclude-file", value<vector<string> >(&po.excludeFiles)->composing(),
+    ("exclude-file",
+     value<std::vector<std::string>>(&po.excludeFiles)->composing(),
      "files to exclude from the input, even if parse-on-demand finds it")
     ("exclude-pattern",
-     value<vector<string> >(&po.excludePatterns)->composing(),
+     value<std::vector<std::string>>(&po.excludePatterns)->composing(),
      "regex (in 'find' command's regex command line option format) of files "
      "or directories to exclude from the input, even if parse-on-demand finds "
      "it")
     ("exclude-static-pattern",
-     value<vector<string> >(&po.excludeStaticPatterns)->composing(),
+     value<std::vector<std::string>>(&po.excludeStaticPatterns)->composing(),
      "regex (in 'find' command's regex command line option format) of files "
      "or directories to exclude from static content cache")
     ("exclude-static-dir",
-     value<vector<string> >(&po.excludeStaticDirs)->composing(),
+     value<std::vector<std::string>>(&po.excludeStaticDirs)->composing(),
      "directories to exclude from static content cache")
     ("exclude-static-file",
-     value<vector<string> >(&po.excludeStaticFiles)->composing(),
+     value<std::vector<std::string>>(&po.excludeStaticFiles)->composing(),
      "files to exclude from static content cache")
-    ("cfile", value<vector<string> >(&po.cfiles)->composing(),
+    ("cfile", value<std::vector<std::string>>(&po.cfiles)->composing(),
      "extra static files forced to include without exclusion checking")
-    ("cmodule", value<vector<string> >(&po.cmodules)->composing(),
+    ("cmodule", value<std::vector<std::string>>(&po.cmodules)->composing(),
      "extra directories for static files without exclusion checking")
     ("parse-on-demand", value<bool>(&po.parseOnDemand)->default_value(true),
      "whether to parse files that are not specified from command line")
-    ("branch", value<string>(&po.branch), "SVN branch")
+    ("branch", value<std::string>(&po.branch), "SVN branch")
     ("revision", value<int>(&po.revision), "SVN revision")
-    ("output-dir,o", value<string>(&po.outputDir), "output directory")
-    ("sync-dir", value<string>(&po.syncDir),
+    ("output-dir,o", value<std::string>(&po.outputDir), "output directory")
+    ("sync-dir", value<std::string>(&po.syncDir),
      "Files will be created in this directory first, then sync with output "
      "directory without overwriting identical files. Great for incremental "
      "compilation and build.")
@@ -280,20 +277,18 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
      "whether to generate code errors")
     ("keep-tempdir,k", value<bool>(&po.keepTempDir)->default_value(false),
      "whether to keep the temporary directory")
-    ("db-stats", value<string>(&po.dbStats),
-     "database connection string to save code errors: "
-     "<username>:<password>@<host>:<port>/<db>")
-    ("no-type-inference",
-     value<bool>(&po.noTypeInference)->default_value(false),
-     "turn off type inference for C++ code generation")
-    ("config,c", value<vector<string> >(&po.config)->composing(),
+    ("config,c", value<std::vector<std::string>>(&po.config)->composing(),
      "config file name")
-    ("config-dir", value<string>(&po.configDir),
+    ("config-dir", value<std::string>(&po.configDir),
      "root directory configuration is based on (for example, "
      "excluded directories may be relative path in configuration.")
-    ("config-value,v", value<vector<string> >(&po.confStrings)->composing(),
+    ("config-value,v",
+     value<std::vector<std::string>>(&po.confStrings)->composing(),
      "individual configuration string in a format of name=value, where "
      "name can be any valid configuration for a config file")
+    ("define,d", value<std::vector<std::string>>(&po.iniStrings)->composing(),
+     "define an ini setting in the same format ( foo[=bar] ) as provided in a "
+     ".ini file")
     ("log,l",
      value<int>(&po.logLevel)->default_value(-1),
      "-1: (default); 0: no logging; 1: errors only; 2: warnings and errors; "
@@ -302,14 +297,11 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
      value<bool>(&po.force)->default_value(true),
      "force to ignore code generation errors and continue compilations")
     ("file-cache",
-     value<string>(&po.filecache),
+     value<std::string>(&po.filecache),
      "if specified, generate a static file cache with this file name")
     ("dump",
      value<bool>(&po.dump)->default_value(false),
      "dump the program graph")
-    ("docjson",
-     value<string>(&po.docjson)->default_value(""),
-     "Filename to generate a JSON file for PHP docs")
     ("coredump",
      value<bool>(&po.coredump)->default_value(false),
      "turn on coredump")
@@ -317,9 +309,6 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
      value<bool>(&po.nofork)->default_value(false),
      "forking is needed for large compilation to release memory before g++"
      "compilation. turning off forking can help gdb debugging.")
-    ("opts",
-     value<string>(&po.optimizations)->default_value(""),
-     "Set optimizations to enable/disable")
     ("compiler-id", "display the git hash for the compiler id")
     ("repo-schema", "display the repo schema id used by this app")
     ;
@@ -328,11 +317,38 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
   p.add("inputs", -1);
   variables_map vm;
   try {
-    store(command_line_parser(argc, argv).options(desc).positional(p).run(),
-          vm);
-    notify(vm);
+    auto opts = command_line_parser(argc, argv).options(desc)
+                                               .positional(p).run();
+    try {
+      store(opts, vm);
+      notify(vm);
+#if defined(BOOST_VERSION) && BOOST_VERSION >= 105000 && BOOST_VERSION <= 105400
+    } catch (const error_with_option_name &e) {
+      std::string wrong_name = e.get_option_name();
+      std::string right_name = get_right_option_name(opts, wrong_name);
+      std::string message = e.what();
+      if (right_name != "") {
+        boost::replace_all(message, wrong_name, right_name);
+      }
+      Logger::Error("Error in command line: %s", message.c_str());
+      cout << desc << "\n";
+      return -1;
+#endif
+    } catch (const error& e) {
+      Logger::Error("Error in command line: %s", e.what());
+      cout << desc << "\n";
+      return -1;
+    }
   } catch (const unknown_option& e) {
-    Logger::Error("Error in command line: %s\n\n", e.what());
+    Logger::Error("Error in command line: %s", e.what());
+    cout << desc << "\n";
+    return -1;
+  } catch (const error& e) {
+    Logger::Error("Error in command line: %s", e.what());
+    cout << desc << "\n";
+    return -1;
+  } catch (...) {
+    Logger::Error("Error in command line parsing.");
     cout << desc << "\n";
     return -1;
   }
@@ -342,7 +358,7 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
   }
   if (vm.count("version")) {
     cout << "HipHop Repo Compiler";
-    cout << " " << k_HHVM_VERSION.c_str();
+    cout << " " << HHVM_VERSION;
     cout << " (" << (debug ? "dbg" : "rel") << ")\n";
     cout << "Compiler: " << kCompilerId << "\n";
     cout << "Repo schema: " << kRepoSchemaId << "\n";
@@ -359,8 +375,20 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
     return 1;
   }
 
+  if (po.target != "run"
+      && po.target != "lint"
+      && po.target != "php"
+      && po.target != "hhbc"
+      && po.target != "filecache") {
+    Logger::Error("Error in command line: target '%s' is not supported.",
+                  po.target.c_str());
+    // desc[ription] is the --help output
+    cout << desc << "\n";
+    return -1;
+  }
+
   if ((po.target == "hhbc" || po.target == "run") &&
-      po.format.find("exe") == string::npos) {
+      po.format.find("exe") == std::string::npos) {
     if (po.program == "program") {
       po.program = "hhvm.hhbc";
     }
@@ -375,21 +403,29 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
     Logger::LogLevel = Logger::LogInfo;
   }
 
+  MemoryManager::TlsWrapper::getCheck();
   IniSetting::Map ini = IniSetting::Map::object;
   Hdf config;
-  for (auto& c : po.config) {
-    Config::Parse(c, ini, config);
+  for (auto& file : po.config) {
+    Config::ParseConfigFile(file, ini, config);
+  }
+  for (unsigned int i = 0; i < po.iniStrings.size(); i++) {
+    Config::ParseIniString(po.iniStrings[i].c_str(), ini);
   }
   for (unsigned int i = 0; i < po.confStrings.size(); i++) {
-    config.fromString(po.confStrings[i].c_str());
+    Config::ParseHdfString(po.confStrings[i].c_str(), config);
   }
   Option::Load(ini, config);
   IniSetting::Map iniR = IniSetting::Map::object;
   Hdf runtime = config["Runtime"];
+  // The configuration command line strings were already processed above
+  // Don't process them again.
   RuntimeOption::Load(iniR, runtime);
+  RuntimeOption::EvalJit = false;
+
   initialize_repo();
 
-  vector<string> badnodes;
+  std::vector<std::string> badnodes;
   config.lint(badnodes);
   for (unsigned int i = 0; i < badnodes.size(); i++) {
     Logger::Error("Possible bad config node: %s", badnodes[i].c_str());
@@ -434,10 +470,6 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
       (format_pattern(po.excludeStaticPatterns[i], true));
   }
 
-  if (po.target == "hhbc" || po.target == "run") {
-    Option::AnalyzePerfectVirtuals = false;
-  }
-
   Option::ProgramName = po.program;
 
   if (po.format.empty()) {
@@ -450,18 +482,6 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
     }
   }
 
-  if (!po.docjson.empty()) {
-    if (po.target != "run" &&
-        po.target != "hhbc" &&
-        po.target != "analyze") {
-      Logger::Error(
-        "Cannot generate doc JSON file unless target is "
-        "'hhbc', 'run', or 'analyze'");
-    } else {
-      Option::DocJson = po.docjson;
-    }
-  }
-
   if (po.optimizeLevel == -1) {
     po.optimizeLevel = 1;
   }
@@ -471,7 +491,6 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
   Option::PostOptimization = true;
   if (po.optimizeLevel == 0) {
     // --optimize-level=0 is equivalent to --opts=none
-    po.optimizations = "none";
     Option::ParseTimeOpts = false;
   }
 
@@ -484,7 +503,15 @@ int prepareOptions(CompilerOptions &po, int argc, char **argv) {
 
 int process(const CompilerOptions &po) {
   if (po.coredump) {
-#if defined(__APPLE__) || defined(__FreeBSD__)
+#if defined(__MINGW__) || defined(_MSC_VER)
+/**
+ * Windows actually does core dump size and control at a system, not an app
+ * level.  So we do nothing here and are at the mercy of Dr. Watson
+ *
+ * Cygwin has a compat layer in place and does its own core dumping, so we
+ * still call setrlimit for core dumps
+ */
+#elif defined(__APPLE__) || defined(__FreeBSD__) || defined(__CYGWIN__)
     struct rlimit rl;
     getrlimit(RLIMIT_CORE, &rl);
     rl.rlim_cur = 80000000LL;
@@ -509,25 +536,15 @@ int process(const CompilerOptions &po) {
   }
 
   register_process_init();
-  init_thread_locals();
 
   Timer timer(Timer::WallTime);
-  AnalysisResultPtr ar;
-
   // prepare a package
   Package package(po.inputDir.c_str());
-  ar = package.getAnalysisResult();
+  AnalysisResultPtr ar = package.getAnalysisResult();
 
   hhbcTargetInit(po, ar);
 
-  std::string errs;
-  if (!AliasManager::parseOptimizations(po.optimizations, errs)) {
-    Logger::Error("%s\n", errs.c_str());
-    return false;
-  }
-
   // one time initialization
-  Type::InitTypeHintMap();
   BuiltinSymbols::LoadSuperGlobals();
   ClassInfo::Load();
 
@@ -547,6 +564,8 @@ int process(const CompilerOptions &po) {
         return false;
       }
     }
+  } else {
+    hphp_process_init();
   }
 
   {
@@ -592,7 +611,7 @@ int process(const CompilerOptions &po) {
       if (!package.parse(!po.force)) {
         return 1;
       }
-      if (Option::WholeProgram || po.target == "analyze") {
+      if (Option::WholeProgram) {
         Timer timer(Timer::WallTime, "analyzeProgram");
         ar->analyzeProgram();
       }
@@ -601,7 +620,7 @@ int process(const CompilerOptions &po) {
 
   // saving file cache
   AsyncFileCacheSaver fileCacheThread(&package, po.filecache.c_str());
-  if (po.target != "analyze" && !po.filecache.empty()) {
+  if (!po.filecache.empty()) {
     fileCacheThread.start();
   }
 
@@ -609,15 +628,29 @@ int process(const CompilerOptions &po) {
     ar->dump();
   }
 
+  ar->setFinish([&po,&timer,&package](AnalysisResultPtr ar) {
+      if (Option::DumpAst) {
+        ar->dump();
+      }
+
+      // saving stats
+      if (po.genStats) {
+        int seconds = timer.getMicroSeconds() / 1000000;
+
+        Logger::Info("saving code errors and stats...");
+        Timer timer(Timer::WallTime, "saving stats");
+        package.saveStatsToFile((po.outputDir + "/Stats.js").c_str(), seconds);
+      }
+      package.resetAr();
+    });
+
   int ret = 0;
-  if (po.target == "analyze") {
-    ret = analyzeTarget(po, ar);
-  } else if (po.target == "php") {
+  if (po.target == "php") {
     ret = phpTarget(po, ar);
   } else if (po.target == "hhbc") {
-    ret = hhbcTarget(po, ar, fileCacheThread);
+    ret = hhbcTarget(po, std::move(ar), fileCacheThread);
   } else if (po.target == "run") {
-    ret = runTargetCheck(po, ar, fileCacheThread);
+    ret = runTargetCheck(po, std::move(ar), fileCacheThread);
   } else if (po.target == "filecache") {
     // do nothing
   } else {
@@ -625,40 +658,6 @@ int process(const CompilerOptions &po) {
     return 1;
   }
 
-  if (Option::DumpAst) {
-    ar->dump();
-  }
-
-  if (!Option::DocJson.empty()) {
-    Timer timer(Timer::WallTime, "Saving doc JSON file");
-    ar->docJson(Option::DocJson);
-  }
-
-  // saving stats
-  if (po.target == "analyze" || po.genStats || !po.dbStats.empty()) {
-    int seconds = timer.getMicroSeconds() / 1000000;
-
-    Logger::Info("saving code errors and stats...");
-    Timer timer(Timer::WallTime, "saving stats");
-
-    if (!po.dbStats.empty()) {
-      try {
-        ServerDataPtr server = ServerData::Create(po.dbStats);
-        int runId = package.saveStatsToDB(server, seconds, po.branch,
-                                          po.revision);
-        package.commitStats(server, runId);
-      } catch (const DatabaseException& e) {
-        Logger::Error("%s", e.what());
-      }
-    } else {
-      Compiler::SaveErrors(ar, (po.outputDir + "/CodeError.js").c_str());
-      package.saveStatsToFile((po.outputDir + "/Stats.js").c_str(), seconds);
-    }
-  } else if (Compiler::HasError()) {
-    Logger::Info("saving code errors...");
-    Compiler::SaveErrors(ar, (po.outputDir + "/CodeError.js").c_str());
-  }
-
   if (!po.filecache.empty()) {
     fileCacheThread.waitForEnd();
   }
@@ -670,7 +669,7 @@ int process(const CompilerOptions &po) {
 int lintTarget(const CompilerOptions &po) {
   int ret = 0;
   for (unsigned int i = 0; i < po.inputs.size(); i++) {
-    string filename = po.inputDir + "/" + po.inputs[i];
+    std::string filename = po.inputDir + "/" + po.inputs[i];
     try {
       Scanner scanner(filename, Option::GetScannerType());
       Compiler::Parser parser(scanner, filename.c_str(),
@@ -692,33 +691,12 @@ int lintTarget(const CompilerOptions &po) {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-int analyzeTarget(const CompilerOptions &po, AnalysisResultPtr ar) {
-  int ret = 0;
-
-  if (!po.noTypeInference) {
-    Option::GenerateInferredTypes = true;
-  }
+static void wholeProgramPasses(const CompilerOptions& po,
+                               AnalysisResultPtr ar) {
   if (Option::PreOptimization) {
     Timer timer(Timer::WallTime, "pre-optimizing");
     ar->preOptimize();
   }
-
-  if (!Option::AllVolatile) {
-    Timer timer(Timer::WallTime, "analyze includes");
-    ar->analyzeIncludes();
-  }
-
-  if (Option::GenerateInferredTypes) {
-    Timer timer(Timer::WallTime, "inferring types");
-    ar->inferTypes();
-  }
-  if (Option::PostOptimization) {
-    Timer timer(Timer::WallTime, "post-optimizing");
-    ar->postOptimize();
-  }
-  ar->analyzeProgramFinal();
-
-  return ret;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -728,44 +706,35 @@ int phpTarget(const CompilerOptions &po, AnalysisResultPtr ar) {
 
   // format
   int formatCount = 0;
-  if (po.format.find("pickled") != string::npos) {
+  if (po.format.find("pickled") != std::string::npos) {
     Option::GeneratePickledPHP = true;
     formatCount++;
   }
-  if (po.format.find("inlined") != string::npos) {
+  if (po.format.find("inlined") != std::string::npos) {
     Option::GenerateInlinedPHP = true;
     formatCount++;
   }
-  if (po.format.find("trimmed") != string::npos) {
+  if (po.format.find("trimmed") != std::string::npos) {
     Option::GenerateTrimmedPHP = true;
     formatCount++;
   }
-  if (po.format.find("typeinfo") != string::npos) {
-    Option::GenerateInferredTypes = true;
-  }
   if (formatCount == 0) {
     Logger::Error("Unknown format for PHP target: %s", po.format.c_str());
     return 1;
   }
 
-  // analyze
-  if (Option::GenerateInferredTypes || Option::ConvertSuperGlobals) {
-    Logger::Info("inferring types...");
-    ar->inferTypes();
-  }
-
   // generate
   ar->setOutputPath(po.outputDir);
   if (Option::GeneratePickledPHP) {
     Logger::Info("creating pickled PHP files...");
-    string outputDir = po.outputDir;
+    std::string outputDir = po.outputDir;
     if (formatCount > 1) outputDir += "/pickled";
     mkdir(outputDir.c_str(), 0777);
     ar->outputAllPHP(CodeGenerator::PickledPHP);
   }
   if (Option::GenerateInlinedPHP) {
     Logger::Info("creating inlined PHP files...");
-    string outputDir = po.outputDir;
+    std::string outputDir = po.outputDir;
     if (formatCount > 1) outputDir += "/inlined";
     mkdir(outputDir.c_str(), 0777);
     if (!ar->outputAllPHP(CodeGenerator::InlinedPHP)) {
@@ -774,7 +743,7 @@ int phpTarget(const CompilerOptions &po, AnalysisResultPtr ar) {
   }
   if (Option::GenerateTrimmedPHP) {
     Logger::Info("creating trimmed PHP files...");
-    string outputDir = po.outputDir;
+    std::string outputDir = po.outputDir;
     if (formatCount > 1) outputDir += "/trimmed";
     mkdir(outputDir.c_str(), 0777);
     if (!ar->outputAllPHP(CodeGenerator::TrimmedPHP)) {
@@ -795,7 +764,7 @@ void hhbcTargetInit(const CompilerOptions &po, AnalysisResultPtr ar) {
   }
   // Propagate relevant compiler-specific options to the runtime.
   RuntimeOption::RepoCentralPath = ar->getOutputPath() + '/' + po.program;
-  if (po.format.find("exe") != string::npos) {
+  if (po.format.find("exe") != std::string::npos) {
     RuntimeOption::RepoCentralPath += ".hhbc";
   }
   unlink(RuntimeOption::RepoCentralPath.c_str());
@@ -803,6 +772,9 @@ void hhbcTargetInit(const CompilerOptions &po, AnalysisResultPtr ar) {
   RuntimeOption::RepoDebugInfo = Option::RepoDebugInfo;
   RuntimeOption::RepoJournal = "memory";
   RuntimeOption::EnableHipHopSyntax = Option::EnableHipHopSyntax;
+  if (Option::HardReturnTypeHints) {
+    RuntimeOption::EvalCheckReturnTypeHints = 3;
+  }
   RuntimeOption::EnableZendCompat = Option::EnableZendCompat;
   RuntimeOption::EvalJitEnableRenameFunction = Option::JitEnableRenameFunction;
   RuntimeOption::IntsOverflowToInts = Option::IntsOverflowToInts;
@@ -814,22 +786,22 @@ void hhbcTargetInit(const CompilerOptions &po, AnalysisResultPtr ar) {
   RuntimeOption::RepoCommit = false;
 }
 
-int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr ar,
+int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr&& ar,
                AsyncFileCacheSaver &fcThread) {
   int ret = 0;
   int formatCount = 0;
   const char *type = 0;
-  if (po.format.find("text") != string::npos) {
+  if (po.format.find("text") != std::string::npos) {
     Option::GenerateTextHHBC = true;
     type = "creating text HHBC files";
     formatCount++;
   }
-  if (po.format.find("binary") != string::npos) {
+  if (po.format.find("binary") != std::string::npos) {
     Option::GenerateBinaryHHBC = true;
     type = "creating binary HHBC files";
     formatCount++;
   }
-  if (po.format.find("exe") != string::npos) {
+  if (po.format.find("exe") != std::string::npos) {
     Option::GenerateBinaryHHBC = true;
     type = "creating binary HHBC files";
     formatCount++;
@@ -844,14 +816,15 @@ int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr ar,
      hoistable */
   SystemLib::s_inited = true;
   RuntimeOption::RepoCommit = true;
-  Option::AutoInline = -1;
 
   if (po.optimizeLevel > 0) {
-    ret = analyzeTarget(po, ar);
+    ret = 0;
+    wholeProgramPasses(po, ar);
+    ar->analyzeProgramFinal();
   }
 
   Timer timer(Timer::WallTime, type);
-  Compiler::emitAllHHBC(ar);
+  Compiler::emitAllHHBC(std::move(ar));
 
   if (!po.syncDir.empty()) {
     if (!po.filecache.empty()) {
@@ -861,21 +834,21 @@ int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr ar,
     boost::filesystem::remove_all(po.syncDir);
   }
 
-  if (!ret && po.format.find("exe") != string::npos) {
+  if (!ret && po.format.find("exe") != std::string::npos) {
     /*
      * We need to create an executable with the repo
      * embedded in it.
      * Copy ourself, and embed the repo as a section
      * named "repo".
      */
-    string exe = po.outputDir + '/' + po.program;
-    string repo = "repo=" + exe + ".hhbc";
-    string buf = current_executable_path();
+    std::string exe = po.outputDir + '/' + po.program;
+    std::string repo = "repo=" + exe + ".hhbc";
+    std::string buf = current_executable_path();
     if (buf.empty()) return -1;
 
     const char *argv[] = { "objcopy", "--add-section", repo.c_str(),
                            buf.c_str(), exe.c_str(), 0 };
-    string out;
+    std::string out;
     ret = Process::Exec(argv[0], argv, nullptr, out, nullptr) ? 0 : 1;
   }
 
@@ -884,10 +857,10 @@ int hhbcTarget(const CompilerOptions &po, AnalysisResultPtr ar,
 
 ///////////////////////////////////////////////////////////////////////////////
 
-int runTargetCheck(const CompilerOptions &po, AnalysisResultPtr ar,
+int runTargetCheck(const CompilerOptions &po, AnalysisResultPtr&& ar,
                    AsyncFileCacheSaver &fcThread) {
   // generate code
-  if (hhbcTarget(po, ar, fcThread)) {
+  if (hhbcTarget(po, std::move(ar), fcThread)) {
     return 1;
   }
 
@@ -911,9 +884,9 @@ int runTarget(const CompilerOptions &po) {
   }
 
   // run the executable
-  string cmd;
-  if (po.format.find("exe") == string::npos) {
-    string buf = current_executable_path();
+  std::string cmd;
+  if (po.format.find("exe") == std::string::npos) {
+    std::string buf = current_executable_path();
     if (buf.empty()) return -1;
 
     cmd += buf;
@@ -923,7 +896,7 @@ int runTarget(const CompilerOptions &po) {
     cmd += " -vRepo.Local.Mode=r- -vRepo.Local.Path=";
   }
   cmd += po.outputDir + '/' + po.program;
-  cmd += string(" --file ") +
+  cmd += std::string(" --file ") +
     (po.inputs.size() == 1 ? po.inputs[0] : "") +
     " " + po.programArgs;
   Logger::Info("running executable: %s", cmd.c_str());
@@ -944,12 +917,11 @@ void createOutputDirectory(CompilerOptions &po) {
     if (!t) {
       t = "/tmp";
     }
-    string temp = t;
+    std::string temp = t;
     temp += "/hphp_XXXXXX";
-    char path[PATH_MAX + 1];
-    strncpy(path, temp.c_str(), PATH_MAX);
-    path[PATH_MAX] = '\0';
-    po.outputDir = mkdtemp(path);
+    std::vector<char> path(begin(temp), end(temp));
+    path.push_back('\0');
+    po.outputDir = mkdtemp(&path[0]);
     Logger::Info("creating temporary directory %s ...", po.outputDir.c_str());
   }
   mkdir(po.outputDir.c_str(), 0777);
