@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,6 +18,8 @@
 #define incl_HPHP_COMPILER_PARSER_H_
 
 #include <functional>
+#include <stack>
+#include <string>
 #include <vector>
 
 #include "hphp/runtime/base/exceptions.h"
@@ -29,7 +31,9 @@
 #include "hphp/compiler/expression/scalar_expression.h"
 #include "hphp/compiler/statement/statement.h"
 #include "hphp/compiler/statement/statement_list.h"
+#include "hphp/compiler/expression/object_property_expression.h"
 #include "hphp/util/logger.h"
+#include "hphp/parser/parse-time-fatal-exception.h"
 
 #ifdef HPHP_PARSER_NS
 #undef HPHP_PARSER_NS
@@ -39,8 +43,6 @@
 #ifdef HPHP_PARSER_ERROR
 #undef HPHP_PARSER_ERROR
 #endif
-#define HPHP_PARSER_ERROR(fmt, p, args...)  \
-  throw HPHP::ParseTimeFatalException((p)->file(), (p)->line1(), fmt, ##args)
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -49,12 +51,17 @@ DECLARE_BOOST_TYPES(Expression);
 DECLARE_BOOST_TYPES(Statement);
 DECLARE_BOOST_TYPES(StatementList);
 DECLARE_BOOST_TYPES(LabelScope);
-DECLARE_BOOST_TYPES(Location);
 DECLARE_BOOST_TYPES(AnalysisResult);
 DECLARE_BOOST_TYPES(BlockScope);
 DECLARE_BOOST_TYPES(TypeAnnotation);
 
 namespace Compiler {
+
+enum class ThisContextError {
+  Assign,
+  NullSafeBase
+};
+
 ///////////////////////////////////////////////////////////////////////////////
 // scanner
 
@@ -106,6 +113,7 @@ public:
 ///////////////////////////////////////////////////////////////////////////////
 
 DECLARE_BOOST_TYPES(Parser);
+
 class Parser : public ParserBase {
 public:
   static StatementListPtr ParseString(const String& input, AnalysisResultPtr ar,
@@ -118,8 +126,11 @@ public:
 
   // implementing ParserBase
   virtual bool parseImpl();
+  virtual bool parseImpl5();
+  virtual bool parseImpl7();
   bool parse();
-  virtual void error(const char* fmt, ...) ATTRIBUTE_PRINTF(2,3);
+  virtual void error(ATTRIBUTE_PRINTF_STRING const char* fmt, ...)
+    ATTRIBUTE_PRINTF(2,3);
   IMPLEMENT_XHP_ATTRIBUTES;
 
   virtual void fatal(const Location* loc, const char* msg);
@@ -140,9 +151,13 @@ public:
   void onStaticVariable(Token &out, Token *exprs, Token &var, Token *value);
   void onClassVariableModifer(Token &mod) {}
   void onClassVariableStart(Token &out, Token *modifiers, Token &decl,
-                            Token *type);
+                            Token *type, bool abstract = false,
+                            bool typeconst = false,
+                            const TypeAnnotationPtr& typeAnnot = nullptr);
   void onClassVariable(Token &out, Token *exprs, Token &var, Token *value);
   void onClassConstant(Token &out, Token *exprs, Token &var, Token &value);
+  void onClassAbstractConstant(Token &out, Token *exprs, Token &var);
+  void onClassTypeConstant(Token &out, Token &var, Token &value);
   void onSimpleVariable(Token &out, Token &var);
   void onSynthesizedVariable(Token &out, Token &var) {
     onSimpleVariable(out, var);
@@ -157,15 +172,21 @@ public:
   void onEncapsList(Token &out, int type, Token &list);
   void addEncap(Token &out, Token *list, Token &expr, int type);
   void encapRefDim(Token &out, Token &var, Token &offset);
-  void encapObjProp(Token &out, Token &var, Token &name);
+  void encapObjProp(Token &out, Token &var,
+                    PropAccessType propAccessType, Token &name);
   void encapArray(Token &out, Token &var, Token &expr);
   void onConst(Token &out, Token &name, Token &value);
   void onConstantValue(Token &out, Token &constant);
   void onScalar(Token &out, int type, Token &scalar);
   void onExprListElem(Token &out, Token *exprs, Token &expr);
 
-  void onObjectProperty(Token &out, Token &base, Token &prop);
-  void onObjectMethodCall(Token &out, Token &base, Token &prop, Token &params);
+  void onObjectProperty(Token &out, Token &base,
+                        PropAccessType propAccessType, Token &prop);
+  void onObjectMethodCall(Token &out, Token &base, bool nullsafe, Token &prop,
+                          Token &params);
+
+  void checkClassDeclName(const std::string& name);
+  void checkAllowedInWriteContext(ExpressionPtr e);
 
   void onListAssignment(Token &out, Token &vars, Token *expr,
                         bool rhsFirst = false);
@@ -178,6 +199,7 @@ public:
   void onUnaryOpExp(Token &out, Token &operand, int op, bool front);
   void onBinaryOpExp(Token &out, Token &operand1, Token &operand2, int op);
   void onQOp(Token &out, Token &exprCond, Token *expYes, Token &expNo);
+  void onNullCoalesce(Token &out, Token &expFirst, Token &expSecond);
   void onArray(Token &out, Token &pairs, int op = T_ARRAY);
   void onArrayPair(Token &out, Token *pairs, Token *name, Token &value,
                    bool ref);
@@ -197,9 +219,15 @@ public:
                bool ref, Token *defValue, Token *attr, Token *modifier);
   void onClassStart(int type, Token &name);
   void onClass(Token &out, int type, Token &name, Token &base,
-               Token &baseInterface, Token &stmt, Token *attr);
+               Token &baseInterface, Token &stmt, Token *attr,
+               Token *enumTy);
+  void onClassExpressionStart();
+  void onClassExpression(Token &out, Token &args, Token &base,
+                         Token &baseInterface, Token &stmt);
   void onInterface(Token &out, Token &name, Token &base, Token &stmt,
                    Token *attr);
+  void onEnum(Token &out, Token &name, Token &baseTy,
+              Token &stmt, Token *attr);
   void onInterfaceName(Token &out, Token *names, Token &name);
   void onClassRequire(Token &out, Token &name, bool isClass);
   void onTraitUse(Token &out, Token &traits, Token &rules);
@@ -234,13 +262,15 @@ public:
   void onCase(Token &out, Token &cases, Token *cond, Token &stmt);
   void onBreakContinue(Token &out, bool isBreak, Token *expr);
   void onReturn(Token &out, Token *expr);
-  void onYield(Token &out, Token &expr);
-  void onYieldPair(Token &out, Token &key, Token &val);
+  void onYield(Token &out, Token *expr);
+  void onYieldFrom(Token &out, Token *expr);
+  void onYieldPair(Token &out, Token *key, Token *val);
   void onYieldBreak(Token &out);
   void onAwait(Token &out, Token &expr);
   void onGlobal(Token &out, Token &expr);
   void onGlobalVar(Token &out, Token *exprs, Token &expr);
   void onStatic(Token &out, Token &expr);
+  void onHashBang(Token &out, Token &text);
   void onEcho(Token &out, Token &expr, bool html);
   void onUnset(Token &out, Token &expr);
   void onExpStatement(Token &out, Token &expr);
@@ -261,46 +291,71 @@ public:
                   Token& ref,
                   Token& params,
                   Token& cparams,
-                  Token& stmts);
+                  Token& stmts,
+                  Token& ret);
   Token onExprForLambda(const Token& expr);
   void onClosureParam(Token &out, Token *params, Token &param, bool ref);
 
   void onLabel(Token &out, Token &label);
   void onGoto(Token &out, Token &label, bool limited);
-  void onTypedef(Token& out, const Token& name, const Token& type);
+  void setTypeVars(Token &out, const Token &name);
+  void onTypedef(Token& out, const Token& name, const Token& type,
+                 const Token* attr = nullptr);
 
   void onTypeAnnotation(Token& out, const Token& name, const Token& typeArgs);
   void onTypeList(Token& type1, const Token& type2);
   void onTypeSpecialization(Token& type, char specialization);
-
-  // for language integrated query expressions
-  void onQuery(Token &out, Token &head, Token &body);
-  void onQueryBody(Token &out, Token *clauses, Token &select, Token *cont);
-  void onQueryBodyClause(Token &out, Token *clauses, Token &clause);
-  void onFromClause(Token &out, Token &var, Token &coll);
-  void onLetClause(Token &out, Token &var, Token &expr);
-  void onWhereClause(Token &out, Token &expr);
-  void onJoinClause(Token &out, Token &var, Token &coll, Token &left,
-    Token &right);
-  void onJoinIntoClause(Token &out, Token &var, Token &coll, Token &left,
-    Token &right, Token &group);
-  void onOrderbyClause(Token &out, Token &orderings);
-  void onOrdering(Token &out, Token *orderings, Token &ordering);
-  void onOrderingExpr(Token &out, Token &expr, Token *direction);
-  void onSelectClause(Token &out, Token &expr);
-  void onGroupClause(Token &out, Token &coll, Token &key);
-  void onIntoClause(Token &out, Token &var, Token &query);
+  void onClsCnsShapeField(Token& out, const Token& cls, const Token& cns,
+    const Token& value);
+  void onShape(Token& out, const Token& shapeMemberList);
 
   // for namespace support
   void onNamespaceStart(const std::string &ns, bool file_scope = false);
   void onNamespaceEnd();
   void nns(int token = 0, const std::string& text = std::string());
+  std::string nsClassDecl(const std::string &name);
   std::string nsDecl(const std::string &name);
   std::string resolve(const std::string &ns, bool cls);
 
-  void onUse(const std::string &ns, const std::string &as);
-  void onUseFunction(const std::string &fn, const std::string &as);
-  void onUseConst(const std::string &cnst, const std::string &as);
+  typedef void (Parser::*UseDeclarationConsumer)
+    (const std::string&, const std::string&);
+
+  void onUseDeclaration(Token& out, const std::string &ns,
+                                    const std::string &as);
+  void onMixedUseDeclaration(Token &out,
+                             Token &use, UseDeclarationConsumer f);
+
+  /*
+   * The consumer parameter here vs the mixed_consumer in the
+   * UseDeclarationStatementFragment is annoying. The gist is that the following
+   * are both valid:
+   *
+   *   use function Foo\{Bar, Baz};
+   *   use Foo\{function Bar, function Baz};
+   *
+   * This means that we need to track the type of the use with each individual
+   * element in the latter "mixed" case, but also need to be able to specify it
+   * for the whole clause in the former case. Thankfully, you can't mix the two,
+   * so we can have some assertions around this.
+   *
+   * In the former case, the mixed_consumer member will always be nullptr, and
+   * the consumer parameter to these two following functions must be specified.
+   * In the latter case, the mixed_consumer member will always be specified, and
+   * the consumer parameter to these two following function must be nullptr.
+   *
+   * This means exactly one will be non-null, and we can assert that the other
+   * is null.
+   */
+  void onUse(const Token &tok, UseDeclarationConsumer f);
+  void onGroupUse(const std::string &prefix, const Token &tok,
+                  UseDeclarationConsumer f);
+
+  void useClass(const std::string &fn, const std::string &as);
+  void useFunction(const std::string &fn, const std::string &as);
+  void useConst(const std::string &cnst, const std::string &as);
+
+  void onDeclare(Token& out, Token& block);
+  void onDeclareList(Token& out, Token& ident, Token& value);
 
   /*
    * Get the current label scope. A new label scope is demarcated by
@@ -342,18 +397,27 @@ public:
 private:
   struct FunctionContext {
     FunctionContext()
-      : hasNonEmptyReturn(false)
+      : hasCallToGetArgs(false)
+      , hasNonEmptyReturn(false)
       , isGenerator(false)
       , isAsync(false)
+      , mayCallSetFrameMetadata(false)
     {}
 
-    void checkFinalAssertions() {
-      assert(!isGenerator || !hasNonEmptyReturn);
-    }
+    // Function contains a call to func_num_args, func_get_args or func_get_arg.
+    bool hasCallToGetArgs;
 
-    bool hasNonEmptyReturn; // function contains a non-empty return statement
-    bool isGenerator;       // function determined to be a generator
-    bool isAsync;           // function determined to be async
+    // Function contains a non-empty return statement.
+    bool hasNonEmptyReturn;
+
+    // Function determined to be a generator.
+    bool isGenerator;
+
+    // Function determined to be async.
+    bool isAsync;
+
+    // Function may contain a call to set_frame_metadata.
+    bool mayCallSetFrameMetadata;
   };
 
   enum class FunctionType {
@@ -362,23 +426,29 @@ private:
     Closure,
   };
 
+  struct ClassContext {
+    ClassContext(int type, std::string name)
+      : type(type), name(name) {}
+
+    int type;
+    std::string name;
+  };
+
   AnalysisResultPtr m_ar;
   FileScopePtr m_file;
   std::vector<std::string> m_comments; // for docComment stack
-  std::vector<BlockScopePtrVec> m_scopes;
-  std::vector<LabelScopePtrVec> m_labelScopes;
+  std::vector<std::vector<BlockScopePtr>> m_scopes;
+  std::vector<std::vector<LabelScopePtr>> m_labelScopes;
   std::vector<FunctionContext> m_funcContexts;
   std::vector<ScalarExpressionPtr> m_compilerHaltOffsetVec;
-  std::string m_clsName; // for T_CLASS_C inside a closure
+  std::stack<ClassContext> m_clsContexts;
   std::string m_funcName;
   std::string m_containingFuncName;
-  bool m_inTrait;
 
   // parser output
   StatementListPtr m_tree;
   std::string m_error;
 
-  std::vector<bool> m_hasCallToGetArgs;
   std::vector<StringToExpressionPtrVecMap> m_staticVars;
   bool m_lambdaMode;
   bool m_closureGenerator;
@@ -390,6 +460,9 @@ private:
   void newScope();
   void completeScope(BlockScopePtr inner);
 
+  const std::string& clsName() const;
+  bool inTrait() const;
+
   void setHasNonEmptyReturn(ConstructPtr blame);
 
   void invalidYield();
@@ -398,13 +471,14 @@ private:
   void invalidAwait();
   void setIsAsync();
 
-  static bool canBeAsyncOrGenerator(string funcName, string clsName);
-  void checkFunctionContext(string funcName,
+  static bool canBeAsyncOrGenerator(const std::string& funcName,
+                                    const std::string& clsName);
+  void checkFunctionContext(const std::string& funcName,
                             FunctionContext& funcContext,
                             ModifierExpressionPtr modifiers,
                             int returnsRef);
 
-  string getFunctionName(FunctionType type, Token* name);
+  std::string getFunctionName(FunctionType type, Token* name);
   void prepareConstructorParameters(StatementListPtr stmts,
                                     ExpressionListPtr params,
                                     bool isAbstract);
@@ -412,13 +486,17 @@ private:
                         Token *modifiers, Token &ret,
                         Token &ref, Token *name, Token &params,
                         Token &stmt, Token *attr, bool reloc);
+  StatementPtr onClassHelper(int type, const std::string &name, Token &base,
+                 Token &baseInterface, Token &stmt, Token *attr,
+                 Token *enumTy);
 
   ExpressionPtr getDynamicVariable(ExpressionPtr exp, bool encap);
   ExpressionPtr createDynamicVariable(ExpressionPtr exp);
 
-  bool hasType(Token &type);
-
-  void checkAssignThis(Token &var);
+  void checkThisContext(const std::string& var, ThisContextError error);
+  void checkThisContext(Token &var, ThisContextError error);
+  void checkThisContext(ExpressionPtr e, ThisContextError error);
+  void checkThisContext(ExpressionListPtr params, ThisContextError error);
 
   void addStatement(StatementPtr stmt, StatementPtr new_stmt);
 
@@ -430,53 +508,78 @@ private:
     InsideNamespace,
   };
 
+public:
   /*
    * An AliasTable maps aliases to names.
    * We use it instead a regular map because it lazily imports a bunch of
    * names into the current namespace when appropriate.
    */
-  class AliasTable {
-  public:
-    struct AliasEntry {
-      std::string alias;
+  struct AliasTable {
+    enum class AliasFlags {
+      None = 0,
+      HH = 0x1,
+      PHP7_ScalarTypes = 0x2,
+      PHP7_EngineExceptions = 0x4,
+    };
+
+    struct AutoAlias {
       std::string name;
+      AliasFlags flags;
     };
 
     enum class AliasType {
+      NONE,
       USE,
+      AUTO_USE,
       DEF
     };
 
-    AliasTable(const hphp_string_imap<std::string>& autoAliases,
-               std::function<bool ()> autoOracle);
+    using AutoAliasMap = std::unordered_map<
+      std::string,
+      AutoAlias,
+      string_hashi,
+      string_eqstri>;
 
-    std::string getName(std::string alias);
-    std::string getDefName(std::string alias);
-    std::string getUseName(std::string alias);
-    bool isAliased(std::string alias);
-    bool isAutoType(std::string alias);
-    bool isUseType(std::string alias);
-    bool isDefType(std::string alias);
-    void set(std::string alias, std::string name, AliasType type);
+    AliasTable(const AutoAliasMap& aliases,
+               std::function<AliasFlags ()> autoOracle);
+
+    std::string getName(const std::string& alias, int line_no);
+    std::string getNameRaw(const std::string& alias);
+    AliasType getType(const std::string& alias);
+    int getLine(const std::string& alias);
+    bool isAliased(const std::string& alias);
+    void set(const std::string& alias,
+             const std::string& name,
+             AliasType type,
+             int line_no);
     void clear();
 
   private:
     struct NameEntry {
       std::string name;
       AliasType type;
+      int line_no;
     };
 
     hphp_string_imap<NameEntry> m_aliases;
-    const hphp_string_imap<std::string>& m_autoAliases;
+    const AutoAliasMap& m_autoAliases;
     // Returns true if stuff should be auto-imported.
-    std::function<bool ()> m_autoOracle;
+    std::function<AliasFlags ()> m_autoOracle;
     void setFalseOracle();
+    const AutoAliasMap& getAutoAliases();
   };
 
+  using AutoAlias = AliasTable::AutoAlias;
+  using AliasType = AliasTable::AliasType;
+  using AliasFlags = AliasTable::AliasFlags;
+  using AutoAliasMap = AliasTable::AutoAliasMap;
+
+private:
   NamespaceState m_nsState;
   bool m_nsFileScope;
   std::string m_namespace; // current namespace
   AliasTable m_nsAliasTable;
+  std::vector<uint32_t> m_nsStack;
 
   // Function aliases
   hphp_string_iset m_fnTable;
@@ -487,11 +590,31 @@ private:
   hphp_string_map<std::string> m_cnstAliasTable;
 
   void registerAlias(std::string name);
-  bool isAutoAliasOn();
-  const hphp_string_imap<std::string>& getAutoAliasedClasses();
-  hphp_string_imap<std::string> getAutoAliasedClassesHelper();
+  AliasFlags getAliasFlags();
+  const AutoAliasMap& getAutoAliasedClasses();
 };
 
+inline Parser::AliasFlags operator|(const Parser::AliasFlags& lhs,
+                                    const Parser::AliasFlags& rhs) {
+  return (Parser::AliasFlags)((unsigned)lhs | (unsigned)rhs);
+}
+
+inline unsigned operator&(const Parser::AliasFlags& lhs,
+                          const Parser::AliasFlags& rhs) {
+  return (unsigned)lhs & (unsigned)rhs;
+}
+
+template<typename... Args>
+inline void HPHP_PARSER_ERROR(const char* fmt,
+                       Parser* p,
+                       Args&&... args) {
+  throw ParseTimeFatalException(p->file(), p->line1(), fmt, args...);
+}
+
+inline void HPHP_PARSER_ERROR(const char* msg, Parser* p) {
+  throw ParseTimeFatalException(p->file(), p->line1(), "%s", msg);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 }}
 
