@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,28 +8,29 @@
  *
  *)
 
-module Json = Hh_json
+open Core
 
 let to_json input =
-  let entries = List.map begin fun (pos, name, type_) ->
+  let entries = List.map input begin fun (pos, name, type_) ->
     let line, start, end_ = Pos.info_pos pos in
-    Json.JAssoc [ "name",  Json.JString name;
-                  "type", Json.JString type_;
-                  "line",  Json.JInt line;
-                  "char_start", Json.JInt start;
-                  "char_end",Json.JInt end_;
-                ]
-    end input in
-  Json.JList entries
+    Hh_json.JSON_Object [
+        "name",  Hh_json.JSON_String name;
+        "type", Hh_json.JSON_String type_;
+        "line",  Hh_json.int_ line;
+        "char_start", Hh_json.int_ start;
+        "char_end", Hh_json.int_ end_;
+    ]
+  end in
+  Hh_json.JSON_Array entries
 
 let print_json res =
-  print_endline (Json.json_to_string (to_json res));
+  print_endline (Hh_json.json_to_string (to_json res));
   ()
 
 let print_readable res =
-  List.iter begin fun (pos, name, type_) ->
+  List.iter res begin fun (pos, name, type_) ->
     print_endline ((Pos.string pos)^" "^name^" ("^type_^")")
-    end res;
+  end;
   ()
 
 let go res output_json =
