@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -23,13 +23,6 @@ let (auto_complete_pos: Pos.t option ref) = ref None
  * not available, but we can use this map to relate names to types *)
 let auto_complete_vars = ref (SMap.empty: Ident.t SMap.t)
 
-(*****************************************************************************)
-(* Argument info mode *)
-(*****************************************************************************)
-let (argument_info_target: (int * int) option ref) = ref None
-let (argument_info_expected: ((string option * string) list) option ref) = ref None
-let (argument_info_position: int option ref) = ref None
-
 
 (*****************************************************************************)
 (* Returns true if this is the identifier we want to auto-complete *)
@@ -40,7 +33,7 @@ type autocomplete_type =
 | Acnew
 | Actype
 | Acclass_get
-| Acvar
+| Acprop
 
 let (argument_global_type: autocomplete_type option ref) = ref None
 let auto_complete_for_global = ref ""
