@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,27 +8,113 @@
  *
  *)
 
-
+open Core
 open Utils
 
+type config = {
+  global_size: int;
+  heap_size : int;
+}
+
+let default_config =
+  let gig = 1024 * 1024 * 1024 in
+  {global_size = gig; heap_size = 20 * gig}
+
 (*****************************************************************************)
 (* Initializes the shared memory. Must be called before forking. *)
 (*****************************************************************************)
-external init: unit -> unit = "hh_shared_init"
+external hh_shared_init: global_size:int -> heap_size:int -> unit
+= "hh_shared_init"
+
+let init config =
+  hh_shared_init
+    ~global_size:config.global_size
+    ~heap_size:config.heap_size
 
 (*****************************************************************************)
 (* The shared memory garbage collector. It must be called every time we
  * free data (cf hh_shared.c for the underlying C implementation).
  *)
 (*****************************************************************************)
-external collect: unit -> unit = "hh_collect"
+external hh_collect: bool -> unit = "hh_collect"
+
+(*****************************************************************************)
+(* Serializes the shared memory and writes it to a file *)
+(*****************************************************************************)
+external save: string -> unit = "hh_save"
+
+external save_dep_table: string -> unit = "hh_save_dep_table"
+
+(*****************************************************************************)
+(* Loads the shared memory by reading from a file *)
+(*****************************************************************************)
+external load: string -> unit = "hh_load"
+
+external load_dep_table: string -> unit = "hh_load_dep_table"
+
+(*****************************************************************************)
+(* The size of the dynamically allocated shared memory section *)
+(*****************************************************************************)
+external heap_size: unit -> int = "hh_heap_size"
+
+(*****************************************************************************)
+(* The number of used slots in our hashtable *)
+(*****************************************************************************)
+external hash_used_slots : unit -> int = "hh_hash_used_slots"
+
+(*****************************************************************************)
+(* The total number of slots in our hashtable *)
+(*****************************************************************************)
+external hash_slots : unit -> int = "hh_hash_slots"
+
+(*****************************************************************************)
+(* The number of used slots in our dependency table *)
+(*****************************************************************************)
+external dep_used_slots : unit -> int = "hh_dep_used_slots"
+
+(*****************************************************************************)
+(* The total number of slots in our dependency table *)
+(*****************************************************************************)
+external dep_slots : unit -> int = "hh_dep_slots"
 
 (*****************************************************************************)
 (* Must be called after the initialization of the hack server is over.
- * (cf serverInit.ml).
- *)
+ * (cf serverInit.ml). *)
 (*****************************************************************************)
-external init_done: unit -> unit = "hh_call_after_init"
+external hh_init_done: unit -> unit = "hh_call_after_init"
+
+let init_done () =
+  hh_init_done ();
+  EventLogger.sharedmem_init_done (heap_size ())
+
+type table_stats = {
+  used_slots : int;
+  slots : int;
+}
+
+let dep_stats () = {
+  used_slots = dep_used_slots ();
+  slots = dep_slots ();
+}
+
+let hash_stats () = {
+  used_slots = hash_used_slots ();
+  slots = hash_slots ();
+}
+
+let collect (effort : [ `gentle | `aggressive ]) =
+  let old_size = heap_size () in
+  Stats.update_max_heap_size old_size;
+  let start_t = Unix.gettimeofday () in
+  hh_collect (effort = `aggressive);
+  let new_size = heap_size () in
+  let time_taken = Unix.gettimeofday () -. start_t in
+  if old_size <> new_size then begin
+    Hh_logger.log
+      "Sharedmem GC: %d bytes before; %d bytes after; in %f seconds"
+      old_size new_size time_taken;
+    EventLogger.sharedmem_gc_ran effort old_size new_size time_taken
+  end
 
 (*****************************************************************************)
 (* Module returning the MD5 of the key. It's because the code in C land
@@ -37,20 +123,23 @@ external init_done: unit -> unit = "hh_call_after_init"
  *)
 (*****************************************************************************)
 
-module Key: sig
+module type Key = sig
 
-  (* The type of keys *)
+  (* The type of keys that OCaml-land callers try to insert *)
+  type userkey
+
+  (* The type of keys that get stored in the C hashtable *)
   type t
 
-  (* The type of old keys *)
+  (* The type of old keys that get stored in the C hashtable *)
   type old
 
   (* The md5 of an old or a new key *)
   type md5
 
   (* Creation/conversion primitives *)
-  val make     : Prefix.t -> string -> t
-  val make_old : Prefix.t -> string -> old
+  val make     : Prefix.t -> userkey -> t
+  val make_old : Prefix.t -> userkey -> old
   val to_old   : t -> old
   val to_new   : old -> t
 
@@ -58,19 +147,27 @@ module Key: sig
   val md5     : t -> md5
   val md5_old : old -> md5
 
-end = struct
-  type t   = string
-  type old = string
-  type md5 = string
+end
 
-  (* The prefix we use for old keys. The prefix garantees that we never
+module KeyFunctor (UserKeyType : sig
+  type t
+  val to_string : t -> string
+end) : Key with type userkey = UserKeyType.t = struct
+
+  type userkey = UserKeyType.t
+  type t       = string
+  type old     = string
+  type md5     = string
+
+  (* The prefix we use for old keys. The prefix guarantees that we never
    * mix old and new data, because a key can never start with the prefix
    * "old_", it always starts with a number (cf Prefix.make()).
    *)
   let old_prefix = "old_"
 
-  let make = Prefix.make_key
-  let make_old prefix x = old_prefix^Prefix.make_key prefix x
+  let make prefix x = Prefix.make_key prefix (UserKeyType.to_string x)
+  let make_old prefix x =
+    old_prefix^Prefix.make_key prefix (UserKeyType.to_string x)
 
   let to_old x = old_prefix^x
 
@@ -108,18 +205,8 @@ end
  * representation).
  *)
 (*****************************************************************************)
-module Raw: functor(Value: sig type t end) -> sig
+module Raw (Key: Key) (Value: sig type t end) = struct
 
-  val hh_shared_init : unit -> unit
-  val hh_add         : Key.md5 -> Value.t -> unit
-  val hh_mem         : Key.md5 -> bool
-  val hh_get         : Key.md5 -> Value.t
-  val hh_remove      : Key.md5 -> unit
-  val hh_move        : Key.md5 -> Key.md5 -> unit
-
-end = functor(Value: sig type t end) -> struct
-
-  external hh_shared_init : unit -> unit               = "hh_shared_init"
   external hh_add         : Key.md5 -> Value.t -> unit = "hh_add"
   external hh_mem         : Key.md5 -> bool            = "hh_mem"
   external hh_get         : Key.md5 -> Value.t         = "hh_get"
@@ -135,14 +222,14 @@ end
  * The "old" representation is the value that was bound to that key in the
  * last round of type-checking.
  * Despite the fact that the same storage is used under the hood, it's good
- * to seperate the two interfaces to make sure we never mix old and new
+ * to separate the two interfaces to make sure we never mix old and new
  * values.
  *)
 (*****************************************************************************)
 
-module New: functor(Value: Value.Type) -> sig
+module New : functor (Key : Key) -> functor(Value: Value.Type) -> sig
   
-  (* Adds a binding to the table, the table is left unchanged in the
+  (* Adds a binding to the table, the table is left unchanged if the
    * key was already bound.
    *)
   val add         : Key.t -> Value.t -> unit
@@ -159,10 +246,10 @@ module New: functor(Value: Value.Type) -> sig
    *)
   val oldify      : Key.t -> unit
 
-end = functor(Value: Value.Type) -> struct
+end = functor (Key : Key) -> functor (Value : Value.Type) -> struct
 
-  module Data      = Serial(Value)
-  module Raw = Raw(Data)
+  module Data = Serial(Value)
+  module Raw = Raw (Key) (Data)
 
   let add key value = Raw.hh_add (Key.md5 key) (Data.make value)
   let mem key = Raw.hh_mem (Key.md5 key)
@@ -196,21 +283,21 @@ end = functor(Value: Value.Type) -> struct
 end
 
 (* Same as new, but for old values *)
-module Old: functor(Value: Value.Type) -> sig
+module Old : functor (Key : Key) -> functor (Value : Value.Type) -> sig
 
   val get         : Key.old -> Value.t option
-  val find_unsafe : Key.old -> Value.t
   val remove      : Key.old -> unit
   val mem         : Key.old -> bool
 
-  (* Takes an old value and moves it back to a "new" one 
+  (* Takes an old value and moves it back to a "new" one
    * (useful for auto-complete).
    *)
   val revive      : Key.old -> unit
-end = functor(Value: Value.Type) -> struct
+
+end = functor (Key : Key) -> functor (Value: Value.Type) -> struct
 
   module Data = Serial(Value)
-  module Raw = Raw(Data)
+  module Raw = Raw (Key) (Data)
 
   let get key =
     let key = Key.md5_old key in
@@ -220,15 +307,10 @@ end = functor(Value: Value.Type) -> struct
 
   let mem key = Raw.hh_mem (Key.md5_old key)
 
-  let remove key = 
+  let remove key =
     if mem key
     then Raw.hh_remove (Key.md5_old key)
 
-  let find_unsafe key = 
-    match get key with 
-    | None -> raise Not_found
-    | Some x -> x
-
   let revive key =
     if mem key
     then
@@ -245,29 +327,47 @@ end
 (*****************************************************************************)
 
 module type S = sig
+  type key
   type t
+  module KeySet : Set.S with type elt = key
+  module KeyMap : MapSig with type key = key
 
-  val add              : string -> t -> unit
-  val get              : string -> t option
-  val get_old          : string -> t option
-  val get_old_batch    : SSet.t -> t option SMap.t
-  val remove_old_batch : SSet.t -> unit
-  val find_unsafe      : string -> t
-  val get_batch        : SSet.t -> t option SMap.t
-  val remove_batch     : SSet.t -> unit
-  val mem              : string -> bool
-  val oldify_batch     : SSet.t -> unit
-  val revive_batch     : SSet.t -> unit
+  val add              : key -> t -> unit
+  val get              : key -> t option
+  val get_old          : key -> t option
+  val get_old_batch    : KeySet.t -> t option KeyMap.t
+  val remove_old_batch : KeySet.t -> unit
+  val find_unsafe      : key -> t
+  val get_batch        : KeySet.t -> t option KeyMap.t
+  val remove_batch     : KeySet.t -> unit
+  val mem              : key -> bool
+  val oldify_batch     : KeySet.t -> unit
+  val revive_batch     : KeySet.t -> unit
+end
+
+(*****************************************************************************)
+(* The interface that all keys need to implement *)
+(*****************************************************************************)
+
+module type UserKeyType = sig
+  type t
+  val to_string : t -> string
+  val compare : t -> t -> int
 end
 
 (*****************************************************************************)
 (* A functor returning an implementation of the S module without caching. *)
 (*****************************************************************************)        
 
-module NoCache = functor(Value:Value.Type) -> struct
-  module New = New(Value)
-  module Old = Old(Value)
+module NoCache (UserKeyType : UserKeyType) (Value : Value.Type) = struct
 
+  module Key = KeyFunctor (UserKeyType)
+  module New = New (Key) (Value)
+  module Old = Old (Key) (Value)
+  module KeySet = Set.Make (UserKeyType)
+  module KeyMap = MyMap (UserKeyType)
+
+  type key = UserKeyType.t
   type t = Value.t
 
   let add x y = New.add (Key.make Value.prefix x) y
@@ -280,42 +380,52 @@ module NoCache = functor(Value:Value.Type) -> struct
     let key = Key.make_old Value.prefix x in
     Old.get key
 
-  let get_old_batch xs = 
-    SSet.fold begin fun str_key acc ->
+  let get_old_batch xs =
+    KeySet.fold begin fun str_key acc ->
       let key = Key.make_old Value.prefix str_key in
-      SMap.add str_key (Old.get key) acc
-    end xs SMap.empty
+      KeyMap.add str_key (Old.get key) acc
+    end xs KeyMap.empty
 
-  let remove_batch xs = 
-    SSet.iter begin fun str_key ->
+  let remove_batch xs =
+    KeySet.iter begin fun str_key ->
       let key = Key.make Value.prefix str_key in
       New.remove key
     end xs
 
   let oldify_batch xs =
-    SSet.iter begin fun str_key ->
+    KeySet.iter begin fun str_key ->
       let key = Key.make Value.prefix str_key in
-      New.oldify key
+      if New.mem key
+      then
+        New.oldify key
+      else
+        let key = Key.make_old Value.prefix str_key in
+        Old.remove key
     end xs
 
   let revive_batch xs =
-    SSet.iter begin fun str_key ->
-      let key = Key.make_old Value.prefix str_key in
-      Old.revive key
+    KeySet.iter begin fun str_key ->
+      let old_key = Key.make_old Value.prefix str_key in
+      if Old.mem old_key
+      then
+        Old.revive old_key
+      else
+        let key = Key.make Value.prefix str_key in
+        New.remove key
     end xs
 
   let get_batch xs = 
-    SSet.fold begin fun str_key acc ->
+    KeySet.fold begin fun str_key acc ->
       let key = Key.make Value.prefix str_key in
       match New.get key with
-      | None -> SMap.add str_key None acc
-      | Some data -> SMap.add str_key (Some data) acc
-    end xs SMap.empty
+      | None -> KeyMap.add str_key None acc
+      | Some data -> KeyMap.add str_key (Some data) acc
+    end xs KeyMap.empty
 
   let mem x = New.mem (Key.make Value.prefix x)
 
   let remove_old_batch xs =
-    SSet.iter begin fun str_key ->
+    KeySet.iter begin fun str_key ->
       let key = Key.make_old Value.prefix str_key in
       Old.remove key
     end xs
@@ -342,21 +452,21 @@ end
 (*****************************************************************************)
       
 module type CacheType = sig
+  type key
   type value
 
-  val add: Key.t -> value -> unit
-  val get: Key.t -> value option
-  val find: Key.t -> value
-  val remove: Key.t -> unit
+  val add: key -> value -> unit
+  val get: key -> value option
+  val remove: key -> unit
   val clear: unit -> unit
 end
 
-
 (*****************************************************************************)
 (* Cache keeping the objects the most frequently used. *)
 (*****************************************************************************)
       
-module FreqCache = functor(Config:ConfigType) -> struct
+module FreqCache (Key : Key) (Config:ConfigType) :
+  CacheType with type key := Key.t and type value := Config.value = struct
 
   type value = Config.value
 
@@ -369,7 +479,6 @@ module FreqCache = functor(Config:ConfigType) -> struct
     Hashtbl.clear cache;
     size := 0
  
-
 (* The collection function is called when we reach twice original capacity
  * in size. When the collection is triggered, we only keep the most recent
  * object.
@@ -421,19 +530,17 @@ module FreqCache = functor(Config:ConfigType) -> struct
     if Hashtbl.mem cache x
     then decr size;
     Hashtbl.remove cache x
-      
+
 end
 
 (*****************************************************************************)
 (* An ordered cache keeps the most recently used objects *)
 (*****************************************************************************)
 
-module OrderedCache = functor(Config:ConfigType) -> struct
-  
-  type value = Config.value
+module OrderedCache (Key : Key) (Config:ConfigType):
+  CacheType with type key := Key.t and type value := Config.value = struct
 
-  let iorder = ref 0
-  let (cache: (Key.t, Config.value) Hashtbl.t) = 
+  let (cache: (Key.t, Config.value) Hashtbl.t) =
     Hashtbl.create Config.capacity
 
   let queue = Queue.create()
@@ -479,7 +586,7 @@ end
 
 let invalidate_callback_list = ref []
 let invalidate_caches () =
-  List.iter begin fun callback -> callback() end !invalidate_callback_list
+  List.iter !invalidate_callback_list begin fun callback -> callback() end
 
 (*****************************************************************************)
 (* A functor returning an implementation of the S module with caching.
@@ -487,8 +594,9 @@ let invalidate_caches () =
  * much time. The caches keep a deserialized version of the types.
  *)
 (*****************************************************************************)        
+module WithCache (UserKeyType : UserKeyType) (Value:Value.Type) = struct
 
-module WithCache = functor(Value:Value.Type) -> struct
+  type key = UserKeyType.t
   type t = Value.t
 
   module ConfValue = struct
@@ -496,15 +604,19 @@ module WithCache = functor(Value:Value.Type) -> struct
     let capacity = 1000
   end
 
+  module Key = KeyFunctor (UserKeyType)
+
   (* Young values cache *)
-  module L1 = OrderedCache(ConfValue)
+  module L1 = OrderedCache (Key) (ConfValue)
   (* Frequent values cache *)
-  module L2 = FreqCache(ConfValue)
+  module L2 = FreqCache (Key) (ConfValue)
 
-  module New = New(Value)
-  module Old = Old(Value)
+  module New = New (Key) (Value)
+  module Old = Old (Key) (Value)
+  module KeySet = Set.Make (UserKeyType)
+  module KeyMap = MyMap (UserKeyType)
 
-  module Direct = NoCache(Value)
+  module Direct = NoCache (UserKeyType) (Value)
 
   let add x y = 
     let x = Key.make Value.prefix x in
@@ -512,10 +624,6 @@ module WithCache = functor(Value:Value.Type) -> struct
     L2.add x y;
     New.add x y
 
-  let force_get = function
-    | Some x -> x
-    | None -> raise Not_found
-        
   let get x = 
     let x = Key.make Value.prefix x in
     match L1.get x with
@@ -559,21 +667,27 @@ module WithCache = functor(Value:Value.Type) -> struct
     New.remove x
 
   let get_batch keys =
-    SSet.fold begin fun key acc ->
-      SMap.add key (get key) acc
-    end keys SMap.empty
+    KeySet.fold begin fun key acc ->
+      KeyMap.add key (get key) acc
+    end keys KeyMap.empty
 
   let oldify_batch keys =
     Direct.oldify_batch keys;
-    SSet.iter begin fun key ->
+    KeySet.iter begin fun key ->
       let key = Key.make Value.prefix key in
       L1.remove key;
       L2.remove key;
     end keys
 
-  let revive_batch = Direct.revive_batch
+  let revive_batch keys =
+    Direct.revive_batch keys;
+    KeySet.iter begin fun x ->
+      let x = Key.make Value.prefix x in
+      L1.remove x;
+      L2.remove x;
+    end keys
 
-  let remove_batch xs = SSet.iter remove xs
+  let remove_batch xs = KeySet.iter remove xs
 
   let () =
     invalidate_callback_list := begin fun () ->
@@ -582,7 +696,6 @@ module WithCache = functor(Value:Value.Type) -> struct
     end :: !invalidate_callback_list
 
   let remove_old x = Old.remove (Key.make_old Value.prefix x)
-  let remove_old_batch = SSet.iter remove_old
+  let remove_old_batch = KeySet.iter remove_old
 
 end
-
