@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -10,7 +10,14 @@
 
 type env = {
   ns_uses: string Utils.SMap.t;
+  ns_fun_uses: string Utils.SMap.t;
+  ns_const_uses: string Utils.SMap.t;
   ns_name: string option;
 }
 
-let empty = { ns_uses = Utils.SMap.empty; ns_name = None }
+let empty = {
+  ns_uses = Utils.SMap.empty;
+  ns_fun_uses = Utils.SMap.empty;
+  ns_const_uses = Utils.SMap.empty;
+  ns_name = None
+}
