@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -7,16 +7,12 @@
  * of patent rights can be found in the PATENTS file in the same directory.
  *
  *)
-open Utils
-
 val empty_file_info: FileInfo.t
 
 val go:
   Worker.t list option ->
-  bool ->
-  FileInfo.t SMap.t ->
-  get_next:(unit -> string list) ->
-  FileInfo.t SMap.t * Errors.t * SSet.t
+  get_next:(unit -> Relative_path.t list) ->
+  FileInfo.t Relative_path.Map.t * Errors.t * Relative_path.Set.t
 
 (* used by hack build *)
-val legacy_php_file_info: (string -> FileInfo.t) ref
+val legacy_php_file_info: (Relative_path.t -> FileInfo.t) ref
