@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,8 +8,7 @@
  *
  *)
 
-val make_genv: 
-  multicore:bool -> ServerArgs.options -> ServerEnv.genv
+val make_genv:
+  ServerArgs.options -> ServerConfig.t -> ServerLocalConfig.t -> ServerEnv.genv
 
-val make_env:
-  ServerArgs.options -> ServerEnv.env
+val make_env: ServerConfig.t -> ServerEnv.env
