@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,9 +8,10 @@
  *
  *)
 
+(* Saves the state that load_mini_script below reads in *)
+val save_state: ServerEnv.env -> string -> unit
+
 (* will parse, name, typecheck, the next set of files
  * and refresh the environment and update the many shared heaps
  *)
-val init:
-  ServerEnv.genv -> ServerEnv.env -> string MultiWorker.nextlist ->
-  ServerEnv.env
+val init: ?load_mini_script:Path.t -> ServerEnv.genv -> ServerEnv.env
