@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,7 +8,11 @@
  *
  *)
 
-let go query oc =
-  let results = SearchService.MasterApi.query query in
-  Marshal.to_channel oc results [];
-  flush oc
+open Core
+
+type result =
+  (Pos.absolute, HackSearchService.search_result_type) SearchUtils.term list
+
+let go query type_ =
+  let results = HackSearchService.MasterApi.query query type_ in
+  List.map results SearchUtils.to_absolute
