@@ -1,6 +1,5 @@
-
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -9,4 +8,5 @@
  *
  *)
 
-let go _ _ _ = ""
+let output_facts _ _ = ()
+let go _ _ = ""
