@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -15,19 +15,19 @@
 open Utils
 
 (* The set of files that failed *)
-type failed = SSet.t
-
-(* The result excepted from the service *)
+type failed = Relative_path.Set.t
+(* The result expected from the service *)
 type result = Errors.t * failed
 
 (*****************************************************************************)
 (* We need to know all the classes defined, because we want to declare
  * the types in their topological order *)
 (*****************************************************************************)
-val get_classes: FileInfo.fast -> SSet.t SMap.t
+val get_classes: FileInfo.fast -> Relative_path.Set.t SMap.t
 
 (*****************************************************************************)
 (* Starts the process *)
 (*****************************************************************************)
-val go: Worker.t list option -> Naming.env -> FileInfo.fast -> result
+val go: Worker.t list option -> bucket_size:int -> Naming.env ->
+  FileInfo.fast -> result
 val merge_decl: result -> result -> result
