@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -8,11 +8,12 @@
  *
  *)
 
-
+open Core
 open Utils
 open Typing_defs
 open Nast
 
+module SN = Naming_special_names
 module Dep = Typing_deps.Dep
 
 (* The following classes are used to make sure we make no typing
@@ -23,86 +24,59 @@ module Dep = Typing_deps.Dep
  * very well isolated.
 *)
 
-(* Module used to represent serialized classes *)
-module Class = struct
-  type t = class_type
-  let prefix = Prefix.make()
-end
-
-(* a function type *)
-module Fun = struct
-  type t = fun_type
-  let prefix = Prefix.make()
-end
-
-module Typedef = struct
-
-  type visibility =
-    | Public
-    | Private
-
-  type tdef = visibility * Typing_defs.tparam list * ty option * ty * Pos.t
-
-  type tdef_or_error =
-    | Error
-    | Ok of tdef
-
-  type t = tdef_or_error
-  let prefix = Prefix.make()
-end
-
-module GConst = struct
-  type t = ty
-  let prefix = Prefix.make()
-end
-
-module Funs = SharedMem.WithCache(Fun)
-module Classes = SharedMem.WithCache(Class)
-module Typedefs = SharedMem.WithCache(Typedef)
-module GConsts = SharedMem.WithCache(GConst)
-
-type funs    = Funs.t
-type classes = Classes.t
+module Funs = Typing_heap.Funs
+module Classes = Typing_heap.Classes
+module Typedefs = Typing_heap.Typedefs
+module GConsts = Typing_heap.GConsts
 
 type fake_members = {
-    last_call : Pos.t option;
-    invalid   : SSet.t;
-    valid     : SSet.t;
-  }
-
-type local = ty list * ty
+  last_call : Pos.t option;
+  invalid   : SSet.t;
+  valid     : SSet.t;
+}
+(* Along with a type, each local variable has a expression id associated with
+ * it. This is used when generating expression dependent types for the 'this'
+ * type. The idea is that if two local variables have the same expression_id
+ * then they refer to the same late bound type, and thus have compatible
+ * 'this' types.
+ *)
+type expression_id = Ident.t
+type local = locl ty list * locl ty * expression_id
 type local_env = fake_members * local IMap.t
 
 type env = {
-    pos     : Pos.t      ;
-    tenv    : ty  IMap.t ;
-    subst   : int IMap.t ;
-    lenv    : local_env ;
-    genv    : genv       ;
-    todo    : tfun list  ;
-    pclasses : SSet.t;
-    pfuns    : SSet.t;
-  }
+  pos     : Pos.t      ;
+  tenv    : locl ty IMap.t ;
+  subst   : int IMap.t ;
+  lenv    : local_env  ;
+  genv    : genv       ;
+  todo    : tfun list  ;
+  in_loop : bool       ;
+  (* when encountering Tunresolved in the supertype, do we allow it to grow?
+   * if false, this allows the opposite, i.e. Tunresolved can grow in the
+   * subtype. *)
+  grow_super : bool      ;
+}
 
 and genv = {
-    mode    : Ast.mode;
-    return  : ty         ;
-    parent  : ty         ;
-    self_id : string     ;
-    self    : ty         ;
-    static  : bool       ;
-    has_yield : bool     ;
-    allow_null_as_void : bool;
-    f_type  : Ast.fun_type;
-    anons   : anon IMap.t;
-    droot   : Typing_deps.Dep.variant option  ;
-    file    : string;
-  }
+  tcopt   : TypecheckerOptions.t;
+  mode    : FileInfo.mode;
+  return  : locl ty;
+  parent_id : string;
+  parent  : decl ty;
+  self_id : string;
+  self    : locl ty;
+  static  : bool;
+  fun_kind : Ast.fun_kind;
+  anons   : anon IMap.t;
+  droot   : Typing_deps.Dep.variant option;
+  file    : Relative_path.t;
+}
 
 (* An anonymous function
  * the environment + the fun parameters + the captured identifiers
 *)
-and anon = env -> fun_params -> env * ty
+and anon = env -> locl fun_params -> env * locl ty
 and tfun = env -> env
 
 let fresh () =
@@ -143,7 +117,6 @@ let get_type env x =
   | None -> env, (Reason.none, Tany)
   | Some ty -> env, ty
 
-
 let get_type_unsafe env x =
   let ty = IMap.get x env.tenv in
   match ty with
@@ -156,45 +129,93 @@ let expand_type env x =
   | _, Tvar x -> get_type env x
   | x -> env, x
 
-let has_type env x =
-  let env, x = get_var env x in
-  IMap.mem x env.tenv
+let expand_type_recorded env set ty =
+  match ty with
+  | r, Tvar x -> begin
+    let env, ty =
+      if ISet.mem x set then env, (r, Tany) else expand_type env ty in
+    let set = ISet.add x set in
+    env, set, ty
+  end
+  | x -> env, set, x
 
-let make_ft env p params ret_ty =
+let make_ft p params ret_ty =
   let arity = List.length params in
   {
-   ft_pos = p;
-   ft_unsafe    = false;
-   ft_abstract  = false;
-   ft_arity_min = arity;
-   ft_arity_max = arity;
-   ft_tparams   = [];
-   ft_params    = params;
-   ft_ret       = ret_ty;
+    ft_pos      = p;
+    ft_deprecated = None;
+    ft_abstract = false;
+    ft_arity    = Fstandard (arity, arity);
+    ft_tparams  = [];
+    ft_params   = params;
+    ft_ret      = ret_ty;
   }
 
+let get_shape_field_name = function
+  | SFlit (_, s) -> s
+  | SFclass_const ((_, s1), (_, s2)) -> s1^"::"^s2
+
+(* When printing out types (by hh_show()), TVars are printed with an
+ * associated identifier. We reindex them (in the order of appearance) to be
+ * consecutive integers starting from zero, because the internal identifier
+ * can change due to unrelated reasons, which breaks tests. *)
+let printable_tvar_ids = ref IMap.empty
+
+let get_printable_tvar_id x =
+  match IMap.get x !printable_tvar_ids with
+    | None ->
+        let res = (IMap.cardinal !printable_tvar_ids) + 1 in
+        printable_tvar_ids := IMap.add
+          x
+          res
+          !printable_tvar_ids;
+       res
+    | Some v -> v
+
+let debug_shape_map fdm f =
+  let o = print_string in
+  let cmp = (fun (k1, _) (k2, _) ->
+     compare (get_shape_field_name k1) (get_shape_field_name k2)) in
+  let fields = List.sort ~cmp (ShapeMap.elements fdm) in
+  let o_field = (fun (k, v) ->
+     o (get_shape_field_name k); o " => "; f v;)
+  in
+  (match fields with
+  | [] -> ()
+  | f::l ->
+     o_field f;
+     List.iter l (fun f -> o ", "; o_field f;))
+
 let rec debug stack env (r, ty) =
   let o = print_string in
   (match r with Reason.Rlost_info _ -> o "~lost" | _ -> ());
   match ty with
   | Tunresolved tyl -> o "intersect("; debugl stack env tyl; o ")"
-  | Ttuple _ -> o "tuple"
-  | Tarray (_, None, None) -> o "array"
-  | Tarray (_, Some x, None) -> o "array<"; debug stack env x; o ">"
-  | Tarray (_, Some x, Some y) -> o "array<"; debug stack env x; o ", ";
+  | Ttuple tyl -> o "tuple("; debugl stack env tyl; o ")"
+  | Tarraykind AKempty -> o "array"
+  | Tarraykind AKany -> o "array"
+  | Tarraykind (AKvec x) -> o "array<"; debug stack env x; o ">"
+  | Tarraykind (AKmap (x, y)) -> o "array<"; debug stack env x; o ", ";
       debug stack env y; o ">"
-  | Tarray _ -> assert false
+  | Tarraykind (AKshape fdm) ->
+    o "array<";
+      debug_shape_map fdm (fun (_tk, tv) -> debug stack env tv);
+    o ">"
+  | Tarraykind (AKtuple fields) ->
+    o "array[";
+      debugl stack env (List.rev (IMap.values fields));
+    o "]"
   | Tmixed -> o "mixed"
-  | Tabstract (x, argl, _)
-  | Tapply (x, argl) ->
-      Printf.printf "App %s" (snd x);
-      o "<"; List.iter (fun x -> debug stack env x; o ", ") argl;
+  | Tabstract (AKnewtype (x, argl), _)
+  | Tclass ((_, x), argl) ->
+      Printf.printf "App %s" x;
+      o "<"; List.iter argl (fun x -> debug stack env x; o ", ");
       o ">"
   | Tany -> o "X"
   | Tanon _ -> o "anonymous"
   | Tfun ft ->
       o "fun ";
-      List.iter (fun (_, x) -> debug stack env x; o ", ") ft.ft_params;
+      List.iter ft.ft_params (fun (_, x) -> debug stack env x; o ", ");
       o " -> ";
       debug stack env ft.ft_ret
   | Toption ty -> o "option("; debug stack env ty; o ")"
@@ -207,29 +228,46 @@ let rec debug stack env (r, ty) =
       | Tstring -> o "Tstring"
       | Tnum -> o "Tnum"
       | Tresource -> o "Tresource"
+      | Tarraykey -> o "Tarraykey"
+      | Tnoreturn -> o "Tnoreturn"
       )
-  | Tgeneric (s, x) ->
+  | Tabstract (AKgeneric(s, super), cstr_opt) ->
+      o "generic ";
       o s;
-      (match x with
-      | None -> ()
-      | Some x -> o "<"; debug stack env x; o ">"
+      (match cstr_opt, super with
+      | None, None -> ()
+      | Some x, _ -> o " as <"; debug stack env x; o ">"
+      | _, Some x -> o " super <"; debug stack env x; o ">"
       )
+  | Tabstract (ak, cstr) ->
+     o "[";  o (AbstractKind.to_string ak); o "]";
+     Option.iter cstr ~f:(debug stack env)
   | Tvar x ->
       let env, x = get_var env x in
       if ISet.mem x stack
-      then o (Ident.debug x)
+      then o (Ident.debug ~normalize:get_printable_tvar_id x)
       else
         let stack = ISet.add x stack in
         let _, y = get_var env x in
-        o "["; o (string_of_int y); o "]";
+        o "["; o (string_of_int (get_printable_tvar_id y)); o "]";
         (match get_type env x with
-        | _, (_, Tany) -> o (Ident.debug x)
+        | _, (_, Tany) -> o (Ident.debug ~normalize:get_printable_tvar_id x)
         | _, ty -> debug stack env ty)
   | Tobject -> o "object"
-  | Tshape fdm -> o "shape(";
-      SMap.iter begin fun k v ->
-        o k; o " => "; debug stack env v
-      end fdm;
+  | Tshape (fields_known, fdm) ->
+      o "shape<";
+      begin match fields_known with
+        | FieldsFullyKnown -> o "FieldsFullyKnown";
+        | FieldsPartiallyKnown unset_fields -> begin
+            o "FieldsPartiallyKnown(unset fields:";
+              ShapeMap.iter begin fun k _ ->
+                o (get_shape_field_name k); o " "
+              end unset_fields;
+            o ")"
+          end
+      end;
+      o ">(";
+        debug_shape_map fdm (fun v -> debug stack env v;);
       o ")"
 
 and debugl stack env x =
@@ -242,31 +280,31 @@ and debugl stack env x =
 let debug env ty = debug ISet.empty env ty; print_newline()
 
 let empty_fake_members = {
-    last_call = None;
-    invalid   = SSet.empty;
-    valid     = SSet.empty;
+  last_call = None;
+  invalid   = SSet.empty;
+  valid     = SSet.empty;
 }
 
 let empty_local = empty_fake_members, IMap.empty
 
-let empty file = {
+let empty tcopt file = {
   pos     = Pos.none;
   tenv    = IMap.empty;
   subst   = IMap.empty;
   lenv    = empty_local;
   todo    = [];
-  pclasses = SSet.empty;
-  pfuns = SSet.empty;
+  in_loop = false;
+  grow_super = true;
   genv    = {
-    mode    = Ast.Mstrict;
+    tcopt   = tcopt;
+    mode    = FileInfo.Mstrict;
     return  = fresh_type();
     self_id = "";
     self    = Reason.none, Tany;
     static  = false;
+    parent_id = "";
     parent  = Reason.none, Tany;
-    has_yield = false;
-    allow_null_as_void = false;
-    f_type  = Ast.FSync;
+    fun_kind = Ast.FSync;
     anons   = IMap.empty;
     droot   = None;
     file    = file;
@@ -277,43 +315,52 @@ let add_class x y =
   Classes.add x y
 
 let add_typedef x y =
-  Typedefs.add x (Typedef.Ok y)
+  Typedefs.add x (Typing_heap.Typedef.Ok y)
 
-let is_typedef env x =
+let is_typedef x =
   match Typedefs.get x with
   | None -> false
   | Some _ -> true
 
+let get_enum x =
+  match Classes.get x with
+  | Some tc when tc.tc_enum_type <> None -> Some tc
+  | _ -> None
+
+let is_enum x = get_enum x <> None
+
+let get_enum_constraint x =
+  match Classes.get x with
+  | None -> None
+  | Some tc ->
+    match tc.tc_enum_type with
+      | None -> None
+      | Some e -> e.te_constraint
+
 let add_typedef_error x =
-  Typedefs.add x Typedef.Error
+  Typedefs.add x Typing_heap.Typedef.Error
 
 (* Adds a new function (global) *)
 let add_fun x ft =
   Funs.add x ft
 
-(* Manipulating the environment *)
 let add_wclass env x =
   let dep = Dep.Class x in
   Typing_deps.add_idep env.genv.droot dep;
-  let env = { env with pclasses = SSet.add x env.pclasses } in
-  env
+  ()
 
 (* When we want to type something with a fresh typing environment *)
 let fresh_tenv env f =
   let genv = env.genv in
-  let genv = { genv with has_yield = false; allow_null_as_void = false } in
-  f { env with todo = []; tenv = IMap.empty; genv = genv }
+  f { env with todo = []; tenv = IMap.empty; genv = genv; in_loop = false }
 
 let get_class env x =
-  let env = add_wclass env x in
-  env, Classes.get x
+  add_wclass env x;
+  Classes.get x
 
 let get_typedef env x =
-  let env = add_wclass env x in
-  env, Typedefs.get x
-
-let class_exists x =
-  Classes.mem x
+  add_wclass env x;
+  Typedefs.get x
 
 let add_extends_dependency env x =
   let dep = Dep.Class x in
@@ -322,20 +369,22 @@ let add_extends_dependency env x =
   ()
 
 let get_class_dep env x =
-  let env = add_wclass env x in
+  add_wclass env x;
   add_extends_dependency env x;
-  let class_ = Classes.get x in
-  match class_ with
-  | None -> env, None
-  | Some class_ ->
-      env, Some class_
+  Classes.get x
+
+let get_typeconst env class_ mid =
+  add_wclass env class_.tc_name;
+  let dep = Dep.Const (class_.tc_name, mid) in
+  Typing_deps.add_idep env.genv.droot dep;
+  SMap.get mid class_.tc_typeconsts
 
 (* Used to access class constants. *)
 let get_const env class_ mid =
-  let env = add_wclass env class_.tc_name in
+  add_wclass env class_.tc_name;
   let dep = Dep.Const (class_.tc_name, mid) in
   Typing_deps.add_idep env.genv.droot dep;
-  env, SMap.get mid class_.tc_consts
+  SMap.get mid class_.tc_consts
 
 (* Used to access "global constants". That is constants that were
  * introduced with "const X = ...;" at topelevel, or "define('X', ...);"
@@ -345,12 +394,12 @@ let get_gconst env cst_name =
   GConsts.get cst_name
 
 let get_static_member is_method env class_ mid =
-  let env = add_wclass env class_.tc_name in
+  add_wclass env class_.tc_name;
   let dep = if is_method then Dep.SMethod (class_.tc_name, mid)
-  else Dep.SCVar (class_.tc_name, mid) in
+  else Dep.SProp (class_.tc_name, mid) in
   Typing_deps.add_idep env.genv.droot dep;
-  env, if is_method then SMap.get mid class_.tc_smethods
-  else SMap.get mid class_.tc_scvars
+  if is_method then SMap.get mid class_.tc_smethods
+  else SMap.get mid class_.tc_sprops
 
 let suggest_member members mid =
   let members = SMap.fold begin fun x ce acc ->
@@ -362,34 +411,41 @@ let suggest_member members mid =
 
 let suggest_static_member is_method class_ mid =
   let mid = String.lowercase mid in
-  let members = if is_method then class_.tc_smethods else class_.tc_scvars in
+  let members = if is_method then class_.tc_smethods else class_.tc_sprops in
   suggest_member members mid
 
-let method_exists class_ mid =
-  SMap.mem mid class_.tc_methods
-
 let get_member is_method env class_ mid =
-  let env = add_wclass env class_.tc_name in
+  add_wclass env class_.tc_name;
   let dep = if is_method then Dep.Method (class_.tc_name, mid)
-  else Dep.CVar (class_.tc_name, mid) in
+  else Dep.Prop (class_.tc_name, mid) in
   Typing_deps.add_idep env.genv.droot dep;
-  env, if is_method then (SMap.get mid class_.tc_methods)
-  else SMap.get mid class_.tc_cvars
+  if is_method then (SMap.get mid class_.tc_methods)
+  else SMap.get mid class_.tc_props
 
 let suggest_member is_method class_ mid =
   let mid = String.lowercase mid in
-  let members = if is_method then class_.tc_methods else class_.tc_cvars in
+  let members = if is_method then class_.tc_methods else class_.tc_props in
   suggest_member members mid
 
 let get_construct env class_ =
-  let env = add_wclass env class_.tc_name in
+  add_wclass env class_.tc_name;
   let dep = Dep.Cstr (class_.tc_name) in
   Typing_deps.add_idep env.genv.droot dep;
-  env, class_.tc_construct
+  class_.tc_construct
 
 let get_todo env =
   env.todo
 
+let grow_super env =
+  env.grow_super
+
+let invert_grow_super env f =
+  let old = env.grow_super in
+  let env = { env with grow_super = not old } in
+  let env = f env in
+  let env = { env with grow_super = old } in
+  env
+
 let get_return env =
   env.genv.return
 
@@ -403,32 +459,25 @@ let with_return env f =
   let env = f env in
   set_return env ret
 
-let has_yield env = env.genv.has_yield
-let allow_null_as_void env = env.genv.allow_null_as_void
 let is_static env = env.genv.static
 let get_self env = env.genv.self
 let get_self_id env = env.genv.self_id
+let is_outside_class env = (env.genv.self_id = "")
 let get_parent env = env.genv.parent
+let get_parent_id env = env.genv.parent_id
 
-let get_fn_type env = env.genv.f_type
+let get_fn_kind env = env.genv.fun_kind
+
+let get_file env = env.genv.file
 
 let get_fun env x =
   let dep = Dep.Fun x in
   Typing_deps.add_idep env.genv.droot dep;
-  let env = { env with pfuns = SSet.add x env.pfuns } in
-  env, Funs.get x
+  Funs.get x
 
-let set_has_yield ?(value=true) env =
-  {env with genv = {env.genv with has_yield = value }}
-
-let set_allow_null_as_void ?(allow=true) env =
+let set_fn_kind env fn_type =
   let genv = env.genv in
-  let genv = { genv with allow_null_as_void = allow } in
-  { env with genv = genv }
-
-let set_fn_type env fn_type =
-  let genv = env.genv in
-  let genv = { genv with f_type = fn_type } in
+  let genv = { genv with fun_kind = fn_type } in
   { env with genv = genv }
 
 let add_todo env x =
@@ -440,6 +489,11 @@ let add_anonymous env x =
   let genv = { genv with anons = IMap.add anon_id x genv.anons } in
   { env with genv = genv }, anon_id
 
+let set_anonymous env anon_id x =
+  let genv = env.genv in
+  let genv = { genv with anons = IMap.add anon_id x genv.anons } in
+  { env with genv = genv }
+
 let get_anonymous env x =
   IMap.get x env.genv.anons
 
@@ -453,6 +507,11 @@ let set_self env x =
   let genv = { genv with self = x } in
   { env with genv = genv }
 
+let set_parent_id env x =
+  let genv = env.genv in
+  let genv = { genv with parent_id = x } in
+  { env with genv = genv }
+
 let set_parent env x =
   let genv = env.genv in
   let genv = { genv with parent = x } in
@@ -475,8 +534,10 @@ let set_root env root =
 
 let get_mode env = env.genv.mode
 
-let is_strict env = get_mode env = Ast.Mstrict
-let is_decl env = get_mode env = Ast.Mdecl
+let is_strict env = get_mode env = FileInfo.Mstrict
+let is_decl env = get_mode env = FileInfo.Mdecl
+
+let get_options env = env.genv.tcopt
 
 (*
 let debug_env env =
@@ -495,7 +556,7 @@ let debug_env env =
  * if($this->x) {
  *   ... $this->x ...
  * }
- * The trick consists in relplacing $this->x with a "fake" local. So that
+ * The trick consists in replacing $this->x with a "fake" local. So that
  * all the logic that normally applies to locals is applied in cases like
  * this. Hence the name: FakeMembers.
  * All the fake members are thrown away at the first call.
@@ -553,13 +614,7 @@ module FakeMembers = struct
     string_of_int obj_name^"->"^member_name
 
   let make_static_id cid member_name =
-    let class_name =
-      match cid with
-      | CIparent -> "parent"
-      | CIself -> "self"
-      | CIstatic -> "static"
-      | CI (_, x) -> x
-    in
+    let class_name = class_id_to_str cid in
     class_name^"::"^member_name
 
   let get env obj member_name =
@@ -594,15 +649,15 @@ module FakeMembers = struct
     let fake_members = { fake_members with valid = valid } in
     { env with lenv = fake_members, locals }
 
-  let make p env obj_name member_name =
+  let make _ env obj_name member_name =
     let my_fake_local_id = make_id obj_name member_name in
     let env = add_member env my_fake_local_id in
     env, Hashtbl.hash my_fake_local_id
 
- let make_static p env class_name member_name =
+ let make_static _ env class_name member_name =
    let my_fake_local_id = make_static_id class_name member_name in
    let env = add_member env my_fake_local_id in
-    env, Hashtbl.hash my_fake_local_id
+   env, Hashtbl.hash my_fake_local_id
 
 end
 
@@ -618,7 +673,7 @@ end
 
 let rec unbind seen env ty =
   let env, ty = expand_type env ty in
-  if List.mem ty seen
+  if List.mem seen ty
   then env, ty
   else
     let seen = ty :: seen in
@@ -630,24 +685,25 @@ let rec unbind seen env ty =
 
 let unbind = unbind []
 
-(* We maintain 2 states for a local, all the types that the
- * local ever had (cf integrate in typing.ml), and the type
- * that the local currently has.
+(* We maintain 3 states for a local, all the types that the
+ * local ever had (cf integrate in typing.ml), the type
+ * that the local currently has, and an expression_id generated from
+ * the last assignment to this local.
  *)
 let set_local env x new_type =
   let fake_members, locals = env.lenv in
   let env, new_type = unbind env new_type in
-  let all_types =
+  let all_types, expr_id =
     match IMap.get x locals with
-    | None -> []
-    | Some (x, _) -> x
+    | None -> [], Ident.tmp()
+    | Some (x, _, y) -> x, y
   in
   let all_types =
-    if List.exists (fun x -> x = new_type) all_types
+    if List.exists all_types (fun x -> x = new_type)
     then all_types
     else new_type :: all_types
   in
-  let local = all_types, new_type in
+  let local = all_types, new_type, expr_id in
   let locals = IMap.add x local locals in
   let env = { env with lenv = fake_members, locals } in
   env
@@ -656,11 +712,25 @@ let get_local env x =
   let lcl = IMap.get x (snd env.lenv) in
   match lcl with
   | None -> env, (Reason.Rnone, Tany)
-  | Some (_, x) -> env, x
+  | Some (_, x, _) -> env, x
+
+let set_local_expr_id env x new_eid =
+  let fake_members, locals = env.lenv in
+  match IMap.get x locals with
+  | Some (all_types, type_, eid) when eid <> new_eid ->
+      let local = all_types, type_, new_eid in
+      let locals = IMap.add x local locals in
+      let env = { env with lenv = fake_members, locals } in
+      env
+  | _ -> env
+
+let get_local_expr_id env x =
+  let lcl = IMap.get x (snd env.lenv) in
+  Option.map lcl ~f:(fun (_, _, x) -> x)
 
 (*****************************************************************************)
 (* This function is called when we are about to type-check a block that will
- * later be fully_integrated (cf Typing.fully_integrage).
+ * later be fully_integrated (cf Typing.fully_integrate).
  * Integration is about keeping track of all the types that a local had in
  * its lifetime. It's necessary to correctly type-check catch blocks.
  * After we type-check a block, we want to take all the types that the local
@@ -690,7 +760,7 @@ let get_local env x =
 
 let freeze_local_env env =
   let (members, locals) = env.lenv in
-  let locals = IMap.map (fun (_, type_) -> [type_], type_) locals in
+  let locals = IMap.map (fun (_, type_, eid) -> [type_], type_, eid) locals in
   let lenv = members, locals in
   { env with lenv = lenv }
 
@@ -702,15 +772,18 @@ let anon anon_lenv env f =
   (* Setting up the environment. *)
   let old_lenv = env.lenv in
   let old_return = get_return env in
-  let old_has_yield = env.genv.has_yield in
-  let outer_f_type = get_fn_type env in
-  let env = set_has_yield ~value:false env in
+  let outer_fun_kind = get_fn_kind env in
   let env = { env with lenv = anon_lenv } in
   (* Typing *)
   let env, result = f env in
   (* Cleaning up the environment. *)
   let env = { env with lenv = old_lenv } in
-  let env = set_has_yield ~value:old_has_yield env in
   let env = set_return env old_return in
-  let env = set_fn_type env outer_f_type in
+  let env = set_fn_kind env outer_fun_kind in
   env, result
+
+let in_loop env f =
+  let old_in_loop = env.in_loop in
+  let env = { env with in_loop = true } in
+  let env = f env in
+  { env with in_loop = old_in_loop }
