@@ -1,5 +1,5 @@
 (**
- * Copyright (c) 2014, Facebook, Inc.
+ * Copyright (c) 2015, Facebook, Inc.
  * All rights reserved.
  *
  * This source code is licensed under the BSD-style license found in the
@@ -13,23 +13,13 @@
 (* Handling where our temporary files go *)
 (*****************************************************************************)
 
-let temp_dir_name =
-  let dir = Filename.temp_dir_name in
-  if dir.[String.length dir - 1] <> '/' then dir ^ "/" else dir
-
-let get_dir ?user:(user=None) () =
-  let user = match user with
-    | None -> Sys.getenv "USER"
-    | Some user -> user in
-  let tmp_dir = temp_dir_name ^ SysConfig.temp_base ^ "_" ^ user in
-  if not (Sys.file_exists tmp_dir)
-  then Unix.mkdir tmp_dir 0o755;
-  tmp_dir
-
 (* The missing counterpart to Filename.temp_file. Put in a random location
  * under get_dir() above. *)
-let temp_dir prefix  =
+let temp_dir parent_dir prefix  =
+  Sys_utils.mkdir_no_fail parent_dir;
   let tmpdir =
-    Printf.sprintf "%s/%s_%06x" (get_dir ()) prefix (Random.bits ()) in
+    Filename.concat
+      parent_dir
+      (Printf.sprintf "%s_%06x" prefix (Random.bits ())) in
   Unix.mkdir tmpdir 0o755;
   tmpdir
