@@ -9,10 +9,14 @@
  *
  */
 
-function show<T>(T $x): void { }
+function show<T>(T $x): void {}
 
 function test(): Vector<(function(int): int)> {
-  $v = Vector {function($x) { return $x; }};
+  $v = Vector {
+    function($x) {
+      return $x;
+    },
+  };
   $v[] = function($x) use ($v) {
     return $v[0]($x);
   };
