@@ -9,12 +9,16 @@
  *
  */
 
-function show<T>(T $x): void { }
+function show<T>(T $x): void {}
 
 function test(): void {
-  $v = Vector {function($x) { return $x; }};
-  $v[0] = function($x) use ($v) { 
-    return $v[0]($x); 
+  $v = Vector {
+    function($x) {
+      return $x;
+    },
+  };
+  $v[0] = function($x) use ($v) {
+    return $v[0]($x);
   };
   $v[0](1);
 }
