@@ -8,7 +8,9 @@
  * of patent rights can be found in the PATENTS file in the same directory.
  *
  */
-
-class B<T as B> {
-  public function get(): T { return $this; }
+class A extends B<A> {}
+class B<T as B<A>> {
+  public function get(): T {
+    return $this;
+  }
 }
