@@ -1,6 +1,5 @@
 //// newtype.php
 <?hh // strict
-// Copyright 2004-present Facebook. All Rights Reserved.
 
 newtype bar = bool;
 const bar truthy_foo = true;
@@ -12,7 +11,7 @@ abstract class Enum<T> {
 }
 
 newtype foo = bar;
-class Foo {
+class C1 {
   const foo FOO = truthy_foo;
 }
 
@@ -20,6 +19,6 @@ class Foo {
 <?hh // strict
 
 // Should fail because foo isn't an int or a string
-class Bar extends Enum<foo> {
-  const foo FOO = Foo::FOO;
+class C2 extends Enum<foo> {
+  const foo FOO = C1::FOO;
 }
