@@ -9,8 +9,6 @@
  *
  */
 
-class A extends B {
-}
+class A extends B {}
 
-class B extends A {
-}
+class B extends A {}
