@@ -14,28 +14,21 @@
  */
 
 class A {
-  public function f1(): void {
-  }
-  public static function f2(): void {
-  }
+  public function f1(): void {}
+  public static function f2(): void {}
 }
 
 class B extends A {
-  public function f1(): void {
-  }
-  public static function f2(): void {
-  }
+  public function f1(): void {}
+  public static function f2(): void {}
   public function test1(A $x): void {
     // UNSAFE
     $x::f1();
   }
-  public static function test2(): void {
-  }
+  public static function test2(): void {}
 }
 
 class C {
-  public function f3(): void {
-  }
-  public static function f4(): void {
-  }
+  public function f3(): void {}
+  public static function f4(): void {}
 }
