@@ -9,13 +9,11 @@
  *
  */
 
-class B {
-}
+class B {}
 
 class A extends B {
 
-  public function foo(): void {
-  }
+  public function foo(): void {}
 
   public static function bar(): void {
     $x = new parent();
