@@ -11,8 +11,7 @@
 
 class A {
 
-  public function foo(): void {
-  }
+  public function foo(): void {}
 
   public static function bar(): void {
     $x = new self();
