@@ -8,11 +8,10 @@
  * of patent rights can be found in the PATENTS file in the same directory.
  *
  */
-
+<<__ConsistentConstruct>>
 abstract class A {
 
-  public function foo(): void {
-  }
+  public function foo(): void {}
 
   public static function bar(): void {
     $x = new static();
@@ -20,4 +19,3 @@ abstract class A {
     echo 'test';
   }
 }
-
