@@ -9,14 +9,13 @@
  *
  */
 
+<<__ConsistentConstruct>>
 abstract class A {
 
-  public function foo(): void {
-  }
+  public function foo(): void {}
 
   public static function bar(): void {
     $x = new static();
     $x->xxx();
   }
 }
-
