@@ -13,11 +13,10 @@ interface I {}
 class A implements I {
   public function get(): void {}
 }
-class B implements I {
-}
+class B implements I {}
 class Z {}
 
 function bar(): void {
-  $x = true? new A(): new B();
+  $x = true ? new A() : new B();
   $x->get();
 }
