@@ -15,12 +15,12 @@ class A {
   private int $x;
 
   public function __construct(int $x) {
-    if(true) {
+    if (true) {
       $this->x = $x;
     } else {
-      $y = $this->init();
+      $y = f();
+      $this->init();
     }
-
   }
 
   private function init(): void {
@@ -37,3 +37,7 @@ class B extends A {
   }
 
 }
+
+function f(): int {
+  return 10;
+}
