@@ -1,6 +1,6 @@
 <?hh
 
-function f(): Continuation<int> {
+function f(): Generator<int, int, void> {
   foreach (array(1) as $x) {
     {
       // UNSAFE
