@@ -10,10 +10,7 @@
  */
 
 async function as_tuple(): Awaitable<(int, string)> {
-  $ret = await genva(
-    gen_int(),
-    gen_string()
-  );
+  $ret = await genva(gen_int(), gen_string());
   return $ret;
 }
 
@@ -22,10 +19,7 @@ async function maybe_null(): Awaitable<(int, ?int)> {
   if (flip_coin()) {
     $gen = gen_int();
   }
-  $ret = await genva(
-    gen_int(),
-    $gen
-  );
+  $ret = await genva(gen_int(), $gen);
   return $ret;
 }
 
@@ -34,7 +28,7 @@ async function def_null(): Awaitable<(int, ?int, ?string, ?int)> {
     gen_int(),
     gen_int_opt(),
     null,
-    coin_flip() ? gen_int() : null
+    coin_flip() ? gen_int() : null,
   );
   return $ret;
 }
@@ -47,6 +41,7 @@ async function gen_int_opt(): Awaitable<?int> {
   if (coin_flip()) {
     return 123;
   }
+  return null;
 }
 
 async function gen_string(): Awaitable<string> {
