@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,18 +22,18 @@
 #include <memory>
 #include <type_traits>
 
-#include "folly/gen/Base.h"
-#include "folly/Conv.h"
-#include "folly/Optional.h"
-#include "folly/Memory.h"
+#include <folly/gen/Base.h>
+#include <folly/Conv.h>
+#include <folly/Optional.h>
+#include <folly/Memory.h>
 
 #include "hphp/runtime/base/repo-auth-type.h"
 #include "hphp/runtime/base/repo-auth-type-array.h"
 #include "hphp/runtime/base/repo-auth-type-codec.h"
-#include "hphp/runtime/vm/unit.h"
+#include "hphp/runtime/vm/bytecode.h"
 #include "hphp/runtime/vm/func-emitter.h"
 #include "hphp/runtime/vm/preclass-emitter.h"
-#include "hphp/runtime/vm/bytecode.h"
+#include "hphp/runtime/vm/unit-emitter.h"
 #include "hphp/hhbbc/representation.h"
 #include "hphp/hhbbc/cfg.h"
 #include "hphp/hhbbc/unit-util.h"
@@ -49,6 +49,7 @@ namespace {
 //////////////////////////////////////////////////////////////////////
 
 const StaticString s_empty("");
+const StaticString s_invoke("__invoke");
 
 //////////////////////////////////////////////////////////////////////
 
@@ -283,10 +284,9 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
 
     auto emit_srcloc = [&] {
       if (!inst.srcLoc.isValid()) return;
-      Location loc;
-      loc.first(inst.srcLoc.start.line, inst.srcLoc.start.col);
-      loc.last(inst.srcLoc.past.line, inst.srcLoc.past.col);
-      ue.recordSourceLocation(&loc, startOffset);
+      Location::Range loc(inst.srcLoc.start.line, inst.srcLoc.start.col,
+                          inst.srcLoc.past.line, inst.srcLoc.past.col);
+      ue.recordSourceLocation(loc, startOffset);
     };
 
     auto pop = [&] (int32_t n) {
@@ -320,7 +320,7 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
       euState.defClsMap[id] = startOffset;
     };
 
-    auto nopdefcls = [&] {
+    auto defclsnop = [&] {
       auto const id = inst.DefCls.arg1;
       always_assert(euState.defClsMap[id] == kInvalidOffset);
       euState.defClsMap[id] = startOffset;
@@ -338,7 +338,7 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
 #define IMM_SA(n)      ue.emitInt32(ue.mergeLitstr(data.str##n));
 #define IMM_RATA(n)    encodeRAT(ue, data.rat);
 #define IMM_AA(n)      ue.emitInt32(ue.mergeArray(data.arr##n));
-#define IMM_OA_IMPL(n) ue.emitByte(static_cast<uint8_t>(data.subop));
+#define IMM_OA_IMPL(n) ue.emitByte(static_cast<uint8_t>(data.subop##n));
 #define IMM_OA(type)   IMM_OA_IMPL
 #define IMM_BA(n)      emit_branch(*data.target);
 #define IMM_VSA(n)     emit_vsa(data.keys);
@@ -356,13 +356,18 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
 
 #define POP_MMANY      pop(count_stack_elems(data.mvec));
 #define POP_C_MMANY    pop(1); pop(count_stack_elems(data.mvec));
-#define POP_R_MMANY    pop(1); pop(count_stack_elems(data.mvec));
-#define POP_V_MMANY    pop(1); pop(count_stack_elems(data.mvec));
+#define POP_R_MMANY    POP_C_MMANY
+#define POP_V_MMANY    POP_C_MMANY
+#define POP_MFINAL     pop(data.arg1);
+#define POP_C_MFINAL   pop(1); pop(data.arg1);
+#define POP_R_MFINAL   POP_C_MFINAL
+#define POP_V_MFINAL   POP_C_MFINAL
 #define POP_CMANY      pop(data.arg##1);
 #define POP_SMANY      pop(data.keys.size());
 #define POP_FMANY      pop(data.arg##1);
 #define POP_CVMANY     pop(data.arg##1);
 #define POP_CVUMANY    pop(data.arg##1);
+#define POP_IDX_A      pop(data.arg2 + 1);
 
 #define PUSH_NOV
 #define PUSH_ONE(x)            push(1);
@@ -370,11 +375,12 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
 #define PUSH_THREE(x, y, z)    push(3);
 #define PUSH_INS_1(x)          push(1);
 #define PUSH_INS_2(x)          push(1);
+#define PUSH_IDX_A             push(data.arg2);
 
 #define O(opcode, imms, inputs, outputs, flags)                   \
     auto emit_##opcode = [&] (const bc::opcode& data) {           \
       if (Op::opcode == Op::DefCls)    defcls();                  \
-      if (Op::opcode == Op::NopDefCls) nopdefcls();               \
+      if (Op::opcode == Op::DefClsNop) defclsnop();               \
       if (isRet(Op::opcode))           ret_assert();              \
       ue.emitOp(Op::opcode);                                      \
       POP_##inputs                                                \
@@ -430,6 +436,11 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
 #undef POP_C_MMANY
 #undef POP_R_MMANY
 #undef POP_V_MMANY
+#undef POP_MFINAL
+#undef POP_C_MFINAL
+#undef POP_R_MFINAL
+#undef POP_V_MFINAL
+#undef POP_IDX_A
 
 #undef PUSH_NOV
 #undef PUSH_ONE
@@ -437,6 +448,7 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
 #undef PUSH_THREE
 #undef PUSH_INS_1
 #undef PUSH_INS_2
+#undef PUSH_IDX_A
 
 #define O(opcode, ...)                                        \
     case Op::opcode:                                          \
@@ -466,7 +478,7 @@ EmitBcInfo emit_bytecode(EmitUnitState& euState,
     for (auto& inst : b->hhbcs) emit_inst(inst);
 
     if (b->fallthrough) {
-      if (boost::next(blockIt) == endBlockIt || blockIt[1] != b->fallthrough) {
+      if (std::next(blockIt) == endBlockIt || blockIt[1] != b->fallthrough) {
         if (b->fallthroughNS) {
           emit_inst(bc::JmpNS { b->fallthrough });
         } else {
@@ -868,6 +880,9 @@ void emit_class(EmitUnitState& state,
   for (auto& x : cls.requirements)       pce->addClassRequirement(x);
   for (auto& x : cls.traitPrecRules)     pce->addTraitPrecRule(x);
   for (auto& x : cls.traitAliasRules)    pce->addTraitAliasRule(x);
+  pce->setNumDeclMethods(cls.numDeclMethods);
+
+  pce->setIfaceVtableSlot(state.index.lookup_iface_vtable_slot(&cls));
 
   for (auto& m : cls.methods) {
     FTRACE(2, "    method: {}\n", m->name->data());
@@ -878,38 +893,81 @@ void emit_class(EmitUnitState& state,
     emit_finish_func(*m, *fe, info);
   }
 
+  std::vector<Type> useVars;
+  if (is_closure(cls)) {
+    auto f = find_method(&cls, s_invoke.get());
+    useVars = state.index.lookup_closure_use_vars(f);
+  }
+  auto uvIt = useVars.begin();
+
   auto const privateProps   = state.index.lookup_private_props(&cls);
   auto const privateStatics = state.index.lookup_private_statics(&cls);
   for (auto& prop : cls.properties) {
-    auto const repoTy = [&] (const PropState& ps) -> RepoAuthType {
-      // TODO(#3599292): we don't currently infer closure use var types.
-      if (is_closure(cls)) return RepoAuthType{};
-      auto it = ps.find(prop.name);
-      if (it == end(ps)) return RepoAuthType{};
-      auto const rat = make_repo_type(*state.index.array_table_builder(),
-                                      it->second);
+    auto const makeRat = [&] (const Type& ty) -> RepoAuthType {
+      if (ty.couldBe(TCls)) {
+        return RepoAuthType{};
+      }
+      auto const rat = make_repo_type(*state.index.array_table_builder(), ty);
       merge_repo_auth_type(ue, rat);
       return rat;
     };
 
+    auto const privPropTy = [&] (const PropState& ps) -> Type {
+      if (is_closure(cls)) {
+        // For closures use variables will be the first properties of the
+        // closure object, in declaration order
+        if (uvIt != useVars.end()) return *uvIt++;
+        return Type{};
+      }
+
+      auto it = ps.find(prop.name);
+      if (it == end(ps)) return Type{};
+      return it->second;
+    };
+
+    Type propTy;
+    auto const attrs = prop.attrs;
+    if (attrs & AttrPrivate) {
+      propTy = privPropTy((attrs & AttrStatic) ? privateStatics : privateProps);
+    } else if ((attrs & AttrPublic) && (attrs & AttrStatic)) {
+      propTy = state.index.lookup_public_static(&cls, prop.name);
+    }
+
     pce->addProperty(
       prop.name,
       prop.attrs,
       prop.typeConstraint,
       prop.docComment,
       &prop.val,
-      (prop.attrs & AttrStatic) ? repoTy(privateStatics) : repoTy(privateProps)
+      makeRat(propTy)
     );
   }
+  assert(uvIt == useVars.end());
 
   for (auto& cconst : cls.constants) {
-    pce->addConstant(
-      cconst.name,
-      cconst.typeConstraint,
-      &cconst.val,
-      cconst.phpCode
-    );
+    if (!cconst.val.hasValue()) {
+      pce->addAbstractConstant(
+        cconst.name,
+        cconst.typeConstraint,
+        cconst.isTypeconst
+      );
+    } else {
+      pce->addConstant(
+        cconst.name,
+        cconst.typeConstraint,
+        &cconst.val.value(),
+        cconst.phpCode,
+        cconst.isTypeconst
+      );
+    }
   }
+
+  pce->setEnumBaseTy(cls.enumBaseTy);
+}
+
+void emit_typealias(UnitEmitter& ue, const php::TypeAlias& alias) {
+  auto const id = ue.addTypeAlias(alias);
+  ue.pushMergeableTypeAlias(HPHP::Unit::MergeKind::TypeAlias, id);
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -923,7 +981,10 @@ std::unique_ptr<UnitEmitter> emit_unit(const Index& index,
 
   auto ue = folly::make_unique<UnitEmitter>(unit.md5);
   FTRACE(1, "  unit {}\n", unit.filename->data());
-  ue->setFilepath(unit.filename);
+  ue->m_filepath = unit.filename;
+  ue->m_preloadPriority = unit.preloadPriority;
+  ue->m_isHHFile = unit.isHHFile;
+  ue->m_useStrictTypes = unit.useStrictTypes;
 
   EmitUnitState state { index };
   state.defClsMap.resize(unit.classes.size(), kInvalidOffset);
@@ -940,9 +1001,9 @@ std::unique_ptr<UnitEmitter> emit_unit(const Index& index,
    * this.)
    */
   if (is_systemlib) {
-    ue->setMergeOnly(true);
+    ue->m_mergeOnly = true;
     auto const tv = make_tv<KindOfInt64>(1);
-    ue->setMainReturn(&tv);
+    ue->m_mainReturn = tv;
   } else {
     /*
      * TODO(#3017265): UnitEmitter is very coupled to emitter.cpp, and
@@ -950,13 +1011,13 @@ std::unique_ptr<UnitEmitter> emit_unit(const Index& index,
      * quite clear.  If you don't set returnSeen things relating to
      * hoistability break.
      */
-    ue->returnSeen();
+    ue->m_returnSeen = true;
   }
 
   emit_pseudomain(state, *ue, unit);
   for (auto& c : unit.classes)     emit_class(state, *ue, *c);
   for (auto& f : unit.funcs)       emit_func(state, *ue, *f);
-  for (auto& t : unit.typeAliases) ue->addTypeAlias(*t);
+  for (auto& t : unit.typeAliases) emit_typealias(*ue, *t);
 
   for (size_t id = 0; id < unit.classes.size(); ++id) {
     // We may not have a DefCls PC if we're a closure, or a
