@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -23,11 +23,12 @@
 #include <map>
 
 #include <boost/variant.hpp>
+#include <tbb/concurrent_hash_map.h>
 
-#include "folly/Optional.h"
+#include <folly/Optional.h>
+#include <folly/Hash.h>
 
 #include "hphp/util/either.h"
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/repo-auth-type-array.h"
 #include "hphp/runtime/vm/type-constraint.h"
 
@@ -40,6 +41,7 @@ namespace HPHP { namespace HHBBC {
 
 struct Type;
 struct Index;
+struct PublicSPropIndexer;
 
 namespace php {
 struct Class;
@@ -122,7 +124,7 @@ struct ClassInfo;
  *
  * These represent handles to program entities that may have variable
  * amounts of information.  For example, we may know the name of a
- * class in a res::Class, but not know for sure which php::Class
+ * class in a res::Class, but do not know for sure which php::Class
  * struct is actually associated with it.
  */
 namespace res {
@@ -163,11 +165,21 @@ struct Class {
    */
   SString name() const;
 
+  /*
+   * Whether this class could possibly be an interface or a trait.
+   *
+   * When returning false, it is known that this class is not an interface
+   * or a trait. When returning true, it's possible that this class is not
+   * an interface or trait but the system cannot tell.
+   */
+  bool couldBeInterfaceOrTrait() const;
+
   /*
    * Returns whether this type has the no override attribute, that is, if it
    * is a final class (explicitly marked by the user or known by the static
    * analysis).
-   * When returning false the class is guaranteed to be final. When returning
+   *
+   * When returning false the class is guaranteed to be final.  When returning
    * true the system cannot tell though the class may still be final.
    */
   bool couldBeOverriden() const;
@@ -190,6 +202,7 @@ private:
 private:
   friend std::string show(const Class&);
   friend struct ::HPHP::HHBBC::Index;
+  friend struct ::HPHP::HHBBC::PublicSPropIndexer;
   borrowed_ptr<const Index> index;
   Either<SString,borrowed_ptr<ClassInfo>> val;
 };
@@ -198,11 +211,10 @@ private:
  * This is an abstraction layer to represent possible runtime function
  * resolutions.
  *
- * Internally, this may only know the name of the function, or we may
- * know exactly which source-code-level function it refers to, or we
- * may only have ruled it down to one of a few functions in a class
- * hierarchy.  The interpreter can treat all these cases the same way
- * using this.
+ * Internally, this may only know the name of the function (or method), or we
+ * may know exactly which source-code-level function it refers to, or we may
+ * only have ruled it down to one of a few functions in a class hierarchy.  The
+ * interpreter can treat all these cases the same way using this.
  */
 struct Func {
   /*
@@ -219,9 +231,29 @@ struct Func {
    */
   SString name() const;
 
+  /*
+   * Returns whether this resolved function could possibly be going through a
+   * magic call, in the magic way.
+   *
+   * That is, if was resolved as part of a direct call to an __call method,
+   * this will say true.  If it was resolved as part as some normal method
+   * call, and we haven't proven that there's no way an __call dispatch could
+   * be involved, this will say false.
+   */
+  bool cantBeMagicCall() const;
+
 private:
   friend struct ::HPHP::HHBBC::Index;
-  using Rep = boost::variant< SString
+  struct FuncName {
+    bool operator==(FuncName o) const { return name == o.name; }
+    SString name;
+  };
+  struct MethodName {
+    bool operator==(MethodName o) const { return name == o.name; }
+    SString name;
+  };
+  using Rep = boost::variant< FuncName
+                            , MethodName
                             , borrowed_ptr<FuncInfo>
                             , borrowed_ptr<FuncFamily>
                             >;
@@ -333,14 +365,8 @@ struct Index {
    * closure.  This function should only be used with class names are
    * guaranteed to be closures (for example, the name supplied to a
    * CreateCl opcode).
-   *
-   * TODO(#3363851): logically this function should never fail to
-   * resolve, but there's a bug somewhere upstream where a closure
-   * class appears more than once but still has AttrUnique.  For now
-   * we handle this case by just returning a vector of all the
-   * possible closures.
    */
-  std::pair<res::Class,std::vector<borrowed_ptr<php::Class>>>
+  std::pair<res::Class,borrowed_ptr<php::Class>>
     resolve_closure_class(Context ctx, SString name) const;
 
   /*
@@ -379,14 +405,9 @@ struct Index {
    * Try to resolve a class method named `name' with a given Context
    * and class type.
    *
-   * Returns: folly::none if we can't figure out which function this
-   * would call.
-   *
    * Pre: clsType.subtypeOf(TCls)
    */
-  folly::Optional<res::Func> resolve_method(Context,
-                                            Type clsType,
-                                            SString name) const;
+  res::Func resolve_method(Context, Type clsType, SString name) const;
 
   /*
    * Try to resolve a class constructor for the supplied class.
@@ -403,9 +424,21 @@ struct Index {
    * This function returns a subtype of Cell, although TypeConstraints
    * at runtime can match reference parameters.  The caller should
    * make sure to handle that case.
+   *
+   * For soft constraints (@), this function returns Cell.
+   *
+   * For some non-soft constraints (such as "Stringish"), this
+   * function may return a Type that is a strict supertype of the
+   * constraint's type.
    */
   Type lookup_constraint(Context, const TypeConstraint&) const;
 
+  /*
+   * If this function returns true, it is safe to assume that Type t
+   * will always satisfy TypeConstraint tc at run time.
+   */
+  bool satisfies_constraint(Context, Type t, const TypeConstraint& tc) const;
+
   /*
    * Lookup what the best known Type for a class constant would be,
    * using a given Index and Context, if a class of that name were
@@ -479,6 +512,32 @@ struct Index {
    */
   PropState lookup_private_statics(borrowed_ptr<const php::Class>) const;
 
+  /*
+   * Lookup the best known type for a public static property, with a given
+   * class and name.
+   *
+   * This function will always return TInitGen before refine_public_statics has
+   * been called, or if the AnalyzePublicStatics option is off.
+   */
+  Type lookup_public_static(Type cls, Type name) const;
+  Type lookup_public_static(borrowed_ptr<const php::Class>, SString name) const;
+
+  /*
+   * Returns whether a public static property is known to be immutable.  This
+   * is used to add AttrPersistent flags to static properties, and relies on
+   * AnalyzePublicStatics (without this flag it will always return false).
+   */
+  bool lookup_public_static_immutable(borrowed_ptr<const php::Class>,
+                                      SString name) const;
+
+  /*
+   * Returns the computed vtable slot for the given class, if it's an interface
+   * that was given a vtable slot. No two interfaces implemented by the same
+   * class will share the same vtable slot. May return kInvalidSlot, if the
+   * given class isn't an interface or if it wasn't assigned a slot.
+   */
+  Slot lookup_iface_vtable_slot(borrowed_ptr<const php::Class>) const;
+
   /*
    * Refine the return type for a function, based on a round of
    * analysis.
@@ -523,6 +582,15 @@ struct Index {
   void refine_private_statics(borrowed_ptr<const php::Class> cls,
                               const PropState&);
 
+  /*
+   * After a whole program pass using PublicSPropIndexer, the types can be
+   * reflected into the index for use during another type inference pass.
+   *
+   * No other threads should be calling functions on this Index or on the
+   * provided PublicSPropIndexer when this function is called.
+   */
+  void refine_public_statics(const PublicSPropIndexer&);
+
 private:
   Index(const Index&) = delete;
   Index& operator=(Index&&) = delete;
@@ -535,6 +603,7 @@ private:
                             borrowed_ptr<const php::Class>) const;
   bool could_be_related(borrowed_ptr<const php::Class>,
                         borrowed_ptr<const php::Class>) const;
+  Type satisfies_constraint_helper(Context, const TypeConstraint&) const;
 
 private:
   std::unique_ptr<IndexData> const m_data;
@@ -542,6 +611,58 @@ private:
 
 //////////////////////////////////////////////////////////////////////
 
+/*
+ * Indexer object used for collecting information about public static property
+ * types.  See analyze_public_statics in whole-program.cpp for details about
+ * how it is used.
+ */
+struct PublicSPropIndexer {
+  explicit PublicSPropIndexer(borrowed_ptr<const Index> index)
+    : m_index(index)
+  {}
+
+  /*
+   * Called by the interpreter during analyze_func_collect when a
+   * PublicSPropIndexer is active.  This function must be called anywhere the
+   * interpreter does something that could change the type of public static
+   * properties named `name' on classes of type `cls' to `val'.
+   *
+   * Note that if cls and name are both too generic this object will have to
+   * give up all information it knows about any public static properties.
+   *
+   * This routine may be safely called concurrently by multiple analysis
+   * threads.
+   */
+  void merge(Context ctx, Type cls, Type name, Type val);
+
+private:
+  friend struct Index;
+
+  struct KnownKey {
+    bool operator==(KnownKey o) const {
+      return cinfo == o.cinfo && prop == o.prop;
+    }
+
+    friend size_t tbb_hasher(KnownKey k) {
+      return folly::hash::hash_combine(k.cinfo, k.prop);
+    }
+
+    borrowed_ptr<ClassInfo> cinfo;
+    SString prop;
+  };
+
+  using UnknownMap = tbb::concurrent_hash_map<SString,Type>;
+  using KnownMap = tbb::concurrent_hash_map<KnownKey,Type>;
+
+private:
+  borrowed_ptr<const Index> m_index;
+  std::atomic<bool> m_everything_bad{false};
+  UnknownMap m_unknown;
+  KnownMap m_known;
+};
+
+//////////////////////////////////////////////////////////////////////
+
 }}
 
 #endif
