@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,12 +18,15 @@
 
 #include <algorithm>
 
-#include "folly/Optional.h"
+#include <folly/Optional.h>
+
+#include "hphp/runtime/base/type-string.h"
 
 #include "hphp/hhbbc/interp-state.h"
 #include "hphp/hhbbc/interp.h"
 #include "hphp/hhbbc/representation.h"
 #include "hphp/hhbbc/type-system.h"
+#include "hphp/hhbbc/func-util.h"
 
 namespace HPHP { namespace HHBBC {
 
@@ -33,23 +36,24 @@ TRACE_SET_MOD(hhbbc);
 
 const StaticString s_extract("extract");
 const StaticString s_extract_sl("__SystemLib\\extract");
+const StaticString s_assert("assert");
+const StaticString s_assert_sl("__SystemLib\\assert");
+const StaticString s_parse_str("parse_str");
+const StaticString s_parse_str_sl("__SystemLib\\parse_str");
 const StaticString s_compact("compact");
 const StaticString s_compact_sl("__SystemLib\\compact_sl");
 const StaticString s_get_defined_vars("get_defined_vars");
 const StaticString s_get_defined_vars_sl("__SystemLib\\get_defined_vars");
 
-const StaticString s_http_response_header("http_response_header");
-const StaticString s_php_errormsg("php_errormsg");
-
 //////////////////////////////////////////////////////////////////////
 
 /*
  * Interpreter Step State.
  *
- * This struct gives interpreter functions access to shared state.
- * It's not in interp-state.h because it's part of the internal
- * implementation of interpreter routines.  The publicized state as
- * results of interpretation are in that header and interp.h.
+ * This struct gives interpreter functions access to shared state.  It's not in
+ * interp-state.h because it's part of the internal implementation of
+ * interpreter routines.  The publicized state as results of interpretation are
+ * in that header and interp.h.
  */
 struct ISS {
   explicit ISS(Interp& bag,
@@ -77,74 +81,60 @@ struct ISS {
 
 namespace {
 
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-function"
+#endif
+
 void nothrow(ISS& env) {
   FTRACE(2, "    nothrow\n");
   env.flags.wasPEI = false;
 }
 
-void calledNoReturn(ISS& env)    { env.flags.calledNoReturn = true; }
-void constprop(ISS& env)         { env.flags.canConstProp = true; }
-void nofallthrough(ISS& env)     {
+void unreachable(ISS& env)    { env.state.unreachable = true; }
+void constprop(ISS& env)      { env.flags.canConstProp = true; }
+
+void jmp_nofallthrough(ISS& env) {
   env.flags.jmpFlag = StepFlags::JmpFlags::Taken;
 }
-void never_taken(ISS& env)       {
+void jmp_nevertaken(ISS& env) {
   env.flags.jmpFlag = StepFlags::JmpFlags::Fallthrough;
 }
+
 void readUnknownLocals(ISS& env) { env.flags.mayReadLocalSet.set(); }
 void readAllLocals(ISS& env)     { env.flags.mayReadLocalSet.set(); }
 
-void doRet(ISS& env, Type t) {
-  readAllLocals(env);
-  assert(env.state.stack.empty());
-  env.flags.returned = t;
-}
-
 void killLocals(ISS& env) {
   FTRACE(2, "    killLocals\n");
   readUnknownLocals(env);
   for (auto& l : env.state.locals) l = TGen;
 }
 
-// Force non-ref locals to TCell.  Used when something modifies an
-// unknown local's value, without changing reffiness.
-void loseNonRefLocalTypes(ISS& env) {
-  readUnknownLocals(env);
-  FTRACE(2, "    loseNonRefLocalTypes\n");
-  for (auto& l : env.state.locals) {
-    if (l.subtypeOf(TCell)) l = TCell;
-  }
-}
-
-void boxUnknownLocal(ISS& env) {
-  readUnknownLocals(env);
-  FTRACE(2, "   boxUnknownLocal\n");
-  for (auto& l : env.state.locals) {
-    if (!l.subtypeOf(TRef)) l = TGen;
-  }
-}
-
-void unsetUnknownLocal(ISS& env) {
-  readUnknownLocals(env);
-  FTRACE(2, "  unsetUnknownLocal\n");
-  for (auto& l : env.state.locals) l = union_of(l, TUninit);
+void doRet(ISS& env, Type t) {
+  readAllLocals(env);
+  assert(env.state.stack.empty());
+  env.flags.returned = t;
 }
 
 void specialFunctionEffects(ISS& env, SString name) {
-  // extract() trashes the local variable environment.
-  if (name->isame(s_extract_sl.get())
-      || (!options.DisallowDynamicVarEnvFuncs && name->isame(s_extract.get()))) {
+  auto special_fn = [&] (const StaticString& fn1, const StaticString& fn2) {
+    return name->isame(fn1.get()) ||
+      (!options.DisallowDynamicVarEnvFuncs && name->isame(fn2.get()));
+  };
+
+  // extract() and parse_str() can trash the local variable environment.
+  if (special_fn(s_extract_sl, s_extract) ||
+      special_fn(s_parse_str_sl, s_parse_str)) {
     readUnknownLocals(env);
     killLocals(env);
     return;
   }
-  // compact() and get_defined_vars() read the local variable
+  // assert(), compact() and get_defined_vars() read the local variable
   // environment.  We could check which locals for compact, but for
   // now we just include them all.
-  if (name->isame(s_get_defined_vars_sl.get()) ||
-      name->isame(s_compact_sl.get()) ||
-      (!options.DisallowDynamicVarEnvFuncs && name->isame(s_compact.get())) ||
-      (!options.DisallowDynamicVarEnvFuncs && name->isame(s_get_defined_vars.get()))
-     ) {
+  if (special_fn(s_get_defined_vars_sl, s_get_defined_vars) ||
+      special_fn(s_compact_sl, s_compact) ||
+      special_fn(s_assert_sl, s_assert)) {
     readUnknownLocals(env);
     return;
   }
@@ -207,14 +197,9 @@ Type popF(ISS& env)  { return popT(env); }
 Type popCV(ISS& env) { return popT(env); }
 Type popU(ISS& env)  { return popT(env); }
 
-void popFlav(ISS& env, Flavor flav) {
-  switch (flav) {
-  case Flavor::C: popC(env); break;
-  case Flavor::V: popV(env); break;
-  case Flavor::U: popU(env); break;
-  case Flavor::F: popF(env); break;
-  case Flavor::R: popR(env); break;
-  case Flavor::A: popA(env); break;
+void discard(ISS& env, int n) {
+  for (auto i = 0; i < n; ++i) {
+    popT(env);
   }
 }
 
@@ -283,15 +268,18 @@ void mayReadLocal(ISS& env, uint32_t id) {
 
 Type locRaw(ISS& env, borrowed_ptr<const php::Local> l) {
   mayReadLocal(env, l->id);
-  return env.state.locals[l->id];
+  auto ret = env.state.locals[l->id];
+  if (is_volatile_local(env.ctx.func, l)) {
+    always_assert_flog(ret == TGen, "volatile local was not TGen");
+  }
+  return ret;
 }
 
 void setLocRaw(ISS& env, borrowed_ptr<const php::Local> l, Type t) {
   mayReadLocal(env, l->id);
-  if (l->name && (l->name->same(s_http_response_header.get()) ||
-                  l->name->same(s_php_errormsg.get()))) {
+  if (is_volatile_local(env.ctx.func, l)) {
     auto current = env.state.locals[l->id];
-    assert(current == TGen || current == TCell);
+    always_assert_flog(current == TGen, "volatile local was not TGen");
     return;
   }
   env.state.locals[l->id] = t;
@@ -317,6 +305,10 @@ Type derefLoc(ISS& env, borrowed_ptr<const php::Local> l) {
 
 void ensureInit(ISS& env, borrowed_ptr<const php::Local> l) {
   auto t = locRaw(env, l);
+  if (is_volatile_local(env.ctx.func, l)) {
+    always_assert_flog(t == TGen, "volatile local was not TGen");
+    return;
+  }
   if (t.couldBe(TUninit)) {
     if (t.subtypeOf(TUninit)) return setLocRaw(env, l, TInitNull);
     if (t.subtypeOf(TCell))   return setLocRaw(env, l, remove_uninit(t));
@@ -335,6 +327,10 @@ bool locCouldBeUninit(ISS& env, borrowed_ptr<const php::Local> l) {
  */
 void setLoc(ISS& env, borrowed_ptr<const php::Local> l, Type t) {
   auto v = locRaw(env, l);
+  if (is_volatile_local(env.ctx.func, l)) {
+    always_assert_flog(v == TGen, "volatile local was not TGen");
+    return;
+  }
   if (v.subtypeOf(TCell)) env.state.locals[l->id] = t;
 }
 
@@ -348,6 +344,30 @@ borrowed_ptr<php::Local> findLocal(ISS& env, SString name) {
   return nullptr;
 }
 
+// Force non-ref locals to TCell.  Used when something modifies an
+// unknown local's value, without changing reffiness.
+void loseNonRefLocalTypes(ISS& env) {
+  readUnknownLocals(env);
+  FTRACE(2, "    loseNonRefLocalTypes\n");
+  for (auto& l : env.state.locals) {
+    if (l.subtypeOf(TCell)) l = TCell;
+  }
+}
+
+void boxUnknownLocal(ISS& env) {
+  readUnknownLocals(env);
+  FTRACE(2, "   boxUnknownLocal\n");
+  for (auto& l : env.state.locals) {
+    if (!l.subtypeOf(TRef)) l = TGen;
+  }
+}
+
+void unsetUnknownLocal(ISS& env) {
+  readUnknownLocals(env);
+  FTRACE(2, "  unsetUnknownLocal\n");
+  for (auto& l : env.state.locals) l = union_of(l, TUninit);
+}
+
 //////////////////////////////////////////////////////////////////////
 // iterators
 
@@ -387,6 +407,14 @@ folly::Optional<Type> selfCls(ISS& env) {
   return folly::none;
 }
 
+folly::Optional<Type> selfClsExact(ISS& env) {
+  if (!env.ctx.cls) return folly::none;
+  if (auto const rcls = env.index.resolve_class(env.ctx, env.ctx.cls->name)) {
+    return clsExact(*rcls);
+  }
+  return folly::none;
+}
+
 //////////////////////////////////////////////////////////////////////
 // properties on $this
 
@@ -578,6 +606,9 @@ void loseNonRefSelfPropTypes(ISS& env) {
   }
 }
 
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 }
 
 //////////////////////////////////////////////////////////////////////
