@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -23,9 +23,8 @@
 #include <utility>
 #include <vector>
 
-#include "folly/Lazy.h"
+#include <folly/Lazy.h>
 
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/array-init.h"
 
 #include "hphp/hhbbc/hhbbc.h"
@@ -34,7 +33,7 @@
 #include "hphp/hhbbc/parse.h"
 #include "hphp/hhbbc/index.h"
 #include "hphp/runtime/vm/as.h"
-#include "hphp/runtime/vm/unit.h"
+#include "hphp/runtime/vm/unit-emitter.h"
 
 namespace HPHP { namespace HHBBC {
 
@@ -58,7 +57,7 @@ const StaticString s_IAA("IAA");
 const StaticString s_IB("IB");
 const StaticString s_NonUnique("NonUnique");
 const StaticString s_NonUniqueA("NonUniqueA");
-const StaticString s_WaitHandle("WaitHandle");
+const StaticString s_WaitHandle("HH\\WaitHandle");
 
 // A test program so we can actually test things involving object or
 // class types.
@@ -74,7 +73,7 @@ std::unique_ptr<php::Unit> make_test_unit() {
     # only one we have to make sure the type system can see for unit
     # test purposes, so we can just define it here.  We don't need to
     # give it any of its functions currently.
-    .class [abstract unique builtin] WaitHandle {
+    .class [abstract unique builtin] HH\WaitHandle {
     }
 
     .class [interface unique] IBase {
@@ -166,11 +165,11 @@ std::unique_ptr<php::Program> make_program() {
 
 //////////////////////////////////////////////////////////////////////
 
-auto const test_empty_array = folly::lazy([&] {
+auto const test_empty_array = folly::lazy([] {
   return staticEmptyArray();
 });
 
-auto const test_array_map_value = folly::lazy([&] {
+auto const test_array_map_value = folly::lazy([] {
   auto ar = make_map_array(
     s_A.get(), s_B.get(),
     s_test.get(), 12
@@ -178,7 +177,7 @@ auto const test_array_map_value = folly::lazy([&] {
   return ArrayData::GetScalarArray(ar.get());
 });
 
-auto const test_array_packed_value = folly::lazy([&] {
+auto const test_array_packed_value = folly::lazy([] {
   auto ar = make_packed_array(
     42,
     23,
@@ -187,7 +186,7 @@ auto const test_array_packed_value = folly::lazy([&] {
   return ArrayData::GetScalarArray(ar.get());
 });
 
-auto const test_array_packed_value2 = folly::lazy([&] {
+auto const test_array_packed_value2 = folly::lazy([] {
   auto ar = make_packed_array(
     42,
     23.0,
@@ -196,7 +195,7 @@ auto const test_array_packed_value2 = folly::lazy([&] {
   return ArrayData::GetScalarArray(ar.get());
 });
 
-auto const test_array_packed_value3 = folly::lazy([&] {
+auto const test_array_packed_value3 = folly::lazy([] {
   auto ar = make_packed_array(
     1,
     2,
@@ -207,7 +206,7 @@ auto const test_array_packed_value3 = folly::lazy([&] {
   return ArrayData::GetScalarArray(ar.get());
 });
 
-auto const with_data = folly::lazy([&] {
+auto const with_data = folly::lazy([] {
   return std::vector<Type> {
     ival(2),
     dval(2.0),
@@ -281,7 +280,7 @@ auto const non_opt_unions = {
 
 auto const all_unions = boost::join(optionals, non_opt_unions);
 
-auto const all = folly::lazy([&] {
+auto const all = folly::lazy([] {
   std::vector<Type> ret;
   auto const wdata = with_data();
   ret.insert(end(ret), begin(primitives), end(primitives));
@@ -303,7 +302,7 @@ std::vector<Type> all_with_waithandles(const Index& index) {
   return ret;
 }
 
-auto const specialized_array_examples = folly::lazy([&] {
+auto const specialized_array_examples = folly::lazy([] {
   auto ret = std::vector<Type>{};
 
   auto test_map_a          = StructMap{};
@@ -1300,22 +1299,18 @@ TEST(Type, Interface) {
   auto const subObjAATy  = subObj(*clsAA);
   auto const subClsAATy  = subCls(*clsAA);
 
-  // we don't support interfaces quite yet so let's put few tests
-  // that will fail once interfaces are supported
-
-  // first 2 are "not precise" - should be true
-  EXPECT_FALSE(subClsATy.subtypeOf(objcls(subObjIATy)));
-  EXPECT_FALSE(objcls(subObjATy).strictSubtypeOf(subClsIATy));
+  EXPECT_TRUE(subClsATy.subtypeOf(objcls(subObjIATy)));
   EXPECT_TRUE(subClsATy.couldBe(objcls(subObjIATy)));
-
-  // first 2 are "not precise" - should be true
-  EXPECT_FALSE(subClsAATy.subtypeOf(objcls(subObjIAATy)));
-  EXPECT_FALSE(objcls(subObjAATy).strictSubtypeOf(objcls(subObjIAATy)));
+  EXPECT_TRUE(objcls(subObjATy).strictSubtypeOf(subClsIATy));
+  EXPECT_TRUE(subClsAATy.subtypeOf(objcls(subObjIAATy)));
   EXPECT_TRUE(subClsAATy.couldBe(objcls(subObjIAATy)));
+  EXPECT_TRUE(objcls(subObjAATy).strictSubtypeOf(objcls(subObjIAATy)));
 
-  // 3rd one is not precise - should be false
   EXPECT_FALSE(subClsATy.subtypeOf(objcls(subObjIAATy)));
   EXPECT_FALSE(objcls(subObjATy).strictSubtypeOf(objcls(subObjIAATy)));
+
+  // We don't support couldBe intelligently for interfaces quite yet, so here's
+  // a test that will start failing if we ever do:
   EXPECT_TRUE(clsExactATy.couldBe(objcls(subObjIAATy)));
 }
 
