@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,9 +22,10 @@
 
 #include <boost/container/flat_map.hpp>
 
-#include "folly/Optional.h"
+#include <folly/Optional.h>
+
+#include "hphp/util/copy-ptr.h"
 
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/repo-auth-type.h"
 #include "hphp/runtime/base/repo-auth-type-array.h"
 
@@ -88,27 +89,26 @@ struct Type;
  *
  *   {Init,}Prim
  *
- *       "Primitive" types---these can be represented in a TypedValue
- *       without a pointer to the heap.
+ *       "Primitive" types---these can be represented in a TypedValue without a
+ *       pointer to the heap.
  *
  *   {Init,}Unc
  *
- *       "Uncounted" types---values of these types don't require
- *       reference counting.
+ *       "Uncounted" types---values of these types don't require reference
+ *       counting.
  *
  *   WaitH<T>
  *
- *       A WaitHandle that is known will either return a value of type
- *       T from its join() method (or Await), or else throw an
- *       exception.
+ *       A WaitHandle that is known will either return a value of type T from
+ *       its join() method (or Await), or else throw an exception.
  *
  * Array types:
  *
- *   Arrays are divided along two dimensions: counted or uncounted,
- *   and empty or non-empty.  Unions of either are allowed.  The
- *   naming convention is {S,C,}Arr{N,E,}, where leaving out either
- *   bit means it's unknown along that dimension.  All arrays are
- *   subtypes of the Arr type.  The lattice here looks like this:
+ *   Arrays are divided along two dimensions: counted or uncounted, and empty
+ *   or non-empty.  Unions of either are allowed.  The naming convention is
+ *   {S,C,}Arr{N,E,}, where leaving out either bit means it's unknown along
+ *   that dimension.  All arrays are subtypes of the Arr type.  The lattice
+ *   here looks like this:
  *
  *                         Arr
  *                          |
@@ -122,16 +122,15 @@ struct Type;
  *              | /   \  |     | /   \  |
  *             SArrN  CArrN   CArrE  SArrE
  *
- *   "Specialized" array types may be found as subtypes of any of the
- *   above types except SArrE and CArrE, or of optional versions of
- *   the above types (e.g. as a subtype of ?ArrN).  The information
- *   about additional structure is dispayed in parenthesis, and
- *   probably best explained by some examples:
+ *   "Specialized" array types may be found as subtypes of any of the above
+ *   types except SArrE and CArrE, or of optional versions of the above types
+ *   (e.g. as a subtype of ?ArrN).  The information about additional structure
+ *   is dispayed in parenthesis, and probably best explained by some examples:
  *
  *     SArrN(Bool,Int)
  *
- *         Tuple-like static two-element array, with integer keys 0
- *         and 1, containing a Bool and an Int with unknown values.
+ *         Tuple-like static two-element array, with integer keys 0 and 1,
+ *         containing a Bool and an Int with unknown values.
  *
  *     Arr(Int,Dbl)
  *
@@ -140,39 +139,38 @@ struct Type;
  *
  *     CArrN([Bool])
  *
- *         Non-empty reference counted array with contiguous
- *         zero-based integer keys, unknown size, values all are
- *         subtypes of Bool.
+ *         Non-empty reference counted array with contiguous zero-based integer
+ *         keys, unknown size, values all are subtypes of Bool.
  *
  *     ArrN(x:Int,y:Int)
  *
- *         Struct-like array with known fields "x" and "y" that have
- *         Int values, and no other fields.  Struct-like arrays always
- *         have known string keys, and the type contains only those
- *         array values with exactly the given key set.  The order of
- *         the array elements is not tracked in this type system.
+ *         Struct-like array with known fields "x" and "y" that have Int
+ *         values, and no other fields.  Struct-like arrays always have known
+ *         string keys, and the type contains only those array values with
+ *         exactly the given key set.  The order of the array elements is not
+ *         tracked in this type system.
  *
  *     Arr([SStr:InitCell])
  *
  *         Possibly empty map-like array with unknown keys, but all
- *         non-reference counted strings, all values InitCell.  In
- *         this case the array itself may or may not be static.
+ *         non-reference counted strings, all values InitCell.  In this case
+ *         the array itself may or may not be static.
  *
- *         Note that struct-like arrays will be subtypes of map-like
- *         arrays with string keys.
+ *         Note that struct-like arrays will be subtypes of map-like arrays
+ *         with string keys.
  *
  *     ArrN([Int:InitPrim])
  *
- *         Map-like array with only integer keys (not-necessarily
- *         contiguous) and values that are all subtypes of InitPrim.
- *         Note that the keys *may* be contiguous integers, so for
- *         example Arr([InitPrim]) <: Arr([Int => InitPrim]).
+ *         Map-like array with only integer keys (not-necessarily contiguous)
+ *         and values that are all subtypes of InitPrim.  Note that the keys
+ *         *may* be contiguous integers, so for example Arr([InitPrim]) <:
+ *         Arr([Int => InitPrim]).
  *
  *     Arr([InitCell:InitCell])
  *
- *         Map-like array with either integer or string keys, and
- *         InitCell values, or empty.  Essentially this is the most
- *         generic array that can't contain php references.
+ *         Map-like array with either integer or string keys, and InitCell
+ *         values, or empty.  Essentially this is the most generic array that
+ *         can't contain php references.
  *
  *  TODO(#3774082): we should have a Str|Int type.
  */
@@ -352,6 +350,8 @@ private:
   friend Type wait_handle(const Index&, Type);
   friend bool is_specialized_wait_handle(const Type&);
   friend bool is_specialized_array(const Type&);
+  friend bool is_specialized_obj(const Type&);
+  friend bool is_specialized_cls(const Type&);
   friend bool is_ref_with_inner(const Type&);
   friend Type wait_handle_inner(const Type&);
   friend Type sval(SString);
@@ -379,6 +379,7 @@ private:
   friend DCls dcls_of(Type);
   friend Type union_of(Type, Type);
   friend Type widening_union(const Type&, const Type&);
+  friend Type promote_emptyish(Type, Type);
   friend Type opt(Type);
   friend Type unopt(Type);
   friend bool is_opt(Type);
@@ -638,10 +639,16 @@ Type unopt(Type t);
 bool is_opt(Type t);
 
 /*
- * Returns true if type 't' represents a "specialized" object, that is
- * an object of a known class, or an optional object of a known class.
+ * Returns true if type 't' represents a "specialized" object, that is an
+ * object of a known class, or an optional object of a known class.
  */
-bool is_specialized_obj(Type t);
+bool is_specialized_obj(const Type&);
+
+/*
+ * Returns true if type 't' represents a "specialized" class---i.e. a class
+ * with a DCls structure.
+ */
+bool is_specialized_cls(const Type&);
 
 /*
  * Returns whether `t' is a WaitH<T> or ?WaitH<T> for some T.
@@ -663,7 +670,7 @@ bool is_specialized_array(const Type& t);
  *
  * Pre: t.subtypeOf(TObj)
  */
-Type objcls(Type t);
+Type objcls(const Type& t);
 
 /*
  * If the type t has a known constant value, return it as a Cell.
@@ -691,7 +698,7 @@ DObj dobj_of(const Type& t);
 /*
  * Return the DCls structure for a strict subtype of TCls.
  *
- * Pre: t.strictSubtypeOf(TCls)
+ * Pre: is_specialized_cls(t)
  */
 DCls dcls_of(Type t);
 
@@ -747,6 +754,18 @@ Type union_of(Type a, Type b);
  */
 Type widening_union(const Type& a, const Type& b);
 
+/*
+ * A sort of union operation that also attempts to remove "emptyish" types from
+ * union_of(a, b).  This is useful for promoting emptyish types (sempty(),
+ * false, and null) to stdClass or to arrays, in member instructions.
+ *
+ * This function currently doesn't give specific guarantees about exactly when
+ * the emptyish types will not be part of the return type (informally it only
+ * happens in the "easy" cases right now), so you should not use it in
+ * situations where union_of(a, b) would not also be correct.
+ */
+Type promote_emptyish(Type a, Type b);
+
 /*
  * Returns the smallest type that `a' is a subtype of, from the
  * following set: TGen, TInitCell, TRef, TUninit, TCls.
@@ -831,6 +850,7 @@ std::pair<Type,Type> iter_types(const Type&);
  * sure these things are merged into the appropriate unit or repo.
  *
  * Pre: !t.couldBe(TCls)
+ *      !t.subtypeOf(TBottom)
  */
 RepoAuthType make_repo_type(ArrayTypeTable::Builder&, const Type& t);
 
