@@ -8,6 +8,10 @@
 #define FALSE 0
 #endif
 
+#ifdef _MSC_VER
+typedef int BOOL;
+#else
 typedef char BOOL;
+#endif
 
 #endif /* incl_HPHP_NEO_BOOL_H_ */
