@@ -12,7 +12,6 @@
 #include "cs_config.h"
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -20,6 +19,16 @@
 #include <limits.h>
 #include <stdarg.h>
 #include <sys/stat.h>
+
+#ifdef _MSC_VER
+#include <windows.h>
+#include <direct.h>
+#include <io.h>
+#define PATH_MAX MAX_PATH
+#else
+#include <unistd.h>
+#endif
+
 #include "neo_misc.h"
 #include "neo_err.h"
 #include "neo_rand.h"
@@ -318,90 +327,6 @@ static int _walk_hdf (HDF *hdf, const char *name, HDF **node)
   return 0;
 }
 
-int hdf_get_int_value (HDF *hdf, const char *name, int defval)
-{
-  HDF *node;
-  int v;
-  char *n;
-
-  if ((_walk_hdf(hdf, name, &node) == 0) && (node->value != NULL))
-  {
-    v = strtol (node->value, &n, 10);
-    if (node->value == n) v = defval;
-    return v;
-  }
-  return defval;
-}
-
-/* This should return a const char *, but changing this would have big
- * repurcussions for any C code using this function, so no change for now */
-char* hdf_get_value (HDF *hdf, const char *name, const char *defval)
-{
-  HDF *node;
-
-  if ((_walk_hdf(hdf, name, &node) == 0) && (node->value != NULL))
-  {
-    return node->value;
-  }
-  return (char *)defval;
-}
-
-char* hdf_get_valuevf (HDF *hdf, const char *namefmt, va_list ap)
-{
-  HDF *node;
-  char *name;
-
-  name = vsprintf_alloc(namefmt, ap);
-  if (name == NULL) return NULL;
-  if ((_walk_hdf(hdf, name, &node) == 0) && (node->value != NULL))
-  {
-    free(name);
-    return node->value;
-  }
-  free(name);
-  return NULL;
-}
-
-char* hdf_get_valuef (HDF *hdf, const char *namefmt, ...)
-{
-  char *val;
-  va_list ap;
-
-  va_start(ap, namefmt);
-  val = hdf_get_valuevf(hdf, namefmt, ap);
-  va_end(ap);
-  return val;
-}
-
-NEOERR* hdf_get_copy (HDF *hdf, const char *name, char **value,
-                      const char *defval)
-{
-  HDF *node;
-
-  if ((_walk_hdf(hdf, name, &node) == 0) && (node->value != NULL))
-  {
-    *value = strdup(node->value);
-    if (*value == NULL)
-    {
-      return nerr_raise (NERR_NOMEM, "Unable to allocate copy of %s", name);
-    }
-  }
-  else
-  {
-    if (defval == NULL)
-      *value = NULL;
-    else
-    {
-      *value = strdup(defval);
-      if (*value == NULL)
-      {
-	return nerr_raise (NERR_NOMEM, "Unable to allocate copy of %s", name);
-      }
-    }
-  }
-  return STATUS_OK;
-}
-
 HDF* hdf_get_obj (HDF *hdf, const char *name)
 {
   HDF *obj;
@@ -418,73 +343,6 @@ HDF* hdf_get_child (HDF *hdf, const char *name)
   return obj;
 }
 
-HDF_ATTR* hdf_get_attr (HDF *hdf, const char *name)
-{
-  HDF *obj;
-  _walk_hdf(hdf, name, &obj);
-  if (obj != NULL) return obj->attr;
-  return NULL;
-}
-
-NEOERR* hdf_set_attr (HDF *hdf, const char *name, const char *key,
-                      const char *value)
-{
-  HDF *obj;
-  HDF_ATTR *attr, *last;
-
-  _walk_hdf(hdf, name, &obj);
-  if (obj == NULL)
-    return nerr_raise(NERR_ASSERT, "Unable to set attribute on none existant node");
-
-  if (obj->attr != NULL)
-  {
-    attr = obj->attr;
-    last = attr;
-    while (attr != NULL)
-    {
-      if (!strcmp(attr->key, key))
-      {
-	if (attr->value) free(attr->value);
-	/* a set of NULL deletes the attr */
-	if (value == NULL)
-	{
-	  if (attr == obj->attr)
-	    obj->attr = attr->next;
-	  else
-	    last->next = attr->next;
-	  free(attr->key);
-	  free(attr);
-	  return STATUS_OK;
-	}
-	attr->value = strdup(value);
-	if (attr->value == NULL)
-	  return nerr_raise(NERR_NOMEM, "Unable to set attr %s to %s", key, value);
-	return STATUS_OK;
-      }
-      last = attr;
-      attr = attr->next;
-    }
-    last->next = (HDF_ATTR *) calloc(1, sizeof(HDF_ATTR));
-    if (last->next == NULL)
-      return nerr_raise(NERR_NOMEM, "Unable to set attr %s to %s", key, value);
-    attr = last->next;
-  }
-  else
-  {
-    if (value == NULL) return STATUS_OK;
-    obj->attr = (HDF_ATTR *) calloc(1, sizeof(HDF_ATTR));
-    if (obj->attr == NULL)
-      return nerr_raise(NERR_NOMEM, "Unable to set attr %s to %s", key, value);
-    attr = obj->attr;
-  }
-  attr->key = strdup(key);
-  attr->value = strdup(value);
-  if (attr->key == NULL || attr->value == NULL)
-    return nerr_raise(NERR_NOMEM, "Unable to set attr %s to %s", key, value);
-
-  return STATUS_OK;
-}
-
 void hdf_set_visited (HDF *hdf, int visited) {
   if (hdf) hdf->visited = visited;
 }
@@ -512,18 +370,6 @@ HDF* hdf_obj_next (HDF *hdf)
   return hdf->next;
 }
 
-HDF* hdf_obj_top (HDF *hdf)
-{
-  if (hdf == NULL) return NULL;
-  return hdf->top;
-}
-
-HDF_ATTR* hdf_obj_attr (HDF *hdf)
-{
-  if (hdf == NULL) return NULL;
-  return hdf->attr;
-}
-
 char* hdf_obj_name (HDF *hdf)
 {
   if (hdf == NULL) return NULL;
@@ -868,69 +714,6 @@ NEOERR* hdf_set_value_attr (HDF *hdf, const char *name, const char *value,
   return nerr_pass(_set_value (hdf, name, value, 1, 1, 0, attr, NULL));
 }
 
-NEOERR* hdf_set_symlink (HDF *hdf, const char *src, const char *dest)
-{
-  return nerr_pass(_set_value (hdf, src, dest, 1, 1, 1, NULL, NULL));
-}
-
-NEOERR* hdf_set_int_value (HDF *hdf, const char *name, int value)
-{
-  char buf[256];
-
-  snprintf (buf, sizeof(buf), "%d", value);
-  return nerr_pass(_set_value (hdf, name, buf, 1, 1, 0, NULL, NULL));
-}
-
-NEOERR* hdf_set_buf (HDF *hdf, const char *name, char *value)
-{
-  return nerr_pass(_set_value (hdf, name, value, 0, 1, 0, NULL, NULL));
-}
-
-NEOERR* hdf_set_copy (HDF *hdf, const char *dest, const char *src)
-{
-  HDF *node;
-  if ((_walk_hdf(hdf, src, &node) == 0) && (node->value != NULL))
-  {
-    return nerr_pass(_set_value (hdf, dest, node->value, 0, 0, 0, NULL, NULL));
-  }
-  return nerr_raise (NERR_NOT_FOUND, "Unable to find %s", src);
-}
-
-NEOERR* hdf_set_valuevf (HDF *hdf, const char *fmt, va_list ap)
-{
-  NEOERR *err;
-  char *k;
-  char *v;
-
-  k = vsprintf_alloc(fmt, ap);
-  if (k == NULL)
-  {
-    return nerr_raise(NERR_NOMEM, "Unable to allocate memory for format string");
-  }
-  v = strchr(k, '=');
-  if (v == NULL)
-  {
-    err = nerr_raise(NERR_ASSERT, "No equals found: %s", k);
-    free(k);
-    return err;
-  }
-  *v++ = '\0';
-  err = hdf_set_value(hdf, k, v);
-  free(k);
-  return nerr_pass(err);
-}
-
-NEOERR* hdf_set_valuef (HDF *hdf, const char *fmt, ...)
-{
-  NEOERR *err;
-  va_list ap;
-
-  va_start(ap, fmt);
-  err = hdf_set_valuevf(hdf, fmt, ap);
-  va_end(ap);
-  return nerr_pass(err);
-}
-
 NEOERR* hdf_get_node (HDF *hdf, const char *name, HDF **ret)
 {
   _walk_hdf(hdf, name, ret);
@@ -941,44 +724,6 @@ NEOERR* hdf_get_node (HDF *hdf, const char *name, HDF **ret)
   return STATUS_OK;
 }
 
-/* Ok, this version avoids the bubble sort by walking the level once to
- * load them all into a ULIST, qsort'ing the list, and then dumping them
- * back out... */
-NEOERR *hdf_sort_obj (HDF *h, int (*compareFunc)(const void *, const void *))
-{
-  NEOERR *err = STATUS_OK;
-  ULIST *level = NULL;
-  HDF *p, *c;
-  int x;
-
-  if (h == NULL) return STATUS_OK;
-  c = h->child;
-  if (c == NULL) return STATUS_OK;
-
-  do {
-    err = uListInit(&level, 40, 0);
-    if (err) return nerr_pass(err);
-    for (p = c; p; p = p->next) {
-      err = uListAppend(level, p);
-      if (err) break;
-    }
-    err = uListSort(level, compareFunc);
-    if (err) break;
-    uListGet(level, 0, (void *)&c);
-    h->child = c;
-    for (x = 1; x < uListLength(level); x++)
-    {
-      uListGet(level, x, (void *)&p);
-      c->next = p;
-      p->next = NULL;
-      c = p;
-    }
-    h->last_child = c;
-  } while (0);
-  uListDestroy(&level, 0);
-  return nerr_pass(err);
-}
-
 NEOERR* hdf_remove_tree (HDF *hdf, const char *name)
 {
   HDF *hp = hdf;
@@ -1306,11 +1051,6 @@ NEOERR* hdf_dump_str (HDF *hdf, const char *prefix, int dtype, NEOSTRING *str)
   return nerr_pass(hdf_dump_cb(hdf, prefix, dtype, 0, str, _string_dump_cb));
 }
 
-NEOERR* hdf_dump(HDF *hdf, const char *prefix)
-{
-  return nerr_pass(hdf_dump_cb(hdf, prefix, DUMP_TYPE_DOTTED, 0, stdout, _fp_dump_cb));
-}
-
 NEOERR* hdf_dump_format (HDF *hdf, int lvl, FILE *fp)
 {
   return nerr_pass(hdf_dump_cb(hdf, "", DUMP_TYPE_PRETTY, 0, fp, _fp_dump_cb));
@@ -1335,38 +1075,6 @@ NEOERR *hdf_write_file (HDF *hdf, const char *path)
   return nerr_pass(err);
 }
 
-NEOERR *hdf_write_file_atomic (HDF *hdf, const char *path)
-{
-  NEOERR *err;
-  FILE *fp;
-  char tpath[PATH_BUF_SIZE];
-  static int count = 0;
-
-  snprintf(tpath, sizeof(tpath), "%s.%5.5f.%d", path, ne_timef(), count++);
-
-  fp = fopen(tpath, "w");
-  if (fp == NULL)
-    return nerr_raise_errno (NERR_IO, "Unable to open %s for writing", tpath);
-
-  err = hdf_dump_format (hdf, 0, fp);
-
-  fclose (fp);
-
-  if (err)
-  {
-    unlink(tpath);
-    return nerr_pass(err);
-  }
-  if (rename(tpath, path) == -1)
-  {
-    unlink (tpath);
-    return nerr_raise_errno (NERR_IO, "Unable to rename file %s to %s",
-	tpath, path);
-  }
-
-  return STATUS_OK;
-}
-
 NEOERR *hdf_write_string (HDF *hdf, char **s)
 {
   NEOSTRING str;
@@ -1776,7 +1484,11 @@ static NEOERR* _hdf_read_string (HDF *hdf, const char **str, NEOSTRING *line,
 	s+=2;
 	value = neos_strip(s);
 
+#ifdef _MSC_VER
+        FILE *f = _popen(value, "r");
+#else
         FILE *f = popen(value, "r");
+#endif
 	if (f == NULL)
         {
 	  err = nerr_raise(NERR_PARSE,
@@ -1919,18 +1631,6 @@ NEOERR * hdf_read_string (HDF *hdf, const char *str)
   return nerr_pass(err);
 }
 
-NEOERR * hdf_read_string_ignore (HDF *hdf, const char *str, int ignore)
-{
-  NEOERR *err;
-  int lineno = 0;
-  NEOSTRING line;
-  string_init(&line);
-  err = _hdf_read_string(hdf, &str, &line, "<string>", &lineno,
-                         (ignore ? INCLUDE_IGNORE : INCLUDE_ERROR), 0);
-  string_clear(&line);
-  return nerr_pass(err);
-}
-
 /* The search path is part of the HDF by convention */
 NEOERR* hdf_search_path (HDF *hdf, const char *path, char *full, int full_len)
 {
@@ -1973,29 +1673,20 @@ static NEOERR* hdf_read_file_internal (HDF *hdf, const char *path,
   char fpath[PATH_BUF_SIZE];
   char *ibuf = NULL;
   const char *ptr = NULL;
-  HDF *top = hdf->top;
   NEOSTRING line;
 
   string_init(&line);
 
   if (path == NULL)
     return nerr_raise(NERR_ASSERT, "Can't read NULL file");
-
-  if (top->fileload)
+  if (path[0] != '/')
   {
-    err = top->fileload(top->fileload_ctx, hdf, path, &ibuf);
+    err = hdf_search_path (hdf, path, fpath, PATH_BUF_SIZE);
+    if (err != STATUS_OK) return nerr_pass(err);
+    path = fpath;
   }
-  else
-  {
-    if (path[0] != '/')
-    {
-      err = hdf_search_path (hdf, path, fpath, PATH_BUF_SIZE);
-      if (err != STATUS_OK) return nerr_pass(err);
-      path = fpath;
-    }
 
-    err = ne_load_file (path, &ibuf);
-  }
+  err = ne_load_file (path, &ibuf);
   if (err) return nerr_pass(err);
 
   ptr = ibuf;
@@ -2012,11 +1703,3 @@ NEOERR* hdf_read_file (HDF *hdf, const char *path)
   return nerr_pass(err);
 }
 
-void hdf_register_fileload(HDF *hdf, void *ctx, HDFFILELOAD fileload)
-{
-  if (hdf == NULL) return;
-  if (hdf->top != NULL) hdf = hdf->top;
-  hdf->fileload_ctx = ctx;
-  hdf->fileload = fileload;
-}
-
