@@ -1,5 +1,6 @@
 #!/bin/sh
 
+unset CDPATH
 DIR="$( cd "$( dirname "$0" )" && pwd )"
 
 if [ -z "${INSTALL_DIR}" ]; then
@@ -15,9 +16,7 @@ else
   INFILE=hphp.ll
   OUTFILE=${INSTALL_DIR}/lex.yy.cpp
 
-  EXTERNAL_TOOLS_ROOT=`readlink -f ${FBCODE_DIR}/third-party/centos5.2-native/`
-  FLEX_DIR=${EXTERNAL_TOOLS_ROOT}/flex/flex-2.5.35/
-  FLEX=${FLEX_DIR}/bin/flex
+  FLEX=$(readlink -f $(ls -t ${FBCODE_DIR}/third-party2/flex/2.5.35/centos5.2-native/*/bin/flex | head -1))
 fi
 
 $FLEX -i -f -Phphp -R -8 --bison-locations -o${OUTFILE} ${INFILE}
@@ -25,18 +24,19 @@ if [ $? -ne 0 ] ; then
   exit 1
 fi
 
+SED=`which sed`
+if [ ! -x "$SED" ]; then
+  echo "sed not found" 1>&2
+  exit 1
+fi
+
+$SED -i \
+  -e "1i// @""generated" \
+  -e "s@/.*lex.yy.cpp@lex.yy.cpp@g" \
+  -e "s@/.*hphp.ll@hphp.ll@g" \
+  $OUTFILE
+
 # We still want the files in our tree since they are checked in
 if [ -n "${INSTALL_DIR}" ]; then
-
-  SED=`which sed`
-  if [ ! -x "$SED" ]; then
-    echo "sed not found" 1>&2
-    exit 1
-  fi
-  
-  $SED -i \
-    -e "1i// @""generated" \
-    -e "s@/.*lex.yy.cpp@lex.yy.cpp@" \
-    $OUTFILE
   cp $OUTFILE ${DIR}/lex.yy.cpp
 fi
