@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,27 +17,26 @@
 
 #include <vector>
 #include <array>
-#include <boost/lexical_cast.hpp>
 #include <tbb/concurrent_hash_map.h>
 
 #include "hphp/util/exception.h"
 #include "hphp/runtime/base/array-init.h"
 #include "hphp/runtime/base/empty-array.h"
 #include "hphp/runtime/base/packed-array.h"
+#include "hphp/runtime/base/struct-array.h"
 #include "hphp/runtime/base/array-common.h"
 #include "hphp/runtime/base/array-iterator.h"
 #include "hphp/runtime/base/type-conversions.h"
 #include "hphp/runtime/base/builtin-functions.h"
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/variable-serializer.h"
 #include "hphp/runtime/base/runtime-option.h"
-#include "hphp/runtime/base/macros.h"
 #include "hphp/runtime/base/apc-local-array.h"
 #include "hphp/runtime/base/comparisons.h"
-#include "hphp/runtime/vm/name-value-table-wrapper.h"
+#include "hphp/runtime/vm/globals-array.h"
 #include "hphp/runtime/base/proxy-array.h"
 #include "hphp/runtime/base/thread-info.h"
 #include "hphp/runtime/base/mixed-array.h"
+#include "hphp/zend/zend-string.h"
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -47,29 +46,43 @@ static_assert(
   "Performance is sensitive to sizeof(ArrayData)."
   " Make sure you changed it with good reason and then update this assert.");
 
-typedef tbb::concurrent_hash_map<std::string, ArrayData*> ArrayDataMap;
+using ArrayDataMap = tbb::concurrent_hash_map<ArrayData::ScalarArrayKey,
+                                              ArrayData*,
+                                              ArrayData::ScalarHash>;
 static ArrayDataMap s_arrayDataMap;
 
+ArrayData::ScalarArrayKey ArrayData::GetScalarArrayKey(const char* str,
+                                                       size_t sz) {
+  return MD5(string_md5(str, sz).c_str());
+}
+
+ArrayData::ScalarArrayKey ArrayData::GetScalarArrayKey(ArrayData* arr) {
+  VariableSerializer vs(VariableSerializer::Type::Serialize);
+  auto s = vs.serializeValue(VarNR(arr), false /* limit */);
+  return GetScalarArrayKey(s.data(), s.size());
+}
+
 ArrayData* ArrayData::GetScalarArray(ArrayData* arr) {
   if (arr->empty()) return staticEmptyArray();
-  auto key = f_serialize(arr).toCppString();
+  auto key = GetScalarArrayKey(arr);
   return GetScalarArray(arr, key);
 }
 
-ArrayData* ArrayData::GetScalarArray(ArrayData* arr, const std::string& key) {
+ArrayData* ArrayData::GetScalarArray(ArrayData* arr,
+                                     const ScalarArrayKey& key) {
   if (arr->empty()) return staticEmptyArray();
-  assert(key == f_serialize(arr).toCppString());
+  assert(key == GetScalarArrayKey(arr));
 
   ArrayDataMap::accessor acc;
   if (s_arrayDataMap.insert(acc, key)) {
     ArrayData* ad;
 
     if (arr->isVectorData() && !arr->isPacked()) {
-      ad = PackedArray::NonSmartConvert(arr);
+      ad = PackedArray::ConvertStatic(arr);
     } else {
-      ad = arr->nonSmartCopy();
+      ad = arr->copyStatic();
     }
-    ad->setStatic();
+    assert(ad->isStatic());
     ad->onSetEvalScalar();
     acc->second = ad;
   }
@@ -79,37 +92,27 @@ ArrayData* ArrayData::GetScalarArray(ArrayData* arr, const std::string& key) {
 //////////////////////////////////////////////////////////////////////
 
 static ArrayData* ZSetIntThrow(ArrayData* ad, int64_t k, RefData* v) {
-  throw FatalErrorException("Unimplemented ArrayData::ZSetInt");
+  raise_fatal_error("Unimplemented ArrayData::ZSetInt");
 }
 
 static ArrayData* ZSetStrThrow(ArrayData* ad, StringData* k, RefData* v) {
-  throw FatalErrorException("Unimplemented ArrayData::ZSetStr");
+  raise_fatal_error("Unimplemented ArrayData::ZSetStr");
 }
 
 static ArrayData* ZAppendThrow(ArrayData* ad, RefData* v, int64_t* key_ptr) {
-  throw FatalErrorException("Unimplemented ArrayData::ZAppend");
+  raise_fatal_error("Unimplemented ArrayData::ZAppend");
 }
 
 //////////////////////////////////////////////////////////////////////
 
 #define DISPATCH(entry)                         \
   { PackedArray::entry,                         \
+    StructArray::entry,                         \
     MixedArray::entry,                          \
-    APCLocalArray::entry,                       \
     EmptyArray::entry,                          \
-    NameValueTableWrapper::entry,               \
-    ProxyArray::entry,                          \
-    MixedArray::entry                           \
-  },
-
-#define DISPATCH_INTMAP_SPECIALIZED(entry)      \
-  { PackedArray::entry,                         \
-    MixedArray::entry,                          \
     APCLocalArray::entry,                       \
-    EmptyArray::entry,                          \
-    NameValueTableWrapper::entry,               \
-    ProxyArray::entry,                          \
-    MixedArray::entry##Impl<ArrayData::kIntMapKind>   \
+    GlobalsArray::entry,                        \
+    ProxyArray::entry                           \
   },
 
 /*
@@ -137,15 +140,13 @@ static ArrayData* ZAppendThrow(ArrayData* ad, RefData* v, int64_t* key_ptr) {
  *   array doesn't have to copy if it was asked to remove an element
  *   that doesn't exist.
  *
- *   When a function with these semantics returns a new array, the new
- *   array is not yet incref'd (for historical reasons relating to our
- *   smart pointers).  Correctly using functions with these semantics
- *   usually involves checking whether the return value is the same
- *   pointer to be able to conditionally incref it.  TODO(#2926276):
- *   we want to change this to make callsites cheaper.
+ *   When a function with these semantics returns a new array, the new array is
+ *   already incref'd. In a few cases, an existing array (different than the
+ *   source array) may be returned. In this case, the array will already be
+ *   incref'd.
  */
 
-extern const ArrayFunctions g_array_funcs = {
+extern const ArrayFunctions g_array_funcs_unmodified = {
   /*
    * void Release(ArrayData*)
    *
@@ -169,7 +170,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   must not be an integer-like string.  Returns nullptr if the key
    *   is not in the array.
    */
-  DISPATCH_INTMAP_SPECIALIZED(NvGetStr)
+  DISPATCH(NvGetStr)
 
   /*
    * void NvGetKey(const ArrayData*, TypedValue* out, ssize_t pos)
@@ -194,20 +195,20 @@ extern const ArrayFunctions g_array_funcs = {
    *   be an integer-like string.  This function has copy/grow
    *   semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(SetStr)
+  DISPATCH(SetStr)
 
   /*
    * size_t Vsize(const ArrayData*)
    *
-   *   This entry point essentially is only for NameValueTableWrapper;
+   *   This entry point essentially is only for GlobalsArray and ProxyArray;
    *   all the other cases are not_reached().
    *
-   *   Because of particulars of how NameValueTableWrapper works,
+   *   Because of particulars of how GlobalsArray works,
    *   determining the size of the array is an O(N) operation---we set
    *   the size field in the generic ArrayData header to -1 in that
    *   case and dispatch through this entry point.  ProxyArray also
    *   always involves virtual size, because of the possibility that
-   *   it could be proxying a NameValueTableWrapper.
+   *   it could be proxying a GlobalsArray.
    */
   DISPATCH(Vsize)
 
@@ -243,7 +244,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   supplied string key.  The string must not be an integer-like
    *   string.
    */
-  DISPATCH_INTMAP_SPECIALIZED(ExistsStr)
+  DISPATCH(ExistsStr)
 
   /*
    * ArrayData* LvalInt(ArrayData*, int64_t k, Variant*& out, bool copy)
@@ -262,7 +263,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   to point to it.  The string `key' may not be an integer-like
    *   string.  This function has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(LvalStr)
+  DISPATCH(LvalStr)
 
   /*
    * ArrayData* LvalNew(ArrayData*, Variant*& out, bool copy)
@@ -273,7 +274,20 @@ extern const ArrayFunctions g_array_funcs = {
    *   function sets out to point to the lvalBlackHole.  This function
    *   has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(LvalNew)
+  DISPATCH(LvalNew)
+
+  /*
+   * ArrayData* LvalNewRef(ArrayData*, Variant*& out, bool copy)
+   */
+  {
+    PackedArray::LvalNewRef,
+    StructArray::LvalNew,
+    MixedArray::LvalNew,
+    EmptyArray::LvalNew,
+    APCLocalArray::LvalNew,
+    GlobalsArray::LvalNew,
+    ProxyArray::LvalNew,
+  },
 
   /*
    * ArrayData* SetRefInt(ArrayData*, int64_t key, Variant& v, bool copy)
@@ -282,7 +296,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   boxed, and then insert a KindOfRef that points to v's RefData.
    *   This function has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(SetRefInt)
+  DISPATCH(SetRefInt)
 
   /*
    * ArrayData* SetRefStr(ArrayData*, StringData* key, Variant& v, bool copy)
@@ -292,7 +306,7 @@ extern const ArrayFunctions g_array_funcs = {
    *  then insert a KindOfRef that points to v's RefData.  This
    *  function has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(SetRefStr)
+  DISPATCH(SetRefStr)
 
   /*
    * ArrayData* AddInt(ArrayData*, int64_t key, Cell, bool copy)
@@ -304,7 +318,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   operation more efficient.
    */
   DISPATCH(SetInt)
-  DISPATCH_INTMAP_SPECIALIZED(SetStr)
+  DISPATCH(SetStr)
 
   /*
    * ArrayData* RemoveInt(ArrayData*, int64_t key, bool copy)
@@ -322,38 +336,46 @@ extern const ArrayFunctions g_array_funcs = {
    *   entry for that element, this function does not remove it, and
    *   may or may not cow.  This function has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(RemoveStr)
+  DISPATCH(RemoveStr)
 
   /*
-   * ssize_t IterBegin(const ArrayData*)
    * ssize_t IterEnd(const ArrayData*)
    *
-   *   Array positions are represented as an opaque ssize_t.
-   *   IterBegin returns the position of the first element in the
-   *   array, and IterEnd returns the position of the last element in
-   *   the array.  Either function may return ArrayData::invalid_index
-   *   if there is no first or last element.
+   *   Returns the canonical invalid position for this array.  Note
+   *   that if elements are added or removed from the array, the value
+   *   of the array's canonical invalid position may change.
+   *
+   * ssize_t IterBegin(const ArrayData*)
+   *
+   *   Returns the position of the first element, or the canonical
+   *   invalid position if this array is empty.
+   *
+   * ssize_t IterLast(const ArrayData*)
+   *
+   *   Returns the position of the last element, or the canonical
+   *   invalid position if this array is empty.
    */
   DISPATCH(IterBegin)
+  DISPATCH(IterLast)
   DISPATCH(IterEnd)
 
   /*
-   * ssize_t IterAdvance(const ArrayData*, ssize_t pos)
+   * ssize_t IterAdvance(const ArrayData*, size_t pos)
    *
-   *   Advance `pos' to the next position in the array.  `pos' may be
-   *   invalid_index, in which case this function returns the position
-   *   of the first element in the array.  Returns invalid_index if
-   *   there is no next position in the array.
+   *   Returns the position of the element that comes after pos, or the
+   *   canonical invalid position if there are no more elements after pos.
+   *   If pos is the canonical invalid position, this method will return
+   *   the canonical invalid position.
    */
   DISPATCH(IterAdvance)
 
   /*
-   * ssize_t IterRewind(const ArrayData*, ssize_t pos)
+   * ssize_t IterRewind(const ArrayData*, size_t pos)
    *
-   *   Move `pos' to the position of the previous element in the
-   *   array.  `pos' may be invalid_index, in which case this function
-   *   returns invalid_index.  Returns invalid_index if there is no
-   *   previous position in the array.
+   *   Returns the position of the element that comes before pos, or the
+   *   canonical invalid position if there are no elements before pos. If
+   *   pos is the canonical invalid position, no guarantees are made about
+   *   what this method returns.
    */
   DISPATCH(IterRewind)
 
@@ -380,13 +402,13 @@ extern const ArrayFunctions g_array_funcs = {
    *
    *   Pre: fp.getContainer() == ad
    */
-  DISPATCH_INTMAP_SPECIALIZED(AdvanceMArrayIter)
+  DISPATCH(AdvanceMArrayIter)
 
   /*
-   * ArrayData* EscalateForSort(ArrayData*)
+   * ArrayData* EscalateForSort(ArrayData*, SortFunction)
    *
    *   Must be called before calling any of the sort routines on an
-   *   array.  This gives arrays a chance to change to a kind that
+   *   array. This gives arrays a chance to change to a kind that
    *   supports sorting.
    */
   DISPATCH(EscalateForSort)
@@ -450,7 +472,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   Explicitly request that an array be copied.  This API does
    *   /not/ actually guarantee a copy occurs.
    *
-   *   (E.g. NameValueTableWrapper doesn't copy here.)
+   *   (E.g. GlobalsArray doesn't copy here.)
    */
   DISPATCH(Copy)
 
@@ -465,14 +487,14 @@ extern const ArrayFunctions g_array_funcs = {
   DISPATCH(CopyWithStrongIterators)
 
   /*
-   * ArrayData* NonSmartCopy(const ArrayData*)
+   * ArrayData* CopyStatic(const ArrayData*)
    *
    *   Copy an array, allocating the new array with malloc() instead
    *   of from the request local allocator.  This function does
    *   guarantee the returned array is a new copy---but it may throw a
    *   fatal error if this cannot be accomplished (e.g. for $GLOBALS).
    */
-  DISPATCH(NonSmartCopy)
+  DISPATCH(CopyStatic)
 
   /*
    * ArrayData* Append(ArrayData*, const Variant& v, bool copy)
@@ -481,7 +503,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   integer key.  If there is no next available integer key, no
    *   value is appended.  This function has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(Append)
+  DISPATCH(Append)
 
   /*
    * ArrayData* AppendRef(ArrayData*, Variant& v, bool copy)
@@ -492,7 +514,7 @@ extern const ArrayFunctions g_array_funcs = {
    *   If there is no next available integer key, this function does
    *   not append a value.  This function has copy/grow semantics.
    */
-  DISPATCH_INTMAP_SPECIALIZED(AppendRef)
+  DISPATCH(AppendRef)
 
   /*
    * ArrayData* AppendWithRef(ArrayData*, const Variant& v, bool copy)
@@ -504,16 +526,14 @@ extern const ArrayFunctions g_array_funcs = {
    *   referenced"---i.e. if `v' is already KindOfRef and
    *   RefData::isReferenced is true.
    */
-  DISPATCH_INTMAP_SPECIALIZED(AppendWithRef)
+  DISPATCH(AppendWithRef)
 
   /*
    * ArrayData* PlusEq(ArrayData*, const ArrayData* elems)
    *
    *    Performs array addition, logically mutating the first array.
    *    It may return a new array if the array needed to grow, or if
-   *    it needed to COW because hasMultipleRefs was true---in this
-   *    case the new returned array will already have a reference
-   *    count of 1.
+   *    it needed to COW because cowCheck() was true.
    */
   DISPATCH(PlusEq)
 
@@ -522,28 +542,26 @@ extern const ArrayFunctions g_array_funcs = {
    *
    *   Perform part of the semantics of the php function array_merge.
    *   (Renumbering keys is not done by this routine currently.)
-   *
-   *   This function always produces a new array with reference count 1.
    */
   DISPATCH(Merge)
 
   /*
    * ArrayData* Pop(ArrayData*, Variant& value);
    *
-   *   Remove the last element from the array and assign it to
-   *   `value'.  This function may return a new (not yet incref'd)
-   *   array if it decided to COW due to hasMultipleRefs().
+   *   Remove the last element from the array and assign it to `value'.  This
+   *   function may return a new array if it decided to COW due to
+   *   cowCheck().
    */
-  DISPATCH_INTMAP_SPECIALIZED(Pop)
+  DISPATCH(Pop)
 
   /*
    * ArrayData* Dequeue(ArrayData*, Variant& value)
    *
-   *   Remove the first element from the array and assign it to
-   *   `value'.  This function may return a new (not yet incref'd)
-   *   array if it decided to COW due to hasMultipleRefs().
+   *   Remove the first element from the array and assign it to `value'.  This
+   *   function may return a new array if it decided to COW due to
+   *   cowCheck().
    */
-  DISPATCH_INTMAP_SPECIALIZED(Dequeue)
+  DISPATCH(Dequeue)
 
   /*
    * ArrayData* Prepend(ArrayData*, const Variant& `v', bool copy)
@@ -558,7 +576,7 @@ extern const ArrayFunctions g_array_funcs = {
    *
    *   Renumber integer keys on the array in place.
    */
-  DISPATCH_INTMAP_SPECIALIZED(Renumber)
+  DISPATCH(Renumber)
 
   /*
    * void OnSetEvalScalar(ArrayData*)
@@ -587,31 +605,42 @@ extern const ArrayFunctions g_array_funcs = {
    *   These functions are part of the zend compat layer but their
    *   effects currently aren't documented.
    */
-  { &PackedArray::ZSetInt,
+  {
+    &PackedArray::ZSetInt,
+    &StructArray::ZSetInt,
     &MixedArray::ZSetInt,
     &ZSetIntThrow,
     &ZSetIntThrow,
     &ZSetIntThrow,
     &ProxyArray::ZSetInt,
-    &MixedArray::ZSetInt },
-  { &PackedArray::ZSetStr,
+  },
+
+  {
+    &PackedArray::ZSetStr,
+    &StructArray::ZSetStr,
     &MixedArray::ZSetStr,
     &ZSetStrThrow,
     &ZSetStrThrow,
     &ZSetStrThrow,
     &ProxyArray::ZSetStr,
-    &MixedArray::ZSetStr },
-  { &PackedArray::ZAppend,
+  },
+
+  {
+    &PackedArray::ZAppend,
+    &StructArray::ZAppend,
     &MixedArray::ZAppend,
     &ZAppendThrow,
     &ZAppendThrow,
     &ZAppendThrow,
     &ProxyArray::ZAppend,
-    &MixedArray::ZAppend },
+  },
 };
 
+// We create a copy so that we can install instrumentation shim-functions
+// instrument g_array_funcs at runtime.
+ArrayFunctions g_array_funcs = g_array_funcs_unmodified;
+
 #undef DISPATCH
-#undef DISPATCH_INTMAP_SPECIALIZED
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -640,8 +669,12 @@ ArrayData *ArrayData::Create(const Variant& value) {
 
 ArrayData *ArrayData::Create(const Variant& name, const Variant& value) {
   ArrayInit init(1, ArrayInit::Map{});
-  // There is no toKey() call on name.
-  init.set(name, value);
+  DEBUG_ONLY int64_t unused;
+  assertx(name.isString() ?
+         !name.getStringData()->isStrictlyInteger(unused) :
+         name.isInteger());
+
+  init.setValidKey(name, value);
   return init.create();
 }
 
@@ -653,7 +686,11 @@ ArrayData *ArrayData::CreateRef(Variant& value) {
 
 ArrayData *ArrayData::CreateRef(const Variant& name, Variant& value) {
   ArrayInit init(1, ArrayInit::Map{});
-  // There is no toKey() call on name.
+  DEBUG_ONLY int64_t unused;
+  assertx(name.isString() ?
+         !name.getStringData()->isStrictlyInteger(unused) :
+         name.isInteger());
+
   init.setRef(name, value, true);
   return init.create();
 }
@@ -670,16 +707,16 @@ int ArrayData::compare(const ArrayData *v2) const {
   if (count1 > count2) return 1;
   if (count1 == 0) return 0;
 
-  // prevent circular referenced objects/arrays or deep ones
-  DECLARE_THREAD_INFO; check_recursion(info);
+  // Prevent circular referenced objects/arrays or deep ones.
+  check_recursion_error();
 
   for (ArrayIter iter(this); iter; ++iter) {
     auto key = iter.first();
     if (!v2->exists(key)) return 1;
     auto value1 = iter.second();
     auto value2 = v2->get(key);
-    if (HPHP::more(value1, value2)) return 1;
-    if (HPHP::less(value1, value2)) return -1;
+    auto cmp = HPHP::compare(value1, value2);
+    if (cmp != 0) return cmp;
   }
 
   return 0;
@@ -694,8 +731,8 @@ bool ArrayData::equal(const ArrayData *v2, bool strict) const {
   if (count1 != count2) return false;
   if (count1 == 0) return true;
 
-  // prevent circular referenced objects/arrays or deep ones
-  DECLARE_THREAD_INFO; check_recursion(info);
+  // Prevent circular referenced objects/arrays or deep ones.
+  check_recursion_error();
 
   if (strict) {
     for (ArrayIter iter1(this), iter2(v2); iter1; ++iter1, ++iter2) {
@@ -719,23 +756,24 @@ bool ArrayData::equal(const ArrayData *v2, bool strict) const {
 
 Variant ArrayData::reset() {
   setPosition(iter_begin());
-  return m_pos != invalid_index ? getValue(m_pos) : Variant(false);
-}
-
-Variant ArrayData::prev() {
-  if (m_pos != invalid_index) {
-    setPosition(iter_rewind(m_pos));
-    if (m_pos != invalid_index) {
-      return getValue(m_pos);
-    }
-  }
-  return Variant(false);
+  return m_pos != iter_end() ? getValue(m_pos) : Variant(false);
 }
 
 Variant ArrayData::next() {
-  if (m_pos != invalid_index) {
-    setPosition(iter_advance(m_pos));
-    if (m_pos != invalid_index) {
+  // We call iter_advance() without checking if m_pos is the canonical invalid
+  // position. This is okay, since all IterAdvance() impls handle this
+  // correctly, but it means that EmptyArray::IterAdvance() is reachable.
+  setPosition(iter_advance(m_pos));
+  return m_pos != iter_end() ? getValue(m_pos) : Variant(false);
+}
+
+Variant ArrayData::prev() {
+  // We only call iter_rewind() if m_pos is not the canonical invalid position.
+  // Thus, EmptyArray::IterRewind() is not reachable.
+  auto pos_limit = iter_end();
+  if (m_pos != pos_limit) {
+    setPosition(iter_rewind(m_pos));
+    if (m_pos != pos_limit) {
       return getValue(m_pos);
     }
   }
@@ -743,20 +781,20 @@ Variant ArrayData::next() {
 }
 
 Variant ArrayData::end() {
-  setPosition(iter_end());
-  return m_pos != invalid_index ? getValue(m_pos) : Variant(false);
+  setPosition(iter_last());
+  return m_pos != iter_end() ? getValue(m_pos) : Variant(false);
 }
 
 Variant ArrayData::key() const {
-  return m_pos != invalid_index ? getKey(m_pos) : uninit_null();
+  return m_pos != iter_end() ? getKey(m_pos) : uninit_null();
 }
 
 Variant ArrayData::value(int32_t &pos) const {
-  return pos != invalid_index ? getValue(pos) : Variant(false);
+  return pos != iter_end() ? getValue(pos) : Variant(false);
 }
 
 Variant ArrayData::current() const {
-  return m_pos != invalid_index ? getValue(m_pos) : Variant(false);
+  return m_pos != iter_end() ? getValue(m_pos) : Variant(false);
 }
 
 const StaticString
@@ -764,7 +802,7 @@ const StaticString
   s_key("key");
 
 Variant ArrayData::each() {
-  if (m_pos != invalid_index) {
+  if (m_pos != iter_end()) {
     ArrayInit ret(4, ArrayInit::Mixed{});
     Variant key(getKey(m_pos));
     Variant value(getValue(m_pos));
@@ -781,36 +819,6 @@ Variant ArrayData::each() {
 ///////////////////////////////////////////////////////////////////////////////
 // helpers
 
-void ArrayData::serializeImpl(VariableSerializer *serializer) const {
-  serializer->writeArrayHeader(size(), isVectorData());
-  for (ArrayIter iter(this); iter; ++iter) {
-    serializer->writeArrayKey(iter.first());
-    serializer->writeArrayValue(iter.secondRef());
-  }
-  serializer->writeArrayFooter();
-}
-
-void ArrayData::serialize(VariableSerializer *serializer,
-                          bool skipNestCheck /* = false */) const {
-  if (size() == 0) {
-    serializer->writeArrayHeader(0, isVectorData());
-    serializer->writeArrayFooter();
-    return;
-  }
-  if (!skipNestCheck) {
-    if (serializer->incNestedLevel((void*)this)) {
-      serializer->writeOverflow((void*)this);
-    } else {
-      serializeImpl(serializer);
-    }
-    serializer->decNestedLevel((void*)this);
-  } else {
-    // If isObject, the array is temporary and we should not check or save
-    // its pointer.
-    serializeImpl(serializer);
-  }
-}
-
 const Variant& ArrayData::get(const Variant& k, bool error) const {
   assert(IsValidKey(k));
   auto const cell = k.asCell();
@@ -829,12 +837,12 @@ const Variant& ArrayData::getNotFound(const StringData* k) {
 }
 
 const Variant& ArrayData::getNotFound(int64_t k, bool error) const {
-  return error && m_kind != kNvtwKind ? getNotFound(k) :
+  return error && kind() != kGlobalsKind ? getNotFound(k) :
          null_variant;
 }
 
 const Variant& ArrayData::getNotFound(const StringData* k, bool error) const {
-  return error && m_kind != kNvtwKind ? getNotFound(k) :
+  return error && kind() != kGlobalsKind ? getNotFound(k) :
          null_variant;
 }
 
@@ -851,12 +859,12 @@ const Variant& ArrayData::getNotFound(const Variant& k) {
 const char* ArrayData::kindToString(ArrayKind kind) {
   std::array<const char*,7> names = {{
     "PackedKind",
+    "StructKind",
     "MixedKind",
-    "SharedKind",
     "EmptyKind",
-    "NvtwKind",
+    "ApcKind",
+    "GlobalsKind",
     "ProxyKind",
-    "IntMapKind",
   }};
   static_assert(names.size() == kNumKinds, "add new kinds here");
   return names[kind];
