@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,12 +20,11 @@
 #include <array>
 #include <cstdint>
 
+#include "hphp/runtime/base/array-data-defs.h"
+#include "hphp/runtime/base/req-containers.h"
+#include "hphp/runtime/base/req-ptr.h"
+#include "hphp/runtime/base/type-variant.h"
 #include "hphp/util/tls-pod-bag.h"
-#include "hphp/runtime/base/types.h"
-#include "hphp/runtime/base/smart-ptr.h"
-#include "hphp/runtime/base/complex-types.h"
-#include "hphp/runtime/base/smart-containers.h"
-#include "hphp/runtime/base/mixed-array.h"
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -34,14 +33,13 @@ struct TypedValue;
 class BaseVector;
 class BaseMap;
 class BaseSet;
-class c_ImmVector;
-class c_ImmSet;
-class c_Pair;
 struct Iter;
+struct MixedArray;
 
 enum class IterNextIndex : uint16_t {
   ArrayPacked = 0,
   ArrayMixed,
+  ArrayStruct,
   Array,
   Vector,
   ImmVector,
@@ -79,7 +77,7 @@ struct ArrayIter {
    * Constructors.  Note that sometimes ArrayIter objects are created
    * without running their C++ constructor.  (See new_iter_array.)
    */
-  ArrayIter() : m_pos(ArrayData::invalid_index) {
+  ArrayIter() {
     m_data = nullptr;
   }
   explicit ArrayIter(const ArrayData* data);
@@ -87,8 +85,6 @@ struct ArrayIter {
     setArrayData(data);
     if (data) {
       m_pos = data->iter_begin();
-    } else {
-      m_pos = ArrayData::invalid_index;
     }
   }
   explicit ArrayIter(const MixedArray*) = delete;
@@ -96,7 +92,7 @@ struct ArrayIter {
   explicit ArrayIter(ObjectData* obj);
   ArrayIter(ObjectData* obj, NoInc);
   explicit ArrayIter(const Object& obj);
-  explicit ArrayIter(const Cell& c);
+  explicit ArrayIter(Cell);
   explicit ArrayIter(const Variant& v);
 
   // Copy ctor
@@ -130,19 +126,21 @@ struct ArrayIter {
 
   explicit operator bool() { return !end(); }
   void operator++() { next(); }
-  bool end() {
+  bool end() const {
     if (LIKELY(hasArrayData())) {
-      return m_pos == ArrayData::invalid_index;
+      auto* ad = getArrayData();
+      return !ad || m_pos == ad->iter_end();
+      return endHelper();
     }
     return endHelper();
   }
-  bool endHelper();
+  bool endHelper() const;
 
   void next() {
     if (LIKELY(hasArrayData())) {
       const ArrayData* ad = getArrayData();
       assert(ad);
-      assert(m_pos != ArrayData::invalid_index);
+      assert(m_pos != ad->iter_end());
       m_pos = ad->iter_advance(m_pos);
       return;
     }
@@ -154,7 +152,7 @@ struct ArrayIter {
     if (LIKELY(hasArrayData())) {
       const ArrayData* ad = getArrayData();
       assert(ad);
-      assert(m_pos != ArrayData::invalid_index);
+      assert(m_pos != ad->iter_end());
       return ad->getKey(m_pos);
     }
     return firstHelper();
@@ -162,7 +160,7 @@ struct ArrayIter {
   Variant firstHelper();
   void nvFirst(TypedValue* out) {
     const ArrayData* ad = getArrayData();
-    assert(ad && m_pos != ArrayData::invalid_index);
+    assert(ad && m_pos != ad->iter_end());
     const_cast<ArrayData*>(ad)->nvGetKey(out, m_pos);
   }
 
@@ -187,17 +185,17 @@ struct ArrayIter {
   // Inline version of secondRef.  Only for use in iterator helpers.
   const TypedValue* nvSecond() const {
     const ArrayData* ad = getArrayData();
-    assert(ad && m_pos != ArrayData::invalid_index);
+    assert(ad && m_pos != ad->iter_end());
     return ad->getValueRef(m_pos).asTypedValue();
   }
 
   bool hasArrayData() const {
     return !((intptr_t)m_data & 1);
   }
-  bool hasCollection() {
+  bool hasCollection() const {
     return (!hasArrayData() && getObject()->isCollection());
   }
-  bool hasIteratorObj() {
+  bool hasIteratorObj() const {
     return (!hasArrayData() && !getObject()->isCollection());
   }
 
@@ -281,11 +279,12 @@ struct ArrayIter {
     return m_nextHelperIdx;
   }
 
-  ObjectData* getObject() {
+  ObjectData* getObject() const {
     assert(!hasArrayData());
     return (ObjectData*)((intptr_t)m_obj & ~1);
   }
 
+  template<typename F> void scan(F& mark) const;
 private:
   friend int64_t new_iter_array(Iter*, ArrayData*, TypedValue*);
   template<bool withRef>
@@ -297,7 +296,7 @@ private:
   template <bool incRef>
   void objInit(ObjectData* obj);
 
-  void cellInit(const Cell& c);
+  void cellInit(Cell);
 
   static void VectorInit(ArrayIter* iter, ObjectData* obj);
   static void MapInit(ArrayIter* iter, ObjectData* obj);
@@ -309,50 +308,10 @@ private:
   static void IteratorObjInit(ArrayIter* iter, ObjectData* obj);
 
   typedef void(*InitFuncPtr)(ArrayIter*,ObjectData*);
-  static const InitFuncPtr initFuncTable[Collection::MaxNumTypes];
+  static const InitFuncPtr initFuncTable[];
 
   void destruct();
 
-  BaseVector* getVector() {
-    assert(hasCollection());
-    assert(getCollectionType() == Collection::VectorType ||
-           getCollectionType() == Collection::ImmVectorType);
-    return (BaseVector*)((intptr_t)m_obj & ~1);
-  }
-  BaseMap* getMap() {
-    assert(hasCollection());
-    assert(Collection::isMapType(getCollectionType()));
-    return (BaseMap*)((intptr_t)m_obj & ~1);
-  }
-  BaseSet* getSet() {
-    assert(hasCollection());
-    assert(getCollectionType() == Collection::SetType ||
-           getCollectionType() == Collection::ImmSetType);
-    return (BaseSet*)((intptr_t)m_obj & ~1);
-  }
-  c_Pair* getPair() {
-    assert(hasCollection() && getCollectionType() == Collection::PairType);
-    return (c_Pair*)((intptr_t)m_obj & ~1);
-  }
-  c_ImmVector* getImmVector() {
-    assert(hasCollection() &&
-           getCollectionType() == Collection::ImmVectorType);
-
-    return (c_ImmVector*)((intptr_t)m_obj & ~1);
-  }
-  c_ImmSet* getImmSet() {
-    assert(hasCollection() && getCollectionType() == Collection::ImmSetType);
-    return (c_ImmSet*)((intptr_t)m_obj & ~1);
-  }
-  Collection::Type getCollectionType() {
-    ObjectData* obj = getObject();
-    return obj->getCollectionType();
-  }
-  ObjectData* getIteratorObj() {
-    assert(hasIteratorObj());
-    return getObject();
-  }
-
   void setArrayData(const ArrayData* ad) {
     assert((intptr_t(ad) & 1) == 0);
     m_data = ad;
@@ -378,6 +337,8 @@ private:
     ObjectData* m_obj;
   };
  public:
+  // m_pos is used by the array implementation to track the current position
+  // in the array. Beware that when m_data is null, m_pos is uninitialized.
   ssize_t m_pos;
  private:
   int m_version;
@@ -468,7 +429,7 @@ struct MArrayIter {
   Variant& val() {
     ArrayData* data = getArray();
     assert(data && data == getContainer());
-    assert(!data->hasMultipleRefs() || data->noCopyOnWrite());
+    assert(!data->cowCheck() || data->noCopyOnWrite());
     assert(!getResetFlag());
     assert(data->validMArrayIter(*this));
     // Normally it's not ok to modify the return value of getValueRef,
@@ -523,10 +484,11 @@ struct MArrayIter {
   bool getResetFlag() const { return m_resetFlag; }
   void setResetFlag(bool reset) { m_resetFlag = reset; }
 
+  template<typename F> void scan(F& mark) const;
 private:
   ArrayData* getData() const {
     assert(hasRef());
-    return m_ref->tv()->m_type == KindOfArray
+    return isArrayType(m_ref->tv()->m_type)
       ? m_ref->tv()->m_data.parr
       : nullptr;
   }
@@ -551,8 +513,8 @@ private:
     ArrayData* m_data;
   };
 public:
-  // m_pos is an opaque value used by the array implementation to track the
-  // current position in the array.
+  // m_pos is used by the array implementation to track the current position
+  // in the array.
   ssize_t m_pos;
 private:
   // m_container keeps track of which array we're "registered" with. Normally
@@ -609,9 +571,16 @@ private:
 struct MIterTable {
   struct Ent { ArrayData* array; MArrayIter* iter; };
 
+  void clear() {
+    ents.fill({nullptr, nullptr});
+    if (!extras.empty()) {
+      extras.release_if([] (const MIterTable::Ent& e) { return true; });
+    }
+  }
+
   std::array<Ent,7> ents;
   // Slow path: we expect this `extras' list to rarely be allocated.
-  TlsPodBag<Ent,smart::Allocator<Ent>> extras;
+  TlsPodBag<Ent,req::Allocator<Ent>> extras;
 };
 static_assert(sizeof(MIterTable) == 2*64, "");
 extern __thread MIterTable tl_miter_table;
@@ -621,8 +590,7 @@ ArrayData* move_strong_iterators(ArrayData* dest, ArrayData* src);
 
 //////////////////////////////////////////////////////////////////////
 
-class CufIter {
- public:
+struct CufIter {
   CufIter() : m_func(nullptr), m_ctx(nullptr), m_name(nullptr) {}
   ~CufIter();
   const Func* func() const { return m_func; }
@@ -636,16 +604,23 @@ class CufIter {
   }
   void setName(StringData* name) { m_name = name; }
 
-  static uint32_t funcOff() { return offsetof(CufIter, m_func); }
-  static uint32_t ctxOff()  { return offsetof(CufIter, m_ctx); }
-  static uint32_t nameOff() { return offsetof(CufIter, m_name); }
+  static constexpr uint32_t funcOff() { return offsetof(CufIter, m_func); }
+  static constexpr uint32_t ctxOff()  { return offsetof(CufIter, m_ctx); }
+  static constexpr uint32_t nameOff() { return offsetof(CufIter, m_name); }
+
+  template<class F> void scan(F& mark) const {
+    if (m_ctx && intptr_t(m_ctx) % 2 == 0) {
+      mark(reinterpret_cast<const ObjectData*>(m_ctx));
+    }
+    mark(m_name);
+  }
  private:
   const Func* m_func;
   void* m_ctx;
   StringData* m_name;
 };
 
-struct Iter {
+struct alignas(16) Iter {
   const ArrayIter&   arr() const { return m_u.aiter; }
   const MArrayIter& marr() const { return m_u.maiter; }
   const CufIter&     cuf() const { return m_u.cufiter; }
@@ -666,7 +641,7 @@ private:
     MArrayIter maiter;
     CufIter cufiter;
   } m_u;
-} __attribute__ ((aligned(16)));
+};
 
 //////////////////////////////////////////////////////////////////////
 
