@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,10 +17,9 @@
 #include "hphp/runtime/base/class-info.h"
 #include <vector>
 #include "hphp/runtime/base/static-string-table.h"
+#include "hphp/runtime/base/array-data-defs.h"
 #include "hphp/runtime/base/array-util.h"
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/externals.h"
-#include "hphp/runtime/base/hphp-system.h"
 #include "hphp/runtime/base/variable-serializer.h"
 #include "hphp/runtime/base/variable-unserializer.h"
 #include "hphp/runtime/ext/extension.h"
@@ -49,9 +48,6 @@ const ClassInfo::MethodInfo *ClassInfo::FindFunction(const String& name) {
   assert(s_loaded);
 
   const MethodInfo *ret = s_systemFuncs->getMethodInfo(name);
-  if (ret == nullptr) {
-    ret = g_context->findFunctionInfo(name);
-  }
   return ret;
 }
 
@@ -59,13 +55,6 @@ const ClassInfo *ClassInfo::FindClassInterfaceOrTrait(const String& name) {
   assert(!name.isNull());
   assert(s_loaded);
 
-  const ClassInfo *r;
-  if ((r = g_context->findClassInfo(name)) != nullptr ||
-      (r = g_context->findInterfaceInfo(name)) != nullptr ||
-      (r = g_context->findTraitInfo(name)) != nullptr) {
-    return r;
-  }
-
   ClassMap::const_iterator iter = s_class_like.find(name);
   if (iter != s_class_like.end()) {
     return iter->second->getDeclared();
@@ -169,30 +158,18 @@ Array ClassInfo::GetClassLike(unsigned mask, unsigned value) {
   return ret;
 }
 
-ClassInfo::ConstantInfo::ConstantInfo() :
-    valueLen(0), callback(nullptr), deferred(true) {
-}
-
-const Variant& ClassInfo::ConstantInfo::getDeferredValue() const {
-  assert(deferred);
-  if (callback) {
-    const Variant& (*f)()=(const Variant&(*)())callback;
-    return (*f)();
-  }
-  EnvConstants* g = get_env_constants();
-  return g->stgv_Variant[valueLen];
+ClassInfo::ConstantInfo::ConstantInfo() : valueLen(0) {
 }
 
 Variant ClassInfo::ConstantInfo::getValue() const {
-  if (deferred) {
-    return getDeferredValue();
-  }
   if (!svalue.empty()) {
     try {
       VariableUnserializer vu(svalue.data(), svalue.size(),
                               VariableUnserializer::Type::Serialize);
       return vu.unserialize();
-    } catch (Exception &e) {
+    } catch (ResourceExceededException&) {
+      throw;
+    } catch (Exception&) {
       assert(false);
     }
   }
@@ -203,13 +180,11 @@ void ClassInfo::ConstantInfo::setValue(const Variant& value) {
   VariableSerializer vs(VariableSerializer::Type::Serialize);
   String s = vs.serialize(value, true);
   svalue = std::string(s.data(), s.size());
-  deferred = false;
 }
 
 void ClassInfo::ConstantInfo::setStaticValue(const Variant& v) {
   value = v;
   value.setEvalScalar();
-  deferred = false;
 }
 
 void ClassInfo::InitializeSystemConstants() {
@@ -217,25 +192,19 @@ void ClassInfo::InitializeSystemConstants() {
   const ConstantMap &scm = s_systemFuncs->getConstants();
   for (ConstantMap::const_iterator it = scm.begin(); it != scm.end(); ++it) {
     ConstantInfo* ci = it->second;
-    if (ci->isDynamic()) {
-      Unit::defDynamicSystemConstant(ci->name.get(), ci);
-    } else {
-      Variant v = ci->getValue();
-      bool DEBUG_ONLY res = Unit::defCns(ci->name.get(),
-                                             v.asTypedValue(), true);
-      assert(res);
-    }
+    Variant v = ci->getValue();
+    bool DEBUG_ONLY res = Unit::defCns(ci->name.get(),
+                                           v.asTypedValue(), true);
+    assert(res);
   }
 }
 
-Array ClassInfo::GetSystemConstants(bool get_dynamic_constants /* = false */) {
+Array ClassInfo::GetSystemConstants() {
   assert(s_loaded);
   Array res;
   const ConstantMap &scm = s_systemFuncs->getConstants();
   for (ConstantMap::const_iterator it = scm.begin(); it != scm.end(); ++it) {
-    if (get_dynamic_constants || !it->second->isDynamic()) {
-      res.set(it->second->name, it->second->getValue());
-    }
+    res.set(it->second->name, it->second->getValue());
   }
   return res;
 }
@@ -298,112 +267,6 @@ bool ClassInfo::GetClassMethods(MethodVec &ret, const ClassInfo *classInfo) {
   return true;
 }
 
-void ClassInfo::GetClassSymbolNames(const Array& names, bool interface, bool trait,
-                                    std::vector<String> &classes,
-                                    std::vector<String> *clsMethods,
-                                    std::vector<String> *clsProperties,
-                                    std::vector<String> *clsConstants) {
-  if (clsMethods || clsProperties || clsConstants) {
-    for (ArrayIter iter(names); iter; ++iter) {
-      String clsname = iter.second().toString();
-      classes.push_back(clsname);
-
-      const ClassInfo *cls;
-      if (interface) {
-        cls = FindInterface(clsname.data());
-      } else if (trait) {
-        cls = FindTrait(clsname.data());
-      } else {
-        try {
-          cls = FindClass(clsname.data());
-        } catch (Exception &e) {
-          Logger::Error("Caught exception %s", e.getMessage().c_str());
-          continue;
-        } catch(...) {
-          Logger::Error("Caught unknown exception");
-          continue;
-        }
-      }
-      assert(cls);
-      if (clsMethods) {
-        const ClassInfo::MethodVec &methods = cls->getMethodsVec();
-        for (unsigned int i = 0; i < methods.size(); i++) {
-          clsMethods->push_back(clsname + "::" + methods[i]->name);
-        }
-      }
-      if (clsProperties) {
-        const ClassInfo::PropertyVec &properties = cls->getPropertiesVec();
-        for (ClassInfo::PropertyVec::const_iterator iter = properties.begin();
-             iter != properties.end(); ++iter) {
-          clsProperties->push_back(clsname + "::$" + (*iter)->name);
-        }
-      }
-      if (clsConstants) {
-        const ClassInfo::ConstantVec &constants = cls->getConstantsVec();
-        for (ClassInfo::ConstantVec::const_iterator iter = constants.begin();
-             iter != constants.end(); ++iter) {
-          clsConstants->push_back(clsname + "::" + (*iter)->name);
-        }
-      }
-    }
-  } else {
-    for (ArrayIter iter(names); iter; ++iter) {
-      classes.push_back(iter.second().toString());
-    }
-  }
-}
-
-void ClassInfo::GetSymbolNames(std::vector<String> &classes,
-                               std::vector<String> &functions,
-                               std::vector<String> &constants,
-                               std::vector<String> *clsMethods,
-                               std::vector<String> *clsProperties,
-                               std::vector<String> *clsConstants) {
-  static unsigned int methodSize = 128;
-  static unsigned int propSize   = 128;
-  static unsigned int constSize  = 128;
-
-  if (clsMethods) {
-    clsMethods->reserve(methodSize);
-  }
-  if (clsProperties) {
-    clsProperties->reserve(propSize);
-  }
-  if (clsConstants) {
-    clsConstants->reserve(constSize);
-  }
-
-  GetClassSymbolNames(GetClasses(), false, false, classes,
-                      clsMethods, clsProperties, clsConstants);
-  GetClassSymbolNames(GetInterfaces(), true, false, classes,
-                      clsMethods, clsProperties, clsConstants);
-
-  if (clsMethods && methodSize < clsMethods->size()) {
-    methodSize = clsMethods->size();
-  }
-  if (clsProperties && propSize < clsProperties->size()) {
-    propSize = clsProperties->size();
-  }
-  if (constSize && constSize < clsConstants->size()) {
-    constSize = clsConstants->size();
-  }
-
-  Array funcs1 = Unit::getSystemFunctions();
-  Array funcs2 = Unit::getUserFunctions();
-  functions.reserve(funcs1.size() + funcs2.size());
-  for (ArrayIter iter(funcs1); iter; ++iter) {
-    functions.push_back(iter.second().toString());
-  }
-  for (ArrayIter iter(funcs2); iter; ++iter) {
-    functions.push_back(iter.second().toString());
-  }
-  Array consts = lookupDefinedConstants();
-  constants.reserve(consts.size());
-  for (ArrayIter iter(consts); iter; ++iter) {
-    constants.push_back(iter.first().toString());
-  }
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 // ClassInfo
 
@@ -472,7 +335,7 @@ static String staticString(const char *s) {
   if (!s) {
     return String();
   }
-  return makeStaticString(s);
+  return String{makeStaticString(s)};
 }
 
 void ClassInfo::ReadUserAttributes(const char **&p,
@@ -520,7 +383,9 @@ ClassInfo::MethodInfo::MethodInfo(const char **&p) {
     docComment = *p++;
 
     if (attribute & IsSystem) {
-      returnType = (DataType)(int64_t)(*p++);
+      auto dt = static_cast<DataType>((int64_t)(*p++));
+      returnType = dt == kInvalidDataType ? folly::none
+                                          : MaybeDataType(dt);
     }
     while (*p) {
       ParameterInfo *parameter = new ParameterInfo();
@@ -528,7 +393,9 @@ ClassInfo::MethodInfo::MethodInfo(const char **&p) {
       parameter->name = *p++;
       parameter->type = *p++;
       if (attribute & IsSystem) {
-        parameter->argType = (DataType)(int64_t)(*p++);
+        auto dt = static_cast<DataType>((int64_t)(*p++));
+        parameter->argType = dt == kInvalidDataType ? folly::none
+                                                    : MaybeDataType(dt);
       }
       parameter->value = *p++;
       parameter->valueLen = (int64_t)*p++;
@@ -571,7 +438,7 @@ ClassInfoUnique::ClassInfoUnique(const char **&p) {
 
   // ClassInfoUnique is only created by ClassInfo::Load(), which is called
   // from hphp_process_init() in the thread-neutral initialization phase.
-  // It is OK to create StaticStrings here, and throw the smart ptrs away,
+  // It is OK to create StaticStrings here, and throw the String wrapper away,
   // because the underlying static StringData will not be released.
   m_name = staticString(*p++);
   m_parent = staticString(*p++);
@@ -608,7 +475,9 @@ ClassInfoUnique::ClassInfoUnique(const char **&p) {
     PropertyInfo *property = new PropertyInfo();
     property->attribute = (Attribute)(int64_t)(*p++);
     property->name = staticString(*p++);
-    property->type = DataType((int)uintptr_t(*p++));
+    auto dt = static_cast<DataType>((int)uintptr_t(*p++));
+    property->type = dt == kInvalidDataType ? folly::none
+                                            : MaybeDataType(dt);
     property->owner = this;
     assert(m_properties.find(property->name) == m_properties.end());
     m_properties[property->name] = property;
@@ -622,29 +491,28 @@ ClassInfoUnique::ClassInfoUnique(const char **&p) {
     const char *len_or_cw = *p++;
     constant->valueText = *p++;
 
+    assert(constant->valueText);
     if (uintptr_t(constant->valueText) > 0x100) {
+      // Serialized value from an IDL entry with a value: element
       constant->valueLen = (int64_t)len_or_cw;
       VariableUnserializer vu(constant->valueText,
                               constant->valueLen,
                               VariableUnserializer::Type::Serialize);
       try {
         constant->setStaticValue(vu.unserialize());
-      } catch (Exception &e) {
+      } catch (ResourceExceededException&) {
+        throw;
+      } catch (Exception&) {
         assert(false);
       }
-    } else if (constant->valueText) {
+    } else {
       DataType dt = DataType((int)uintptr_t(constant->valueText) - 2);
+      assert(dt != kInvalidDataType);
       constant->valueLen = 0;
       constant->valueText = nullptr;
       Variant v;
-      if (dt == KindOfUnknown) {
-        constant->valueLen = intptr_t(len_or_cw);
-      } else {
-        v = ClassInfo::GetVariant(dt, len_or_cw);
-        constant->setStaticValue(v);
-      }
-    } else {
-      constant->callback = (void*)len_or_cw;
+      v = ClassInfo::GetVariant(dt, len_or_cw);
+      constant->setStaticValue(v);
     }
 
     assert(m_constants.find(constant->name) == m_constants.end());
