@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,35 +22,38 @@
 #include <list>
 #include <utility>
 
-#include "folly/MapUtil.h"
+#include <folly/MapUtil.h>
+#include <folly/Format.h>
+#include <folly/Likely.h>
 
 #include "hphp/util/logger.h"
 #include "hphp/util/process.h"
 #include "hphp/util/text-color.h"
+#include "hphp/util/service-data.h"
 #include "hphp/runtime/base/request-event-handler.h"
 #include "hphp/runtime/base/array-init.h"
 #include "hphp/runtime/base/debuggable.h"
 #include "hphp/runtime/base/array-iterator.h"
 #include "hphp/runtime/base/memory-manager.h"
 #include "hphp/runtime/base/sweepable.h"
-#include "hphp/runtime/server/server-stats.h"
 #include "hphp/runtime/base/request-local.h"
 #include "hphp/runtime/base/builtin-functions.h"
 #include "hphp/runtime/base/comparisons.h"
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/externals.h"
+#include "hphp/runtime/base/program-functions.h"
 #include "hphp/runtime/base/runtime-option.h"
 #include "hphp/runtime/base/type-conversions.h"
-#include "hphp/runtime/debugger/debugger.h"
 #include "hphp/runtime/base/unit-cache.h"
-#include "hphp/runtime/ext/ext_string.h"
+#include "hphp/runtime/debugger/debugger.h"
+#include "hphp/runtime/base/system-profiler.h"
 #include "hphp/runtime/ext/std/ext_std_output.h"
+#include "hphp/runtime/ext/string/ext_string.h"
+#include "hphp/runtime/server/server-stats.h"
 #include "hphp/runtime/vm/jit/translator-inline.h"
 #include "hphp/runtime/vm/jit/translator.h"
 #include "hphp/runtime/vm/debugger-hook.h"
 #include "hphp/runtime/vm/event-hook.h"
 #include "hphp/runtime/vm/runtime.h"
-#include "hphp/system/constants.h"
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -59,8 +62,7 @@ IMPLEMENT_THREAD_LOCAL_NO_CHECK(ExecutionContext, g_context);
 
 ExecutionContext::ExecutionContext()
   : m_transport(nullptr)
-  , m_cwd(Process::CurrentWorkingDirectory)
-  , m_out(nullptr)
+  , m_sb(nullptr)
   , m_implicitFlush(false)
   , m_protectedLevel(0)
   , m_stdout(nullptr)
@@ -69,50 +71,67 @@ ExecutionContext::ExecutionContext()
   , m_errorState(ExecutionContext::ErrorState::NoError)
   , m_lastErrorNum(0)
   , m_throwAllErrors(false)
+  , m_pageletTasksStarted(0)
   , m_vhost(nullptr)
   , m_globalVarEnv(nullptr)
   , m_lambdaCounter(0)
   , m_nesting(0)
-  , m_breakPointFilter(nullptr)
-  , m_lastLocFilter(nullptr)
   , m_dbgNoBreak(false)
-  , m_coverPrevLine(-1)
-  , m_coverPrevUnit(nullptr)
-  , m_lastErrorPath("")
+  , m_unwindingCppException(false)
+  , m_lastErrorPath(staticEmptyString())
   , m_lastErrorLine(0)
   , m_executingSetprofileCallback(false)
 {
-  // We want this to run on every request, instead of just once per thread
-  auto hasSystemDefault = IniSetting::ResetSystemDefault("memory_limit");
-  if (!hasSystemDefault) {
-    auto max_mem = std::to_string(RuntimeOption::RequestMemoryMaxBytes);
-    IniSetting::SetUser("memory_limit", max_mem);
-  }
+  resetCoverageCounters();
+  // We don't want a new execution context to cause any request-heap
+  // allocations (because it will cause us to hold a slab, even while idle).
+  static auto s_cwd = makeStaticString(Process::CurrentWorkingDirectory);
+  m_cwd = s_cwd;
+  RID().setMemoryLimit(std::to_string(RuntimeOption::RequestMemoryMaxBytes));
+  RID().setErrorReportingLevel(RuntimeOption::RuntimeErrorReportingLevel);
 
-  // This one is hot so we don't want to go through the ini_set() machinery to
-  // change it in error_reporting(). Because of that, we have to set it back to
-  // the default on every request.
-  hasSystemDefault = IniSetting::ResetSystemDefault("error_reporting");
-  if (!hasSystemDefault) {
-    ThreadInfo::s_threadInfo.getNoCheck()->m_reqInjectionData.
-      setErrorReportingLevel(RuntimeOption::RuntimeErrorReportingLevel);
+  VariableSerializer::serializationSizeLimit =
+    RuntimeOption::SerializationSizeLimit;
+}
+
+// See header for why this is required.
+#ifndef _MSC_VER
+template<>
+#endif
+void ThreadLocalNoCheck<ExecutionContext>::destroy() {
+  if (!isNull()) {
+    getNoCheck()->sweep();
+    setNull();
   }
 }
 
-ExecutionContext::~ExecutionContext() {
-  // Discard any ConstInfo objects that were created to support reflection.
-  for (ConstInfoMap::const_iterator it = m_constInfo.begin();
-       it != m_constInfo.end(); ++it) {
-    delete it->second;
-  }
+
+void ExecutionContext::cleanup() {
+  manageAPCHandle();
 
   // Discard all units that were created via create_function().
   for (auto& v : m_createdFuncs) delete v;
+  m_createdFuncs.clear();
 
-  delete m_breakPointFilter;
-  delete m_lastLocFilter;
-  obFlushAll();
-  for (auto& b : m_buffers) delete b;
+  always_assert(m_activeSims.empty());
+}
+
+void ExecutionContext::sweep() {
+  cleanup();
+}
+
+ExecutionContext::~ExecutionContext() {
+  // When we destroy the execution context will call destructors on any objects
+  // in the userErrorHandlers and userExceptionHandlers vectors. If these
+  // destructors call restore_*_handler() they can trigger a pop_back() on the
+  // vector resulting in double destruction. There's no reason for code to do
+  // this but we should still avoid crashing.
+  // N.B.: This is already taken care of for us if EnableObjDestructCall is on
+  if (!RuntimeOption::EnableObjDestructCall) {
+    while (!m_userErrorHandlers.empty()) m_userErrorHandlers.pop_back();
+    while (!m_userExceptionHandlers.empty()) m_userExceptionHandlers.pop_back();
+  }
+  cleanup();
 }
 
 void ExecutionContext::backupSession() {
@@ -142,8 +161,7 @@ String ExecutionContext::getMimeType() const {
       mimetype = mimetype.substr(0, pos);
     }
   } else if (m_transport && m_transport->getUseDefaultContentType()) {
-    mimetype =
-        ThreadInfo::s_threadInfo->m_reqInjectionData.getDefaultMimeType();
+    mimetype = RID().getDefaultMimeType();
   }
   return mimetype;
 }
@@ -205,8 +223,13 @@ size_t ExecutionContext::getStdoutBytesWritten() const {
 }
 
 void ExecutionContext::write(const char *s, int len) {
-  if (m_out) {
-    m_out->append(s, len);
+  if (m_sb) {
+    m_sb->append(s, len);
+    if (m_out && m_out->chunk_size > 0) {
+      if (m_sb->size() >= m_out->chunk_size) {
+        obFlush();
+      }
+    }
   } else {
     writeStdout(s, len);
   }
@@ -220,16 +243,19 @@ void ExecutionContext::obProtect(bool on) {
   m_protectedLevel = on ? m_buffers.size() : 0;
 }
 
-void ExecutionContext::obStart(const Variant& handler /* = null */) {
-  OutputBuffer *ob = new OutputBuffer();
-  ob->handler = handler;
-  m_buffers.push_back(ob);
+void ExecutionContext::obStart(const Variant& handler /* = null */,
+                               int chunk_size /* = 0 */) {
+  if (m_insideOBHandler) {
+    raise_error("ob_start(): Cannot use output buffering "
+                "in output buffering display handlers");
+  }
+  m_buffers.emplace_back(Variant(handler), chunk_size);
   resetCurrentBuffer();
 }
 
 String ExecutionContext::obCopyContents() {
   if (!m_buffers.empty()) {
-    StringBuffer &oss = m_buffers.back()->oss;
+    StringBuffer &oss = m_buffers.back().oss;
     if (!oss.empty()) {
       return oss.copy();
     }
@@ -239,7 +265,7 @@ String ExecutionContext::obCopyContents() {
 
 String ExecutionContext::obDetachContents() {
   if (!m_buffers.empty()) {
-    StringBuffer &oss = m_buffers.back()->oss;
+    StringBuffer &oss = m_buffers.back().oss;
     if (!oss.empty()) {
       return oss.detach();
     }
@@ -251,13 +277,15 @@ int ExecutionContext::obGetContentLength() {
   if (m_buffers.empty()) {
     return 0;
   }
-  return m_buffers.back()->oss.size();
+  return m_buffers.back().oss.size();
 }
 
 void ExecutionContext::obClean(int handler_flag) {
   if (!m_buffers.empty()) {
-    OutputBuffer *last = m_buffers.back();
+    OutputBuffer *last = &m_buffers.back();
     if (!last->handler.isNull()) {
+      m_insideOBHandler = true;
+      SCOPE_EXIT { m_insideOBHandler = false; };
       vm_call_user_func(last->handler,
                         make_packed_array(last->oss.detach(), handler_flag));
     }
@@ -267,45 +295,60 @@ void ExecutionContext::obClean(int handler_flag) {
 
 bool ExecutionContext::obFlush() {
   assert(m_protectedLevel >= 0);
-  if ((int)m_buffers.size() > m_protectedLevel) {
-    std::list<OutputBuffer*>::const_iterator iter = m_buffers.end();
-    OutputBuffer *last = *(--iter);
-    const int flag = k_PHP_OUTPUT_HANDLER_START | k_PHP_OUTPUT_HANDLER_END;
-    if (iter != m_buffers.begin()) {
-      OutputBuffer *prev = *(--iter);
-      if (last->handler.isNull()) {
-        prev->oss.absorb(last->oss);
-      } else {
-        try {
-          Variant tout = vm_call_user_func(
-            last->handler, make_packed_array(last->oss.detach(), flag)
-          );
-          prev->oss.append(tout.toString());
-          last->oss.clear();
-        } catch (...) {
-          prev->oss.absorb(last->oss);
-        }
-      }
-      return true;
-    }
 
-    if (!last->handler.isNull()) {
+  if ((int)m_buffers.size() <= m_protectedLevel) {
+    return false;
+  }
+
+  auto iter = m_buffers.end();
+  OutputBuffer& last = *(--iter);
+
+  const int flag = k_PHP_OUTPUT_HANDLER_START | k_PHP_OUTPUT_HANDLER_END;
+
+  if (iter != m_buffers.begin()) {
+    OutputBuffer& prev = *(--iter);
+    if (last.handler.isNull()) {
+      prev.oss.absorb(last.oss);
+    } else {
+      auto str = last.oss.detach();
       try {
-        Variant tout = vm_call_user_func(
-          last->handler, make_packed_array(last->oss.detach(), flag)
-        );
-        String sout = tout.toString();
-        writeStdout(sout.data(), sout.size());
-        last->oss.clear();
-        return true;
-      } catch (...) {}
+        Variant tout;
+        {
+          m_insideOBHandler = true;
+          SCOPE_EXIT { m_insideOBHandler = false; };
+          tout = vm_call_user_func(
+            last.handler, make_packed_array(str, flag)
+          );
+        }
+        prev.oss.append(tout.toString());
+      } catch (...) {
+        prev.oss.append(str);
+        throw;
+      }
     }
-
-    writeStdout(last->oss.data(), last->oss.size());
-    last->oss.clear();
     return true;
   }
-  return false;
+
+  auto str = last.oss.detach();
+  if (!last.handler.isNull()) {
+    try {
+      Variant tout;
+      {
+        m_insideOBHandler = true;
+        SCOPE_EXIT { m_insideOBHandler = false; };
+        tout = vm_call_user_func(
+          last.handler, make_packed_array(str, flag)
+        );
+      }
+      str = tout.toString();
+    } catch (...) {
+      writeStdout(str.data(), str.size());
+      throw;
+    }
+  }
+
+  writeStdout(str.data(), str.size());
+  return true;
 }
 
 void ExecutionContext::obFlushAll() {
@@ -315,7 +358,6 @@ void ExecutionContext::obFlushAll() {
 bool ExecutionContext::obEnd() {
   assert(m_protectedLevel >= 0);
   if ((int)m_buffers.size() > m_protectedLevel) {
-    delete m_buffers.back();
     m_buffers.pop_back();
     resetCurrentBuffer();
     if (m_implicitFlush) flush();
@@ -339,28 +381,32 @@ const StaticString
   s_type("type"),
   s_name("name"),
   s_args("args"),
+  s_chunk_size("chunk_size"),
+  s_buffer_used("buffer_used"),
   s_default_output_handler("default output handler");
 
 Array ExecutionContext::obGetStatus(bool full) {
   Array ret = Array::Create();
-  std::list<OutputBuffer*>::const_iterator iter = m_buffers.begin();
   int level = 0;
-  for (; iter != m_buffers.end(); ++iter, ++level) {
+  for (auto& buffer : m_buffers) {
     Array status;
-    status.set(s_level, level);
-    if (level < m_protectedLevel) {
-      status.set(s_type, 1);
+    if (level < m_protectedLevel || buffer.handler.isNull()) {
       status.set(s_name, s_default_output_handler);
-    } else {
       status.set(s_type, 0);
-      status.set(s_name, (*iter)->handler);
+    } else {
+      status.set(s_name, buffer.handler);
+      status.set(s_type, 1);
     }
+    status.set(s_level, level);
+    status.set(s_chunk_size, buffer.chunk_size);
+    status.set(s_buffer_used, static_cast<uint64_t>(buffer.oss.size()));
 
     if (full) {
       ret.append(status);
     } else {
       ret = std::move(status);
     }
+    level++;
   }
   return ret;
 }
@@ -372,7 +418,7 @@ void ExecutionContext::obSetImplicitFlush(bool on) {
 Array ExecutionContext::obGetHandlers() {
   Array ret;
   for (auto& ob : m_buffers) {
-    auto& handler = ob->handler;
+    auto& handler = ob.handler;
     ret.append(handler.isNull() ? s_default_output_handler : handler);
   }
   return ret;
@@ -385,7 +431,7 @@ void ExecutionContext::flush() {
              (m_transport == nullptr ||
               (m_transport->getHTTPVersion() == "1.1" &&
                m_transport->getMethod() != Transport::Method::HEAD))) {
-    StringBuffer &oss = m_buffers.front()->oss;
+    StringBuffer &oss = m_buffers.front().oss;
     if (!oss.empty()) {
       if (m_transport) {
         m_transport->sendRaw((void*)oss.data(), oss.size(), 200, false, true);
@@ -400,9 +446,11 @@ void ExecutionContext::flush() {
 
 void ExecutionContext::resetCurrentBuffer() {
   if (m_buffers.empty()) {
+    m_sb = nullptr;
     m_out = nullptr;
   } else {
-    m_out = &m_buffers.back()->oss;
+    m_sb = &m_buffers.back().oss;
+    m_out = &m_buffers.back();
   }
 }
 
@@ -434,8 +482,13 @@ bool ExecutionContext::removeShutdownFunction(const Variant& function,
   return ret;
 }
 
+bool ExecutionContext::hasShutdownFunctions(ShutdownType type) {
+  return !m_shutdowns.isNull() && m_shutdowns.exists(type) &&
+    m_shutdowns[type].toArray().size() >= 1;
+}
+
 Variant ExecutionContext::pushUserErrorHandler(const Variant& function,
-                                                   int error_types) {
+                                               int error_types) {
   Variant ret;
   if (!m_userErrorHandlers.empty()) {
     ret = m_userErrorHandlers.back().first;
@@ -465,47 +518,75 @@ void ExecutionContext::popUserExceptionHandler() {
   }
 }
 
-void ExecutionContext::registerRequestEventHandler
-(RequestEventHandler *handler) {
-  assert(handler);
-  if (m_requestEventHandlerSet.find(handler) ==
-      m_requestEventHandlerSet.end()) {
-    m_requestEventHandlerSet.insert(handler);
-    m_requestEventHandlers.push_back(handler);
+std::size_t ExecutionContext::registerRequestEventHandler(
+  RequestEventHandler *handler) {
+  assert(handler && handler->getInited());
+  m_requestEventHandlers.push_back(handler);
+  return m_requestEventHandlers.size()-1;
+}
+
+void ExecutionContext::unregisterRequestEventHandler(
+  RequestEventHandler* handler,
+  std::size_t index) {
+  assert(index < m_requestEventHandlers.size() &&
+         m_requestEventHandlers[index] == handler);
+  assert(!handler->getInited());
+  if (index == m_requestEventHandlers.size()-1) {
+    m_requestEventHandlers.pop_back();
   } else {
-    assert(false);
+    m_requestEventHandlers[index] = nullptr;
   }
 }
 
 static bool requestEventHandlerPriorityComp(RequestEventHandler *a,
                                             RequestEventHandler *b) {
-  return a->priority() < b->priority();
+  if (!a) return b;
+  else if (!b) return false;
+  else return a->priority() < b->priority();
 }
 
 void ExecutionContext::onRequestShutdown() {
-  // Sort handlers by priority so that lower priority values get shutdown
-  // first
-  sort(m_requestEventHandlers.begin(), m_requestEventHandlers.end(),
-       requestEventHandlerPriorityComp);
-  for (unsigned int i = 0; i < m_requestEventHandlers.size(); i++) {
-    RequestEventHandler *handler = m_requestEventHandlers[i];
-    assert(handler->getInited());
-    if (handler->getInited()) {
+  while (!m_requestEventHandlers.empty()) {
+    // handlers could cause other handlers to be registered,
+    // so need to repeat until done
+    decltype(m_requestEventHandlers) tmp;
+    tmp.swap(m_requestEventHandlers);
+
+    // Sort handlers by priority so that lower priority values get shutdown
+    // first
+    sort(tmp.begin(), tmp.end(),
+         requestEventHandlerPriorityComp);
+    for (auto* handler : tmp) {
+      if (!handler) continue;
+      assert(handler->getInited());
       handler->requestShutdown();
       handler->setInited(false);
     }
   }
-  m_requestEventHandlers.clear();
-  m_requestEventHandlerSet.clear();
 }
 
-void ExecutionContext::executeFunctions(const Array& funcs) {
-  ThreadInfo::s_threadInfo->m_reqInjectionData.resetTimer(
-    RuntimeOption::PspTimeoutSeconds);
+void ExecutionContext::executeFunctions(ShutdownType type) {
+  RID().resetTimer(RuntimeOption::PspTimeoutSeconds);
+  RID().resetCPUTimer(RuntimeOption::PspCpuTimeoutSeconds);
 
-  for (ArrayIter iter(funcs); iter; ++iter) {
-    Array callback = iter.second().toArray();
-    vm_call_user_func(callback[s_name], callback[s_args].toArray());
+  if (!m_shutdowns.isNull() && m_shutdowns.exists(type)) {
+    SCOPE_EXIT {
+      try { m_shutdowns.remove(type); } catch (...) {}
+    };
+    // We mustn't destroy any callbacks until we're done with all
+    // of them. So hold them in tmp.
+    Array tmp;
+    while (true) {
+      auto& var = m_shutdowns.lvalAt(type);
+      if (!var.isArray()) break;
+      auto funcs = var.toArray();
+      var.unset();
+      for (int pos = 0; pos < funcs.size(); ++pos) {
+        Array callback = funcs[pos].toArray();
+        vm_call_user_func(callback[s_name], callback[s_args].toArray());
+      }
+      tmp.append(funcs);
+    }
   }
 }
 
@@ -515,33 +596,22 @@ void ExecutionContext::onShutdownPreSend() {
     try { obFlushAll(); } catch (...) {}
   };
 
-  if (!m_shutdowns.isNull() && m_shutdowns.exists(ShutDown)) {
-    SCOPE_EXIT {
-      try { m_shutdowns.remove(ShutDown); } catch (...) {}
-    };
-    executeFunctions(m_shutdowns[ShutDown].toArray());
-  }
+  MM().resetCouldOOM(isStandardRequest());
+  executeFunctions(ShutDown);
 }
 
 extern void ext_session_request_shutdown();
 
 void ExecutionContext::onShutdownPostSend() {
   ServerStats::SetThreadMode(ServerStats::ThreadMode::PostProcessing);
-  MM().resetCouldOOM();
+  MM().resetCouldOOM(isStandardRequest());
   try {
     try {
       ServerStatsHelper ssh("psp", ServerStatsHelper::TRACK_HWINST);
-      if (!m_shutdowns.isNull()) {
-        if (m_shutdowns.exists(PostSend)) {
-          SCOPE_EXIT {
-            try { m_shutdowns.remove(PostSend); } catch (...) {}
-          };
-          executeFunctions(m_shutdowns[PostSend].toArray());
-        }
-      }
+      executeFunctions(PostSend);
     } catch (...) {
       try {
-        bump_counter_and_rethrow();
+        bump_counter_and_rethrow(true /* isPsp */);
       } catch (const ExitException &e) {
         // do nothing
       } catch (const Exception &e) {
@@ -570,11 +640,11 @@ void ExecutionContext::onShutdownPostSend() {
 bool ExecutionContext::errorNeedsHandling(int errnum,
                                               bool callUserHandler,
                                               ErrorThrowMode mode) {
-  if (m_throwAllErrors) {
-    throw errnum;
+  if (UNLIKELY(m_throwAllErrors)) {
+    throw Exception(folly::sformat("throwAllErrors: {}", errnum));
   }
   if (mode != ErrorThrowMode::Never || errorNeedsLogging(errnum) ||
-      ThreadInfo::s_threadInfo->m_reqInjectionData.hasTrackErrors()) {
+      RID().hasTrackErrors()) {
     return true;
   }
   if (callUserHandler) {
@@ -587,8 +657,9 @@ bool ExecutionContext::errorNeedsHandling(int errnum,
 }
 
 bool ExecutionContext::errorNeedsLogging(int errnum) {
-  auto level = ThreadInfo::s_threadInfo.getNoCheck()->
-    m_reqInjectionData.getErrorReportingLevel();
+  auto level =
+    RID().getErrorReportingLevel() |
+    RuntimeOption::ForceErrorReportingLevel;
   return RuntimeOption::NoSilencer || (level & errnum) != 0;
 }
 
@@ -633,47 +704,61 @@ void ExecutionContext::handleError(const std::string& msg,
     break;
   }
 
+  // Potentially upgrade the error to E_USER_ERROR
+  if (errnum & RuntimeOption::ErrorUpgradeLevel &
+      static_cast<int>(ErrorMode::UPGRADEABLE_ERROR)) {
+    errnum = static_cast<int>(ErrorMode::USER_ERROR);
+    mode = ErrorThrowMode::IfUnhandled;
+  }
+
   ErrorStateHelper esh(this, newErrorState);
   auto const ee = skipFrame ?
     ExtendedException(ExtendedException::SkipFrame{}, msg) :
     ExtendedException(msg);
-  recordLastError(ee, errnum);
   bool handled = false;
   if (callUserHandler) {
     handled = callUserErrorHandler(ee, errnum, false);
   }
+
+  if (!handled) {
+    recordLastError(ee, errnum);
+  }
+
+  if (g_system_profiler) {
+    g_system_profiler->errorCallBack(ee, errnum, msg);
+  }
+
   if (mode == ErrorThrowMode::Always ||
       (mode == ErrorThrowMode::IfUnhandled && !handled)) {
-    DEBUGGER_ATTACHED_ONLY(phpDebuggerErrorHook(msg));
-    auto exn = FatalErrorException(msg, ee.getBackTrace());
-    exn.setSilent(!errorNeedsLogging(errnum));
-    throw exn;
+    DEBUGGER_ATTACHED_ONLY(phpDebuggerErrorHook(ee, errnum, msg));
+    bool isRecoverable =
+      errnum == static_cast<int>(ErrorMode::RECOVERABLE_ERROR);
+    raise_fatal_error(msg.c_str(), ee.getBacktrace(), isRecoverable,
+                      !errorNeedsLogging(errnum) /* silent */);
+    not_reached();
   }
   if (!handled) {
-    if (ThreadInfo::s_threadInfo->m_reqInjectionData.hasTrackErrors()) {
+    VMRegAnchor _;
+    auto fp = vmfp();
+
+    if (RID().hasTrackErrors() && fp) {
       // Set $php_errormsg in the parent scope
-      Variant varFrom(ee.getMessage());
-      const auto tvFrom(varFrom.asTypedValue());
-      VMRegAnchor _;
-      auto fp = vmfp();
+      Variant msg(ee.getMessage());
       if (fp->func()->isBuiltin()) {
         fp = getPrevVMState(fp);
       }
       assert(fp);
       auto id = fp->func()->lookupVarId(s_php_errormsg.get());
       if (id != kInvalidId) {
-        auto tvTo = frame_local(fp, id);
-        if (tvTo->m_type == KindOfRef) {
-          tvTo = tvTo->m_data.pref->tv();
-        }
-        tvDup(*tvFrom, *tvTo);
-      } else if (fp->hasVarEnv()) {
-        g_context->setVar(s_php_errormsg.get(), tvFrom);
+        auto local = frame_local(fp, id);
+        tvSet(*msg.asTypedValue(), *tvToCell(local));
+      } else if ((fp->func()->attrs() & AttrMayUseVV) && fp->hasVarEnv()) {
+        fp->getVarEnv()->set(s_php_errormsg.get(), msg.asTypedValue());
       }
     }
 
     if (errorNeedsLogging(errnum)) {
-      DEBUGGER_ATTACHED_ONLY(phpDebuggerErrorHook(ee.getMessage()));
+      DEBUGGER_ATTACHED_ONLY(phpDebuggerErrorHook(ee, errnum, ee.getMessage()));
       auto fileAndLine = ee.getFileAndLine();
       Logger::Log(Logger::LogError, prefix.c_str(), ee,
                   fileAndLine.first.c_str(), fileAndLine.second);
@@ -692,31 +777,57 @@ bool ExecutionContext::callUserErrorHandler(const Exception &e, int errnum,
   }
   if (!m_userErrorHandlers.empty() &&
       (m_userErrorHandlers.back().second & errnum) != 0) {
-    int errline = 0;
-    String errfile;
-    Array backtrace;
+    auto fileAndLine = std::make_pair(empty_string(), 0);
+    Variant backtrace;
     if (auto const ee = dynamic_cast<const ExtendedException*>(&e)) {
-      Array arr = ee->getBackTrace();
-      if (!arr.isNull()) {
-        backtrace = arr;
-        Array top = backtrace.rvalAt(0).toArray();
-        if (!top.isNull()) {
-          errfile = top.rvalAt(s_file);
-          errline = top.rvalAt(s_line).toInt64();
-        }
-      }
+      fileAndLine = ee->getFileAndLine();
+      backtrace = ee->getBacktrace();
     }
     try {
       ErrorStateHelper esh(this, ErrorState::ExecutingUserHandler);
-      Array dummyContext = Array::Create();
+      auto const ar = g_context->getFrameAtDepth(0);
+      auto const context = ar ? getDefinedVariables(ar) : empty_array();
       if (!same(vm_call_user_func
                 (m_userErrorHandlers.back().first,
-                 make_packed_array(errnum, String(e.getMessage()), errfile,
-                     errline, dummyContext, backtrace)),
+                 make_packed_array(errnum, String(e.getMessage()),
+                     fileAndLine.first, fileAndLine.second, context,
+                     backtrace)),
                 false)) {
         return true;
       }
+    } catch (const RequestTimeoutException& e) {
+      static auto requestErrorHandlerTimeoutCounter =
+          ServiceData::createTimeseries("requests_timed_out_error_handler",
+                                        {ServiceData::StatsType::COUNT});
+      requestErrorHandlerTimeoutCounter->addValue(1);
+      ServerStats::Log("request.timed_out.error_handler", 1);
+
+      if (!swallowExceptions) throw;
+    } catch (const RequestCPUTimeoutException& e) {
+      static auto requestErrorHandlerCPUTimeoutCounter =
+          ServiceData::createTimeseries("requests_cpu_timed_out_error_handler",
+                                        {ServiceData::StatsType::COUNT});
+      requestErrorHandlerCPUTimeoutCounter->addValue(1);
+      ServerStats::Log("request.cpu_timed_out.error_handler", 1);
+
+      if (!swallowExceptions) throw;
+    } catch (const RequestMemoryExceededException& e) {
+      static auto requestErrorHandlerMemoryExceededCounter =
+          ServiceData::createTimeseries(
+              "requests_memory_exceeded_error_handler",
+              {ServiceData::StatsType::COUNT});
+      requestErrorHandlerMemoryExceededCounter->addValue(1);
+      ServerStats::Log("request.memory_exceeded.error_handler", 1);
+
+      if (!swallowExceptions) throw;
     } catch (...) {
+      static auto requestErrorHandlerOtherExceptionCounter =
+          ServiceData::createTimeseries(
+              "requests_other_exception_error_handler",
+              {ServiceData::StatsType::COUNT});
+      requestErrorHandlerOtherExceptionCounter->addValue(1);
+      ServerStats::Log("request.other_exception.error_handler", 1);
+
       if (!swallowExceptions) throw;
     }
   }
@@ -724,7 +835,17 @@ bool ExecutionContext::callUserErrorHandler(const Exception &e, int errnum,
 }
 
 bool ExecutionContext::onFatalError(const Exception &e) {
-  int errnum = static_cast<int>(ErrorConstants::ErrorModes::FATAL_ERROR);
+  MM().resetCouldOOM(isStandardRequest());
+  RID().resetTimer();
+
+  auto prefix = "\nFatal error: ";
+  auto errnum = static_cast<int>(ErrorMode::FATAL_ERROR);
+  auto const fatal = dynamic_cast<const FatalErrorException*>(&e);
+  if (fatal && fatal->isRecoverable()) {
+     prefix = "\nCatchable fatal error: ";
+     errnum = static_cast<int>(ErrorMode::RECOVERABLE_ERROR);
+  }
+
   recordLastError(e, errnum);
 
   bool silenced = false;
@@ -735,16 +856,16 @@ bool ExecutionContext::onFatalError(const Exception &e) {
   }
   // need to silence even with the AlwaysLogUnhandledExceptions flag set
   if (!silenced && RuntimeOption::AlwaysLogUnhandledExceptions) {
-    Logger::Log(Logger::LogError, "\nFatal error: ", e,
-                fileAndLine.first.c_str(), fileAndLine.second);
+    Logger::Log(Logger::LogError, prefix, e, fileAndLine.first.c_str(),
+                fileAndLine.second);
   }
   bool handled = false;
   if (RuntimeOption::CallUserHandlerOnFatals) {
     handled = callUserErrorHandler(e, errnum, true);
   }
   if (!handled && !silenced && !RuntimeOption::AlwaysLogUnhandledExceptions) {
-    Logger::Log(Logger::LogError, "\nFatal error: ", e,
-                fileAndLine.first.c_str(), fileAndLine.second);
+    Logger::Log(Logger::LogError, prefix, e, fileAndLine.first.c_str(),
+                fileAndLine.second);
   }
   return handled;
 }
@@ -755,7 +876,7 @@ bool ExecutionContext::onUnhandledException(Object e) {
     Logger::Error("\nFatal error: Uncaught %s", err.data());
   }
 
-  if (e.instanceof(SystemLib::s_ExceptionClass)) {
+  if (e.instanceof(SystemLib::s_ThrowableClass)) {
     // user thrown exception
     if (!m_userExceptionHandlers.empty()) {
       if (!same(vm_call_user_func
@@ -790,8 +911,7 @@ void ExecutionContext::debuggerInfo(
     info.emplace_back("Max Memory", IDebuggable::FormatSize(newInt));
   }
   info.emplace_back("Max Time",
-    IDebuggable::FormatTime(ThreadInfo::s_threadInfo.getNoCheck()->
-                 m_reqInjectionData.getTimeout() * 1000));
+                    IDebuggable::FormatTime(RID().getTimeout() * 1000));
 }
 
 ///////////////////////////////////////////////////////////////////////////////
