@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -19,7 +19,7 @@
 #include <cstdlib>
 #include <cstring>
 
-#include "folly/Likely.h"
+#include <folly/Likely.h>
 
 #include "hphp/runtime/base/tv-helpers.h"
 #include "hphp/runtime/base/mixed-array.h"
@@ -35,7 +35,9 @@ namespace HPHP {
 
 bool PackedArray::checkInvariants(const ArrayData* arr) {
   assert(arr->isPacked());
-  assert(arr->m_size <= packedCodeToCap(arr->m_packedCapCode));
+  assert(arr->checkCount());
+  assert(arr->m_size <= arr->cap());
+  assert(arr->m_pos >= 0 && arr->m_pos <= arr->m_size);
   static_assert(ArrayData::kPackedKind == 0, "");
   // Note that m_pos < m_size is not an invariant, because an array
   // that grows will only adjust m_size to zero on the old array.
@@ -57,29 +59,23 @@ bool PackedArray::checkInvariants(const ArrayData* arr) {
 
 ALWAYS_INLINE
 MixedArray* PackedArray::ToMixedHeader(const ArrayData* old,
-                                      size_t neededSize) {
+                                       size_t neededSize) {
   assert(PackedArray::checkInvariants(old));
 
   auto const oldSize = old->m_size;
-  auto const cmret   = computeCapAndMask(neededSize);
-  auto const cap     = cmret.first;
-  auto const mask    = cmret.second;
-  auto const ad      = smartAllocArray(cap, mask);
+  auto const scale   = computeScaleFromSize(neededSize);
+  auto const ad      = reqAllocArray(scale);
+  ad->m_sizeAndPos   = oldSize | int64_t{old->m_pos} << 32;
+  ad->m_hdr.init(HeaderKind::Mixed, 1);
+  ad->m_scale_used   = scale | uint64_t{oldSize} << 32; // used=oldSize
+  ad->m_nextKI       = oldSize;
 
-  auto const shiftedSize = uint64_t{oldSize} << 32;
-  ad->m_kindAndSize      = shiftedSize | MixedArray::kMixedKind << 24;
-  ad->m_posAndCount      = static_cast<uint32_t>(old->m_pos);  // zero count
-  ad->m_capAndUsed       = shiftedSize | cap;
-  ad->m_tableMask        = mask;
-  ad->m_nextKI           = oldSize;
-
-  assert(ad->m_kind == ArrayData::kMixedKind);
   assert(ad->m_size == oldSize);
   assert(ad->m_pos == old->m_pos);
-  assert(ad->m_count == 0);
+  assert(ad->kind() == ArrayData::kMixedKind);
+  assert(ad->hasExactlyOneRef());
   assert(ad->m_used == oldSize);
-  assert(ad->m_cap == cap);
-  assert(ad->m_tableMask == mask);
+  assert(ad->m_scale == scale);
   assert(ad->m_nextKI == oldSize);
   // Can't checkInvariants yet, since we haven't populated the payload.
   return ad;
@@ -96,7 +92,7 @@ MixedArray* PackedArray::ToMixedHeader(const ArrayData* old,
 MixedArray* PackedArray::ToMixed(ArrayData* old) {
   auto const oldSize = old->m_size;
   auto const ad      = ToMixedHeader(old, oldSize + 1);
-  auto const mask    = ad->m_tableMask;
+  auto const mask    = ad->mask();
   auto dstData       = ad->data();
   auto dstHash       = ad->hashTab();
   auto const srcData = packedData(old);
@@ -112,11 +108,11 @@ MixedArray* PackedArray::ToMixed(ArrayData* old) {
   for (; i <= mask; ++i) {
     *dstHash++ = MixedArray::Empty;
   }
-
-  old->m_size = 0;
+  old->m_sizeAndPos = 0;
 
   assert(ad->checkInvariants());
   assert(!ad->isFull());
+  assert(ad->hasExactlyOneRef());
   return ad;
 }
 
@@ -143,13 +139,14 @@ MixedArray* PackedArray::ToMixedCopy(const ArrayData* old) {
     ++dstData;
     ++dstHash;
   }
-  auto const mask = ad->m_tableMask;
-  for (; i <= mask; ++i) {
+  auto const n = ad->hashSize();
+  for (; i < n; ++i) {
     *dstHash++ = MixedArray::Empty;
   }
 
   assert(ad->checkInvariants());
   assert(!ad->isFull());
+  assert(ad->hasExactlyOneRef());
   return ad;
 }
 
@@ -157,17 +154,13 @@ MixedArray* PackedArray::ToMixedCopy(const ArrayData* old) {
  * Convert to mixed, reserving space for at least `neededSize' elems.
  * The `neededSize' should include old->size(), but may be equal to
  * it.
- *
- * Unlike the other ToMixed functions, the returned array already has
- * a reference count of 1.
  */
 MixedArray* PackedArray::ToMixedCopyReserve(const ArrayData* old,
                                            size_t neededSize) {
   assert(neededSize >= old->m_size);
   auto const ad      = ToMixedHeader(old, neededSize);
-  ad->m_count = 1;
   auto const oldSize = old->m_size;
-  auto const mask    = ad->m_tableMask;
+  auto const mask    = ad->mask();
   auto dstData       = ad->data();
   auto dstHash       = ad->hashTab();
   auto const srcData = packedData(old);
@@ -185,79 +178,95 @@ MixedArray* PackedArray::ToMixedCopyReserve(const ArrayData* old,
   }
 
   assert(ad->checkInvariants());
+  assert(ad->hasExactlyOneRef());
   return ad;
 }
 
 NEVER_INLINE
 ArrayData* PackedArray::Grow(ArrayData* old) {
   assert(checkInvariants(old));
-  assert(old->m_size == packedCodeToCap(old->m_packedCapCode));
+  assert(old->m_size == old->cap());
   DEBUG_ONLY auto const oldPos = old->m_pos;
 
   ArrayData* ad;
-  if (LIKELY(old->m_packedCapCode <= kPackedCapCodeThreshold / 2)) {
-    assert(old->m_packedCapCode == packedCodeToCap(old->m_packedCapCode));
-    auto const cap = old->m_packedCapCode * 2;
+  uint32_t oldCapCode = old->m_hdr.aux.code;
+  auto cap = oldCapCode * 2;
+  if (LIKELY(cap <= CapCode::Threshold)) {
+    assert(oldCapCode == old->cap());
+    // We add 1 to the cap, to make it use up all the memory to be allocated, if
+    // the original cap has been maximized.
+    if (auto capUpdated = getMaxCapInPlaceFast(++cap)) {
+      cap = capUpdated;
+    }
     ad = static_cast<ArrayData*>(
-      MM().objMallocLogged(sizeof(ArrayData) + cap * sizeof(TypedValue))
+      MM().objMalloc(sizeof(ArrayData) + cap * sizeof(TypedValue))
     );
-    auto const oldSize = old->m_size;
-    assert(cap == packedCodeToCap(cap));
-    ad->m_kindAndSize = uint64_t{oldSize} << 32 | cap;
-    assert(ad->m_kind == ArrayData::kPackedKind);
-    assert(ad->m_size == oldSize);
-    assert(packedCodeToCap(ad->m_packedCapCode) == cap);
+    assert(cap == CapCode::ceil(cap).code);
+    ad->m_sizeAndPos = old->m_sizeAndPos;
+    ad->m_hdr.init(CapCode::exact(cap), old->m_hdr.kind, 1);
+    assert(ad->isPacked());
+    assert(ad->m_size == old->m_size);
+    assert(ad->cap() == cap);
   } else {
     ad = GrowHelper(old);
   }
 
-  auto const oldPosUnsigned = uint64_t{static_cast<uint32_t>(old->m_pos)};
-  ad->m_posAndCount = oldPosUnsigned;
-
   if (UNLIKELY(strong_iterators_exist())) {
     move_strong_iterators(ad, old);
   }
 
-  // Steal the old array payload.  At the time of this writing, it was
-  // better not to reuse the memcpy return value here because gcc had
-  // `ad' in a callee saved register anyway.  The reg-to-reg move was
-  // smaller than subtracting sizeof(ArrayData) from rax to return.
   auto const oldSize = old->m_size;
-  old->m_size = 0;
-  std::memcpy(packedData(ad), packedData(old), oldSize * sizeof(TypedValue));
+  old->m_sizeAndPos = 0;                // zombie
 
-  // TODO(#2926276): it would be good to refactor callers to expect
-  // our refcount to start at 1.
+  memcpy16_inline(packedData(ad), packedData(old),
+                  oldSize * sizeof(TypedValue));
 
   assert(ad->m_pos == oldPos);
-  assert(ad->m_count == 0);
+  assert(ad->hasExactlyOneRef());
   assert(checkInvariants(ad));
   return ad;
 }
 
 NEVER_INLINE
 ArrayData* PackedArray::GrowHelper(ArrayData* old) {
-  auto const oldCap = packedCodeToCap(old->m_packedCapCode);
-  static_assert(kMaxPackedCap >= MixedArray::MaxSize, "");
+  auto const oldCap = old->cap();
+  static_assert(CapCode::Max >= MixedArray::MaxSize, "");
   if (UNLIKELY(oldCap > MixedArray::MaxSize / 2)) return nullptr;
-  auto const cap = roundUpPackedCap(oldCap * 2);
-  // The capacity should not change if it round trips into
-  // encoded form and back
-  ArrayData* ad = static_cast<ArrayData*>(
-    MM().objMallocLogged(sizeof(ArrayData) + cap * sizeof(TypedValue))
-  );
-  auto const capCode = packedCapToCode(cap);
-  auto const oldSize = old->m_size;
-  ad->m_kindAndSize = uint64_t{oldSize} << 32 | capCode;
-  assert(ad->m_kind == ArrayData::kPackedKind);
-  assert(ad->m_size == oldSize);
-  assert(packedCodeToCap(ad->m_packedCapCode) == cap);
+  assert(CapCode::ceil(oldCap*2).decode() > CapCode::Threshold);
+  auto ad = MixedArray::MakeReserveSlow(oldCap*2); // pos=size=kind=0
+  if (UNLIKELY(ad == nullptr)) return nullptr;
+  // ad->m_hdr is already set correctly in MakeReserveSlow
+  ad->m_sizeAndPos = old->m_sizeAndPos;
+  assert(ad->isPacked());
+  assert(ad->m_size == old->m_size);
+  assert(ad->cap() >= oldCap*2);
+  assert(ad->hasExactlyOneRef());
   return ad;
 }
 
+/*
+ * Get the maximum possible capacity without reallocation. Return 0 if we don't
+ * have a quick way to get a better cap. Currently this only works for caps
+ * within CapCode::Threshold. It should be pretty fast.
+ */
+uint32_t PackedArray::getMaxCapInPlaceFast(uint32_t cap) {
+  if (UNLIKELY(cap > CapCode::Threshold)) {
+    return 0;
+  }
+  static_assert(sizeof(TypedValue) == 16, "sizeof TypedValue changed?");
+  static_assert(sizeof(ArrayData) == 16, "sizeof ArrayData changed?");
+  assert((cap + 1) * 16U <= kMaxSmallSize);
+  uint32_t newCap = (MemoryManager::smallSizeClass((cap + 1) << 4) >> 4) - 1;
+  if (UNLIKELY(newCap > CapCode::Threshold)) {
+    newCap = CapCode::floor(newCap).decode();
+  }
+  assert(newCap >= cap && CapCode::encodable(newCap));
+  return newCap > cap ? newCap : 0;
+}
+
 NEVER_INLINE
 ArrayData* PackedArray::CopyAndResizeIfNeededSlow(const ArrayData* adIn) {
-  assert(adIn->m_size == packedCodeToCap(adIn->m_packedCapCode));
+  assert(adIn->m_size == adIn->cap());
   // Note: this path will have to handle splitting strong iterators
   // later when we combine copy & grow into one operation.
   // For now I'm just making use of copyPacked to do it for me before
@@ -265,14 +274,14 @@ ArrayData* PackedArray::CopyAndResizeIfNeededSlow(const ArrayData* adIn) {
   auto const copy = PackedArray::Copy(adIn);
   auto const ret  = PackedArray::Grow(copy);
   assert(ret != copy);
-  assert(copy->getCount() == 0);
+  assert(copy->hasExactlyOneRef());
   PackedArray::Release(copy);
   return ret;
 }
 
 ALWAYS_INLINE
 ArrayData* PackedArray::CopyAndResizeIfNeeded(const ArrayData* adIn) {
-  if (LIKELY(sizeLessThanPackedCapCode(adIn->m_size, adIn->m_packedCapCode))) {
+  if (LIKELY(adIn->cap() > adIn->m_size)) {
     return Copy(adIn);
   }
   return CopyAndResizeIfNeededSlow(adIn);
@@ -280,40 +289,54 @@ ArrayData* PackedArray::CopyAndResizeIfNeeded(const ArrayData* adIn) {
 
 ALWAYS_INLINE
 ArrayData* PackedArray::ResizeIfNeeded(ArrayData* adIn) {
-  if (LIKELY(sizeLessThanPackedCapCode(adIn->m_size, adIn->m_packedCapCode))) {
+  if (LIKELY(adIn->cap() > adIn->m_size)) {
     return adIn;
   }
   return Grow(adIn);
 }
 
 //////////////////////////////////////////////////////////////////////
+ALWAYS_INLINE
+void PackedArray::CopyPackedHelper(const ArrayData* adIn, ArrayData* ad,
+                                   RefCount initial_count) {
+  // Copy everything from `adIn' to `ad', including refcount, etc.
+  auto const size = adIn->m_size;
+  static_assert(sizeof(ArrayData) == 16 && sizeof(TypedValue) == 16, "");
+  memcpy16_inline(ad, adIn, (size + 1) * 16);
+  ad->m_hdr.init(adIn->m_hdr, initial_count);
+
+  // Copy counted types correctly, especially RefData.
+  auto data = packedData(ad);
+  for (uint32_t i = 0; i < size; ++i) {
+    auto pTv = data + i;
+    if (UNLIKELY(pTv->m_type == KindOfRef)) {
+      auto ref = pTv->m_data.pref;
+      // See also tvDupFlatternVars()
+      if (!ref->isReferenced() && ref->tv()->m_data.parr != adIn) {
+        cellDup(*ref->tv(), *pTv);
+        continue;
+      }
+    }
+    tvRefcountedIncRef(pTv);
+  }
+}
 
 NEVER_INLINE
 ArrayData* PackedArray::Copy(const ArrayData* adIn) {
   assert(checkInvariants(adIn));
 
-  auto const cap = packedCodeToCap(adIn->m_packedCapCode);
-
+  auto const cap = adIn->cap();
   auto const ad = static_cast<ArrayData*>(
-    MM().objMallocLogged(sizeof(ArrayData) + cap * sizeof(TypedValue))
+    MM().objMalloc(sizeof(ArrayData) + cap * sizeof(TypedValue))
   );
-  auto const size = adIn->m_size;
-  auto const capCode = adIn->m_packedCapCode;
-  ad->m_kindAndSize = uint64_t{size} << 32 | capCode; // zero kind
-  ad->m_posAndCount = static_cast<uint32_t>(adIn->m_pos);
 
-  auto const srcData = packedData(adIn);
-  auto const stop    = srcData + size;
-  auto targetData    = reinterpret_cast<TypedValue*>(ad + 1);
-  for (auto ptr = srcData; ptr != stop; ++ptr, ++targetData) {
-    tvDupFlattenVars(ptr, targetData, adIn);
-  }
+  CopyPackedHelper(adIn, ad, 1);
 
-  assert(ad->m_kind == ArrayData::kPackedKind);
-  assert(packedCodeToCap(ad->m_packedCapCode) == cap);
-  assert(ad->m_size == size);
+  assert(ad->isPacked());
+  assert(ad->cap() == adIn->cap());
+  assert(ad->m_size == adIn->m_size);
   assert(ad->m_pos == adIn->m_pos);
-  assert(ad->m_count == 0);
+  assert(ad->hasExactlyOneRef());
   assert(checkInvariants(ad));
   return ad;
 }
@@ -327,144 +350,132 @@ ArrayData* PackedArray::CopyWithStrongIterators(const ArrayData* ad) {
   return cpy;
 }
 
-ArrayData* PackedArray::NonSmartCopy(const ArrayData* adIn) {
+NEVER_INLINE
+ArrayData* PackedArray::CopyStatic(const ArrayData* adIn) {
   assert(checkInvariants(adIn));
 
   ArrayData* ad;
-  if (LIKELY(adIn->m_size <= kPackedCapCodeThreshold)) {
-    // There's no reason to use the full capacity, since non-smart
+  if (LIKELY(adIn->m_size <= CapCode::Threshold)) {
+    // There's no reason to use the full capacity, since static/uncounted
     // arrays are not mutable.
     auto const cap = adIn->m_size;
     ad = static_cast<ArrayData*>(
       std::malloc(sizeof(ArrayData) + cap * sizeof(TypedValue))
     );
-    auto const size = adIn->m_size;
-    assert(cap == packedCodeToCap(cap));
-    ad->m_kindAndSize = uint64_t{size} << 32 | cap; // zero kind
-    assert(ad->m_kind == ArrayData::kPackedKind);
-    assert(packedCodeToCap(ad->m_packedCapCode) == cap);
-    assert(ad->m_size == size);
+    assert(cap == CapCode::ceil(cap).code);
+    ad->m_sizeAndPos = adIn->m_sizeAndPos;
   } else {
-    ad = NonSmartCopyHelper(adIn);
-  }
-
-  ad->m_posAndCount  = static_cast<uint32_t>(adIn->m_pos);
-  auto const srcData = packedData(adIn);
-  auto const size    = adIn->m_size;
-  auto const stop    = srcData + size;
-  auto targetData    = reinterpret_cast<TypedValue*>(ad + 1);
-  for (auto ptr = srcData; ptr != stop; ++ptr, ++targetData) {
-    tvDupFlattenVars(ptr, targetData, adIn);
+    ad = CopyStaticHelper(adIn);
   }
+  CopyPackedHelper(adIn, ad, StaticValue);
 
+  assert(ad->isPacked());
+  assert(ad->cap() == adIn->cap());
+  assert(ad->m_size == adIn->m_size);
   assert(ad->m_pos == adIn->m_pos);
-  assert(ad->m_count == 0);
+  assert(ad->isStatic());
   assert(checkInvariants(ad));
   return ad;
 }
 
 NEVER_INLINE
-ArrayData* PackedArray::NonSmartCopyHelper(const ArrayData* adIn) {
-  auto const cap = roundUpPackedCap(adIn->m_size);
+ArrayData* PackedArray::CopyStaticHelper(const ArrayData* adIn) {
+  auto const fpcap = CapCode::ceil(adIn->m_size);
+  auto const cap = fpcap.decode();
   auto const ad = static_cast<ArrayData*>(
     std::malloc(sizeof(ArrayData) + cap * sizeof(TypedValue))
   );
-  auto const capCode = packedCapToCode(cap);
-  auto const size = adIn->m_size;
-  ad->m_kindAndSize = uint64_t{size} << 32 | capCode; // zero kind
-  assert(ad->m_kind == ArrayData::kPackedKind);
-  assert(packedCodeToCap(ad->m_packedCapCode) == cap);
-  assert(ad->m_size == size);
+  ad->m_sizeAndPos = adIn->m_sizeAndPos;
+  ad->m_hdr.init(fpcap, HeaderKind::Packed, StaticValue);
+  assert(ad->isPacked());
+  assert(ad->cap() == cap);
+  assert(ad->m_size == adIn->m_size);
+  assert(ad->isStatic());
   return ad;
 }
 
-ArrayData* PackedArray::NonSmartConvert(const ArrayData* arr) {
+ArrayData* PackedArray::ConvertStatic(const ArrayData* arr) {
   assert(arr->isVectorData());
 
   ArrayData* ad;
-  if (LIKELY(arr->m_size <= kPackedCapCodeThreshold)) {
+  if (LIKELY(arr->m_size <= CapCode::Threshold)) {
     auto const cap = arr->m_size;
     ad = static_cast<ArrayData*>(
       std::malloc(sizeof(ArrayData) + cap * sizeof(TypedValue))
     );
-    auto const size = arr->m_size;
-    assert(cap == packedCodeToCap(cap));
-    ad->m_kindAndSize = uint64_t{size} << 32 | cap; // zero kind
-    assert(ad->m_kind == ArrayData::kPackedKind);
-    assert(packedCodeToCap(ad->m_packedCapCode) == cap);
-    assert(ad->m_size == size);
+    assert(cap == CapCode::ceil(cap).code);
+    ad->m_sizeAndPos = arr->m_sizeAndPos;
+    ad->m_hdr.init(CapCode::exact(cap), HeaderKind::Packed, StaticValue);
+    assert(ad->isPacked());
+    assert(ad->cap() == cap);
+    assert(ad->m_size == arr->m_size);
   } else {
-    ad = NonSmartConvertHelper(arr);
+    ad = ConvertStaticHelper(arr);
   }
-
-  ad->m_posAndCount = static_cast<uint32_t>(arr->m_pos);
   auto data = reinterpret_cast<TypedValue*>(ad + 1);
-  for (auto pos = arr->iter_begin();
-      pos != ArrayData::invalid_index;
-      pos = arr->iter_advance(pos), ++data) {
+  auto pos_limit = arr->iter_end();
+  for (auto pos = arr->iter_begin(); pos != pos_limit;
+       pos = arr->iter_advance(pos), ++data) {
     tvDupFlattenVars(arr->getValueRef(pos).asTypedValue(), data, arr);
   }
-
   assert(ad->m_pos == arr->m_pos);
-  assert(ad->m_count == 0);
+  assert(ad->isStatic());
   assert(checkInvariants(ad));
   return ad;
 }
 
 NEVER_INLINE
-ArrayData* PackedArray::NonSmartConvertHelper(const ArrayData* arr) {
-  auto const cap = roundUpPackedCap(arr->m_size);
+ArrayData* PackedArray::ConvertStaticHelper(const ArrayData* arr) {
+  auto const fpcap = CapCode::ceil(arr->m_size);
+  auto const cap = fpcap.decode();
   auto const ad = static_cast<ArrayData*>(
     std::malloc(sizeof(ArrayData) + cap * sizeof(TypedValue))
   );
-  auto const capCode = packedCapToCode(cap);
-  auto const size = arr->m_size;
-  ad->m_kindAndSize = uint64_t{size} << 32 | capCode; // zero kind
-  assert(ad->m_kind == ArrayData::kPackedKind);
-  assert(packedCodeToCap(ad->m_packedCapCode) == cap);
-  assert(ad->m_size == size);
+  ad->m_sizeAndPos = arr->m_sizeAndPos;
+  ad->m_hdr.init(fpcap, HeaderKind::Packed, StaticValue);
+  assert(ad->isPacked());
+  assert(ad->cap() == cap);
+  assert(ad->m_size == arr->m_size);
+  assert(ad->isStatic());
   return ad;
 }
 
-//////////////////////////////////////////////////////////////////////
-
 ArrayData* MixedArray::MakeReserve(uint32_t capacity) {
   ArrayData* ad;
-  if (LIKELY(capacity <= kPackedCapCodeThreshold)) {
+  if (LIKELY(capacity <= CapCode::Threshold)) {
     auto const kSmallSize = MixedArray::SmallSize;
     auto const cap = std::max(capacity, kSmallSize);
     ad = static_cast<ArrayData*>(
-      MM().objMallocLogged(sizeof(ArrayData) + sizeof(TypedValue) * cap)
+      MM().objMalloc(sizeof(ArrayData) + sizeof(TypedValue) * cap)
     );
-    assert(cap == packedCodeToCap(cap));
-    ad->m_kindAndSize = cap;    // zeros m_size and m_kind
-    assert(ad->m_kind == kPackedKind);
-    assert(packedCodeToCap(ad->m_packedCapCode) == cap);
+    assert(cap == CapCode::ceil(cap).code);
+    ad->m_sizeAndPos = 0; // size=0, pos=0
+    ad->m_hdr.init(CapCode::exact(cap), HeaderKind::Packed, 1);
+    assert(ad->isPacked());
+    assert(ad->cap() == cap);
     assert(ad->m_size == 0);
   } else {
-    ad = MakeReserveSlow(capacity);
+    ad = MakeReserveSlow(capacity); // size=pos=kind=0
   }
 
-  ad->m_posAndCount = uint64_t{1} << 32 |
-                        static_cast<uint32_t>(ArrayData::invalid_index);
-
-  assert(ad->m_count == 1);
-  assert(ad->m_pos == ArrayData::invalid_index);
+  assert(ad->hasExactlyOneRef());
+  assert(ad->m_pos == 0);
   assert(PackedArray::checkInvariants(ad));
   return ad;
 }
 
 NEVER_INLINE
 ArrayData* MixedArray::MakeReserveSlow(uint32_t capacity) {
-  auto const cap = roundUpPackedCap(capacity);
-  auto const ad = static_cast<ArrayData*>(
-    MM().objMallocLogged(sizeof(ArrayData) + sizeof(TypedValue) * cap)
-  );
-  auto const capCode = packedCapToCode(cap);
-  ad->m_kindAndSize = capCode;    // zeros m_size and m_kind
-  assert(ad->m_kind == kPackedKind);
-  assert(packedCodeToCap(ad->m_packedCapCode) == cap);
+  auto const fpcap = CapCode::ceil(capacity);
+  auto const cap = fpcap.decode();
+  auto const requestSize = sizeof(ArrayData) + sizeof(TypedValue) * cap;
+  auto const ad = static_cast<ArrayData*>(MM().objMalloc(requestSize));
+  ad->m_sizeAndPos = 0;
+  ad->m_hdr.init(fpcap, HeaderKind::Packed, 1);
+  assert(ad->isPacked());
+  assert(ad->cap() == cap);
   assert(ad->m_size == 0);
+  assert(ad->hasExactlyOneRef());
   return ad;
 }
 
@@ -472,19 +483,18 @@ NEVER_INLINE
 void PackedArray::Release(ArrayData* ad) {
   assert(checkInvariants(ad));
   assert(ad->isRefCounted());
+  assert(ad->hasExactlyOneRef());
 
   auto const size = ad->m_size;
   auto const data = packedData(ad);
   auto const stop = data + size;
   for (auto ptr = data; ptr != stop; ++ptr) {
-    tvRefcountedDecRef(*ptr);
+    tvRefcountedDecRef(ptr);
   }
   if (UNLIKELY(strong_iterators_exist())) {
     free_strong_iterators(ad);
   }
-
-  auto const cap = packedCodeToCap(ad->m_packedCapCode);
-  MM().objFreeLogged(ad, sizeof(ArrayData) + sizeof(TypedValue) * cap);
+  MM().objFree(ad, heapSize(ad));
 }
 
 const TypedValue* PackedArray::NvGetInt(const ArrayData* ad, int64_t ki) {
@@ -492,18 +502,26 @@ const TypedValue* PackedArray::NvGetInt(const ArrayData* ad, int64_t ki) {
   return LIKELY(size_t(ki) < ad->m_size) ? &data[ki] : nullptr;
 }
 
+const TypedValue*
+PackedArray::NvGetStr(const ArrayData* ad, const StringData* s) {
+  return nullptr;
+}
+
 void PackedArray::NvGetKey(const ArrayData* ad, TypedValue* out, ssize_t pos) {
   assert(checkInvariants(ad));
-  assert(pos != ArrayData::invalid_index);
+  assert(pos != ad->m_size);
   out->m_data.num = pos;
   out->m_type = KindOfInt64;
 }
 
-size_t PackedArray::Vsize(const ArrayData*) { not_reached(); }
+size_t PackedArray::Vsize(const ArrayData*) {
+  // PackedArray always has a valid m_size so it's an error to get here.
+  always_assert(false);
+}
 
 const Variant& PackedArray::GetValueRef(const ArrayData* ad, ssize_t pos) {
   assert(checkInvariants(ad));
-  assert(pos != ArrayData::invalid_index);
+  assert(pos != ad->m_size);
   return tvAsCVarRef(&packedData(ad)[pos]);
 }
 
@@ -512,6 +530,10 @@ bool PackedArray::ExistsInt(const ArrayData* ad, int64_t k) {
   return size_t(k) < ad->m_size;
 }
 
+bool PackedArray::ExistsStr(const ArrayData* ad, const StringData* s) {
+  return false;
+}
+
 ArrayData* PackedArray::LvalInt(ArrayData* adIn,
                                 int64_t k,
                                 Variant*& ret,
@@ -548,14 +570,16 @@ ArrayData* PackedArray::LvalNew(ArrayData* adIn, Variant*& ret, bool copy) {
   assert(checkInvariants(adIn));
   auto const ad = copy ? CopyAndResizeIfNeeded(adIn)
                        : ResizeIfNeeded(adIn);
-  if (UNLIKELY(!ad)) {
-    auto const mixed = copy ? ToMixedCopy(adIn) : ToMixed(ad);
-    return MixedArray::LvalNew(mixed, ret, copy);
-  }
+  auto& tv = packedData(ad)[ad->m_size++];
+  tv.m_type = KindOfNull;
+  ret = &tvAsVariant(&tv);
+  return ad;
+}
 
-  if (ad->m_pos == ArrayData::invalid_index) {
-    ad->m_pos = ad->m_size;
-  }
+ArrayData* PackedArray::LvalNewRef(ArrayData* adIn, Variant*& ret, bool copy) {
+  assert(checkInvariants(adIn));
+  auto const ad = copy ? CopyAndResizeIfNeeded(adIn)
+                       : ResizeIfNeeded(adIn);
   auto& tv = packedData(ad)[ad->m_size++];
   tv.m_type = KindOfNull;
   ret = &tvAsVariant(&tv);
@@ -572,12 +596,12 @@ PackedArray::SetInt(ArrayData* adIn, int64_t k, Cell v, bool copy) {
   if (size_t(k) < adIn->m_size) {
     auto const ad = copy ? Copy(adIn) : adIn;
     auto& dst = *tvToCell(&packedData(ad)[k]);
-    cellSet(v, dst);
     // TODO(#3888164): we should restructure things so we don't have to
     // check KindOfUninit here.
-    if (UNLIKELY(dst.m_type == KindOfUninit)) {
-      dst.m_type = KindOfNull;
+    if (UNLIKELY(v.m_type == KindOfUninit)) {
+      v = make_tv<KindOfNull>();
     }
+    cellSet(v, dst);
     return ad;
   }
 
@@ -591,9 +615,7 @@ PackedArray::SetInt(ArrayData* adIn, int64_t k, Cell v, bool copy) {
   return mixed->addVal(k, v);
 }
 
-ArrayData* PackedArray::SetStr(ArrayData* adIn,
-                               StringData* k,
-                               Cell v,
+ArrayData* PackedArray::SetStr(ArrayData* adIn, StringData* k, Cell v,
                                bool copy) {
   // We must convert to mixed, but can call addVal since the key must
   // not exist.
@@ -601,9 +623,7 @@ ArrayData* PackedArray::SetStr(ArrayData* adIn,
   return mixed->addVal(k, v);
 }
 
-ArrayData* PackedArray::SetRefInt(ArrayData* adIn,
-                                  int64_t k,
-                                  Variant& v,
+ArrayData* PackedArray::SetRefInt(ArrayData* adIn, int64_t k, Variant& v,
                                   bool copy) {
   assert(checkInvariants(adIn));
 
@@ -653,43 +673,50 @@ PackedArray::RemoveStr(ArrayData* adIn, const StringData*, bool) {
 
 ssize_t PackedArray::IterBegin(const ArrayData* ad) {
   assert(checkInvariants(ad));
-  return ad->m_size ? 0 : ArrayData::invalid_index;
+  return 0;
+}
+
+ssize_t PackedArray::IterLast(const ArrayData* ad) {
+  assert(checkInvariants(ad));
+  return ad->m_size ? ad->m_size - 1 : 0;
 }
 
 ssize_t PackedArray::IterEnd(const ArrayData* ad) {
   assert(checkInvariants(ad));
-  static_assert(ArrayData::invalid_index == -1, "");
-  return static_cast<ssize_t>(ad->m_size) - 1;
+  return ad->m_size;
 }
 
 ssize_t PackedArray::IterAdvance(const ArrayData* ad, ssize_t pos) {
   assert(checkInvariants(ad));
-  if (size_t(++pos) < ad->m_size) {
-    return pos;
+  if (pos < ad->m_size) {
+    ++pos;
   }
-  return ArrayData::invalid_index;
+  return pos;
 }
 
 ssize_t PackedArray::IterRewind(const ArrayData* ad, ssize_t pos) {
   assert(checkInvariants(ad));
-  if (pos == ArrayData::invalid_index) return ArrayData::invalid_index;
-  return pos - 1;
+  if (pos > 0) {
+    return pos - 1;
+  }
+  return ad->m_size;
 }
 
 bool PackedArray::AdvanceMArrayIter(ArrayData* ad, MArrayIter& fp) {
   assert(checkInvariants(ad));
   if (fp.getResetFlag()) {
     fp.setResetFlag(false);
-    fp.m_pos = ArrayData::invalid_index;
-  } else if (fp.m_pos == ArrayData::invalid_index) {
+    fp.m_pos = 0;
+  } else if (fp.m_pos == ad->m_size) {
+    return false;
+  } else {
+    fp.m_pos = IterAdvance(ad, fp.m_pos);
+  }
+  if (fp.m_pos == ad->m_size) {
     return false;
   }
-  fp.m_pos = IterAdvance(ad, fp.m_pos);
-  if (fp.m_pos == ArrayData::invalid_index) {
-    return false;
-  }
-  // To conform to PHP behavior, we need to set the internal
-  // cursor to point to the next element.
+  // We set ad's internal cursor to point to the next element
+  // to conform with PHP5 behavior
   ad->m_pos = IterAdvance(ad, fp.m_pos);
   return true;
 }
@@ -698,14 +725,6 @@ ArrayData* PackedArray::Append(ArrayData* adIn, const Variant& v, bool copy) {
   assert(checkInvariants(adIn));
   auto const ad = copy ? CopyAndResizeIfNeeded(adIn)
                        : ResizeIfNeeded(adIn);
-  if (UNLIKELY(!ad)) {
-    auto const mixed = copy ? ToMixedCopy(adIn) : ToMixed(adIn);
-    return MixedArray::Append(mixed, v, copy);
-  }
-
-  if (ad->m_pos == ArrayData::invalid_index) {
-    ad->m_pos = ad->m_size;
-  }
   auto& dst = packedData(ad)[ad->m_size++];
   cellDup(*v.asCell(), dst);
   // TODO(#3888164): restructure this so we don't need KindOfUninit checks.
@@ -719,14 +738,6 @@ ArrayData* PackedArray::AppendRef(ArrayData* adIn,
   assert(checkInvariants(adIn));
   auto const ad = copy ? CopyAndResizeIfNeeded(adIn)
                        : ResizeIfNeeded(adIn);
-  if (UNLIKELY(!ad)) {
-    auto const mixed = copy ? ToMixedCopy(adIn) : ToMixed(adIn);
-    return MixedArray::AppendRef(mixed, v, copy);
-  }
-
-  if (ad->m_pos == ArrayData::invalid_index) {
-    ad->m_pos = ad->m_size;
-  }
   auto& dst = packedData(ad)[ad->m_size++];
   dst.m_data.pref = v.asRef()->m_data.pref;
   dst.m_type = KindOfRef;
@@ -740,15 +751,6 @@ ArrayData* PackedArray::AppendWithRef(ArrayData* adIn,
   assert(checkInvariants(adIn));
   auto const ad = copy ? CopyAndResizeIfNeeded(adIn)
                        : ResizeIfNeeded(adIn);
-  if (UNLIKELY(!ad)) {
-    auto const mixed = copy ? ToMixedCopy(adIn) : ToMixed(adIn);
-    // XXX: constness
-    return MixedArray::AppendRef(mixed, const_cast<Variant&>(v), copy);
-  }
-
-  if (ad->m_pos == ArrayData::invalid_index) {
-    ad->m_pos = ad->m_size;
-  }
   auto& dst = packedData(ad)[ad->m_size++];
   dst.m_type = KindOfNull;
   tvAsVariant(&dst).setWithRef(v);
@@ -762,7 +764,7 @@ ArrayData* PackedArray::PlusEq(ArrayData* adIn, const ArrayData* elems) {
   try {
     auto const ret = MixedArray::PlusEq(mixed, elems);
     assert(ret == mixed);
-    assert(!mixed->hasMultipleRefs());
+    assert(mixed->hasExactlyOneRef());
     return ret;
   } catch (...) {
     MixedArray::Release(mixed);
@@ -778,12 +780,14 @@ ArrayData* PackedArray::Merge(ArrayData* adIn, const ArrayData* elems) {
 }
 
 static void adjustMArrayIter(ArrayData* ad, ssize_t pos) {
+  assert(ad->isPacked());
   for_each_strong_iterator([&] (MIterTable::Ent& miEnt) {
     if (miEnt.array != ad) return;
     auto const iter = miEnt.iter;
+    if (iter->getResetFlag()) return;
     if (iter->m_pos == pos) {
-      if (pos - 1 < 0) {
-        iter->m_pos = ArrayData::invalid_index;
+      if (pos <= 0) {
+        iter->m_pos = ad->getSize();
         iter->setResetFlag(true);
       } else {
         iter->m_pos = pos - 1;
@@ -795,11 +799,11 @@ static void adjustMArrayIter(ArrayData* ad, ssize_t pos) {
 ArrayData* PackedArray::Pop(ArrayData* adIn, Variant& value) {
   assert(checkInvariants(adIn));
 
-  auto const ad = adIn->hasMultipleRefs() ? Copy(adIn) : adIn;
+  auto const ad = adIn->cowCheck() ? Copy(adIn) : adIn;
 
   if (UNLIKELY(ad->m_size == 0)) {
+    assert(ad->m_pos == 0);
     value = uninit_null();
-    ad->m_pos = ArrayData::invalid_index;
     return ad;
   }
 
@@ -812,7 +816,7 @@ ArrayData* PackedArray::Pop(ArrayData* adIn, Variant& value) {
   auto const oldType = tv.m_type;
   auto const oldDatum = tv.m_data.num;
   ad->m_size = oldSize - 1;
-  ad->m_pos = oldSize - 1 > 0 ? 0 : ArrayData::invalid_index;
+  ad->m_pos = 0;
   tvRefcountedDecRefHelper(oldType, oldDatum);
   return ad;
 }
@@ -820,7 +824,7 @@ ArrayData* PackedArray::Pop(ArrayData* adIn, Variant& value) {
 ArrayData* PackedArray::Dequeue(ArrayData* adIn, Variant& value) {
   assert(checkInvariants(adIn));
 
-  auto const ad = adIn->hasMultipleRefs() ? Copy(adIn) : adIn;
+  auto const ad = adIn->cowCheck() ? Copy(adIn) : adIn;
   // To conform to PHP behavior, we invalidate all strong iterators when an
   // element is removed from the beginning of the array.
   if (UNLIKELY(strong_iterators_exist())) {
@@ -829,7 +833,6 @@ ArrayData* PackedArray::Dequeue(ArrayData* adIn, Variant& value) {
 
   if (UNLIKELY(ad->m_size == 0)) {
     value = uninit_null();
-    ad->m_pos = ArrayData::invalid_index;
     return ad;
   }
 
@@ -840,7 +843,7 @@ ArrayData* PackedArray::Dequeue(ArrayData* adIn, Variant& value) {
   value = std::move(tvAsVariant(data)); // no incref+decref
   std::memmove(data, data + 1, n * sizeof *data);
   ad->m_size = n;
-  ad->m_pos = n > 0 ? 0 : ArrayData::invalid_index;
+  ad->m_pos = 0;
   return ad;
 }
 
@@ -849,8 +852,8 @@ ArrayData* PackedArray::Prepend(ArrayData* adIn,
                                 bool copy) {
   assert(checkInvariants(adIn));
 
-  auto const ad = adIn->hasMultipleRefs() ? CopyAndResizeIfNeeded(adIn)
-                                          : ResizeIfNeeded(adIn);
+  auto const ad = adIn->cowCheck() ? CopyAndResizeIfNeeded(adIn)
+                                   : ResizeIfNeeded(adIn);
   // To conform to PHP behavior, we invalidate all strong iterators when an
   // element is added to the beginning of the array.
   if (UNLIKELY(strong_iterators_exist())) {
@@ -876,35 +879,22 @@ void PackedArray::OnSetEvalScalar(ArrayData* ad) {
   }
 }
 
-ArrayData* PackedArray::EscalateForSort(ArrayData* ad) {
-  // Note: ToMixedCopy also grows so we have !isFull.  We could use
-  // ToMixedCopyReserve?
-  assert(checkInvariants(ad));
-  return ToMixedCopy(ad);
+void PackedArray::Ksort(ArrayData* ad, int flags, bool ascending) {
+  assert(ad->getSize() <= 1 || ascending);
 }
 
-void PackedArray::Ksort(ArrayData*, int, bool) {
-  not_reached();
+void PackedArray::Asort(ArrayData* ad, int, bool) {
+  assert(ad->getSize() <= 1);
 }
 
-void PackedArray::Sort(ArrayData*, int, bool) {
-  not_reached();
+bool PackedArray::Uksort(ArrayData* ad, const Variant&) {
+  assert(ad->getSize() <= 1);
+  return true;
 }
 
-void PackedArray::Asort(ArrayData*, int, bool) {
-  not_reached();
-}
-
-bool PackedArray::Uksort(ArrayData*, const Variant&) {
-  not_reached();
-}
-
-bool PackedArray::Usort(ArrayData*, const Variant&) {
-  not_reached();
-}
-
-bool PackedArray::Uasort(ArrayData*, const Variant&) {
-  not_reached();
+bool PackedArray::Uasort(ArrayData* ad, const Variant&) {
+  assert(ad->getSize() <= 1);
+  return true;
 }
 
 ArrayData* PackedArray::ZSetInt(ArrayData* ad, int64_t k, RefData* v) {
