@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 #ifndef incl_HPHP_RUNTIME_OPTION_H_
 #define incl_HPHP_RUNTIME_OPTION_H_
 
-#include "folly/dynamic.h"
+#include <folly/dynamic.h>
 
 #include <unordered_map>
 #include <vector>
@@ -40,8 +40,7 @@ struct IpBlockMap;
 struct SatelliteServerInfo;
 struct FilesMatch;
 struct Hdf;
-// Can we make sure this equals IniSetting::Map?
-typedef folly::dynamic IniSettingMap;
+class IniSettingMap;
 
 constexpr int kDefaultInitialStaticStringTableSize = 500000;
 
@@ -51,9 +50,11 @@ constexpr int kDefaultInitialStaticStringTableSize = 500000;
  */
 class RuntimeOption {
 public:
-  static void Load(const IniSettingMap &ini,
-                   Hdf& config,
-                   std::vector<std::string>* overwrites = nullptr);
+  static void Load(
+    IniSettingMap &ini, Hdf& config,
+    const std::vector<std::string>& iniClis = std::vector<std::string>(),
+    const std::vector<std::string>& hdfClis = std::vector<std::string>(),
+    std::vector<std::string>* messages = nullptr);
 
   static bool ServerExecutionMode() {
     return strcmp(ExecutionMode, "srv") == 0;
@@ -63,6 +64,14 @@ public:
     return strcmp(ExecutionMode, "cli") == 0;
   }
 
+  static void ReadSatelliteInfo(
+    const IniSettingMap& ini,
+    const Hdf& hdf,
+    std::vector<std::shared_ptr<SatelliteServerInfo>>& infos,
+    std::string& xboxPassword,
+    std::set<std::string>& xboxPasswords
+  );
+
   static const char *ExecutionMode;
   static std::string BuildId;
   static std::string InstanceId;
@@ -74,9 +83,12 @@ public:
   static bool AlwaysEscapeLog;
   static bool AlwaysLogUnhandledExceptions;
   static bool NoSilencer;
+  static int ErrorUpgradeLevel; // Bitmask of errors to upgrade to E_USER_ERROR
   static bool CallUserHandlerOnFatals;
   static bool ThrowExceptionOnBadMethodCall;
+  static bool LogNativeStackOnOOM;
   static int RuntimeErrorReportingLevel;
+  static int ForceErrorReportingLevel; // Bitmask ORed with the reporting level
 
   static std::string ServerUser; // run server under this user account
 
@@ -85,8 +97,7 @@ public:
   static bool NoInfiniteRecursionDetection;
   static bool WarnTooManyArguments;
   static bool EnableHipHopErrors;
-  static bool AssertActive;
-  static bool AssertWarning;
+  static bool AssertEmitted;
   static int64_t NoticeFrequency; // output 1 out of NoticeFrequency notices
   static int64_t WarningFrequency;
   static int RaiseDebuggingFrequency;
@@ -94,24 +105,27 @@ public:
   static int64_t StringOffsetLimit;
 
   static std::string AccessLogDefaultFormat;
-  static std::vector<AccessLogFileData> AccessLogs;
+  static std::map<std::string, AccessLogFileData> AccessLogs;
 
   static std::string AdminLogFormat;
   static std::string AdminLogFile;
   static std::string AdminLogSymLink;
 
-  static std::string Tier;
+  static std::map<std::string, AccessLogFileData> RPCLogs;
+
   static std::string Host;
   static std::string DefaultServerNameSuffix;
   static std::string ServerType;
   static std::string ServerIP;
   static std::string ServerFileSocket;
-  static std::string ServerPrimaryIP;
+  static const std::string& GetServerPrimaryIPv4();
+  static const std::string& GetServerPrimaryIPv6();
   static int ServerPort;
   static int ServerPortFd;
   static int ServerBacklog;
   static int ServerConnectionLimit;
   static int ServerThreadCount;
+  static int QueuedJobsReleaseRate;
   static int ServerWarmupThrottleRequestCount;
   static bool ServerThreadRoundRobin;
   static int ServerThreadDropCacheTimeoutSeconds;
@@ -121,6 +135,7 @@ public:
   static bool ServerHttpSafeMode;
   static bool ServerStatCache;
   static bool ServerFixPathInfo;
+  static bool ServerAddVaryEncoding;
   static std::vector<std::string> ServerWarmupRequests;
   static boost::container::flat_set<std::string> ServerHighPriorityEndPoints;
   static bool ServerExitOnBindFail;
@@ -130,9 +145,10 @@ public:
   static int PageletServerQueueLimit;
   static bool PageletServerThreadDropStack;
 
-  static int FiberCount;
   static int RequestTimeoutSeconds;
   static int PspTimeoutSeconds;
+  static int PspCpuTimeoutSeconds;
+  static int64_t MaxRequestAgeFactor;
   static int64_t ServerMemoryHeadRoom;
   static int64_t RequestMemoryMaxBytes;
   static int64_t ImageMemoryMaxBytes;
@@ -146,10 +162,10 @@ public:
   static int ServerShutdownListenNoWork;
   static std::vector<std::string> ServerNextProtocols;
   static int GzipCompressionLevel;
+  static int GzipMaxCompressionLevel;
   static std::string ForceCompressionURL;
   static std::string ForceCompressionCookie;
   static std::string ForceCompressionParam;
-  static bool EnableMagicQuotesGpc;
   static bool EnableKeepAlive;
   static bool ExposeHPHP;
   static bool ExposeXFBServer;
@@ -167,6 +183,7 @@ public:
   static std::string UploadTmpDir;
   static bool EnableFileUploads;
   static bool EnableUploadProgress;
+  static int64_t MaxFileUploads;
   static int Rfc1867Freq;
   static std::string Rfc1867Prefix;
   static std::string Rfc1867Name;
@@ -227,13 +244,10 @@ public:
   static bool Utf8izeReplace;
 
   static std::string StartupDocument;
-  static std::string WarmupDocument;
   static std::string RequestInitFunction;
   static std::string RequestInitDocument;
   static std::string AutoPrependFile;
   static std::string AutoAppendFile;
-  static std::vector<std::string> ThreadDocuments;
-  static std::vector<std::string> ThreadLoopDocuments;
 
   static bool SafeFileAccess;
   static std::vector<std::string> AllowedDirectories;
@@ -257,12 +271,17 @@ public:
   static std::string AdminPassword;
   static std::set<std::string> AdminPasswords;
 
-  static std::string ProxyOrigin;
+  /*
+   * Options related to reverse proxying. ProxyOriginRaw and ProxyPercentageRaw
+   * may be mutated by background threads and should only be read or written
+   * using the helper functions defined with HttpRequestHandler.
+   */
+  static std::string ProxyOriginRaw;
+  static int ProxyPercentageRaw;
   static int ProxyRetry;
   static bool UseServeURLs;
   static std::set<std::string> ServeURLs;
   static bool UseProxyURLs;
-  static int ProxyPercentage;
   static std::set<std::string> ProxyURLs;
   static std::vector<std::string> ProxyPatterns;
   static bool AlwaysUseRelativePath;
@@ -280,16 +299,15 @@ public:
   static std::string ProfilerOutputDir;
   static std::string CoreDumpEmail;
   static bool CoreDumpReport;
-  static std::string CoreDumpReportDirectory;
+  static std::string StackTraceFilename;
   static bool LocalMemcache;
   static bool MemcacheReadOnly;
+  static int StackTraceTimeout;
 
   static bool EnableStats;
   static bool EnableAPCStats;
   static bool EnableWebStats;
   static bool EnableMemoryStats;
-  static bool EnableMemcacheStats;
-  static bool EnableMemcacheKeyStats;
   static bool EnableSQLStats;
   static bool EnableSQLTableStats;
   static bool EnableNetworkIOStatus;
@@ -314,8 +332,7 @@ public:
   static bool WarnOnCollectionToArray;
   static bool UseDirectCopy;
 
-  static bool EnableDnsCache;
-  static int DnsCacheTTL;
+  static bool DisableSmallAllocator;
 
   static std::map<std::string, std::string> ServerVariables;
 
@@ -333,29 +350,52 @@ public:
   static bool EnableAspTags;
   static bool EnableXHP;
   static bool EnableObjDestructCall;
-  static bool EnableEmitSwitch;
   static bool EnableEmitterStats;
+  static bool EnableIntrinsicsExtension;
   static bool CheckSymLink;
-  static int MaxUserFunctionId;
   static bool EnableArgsInBacktraces;
   static bool EnableZendCompat;
   static bool EnableZendSorting;
+  static bool EnableZendIniCompat;
   static bool TimeoutsUseWallTime;
   static bool CheckFlushOnUserClose;
   static bool EvalAuthoritativeMode;
   static bool IntsOverflowToInts;
   static HackStrictOption StrictArrayFillKeys;
   static HackStrictOption DisallowDynamicVarEnvFuncs;
+  static HackStrictOption IconvIgnoreCorrect;
+  static HackStrictOption MinMaxAllowDegenerate;
+  static bool LookForTypechecker;
+  static bool AutoTypecheck;
+  static bool AutoprimeGenerators;
+
+  // ENABLED (1) selects PHP7 behavior.
+  static bool PHP7_DeprecateOldStyleCtors;
+  static bool PHP7_IntSemantics;
+  static bool PHP7_LTR_assign;
+  static bool PHP7_NoHexNumerics;
+  static bool PHP7_ScalarTypes;
+  static bool PHP7_EngineExceptions;
+  static bool PHP7_UVS;
+
+  static int64_t HeapSizeMB;
+  static int64_t HeapResetCountBase;
+  static int64_t HeapResetCountMultiple;
+  static int64_t HeapLowWaterMark;
+  static int64_t HeapHighWaterMark;
 
   static int GetScannerType();
 
-  static bool GetServerCustomBoolSetting(const std::string &settingName,
-                                         bool &val);
-
   static std::set<std::string, stdltistr> DynamicInvokeFunctions;
 
   static const uint32_t kPCREInitialTableSize = 96 * 1024;
 
+  static std::string ExtensionDir;
+  static std::vector<std::string> Extensions;
+  static std::string DynamicExtensionPath;
+  static std::vector<std::string> DynamicExtensions;
+
+
 #define EVALFLAGS()                                                     \
   /* F(type, name, defaultVal) */                                       \
   /*                                                                    \
@@ -369,46 +409,88 @@ public:
   F(uint32_t, VMInitialGlobalTableSize,                                 \
     kEvalVMInitialGlobalTableSizeDefault)                               \
   F(bool, Jit,                         evalJitDefault())                \
+  F(bool, JitEvaledCode,               true)                            \
   F(bool, SimulateARM,                 simulateARMDefault())            \
+  F(uint32_t, JitLLVM,                 jitLLVMDefault())                \
+  F(uint32_t, JitLLVMKeepSize,         0)                               \
+  F(uint32_t, JitLLVMOptLevel,         2)                               \
+  F(uint32_t, JitLLVMSizeLevel,        0)                               \
+  F(bool,     JitLLVMBBVectorize,      false)                           \
+  F(bool,     JitLLVMBasicOpt,         true)                            \
+  F(string,   JitLLVMCompare,          "")                              \
+  F(bool,     JitLLVMCondTail,         true)                            \
+  F(bool,     JitLLVMCounters,         false)                           \
+  F(bool,     JitLLVMDiscard,          false)                           \
+  F(bool,     JitLLVMFastISel,         false)                           \
+  F(bool,     JitLLVMLoadCombine,      false)                           \
+  F(bool,     JitLLVMMinSize,          true)                            \
+  F(bool,     JitLLVMOptSize,          true)                            \
+  F(bool,     JitLLVMPrintAfterAll,    false)                           \
+  F(bool,     JitLLVMRetOpt,           true)                            \
+  F(bool,     JitLLVMSLPVectorize,     jitLLVMSLPVectorizeDefault())    \
+  F(uint32_t, JitLLVMSplitHotCold,     1)                               \
+  F(bool,     JitLLVMVolatileIncDec,   true)                            \
+  F(string,   JitLLVMAttrs,            "")                              \
+  F(string,   JitCPU,                  "native")                        \
   F(bool, JitRequireWriteLease,        false)                           \
-  F(uint64_t, JitAHotSize,             4 << 20)                         \
-  F(uint64_t, JitASize,                60 << 20)                        \
-  F(uint64_t, JitAMaxUsage,            maxUsageDef())                   \
-  F(uint64_t, JitAProfSize,            64 << 20)                        \
-  F(uint64_t, JitAColdSize,            24 << 20)                        \
-  F(uint64_t, JitAFrozenSize,          40 << 20)                        \
-  F(uint64_t, JitGlobalDataSize,       kJitGlobalDataDef)               \
-  F(uint64_t, JitRelocationSize,       1 << 20)                         \
+  F(uint64_t, JitRelocationSize,       kJitRelocationSizeDefault)       \
+  F(bool, JitTimer,                    kJitTimerDefault)                \
+  F(bool, RecordSubprocessTimes,       false)                           \
   F(bool, AllowHhas,                   false)                           \
+  F(string, UseExternalEmitter,        "")                              \
+  /* ExternalEmitterFallback:
+     0 - No fallback; fail when external emitter fails
+     1 - Fallback to builtin emitter if external emitter fails,
+         but log a diagnostic
+     2 - Fallback to builtin emitter if external emitter fails and
+         don't log anything */                                          \
+  F(int, ExternalEmitterFallback,      0)                               \
+  F(bool, ExternalEmitterAllowPartial, false)                           \
+  F(bool, EmitSwitch,                  true)                            \
+  F(bool, EmitNewMInstrs,              newMInstrsDefault())             \
+  F(bool, LogThreadCreateBacktraces,   false)                           \
+  F(bool, FailJitPrologs,              false)                           \
   /* CheckReturnTypeHints:
-     0 - no checks or enforcement
-     1 - raises E_WARNING if regular type hint or soft type hint fails
-     2 - raises E_RECOVERABLE_ERROR if regular type hint fails, raises
-         E_WARNING if soft type hint fails; note that in repo mode the
-         error handler is not allowed to resume on recoverable errors */ \
+     0 - No checks or enforcement for return type hints.
+     1 - Raises E_WARNING if a return type hint fails.
+     2 - Raises E_RECOVERABLE_ERROR if regular return type hint fails,
+         raises E_WARNING if soft return type hint fails. If a regular
+         return type hint fails, it's possible for execution to resume
+         normally if the user error handler doesn't throw and returns
+         something other than boolean false.
+     3 - Same as 2, except if a regular type hint fails the runtime
+         will not allow execution to resume normally; if the user
+         error handler returns something other than boolean false,
+         the runtime will throw a fatal error (this goes together
+         with Option::HardReturnTypeHints). */                          \
   F(int32_t, CheckReturnTypeHints,     2)                               \
+  F(bool, SoftClosureReturnTypeHints,  false)                           \
+  /* HackArrayWarnFrequency:
+     0 - no warnings
+     [1-UINT32_MAX] - raise warning every X times
+  */                                                                    \
+  F(uint32_t, HackArrayWarnFrequency,  0)                               \
+  F(bool, AllowScopeBinding,           true)                            \
   F(bool, JitNoGdb,                    true)                            \
   F(bool, SpinOnCrash,                 false)                           \
   F(uint32_t, DumpRingBufferOnCrash,   0)                               \
   F(bool, PerfPidMap,                  true)                            \
   F(bool, PerfDataMap,                 false)                           \
   F(bool, KeepPerfPidMap,              false)                           \
-  F(bool, RuntimeTypeProfile,          false)                           \
-  F(int32_t, RuntimeTypeProfileLoggingFreq,  0)                         \
+  F(int32_t, PerfRelocate,             0)                               \
   F(uint32_t, JitTargetCacheSize,      64 << 20)                        \
-  F(uint32_t, HHBCArenaChunkSize,      64 << 20)                        \
+  F(uint32_t, HHBCArenaChunkSize,      10 << 20)                        \
   F(bool, ProfileBC,                   false)                           \
   F(bool, ProfileHeapAcrossRequests,   false)                           \
   F(bool, ProfileHWEnable,             true)                            \
-  F(string, ProfileHWEvents,      std::string(""))                      \
+  F(string, ProfileHWEvents,           std::string(""))                 \
   F(bool, JitAlwaysInterpOne,          false)                           \
   F(uint32_t, JitMaxTranslations,      12)                              \
   F(uint64_t, JitGlobalTranslationLimit, -1)                            \
-  F(bool, JitTrampolines,              true)                            \
-  F(string, JitProfilePath,       std::string(""))                      \
-  F(bool, JitTypePrediction,           true)                            \
-  F(int32_t, JitStressTypePredPercent, 0)                               \
-  F(uint32_t, JitWarmupRequests,       kDefaultWarmupRequests)          \
+  F(uint32_t, JitMaxRegionInstrs,      1000)                            \
+  F(uint32_t, JitProfileInterpRequests, kDefaultProfileInterpRequests)  \
+  F(bool, JitProfileWarmupRequests,    false)                           \
+  F(uint32_t, NumSingleJitRequests,    nsjrDefault())                   \
   F(uint32_t, JitProfileRequests,      kDefaultProfileRequests)         \
   F(bool, JitProfileRecord,            false)                           \
   F(uint32_t, GdbSyncChunks,           128)                             \
@@ -423,68 +505,84 @@ public:
   F(bool, JitDisabledByHphpd,          false)                           \
   F(bool, JitTransCounters,            false)                           \
   F(bool, JitPseudomain,               jitPseudomainDefault())          \
-  F(bool, HHIRBytecodeControlFlow,     true)                            \
-  F(bool, HHIRCse,                     true)                            \
+  F(bool, HHIRLICM,                    false)                           \
   F(bool, HHIRSimplification,          true)                            \
   F(bool, HHIRGenOpts,                 true)                            \
-  F(bool, HHIRJumpOpts,                true)                            \
   F(bool, HHIRRefcountOpts,            true)                            \
-  F(bool, HHIRRefcountOptsAlwaysSink,  false)                           \
-  F(bool, HHIRExtraOptPass,            true)                            \
-  F(uint32_t, HHIRNumFreeRegs,         64)                              \
   F(bool, HHIREnableGenTimeInlining,   true)                            \
+  F(uint32_t, HHIRInliningMaxBindJmps, 0)                               \
+  F(uint32_t, HHIRInliningMaxReturns,  3)                               \
   F(uint32_t, HHIRInliningMaxCost,     13)                              \
-  F(uint32_t, HHIRAlwaysInlineMaxCost, 10)                              \
+  F(uint32_t, HHIRPGOInliningMaxCost,  6)                               \
   F(uint32_t, HHIRInliningMaxDepth,    4)                               \
-  F(uint32_t, HHIRInliningMaxReturnDecRefs, 3)                          \
+  F(uint32_t, HHIRInliningMaxReturnDecRefs, 6)                          \
   F(bool, HHIRInlineFrameOpts,         true)                            \
   F(bool, HHIRInlineSingletons,        true)                            \
-  /* 1 (the default) gives most asserts. 2 adds less commonly           \
-   * useful/more expensive asserts. */                                  \
-  F(uint32_t, HHIRGenerateAsserts,     debug)                           \
+  F(std::string, InlineRegionMode,     "both")                          \
+  F(bool, HHIRGenerateAsserts,         debug)                           \
   F(bool, HHIRDirectExit,              true)                            \
   F(bool, HHIRDeadCodeElim,            true)                            \
+  F(bool, HHIRGlobalValueNumbering,    true)                            \
+  F(bool, HHIRTypeCheckHoisting,       false) /* Task: 7568599 */       \
   F(bool, HHIRPredictionOpts,          true)                            \
-  F(bool, HHIRStressCodegenBlocks,     false)                           \
+  F(bool, HHIRMemoryOpts,              true)                            \
+  F(bool, HHIRStorePRE,                true)                            \
+  F(bool, HHIROutlineGenericIncDecRef, true)                            \
   /* Register allocation flags */                                       \
-  F(bool, HHIREnableCalleeSavedOpt,    true)                            \
   F(bool, HHIREnablePreColoring,       true)                            \
   F(bool, HHIREnableCoalescing,        true)                            \
   F(bool, HHIRAllocSIMDRegs,           true)                            \
+  F(bool, HHIRStressSpill,             false)                           \
   /* Region compiler flags */                                           \
-  F(bool,     JitLoops,                false)                           \
   F(string,   JitRegionSelector,       regionSelectorDefault())         \
   F(bool,     JitPGO,                  pgoDefault())                    \
-  F(string,   JitPGORegionSelector,    "hottrace")                      \
-  F(uint64_t, JitPGOThreshold,         kDefaultJitPGOThreshold)         \
-  F(bool,     JitPGOHotOnly,           ServerExecutionMode())           \
+  F(string,   JitPGORegionSelector,    pgoRegionSelectorDefault())      \
+  F(uint64_t, JitPGOThreshold,         pgoThresholdDefault())           \
+  F(bool,     JitPGOHotOnly,           false)                           \
+  F(bool,     JitPGOCFGHotFuncOnly,    false)                           \
   F(bool,     JitPGOUsePostConditions, true)                            \
   F(uint32_t, JitUnlikelyDecRefPercent,10)                              \
   F(uint32_t, JitPGOReleaseVVMinPercent, 10)                            \
-  F(uint32_t, HotFuncThreshold,        80)                              \
-  F(bool, HHIRValidateRefCount,        debug)                           \
-  F(bool, HHIRRelaxGuards,             true)                            \
+  F(bool,     JitPGOArrayGetStress,    false)                           \
+  F(uint32_t, JitPGOMinBlockCountPercent, 0)                            \
+  F(double,   JitPGOMinArcProbability, 0.0)                             \
+  F(uint32_t, JitPGOMaxFuncSizeDupBody, 80)                             \
+  F(uint32_t, JitPGORelaxPercent,      100)                             \
+  F(bool,     JitLoops,                true)                            \
+  F(uint32_t, HotFuncCount,            4100)                            \
+  F(bool, RegionRelaxGuards,           true)                            \
   /* DumpBytecode =1 dumps user php, =2 dumps systemlib & user php */   \
   F(int32_t, DumpBytecode,             0)                               \
   F(bool, DumpHhas,                    false)                           \
   F(bool, DumpTC,                      false)                           \
   F(bool, DumpTCAnchors,               false)                           \
+  F(uint32_t, DumpIR,                  0)                               \
+  F(bool, DumpRegion,                  false)                           \
   F(bool, DumpAst,                     false)                           \
-  F(bool, MapTCHuge,                   hugePagesSoundNice())            \
-  F(bool, MapHotTextHuge,              hugePagesSoundNice())            \
   F(bool, MapTgtCacheHuge,             false)                           \
+  F(uint32_t, MaxHotTextHugePages,     hugePagesSoundNice() ? 1 : 0)    \
   F(int32_t, MaxLowMemHugePages,       hugePagesSoundNice() ? 8 : 0)    \
-  F(uint32_t, TCNumHugeHotMB,          16)                              \
-  F(uint32_t, TCNumHugeColdMB,         4)                               \
   F(bool, RandomHotFuncs,              false)                           \
+  F(bool, CheckHeapOnAlloc,            false)                           \
+  F(bool, EnableGC,                    false)                           \
+  /*
+    Run GC on every allocation/deallocation with probability 1/N (0 to
+    disable). Requires EnableGC=true with debug build.
+  */                                                                    \
+  F(uint32_t, EagerGCProbability,   0)                                  \
   F(bool, DisableSomeRepoAuthNotices,  true)                            \
-  F(bool, CheckRepoAuthDeserialize,    false)                           \
   F(uint32_t, InitialNamedEntityTableSize,  30000)                      \
   F(uint32_t, InitialStaticStringTableSize,                             \
                         kDefaultInitialStaticStringTableSize)           \
   F(uint32_t, PCRETableSize, kPCREInitialTableSize)                     \
+  F(uint64_t, PCREExpireInterval, 2 * 60 * 60)                          \
+  F(string, PCRECacheType, std::string("static"))                       \
   F(bool, EnableNuma, ServerExecutionMode())                            \
   F(bool, EnableNumaLocal, ServerExecutionMode())                       \
+  F(bool, DisableStructArray, true)                                     \
+  F(bool, EnableCallBuiltin, true)                                      \
+  F(bool, EnableReusableTC,   reuseTCDefault())                         \
+  F(uint32_t, ReusableTCPadding, 128)                                   \
   /* */
 
 private:
@@ -513,6 +611,7 @@ public:
   static bool RepoCommit;
   static bool RepoDebugInfo;
   static bool RepoAuthoritative;
+  static bool RepoPreload;
 
   // Sandbox options
   static bool SandboxMode;
