@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,15 +15,33 @@
 */
 
 #include "hphp/runtime/base/ssl-socket.h"
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/runtime-error.h"
-#include "folly/String.h"
+#include "hphp/runtime/base/req-ptr.h"
+#include "hphp/runtime/base/string-util.h"
+#include "hphp/runtime/server/server-stats.h"
+#include <folly/String.h>
 #include <poll.h>
 #include <sys/time.h>
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
 
+bool SSLSocketData::closeImpl() {
+  if (m_ssl_active) {
+    SSL_shutdown(m_handle);
+    m_ssl_active = false;
+  }
+  if (m_handle) {
+    SSL_free(m_handle);
+    m_handle = nullptr;
+  }
+  return SocketData::closeImpl();
+}
+
+SSLSocketData::~SSLSocketData() {
+  SSLSocketData::closeImpl();
+}
+
 Mutex SSLSocket::s_mutex;
 int SSLSocket::s_ex_data_index = -1;
 int SSLSocket::GetSSLExDataIndex() {
@@ -43,7 +61,7 @@ const StaticString
   s_allow_self_signed("allow_self_signed"),
   s_verify_depth("verify_depth");
 
-static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx) {
+int SSLSocket::verifyCallback(int preverify_ok, X509_STORE_CTX *ctx) {
   int ret = preverify_ok;
 
   /* determine the status for the current cert */
@@ -59,12 +77,12 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx) {
 
   /* if allow_self_signed is set, make sure that verification succeeds */
   if (err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT &&
-      stream->getContext()[s_allow_self_signed].toBoolean()) {
+      stream->m_context[s_allow_self_signed].toBoolean()) {
     ret = 1;
   }
 
   /* check the depth */
-  Variant vdepth = stream->getContext()[s_verify_depth];
+  Variant vdepth = stream->m_context[s_verify_depth];
   if (vdepth.toBoolean() && depth > vdepth.toInt64()) {
     ret = 0;
     X509_STORE_CTX_set_error(ctx, X509_V_ERR_CERT_CHAIN_TOO_LONG);
@@ -75,10 +93,10 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx) {
 
 const StaticString s_passphrase("passphrase");
 
-static int passwd_callback(char *buf, int num, int verify, void *data) {
+int SSLSocket::passwdCallback(char *buf, int num, int verify, void *data) {
   /* TODO: could expand this to make a callback into PHP user-space */
   SSLSocket *stream = (SSLSocket *)data;
-  String passphrase = stream->getContext()[s_passphrase].toString();
+  String passphrase = stream->m_context[s_passphrase].toString();
   if (!passphrase.empty() && passphrase.size() < num - 1) {
     memcpy(buf, passphrase.data(), passphrase.size() + 1);
     return passphrase.size();
@@ -99,18 +117,24 @@ SSL *SSLSocket::createSSL(SSL_CTX *ctx) {
   /* look at options in the stream and set appropriate verification flags */
   if (m_context[s_verify_peer].toBoolean()) {
     /* turn on verification callback */
-    SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, verify_callback);
+    SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, verifyCallback);
 
     /* CA stuff */
     String cafile = m_context[s_cafile].toString();
     String capath = m_context[s_capath].toString();
 
     if (!cafile.empty() || !capath.empty()) {
-      if (!SSL_CTX_load_verify_locations(ctx, cafile.data(), capath.data())) {
+      const char* cafileptr = cafile.empty() ? nullptr : cafile.data();
+      const char* capathptr = capath.empty() ? nullptr : capath.data();
+      if (!SSL_CTX_load_verify_locations(ctx, cafileptr, capathptr)) {
         raise_warning("Unable to set verify locations `%s' `%s'",
                       cafile.data(), capath.data());
         return nullptr;
       }
+    } else if (m_data->m_client && !SSL_CTX_set_default_verify_paths(ctx)) {
+      raise_warning(
+        "Unable to set default verify locations and no CA settings specified");
+      return nullptr;
     }
 
     int64_t depth = m_context[s_verify_depth].toInt64();
@@ -124,7 +148,7 @@ SSL *SSLSocket::createSSL(SSL_CTX *ctx) {
   /* callback for the passphrase (for localcert) */
   if (!m_context[s_passphrase].toString().empty()) {
     SSL_CTX_set_default_passwd_cb_userdata(ctx, this);
-    SSL_CTX_set_default_passwd_cb(ctx, passwd_callback);
+    SSL_CTX_set_default_passwd_cb(ctx, passwdCallback);
   }
 
   String cipherlist = m_context[s_ciphers].toString();
@@ -179,47 +203,80 @@ SSL *SSLSocket::createSSL(SSL_CTX *ctx) {
 // constructors and destructor
 
 SSLSocket::SSLSocket()
-    : m_handle(nullptr), m_ssl_active(false), m_method((CryptoMethod)-1),
-      m_client(false), m_connect_timeout(0), m_enable_on_connect(false),
-      m_state_set(false), m_is_blocked(true) {
+: Socket(std::make_shared<SSLSocketData>()),
+  m_data(static_cast<SSLSocketData*>(getSocketData()))
+{}
+
+SSLSocket::SSLSocket(std::shared_ptr<SSLSocketData> data)
+: Socket(data),
+  m_data(static_cast<SSLSocketData*>(getSocketData()))
+{}
+
+StaticString s_ssl("ssl");
+
+SSLSocket::SSLSocket(int sockfd, int type, const req::ptr<StreamContext>& ctx,
+                     const char *address /* = NULL */, int port /* = 0 */)
+: Socket(std::make_shared<SSLSocketData>(port, type), sockfd, type, address, port),
+  m_data(static_cast<SSLSocketData*>(getSocketData()))
+{
+  if (!ctx) {
+    return;
+  }
+  this->setStreamContext(ctx);
+  this->m_context = ctx->getOptions()[s_ssl].toArray();
 }
 
-SSLSocket::SSLSocket(int sockfd, int type, const char *address /* = NULL */,
-                     int port /* = 0 */)
-    : Socket(sockfd, type, address, port),
-      m_handle(nullptr), m_ssl_active(false), m_method((CryptoMethod)-1),
-      m_client(false), m_connect_timeout(0), m_enable_on_connect(false),
-      m_state_set(false), m_is_blocked(true) {
-}
-
-SSLSocket::~SSLSocket() {
-  SSLSocket::closeImpl();
-}
+SSLSocket::~SSLSocket() { }
 
 void SSLSocket::sweep() {
   SSLSocket::closeImpl();
   File::sweep();
-  SSLSocket::operator delete(this);
+  m_data = nullptr;
+}
+
+bool SSLSocket::enableCrypto(CryptoMethod method) {
+  if (this->m_data->m_method != CryptoMethod::NoCrypto) {
+    raise_warning("SSL/TLS already set-up for this stream");
+    return false;
+  }
+  this->m_data->m_method = method;
+  return setupCrypto() && enableCrypto();
+}
+
+bool SSLSocket::disableCrypto() {
+  if (!this->m_data->m_ssl_active) {
+    return false;
+  }
+  this->m_data->m_method = CryptoMethod::NoCrypto;
+  SSL_shutdown(this->m_data->m_handle);
+  this->m_data->m_ssl_active = false;
+  return true;
 }
 
 bool SSLSocket::onConnect() {
+  if (this->m_data->m_method == CryptoMethod::NoCrypto) {
+    return true;
+  }
   return setupCrypto() && enableCrypto();
 }
 
 bool SSLSocket::onAccept() {
-  if (m_fd >= 0 && m_enable_on_connect) {
-    switch (m_method) {
+  if (m_data->m_method == CryptoMethod::NoCrypto) {
+    return true;
+  }
+  if (getFd() >= 0 && m_data->m_enable_on_connect) {
+    switch (m_data->m_method) {
     case CryptoMethod::ClientSSLv23:
-      m_method = CryptoMethod::ServerSSLv23;
+      m_data->m_method = CryptoMethod::ServerSSLv23;
       break;
     case CryptoMethod::ClientSSLv2:
-      m_method = CryptoMethod::ServerSSLv2;
+      m_data->m_method = CryptoMethod::ServerSSLv2;
       break;
     case CryptoMethod::ClientSSLv3:
-      m_method = CryptoMethod::ServerSSLv3;
+      m_data->m_method = CryptoMethod::ServerSSLv3;
       break;
     case CryptoMethod::ClientTLS:
-      m_method = CryptoMethod::ServerTLS;
+      m_data->m_method = CryptoMethod::ServerTLS;
       break;
     default:
       assert(false);
@@ -243,7 +300,7 @@ bool SSLSocket::handleError(int64_t nr_bytes, bool is_init) {
   unsigned long ecode;
 
   bool retry = true;
-  int err = SSL_get_error(m_handle, nr_bytes);
+  int err = SSL_get_error(m_data->m_handle, nr_bytes);
   switch (err) {
   case SSL_ERROR_ZERO_RETURN:
     /* SSL terminated (but socket may still be active) */
@@ -254,7 +311,7 @@ bool SSLSocket::handleError(int64_t nr_bytes, bool is_init) {
     /* re-negotiation, or perhaps the SSL layer needs more
      * packets: retry in next iteration */
     errno = EAGAIN;
-    retry = (is_init || m_is_blocked);
+    retry = (is_init || m_data->m_is_blocked);
     break;
   case SSL_ERROR_SYSCALL:
     if (ERR_peek_error() == 0) {
@@ -262,8 +319,9 @@ bool SSLSocket::handleError(int64_t nr_bytes, bool is_init) {
         if (ERR_get_error()) {
           raise_warning("SSL: fatal protocol error");
         }
-        SSL_set_shutdown(m_handle, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
-        m_eof = true;
+        SSL_set_shutdown(m_data->m_handle,
+                         SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+        setEof(true);
         retry = false;
       } else {
         raise_warning("SSL: %s", folly::errnoStr(errno).c_str());
@@ -308,7 +366,9 @@ bool SSLSocket::handleError(int64_t nr_bytes, bool is_init) {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-SSLSocket *SSLSocket::Create(const HostURL &hosturl, double timeout) {
+req::ptr<SSLSocket> SSLSocket::Create(
+  int fd, int domain, const HostURL &hosturl, double timeout,
+  const req::ptr<StreamContext>& ctx) {
   CryptoMethod method;
   const std::string scheme = hosturl.getScheme();
 
@@ -320,55 +380,51 @@ SSLSocket *SSLSocket::Create(const HostURL &hosturl, double timeout) {
     method = CryptoMethod::ClientSSLv3;
   } else if (scheme == "tls") {
     method = CryptoMethod::ClientTLS;
+  } else if (
+    scheme == "tcp"
+    || (scheme.empty() && (domain == AF_INET || domain == AF_INET6))
+  ) {
+    method = CryptoMethod::NoCrypto;
   } else {
     return nullptr;
   }
 
-  int domain = hosturl.isIPv6() ? AF_INET6 : AF_INET;
-  int type = SOCK_STREAM;
-  SSLSocket *sock = new SSLSocket(socket(domain, type, 0), domain,
-                                  hosturl.getHost().c_str(),
-                                  hosturl.getPort());
-  sock->m_method = method;
-  sock->m_connect_timeout = timeout;
-  sock->m_enable_on_connect = true;
+  auto sock = req::make<SSLSocket>(
+    fd, domain, ctx, hosturl.getHost().c_str(), hosturl.getPort());
+
+  sock->m_data->m_method = method;
+  sock->m_data->m_connect_timeout = timeout;
+  sock->m_data->m_enable_on_connect = true;
 
   return sock;
 }
 
-bool SSLSocket::close() {
-  invokeFiltersOnClose();
-  return closeImpl();
-}
+bool SSLSocket::waitForData() {
+  if (m_data->m_ssl_active && SSL_pending(m_data->m_handle)) {
+    return true;
+  }
 
-bool SSLSocket::closeImpl() {
-  if (m_ssl_active) {
-    SSL_shutdown(m_handle);
-    m_ssl_active = false;
-  }
-  if (m_handle) {
-    SSL_free(m_handle);
-    m_handle = nullptr;
-  }
-  return Socket::closeImpl();
+  return Socket::waitForData();
 }
 
 int64_t SSLSocket::readImpl(char *buffer, int64_t length) {
   int64_t nr_bytes = 0;
-  if (m_ssl_active) {
+  if (m_data->m_ssl_active) {
+    IOStatusHelper io("sslsocket::recv", getAddress().c_str(), getPort());
+
     bool retry = true;
     do {
-      if (m_is_blocked) {
-        Socket::waitForData();
-        if (m_timedOut) {
+      if (m_data->m_is_blocked) {
+        waitForData();
+        if (timedOut()) {
           break;
         }
         // could get here and we only have parts of an SSL packet
       }
-      nr_bytes = SSL_read(m_handle, buffer, length);
+      nr_bytes = SSL_read(m_data->m_handle, buffer, length);
       if (nr_bytes > 0) break; /* we got the data */
       retry = handleError(nr_bytes, false);
-      m_eof = (!retry && errno != EAGAIN && !SSL_pending(m_handle));
+      setEof(!retry && errno != EAGAIN && !SSL_pending(m_data->m_handle));
     } while (retry);
   } else {
     nr_bytes = Socket::readImpl(buffer, length);
@@ -378,10 +434,12 @@ int64_t SSLSocket::readImpl(char *buffer, int64_t length) {
 
 int64_t SSLSocket::writeImpl(const char *buffer, int64_t length) {
   int didwrite;
-  if (m_ssl_active) {
+  if (m_data->m_ssl_active) {
+    IOStatusHelper io("sslsocket::send", getAddress().c_str(), getPort());
+
     bool retry = true;
     do {
-      didwrite = SSL_write(m_handle, buffer, length);
+      didwrite = SSL_write(m_data->m_handle, buffer, length);
       if (didwrite > 0) break;
       retry = handleError(didwrite, false);
     } while (retry);
@@ -392,11 +450,15 @@ int64_t SSLSocket::writeImpl(const char *buffer, int64_t length) {
 }
 
 bool SSLSocket::setupCrypto(SSLSocket *session /* = NULL */) {
-  if (m_handle) {
+  if (m_data->m_handle) {
     raise_warning("SSL/TLS already set-up for this stream");
     return false;
   }
 
+  if (!m_context.exists(s_verify_peer)) {
+    m_context.add(s_verify_peer, true);
+  }
+
   /* need to do slightly different things, based on client/server method,
    * so lets remember which method was selected */
 #if OPENSSL_VERSION_NUMBER < 0x00909000L
@@ -404,36 +466,44 @@ bool SSLSocket::setupCrypto(SSLSocket *session /* = NULL */) {
 #else
   const SSL_METHOD *smethod;
 #endif
-  switch (m_method) {
+  switch (m_data->m_method) {
   case CryptoMethod::ClientSSLv23:
-    m_client = true;
+    m_data->m_client = true;
     smethod = SSLv23_client_method();
     break;
-  case CryptoMethod::ClientSSLv3:
-    m_client = true;
-    smethod = SSLv3_client_method();
-    break;
   case CryptoMethod::ClientTLS:
-    m_client = true;
+    m_data->m_client = true;
     smethod = TLSv1_client_method();
     break;
   case CryptoMethod::ServerSSLv23:
-    m_client = false;
+    m_data->m_client = false;
     smethod = SSLv23_server_method();
     break;
+
+#ifndef OPENSSL_NO_SSL3
+  case CryptoMethod::ClientSSLv3:
+    m_data->m_client = true;
+    smethod = SSLv3_client_method();
+    break;
   case CryptoMethod::ServerSSLv3:
-    m_client = false;
+    m_data->m_client = false;
     smethod = SSLv3_server_method();
     break;
+#else
+  case CryptoMethod::ClientSSLv3:
+  case CryptoMethod::ServerSSLv3:
+    raise_warning("OpenSSL library does not support SSL3 protocol");
+    return false;
+#endif
 
   /* SSLv2 protocol might be disabled in the OpenSSL library */
 #ifndef OPENSSL_NO_SSL2
   case CryptoMethod::ClientSSLv2:
-    m_client = true;
+    m_data->m_client = true;
     smethod = SSLv2_client_method();
     break;
   case CryptoMethod::ServerSSLv2:
-    m_client = false;
+    m_data->m_client = false;
     smethod = SSLv2_server_method();
     break;
 #else
@@ -445,7 +515,7 @@ bool SSLSocket::setupCrypto(SSLSocket *session /* = NULL */) {
 #endif
 
   case CryptoMethod::ServerTLS:
-    m_client = false;
+    m_data->m_client = false;
     smethod = TLSv1_server_method();
     break;
   default:
@@ -459,18 +529,18 @@ bool SSLSocket::setupCrypto(SSLSocket *session /* = NULL */) {
   }
 
   SSL_CTX_set_options(ctx, SSL_OP_ALL);
-  m_handle = createSSL(ctx);
-  if (m_handle == nullptr) {
+  m_data->m_handle = createSSL(ctx);
+  if (m_data->m_handle == nullptr) {
     raise_warning("failed to create an SSL handle");
     SSL_CTX_free(ctx);
     return false;
   }
 
-  if (!SSL_set_fd(m_handle, m_fd)) {
+  if (!SSL_set_fd(m_data->m_handle, getFd())) {
     handleError(0, true);
   }
   if (session) {
-    SSL_copy_session_id(m_handle, session->m_handle);
+    SSL_copy_session_id(m_data->m_handle, session->m_data->m_handle);
   }
   return true;
 }
@@ -488,7 +558,7 @@ bool SSLSocket::applyVerificationPolicy(X509 *peer) {
     return false;
   }
 
-  int err = SSL_get_verify_result(m_handle);
+  int err = SSL_get_verify_result(m_data->m_handle);
   switch (err) {
   case X509_V_OK:
     /* fine */
@@ -521,6 +591,10 @@ bool SSLSocket::applyVerificationPolicy(X509 *peer) {
     } else if (name_len != (int)strlen(buf)) {
       raise_warning("Peer certificate CN=`%.*s' is malformed", name_len, buf);
       return false;
+    } else if (name_len == sizeof(buf)) {
+      raise_warning("Peer certificate CN=`%.*s' is too long to verify",
+                    name_len, buf);
+      return false;
     }
 
     bool match = (strcmp(cnmatch.c_str(), buf) == 0);
@@ -551,41 +625,41 @@ const StaticString
   s_peer_certificate_chain("peer_certificate_chain");
 
 bool SSLSocket::enableCrypto(bool activate /* = true */) {
-  if (activate && !m_ssl_active) {
-    double timeout = m_connect_timeout;
-    bool blocked = m_is_blocked;
-    if (!m_state_set) {
-      if (m_client) {
-        SSL_set_connect_state(m_handle);
+  if (activate && !m_data->m_ssl_active) {
+    double timeout = m_data->m_connect_timeout;
+    bool blocked = m_data->m_is_blocked;
+    if (!m_data->m_state_set) {
+      if (m_data->m_client) {
+        SSL_set_connect_state(m_data->m_handle);
       } else {
-        SSL_set_accept_state(m_handle);
+        SSL_set_accept_state(m_data->m_handle);
       }
-      m_state_set = true;
+      m_data->m_state_set = true;
     }
 
-    if (m_client && setBlocking(false)) {
-      m_is_blocked = false;
+    if (m_data->m_client && setBlocking(false)) {
+      m_data->m_is_blocked = false;
     }
 
     int n;
     bool retry = true;
     do {
-      if (m_client) {
+      if (m_data->m_client) {
         struct timeval tvs, tve;
         struct timezone tz;
 
         gettimeofday(&tvs, &tz);
-        n = SSL_connect(m_handle);
+        n = SSL_connect(m_data->m_handle);
         gettimeofday(&tve, &tz);
 
         timeout -= (tve.tv_sec + (double) tve.tv_usec / 1000000) -
           (tvs.tv_sec + (double) tvs.tv_usec / 1000000);
         if (timeout < 0) {
           raise_warning("SSL: connection timeout");
-          return -1;
+          return false;
         }
       } else {
-        n = SSL_accept(m_handle);
+        n = SSL_accept(m_data->m_handle);
       }
 
       if (n <= 0) {
@@ -595,32 +669,34 @@ bool SSLSocket::enableCrypto(bool activate /* = true */) {
       }
     } while (retry);
 
-    if (m_client && m_is_blocked != blocked && setBlocking(blocked)) {
-      m_is_blocked = blocked;
+    if (m_data->m_client &&
+        m_data->m_is_blocked != blocked &&
+        setBlocking(blocked)) {
+      m_data->m_is_blocked = blocked;
     }
 
     if (n == 1) {
-      X509 *peer_cert = SSL_get_peer_certificate(m_handle);
+      X509 *peer_cert = SSL_get_peer_certificate(m_data->m_handle);
       if (!applyVerificationPolicy(peer_cert)) {
-        SSL_shutdown(m_handle);
+        SSL_shutdown(m_data->m_handle);
       } else {
-        m_ssl_active = true;
+        m_data->m_ssl_active = true;
 
         /* allow the script to capture the peer cert
          * and/or the certificate chain */
         if (m_context[s_capture_peer_cert].toBoolean()) {
-          Resource cert(new Certificate(peer_cert));
-          m_context.set(s_peer_certificate, cert);
+          m_context.set(s_peer_certificate,
+                        Variant(req::make<Certificate>(peer_cert)));
           peer_cert = nullptr;
         }
 
         if (m_context[s_capture_peer_cert_chain].toBoolean()) {
           Array arr;
-          STACK_OF(X509) *chain = SSL_get_peer_cert_chain(m_handle);
+          STACK_OF(X509) *chain = SSL_get_peer_cert_chain(m_data->m_handle);
           if (chain) {
             for (int i = 0; i < sk_X509_num(chain); i++) {
               X509 *mycert = X509_dup(sk_X509_value(chain, i));
-              arr.append(Resource(new Certificate(mycert)));
+              arr.append(Variant(req::make<Certificate>(mycert)));
             }
           }
           m_context.set(s_peer_certificate_chain, arr);
@@ -636,30 +712,30 @@ bool SSLSocket::enableCrypto(bool activate /* = true */) {
 
     return n >= 0;
 
-  } else if (!activate && m_ssl_active) {
+  } else if (!activate && m_data->m_ssl_active) {
     /* deactivate - common for server/client */
-    SSL_shutdown(m_handle);
-    m_ssl_active = false;
+    SSL_shutdown(m_data->m_handle);
+    m_data->m_ssl_active = false;
   }
   return true;
 }
 
 bool SSLSocket::checkLiveness() {
-  if (m_fd == -1) {
+  if (getFd() == -1) {
     return false;
   }
 
   pollfd p;
-  p.fd = m_fd;
+  p.fd = getFd();
   p.events = POLLIN | POLLERR | POLLHUP | POLLPRI;
   p.revents = 0;
   if (poll(&p, 1, 0) > 0 && p.revents > 0) {
     char buf;
-    if (m_ssl_active) {
+    if (m_data->m_ssl_active) {
       while (true) {
-        int n = SSL_peek(m_handle, &buf, sizeof(buf));
+        int n = SSL_peek(m_data->m_handle, &buf, sizeof(buf));
         if (n <= 0) {
-          int err = SSL_get_error(m_handle, n);
+          int err = SSL_get_error(m_data->m_handle, n);
           if (err == SSL_ERROR_SYSCALL) {
             return errno == EAGAIN;
           }
@@ -676,7 +752,7 @@ bool SSLSocket::checkLiveness() {
          * have set the alive flag appropriately */
         break;
       }
-    } else if (0 == recv(m_fd, &buf, sizeof(buf), MSG_PEEK) &&
+    } else if (0 == recv(getFd(), &buf, sizeof(buf), MSG_PEEK) &&
                errno != EAGAIN) {
       return false;
     }
@@ -687,16 +763,21 @@ bool SSLSocket::checkLiveness() {
 ///////////////////////////////////////////////////////////////////////////////
 // Certificate
 
-const StaticString s_file("file://");
+IMPLEMENT_RESOURCE_ALLOCATION(Certificate)
 
 BIO *Certificate::ReadData(const Variant& var, bool *file /* = NULL */) {
   if (var.isString() || var.isObject()) {
     String svar = var.toString();
-    if (svar.substr(0, 7) == s_file) {
+    if (StringUtil::IsFileUrl(svar)) {
+      String decoded = StringUtil::DecodeFileUrl(svar);
+      if (decoded.empty()) {
+        raise_warning("invalid file URL, %s", svar.data());
+        return nullptr;
+      }
       if (file) *file = true;
-      BIO *ret = BIO_new_file((char*)svar.substr(7).data(), "r");
+      BIO *ret = BIO_new_file(decoded.data(), "r");
       if (ret == nullptr) {
-        raise_warning("error opening the file, %s", svar.data());
+        raise_warning("error opening the file, %s", decoded.data());
       }
       return ret;
     }
@@ -708,14 +789,14 @@ BIO *Certificate::ReadData(const Variant& var, bool *file /* = NULL */) {
 }
 
 
-Resource Certificate::Get(const Variant& var) {
+req::ptr<Certificate> Certificate::Get(const Variant& var) {
   if (var.isResource()) {
-    return var.toResource();
+    return dyn_cast_or_null<Certificate>(var);
   }
   if (var.isString() || var.isObject()) {
     bool file;
     BIO *in = ReadData(var, &file);
-    if (in == nullptr) return Resource();
+    if (in == nullptr) return nullptr;
 
     X509 *cert;
     /*
@@ -729,10 +810,10 @@ Resource Certificate::Get(const Variant& var) {
     cert = PEM_read_bio_X509(in, nullptr, nullptr, nullptr);
     BIO_free(in);
     if (cert) {
-      return Resource(new Certificate(cert));
+      return req::make<Certificate>(cert);
     }
   }
-  return Resource();
+  return nullptr;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
