@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,16 +17,18 @@
 #ifndef incl_HPHP_STRING_DATA_H_
 #define incl_HPHP_STRING_DATA_H_
 
-#include "hphp/util/slice.h"
-#include "hphp/util/hash.h"
+#include <folly/Range.h>
+
 #include "hphp/util/alloc.h"
+#include "hphp/util/bstring.h"
+#include "hphp/util/hash.h"
 #include "hphp/util/word-mem.h"
 
-#include "hphp/runtime/base/types.h"
+#include "hphp/runtime/base/cap-code.h"
 #include "hphp/runtime/base/countable.h"
-#include "hphp/runtime/base/macros.h"
-#include "hphp/runtime/base/bstring.h"
+#include "hphp/runtime/base/datatype.h"
 #include "hphp/runtime/base/exceptions.h"
+#include "hphp/runtime/base/memory-manager.h"
 
 namespace HPHP {
 
@@ -42,8 +44,8 @@ class String;
 enum AttachStringMode { AttachString };
 
 // const char* points to client-owned memory, StringData will copy it
-// at construct-time using smart_malloc.  This is only ok when the StringData
-// itself was smart-allocated.
+// at construct-time using req::malloc.  This is only ok when the StringData
+// itself was request-allocated.
 enum CopyStringMode { CopyString };
 
 /*
@@ -52,12 +54,12 @@ enum CopyStringMode { CopyString };
  * StringData's have two different modes, not all of which we want to
  * keep forever.  The main mode is Flat, which means StringData is a
  * header in a contiguous allocation with the character array for the
- * string.  The other is for APCString-backed StringDatas.
+ * string.  The other (Proxy) is for APCString-backed StringDatas.
  *
  * StringDatas can also be allocated in multiple ways.  Normally, they
  * are created through one of the Make overloads, which drops them in
  * the request-local heap.  They can also be low-malloced (for static
- * strings), or malloc'd (MakeMalloc) for APC strings.
+ * strings), or malloc'd (MakeMalloc) for APC shared or uncounted strings.
  *
  * Here's a breakdown of string modes, and which configurations are
  * allowed in which allocation mode:
@@ -65,89 +67,74 @@ enum CopyStringMode { CopyString };
  *          | Static | Malloced | Normal (request local)
  *          +--------+----------+-----------------------
  *   Flat   |   X    |     X    |    X
- *   Shared |        |          |    X
+ *   Proxy  |        |          |    X
  */
 struct StringData {
+  friend class APCString;
+  friend StringData* allocFlatSmallImpl(size_t len);
+  friend StringData* allocFlatSlowImpl(size_t len);
+
   /*
    * Max length of a string, not counting the terminal 0.
    *
-   * This is MAX_INT-1 to avoid this kind of hazard in client code:
-   *
-   *   int size = string_data->size();
-   *   ... = size + 1; // oops, wraparound.
+   * This is smaller than MAX_INT, and we want a CapCode to precisely encode it.
    */
-  static constexpr uint32_t MaxSize = 0x7ffffffe; // 2^31-2
-  static constexpr uint32_t MaxCap = MaxSize + 1;
+  static constexpr uint32_t MaxSize = 0x7ff00000; // 11 bits of 1's
 
   /*
-   * Creates an empty request-local string with an unspecified amount
-   * of reserved space.
+   * Creates an empty request-local string with an unspecified amount of
+   * reserved space. Ref-count is pre-initialized to 1.
    */
   static StringData* Make();
 
   /*
-   * Constructors that copy the string memory into this StringData,
-   * for request-local strings.
+   * Constructors that copy the string memory into this StringData, for
+   * request-local strings. Ref-count is pre-initialized to 1.
    *
    * Most strings are created this way.
    */
   static StringData* Make(const char* data);
+  static StringData* Make(const std::string& data);
   static StringData* Make(const char* data, CopyStringMode);
   static StringData* Make(const char* data, size_t len, CopyStringMode);
   static StringData* Make(const StringData* s, CopyStringMode);
-  static StringData* Make(StringSlice r1, CopyStringMode);
+  static StringData* Make(folly::StringPiece r1, CopyStringMode);
 
   /*
    * Attach constructors for request-local strings.
    *
-   * These do the same thing as the above CopyStringMode constructors,
-   * except that it will also free `data'.
+   * These do the same thing as the above CopyStringMode constructors, except
+   * that it will also free `data'. Ref-count is pre-initialized to 1.
    */
   static StringData* Make(char* data, AttachStringMode);
   static StringData* Make(char* data, size_t len, AttachStringMode);
 
   /*
    * Create a new request-local string by concatenating two existing
-   * strings.
+   * strings. Ref-count is pre-initialized to 1.
    */
   static StringData* Make(const StringData* s1, const StringData* s2);
-  static StringData* Make(const StringData* s1, StringSlice s2);
+  static StringData* Make(const StringData* s1, folly::StringPiece s2);
   static StringData* Make(const StringData* s1, const char* lit2);
-  static StringData* Make(StringSlice s1, const char* lit2);
-  static StringData* Make(StringSlice s1, StringSlice s2);
-  static StringData* Make(StringSlice s1, StringSlice s2,
-                          StringSlice s3);
-  static StringData* Make(StringSlice s1, StringSlice s2,
-                          StringSlice s3, StringSlice s4);
+  static StringData* Make(folly::StringPiece s1, const char* lit2);
+  static StringData* Make(folly::StringPiece s1, folly::StringPiece s2);
+  static StringData* Make(folly::StringPiece s1, folly::StringPiece s2,
+                          folly::StringPiece s3);
+  static StringData* Make(folly::StringPiece s1, folly::StringPiece s2,
+                          folly::StringPiece s3, folly::StringPiece s4);
 
   /*
-   * Create a new request-local empty string big enough to hold
-   * strings of length `reserve' (not counting the \0 terminator).
+   * Create a new request-local empty string big enough to hold strings of
+   * length `reserve' (not counting the \0 terminator). Ref-count is
+   * pre-initialized to 1.
    */
   static StringData* Make(size_t reserve);
 
   /*
-   * Create a request-local StringData that wraps an APCString
-   * that contains a string.
+   * Create a request-local "Proxy" StringData that wraps an APCString.
+   * Ref-count is pre-initialized to 1.
    */
-  static StringData* Make(APCString* shared);
-
-  /*
-   * Create a StringData that is allocated by malloc, instead of the
-   * smart allocator.
-   *
-   * This is essentially only used for APC, for non-request local
-   * StringDatas.
-   *
-   * StringDatas allocated with this function must be freed by calling
-   * destruct(), instead of release().
-   *
-   * Important: no string functions which change the StringData may be
-   * called on the returned pointer (e.g. append).  These functions
-   * below are marked by saying they require the string to be request
-   * local.
-   */
-  static StringData* MakeMalloced(const char* data, size_t len);
+  static StringData* MakeProxy(const APCString* apcstr);
 
   /*
    * Allocate a string with malloc, using the low-memory allocator if
@@ -157,14 +144,14 @@ struct StringData {
    * StringData is not expected to be reference counted, and must be
    * deallocated using destructStatic.
    */
-  static StringData* MakeStatic(StringSlice);
+  static StringData* MakeStatic(folly::StringPiece);
 
   /*
-   * Same as MakeStatic but the string alloated will *not* be in the static
-   * string table and will be deleted once the root goes out of scope.
-   * Currently only used by APC.
+   * Same as MakeStatic but the string allocated will *not* be in the static
+   * string table, will not be in low-memory, and should be deleted using
+   * destructUncounted once the root goes out of scope.
    */
-  static StringData* MakeUncounted(StringSlice);
+  static StringData* MakeUncounted(folly::StringPiece);
 
   /*
    * Same as MakeStatic but initializes the empty string in aligned storage.
@@ -173,29 +160,25 @@ struct StringData {
   static StringData* MakeEmpty();
 
   /*
-   * Offset accessor for the JIT compiler.
+   * Offset accessors for the JIT compiler.
    */
+  static constexpr ptrdiff_t dataOff() { return offsetof(StringData, m_data); }
   static constexpr ptrdiff_t sizeOff() { return offsetof(StringData, m_len); }
 
   /*
-   * Shared StringData's have a sweep list running through them for
+   * Proxy StringData's have a sweep list running through them for
    * decrefing the APCString they are fronting.  This function
    * must be called at request cleanup time to handle this.
    */
-  static void sweepAll();
+  static unsigned sweepAll();
 
   /*
-   * Called to return a StringData to the smart allocator.  This is
+   * Called to return a StringData to the request allocator.  This is
    * normally called when the reference count goes to zero (e.g. with
    * a helper like decRefStr).
    */
-  void release();
-
-  /*
-   * StringData objects allocated with MakeMalloced should be freed
-   * using this function instead of release().
-   */
-  void destruct();
+  void release() noexcept;
+  size_t heapSize() const;
 
   /*
    * StringData objects allocated with MakeStatic should be freed
@@ -204,47 +187,56 @@ struct StringData {
   void destructStatic();
 
   /*
-   * Reference-counting related.
+   * StringData objects allocated with MakeUncounted should be freed
+   * using this function.
    */
-  IMPLEMENT_COUNTABLE_METHODS_NO_STATIC
-  void setRefCount(RefCount n);
-  bool isStatic() const;
-  bool isUncounted() const;
+  void destructUncounted();
 
   /*
-   * Append the supplied range to this string.  If there is not
-   * sufficient capacity in this string to contain the range, a new
-   * string may be returned.
+   * Reference-counting related.
+   */
+  IMPLEMENT_COUNTABLE_METHODS
+
+  bool kindIsValid() const { return m_hdr.kind == HeaderKind::String; }
+
+  /*
+   * Append the supplied range to this string.  If there is not sufficient
+   * capacity in this string to contain the range, a new string may be
+   * returned. The new string's reference count will be pre-initialized to 1.
    *
    * Pre: !hasMultipleRefs()
    * Pre: the string is request-local
    */
-  StringData* append(StringSlice r);
-  StringData* append(StringSlice r1, StringSlice r2);
-  StringData* append(StringSlice r1, StringSlice r2, StringSlice r3);
+  StringData* append(folly::StringPiece r);
+  StringData* append(folly::StringPiece r1, folly::StringPiece r2);
+  StringData* append(folly::StringPiece r1,
+                     folly::StringPiece r2,
+                     folly::StringPiece r3);
 
   /*
    * Reserve space for a string of length `maxLen' (not counting null
    * terminator).
    *
-   * May not be called for strings created with MakeMalloced or
+   * May not be called for strings created with MakeUncounted or
    * MakeStatic.
    *
-   * Returns: possibly a new StringData, if we had to reallocate.  The
-   * returned pointer is not yet incref'd.
+   * Returns: possibly a new StringData, if we had to reallocate.  The new
+   * string's reference count will be pre-initialized to 1.
    */
   StringData* reserve(size_t maxLen);
 
   /*
    * Shrink a string down to length `len` (not counting null terminator).
    *
-   * May not be called for strings created with MakeMalloced or
+   * May not be called for strings created with MakeUncounted or
    * MakeStatic.
    *
-   * Returns: possibly a new StringData, if we decided to reallocate. The
-   * returned pointer is not yet incref'd.
+   * Returns: possibly a new StringData, if we decided to reallocate. The new
+   * string's reference count is be pre-initialized to 1.  shrinkImpl
+   * always returns a new StringData.
    */
   StringData* shrink(size_t len);
+  StringData* shrinkImpl(size_t len);
 
   /*
    * Returns a slice with extents sized to the *string* that this
@@ -254,7 +246,7 @@ struct StringData {
    * include a null-terminator if possible.  (We would like to make
    * this unnecessary eventually.)
    */
-  StringSlice slice() const;
+  folly::StringPiece slice() const;
 
   /*
    * Returns a mutable slice with extents sized to the *buffer* this
@@ -264,7 +256,7 @@ struct StringData {
    * Note: please do not introduce new uses of this API that write
    * nulls 1 byte past slice.len---we want to weed those out.
    */
-  MutableSlice bufferSlice();
+  folly::MutableStringPiece bufferSlice();
 
   /*
    * If external users of this object want to modify it (e.g. through
@@ -302,7 +294,7 @@ struct StringData {
    * Accessor for the length of a string.
    *
    * Note: size() returns a signed int for historical reasons.  It is
-   * guaranteed to be greater than zero and less than MaxSize.
+   * guaranteed to be in the range (0 <= size() <= MaxSize)
    */
   int size() const;
 
@@ -312,10 +304,8 @@ struct StringData {
   bool empty() const;
 
   /*
-   * Return the capacity of this string's buffer, including the space
+   * Return the capacity of this string's buffer, not including the space
    * for the null terminator.
-   *
-   * For shared strings, returns zero.
    */
   uint32_t capacity() const;
 
@@ -326,12 +316,17 @@ struct StringData {
    * The allow_errors flag is a boolean that does something currently
    * undocumented.
    *
+   * If overflow is set its value is initialized to either zero to
+   * indicate that no overflow occurred or 1/-1 to inidicate the direction
+   * of overflow.
+   *
    * Returns: KindOfNull, KindOfInt64 or KindOfDouble.  The int64_t or
    * double out reference params are populated in the latter two cases
    * with the numeric value of the string.  The KindOfNull case
    * indicates the string is not numeric.
    */
-  DataType isNumericWithVal(int64_t&, double&, int allowErrors) const;
+  DataType isNumericWithVal(int64_t&, double&, int allowErrors,
+                            int* overflow = nullptr) const;
 
   /*
    * Returns true if this string is numeric.
@@ -363,7 +358,7 @@ struct StringData {
    * Change the character at offset `offset' to `c'.
    *
    * May return a reallocated StringData* if this string was a shared
-   * string.
+   * string. The new string's reference count is pre-initialized to 1.
    *
    * Pre: offset >= 0 && offset < size()
    *      !hasMultipleRefs()
@@ -380,8 +375,9 @@ struct StringData {
   StringData* getChar(int offset) const;
 
   /*
-   * Increment this string in the manner of php's ++ operator.  May
-   * return a new string if it had to resize.
+   * Increment this string in the manner of php's ++ operator.  May return a new
+   * string if it had to resize. The new string's reference count is
+   * pre-initialized to 1.
    *
    * Pre: !isStatic() && !isEmpty()
    *      string must be request local
@@ -404,6 +400,7 @@ struct StringData {
    * Returns: case insensitive hash value for this string.
    */
   strhash_t hash() const;
+  NEVER_INLINE strhash_t hashHelper() const;
 
   /*
    * Equality comparison, in the sense of php's string == operator.
@@ -439,27 +436,34 @@ struct StringData {
    */
   void dump() const;
 
+  static StringData* node2str(StringDataNode* node) {
+    return reinterpret_cast<StringData*>(
+      uintptr_t(node) - offsetof(Proxy, node)
+                   - sizeof(StringData)
+    );
+  }
+  bool isProxy() const;
+
 private:
-  struct SharedPayload {
-    SweepNode node;
-    APCString* shared;
+  struct Proxy {
+    StringDataNode node;
+    const APCString* apcstr;
   };
 
 private:
-  static StringData* MakeShared(StringSlice sl, bool trueStatic);
-  static StringData* MakeSVSlowPath(APCString*, uint32_t len);
+  static StringData* MakeShared(folly::StringPiece sl, bool trueStatic);
+  static StringData* MakeProxySlowPath(const APCString*);
 
   StringData(const StringData&) = delete;
   StringData& operator=(const StringData&) = delete;
   ~StringData() = delete;
 
 private:
-  const void* voidPayload() const;
-  void* voidPayload();
-  const SharedPayload* sharedPayload() const;
-  SharedPayload* sharedPayload();
+  const void* payload() const;
+  void* payload();
+  const Proxy* proxy() const;
+  Proxy* proxy();
 
-  bool isShared() const;
   bool isFlat() const;
   bool isImmutable() const;
 
@@ -469,11 +473,8 @@ private:
   void enlist();
   void delist();
   void incrementHelper();
-  strhash_t hashHelper() const NEVER_INLINE;
   bool checkSane() const;
-  void preCompute() const;
-  void setStatic() const;
-  void setUncounted() const;
+  void preCompute();
 
 private:
   char* m_data;
@@ -481,22 +482,14 @@ private:
   // We have the next fields blocked into qword-size unions so
   // StringData initialization can do fewer stores to initialize the
   // fields.  (gcc does not combine the stores itself.)
+  HeaderWord<CapCode,Counted::Maybe> m_hdr;
   union {
     struct {
       uint32_t m_len;
-      mutable RefCount m_count;
-    };
-    uint64_t m_lenAndCount;
-  };
-  union {
-    struct {
-      int32_t m_cap;
       mutable strhash_t m_hash;   // precompute hash codes for static strings
     };
-    uint64_t m_capAndHash;
+    uint64_t m_lenAndHash;
   };
-
-  friend class APCString;
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -505,15 +498,11 @@ private:
  * A reasonable length to reserve for small strings.  This is the
  * default reserve size for StringData::Make(), also.
  */
-const uint32_t SmallStringReserve = 64 - sizeof(StringData) - 1;
+constexpr uint32_t SmallStringReserve = 64 - sizeof(StringData) - 1;
 
 /*
  * DecRef a string s, calling release if its reference count goes to
  * zero.
- *
- * Pre: either s must have been allocated as a request-local string,
- * or it must be a static string.  (I.e. it can not be created with
- * MakeMalloced.)
  */
 void decRefStr(StringData* s);
 
@@ -549,16 +538,29 @@ ALWAYS_INLINE StringData* staticEmptyString() {
 }
 
 namespace folly {
-template<> struct FormatValue<HPHP::StringData> {
-  explicit FormatValue(const HPHP::StringData& str) : m_val(str) {}
+template<> struct FormatValue<const HPHP::StringData*> {
+  explicit FormatValue(const HPHP::StringData* str) : m_val(str) {}
 
   template<typename Callback>
   void format(FormatArg& arg, Callback& cb) const {
-    format_value::formatString(m_val.data(), arg, cb);
+    auto piece = folly::StringPiece(m_val->data(), m_val->size());
+    format_value::formatString(piece, arg, cb);
   }
 
  private:
-  const HPHP::StringData& m_val;
+  const HPHP::StringData* m_val;
+};
+
+template<> struct FormatValue<HPHP::StringData*> {
+  explicit FormatValue(const HPHP::StringData* str) : m_val(str) {}
+
+  template<typename Callback>
+  void format(FormatArg& arg, Callback& cb) const {
+    FormatValue<const HPHP::StringData*>(m_val).format(arg, cb);
+  }
+
+ private:
+  const HPHP::StringData* m_val;
 };
 }
 
