@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -19,12 +19,13 @@
 #include <limits>
 #include <algorithm>
 
-#include "folly/ScopeGuard.h"
+#include <folly/ScopeGuard.h>
 
+#include "hphp/runtime/base/array-data-defs.h"
 #include "hphp/runtime/base/strings.h"
 #include "hphp/runtime/base/runtime-error.h"
 #include "hphp/runtime/base/tv-conversions.h"
-#include "hphp/runtime/ext/ext_math.h"
+#include "hphp/runtime/ext/std/ext_std_math.h"
 #include "hphp/util/overflow.h"
 
 namespace HPHP {
@@ -48,15 +49,32 @@ TypedNum numericConvHelper(Cell cell) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfString:
-  case KindOfStaticString: return stringToNumeric(cell.m_data.pstr);
-  case KindOfBoolean:      return make_int(cell.m_data.num);
-  case KindOfUninit:
-  case KindOfNull:         return make_int(0);
-  case KindOfObject:       return make_int(cell.m_data.pobj->o_toInt64());
-  case KindOfResource:     return make_int(cell.m_data.pres->o_toInt64());
-  case KindOfArray:        throw_bad_array_operand();
-  default:                 break;
+    case KindOfUninit:
+    case KindOfNull:
+      return make_int(0);
+
+    case KindOfBoolean:
+      return make_int(cell.m_data.num);
+
+    case KindOfString:
+    case KindOfStaticString:
+      return stringToNumeric(cell.m_data.pstr);
+
+    case KindOfPersistentArray:
+    case KindOfArray:
+      throw_bad_array_operand();
+
+    case KindOfObject:
+      return make_int(cell.m_data.pobj->toInt64());
+
+    case KindOfResource:
+      return make_int(cell.m_data.pres->data()->o_toInt64());
+
+    case KindOfInt64:
+    case KindOfDouble:
+    case KindOfRef:
+    case KindOfClass:
+      break;
   }
   not_reached();
 }
@@ -82,7 +100,7 @@ again:
     }
   }
 
-  if (c1.m_type == KindOfArray && c2.m_type == KindOfArray) {
+  if (isArrayType(c1.m_type) && isArrayType(c2.m_type)) {
     return make_tv<KindOfArray>(o(c1.m_data.parr, c2.m_data.parr));
   }
 
@@ -95,7 +113,7 @@ again:
 // returns the overflowed value.
 template<class Op, class Check, class Over>
 Cell cellArithO(Op o, Check ck, Over ov, Cell c1, Cell c2) {
-  if (c1.m_type == KindOfArray && c2.m_type == KindOfArray) {
+  if (isArrayType(c1.m_type) && isArrayType(c2.m_type)) {
     return cellArith(o, c1, c2);
   }
 
@@ -153,7 +171,12 @@ struct Div {
   Cell operator()(int64_t t, int64_t u) const {
     if (UNLIKELY(u == 0)) {
       raise_warning(Strings::DIVISION_BY_ZERO);
-      return make_tv<KindOfBoolean>(false);
+      if (RuntimeOption::PHP7_IntSemantics) {
+        // PHP7 uses the IEEE definition (+/- INF and NAN).
+        return make_dbl(t / 0.0);
+      } else {
+        return make_tv<KindOfBoolean>(false);
+      }
     }
 
     // Avoid SIGFPE when dividing the miniumum respresentable integer
@@ -173,7 +196,12 @@ struct Div {
   >::type operator()(T t, U u) const {
     if (UNLIKELY(u == 0)) {
       raise_warning(Strings::DIVISION_BY_ZERO);
-      return make_tv<KindOfBoolean>(false);
+      if (RuntimeOption::PHP7_IntSemantics) {
+        // PHP7 uses the IEEE definition (+/- INF and NAN).
+        return make_dbl(t / u);
+      } else {
+        return make_tv<KindOfBoolean>(false);
+      }
     }
     return make_dbl(t / u);
   }
@@ -217,11 +245,12 @@ again:
     }
   }
 
-  if (c1.m_type == KindOfArray && c2.m_type == KindOfArray) {
+  if (isArrayType(c1.m_type) && isArrayType(c2.m_type)) {
     auto const ad1    = c1.m_data.parr;
     auto const newArr = op(ad1, c2.m_data.parr);
     if (newArr != ad1) {
       c1.m_data.parr = newArr;
+      c1.m_type = KindOfArray;
       decRefArr(ad1);
     }
     return;
@@ -286,7 +315,6 @@ StringData* stringBitOp(BitOp bop, SzOp sop, StringData* s1, StringData* s2) {
   }
   newStr->setSize(newLen);
 
-  newStr->setRefCount(1);
   return newStr;
 }
 
@@ -295,7 +323,7 @@ Cell cellBitOp(StrLenOp strLenOp, Cell c1, Cell c2) {
   assert(cellIsPlausible(c1));
   assert(cellIsPlausible(c2));
 
-  if (IS_STRING_TYPE(c1.m_type) && IS_STRING_TYPE(c2.m_type)) {
+  if (isStringType(c1.m_type) && isStringType(c2.m_type)) {
     return make_tv<KindOfString>(
       stringBitOp(
         BitOp<char>(),
@@ -312,13 +340,16 @@ Cell cellBitOp(StrLenOp strLenOp, Cell c1, Cell c2) {
 template<class Op>
 void cellBitOpEq(Op op, Cell& c1, Cell c2) {
   auto const result = op(c1, c2);
-  cellSet(result, c1);
+  auto const type = c1.m_type;
+  auto const data = c1.m_data.num;
+  tvCopy(result, c1);
+  tvRefcountedDecRefHelper(type, data);
 }
 
 // Op must implement the interface described for cellIncDecOp.
 template<class Op>
 void stringIncDecOp(Op op, Cell& cell) {
-  assert(IS_STRING_TYPE(cell.m_type));
+  assert(isStringType(cell.m_type));
 
   auto const sd = cell.m_data.pstr;
   if (sd->empty()) {
@@ -329,23 +360,19 @@ void stringIncDecOp(Op op, Cell& cell) {
 
   int64_t ival;
   double dval;
-  auto const dt = sd->isNumericWithVal(ival, dval, true /* allow_errors */);
+  auto const dt = sd->isNumericWithVal(ival, dval, false /* allow_errors */);
 
-  switch (dt) {
-  case KindOfInt64:
+  if (dt == KindOfInt64) {
     decRefStr(sd);
     cellCopy(make_int(ival), cell);
     op.intCase(cell);
-    break;
-  case KindOfDouble:
+  } else if (dt == KindOfDouble) {
     decRefStr(sd);
     cellCopy(make_dbl(dval), cell);
     op.dblCase(cell);
-    break;
-  default:
+  } else {
     assert(dt == KindOfNull);
     op.nonNumericString(cell);
-    break;
   }
 }
 
@@ -365,31 +392,36 @@ void cellIncDecOp(Op op, Cell& cell) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfInt64:
-    op.intCase(cell);
-    break;
-  case KindOfDouble:
-    op.dblCase(cell);
-    break;
+    case KindOfUninit:
+    case KindOfNull:
+      op.nullCase(cell);
+      return;
 
-  case KindOfString:
-  case KindOfStaticString:
-    stringIncDecOp(op, cell);
-    break;
+    case KindOfInt64:
+      op.intCase(cell);
+      return;
 
-  case KindOfUninit:
-  case KindOfNull:
-    op.nullCase(cell);
-    break;
+    case KindOfDouble:
+      op.dblCase(cell);
+      return;
 
-  case KindOfBoolean:
-  case KindOfObject:
-  case KindOfResource:
-  case KindOfArray:
-    break;
-  default:
-    not_reached();
+    case KindOfStaticString:
+    case KindOfString:
+      stringIncDecOp(op, cell);
+      return;
+
+    case KindOfBoolean:
+    case KindOfPersistentArray:
+    case KindOfArray:
+    case KindOfObject:
+    case KindOfResource:
+      return;
+
+    case KindOfRef:
+    case KindOfClass:
+      break;
   }
+  not_reached();
 }
 
 const StaticString s_1("1");
@@ -409,13 +441,12 @@ struct IncBase {
       auto const tmp = StringData::Make(sd, CopyString);
       auto const tmp2 = tmp->increment();
       if (tmp2 != tmp) {
-        assert(tmp->getCount() == 0);
+        assert(tmp->hasExactlyOneRef());
         tmp->release();
         return tmp2;
       }
       return tmp;
     }();
-    newSd->incRefCount();
     decRefStr(sd);
     cellCopy(make_tv<KindOfString>(newSd), cell);
   }
@@ -498,15 +529,19 @@ Cell cellDiv(Cell c1, Cell c2) {
 }
 
 Cell cellPow(Cell c1, Cell c2) {
-  return *f_pow(tvAsVariant(&c1), tvAsVariant(&c2)).asCell();
+  return *HHVM_FN(pow)(tvAsVariant(&c1), tvAsVariant(&c2)).asCell();
 }
 
 Cell cellMod(Cell c1, Cell c2) {
   auto const i1 = cellToInt(c1);
   auto const i2 = cellToInt(c2);
   if (UNLIKELY(i2 == 0)) {
-    raise_warning(Strings::DIVISION_BY_ZERO);
-    return make_tv<KindOfBoolean>(false);
+    if (RuntimeOption::PHP7_IntSemantics) {
+      SystemLib::throwDivisionByZeroErrorObject(Strings::MODULO_BY_ZERO);
+    } else {
+      raise_warning(Strings::DIVISION_BY_ZERO);
+      return make_tv<KindOfBoolean>(false);
+    }
   }
 
   // This is to avoid SIGFPE in the case of INT64_MIN % -1.
@@ -535,11 +570,37 @@ Cell cellBitXor(Cell c1, Cell c2) {
 }
 
 Cell cellShl(Cell c1, Cell c2) {
-  return make_int(cellToInt(c1) << cellToInt(c2));
+  int64_t lhs = cellToInt(c1);
+  int64_t shift = cellToInt(c2);
+
+  if (RuntimeOption::PHP7_IntSemantics) {
+    if (UNLIKELY(shift >= 64)) {
+      return make_int(0);
+    }
+
+    if (UNLIKELY(shift < 0)) {
+      SystemLib::throwArithmeticErrorObject(Strings::NEGATIVE_SHIFT);
+    }
+  }
+
+  return make_int(lhs << (shift & 63));
 }
 
 Cell cellShr(Cell c1, Cell c2) {
-  return make_int(cellToInt(c1) >> cellToInt(c2));
+  int64_t lhs = cellToInt(c1);
+  int64_t shift = cellToInt(c2);
+
+  if (RuntimeOption::PHP7_IntSemantics) {
+    if (UNLIKELY(shift >= 64)) {
+      return make_int(lhs >= 0 ? 0 : -1);
+    }
+
+    if (UNLIKELY(shift < 0)) {
+      SystemLib::throwArithmeticErrorObject(Strings::NEGATIVE_SHIFT);
+    }
+  }
+
+  return make_int(lhs >> (shift & 63));
 }
 
 void cellAddEq(Cell& c1, Cell c2) {
@@ -554,9 +615,9 @@ void cellMulEq(Cell& c1, Cell c2) {
   cellOpEq(MulEq(), c1, c2);
 }
 
-void cellAddEqO(Cell& c1, Cell c2) { cellCopy(cellAddO(c1, c2), c1); }
-void cellSubEqO(Cell& c1, Cell c2) { cellCopy(cellSubO(c1, c2), c1); }
-void cellMulEqO(Cell& c1, Cell c2) { cellCopy(cellMulO(c1, c2), c1); }
+void cellAddEqO(Cell& c1, Cell c2) { cellSet(cellAddO(c1, c2), c1); }
+void cellSubEqO(Cell& c1, Cell c2) { cellSet(cellSubO(c1, c2), c1); }
+void cellMulEqO(Cell& c1, Cell c2) { cellSet(cellMulO(c1, c2), c1); }
 
 void cellDivEq(Cell& c1, Cell c2) {
   assert(cellIsPlausible(c1));
@@ -587,8 +648,8 @@ void cellBitXorEq(Cell& c1, Cell c2) {
   cellBitOpEq(cellBitXor, c1, c2);
 }
 
-void cellShlEq(Cell& c1, Cell c2) { cellCopy(cellShl(c1, c2), c1); }
-void cellShrEq(Cell& c1, Cell c2) { cellCopy(cellShr(c1, c2), c1); }
+void cellShlEq(Cell& c1, Cell c2) { cellSet(cellShl(c1, c2), c1); }
+void cellShrEq(Cell& c1, Cell c2) { cellSet(cellShr(c1, c2), c1); }
 
 void cellInc(Cell& cell) { cellIncDecOp(Inc(), cell); }
 void cellIncO(Cell& cell) { cellIncDecOp(IncO(), cell); }
@@ -599,45 +660,53 @@ void cellBitNot(Cell& cell) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfInt64:
-    cell.m_data.num = ~cell.m_data.num;
-    break;
-  case KindOfDouble:
-    cell.m_type     = KindOfInt64;
-    cell.m_data.num = ~toInt64(cell.m_data.dbl);
-    break;
+    case KindOfInt64:
+      cell.m_data.num = ~cell.m_data.num;
+      break;
 
-  case KindOfString:
-    if (cell.m_data.pstr->hasMultipleRefs()) {
+    case KindOfDouble:
+      cell.m_type     = KindOfInt64;
+      cell.m_data.num = ~toInt64(cell.m_data.dbl);
+      break;
+
+    case KindOfString:
+      if (cell.m_data.pstr->cowCheck()) {
     case KindOfStaticString:
-      auto const newSd = StringData::Make(
-        cell.m_data.pstr->slice(),
-        CopyString
-      );
-      newSd->incRefCount();
-      cell.m_data.pstr->decRefCount(); // can't go to zero
-      cell.m_data.pstr = newSd;
-      cell.m_type = KindOfString;
-    } else {
-      // Unless we go through this branch, the string was just freshly
-      // created, so the following mutation will be safe wrt its
-      // internal hash caching.
-      cell.m_data.pstr->invalidateHash();
-    }
-
-    {
-      auto const sd   = cell.m_data.pstr;
-      auto const len  = sd->size();
-      auto const data = sd->mutableData();
-      assert(sd->hasExactlyOneRef());
-      for (uint32_t i = 0; i < len; ++i) {
-        data[i] = ~data[i];
+        auto const newSd = StringData::Make(
+          cell.m_data.pstr->slice(),
+          CopyString
+        );
+        cell.m_data.pstr->decRefCount(); // can't go to zero
+        cell.m_data.pstr = newSd;
+        cell.m_type = KindOfString;
+      } else {
+        // Unless we go through this branch, the string was just freshly
+        // created, so the following mutation will be safe wrt its
+        // internal hash caching.
+        cell.m_data.pstr->invalidateHash();
       }
-    }
-    break;
 
-  default:
-    raise_error("Unsupported operand type for ~");
+      {
+        auto const sd   = cell.m_data.pstr;
+        auto const len  = sd->size();
+        auto const data = sd->mutableData();
+        assert(sd->hasExactlyOneRef());
+        for (uint32_t i = 0; i < len; ++i) {
+          data[i] = ~data[i];
+        }
+      }
+      break;
+
+    case KindOfUninit:
+    case KindOfNull:
+    case KindOfBoolean:
+    case KindOfPersistentArray:
+    case KindOfArray:
+    case KindOfObject:
+    case KindOfResource:
+    case KindOfRef:
+    case KindOfClass:
+      raise_error("Unsupported operand type for ~");
   }
 }
 
