@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,14 +20,14 @@
 #include "hphp/runtime/base/tv-conversions.h"
 #include "hphp/runtime/base/comparisons.h"
 #include "hphp/runtime/base/type-conversions.h"
-#include "hphp/runtime/ext/ext_datetime.h"
+#include "hphp/runtime/ext/datetime/ext_datetime.h"
 
 namespace HPHP {
 
 //////////////////////////////////////////////////////////////////////
-
-extern bool collectionEquals(const ObjectData*, const ObjectData*);
-
+namespace collections {
+extern bool equals(const ObjectData*, const ObjectData*);
+}
 //////////////////////////////////////////////////////////////////////
 
 namespace {
@@ -47,98 +47,134 @@ namespace {
  */
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, bool val) {
+typename Op::RetType cellRelOp(Op op, Cell cell, bool val) {
   return op(cellToBool(cell), val);
 }
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, int64_t val) {
+typename Op::RetType cellRelOp(Op op, Cell cell, int64_t val) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfUninit:
-  case KindOfNull:           return op(false, !!val);
-  case KindOfBoolean:        return op(!!cell.m_data.num, val != 0);
-  case KindOfInt64:          return op(cell.m_data.num, val);
-  case KindOfDouble:         return op(cell.m_data.dbl, val);
-  case KindOfArray:          return op(true, false);
+    case KindOfUninit:
+    case KindOfNull:
+      return op(false, !!val);
 
-  case KindOfObject:
-    return cell.m_data.pobj->isCollection()
-      ? op.collectionVsNonObj()
-      : op(cell.m_data.pobj->o_toInt64(), val);
+    case KindOfBoolean:
+      return op(!!cell.m_data.num, val != 0);
 
-  case KindOfResource:
-    return op(cell.m_data.pres->o_toInt64(), val);
+    case KindOfInt64:
+      return op(cell.m_data.num, val);
 
-  case KindOfStaticString:
-  case KindOfString:
-    {
+    case KindOfDouble:
+      return op(cell.m_data.dbl, val);
+
+    case KindOfStaticString:
+    case KindOfString: {
       auto const num = stringToNumeric(cell.m_data.pstr);
       return num.m_type == KindOfInt64  ? op(num.m_data.num, val) :
              num.m_type == KindOfDouble ? op(num.m_data.dbl, val) :
              op(0, val);
     }
 
-  default:
-    break;
+    case KindOfPersistentArray:
+    case KindOfArray:
+      return op(true, false);
+
+    case KindOfObject:
+      return cell.m_data.pobj->isCollection()
+        ? op.collectionVsNonObj()
+        : op(cell.m_data.pobj->toInt64(), val);
+
+    case KindOfResource:
+      return op(cell.m_data.pres->data()->o_toInt64(), val);
+
+    case KindOfRef:
+    case KindOfClass:
+      break;
   }
   not_reached();
 }
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, double val) {
+typename Op::RetType cellRelOp(Op op, Cell cell, double val) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfUninit:
-  case KindOfNull:            return op(false, val != 0);
-  case KindOfBoolean:         return op(!!cell.m_data.num, val != 0);
-  case KindOfInt64:           return op(cell.m_data.num, val);
-  case KindOfDouble:          return op(cell.m_data.dbl, val);
-  case KindOfArray:           return op(true, false);
+    case KindOfUninit:
+    case KindOfNull:
+      return op(false, val != 0);
 
-  case KindOfObject:
-    return cell.m_data.pobj->isCollection()
-      ? op.collectionVsNonObj()
-      : op(cell.m_data.pobj->o_toDouble(), val);
+    case KindOfBoolean:
+      return op(!!cell.m_data.num, val != 0);
 
-  case KindOfResource:
-    return op(cell.m_data.pres->o_toDouble(), val);
+    case KindOfInt64:
+      return op(cell.m_data.num, val);
 
-  case KindOfStaticString:
-  case KindOfString:
-    return op(toDouble(cell.m_data.pstr), val);
+    case KindOfDouble:
+      return op(cell.m_data.dbl, val);
 
-  default:
-    break;
+    case KindOfStaticString:
+    case KindOfString: {
+      auto const num = stringToNumeric(cell.m_data.pstr);
+      return num.m_type == KindOfInt64  ? op(num.m_data.num, val) :
+             num.m_type == KindOfDouble ? op(num.m_data.dbl, val) :
+             op(0, val);
+    }
+
+    case KindOfPersistentArray:
+    case KindOfArray:
+      return op(true, false);
+
+    case KindOfObject:
+      return cell.m_data.pobj->isCollection()
+        ? op.collectionVsNonObj()
+        : op(cell.m_data.pobj->toDouble(), val);
+
+    case KindOfResource:
+      return op(cell.m_data.pres->data()->o_toDouble(), val);
+
+    case KindOfRef:
+    case KindOfClass:
+      break;
   }
   not_reached();
 }
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, const StringData* val) {
+typename Op::RetType cellRelOp(Op op, Cell cell, const StringData* val) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfUninit:
-  case KindOfNull:         return op(staticEmptyString(), val);
-  case KindOfBoolean:      return op(!!cell.m_data.num, toBoolean(val));
-  case KindOfDouble:       return op(cell.m_data.dbl, val->toDouble());
-  case KindOfArray:        return op(true, false);
-  case KindOfString:
-  case KindOfStaticString: return op(cell.m_data.pstr, val);
+    case KindOfUninit:
+    case KindOfNull:
+      return op(staticEmptyString(), val);
 
-  case KindOfInt64:
-    {
+    case KindOfInt64: {
       auto const num = stringToNumeric(val);
       return num.m_type == KindOfInt64  ? op(cell.m_data.num, num.m_data.num) :
              num.m_type == KindOfDouble ? op(cell.m_data.num, num.m_data.dbl) :
              op(cell.m_data.num, 0);
     }
+    case KindOfBoolean:
+      return op(!!cell.m_data.num, toBoolean(val));
 
-  case KindOfObject:
-    {
+    case KindOfDouble: {
+      auto const num = stringToNumeric(val);
+      return num.m_type == KindOfInt64  ? op(cell.m_data.dbl, num.m_data.num) :
+             num.m_type == KindOfDouble ? op(cell.m_data.dbl, num.m_data.dbl) :
+             op(cell.m_data.dbl, 0);
+    }
+
+    case KindOfStaticString:
+    case KindOfString:
+      return op(cell.m_data.pstr, val);
+
+    case KindOfPersistentArray:
+    case KindOfArray:
+      return op(true, false);
+
+    case KindOfObject: {
       auto od = cell.m_data.pobj;
       if (od->isCollection()) return op.collectionVsNonObj();
       if (od->hasToString()) {
@@ -148,141 +184,185 @@ bool cellRelOp(Op op, Cell cell, const StringData* val) {
       return op(true, false);
     }
 
-  case KindOfResource:
-    {
+    case KindOfResource: {
       auto const rd = cell.m_data.pres;
-      return op(rd->o_toDouble(), val->toDouble());
+      return op(rd->data()->o_toDouble(), val->toDouble());
     }
 
-  default:
-    break;
+    case KindOfRef:
+    case KindOfClass:
+      break;
   }
   not_reached();
 }
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, const ArrayData* ad) {
+typename Op::RetType cellRelOp(Op op, Cell cell, const ArrayData* ad) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfUninit:
-  case KindOfNull:         return op(false, !ad->empty());
-  case KindOfBoolean:      return op(cell.m_data.num, !ad->empty());
-  case KindOfInt64:        return op(false, true);
-  case KindOfDouble:       return op(false, true);
-  case KindOfArray:        return op(cell.m_data.parr, ad);
-  case KindOfStaticString:
-  case KindOfString:       return op(false, true);
-  case KindOfObject:
-    {
+    case KindOfUninit:
+    case KindOfNull:
+      return op(false, !ad->empty());
+
+    case KindOfBoolean:
+      return op(cell.m_data.num, !ad->empty());
+
+    case KindOfInt64:
+      return op(false, true);
+
+    case KindOfDouble:
+      return op(false, true);
+
+    case KindOfStaticString:
+    case KindOfString:
+      return op(false, true);
+
+    case KindOfPersistentArray:
+    case KindOfArray:
+      return op(cell.m_data.parr, ad);
+
+    case KindOfObject: {
       auto const od = cell.m_data.pobj;
       return od->isCollection()
         ? op.collectionVsNonObj()
         : op(true, false);
     }
-  case KindOfResource:     return op(false, true);
-  default:
-    break;
-  }
 
+    case KindOfResource:
+      return op(false, true);
+
+    case KindOfRef:
+    case KindOfClass:
+      break;
+  }
   not_reached();
 }
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, const ObjectData* od) {
+typename Op::RetType cellRelOp(Op op, Cell cell, const ObjectData* od) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfUninit:
-  case KindOfNull:        return op(false, true);
-  case KindOfBoolean:     return op(!!cell.m_data.num, od->o_toBoolean());
-  case KindOfInt64:
-    return od->isCollection() ? op.collectionVsNonObj()
-                              : op(cell.m_data.num, od->o_toInt64());
-  case KindOfDouble:
-    return od->isCollection() ? op.collectionVsNonObj()
-                              : op(cell.m_data.dbl, od->o_toDouble());
-  case KindOfArray:
-      return od->isCollection() ? op.collectionVsNonObj() : op(false, true);
-  case KindOfString:
-  case KindOfStaticString: {
-    auto obj = const_cast<ObjectData*>(od);
-    if (obj->isCollection()) return op.collectionVsNonObj();
-    if (obj->hasToString()) {
-      String str(obj->invokeToString());
-      return op(cell.m_data.pstr, str.get());
+    case KindOfUninit:
+    case KindOfNull:
+      return op(false, true);
+
+    case KindOfBoolean:
+      return op(!!cell.m_data.num, od->toBoolean());
+
+    case KindOfInt64:
+      return od->isCollection() ? op.collectionVsNonObj()
+                                : op(cell.m_data.num, od->toInt64());
+
+    case KindOfDouble:
+      return od->isCollection() ? op.collectionVsNonObj()
+                                : op(cell.m_data.dbl, od->toDouble());
+
+    case KindOfStaticString:
+    case KindOfString: {
+      auto obj = const_cast<ObjectData*>(od);
+      if (obj->isCollection()) return op.collectionVsNonObj();
+      if (obj->hasToString()) {
+        String str(obj->invokeToString());
+        return op(cell.m_data.pstr, str.get());
+      }
+      return op(false, true);
     }
-    return op(false, true);
-  }
-  case KindOfObject:
-    return op(cell.m_data.pobj, od);
-  case KindOfResource:
-    return op(false, true);
-  default:
-    break;
-  }
 
+    case KindOfPersistentArray:
+    case KindOfArray:
+        return od->isCollection() ? op.collectionVsNonObj() : op(false, true);
+
+    case KindOfObject:
+      return op(cell.m_data.pobj, od);
+
+    case KindOfResource:
+      return op(false, true);
+
+    case KindOfRef:
+    case KindOfClass:
+      break;
+  }
   not_reached();
 }
 
 template<class Op>
-bool cellRelOp(Op op, Cell cell, const ResourceData* rd) {
+typename Op::RetType cellRelOp(Op op, Cell cell, const ResourceData* rd) {
   assert(cellIsPlausible(cell));
 
   switch (cell.m_type) {
-  case KindOfUninit:
-  case KindOfNull:        return op(false, true);
-  case KindOfBoolean:     return op(!!cell.m_data.num, rd->o_toBoolean());
-  case KindOfInt64:
-    return op(cell.m_data.num, rd->o_toInt64());
-  case KindOfDouble:
-    return op(cell.m_data.dbl, rd->o_toDouble());
-  case KindOfArray:
-    return op(true, false);
-  case KindOfString:
-  case KindOfStaticString:
-    {
+    case KindOfUninit:
+    case KindOfNull:
+      return op(false, true);
+
+    case KindOfBoolean:
+      return op(!!cell.m_data.num, rd->o_toBoolean());
+
+    case KindOfInt64:
+      return op(cell.m_data.num, rd->o_toInt64());
+
+    case KindOfDouble:
+      return op(cell.m_data.dbl, rd->o_toDouble());
+
+    case KindOfStaticString:
+    case KindOfString: {
       auto const str = cell.m_data.pstr;
       return op(str->toDouble(), rd->o_toDouble());
     }
-  case KindOfObject:
-    return op(true, false);
-  case KindOfResource:
-    return op(cell.m_data.pres, rd);
-  default:
-    break;
-  }
 
+    case KindOfPersistentArray:
+    case KindOfArray:
+      return op(true, false);
+
+    case KindOfObject:
+      return op(true, false);
+
+    case KindOfResource:
+      return op(cell.m_data.pres->data(), rd);
+
+    case KindOfRef:
+    case KindOfClass:
+      break;
+  }
   not_reached();
 }
+template<class Op>
+typename Op::RetType cellRelOp(Op op, Cell cell, const ResourceHdr* r) {
+  return cellRelOp(op, cell, r->data());
+}
 
 template<class Op>
-bool cellRelOp(Op op, Cell c1, Cell c2) {
+typename Op::RetType cellRelOp(Op op, Cell c1, Cell c2) {
   assert(cellIsPlausible(c1));
   assert(cellIsPlausible(c2));
 
   switch (c2.m_type) {
   case KindOfUninit:
   case KindOfNull:
-    return IS_STRING_TYPE(c1.m_type) ? op(c1.m_data.pstr, staticEmptyString()) :
+    return isStringType(c1.m_type) ? op(c1.m_data.pstr, staticEmptyString()) :
            c1.m_type == KindOfObject ? op(true, false) :
            cellRelOp(op, c1, false);
+
   case KindOfInt64:        return cellRelOp(op, c1, c2.m_data.num);
   case KindOfBoolean:      return cellRelOp(op, c1, !!c2.m_data.num);
   case KindOfDouble:       return cellRelOp(op, c1, c2.m_data.dbl);
   case KindOfStaticString:
   case KindOfString:       return cellRelOp(op, c1, c2.m_data.pstr);
+  case KindOfPersistentArray:
   case KindOfArray:        return cellRelOp(op, c1, c2.m_data.parr);
   case KindOfObject:       return cellRelOp(op, c1, c2.m_data.pobj);
   case KindOfResource:     return cellRelOp(op, c1, c2.m_data.pres);
-  default:
+
+  case KindOfRef:
+  case KindOfClass:
     break;
   }
   not_reached();
 }
 
 template<class Op>
-bool tvRelOp(Op op, TypedValue tv1, TypedValue tv2) {
+typename Op::RetType tvRelOp(Op op, TypedValue tv1, TypedValue tv2) {
   assert(tvIsPlausible(tv1));
   assert(tvIsPlausible(tv2));
   return cellRelOp(op, *tvToCell(&tv1), *tvToCell(&tv2));
@@ -299,10 +379,12 @@ bool tvRelOp(Op op, TypedValue tv1, TypedValue tv2) {
  * vs obj function should handle the collection vs collection and
  * collection vs non-collection object cases.)  This is just to handle
  * that php operator == returns false in these cases, while the Lt/Gt
- * operators throw and exception.
+ * operators throw an exception.
  */
 
 struct Eq {
+  using RetType = bool;
+
   template<class T, class U>
   typename std::enable_if<
     !std::is_pointer<T>::value &&
@@ -319,40 +401,24 @@ struct Eq {
   }
 
   bool operator()(const ObjectData* od1, const ObjectData* od2) const {
-    if (od1 == od2) return true;
-    if (od1->isCollection()) {
-      return collectionEquals(od1, od2);
-    }
-    if (UNLIKELY(od1->instanceof(SystemLib::s_DateTimeInterfaceClass)
-        && od2->instanceof(SystemLib::s_DateTimeInterfaceClass))) {
-      return c_DateTime::GetTimestamp(od1) == c_DateTime::GetTimestamp(od2);
-    }
-    if (od1->getVMClass() != od2->getVMClass()) return false;
-    if (UNLIKELY(od1->instanceof(SystemLib::s_ArrayObjectClass))) {
-      // Compare the whole object, not just the array representation
-      Array ar1(ArrayData::Create());
-      Array ar2(ArrayData::Create());
-      od1->o_getArray(ar1, false);
-      od2->o_getArray(ar2, false);
-      return ar1->equal(ar2.get(), false);
-    }
-    if (UNLIKELY(od1->instanceof(SystemLib::s_ClosureClass))) {
-      // First comparison already proves they are different
-      return false;
-    }
-    Array ar1(od1->o_toArray());
-    Array ar2(od2->o_toArray());
-    return ar1->equal(ar2.get(), false);
+    assert(od1);
+    assert(od2);
+    return od1->equal(*od2);
   }
 
   bool operator()(const ResourceData* od1, const ResourceData* od2) const {
     return od1 == od2;
   }
+  bool operator()(const ResourceHdr* od1, const ResourceHdr* od2) const {
+    return od1 == od2;
+  }
 
   bool collectionVsNonObj() const { return false; }
 };
 
 struct Lt {
+  using RetType = bool;
+
   template<class T, class U>
   typename std::enable_if<
     !std::is_pointer<T>::value &&
@@ -369,33 +435,26 @@ struct Lt {
   }
 
   bool operator()(const ObjectData* od1, const ObjectData* od2) const {
-    if (od1->isCollection() || od2->isCollection()) {
-      throw_collection_compare_exception();
-    }
-    if (od1 == od2) return false;
-    if (UNLIKELY(od1->instanceof(SystemLib::s_DateTimeInterfaceClass)
-        && od2->instanceof(SystemLib::s_DateTimeInterfaceClass))) {
-      return c_DateTime::GetTimestamp(od1) < c_DateTime::GetTimestamp(od2);
-    }
-    if (UNLIKELY(od1->instanceof(SystemLib::s_ClosureClass))) {
-      return false;
-    }
-    Array ar1(od1->o_toArray());
-    Array ar2(od2->o_toArray());
-    return (*this)(ar1.get(), ar2.get());
+    assert(od1);
+    assert(od2);
+    return od1->less(*od2);
   }
 
   bool operator()(const ResourceData* rd1, const ResourceData* rd2) const {
     return rd1->o_toInt64() < rd2->o_toInt64();
   }
+  bool operator()(const ResourceHdr* rd1, const ResourceHdr* rd2) const {
+    return rd1->data()->o_toInt64() < rd2->data()->o_toInt64();
+  }
 
   bool collectionVsNonObj() const {
     throw_collection_compare_exception();
-    not_reached();
   }
 };
 
 struct Gt {
+  using RetType = bool;
+
   template<class T, class U>
   typename std::enable_if<
     !std::is_pointer<T>::value &&
@@ -412,29 +471,58 @@ struct Gt {
   }
 
   bool operator()(const ObjectData* od1, const ObjectData* od2) const {
-    if (od1->isCollection() || od2->isCollection()) {
-      throw_collection_compare_exception();
-    }
-    if (od1 == od2) return false;
-    if (UNLIKELY(od1->instanceof(SystemLib::s_DateTimeInterfaceClass)
-        && od2->instanceof(SystemLib::s_DateTimeInterfaceClass))) {
-      return c_DateTime::GetTimestamp(od1) > c_DateTime::GetTimestamp(od2);
-    }
-    if (UNLIKELY(od1->instanceof(SystemLib::s_ClosureClass))) {
-      return false;
-    }
-    Array ar1(od1->o_toArray());
-    Array ar2(od2->o_toArray());
-    return (*this)(ar1.get(), ar2.get());
+    assert(od1);
+    assert(od2);
+    return od1->more(*od2);
   }
 
   bool operator()(const ResourceData* rd1, const ResourceData* rd2) const {
     return rd1->o_toInt64() > rd2->o_toInt64();
   }
+  bool operator()(const ResourceHdr* rd1, const ResourceHdr* rd2) const {
+    return rd1->data()->o_toInt64() > rd2->data()->o_toInt64();
+  }
 
   bool collectionVsNonObj() const {
     throw_collection_compare_exception();
-    not_reached();
+  }
+};
+
+struct Cmp {
+  using RetType = int64_t;
+
+  template<class T, class U>
+  typename std::enable_if<
+    !std::is_pointer<T>::value &&
+    !std::is_pointer<U>::value,
+    int64_t
+  >::type operator()(T t, U u) const {
+    // This ordering is required so that -1 is returned for NaNs (to match PHP7
+    // behavior).
+    return (t == u) ? 0 : ((t > u) ? 1 : -1);
+  }
+
+  int64_t operator()(const StringData* sd1, const StringData* sd2) const {
+    return HPHP::compare(sd1, sd2);
+  }
+
+  int64_t operator()(const ArrayData* ad1, const ArrayData* ad2) const {
+    return HPHP::compare(ad1, ad2);
+  }
+
+  int64_t operator()(const ObjectData* od1, const ObjectData* od2) const {
+    return HPHP::compare(od1, od2);
+  }
+
+  int64_t operator()(const ResourceData* rd1, const ResourceData* rd2) const {
+    return HPHP::compare(rd1->hdr(), rd2->hdr());
+  }
+  int64_t operator()(const ResourceHdr* rd1, const ResourceHdr* rd2) const {
+    return HPHP::compare(rd1, rd2);
+  }
+
+  int64_t collectionVsNonObj() const {
+    throw_collection_compare_exception();
   }
 };
 
@@ -446,39 +534,44 @@ bool cellSame(Cell c1, Cell c2) {
   assert(cellIsPlausible(c1));
   assert(cellIsPlausible(c2));
 
-  bool const null1 = IS_NULL_TYPE(c1.m_type);
-  bool const null2 = IS_NULL_TYPE(c2.m_type);
+  bool const null1 = isNullType(c1.m_type);
+  bool const null2 = isNullType(c2.m_type);
   if (null1 && null2) return true;
   if (null1 || null2) return false;
 
   switch (c1.m_type) {
-  case KindOfInt64:
-  case KindOfBoolean:
-    if (c2.m_type != c1.m_type) return false;
-    return c1.m_data.num == c2.m_data.num;
-  case KindOfDouble:
-    if (c2.m_type != c1.m_type) return false;
-    return c1.m_data.dbl == c2.m_data.dbl;
+    case KindOfBoolean:
+    case KindOfInt64:
+      if (c2.m_type != c1.m_type) return false;
+      return c1.m_data.num == c2.m_data.num;
 
-  case KindOfStaticString:
-  case KindOfString:
-    if (!IS_STRING_TYPE(c2.m_type)) return false;
-    return c1.m_data.pstr->same(c2.m_data.pstr);
+    case KindOfDouble:
+      if (c2.m_type != c1.m_type) return false;
+      return c1.m_data.dbl == c2.m_data.dbl;
 
-  case KindOfArray:
-    if (c2.m_type != KindOfArray) return false;
-    return c1.m_data.parr->equal(c2.m_data.parr, true);
+    case KindOfStaticString:
+    case KindOfString:
+      if (!isStringType(c2.m_type)) return false;
+      return c1.m_data.pstr->same(c2.m_data.pstr);
 
-  case KindOfObject:
-    return c2.m_type == KindOfObject &&
-      c1.m_data.pobj == c2.m_data.pobj;
+    case KindOfPersistentArray:
+    case KindOfArray:
+      if (!isArrayType(c2.m_type)) return false;
+      return c1.m_data.parr->equal(c2.m_data.parr, true);
 
-  case KindOfResource:
-    return c2.m_type == KindOfResource &&
-      c1.m_data.pres == c2.m_data.pres;
+    case KindOfObject:
+      return c2.m_type == KindOfObject &&
+        c1.m_data.pobj == c2.m_data.pobj;
 
-  default:
-    break;
+    case KindOfResource:
+      return c2.m_type == KindOfResource &&
+        c1.m_data.pres == c2.m_data.pres;
+
+    case KindOfUninit:
+    case KindOfNull:
+    case KindOfRef:
+    case KindOfClass:
+      break;
   }
   not_reached();
 }
@@ -518,6 +611,9 @@ bool cellEqual(Cell cell, const ObjectData* val) {
 bool cellEqual(Cell cell, const ResourceData* val) {
   return cellRelOp(Eq(), cell, val);
 }
+bool cellEqual(Cell cell, const ResourceHdr* val) {
+  return cellRelOp(Eq(), cell, val);
+}
 
 bool cellEqual(Cell c1, Cell c2) {
   return cellRelOp(Eq(), c1, c2);
@@ -554,6 +650,9 @@ bool cellLess(Cell cell, const ObjectData* val) {
 bool cellLess(Cell cell, const ResourceData* val) {
   return cellRelOp(Lt(), cell, val);
 }
+bool cellLess(Cell cell, const ResourceHdr* val) {
+  return cellRelOp(Lt(), cell, val);
+}
 
 bool cellLess(Cell c1, Cell c2) {
   return cellRelOp(Lt(), c1, c2);
@@ -590,6 +689,9 @@ bool cellGreater(Cell cell, const ObjectData* val) {
 bool cellGreater(Cell cell, const ResourceData* val) {
   return cellRelOp(Gt(), cell, val);
 }
+bool cellGreater(Cell cell, const ResourceHdr* val) {
+  return cellRelOp(Gt(), cell, val);
+}
 
 bool cellGreater(Cell c1, Cell c2) {
   return cellRelOp(Gt(), c1, c2);
@@ -601,11 +703,52 @@ bool tvGreater(TypedValue tv1, TypedValue tv2) {
 
 //////////////////////////////////////////////////////////////////////
 
+int64_t cellCompare(Cell cell, bool val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell cell, int64_t val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell cell, double val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell cell, const StringData* val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell cell, const ArrayData* val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell cell, const ObjectData* val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell cell, const ResourceData* val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+int64_t cellCompare(Cell cell, const ResourceHdr* val) {
+  return cellRelOp(Cmp(), cell, val);
+}
+
+int64_t cellCompare(Cell c1, Cell c2) {
+  return cellRelOp(Cmp(), c1, c2);
+}
+
+int64_t tvCompare(TypedValue tv1, TypedValue tv2) {
+  return tvRelOp(Cmp(), tv1, tv2);
+}
+
+//////////////////////////////////////////////////////////////////////
+
 bool cellLessOrEqual(Cell c1, Cell c2) {
   assert(cellIsPlausible(c1));
   assert(cellIsPlausible(c2));
 
-  if ((c1.m_type == KindOfArray && c2.m_type == KindOfArray) ||
+  if ((isArrayType(c1.m_type) && isArrayType(c2.m_type)) ||
       (c1.m_type == KindOfObject && c2.m_type == KindOfObject) ||
       (c1.m_type == KindOfResource && c2.m_type == KindOfResource)) {
     return cellLess(c1, c2) || cellEqual(c1, c2);
@@ -624,7 +767,7 @@ bool cellGreaterOrEqual(Cell c1, Cell c2) {
   assert(cellIsPlausible(c1));
   assert(cellIsPlausible(c2));
 
-  if ((c1.m_type == KindOfArray && c2.m_type == KindOfArray) ||
+  if ((isArrayType(c1.m_type) && isArrayType(c2.m_type)) ||
       (c1.m_type == KindOfObject && c2.m_type == KindOfObject) ||
       (c1.m_type == KindOfResource && c2.m_type == KindOfResource)) {
     return cellGreater(c1, c2) || cellEqual(c1, c2);
