@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,13 +22,56 @@
 #include "hphp/runtime/base/resource-data.h"
 #include "hphp/runtime/base/string-data.h"
 #include "hphp/runtime/base/typed-value.h"
-#include "hphp/runtime/base/types.h"
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
 
 struct Variant;
 
+/*
+ * Call a (ref-count related) function on a TypedValue data pointer where we
+ * don't statically know the pointer's type. This is superior to the old idiom
+ * of choosing one of the pointers, calling the method for that type, and
+ * relying on the fact that all the method's have the same code, and therefore
+ * will work correctly even if its actually a different type. This breaks strong
+ * type aliasing and can cause GCC to miscompile code in hard to debug ways.
+ * Instead use manual pointer arithmetic to index to where the ref-count is,
+ * cast it, then call the appropriate generic function. Since we don't use any
+ * of the pointer types, the compiler cannot use type aliasing assumptions to
+ * miscompile.
+ *
+ * If we're in debug mode, switch on the type and call the appropriate function
+ * directly. This is slightly slower, but gives us the benefit of extra
+ * assertions.
+ *
+ * This assumes isRefcountedType() is true.
+ */
+#define TV_GENERIC_DISPATCH_FAST(exp, func)                     \
+  reinterpret_cast<HPHP::HeaderWord<uint16_t,HPHP::Counted::Maybe>*>(\
+      (exp).m_data.num + HPHP::HeaderOffset                     \
+  )->func()
+
+#define TV_GENERIC_DISPATCH_SLOW(exp, func) \
+  [](HPHP::TypedValue tv) {                                     \
+    switch (tv.m_type) {                                        \
+      case HPHP::KindOfStaticString:                            \
+      case HPHP::KindOfString: return tv.m_data.pstr->func();   \
+      case HPHP::KindOfPersistentArray:                             \
+      case HPHP::KindOfArray: return tv.m_data.parr->func();    \
+      case HPHP::KindOfObject: return tv.m_data.pobj->func();   \
+      case HPHP::KindOfResource: return tv.m_data.pres->func(); \
+      case HPHP::KindOfRef: return tv.m_data.pref->func();      \
+      default:                                                  \
+        assert_flog(false, "Bad KindOf: {}", (size_t)tv.m_type);\
+    }                                                           \
+  }(exp)
+
+#ifdef DEBUG
+#define TV_GENERIC_DISPATCH(exp, func) TV_GENERIC_DISPATCH_SLOW(exp, func)
+#else
+#define TV_GENERIC_DISPATCH(exp, func) TV_GENERIC_DISPATCH_FAST(exp, func)
+#endif
+
 /*
  * Assertions on Cells and TypedValues.  Should usually only happen
  * inside an assert().
@@ -41,7 +84,7 @@ bool refIsPlausible(Ref);
  * In a debug build, write garbage into a memory slot for a TypedValue
  * that should not be used anymore.
  */
-inline void tvDebugTrash(TypedValue* tv) {
+ALWAYS_INLINE void tvDebugTrash(TypedValue* tv) {
   if (debug) memset(tv, kTVTrashFill, sizeof *tv);
 }
 
@@ -49,7 +92,7 @@ inline void tvDebugTrash(TypedValue* tv) {
  * Returns: true if the supplied TypedValue is KindOfDouble or
  * KindOfInt64.  I.e. if it is a TypedNum.
  */
-inline bool isTypedNum(const TypedValue& tv) {
+ALWAYS_INLINE bool isTypedNum(const TypedValue& tv) {
   return tv.m_type == KindOfInt64 || tv.m_type == KindOfDouble;
 }
 
@@ -58,16 +101,16 @@ inline bool isTypedNum(const TypedValue& tv) {
  * a non-reference counted type, or is a KindOfString that points to a
  * static string.
  */
-inline bool isUncounted(const TypedValue& tv) {
-  auto const uncounted = !IS_REFCOUNTED_TYPE(tv.m_type) ||
+ALWAYS_INLINE bool isUncounted(const TypedValue& tv) {
+  auto const uncounted = !isRefcountedType(tv.m_type) ||
     (tv.m_type == KindOfString && tv.m_data.pstr->isStatic());
   if (uncounted) assert(cellIsPlausible(tv));
   return uncounted;
 }
 
 // Assumes 'data' is live
-// Assumes 'IS_REFCOUNTED_TYPE(type)'
-void tvDecRefHelper(DataType type, uint64_t datum);
+// Assumes 'isRefcountedType(type)'
+void tvDecRefHelper(DataType type, uint64_t datum) noexcept;
 
 /*
  * Returns true if decreffing the specified TypedValue will free heap-allocated
@@ -80,100 +123,91 @@ bool tvDecRefWillRelease(TypedValue* tv);
  * helper to be called. Note that there are cases where this function returns
  * true but tvDecRefWillRelease() will return false.
  */
-inline bool tvDecRefWillCallHelper(TypedValue* tv) {
-  return IS_REFCOUNTED_TYPE(tv->m_type) &&
-    !tv->m_data.pstr->hasMultipleRefs();
+ALWAYS_INLINE bool tvDecRefWillCallHelper(TypedValue* tv) {
+  return isRefcountedType(tv->m_type) &&
+         TV_GENERIC_DISPATCH(*tv, decWillRelease);
 }
 
-// Assumes 'tv' is live
-inline void tvRefcountedDecRefCell(TypedValue* tv) {
-  assert(tvIsPlausible(*tv));
-  if (IS_REFCOUNTED_TYPE(tv->m_type)) {
-    tvDecRefHelper(tv->m_type, tv->m_data.num);
-  }
-}
-
-inline void tvDecRefStr(TypedValue* tv) {
+ALWAYS_INLINE void tvDecRefStr(TypedValue* tv) {
   assert(tv->m_type == KindOfString);
   decRefStr(tv->m_data.pstr);
 }
 
-inline void tvDecRefArr(TypedValue* tv) {
+ALWAYS_INLINE void tvDecRefArr(TypedValue* tv) {
   assert(tv->m_type == KindOfArray);
   decRefArr(tv->m_data.parr);
 }
 
-inline void tvDecRefObj(TypedValue* tv) {
+ALWAYS_INLINE void tvDecRefObj(TypedValue* tv) {
   assert(tv->m_type == KindOfObject);
   decRefObj(tv->m_data.pobj);
 }
 
-inline void tvDecRefRes(TypedValue* tv) {
+ALWAYS_INLINE void tvDecRefRes(TypedValue* tv) {
   assert(tv->m_type == KindOfResource);
   decRefRes(tv->m_data.pres);
 }
 
 // Assumes 'r' is live and points to a RefData
-inline void tvDecRefRefInternal(RefData* r) {
-  assert(tvIsPlausible(*r->tv()));
-  assert(r->tv()->m_type != KindOfRef);
-  assert(r->getRealCount() > 0);
+ALWAYS_INLINE void tvDecRefRefInternal(RefData* r) {
+  assert(cellIsPlausible(*r->tv()));
   decRefRef(r);
 }
 
 // Assumes 'tv' is live
-inline void tvDecRefRef(TypedValue* tv) {
+ALWAYS_INLINE void tvDecRefRef(TypedValue* tv) {
   assert(tv->m_type == KindOfRef);
   tvDecRefRefInternal(tv->m_data.pref);
 }
 
 // Assumes 'tv' is live
-inline void tvRefcountedDecRefHelper(DataType type, uint64_t datum) {
-  if (IS_REFCOUNTED_TYPE(type)) {
+ALWAYS_INLINE void tvRefcountedDecRefHelper(DataType type, uint64_t datum) {
+  if (isRefcountedType(type)) {
     tvDecRefHelper(type, datum);
   }
 }
 
-inline void tvRefcountedDecRef(TypedValue v) {
-  return tvRefcountedDecRefHelper(v.m_type, v.m_data.num);
-}
-
-// Assumes the value is live, and has a count of at least 2 coming in.
-inline void tvRefcountedDecRefHelperNZ(DataType type, uint64_t datum) {
-  if (IS_REFCOUNTED_TYPE(type)) {
-    auto* asStr = reinterpret_cast<StringData*>(datum);
-    auto const DEBUG_ONLY newCount = asStr->decRefCount();
-    assert(newCount != 0);
+// Assumes 'tv' is live and unlikely to be a refcounted type
+ALWAYS_INLINE void tvUnlikelyRefcountedDecRef(TypedValue tv) {
+  if (UNLIKELY(isRefcountedType(tv.m_type))) {
+    tvDecRefHelper(tv.m_type, tv.m_data.num);
   }
 }
 
-inline void tvRefcountedDecRefNZ(TypedValue v) {
-  return tvRefcountedDecRefHelperNZ(v.m_type, v.m_data.num);
+ALWAYS_INLINE void tvRefcountedDecRef(TypedValue v) {
+  return tvRefcountedDecRefHelper(v.m_type, v.m_data.num);
+}
+
+ALWAYS_INLINE void tvRefcountedDecRefNZ(TypedValue tv) {
+  if (isRefcountedType(tv.m_type)) {
+    TV_GENERIC_DISPATCH(tv, decRefCount);
+  }
 }
 
 // Assumes 'tv' is live
-// Assumes 'IS_REFCOUNTED_TYPE(tv->m_type)'
-inline void tvDecRef(TypedValue* tv) {
+// Assumes 'isRefcountedType(tv->m_type)'
+ALWAYS_INLINE void tvDecRef(TypedValue* tv) {
   tvDecRefHelper(tv->m_type, tv->m_data.num);
 }
 
 // Assumes 'tv' is live
 ALWAYS_INLINE void tvRefcountedDecRef(TypedValue* tv) {
-  if (IS_REFCOUNTED_TYPE(tv->m_type)) {
+  if (isRefcountedType(tv->m_type)) {
     tvDecRef(tv);
+    // If we're in debug mode, turn the entry into null so that the GC doesn't
+    // assert if it tries to follow it.
+    if (debug) tv->m_type = KindOfNull;
   }
 }
 
 // decref when the count is known not to reach zero
 ALWAYS_INLINE void tvDecRefOnly(TypedValue* tv) {
   assert(!tvDecRefWillCallHelper(tv));
-  if (IS_REFCOUNTED_TYPE(tv->m_type)) {
-    tv->m_data.pstr->decRefCount();
-  }
+  tvRefcountedDecRefNZ(*tv);
 }
 
 // Assumes 'tv' is live
-inline TypedValue* tvBox(TypedValue* tv) {
+ALWAYS_INLINE TypedValue* tvBox(TypedValue* tv) {
   assert(tvIsPlausible(*tv));
   assert(tv->m_type != KindOfRef);
   tv->m_data.pref = RefData::Make(*tv);
@@ -182,31 +216,33 @@ inline TypedValue* tvBox(TypedValue* tv) {
 }
 
 // Assumes 'tv' is live
-// Assumes 'IS_REFCOUNTED_TYPE(tv->m_type)'
-inline void tvIncRef(const TypedValue* tv) {
+// Assumes 'isRefcountedType(tv->m_type)'
+ALWAYS_INLINE void tvIncRef(const TypedValue* tv) {
   assert(tvIsPlausible(*tv));
-  assert(IS_REFCOUNTED_TYPE(tv->m_type));
-  tv->m_data.pstr->incRefCount();
+  assert(isRefcountedType(tv->m_type));
+  TV_GENERIC_DISPATCH(*tv, incRefCount);
 }
 
 ALWAYS_INLINE void tvRefcountedIncRef(const TypedValue* tv) {
   assert(tvIsPlausible(*tv));
-  if (IS_REFCOUNTED_TYPE(tv->m_type)) {
+  if (isRefcountedType(tv->m_type)) {
     tvIncRef(tv);
   }
 }
 
 // Assumes 'tv' is live
-// Assumes 'IS_REFCOUNTED_TYPE(tv->m_type)'
+// Assumes 'isRefcountedType(tv->m_type)'
 // Assumes 'tv' is not shared (ie KindOfRef or KindOfObject)
-inline void tvIncRefNotShared(TypedValue* tv) {
+ALWAYS_INLINE void tvIncRefNotShared(TypedValue* tv) {
   assert(tv->m_type == KindOfObject || tv->m_type == KindOfRef);
-  tv->m_data.pobj->incRefCount();
+  tv->m_type == KindOfObject ?
+    tv->m_data.pobj->incRefCount() :
+    tv->m_data.pref->incRefCount();
 }
 
 // Assumes 'tv' is live
 // Assumes 'tv.m_type == KindOfRef'
-inline void tvUnbox(TypedValue* tv) {
+ALWAYS_INLINE void tvUnbox(TypedValue* tv) {
   assert(tvIsPlausible(*tv));
   assert(tv->m_type == KindOfRef);
   RefData* r = tv->m_data.pref;
@@ -225,7 +261,7 @@ inline void tvUnbox(TypedValue* tv) {
  * Copies the m_data and m_type fields, but not m_aux.  (For that you
  * need TypedValue::operator=.)
  */
-inline void tvCopy(const TypedValue& fr, TypedValue& to) {
+ALWAYS_INLINE void tvCopy(const TypedValue& fr, TypedValue& to) {
   assert(tvIsPlausible(fr));
   to.m_data.num = fr.m_data.num;
   to.m_type = fr.m_type;
@@ -235,11 +271,11 @@ inline void tvCopy(const TypedValue& fr, TypedValue& to) {
  * Equivalent of tvCopy for Cells and Vars.  These functions have the
  * same effects as tvCopy, but have some added assertions.
  */
-inline void cellCopy(const Cell& fr, Cell& to) {
+ALWAYS_INLINE void cellCopy(const Cell fr, Cell& to) {
   assert(cellIsPlausible(fr));
   tvCopy(fr, to);
 }
-inline void refCopy(const Ref& fr, Ref& to) {
+ALWAYS_INLINE void refCopy(const Ref fr, Ref& to) {
   assert(refIsPlausible(fr));
   tvCopy(fr, to);
 }
@@ -249,7 +285,7 @@ inline void refCopy(const Ref& fr, Ref& to) {
  * m_type fields, and increments reference count. Does not perform a
  * decRef on to.
  */
-inline void tvDup(const TypedValue& fr, TypedValue& to) {
+ALWAYS_INLINE void tvDup(const TypedValue& fr, TypedValue& to) {
   tvCopy(fr, to);
   tvRefcountedIncRef(&to);
 }
@@ -259,7 +295,7 @@ inline void tvDup(const TypedValue& fr, TypedValue& to) {
  * m_type fields, and increments the reference count. Does not perform
  * a decRef on the value that was overwritten.
  */
-inline void cellDup(const Cell& fr, Cell& to) {
+ALWAYS_INLINE void cellDup(const Cell fr, Cell& to) {
   assert(cellIsPlausible(fr));
   tvCopy(fr, to);
   tvRefcountedIncRef(&to);
@@ -270,48 +306,55 @@ inline void cellDup(const Cell& fr, Cell& to) {
  * m_type fields and increments the reference count. Does not perform
  * as decRef on the value that was overwritten.
  */
-inline void refDup(const Ref& fr, Ref& to) {
+ALWAYS_INLINE void refDup(const Ref fr, Ref& to) {
   assert(refIsPlausible(fr));
-  to.m_data.num = fr.m_data.num;
   to.m_type = KindOfRef;
-  tvIncRefNotShared(&to);
+  to.m_data.pref = fr.m_data.pref;
+  to.m_data.pref->incRefCount();
 }
 
 // Assumes 'tv' is dead
 // NOTE: this helper does not modify tv->m_aux
-inline void tvWriteNull(TypedValue* tv) {
+ALWAYS_INLINE void tvWriteNull(TypedValue* tv) {
   tv->m_type = KindOfNull;
 }
 
 // Assumes 'tv' is dead
 // NOTE: this helper does not modify tv->m_aux
-inline void tvWriteUninit(TypedValue* tv) {
+ALWAYS_INLINE void tvWriteUninit(TypedValue* tv) {
   tv->m_type = KindOfUninit;
 }
 
 // Assumes 'tv' is dead
-inline void tvWriteObject(ObjectData* pobj, TypedValue* tv) {
+ALWAYS_INLINE void tvWriteObject(ObjectData* pobj, TypedValue* tv) {
+  tv->m_type = KindOfObject;
+  tv->m_data.pobj = pobj;
+  tv->m_data.pobj->incRefCount();
+}
+
+// Like tvWriteObject, but does not increment ref-count. Used for transferring
+// object ownership.
+ALWAYS_INLINE void tvMoveObject(ObjectData* pobj, TypedValue* tv) {
   tv->m_type = KindOfObject;
   tv->m_data.pobj = pobj;
-  tvIncRef(tv);
 }
 
 // conditionally unbox tv
-inline Cell* tvToCell(TypedValue* tv) {
+ALWAYS_INLINE Cell* tvToCell(TypedValue* tv) {
   return LIKELY(tv->m_type != KindOfRef) ? tv : tv->m_data.pref->tv();
 }
 
 // conditionally unbox tv, preserve constness.
-inline const Cell* tvToCell(const TypedValue* tv) {
+ALWAYS_INLINE const Cell* tvToCell(const TypedValue* tv) {
   return LIKELY(tv->m_type != KindOfRef) ? tv : tv->m_data.pref->tv();
 }
 
 // assert that tv is cell
-inline Cell* tvAssertCell(TypedValue* tv) {
+ALWAYS_INLINE Cell* tvAssertCell(TypedValue* tv) {
   assert(cellIsPlausible(*tv));
   return tv;
 }
-inline const Cell* tvAssertCell(const TypedValue* tv) {
+ALWAYS_INLINE const Cell* tvAssertCell(const TypedValue* tv) {
   assert(cellIsPlausible(*tv));
   return tv;
 }
@@ -325,7 +368,7 @@ inline const Cell* tvAssertCell(const TypedValue* tv) {
  *
  * `to' must contain a live php value; use cellDup when it doesn't.
  */
-inline void tvSet(const Cell& fr, TypedValue& inTo) {
+ALWAYS_INLINE void tvSet(const Cell fr, TypedValue& inTo) {
   assert(cellIsPlausible(fr));
   Cell* to = tvToCell(&inTo);
   auto const oldType = to->m_type;
@@ -334,12 +377,60 @@ inline void tvSet(const Cell& fr, TypedValue& inTo) {
   tvRefcountedDecRefHelper(oldType, oldDatum);
 }
 
+// like tvSet, but RHS is bool
+ALWAYS_INLINE void tvSetBool(bool val, TypedValue& inTo) {
+  Cell* to = tvToCell(&inTo);
+  auto const oldType = to->m_type;
+  auto const oldDatum = to->m_data.num;
+  to->m_type = KindOfBoolean;
+  to->m_data.num = val;
+  tvRefcountedDecRefHelper(oldType, oldDatum);
+}
+
+// like tvSet, but RHS is int
+ALWAYS_INLINE void tvSetInt(int64_t val, TypedValue& inTo) {
+  Cell* to = tvToCell(&inTo);
+  auto const oldType = to->m_type;
+  auto const oldDatum = to->m_data.num;
+  to->m_type = KindOfInt64;
+  to->m_data.num = val;
+  tvRefcountedDecRefHelper(oldType, oldDatum);
+}
+
+// like tvSet, but RHS is double
+ALWAYS_INLINE void tvSetDouble(double val, TypedValue& inTo) {
+  Cell* to = tvToCell(&inTo);
+  auto const oldType = to->m_type;
+  auto const oldDatum = to->m_data.num;
+  to->m_type = KindOfDouble;
+  to->m_data.dbl = val;
+  tvRefcountedDecRefHelper(oldType, oldDatum);
+}
+
+/*
+ * Assign the value of the Cell in `fr' to `to', decrementing the reference
+ * count of 'to', but not incrementing 'to'.
+ *
+ * If `to' is KindOfRef, places the value of `fr' in the RefData
+ * pointed to by `to'.
+ *
+ * `to' must contain a live php value; use cellCopy when it doesn't.
+ */
+ALWAYS_INLINE void tvMove(const Cell fr, TypedValue& inTo) {
+  assert(cellIsPlausible(fr));
+  Cell* to = tvToCell(&inTo);
+  auto const oldType = to->m_type;
+  auto const oldDatum = to->m_data.num;
+  cellCopy(fr, *to);
+  tvRefcountedDecRefHelper(oldType, oldDatum);
+}
+
 /*
  * Assign null to `to', with appropriate reference count modifications.
  *
  * `to' must contain a live php value; use tvWriteNull when it doesn't.
  */
-inline void cellSetNull(Cell& to) {
+ALWAYS_INLINE void cellSetNull(Cell& to) {
   auto const oldType = to.m_type;
   auto const oldDatum = to.m_data.num;
   tvWriteNull(&to);
@@ -353,7 +444,7 @@ inline void cellSetNull(Cell& to) {
  *
  * `to' must contain a live php value; use tvWriteNull when it doesn't.
  */
-inline void tvSetNull(TypedValue& to) {
+ALWAYS_INLINE void tvSetNull(TypedValue& to) {
   cellSetNull(*tvToCell(&to));
 }
 
@@ -368,7 +459,7 @@ inline void tvSetNull(TypedValue& to) {
  *
  * Post: `to' is a Cell.
  */
-inline void tvSetIgnoreRef(const Cell& fr, TypedValue& to) {
+ALWAYS_INLINE void tvSetIgnoreRef(const Cell fr, TypedValue& to) {
   assert(cellIsPlausible(fr));
   auto const oldType = to.m_type;
   auto const oldDatum = to.m_data.num;
@@ -383,15 +474,47 @@ inline void tvSetIgnoreRef(const Cell& fr, TypedValue& to) {
  * This function has the same effects as tvSetIgnoreRef, with stronger
  * assertions on `to'.
  */
-inline void cellSet(const Cell& fr, Cell& to) {
+ALWAYS_INLINE void cellSet(const Cell fr, Cell& to) {
   assert(cellIsPlausible(fr));
   assert(cellIsPlausible(to));
   tvSetIgnoreRef(fr, to);
 }
 
+/*
+ * Assign the value of the Cell in `fr' to `to', decrementing the reference
+ * count of 'to', but not incrementing 'from'.
+ *
+ * If `to' is KindOfRef, this function will decref the RefData and
+ * replace it with the value in `fr', unlike tvMove.
+ *
+ * `to' must contain a live php value; use cellCopy when it doesnt.
+ *
+ * Post: `to' is a Cell.
+ */
+ALWAYS_INLINE void tvMoveIgnoreRef(const Cell fr, TypedValue& to) {
+  assert(cellIsPlausible(fr));
+  auto const oldType = to.m_type;
+  auto const oldDatum = to.m_data.num;
+  cellCopy(fr, to);
+  tvRefcountedDecRefHelper(oldType, oldDatum);
+}
+
+/*
+ * Assign the value of the Cell in `fr' to the Cell `to', decrementing the
+ * reference count of 'to', but not incrementing 'from'.
+ *
+ * This function has the same effects as tvMoveIgnoreRef, with stronger
+ * assertions on `to'.
+ */
+ALWAYS_INLINE void cellMove(const Cell fr, Cell& to) {
+  assert(cellIsPlausible(fr));
+  assert(cellIsPlausible(to));
+  tvMoveIgnoreRef(fr, to);
+}
+
 // Assumes 'to' and 'fr' are live
 // Assumes that 'fr->m_type == KindOfRef'
-inline void tvBind(const TypedValue* fr, TypedValue* to) {
+ALWAYS_INLINE void tvBind(const TypedValue* fr, TypedValue* to) {
   assert(fr->m_type == KindOfRef);
   DataType oldType = to->m_type;
   uint64_t oldDatum = to->m_data.num;
@@ -400,7 +523,7 @@ inline void tvBind(const TypedValue* fr, TypedValue* to) {
 }
 
 // Assumes 'to' and 'fr' are live
-inline void tvBindRef(RefData* fr, TypedValue* to) {
+ALWAYS_INLINE void tvBindRef(RefData* fr, TypedValue* to) {
   auto const oldType  = to->m_type;
   auto const oldDatum = to->m_data.num;
   fr->incRefCount();
@@ -432,72 +555,71 @@ void tvDupWithRef(const TypedValue& frIn, TypedValue& dst) {
 }
 
 // Assumes 'to' is live
-inline void tvUnset(TypedValue * to) {
-  tvRefcountedDecRef(to);
+ALWAYS_INLINE void tvUnset(TypedValue* to) {
+  auto const oldType = to->m_type;
+  auto const oldDatum = to->m_data.num;
   tvWriteUninit(to);
+  tvRefcountedDecRefHelper(oldType, oldDatum);
 }
 
 /*
  * Returns true if this tv is not a ref-counted type, or if it is a
  * ref-counted type and the object pointed to is static.
  */
-inline bool tvIsStatic(const TypedValue* tv) {
+ALWAYS_INLINE bool tvIsStatic(const TypedValue* tv) {
   assert(tvIsPlausible(*tv));
-  return !IS_REFCOUNTED_TYPE(tv->m_type) ||
-    tv->m_data.pref->m_count == StaticValue;
+  return !isRefcountedType(tv->m_type) || TV_GENERIC_DISPATCH(*tv, isStatic);
 }
 
 /**
- * tvAsVariant and tvAsconst Variant& serve as escape hatches that allow us to call
+ * tvAsVariant and tvAsCVarRef serve as escape hatches that allow us to call
  * into the Variant machinery. Ideally we will use these as little as possible
  * in the long term.
  */
 
 // Assumes 'tv' is live
-inline Variant& tvAsVariant(TypedValue* tv) {
-  // Avoid treating uninitialized TV's as variants. We have some slightly
-  // perverse, but defensible uses where we pass in NULL (and later check
-  // a Variant* against NULL) so tolerate it.
-  assert(nullptr == tv || tvIsPlausible(*tv));
+ALWAYS_INLINE Variant& tvAsVariant(TypedValue* tv) {
+  assert(tv != nullptr);
+  assert(tvIsPlausible(*tv));
   return *(Variant*)(tv);
 }
 
-inline Variant& tvAsUninitializedVariant(TypedValue* tv) {
+ALWAYS_INLINE Variant& tvAsUninitializedVariant(TypedValue* tv) {
   // A special case, for use when constructing a variant and we don't
   // assume initialization.
   return *(Variant*)(tv);
 }
 
 // Assumes 'tv' is live
-inline const Variant& tvAsCVarRef(const TypedValue* tv) {
+ALWAYS_INLINE const Variant& tvAsCVarRef(const TypedValue* tv) {
   return *(const Variant*)(tv);
 }
 
 // Assumes 'tv' is live
-inline Variant& cellAsVariant(Cell& cell) {
+ALWAYS_INLINE Variant& cellAsVariant(Cell& cell) {
   assert(cellIsPlausible(cell));
   return *(Variant*)(&cell);
 }
 
 // Assumes 'tv' is live
-inline const Variant& cellAsCVarRef(const Cell& cell) {
+ALWAYS_INLINE const Variant& cellAsCVarRef(const Cell& cell) {
   assert(cellIsPlausible(cell));
   return *(const Variant*)(&cell);
 }
 
 // Assumes 'tv' is live
-inline Variant& refAsVariant(Ref& ref) {
+ALWAYS_INLINE Variant& refAsVariant(Ref& ref) {
   assert(refIsPlausible(ref));
   return *(Variant*)(&ref);
 }
 
 // Assumes 'tv' is live
-inline const Variant& refAsCVarRef(const Ref& ref) {
+ALWAYS_INLINE const Variant& refAsCVarRef(const Ref& ref) {
   assert(refIsPlausible(ref));
   return *(const Variant*)(&ref);
 }
 
-inline bool tvIsStronglyBound(const TypedValue* tv) {
+ALWAYS_INLINE bool tvIsStronglyBound(const TypedValue* tv) {
   return (tv->m_type == KindOfRef && tv->m_data.pref->isReferenced());
 }
 
@@ -505,7 +627,7 @@ template<class Fun>
 void tvDupFlattenImpl(const TypedValue* fr, TypedValue* to, Fun shouldFlatten) {
   tvCopy(*fr, *to);
   auto type = fr->m_type;
-  if (!IS_REFCOUNTED_TYPE(type)) return;
+  if (!isRefcountedType(type)) return;
   if (type != KindOfRef) {
     tvIncRef(to);
     return;
@@ -519,14 +641,14 @@ void tvDupFlattenImpl(const TypedValue* fr, TypedValue* to, Fun shouldFlatten) {
 }
 
 // Assumes 'fr' is live and 'to' is dead, and does not mutate to->m_aux
-inline void tvDupFlattenVars(const TypedValue* fr, TypedValue* to) {
+ALWAYS_INLINE void tvDupFlattenVars(const TypedValue* fr, TypedValue* to) {
   tvDupFlattenImpl(fr, to, [&](RefData* ref) {
     return !ref->isReferenced();
   });
 }
 
 // Assumes 'fr' is live and 'to' is dead, and does not mutate to->m_aux
-inline void tvDupFlattenVars(const TypedValue* fr, TypedValue* to,
+ALWAYS_INLINE void tvDupFlattenVars(const TypedValue* fr, TypedValue* to,
                              const ArrayData* container) {
   assert(container);
   tvDupFlattenImpl(fr, to, [&](RefData* ref) {
@@ -534,49 +656,65 @@ inline void tvDupFlattenVars(const TypedValue* fr, TypedValue* to,
   });
 }
 
-inline bool cellIsNull(const Cell* tv) {
+ALWAYS_INLINE bool cellIsNull(const Cell* tv) {
   assert(cellIsPlausible(*tv));
-  return IS_NULL_TYPE(tv->m_type);
+  return isNullType(tv->m_type);
 }
 
-inline bool tvIsString(const TypedValue* tv) {
+ALWAYS_INLINE bool tvIsString(const TypedValue* tv) {
   return (tv->m_type & KindOfStringBit) != 0;
 }
 
-inline void tvUnboxIfNeeded(TypedValue* tv) {
+ALWAYS_INLINE void tvUnboxIfNeeded(TypedValue* tv) {
   if (tv->m_type == KindOfRef) tvUnbox(tv);
 }
 
 /*
  * TypedValue conversions that update the tv in place (decrefing and
  * old value, if necessary).
+ *
+ * CastInPlace will forcibly change the value to the new type
+ *   and will not fail. (Though the results may be silly)
+ * CoerceInPlace will attempt to convert the type and
+ *   return false on failure
+ * CoerceOrThrow will attempt to convert the type and
+ *   both raise a warning and throw a TVCoercionException on failure
  */
-void tvCastToBooleanInPlace(TypedValue* tv);
-void tvCastToInt64InPlace(TypedValue* tv);
 void cellCastToInt64InPlace(Cell*);
-void tvCastToDoubleInPlace(TypedValue* tv);
 double tvCastToDouble(TypedValue* tv);
-void tvCastToStringInPlace(TypedValue* tv);
 StringData* tvCastToString(const TypedValue* tv);
-void tvCastToArrayInPlace(TypedValue* tv);
-void tvCastToObjectInPlace(TypedValue* tv);
-void tvCastToNullableObjectInPlace(TypedValue* tv);
-void tvCastToResourceInPlace(TypedValue* tv);
-
 bool tvCanBeCoercedToNumber(TypedValue* tv);
-bool tvCoerceParamToBooleanInPlace(TypedValue* tv);
-bool tvCoerceParamToInt64InPlace(TypedValue* tv);
-bool tvCoerceParamToDoubleInPlace(TypedValue* tv);
-bool tvCoerceParamToStringInPlace(TypedValue* tv);
-bool tvCoerceParamToArrayInPlace(TypedValue* tv);
-bool tvCoerceParamToObjectInPlace(TypedValue* tv);
-bool tvCoerceParamToNullableObjectInPlace(TypedValue* tv);
-bool tvCoerceParamToResourceInPlace(TypedValue* tv);
+
+/*
+ * If the current function (func, a builtin) was called in a strict context then
+ * verify that tv is the correct type for argNum or attempt to convert it to
+ * the correct type, fataling on failure.
+ *
+ * If PHP7_ScalarType is false or EnableHipHopSyntax is true, this call does
+ * nothing.
+ */
+void tvCoerceIfStrict(TypedValue& tv, int64_t argNum, const Func* func);
+
+#define X(kind) \
+void tvCastTo##kind##InPlace(TypedValue* tv); \
+bool tvCoerceParamTo##kind##InPlace(TypedValue* tv); \
+void tvCoerceParamTo##kind##OrThrow(TypedValue* tv, \
+                                    const Func* callee, \
+                                    unsigned int arg_num);
+X(Boolean)
+X(Int64)
+X(Double)
+X(String)
+X(Array)
+X(Object)
+X(NullableObject)
+X(Resource)
+#undef X
 
 typedef void(*RawDestructor)(void*);
-extern const RawDestructor g_destructors[kDestrTableSize];
+extern RawDestructor g_destructors[kDestrTableSize];
 
-inline void tvCastInPlace(TypedValue *tv, DataType DType) {
+ALWAYS_INLINE void tvCastInPlace(TypedValue *tv, DataType DType) {
 #define X(kind) \
   if (DType == KindOf##kind) { tvCastTo##kind##InPlace(tv); return; }
   X(Boolean)
@@ -590,7 +728,7 @@ inline void tvCastInPlace(TypedValue *tv, DataType DType) {
   not_reached();
 }
 
-inline bool tvCoerceParamInPlace(TypedValue* tv, DataType DType) {
+ALWAYS_INLINE bool tvCoerceParamInPlace(TypedValue* tv, DataType DType) {
 #define X(kind) \
   if (DType == KindOf##kind) return tvCoerceParamTo##kind##InPlace(tv);
   X(Boolean)
@@ -604,6 +742,28 @@ inline bool tvCoerceParamInPlace(TypedValue* tv, DataType DType) {
   not_reached();
 }
 
+/*
+ * TVCoercionException is thrown to indicate that a parameter could not be
+ * coerced when calling an HNI builtin function.
+ */
+struct TVCoercionException : public std::runtime_error {
+  TVCoercionException(const Func* func, int arg_num,
+                      DataType actual, DataType expected);
+
+  TypedValue tv() const { return m_tv; }
+private:
+  TypedValue m_tv;
+};
+
+// for debugging & instrumentation only! otherwise, stick to the
+// predicates defined in countable.h
+ALWAYS_INLINE RefCount tvGetCount(const TypedValue* tv) {
+  assert(isRefcountedType(tv->m_type));
+  return reinterpret_cast<const HeaderWord<>*>(
+    reinterpret_cast<const char*>(tv->m_data.parr) + HeaderOffset
+  )->count;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 }
 
