@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,22 +15,31 @@
 */
 
 #include "hphp/runtime/base/type-array.h"
-#include "hphp/runtime/base/base-includes.h"
-#include "hphp/runtime/base/complex-types.h"
+
+#include "hphp/runtime/base/array-init.h"
 #include "hphp/runtime/base/types.h"
-#include "hphp/runtime/base/comparisons.h"
-#include "hphp/util/exception.h"
 #include "hphp/runtime/base/apc-local-array.h"
+#include "hphp/runtime/base/array-util.h"
+#include "hphp/runtime/ext/extension.h"
+#include "hphp/runtime/base/comparisons.h"
+#include "hphp/runtime/base/memory-manager.h"
+#include "hphp/runtime/base/mixed-array.h"
+#include "hphp/runtime/base/mixed-array-defs.h"
+#include "hphp/runtime/base/packed-array.h"
+#include "hphp/runtime/base/runtime-option.h"
+#include "hphp/runtime/base/thread-info.h"
 #include "hphp/runtime/base/variable-serializer.h"
 #include "hphp/runtime/base/variable-unserializer.h"
-#include "hphp/runtime/base/zend-string.h"
-#include "hphp/runtime/base/zend-qsort.h"
 #include "hphp/runtime/base/zend-printf.h"
-#include "hphp/runtime/base/array-util.h"
-#include "hphp/runtime/base/runtime-option.h"
+#include "hphp/runtime/base/zend-qsort.h"
+#include "hphp/runtime/base/zend-string.h"
+
+#include "hphp/parser/hphp.tab.hpp"
+
+#include "hphp/util/exception.h"
+
 #include <unicode/coll.h> // icu
 #include <vector>
-#include "hphp/parser/hphp.tab.hpp"
 
 namespace HPHP {
 
@@ -40,16 +49,14 @@ const StaticString array_string("Array");
 
 void Array::setEvalScalar() const {
   Array* thisPtr = const_cast<Array*>(this);
-  if (!m_px) *thisPtr = ArrayData::Create();
-  if (!m_px->isStatic()) {
-    ArrayData *ad = ArrayData::GetScalarArray(m_px);
-    *thisPtr = ad;
+  if (!m_arr) thisPtr->m_arr = Ptr::attach(ArrayData::Create());
+  if (!m_arr->isStatic()) {
+    thisPtr->m_arr = ArrayData::GetScalarArray(get());
   }
 }
 
 void Array::compileTimeAssertions() {
-  static_assert(sizeof(Array) == sizeof(ArrayBase), "Fix this.");
-  static_assert(offsetof(Array, m_px) == kExpectedMPxOffset, "");
+  static_assert(sizeof(Array) == sizeof(req::ptr<ArrayData>), "Fix this.");
 }
 
 void ArrNR::compileTimeAssertions() {
@@ -60,7 +67,10 @@ void ArrNR::compileTimeAssertions() {
 // constructors
 
 Array Array::Create(const Variant& name, const Variant& var) {
-  return Array(ArrayData::Create(name.isString() ? name.toKey() : name, var));
+  return Array{
+    ArrayData::Create(name.isString() ? name.toKey() : name, var),
+    NoIncRef{}
+  };
 }
 
 Array::~Array() {}
@@ -68,24 +78,14 @@ Array::~Array() {}
 ///////////////////////////////////////////////////////////////////////////////
 // operators
 
-Array &Array::operator=(ArrayData *data) {
-  ArrayBase::operator=(data);
-  return *this;
-}
-
-Array &Array::operator=(const Array& arr) {
-  ArrayBase::operator=(arr.m_px);
-  return *this;
-}
-
 Array &Array::operator=(const Variant& var) {
   return operator=(var.toArray());
 }
 
 // Move assign
 Array& Array::operator=(Variant&& v) {
-  if (v.asTypedValue()->m_type == KindOfArray) {
-    m_px = v.asTypedValue()->m_data.parr;
+  if (isArrayType(v.asTypedValue()->m_type)) {
+    m_arr = req::ptr<ArrayData>::attach(v.asTypedValue()->m_data.parr);
     v.asTypedValue()->m_type = KindOfNull;
   } else {
     *this = const_cast<const Variant&>(v);
@@ -94,11 +94,11 @@ Array& Array::operator=(Variant&& v) {
 }
 
 Array Array::operator+(ArrayData *data) const {
-  return Array(m_px).plusImpl(data);
+  return Array(*this).plusImpl(data);
 }
 
 Array Array::operator+(const Array& arr) const {
-  return Array(m_px).plusImpl(arr.m_px);
+  return Array(*this).plusImpl(arr.get());
 }
 
 Array &Array::operator+=(ArrayData *data) {
@@ -112,14 +112,14 @@ static void throw_bad_array_merge() {
 }
 
 Array &Array::operator+=(const Variant& var) {
-  if (var.getType() != KindOfArray) {
+  if (!var.isArray()) {
     throw_bad_array_merge();
   }
   return operator+=(var.getArrayData());
 }
 
 Array &Array::operator+=(const Array& arr) {
-  return plusImpl(arr.m_px);
+  return plusImpl(arr.get());
 }
 
 Array Array::diff(const Variant& array, bool by_key, bool by_value,
@@ -282,26 +282,26 @@ Array Array::diffImpl(const Array& array, bool by_key, bool by_value, bool match
 // manipulations
 
 String Array::toString() const {
-  if (m_px == nullptr) return empty_string();
+  if (m_arr == nullptr) return empty_string();
   raise_notice("Array to string conversion");
   return array_string;
 }
 
 Array &Array::merge(const Array& arr) {
-  return mergeImpl(arr.m_px);
+  return mergeImpl(arr.get());
 }
 
 Array &Array::plusImpl(ArrayData *data) {
-  if (m_px == nullptr || data == nullptr) {
+  if (m_arr == nullptr || data == nullptr) {
     throw_bad_array_merge();
   }
   if (!data->empty()) {
-    if (m_px->empty()) {
-      ArrayBase::operator=(data);
-    } else if (m_px != data) {
-      auto const escalated = m_px->plusEq(data);
-      if (escalated != m_px) {
-        ArrayBase::operator=(Array::attach(escalated));
+    if (m_arr->empty()) {
+      m_arr = data;
+    } else if (m_arr != data) {
+      auto const escalated = m_arr->plusEq(data);
+      if (escalated != m_arr) {
+        m_arr = Ptr::attach(escalated);
       }
     }
   }
@@ -309,13 +309,14 @@ Array &Array::plusImpl(ArrayData *data) {
 }
 
 Array &Array::mergeImpl(ArrayData *data) {
-  if (m_px == nullptr || data == nullptr) {
+  if (m_arr == nullptr || data == nullptr) {
     throw_bad_array_merge();
   }
   if (!data->empty()) {
-    ArrayBase::operator=(Array::attach(m_px->merge(data)));
+    auto const escalated = m_arr->merge(data);
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   } else {
-    m_px->renumber();
+    m_arr->renumber();
   }
   return *this;
 }
@@ -324,9 +325,9 @@ Array &Array::mergeImpl(ArrayData *data) {
 // comparisons
 
 bool Array::same(const Array& v2) const {
-  if (m_px == nullptr && v2.get() == nullptr) return true;
-  if (m_px && v2.get()) {
-    return m_px->equal(v2.get(), true);
+  if (m_arr == nullptr && v2.get() == nullptr) return true;
+  if (m_arr && v2.get()) {
+    return m_arr->equal(v2.get(), true);
   }
   return false;
 }
@@ -336,31 +337,31 @@ bool Array::same(const Object& v2) const {
 }
 
 bool Array::equal(const Array& v2) const {
-  if (m_px == nullptr || v2.get() == nullptr) {
+  if (m_arr == nullptr || v2.get() == nullptr) {
     return HPHP::equal(toBoolean(), v2.toBoolean());
   }
-  return m_px->equal(v2.get(), false);
+  return m_arr->equal(v2.get(), false);
 }
 
 bool Array::equal(const Object& v2) const {
-  if (m_px == nullptr || v2.get() == nullptr) {
+  if (m_arr == nullptr || v2.get() == nullptr) {
     return HPHP::equal(toBoolean(), v2.toBoolean());
   }
   return false;
 }
 
 bool Array::less(const Array& v2, bool flip /* = false */) const {
-  if (m_px == nullptr || v2.get() == nullptr) {
+  if (m_arr == nullptr || v2.get() == nullptr) {
     return HPHP::less(toBoolean(), v2.toBoolean());
   }
   if (flip) {
-    return v2.get()->compare(m_px) > 0;
+    return v2.get()->compare(get()) > 0;
   }
-  return m_px->compare(v2.get()) < 0;
+  return m_arr->compare(v2.get()) < 0;
 }
 
 bool Array::less(const Object& v2) const {
-  if (m_px == nullptr || v2.get() == nullptr) {
+  if (m_arr == nullptr || v2.get() == nullptr) {
     return HPHP::less(toBoolean(), v2.toBoolean());
   }
   check_collection_compare(v2.get());
@@ -368,27 +369,27 @@ bool Array::less(const Object& v2) const {
 }
 
 bool Array::less(const Variant& v2) const {
-  if (m_px == nullptr || v2.isNull()) {
+  if (m_arr == nullptr || v2.isNull()) {
     return HPHP::less(toBoolean(), v2.toBoolean());
   }
-  if (v2.getType() == KindOfArray) {
-    return m_px->compare(v2.toArray().get()) < 0;
+  if (isArrayType(v2.getType())) {
+    return m_arr->compare(v2.toArray().get()) < 0;
   }
   return HPHP::more(v2, *this);
 }
 
 bool Array::more(const Array& v2, bool flip /* = true */) const {
-  if (m_px == nullptr || v2.get() == nullptr) {
+  if (m_arr == nullptr || v2.get() == nullptr) {
     return HPHP::more(toBoolean(), v2.toBoolean());
   }
   if (flip) {
-    return v2.get()->compare(m_px) < 0;
+    return v2.get()->compare(get()) < 0;
   }
-  return m_px->compare(v2.get()) > 0;
+  return m_arr->compare(v2.get()) > 0;
 }
 
 bool Array::more(const Object& v2) const {
-  if (m_px == nullptr || v2.get() == nullptr) {
+  if (m_arr == nullptr || v2.get() == nullptr) {
     return HPHP::more(toBoolean(), v2.toBoolean());
   }
   check_collection_compare(v2.get());
@@ -396,59 +397,71 @@ bool Array::more(const Object& v2) const {
 }
 
 bool Array::more(const Variant& v2) const {
-  if (m_px == nullptr || v2.isNull()) {
+  if (m_arr == nullptr || v2.isNull()) {
     return HPHP::more(toBoolean(), v2.toBoolean());
   }
-  if (v2.getType() == KindOfArray) {
-    return v2.toArray().get()->compare(m_px) < 0;
+  if (isArrayType(v2.getType())) {
+    return v2.toArray().get()->compare(get()) < 0;
   }
   return HPHP::less(v2, *this);
 }
 
+int Array::compare(const Array& v2, bool flip /* = false */) const {
+  if (m_arr == nullptr || v2.get() == nullptr) {
+    return HPHP::compare(toBoolean(), v2.toBoolean());
+  }
+  if (flip) return -v2.get()->compare(get());
+  return m_arr->compare(v2.get());
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // iterator
 
 ArrayIter Array::begin(const String& context /* = null_string */) const {
-  return ArrayIter(m_px);
+  return ArrayIter(*this);
 }
 
 void Array::escalate() {
-  if (m_px) ArrayBase::operator=(m_px->escalate());
+  if (m_arr) {
+    auto escalated = m_arr->escalate();
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
+  }
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 // offset functions
+#define ACCESSPARAMS_IMPL AccessFlags::Type flags
 
 Variant Array::rvalAt(int key, ACCESSPARAMS_IMPL) const {
-  if (m_px) return m_px->get((int64_t)key, flags & AccessFlags::Error);
+  if (m_arr) return m_arr->get((int64_t)key, flags & AccessFlags::Error);
   return init_null();
 }
 
 const Variant& Array::rvalAtRef(int key, ACCESSPARAMS_IMPL) const {
-  if (m_px) return m_px->get((int64_t)key, flags & AccessFlags::Error);
+  if (m_arr) return m_arr->get((int64_t)key, flags & AccessFlags::Error);
   return null_variant;
 }
 
 Variant Array::rvalAt(int64_t key, ACCESSPARAMS_IMPL) const {
-  if (m_px) return m_px->get(key, flags & AccessFlags::Error);
+  if (m_arr) return m_arr->get(key, flags & AccessFlags::Error);
   return init_null();
 }
 
 const Variant& Array::rvalAtRef(int64_t key, ACCESSPARAMS_IMPL) const {
-  if (m_px) return m_px->get(key, flags & AccessFlags::Error);
+  if (m_arr) return m_arr->get(key, flags & AccessFlags::Error);
   return null_variant;
 }
 
 const Variant& Array::rvalAtRef(const String& key, ACCESSPARAMS_IMPL) const {
-  if (m_px) {
+  if (m_arr) {
     bool error = flags & AccessFlags::Error;
-    if (flags & AccessFlags::Key) return m_px->get(key, error);
-    if (key.isNull()) return m_px->get(staticEmptyString(), error);
+    if (flags & AccessFlags::Key) return m_arr->get(key, error);
+    if (key.isNull()) return m_arr->get(staticEmptyString(), error);
     int64_t n;
     if (!key.get()->isStrictlyInteger(n)) {
-      return m_px->get(key, error);
+      return m_arr->get(key, error);
     } else {
-      return m_px->get(n, error);
+      return m_arr->get(n, error);
     }
   }
   return null_variant;
@@ -459,43 +472,48 @@ Variant Array::rvalAt(const String& key, ACCESSPARAMS_IMPL) const {
 }
 
 const Variant& Array::rvalAtRef(const Variant& key, ACCESSPARAMS_IMPL) const {
-  if (!m_px) return null_variant;
+  if (!m_arr) return null_variant;
   switch (key.getRawType()) {
-  case KindOfUninit:
-  case KindOfNull:
-    return m_px->get(staticEmptyString(), flags & AccessFlags::Error);
-  case KindOfBoolean:
-  case KindOfInt64:
-    return m_px->get(key.asTypedValue()->m_data.num,
-      flags & AccessFlags::Error);
-  case KindOfDouble:
-    return m_px->get((int64_t)key.asTypedValue()->m_data.dbl,
-      flags & AccessFlags::Error);
-  case KindOfStaticString:
-  case KindOfString:
-    {
-      int64_t n;
-      if (!(flags & AccessFlags::Key) &&
-          key.asTypedValue()->m_data.pstr->isStrictlyInteger(n)) {
-        return m_px->get(n, flags & AccessFlags::Error);
+    case KindOfUninit:
+    case KindOfNull:
+      return m_arr->get(staticEmptyString(), flags & AccessFlags::Error);
+
+    case KindOfBoolean:
+    case KindOfInt64:
+      return m_arr->get(key.asTypedValue()->m_data.num,
+                       flags & AccessFlags::Error);
+
+    case KindOfDouble:
+      return m_arr->get((int64_t)key.asTypedValue()->m_data.dbl,
+                       flags & AccessFlags::Error);
+
+    case KindOfStaticString:
+    case KindOfString:
+      {
+        int64_t n;
+        if (!(flags & AccessFlags::Key) &&
+            key.asTypedValue()->m_data.pstr->isStrictlyInteger(n)) {
+          return m_arr->get(n, flags & AccessFlags::Error);
+        }
       }
-    }
-    return m_px->get(key.asCStrRef(), flags & AccessFlags::Error);
-  case KindOfArray:
-    throw_bad_type_exception("Invalid type used as key");
-    break;
-  case KindOfObject:
-    throw_bad_type_exception("Invalid type used as key");
-    break;
-  case KindOfResource:
-    return m_px->get(key.toInt64(), flags & AccessFlags::Error);
-  case KindOfRef:
-    return rvalAtRef(*(key.asTypedValue()->m_data.pref->var()), flags);
-  default:
-    assert(false);
-    break;
+      return m_arr->get(key.asCStrRef(), flags & AccessFlags::Error);
+
+    case KindOfPersistentArray:
+    case KindOfArray:
+    case KindOfObject:
+      throw_bad_type_exception("Invalid type used as key");
+      return null_variant;
+
+    case KindOfResource:
+      return m_arr->get(key.toInt64(), flags & AccessFlags::Error);
+
+    case KindOfRef:
+      return rvalAtRef(*(key.asTypedValue()->m_data.pref->var()), flags);
+
+    case KindOfClass:
+      break;
   }
-  return null_variant;
+  not_reached();
 }
 
 Variant Array::rvalAt(const Variant& key, ACCESSPARAMS_IMPL) const {
@@ -503,11 +521,21 @@ Variant Array::rvalAt(const Variant& key, ACCESSPARAMS_IMPL) const {
 }
 
 Variant &Array::lvalAt() {
-  if (!m_px) ArrayBase::operator=(ArrayData::Create());
+  if (!m_arr) m_arr = Ptr::attach(ArrayData::Create());
   Variant *ret = nullptr;
-  ArrayData *arr = m_px;
-  ArrayData *escalated = arr->lvalNew(ret, arr->hasMultipleRefs());
-  if (escalated != arr) ArrayBase::operator=(escalated);
+  auto arr = m_arr;
+  ArrayData *escalated = arr->lvalNew(ret, arr->cowCheck());
+  if (escalated != arr) m_arr = Ptr::attach(escalated);
+  assert(ret);
+  return *ret;
+}
+
+Variant &Array::lvalAtRef() {
+  if (!m_arr) m_arr = Ptr::attach(ArrayData::Create());
+  Variant *ret = nullptr;
+  auto arr = m_arr;
+  ArrayData *escalated = arr->lvalNewRef(ret, arr->cowCheck());
+  if (escalated != arr) m_arr = Ptr::attach(escalated);
   assert(ret);
   return *ret;
 }
@@ -529,37 +557,34 @@ Variant &Array::lvalAt(const Variant& key, ACCESSPARAMS_IMPL) {
 template<typename T>
 ALWAYS_INLINE
 void Array::setImpl(const T &key, const Variant& v) {
-  if (!m_px) {
-    ArrayData *data = ArrayData::Create(key, v);
-    ArrayBase::operator=(data);
+  if (!m_arr) {
+    m_arr = Ptr::attach(ArrayData::Create(key, v));
   } else {
-    ArrayData *escalated = m_px->set(key, v, (m_px->hasMultipleRefs()));
-    if (escalated != m_px) ArrayBase::operator=(escalated);
+    ArrayData *escalated = m_arr->set(key, v, m_arr->cowCheck());
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   }
 }
 
 template<typename T>
 ALWAYS_INLINE
 void Array::setRefImpl(const T &key, Variant& v) {
-  if (!m_px) {
-    ArrayData *data = ArrayData::CreateRef(key, v);
-    ArrayBase::operator=(data);
+  if (!m_arr) {
+    m_arr = Ptr::attach(ArrayData::CreateRef(key, v));
   } else {
     escalate();
-    ArrayData *escalated = m_px->setRef(key, v, (m_px->hasMultipleRefs()));
-    if (escalated != m_px) ArrayBase::operator=(escalated);
+    ArrayData *escalated = m_arr->setRef(key, v, m_arr->cowCheck());
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   }
 }
 
 template<typename T>
 ALWAYS_INLINE
 void Array::addImpl(const T &key, const Variant& v) {
-  if (!m_px) {
-    ArrayData *data = ArrayData::Create(key, v);
-    ArrayBase::operator=(data);
+  if (!m_arr) {
+    m_arr = Ptr::attach(ArrayData::Create(key, v));
   } else {
-    ArrayData *escalated = m_px->add(key, v, (m_px->hasMultipleRefs()));
-    if (escalated != m_px) ArrayBase::operator=(escalated);
+    ArrayData *escalated = m_arr->add(key, v, m_arr->cowCheck());
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   }
 }
 
@@ -628,12 +653,9 @@ bool Array::exists(const String& key, bool isKey /* = false */) const {
 }
 
 bool Array::exists(const Variant& key, bool isKey /* = false */) const {
-  switch(key.getType()) {
-  case KindOfBoolean:
-  case KindOfInt64:
+  if (isBoolType(key.getType()) ||
+      isIntType(key.getType())) {
     return existsImpl(key.toInt64());
-  default:
-    break;
   }
   if (isKey) return existsImpl(key);
   VarNR k(key.toKey());
@@ -652,13 +674,10 @@ void Array::remove(const String& key, bool isString /* = false */) {
 }
 
 void Array::remove(const Variant& key) {
-  switch(key.getType()) {
-  case KindOfBoolean:
-  case KindOfInt64:
+  if (isBoolType(key.getType()) ||
+      isIntType(key.getType())) {
     removeImpl(key.toInt64());
     return;
-  default:
-    break;
   }
   VarNR k(key.toKey());
   if (!k.isNull()) {
@@ -667,105 +686,57 @@ void Array::remove(const Variant& key) {
 }
 
 const Variant& Array::append(const Variant& v) {
-  if (!m_px) {
-    ArrayBase::operator=(ArrayData::Create(v));
+  if (!m_arr) {
+    m_arr = Ptr::attach(ArrayData::Create(v));
   } else {
-    ArrayData *escalated = m_px->append(v, (m_px->hasMultipleRefs()));
-    if (escalated != m_px) ArrayBase::operator=(escalated);
+    ArrayData *escalated = m_arr->append(v, m_arr->cowCheck());
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   }
   return v;
 }
 
 const Variant& Array::appendRef(Variant& v) {
-  if (!m_px) {
-    ArrayBase::operator=(ArrayData::CreateRef(v));
+  if (!m_arr) {
+    m_arr = Ptr::attach(ArrayData::CreateRef(v));
   } else {
-    ArrayData *escalated = m_px->appendRef(v, (m_px->hasMultipleRefs()));
-    if (escalated != m_px) ArrayBase::operator=(escalated);
+    ArrayData *escalated = m_arr->appendRef(v, m_arr->cowCheck());
+    if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   }
   return v;
 }
 
 const Variant& Array::appendWithRef(const Variant& v) {
-  if (!m_px) ArrayBase::operator=(ArrayData::Create());
-  ArrayData *escalated = m_px->appendWithRef(v, (m_px->hasMultipleRefs()));
-  if (escalated != m_px) ArrayBase::operator=(escalated);
+  if (!m_arr) m_arr = Ptr::attach(ArrayData::Create());
+  ArrayData *escalated = m_arr->appendWithRef(v, m_arr->cowCheck());
+  if (escalated != m_arr) m_arr = Ptr::attach(escalated);
   return v;
 }
 
 Variant Array::pop() {
-  if (m_px) {
+  if (m_arr) {
     Variant ret;
-    ArrayData *newarr = m_px->pop(ret);
-    if (newarr != m_px) ArrayBase::operator=(newarr);
+    ArrayData *newarr = m_arr->pop(ret);
+    if (newarr != m_arr) m_arr = Ptr::attach(newarr);
     return ret;
   }
   return init_null();
 }
 
 Variant Array::dequeue() {
-  if (m_px) {
+  if (m_arr) {
     Variant ret;
-    ArrayData *newarr = m_px->dequeue(ret);
-    if (newarr != m_px) ArrayBase::operator=(newarr);
+    ArrayData *newarr = m_arr->dequeue(ret);
+    if (newarr != m_arr) m_arr = Ptr::attach(newarr);
     return ret;
   }
   return init_null();
 }
 
 void Array::prepend(const Variant& v) {
-  if (!m_px) operator=(Create());
-  assert(m_px);
-  ArrayData *newarr = m_px->prepend(v, (m_px->hasMultipleRefs()));
-  if (newarr != m_px) ArrayBase::operator=(newarr);
-}
-
-///////////////////////////////////////////////////////////////////////////////
-// output functions
-
-void Array::serialize(VariableSerializer *serializer,
-                      bool isObject /* = false */) const {
-  if (m_px) {
-    m_px->serialize(serializer, isObject);
-  } else {
-    serializer->writeNull();
-  }
-}
-
-void Array::unserialize(VariableUnserializer *uns) {
-  int64_t size = uns->readInt();
-  char sep = uns->readChar();
-  if (sep != ':') {
-    throw Exception("Expected ':' but got '%c'", sep);
-  }
-  sep = uns->readChar();
-  if (sep != '{') {
-    throw Exception("Expected '{' but got '%c'", sep);
-  }
-
-  if (size == 0) {
-    operator=(Create());
-  } else {
-    // Pre-allocate an ArrayData of the given size, to avoid escalation in
-    // the middle, which breaks references.
-    operator=(ArrayInit(size, ArrayInit::Mixed{}).toArray());
-    for (int64_t i = 0; i < size; i++) {
-      Variant key(uns->unserializeKey());
-      if (!key.isString() && !key.isInteger()) {
-        throw Exception("Invalid key");
-      }
-      // for apc, we know the key can't exist, but ignore that optimization
-      assert(uns->getType() != VariableUnserializer::Type::APCSerialize ||
-             !exists(key, true));
-      Variant &value = lvalAt(key, AccessFlags::Key);
-      value.unserialize(uns);
-    }
-  }
-
-  sep = uns->readChar();
-  if (sep != '}') {
-    throw Exception("Expected '}' but got '%c'", sep);
-  }
+  if (!m_arr) operator=(Create());
+  assert(m_arr);
+  ArrayData *newarr = m_arr->prepend(v, m_arr->cowCheck());
+  if (newarr != m_arr) m_arr = Ptr::attach(newarr);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -830,7 +801,8 @@ void Array::SortImpl(std::vector<int> &indices, const Array& source,
   opaque.cmp_func = cmp_func;
   opaque.data = data;
   opaque.positions.reserve(count);
-  for (ssize_t pos = source->iter_begin(); pos != ArrayData::invalid_index;
+  auto pos_limit = source->iter_end();
+  for (ssize_t pos = source->iter_begin(); pos != pos_limit;
        pos = source->iter_advance(pos)) {
     opaque.positions.push_back(pos);
   }
@@ -847,9 +819,9 @@ void Array::sort(PFUNC_CMP cmp_func, bool by_key, bool renumber,
   for (int i = 0; i < count; i++) {
     ssize_t pos = opaque.positions[indices[i]];
     if (renumber) {
-      sorted.appendWithRef(m_px->getValueRef(pos));
+      sorted.appendWithRef(m_arr->getValueRef(pos));
     } else {
-      sorted.setWithRef(m_px->getKey(pos), m_px->getValueRef(pos), true);
+      sorted.setWithRef(m_arr->getKey(pos), m_arr->getValueRef(pos), true);
     }
   }
   operator=(sorted);
@@ -875,7 +847,8 @@ bool Array::MultiSort(std::vector<SortData> &data, bool renumber) {
     opaque.positions.reserve(size);
     const Array& arr = *opaque.array;
     if (!arr.empty()) {
-      for (ssize_t pos = arr->iter_begin(); pos != ArrayData::invalid_index;
+      auto pos_limit = arr->iter_end();
+      for (ssize_t pos = arr->iter_begin(); pos != pos_limit;
            pos = arr->iter_advance(pos)) {
         opaque.positions.push_back(pos);
       }
@@ -895,7 +868,6 @@ bool Array::MultiSort(std::vector<SortData> &data, bool renumber) {
   for (unsigned int k = 0; k < data.size(); k++) {
     SortData &opaque = data[k];
     const Array& arr = *opaque.array;
-
     Array sorted;
     for (int i = 0; i < count; i++) {
       ssize_t pos = opaque.positions[indices[i]];
@@ -906,32 +878,38 @@ bool Array::MultiSort(std::vector<SortData> &data, bool renumber) {
         sorted.set(k, arr->getValueRef(pos));
       }
     }
-    *opaque.original = sorted;
+    if (opaque.original->getRawType() == KindOfRef) {
+      *opaque.original->getRefData() = sorted;
+    }
   }
 
   free(indices);
   return true;
 }
 
-int Array::SortRegularAscending(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortRegularAscending(const Variant& v1, const Variant& v2,
+                                const void *data) {
   if (HPHP::less(v1, v2)) return -1;
   if (tvEqual(*v1.asTypedValue(), *v2.asTypedValue())) return 0;
   return 1;
 }
-int Array::SortRegularDescending(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortRegularDescending(const Variant& v1, const Variant& v2,
+                                 const void *data) {
   if (HPHP::less(v1, v2)) return 1;
   if (tvEqual(*v1.asTypedValue(), *v2.asTypedValue())) return 0;
   return -1;
 }
 
-int Array::SortNumericAscending(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortNumericAscending(const Variant& v1, const Variant& v2,
+                                const void *data) {
   double d1 = v1.toDouble();
   double d2 = v2.toDouble();
   if (d1 < d2) return -1;
   if (d1 == d2) return 0;
   return 1;
 }
-int Array::SortNumericDescending(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortNumericDescending(const Variant& v1, const Variant& v2,
+                                 const void *data) {
   double d1 = v1.toDouble();
   double d2 = v2.toDouble();
   if (d1 < d2) return 1;
@@ -939,25 +917,29 @@ int Array::SortNumericDescending(const Variant& v1, const Variant& v2, const voi
   return -1;
 }
 
-int Array::SortStringAscending(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortStringAscending(const Variant& v1, const Variant& v2,
+                               const void *data) {
   String s1 = v1.toString();
   String s2 = v2.toString();
   return string_strcmp(s1.data(), s1.size(), s2.data(), s2.size());
 }
 
-int Array::SortStringAscendingCase(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortStringAscendingCase(const Variant& v1, const Variant& v2,
+                                   const void *data) {
   String s1 = v1.toString();
   String s2 = v2.toString();
   return bstrcasecmp(s1.data(), s1.size(), s2.data(), s2.size());
 }
 
-int Array::SortStringDescending(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortStringDescending(const Variant& v1, const Variant& v2,
+                                const void *data) {
   String s1 = v1.toString();
   String s2 = v2.toString();
   return string_strcmp(s2.data(), s2.size(), s1.data(), s1.size());
 }
 
-int Array::SortStringDescendingCase(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortStringDescendingCase(const Variant& v1, const Variant& v2,
+                                    const void *data) {
   String s1 = v1.toString();
   String s2 = v2.toString();
   return bstrcasecmp(s2.data(), s2.size(), s1.data(), s1.size());
@@ -979,17 +961,33 @@ int Array::SortLocaleStringDescending(const Variant& v1, const Variant& v2,
   return strcoll(s2.data(), s1.data());
 }
 
-int Array::SortNatural(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortNaturalAscending(const Variant& v1, const Variant& v2,
+                                const void *data) {
   String s1 = v1.toString();
   String s2 = v2.toString();
   return string_natural_cmp(s1.data(), s1.size(), s2.data(), s2.size(), 0);
 }
 
-int Array::SortNaturalCase(const Variant& v1, const Variant& v2, const void *data) {
+int Array::SortNaturalDescending(const Variant& v1, const Variant& v2,
+                                 const void *data) {
+  String s1 = v1.toString();
+  String s2 = v2.toString();
+  return string_natural_cmp(s2.data(), s2.size(), s1.data(), s1.size(), 0);
+}
+
+int Array::SortNaturalCaseAscending(const Variant& v1, const Variant& v2,
+                                    const void *data) {
   String s1 = v1.toString();
   String s2 = v2.toString();
   return string_natural_cmp(s1.data(), s1.size(), s2.data(), s2.size(), 1);
 }
 
+int Array::SortNaturalCaseDescending(const Variant& v1, const Variant& v2,
+                                     const void *data) {
+  String s1 = v1.toString();
+  String s2 = v2.toString();
+  return string_natural_cmp(s2.data(), s2.size(), s1.data(), s1.size(), 1);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 }
