@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,12 +17,13 @@
 #ifndef incl_HPHP_STRING_H_
 #define incl_HPHP_STRING_H_
 
-#include "hphp/runtime/base/smart-ptr.h"
-#include "hphp/runtime/base/string-data.h"
-#include "hphp/runtime/base/types.h"
-#include "hphp/runtime/base/typed-value.h"
+#include "hphp/runtime/base/req-ptr.h"
 #include "hphp/runtime/base/static-string-table.h"
+#include "hphp/runtime/base/string-data.h"
+#include "hphp/runtime/base/typed-value.h"
 #include "hphp/util/assertions.h"
+#include "hphp/util/hash-map-typedefs.h"
+#include "hphp/util/functional.h"
 
 #include <algorithm>
 
@@ -31,8 +32,9 @@ namespace HPHP {
 //////////////////////////////////////////////////////////////////////
 
 class Array;
-class String;
 class VarNR;
+class VariableSerializer;
+class VariableUnserializer;
 
 // reserve space for buffer that will be filled in by client.
 enum ReserveStringMode { ReserveString };
@@ -51,8 +53,9 @@ constexpr int kMinShrinkThreshold = 1024;
 
 //////////////////////////////////////////////////////////////////////
 
+// Built strings will have their reference counts pre-initialized to 1.
 StringData* buildStringData(int     n);
-StringData* buildStringData(int64_t   n);
+StringData* buildStringData(int64_t n);
 StringData* buildStringData(double  n);
 
 std::string convDblToStrWithPhpFormat(double n);
@@ -63,11 +66,12 @@ std::string convDblToStrWithPhpFormat(double n);
  * String type wrapping around StringData to implement copy-on-write and
  * literal string handling (to avoid string copying).
  */
-class String : protected SmartPtr<StringData> {
-  typedef SmartPtr<StringData> StringBase;
+class String {
+  req::ptr<StringData> m_str;
 
-  explicit String(StringData* sd, StringBase::NoIncRef) :
-    StringBase(sd, StringBase::NoIncRef{}) { }
+protected:
+  using NoIncRef = req::ptr<StringData>::NoIncRef;
+  String(StringData* sd, NoIncRef) : m_str(sd, NoIncRef{}) {}
 
 public:
   typedef hphp_hash_map<int64_t, const StringData *, int64_hash>
@@ -88,10 +92,10 @@ public:
 
   // create a string from a character
   static String FromChar(char ch) {
-    return makeStaticString(ch);
+    return String{makeStaticString(ch)};
   }
   static String FromCStr(const char* str) {
-    return makeStaticString(str);
+    return String{makeStaticString(str)};
   }
 
   static const StringData *ConvertInteger(int64_t n);
@@ -113,30 +117,24 @@ public:
   String() {}
   ~String();
 
-  StringData* get() const { return m_px; }
-  void reset() { StringBase::reset(); }
+  StringData* get() const { return m_str.get(); }
+  void reset(StringData* str = nullptr) { m_str.reset(str); }
 
   // Transfer ownership of our reference to this StringData.
-  StringData* detach() {
-    StringData* ret = m_px;
-    m_px = 0;
-    return ret;
-  }
+  StringData* detach() { return m_str.detach(); }
 
   /**
    * Constructors
    */
-  /* implicit */ String(StringData *data) : StringBase(data) { }
+  explicit String(StringData *data) : m_str(data) { }
   /* implicit */ String(int     n);
   /* implicit */ String(int64_t n);
   /* implicit */ String(double  n);
-  /* implicit */ String(litstr  s) {
-    if (s) {
-      m_px = StringData::Make(s, CopyString);
-      m_px->setRefCount(1);
-    }
-  }
-  String(const String& str) : StringBase(str.m_px) { }
+  /* implicit */ String(const char* s)
+  : m_str(LIKELY((bool)s) ? StringData::Make(s, CopyString)
+                           : nullptr, NoIncRef{}) { }
+
+  String(const String& str) : m_str(str.m_str) { }
   /* implicit */ String(const StaticString& str);
   /* implicit */ String(char) = delete; // prevent unintentional promotion
 
@@ -145,72 +143,53 @@ public:
   /* implicit */ String(const StringData*) = delete;
 
   // Move ctor
-  /* implicit */ String(String&& str) noexcept : StringBase(std::move(str)) {}
+  /* implicit */ String(String&& str) noexcept : m_str(std::move(str.m_str)) {}
   /* implicit */ String(Variant&& src);
   // Move assign
   String& operator=(String&& src) {
-    static_assert(sizeof(String) == sizeof(StringBase),"Fix this.");
-    StringBase::operator=(std::move(src));
+    m_str = std::move(src.m_str);
+    return *this;
+  }
+  String& operator=(req::ptr<StringData>&& src) {
+    m_str = std::move(src);
     return *this;
   }
 
   // Move assign from Variant
   String& operator=(Variant&& src);
 
-  /* implicit */ String(const std::string &s) { // always make a copy
-    m_px = StringData::Make(s.data(), s.size(), CopyString);
-    m_px->setRefCount(1);
-  }
+  /* implicit */ String(const std::string &s)
+  : m_str(StringData::Make(s.data(), s.size(), CopyString), NoIncRef{}) { }
+
   // attach to null terminated malloc'ed string, maybe free it now.
-  String(char* s, AttachStringMode mode) {
-    if (s) {
-      m_px = StringData::Make(s, mode);
-      m_px->setRefCount(1);
-    }
-  }
+  String(char* s, AttachStringMode mode)
+  : m_str(LIKELY((bool)s) ? StringData::Make(s, mode) : nullptr, NoIncRef{}) {}
+
   // copy a null terminated string
-  String(const char *s, CopyStringMode mode) {
-    if (s) {
-      m_px = StringData::Make(s, mode);
-      m_px->setRefCount(1);
-    }
-  }
+  String(const char *s, CopyStringMode mode)
+  : m_str(LIKELY((bool)s) ? StringData::Make(s, mode) : nullptr, NoIncRef{}) {}
+
   // attach to binary malloc'ed string
-  String(char* s, size_t length, AttachStringMode mode) {
-    if (s) {
-      m_px = StringData::Make(s, length, mode);
-      m_px->setRefCount(1);
-    }
-  }
+  String(char* s, size_t length, AttachStringMode mode)
+  : m_str(LIKELY((bool)s) ? StringData::Make(s, length, mode)
+                          : nullptr, NoIncRef{}) { }
+
   // make copy of binary binary string
-  String(const char *s, size_t length, CopyStringMode mode) {
-    if (s) {
-      m_px = StringData::Make(s, length, mode);
-      m_px->setRefCount(1);
-    }
-  }
+  String(const char *s, size_t length, CopyStringMode mode)
+  : m_str(LIKELY((bool)s) ? StringData::Make(s, length, mode)
+                          : nullptr, NoIncRef{}) { }
+
   // force a copy of a String
-  String(const String& s, CopyStringMode mode) {
-    if (s.m_px) {
-      m_px = StringData::Make(s.c_str(), s.size(), mode);
-      m_px->setRefCount(1);
-    }
-  }
+  String(const String& s, CopyStringMode mode)
+  : m_str(LIKELY((bool)s) ? StringData::Make(s.c_str(), s.size(), mode)
+                          : nullptr, NoIncRef{}) {}
+
   // make an empty string with cap reserve bytes, plus 1 for '\0'
-  String(size_t cap, ReserveStringMode mode) {
-    m_px = StringData::Make(cap);
-    m_px->setRefCount(1);
-  }
+  String(size_t cap, ReserveStringMode mode)
+  : m_str(StringData::Make(cap), NoIncRef{}) { }
 
-  static String attach(const String& s) {
-    String result;
-    result.m_px = s.m_px;
-    return result;
-  }
-
-  static ALWAYS_INLINE String attach(StringData* sd) {
-    assert(sd->isStatic());
-    return String(sd, StringBase::NoIncRef{});
+  static String attach(StringData* sd) {
+    return String(sd, NoIncRef{});
   }
 
   void clear() { reset();}
@@ -218,72 +197,77 @@ public:
    * Informational
    */
   const char *data() const {
-    return m_px ? m_px->data() : "";
+    return m_str ? m_str->data() : "";
   }
+
+  char *mutableData() const {
+    return m_str->mutableData();
+  }
+
 public:
   const String& setSize(int len) {
-    assert(m_px);
-    m_px->setSize(len);
+    assert(m_str);
+    m_str->setSize(len);
     return *this;
   }
   const String& shrink(size_t len) {
-    assert(m_px);
-    if (len < m_px->size() && m_px->size() - len > kMinShrinkThreshold) {
-      StringBase::operator=(m_px->shrink(len));
+    assert(m_str);
+    if (m_str->capacity() - len > kMinShrinkThreshold) {
+      m_str = req::ptr<StringData>::attach(m_str->shrinkImpl(len));
     } else {
       assert(len < StringData::MaxSize);
-      m_px->setSize(len);
+      m_str->setSize(len);
     }
     return *this;
   }
-  MutableSlice reserve(size_t size) {
-    if (!m_px) return MutableSlice("", 0);
-    auto const tmp = m_px->reserve(size);
-    if (UNLIKELY(tmp != m_px)) StringBase::operator=(tmp);
-    return m_px->bufferSlice();
+  folly::MutableStringPiece reserve(size_t size) {
+    if (!m_str) return folly::MutableStringPiece();
+    auto const tmp = m_str->reserve(size);
+    if (UNLIKELY(tmp != m_str)) {
+      m_str = req::ptr<StringData>::attach(tmp);
+    }
+    return m_str->bufferSlice();
   }
   const char *c_str() const {
-    return m_px ? m_px->data() : "";
+    return m_str ? m_str->data() : "";
   }
   bool empty() const {
-    return m_px ? m_px->empty() : true;
+    return m_str ? m_str->empty() : true;
   }
   int size() const {
-    return m_px ? m_px->size() : 0;
+    return m_str ? m_str->size() : 0;
   }
   int length() const {
-    return m_px ? m_px->size() : 0;
+    return m_str ? m_str->size() : 0;
   }
-  StringSlice slice() const {
-    return m_px ? m_px->slice() : StringSlice("", 0);
+  uint32_t capacity() const {
+    return m_str->capacity(); // intentionally skip nullptr check
   }
-  MutableSlice bufferSlice() {
-    return m_px ? m_px->bufferSlice() : MutableSlice("", 0);
+  folly::StringPiece slice() const {
+    return m_str ? m_str->slice() : folly::StringPiece();
   }
-  bool isNull() const {
-    return m_px == nullptr;
+  folly::MutableStringPiece bufferSlice() {
+    return m_str ? m_str->bufferSlice() : folly::MutableStringPiece();
   }
+  bool isNull() const { return !m_str; }
   bool isNumeric() const {
-    return m_px ? m_px->isNumeric() : false;
+    return m_str ? m_str->isNumeric() : false;
   }
   bool isInteger() const {
-    return m_px ? m_px->isInteger() : false;
+    return m_str ? m_str->isInteger() : false;
   }
   bool isZero() const {
-    return m_px ? m_px->isZero() : false;
+    return m_str ? m_str->isZero() : false;
   }
 
-  /**
-   * Take a sub-string from start with specified length. Note, read
-   * http://www.php.net/substr about meanings of negative start or length.
+  /*
+   * Create a sub-string from start with specified length.
+   *
+   * If the start is outside the bounds of the string, or the length is
+   * negative, the empty string is returned.  The range [start, start+length]
+   * gets clamped to [start, size()].
    */
-  String substr(int start, int length = 0x7FFFFFFF,
-                bool nullable = false) const;
-
-  /**
-   * Returns the last token if string is delimited by the specified.
-   */
-  String lastToken(char delimiter);
+  String substr(int start, int length = StringData::MaxSize) const;
 
   /**
    * Find a character or a substring and return its position. "pos" has to be
@@ -300,47 +284,52 @@ public:
   /**
    * Operators
    */
-  String &operator =  (StringData *data);
-  String &operator =  (litstr  v);
-  String &operator =  (const String& v);
-  String &operator =  (const StaticString& v);
-  String &operator =  (const Variant& v);
-  String &operator =  (const std::string &s);
+  String& operator=(StringData *data) {
+    m_str = data;
+    return *this;
+  }
+  String& operator=(const String& v) {
+    m_str = v.m_str;
+    return *this;
+  }
+  String& operator=(const StaticString& v);
+  String& operator=(const char* v);
+  String& operator=(const Variant& v);
+  String& operator=(const std::string &s);
   // These should be members, but g++ doesn't yet support the rvalue
   // reference notation on lhs (http://goo.gl/LuCTo).
-  friend String&& operator+(String&& lhs, litstr rhs);
+  friend String&& operator+(String&& lhs, const char* rhs);
   friend String&& operator+(String&& lhs, String&& rhs);
   friend String operator+(String&& lhs, const String & rhs);
   friend String operator+(const String & lhs, String&& rhs);
-  friend String operator+(const String& lhs, litstr rhs);
+  friend String operator+(const String& lhs, const char* rhs);
   friend String operator+(const String & lhs, const String & rhs);
-  String &operator += (litstr  v);
-  String &operator += (const String& v);
-  String &operator += (const StringSlice& slice);
-  String &operator += (const MutableSlice& slice);
+  String& operator += (const char* v);
+  String& operator += (const String& v);
+  String& operator += (const std::string& v);
+  String& operator += (folly::StringPiece slice);
+  String& operator += (folly::MutableStringPiece slice);
   String  operator |  (const String& v) const = delete;
   String  operator &  (const String& v) const = delete;
   String  operator ^  (const String& v) const = delete;
-  String &operator |= (const String& v) = delete;
-  String &operator &= (const String& v) = delete;
-  String &operator ^= (const String& v) = delete;
+  String& operator |= (const String& v) = delete;
+  String& operator &= (const String& v) = delete;
+  String& operator ^= (const String& v) = delete;
   String  operator ~  () const = delete;
   explicit operator std::string () const { return toCppString(); }
-  explicit operator bool() const {
-    return m_px != nullptr;
-  }
+  explicit operator bool() const { return (bool)m_str; }
 
   /**
    * These are convenient functions for writing extensions, since code
    * generation always uses explicit functions like same(), less() etc. that
    * are type specialized and unambiguous.
    */
-  bool operator == (litstr  v) const = delete;
-  bool operator != (litstr  v) const = delete;
-  bool operator >= (litstr  v) const = delete;
-  bool operator <= (litstr  v) const = delete;
-  bool operator >  (litstr  v) const = delete;
-  bool operator <  (litstr  v) const = delete;
+  bool operator == (const char* v) const = delete;
+  bool operator != (const char* v) const = delete;
+  bool operator >= (const char* v) const = delete;
+  bool operator <= (const char* v) const = delete;
+  bool operator >  (const char* v) const = delete;
+  bool operator <  (const char* v) const = delete;
   bool operator == (const String& v) const;
   bool operator != (const String& v) const;
   bool operator >= (const String& v) const = delete;
@@ -357,43 +346,46 @@ public:
   /**
    * Type conversions
    */
-  bool   toBoolean() const { return m_px ? m_px->toBoolean() : false;}
-  char   toByte   () const { return m_px ? m_px->toByte   () : 0;}
-  short  toInt16  () const { return m_px ? m_px->toInt16  () : 0;}
-  int    toInt32  () const { return m_px ? m_px->toInt32  () : 0;}
-  int64_t  toInt64  () const { return m_px ? m_px->toInt64  () : 0;}
-  double toDouble () const { return m_px ? m_px->toDouble () : 0;}
+  bool   toBoolean() const { return m_str ? m_str->toBoolean() : false;}
+  char   toByte   () const { return m_str ? m_str->toByte   () : 0;}
+  short  toInt16  () const { return m_str ? m_str->toInt16  () : 0;}
+  int    toInt32  () const { return m_str ? m_str->toInt32  () : 0;}
+  int64_t toInt64 () const { return m_str ? m_str->toInt64  () : 0;}
+  double toDouble () const { return m_str ? m_str->toDouble () : 0;}
   VarNR  toKey   () const;
   std::string toCppString() const { return std::string(c_str(), size()); }
 
   /**
    * Comparisons
    */
-  bool same (litstr  v2) const = delete;
+  bool same (const char* v2) const = delete;
   bool same (const StringData *v2) const;
   bool same (const String& v2) const;
   bool same (const Array& v2) const;
   bool same (const Object& v2) const;
   bool same (const Resource& v2) const;
-  bool equal(litstr  v2) const = delete;
+  bool equal(const char* v2) const = delete;
   bool equal(const StringData *v2) const;
   bool equal(const String& v2) const;
   bool equal(const Array& v2) const;
   bool equal(const Object& v2) const;
   bool equal(const Resource& v2) const;
-  bool less (litstr  v2) const = delete;
+  bool less (const char* v2) const = delete;
   bool less (const StringData *v2) const;
   bool less (const String& v2) const;
   bool less (const Array& v2) const;
   bool less (const Object& v2) const;
   bool less (const Resource& v2) const;
-  bool more (litstr  v2) const = delete;
+  bool more (const char* v2) const = delete;
   bool more (const StringData *v2) const;
   bool more (const String& v2) const;
   bool more (const Array& v2) const;
   bool more (const Object& v2) const;
   bool more (const Resource& v2) const;
 
+  int compare(const char* v2) const;
+  int compare(const String& v2) const;
+
   /**
    * Offset
    */
@@ -401,9 +393,11 @@ public:
   String rvalAt(char    key) const { return rvalAtImpl(key);}
   String rvalAt(short   key) const { return rvalAtImpl(key);}
   String rvalAt(int     key) const { return rvalAtImpl(key);}
-  String rvalAt(int64_t   key) const { return rvalAtImpl(key);}
+  String rvalAt(int64_t key) const { return rvalAtImpl(key);}
   String rvalAt(double  key) const { return rvalAtImpl((int64_t)key);}
-  String rvalAt(litstr  key) const { return rvalAtImpl(String(key).toInt32());}
+  String rvalAt(const char* key) const {
+    return rvalAtImpl(String(key).toInt32());
+  }
   String rvalAt(const StringData *key) const {
     not_reached();
     return rvalAtImpl(key ? key->toInt32() : 0);
@@ -419,29 +413,24 @@ public:
   char charAt(int pos) const;
   char operator[](int pos) const { return charAt(pos);}
 
-  /**
-   * Input/Output
-   */
-  void serialize(VariableSerializer *serializer) const;
-  void unserialize(VariableUnserializer *uns, char delimiter0 = '"',
-                   char delimiter1 = '"');
-
   /**
    * Debugging
    */
   void dump() const;
 
  private:
+  static req::ptr<StringData> buildString(int n);
+  static req::ptr<StringData> buildString(int64_t n);
 
   String rvalAtImpl(int key) const {
-    if (m_px) {
-      return m_px->getChar(key);
+    if (m_str) {
+      return String{m_str->getChar(key)};
     }
     return String();
   }
 
   static void compileTimeAssertions() {
-    static_assert(offsetof(String, m_px) == kExpectedMPxOffset, "");
+    static_assert(sizeof(String) == sizeof(req::ptr<StringData>), "");
   }
 };
 
@@ -540,7 +529,11 @@ public:
   explicit StrNR(const StringData *sd) : m_px(const_cast<StringData*>(sd)) {}
   explicit StrNR(const String &s) : m_px(s.get()) {} // XXX
 
-  ~StrNR() {}
+  ~StrNR() {
+    if (debug) {
+      m_px = reinterpret_cast<StringData*>(0xdeadbeeffaceb004);
+    }
+  }
 
   /* implicit */ operator const String&() const { return asString(); }
   const char* data() const { return m_px ? m_px->data() : ""; }
@@ -548,6 +541,10 @@ public:
   int size() const { return m_px ? m_px->size() : 0; }
   bool empty() const { return size() == 0; }
 
+  uint32_t capacity() const {
+    return m_px->capacity(); // intentionally skip nullptr check
+  }
+
   String& asString() {
     return *reinterpret_cast<String*>(this);
   }
@@ -556,7 +553,6 @@ public:
   }
 
   StringData* get() const { return m_px; }
-  StringData* operator->() const { return get(); }
 
 private:
   static void compileTimeAssertions() {
@@ -572,30 +568,29 @@ private:
  * so that they won't be garbage collected by MemoryManager. This is used by
  * constant strings, so they can be pre-allocated before request handling.
  */
-class StaticString : public String {
-public:
-  friend class StringUtil;
-
-  explicit StaticString(litstr s);
-  StaticString(litstr s, int length); // binary string
+struct StaticString : String {
+  explicit StaticString(const char* s);
+  StaticString(const char* s, int length); // binary string
   explicit StaticString(std::string s);
   ~StaticString() {
-    // prevent ~SmartPtr from calling decRefCount after data is released
-    m_px = nullptr;
+    // prevent ~req::ptr from destroying contents.
+    detach();
   }
   StaticString& operator=(const StaticString &str);
-
-private:
-  void insert();
 };
 
-String getDataTypeString(DataType t);
+StaticString getDataTypeString(DataType t);
 
 //////////////////////////////////////////////////////////////////////
 
 inline String::String(const StaticString& str) :
-  StringBase(str.m_px, StringBase::NoIncRef{}) {
-  assert(str.m_px->isStatic());
+  m_str(str.m_str.get(), NoIncRef{}) {
+  assert(str.m_str->isStatic());
+}
+
+inline String& String::operator=(const StaticString& v) {
+  m_str = req::ptr<StringData>::attach(v.m_str.get());
+  return *this;
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -608,4 +603,18 @@ ALWAYS_INLINE String empty_string() {
 
 }
 
+namespace folly {
+template<> struct FormatValue<HPHP::String> {
+  explicit FormatValue(const HPHP::String& str) : m_val(str) {}
+
+  template<typename Callback>
+  void format(FormatArg& arg, Callback& cb) const {
+    FormatValue<HPHP::StringData*>(m_val.get()).format(arg, cb);
+  }
+
+ private:
+  const HPHP::String& m_val;
+};
+}
+
 #endif
