@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,21 +20,26 @@
 #include <fcntl.h>
 #include <memory>
 #include <string>
-#include <ctime>
 #include <cstdlib>
+#include <thread>
 
-#include "folly/ScopeGuard.h"
+#include <folly/ScopeGuard.h>
 
+#include "hphp/util/assertions.h"
 #include "hphp/util/rank.h"
 #include "hphp/util/mutex.h"
-#include "hphp/util/assertions.h"
+#include "hphp/util/process.h"
+#include "hphp/runtime/base/system-profiler.h"
 #include "hphp/runtime/base/runtime-option.h"
 #include "hphp/runtime/base/zend-string.h"
+#include "hphp/runtime/base/string-util.h"
 #include "hphp/runtime/base/file-util.h"
+#include "hphp/runtime/base/plain-file.h"
 #include "hphp/runtime/base/stat-cache.h"
 #include "hphp/runtime/base/stream-wrapper-registry.h"
 #include "hphp/runtime/base/file-stream-wrapper.h"
 #include "hphp/runtime/base/profile-dump.h"
+#include "hphp/runtime/base/program-functions.h"
 #include "hphp/runtime/base/rds.h"
 #include "hphp/runtime/server/source-root-info.h"
 #include "hphp/runtime/vm/debugger-hook.h"
@@ -44,6 +49,7 @@
 
 #ifdef __APPLE__
 #define st_mtim st_mtimespec
+#define st_ctim st_ctimespec
 #endif
 
 namespace HPHP {
@@ -102,14 +108,14 @@ CachedUnit lookupUnitRepoAuth(const StringData* path) {
    */
   MD5 md5;
   if (!Repo::get().findFile(path->data(),
-                            SourceRootInfo::GetCurrentSourceRoot(),
+                            RuntimeOption::SourceRoot,
                             md5)) {
     return acc->second;
   }
 
-  acc->second.unit = Repo::get().loadUnit(path->data(), md5);
+  acc->second.unit = Repo::get().loadUnit(path->data(), md5).release();
   if (acc->second.unit) {
-    acc->second.rdsBitId = RDS::allocBit();
+    acc->second.rdsBitId = rds::allocBit();
   }
   return acc->second;
 }
@@ -117,9 +123,28 @@ CachedUnit lookupUnitRepoAuth(const StringData* path) {
 //////////////////////////////////////////////////////////////////////
 // Non-repo mode unit caching
 
+struct CachedUnitWithFree {
+  CachedUnitWithFree() = delete;
+  explicit CachedUnitWithFree(const CachedUnitWithFree&) = delete;
+  CachedUnitWithFree& operator=(const CachedUnitWithFree&) = delete;
+
+  explicit CachedUnitWithFree(const CachedUnit& src) : cu(src) {}
+  ~CachedUnitWithFree() {
+    if (auto oldUnit = cu.unit) {
+      Treadmill::enqueue([oldUnit] { delete oldUnit; });
+    }
+  }
+  CachedUnit cu;
+};
+
 struct CachedUnitNonRepo {
-  CachedUnit cachedUnit;
+  std::shared_ptr<CachedUnitWithFree> cachedUnit;
+#ifdef _MSC_VER
+  time_t mtime;
+#else
   struct timespec mtime;
+  struct timespec ctime;
+#endif
   ino_t ino;
   dev_t devId;
 };
@@ -132,57 +157,36 @@ using NonRepoUnitCache = RankedCHM<
 >;
 NonRepoUnitCache s_nonRepoUnitCache;
 
-/*
- * When a unit is removed from the unit cache, we wait for a Treadmill round
- * before reclaiming it using this routine.
- *
- * If we have or are in the process of a collecting an hhprof dump then we need
- * to keep these units around even longer, as they might be needed for symbol
- * resolution when that dump is collected by pprof.  In this case, we just pass
- * ownership to the ProfileControler module.
- */
-void reclaimUnit(const Unit* unit) {
-  if (memory_profiling && RuntimeOption::HHProfServerEnabled &&
-      ProfileController::isTracking()) {
-    ProfileController::enqueueOrphanedUnit(unit);
-    return;
-  }
-  delete unit;
-}
-
+#ifndef _MSC_VER
 int64_t timespecCompare(const struct timespec& l,
                         const struct timespec& r) {
   if (l.tv_sec != r.tv_sec) return l.tv_sec - r.tv_sec;
   return l.tv_nsec - r.tv_nsec;
 }
+#endif
 
 bool isChanged(const CachedUnitNonRepo& cu, const struct stat& s) {
   // If the cached unit is null, we always need to consider it out of date (in
   // case someone created the file).  This case should only happen if something
   // successfully stat'd the file, but then it was gone by the time we tried to
   // open() it.
-  return cu.cachedUnit.unit == nullptr ||
+  return !cu.cachedUnit ||
+         cu.cachedUnit->cu.unit == nullptr ||
+#ifdef _MSC_VER
+         cu.mtime - s.st_mtime < 0 ||
+#else
          timespecCompare(cu.mtime, s.st_mtim) < 0 ||
+         timespecCompare(cu.ctime, s.st_ctim) < 0 ||
+#endif
          cu.ino != s.st_ino ||
          cu.devId != s.st_dev;
 }
 
-folly::Optional<String> readFileAsString(const StringData* path,
-                                         off_t fileSize) {
-  if (fileSize > StringData::MaxSize) {
-    throw FatalErrorException(0, "file %s is too big", path->data());
-  }
-
+folly::Optional<String> readFileAsString(const StringData* path) {
   auto const fd = open(path->data(), O_RDONLY);
-  if (!fd) return folly::none;
-  SCOPE_EXIT { close(fd); };
-
-  String str(fileSize, ReserveString);
-  auto const input = str.bufferSlice().ptr;
-  auto const nbytes = read(fd, input, fileSize);
-  str.setSize(fileSize);
-  if (nbytes != fileSize) return folly::none;
-  return str;
+  if (fd < 0) return folly::none;
+  auto file = req::make<PlainFile>(fd);
+  return file->read();
 }
 
 CachedUnit createUnitFromString(const char* path,
@@ -191,31 +195,31 @@ CachedUnit createUnitFromString(const char* path,
     mangleUnitMd5(string_md5(contents.data(), contents.size())).c_str()
   };
   // Try the repo; if it's not already there, invoke the compiler.
-  if (auto const unit = Repo::get().loadUnit(path, md5)) {
-    return CachedUnit { unit, RDS::allocBit() };
+  if (auto unit = Repo::get().loadUnit(path, md5)) {
+    return CachedUnit { unit.release(), rds::allocBit() };
   }
   auto const unit = compile_file(contents.data(), contents.size(), md5, path);
-  return CachedUnit { unit, RDS::allocBit() };
+  return CachedUnit { unit, rds::allocBit() };
 }
 
 CachedUnit createUnitFromUrl(const StringData* const requestedPath) {
   auto const w = Stream::getWrapperFromURI(StrNR(requestedPath));
   if (!w) return CachedUnit{};
-  auto const f = w->open(StrNR(requestedPath), "r", 0, null_variant);
+  auto const f = w->open(StrNR(requestedPath), "r", 0, nullptr);
   if (!f) return CachedUnit{};
   StringBuffer sb;
-  sb.read(f);
+  sb.read(f.get());
   return createUnitFromString(requestedPath->data(), sb.detach());
 }
 
-CachedUnit createUnitFromFile(StringData* const path, off_t fileSize) {
-  auto const contents = readFileAsString(path, fileSize);
+CachedUnit createUnitFromFile(const StringData* const path) {
+  auto const contents = readFileAsString(path);
   return contents ? createUnitFromString(path->data(), *contents)
                   : CachedUnit{};
 }
 
-CachedUnit lookupUnitNonRepoAuth(StringData* requestedPath,
-                                 const struct stat& statInfo) {
+CachedUnit loadUnitNonRepoAuth(StringData* requestedPath,
+                               const struct stat& statInfo) {
   if (strstr(requestedPath->data(), "://") != nullptr) {
     // URL-based units are not currently cached in memory, but the Repo still
     // caches them on disk.
@@ -228,54 +232,92 @@ CachedUnit lookupUnitNonRepoAuth(StringData* requestedPath,
     makeStaticString(
       // XXX: it seems weird we have to do this even though we already ran
       // resolveVmInclude.
-      (requestedPath->data()[0] == '/'
-        ? requestedPath
+      (FileUtil::isAbsolutePath(requestedPath->toCppString())
+       ?  String{requestedPath}
         : String(SourceRootInfo::GetCurrentSourceRoot()) + StrNR(requestedPath)
       ).get()
     );
 
-  NonRepoUnitCache::accessor acc;
-  if (!s_nonRepoUnitCache.insert(acc, path)) {
-    if (!isChanged(acc->second, statInfo)) {
-      return acc->second.cachedUnit;
+  auto const rpath = [&] () -> const StringData* {
+    if (RuntimeOption::CheckSymLink) {
+      std::string rp = StatCache::realpath(path->data());
+      if (rp.size() != 0) {
+        if (rp.size() != path->size() ||
+            memcmp(rp.data(), path->data(), rp.size())) {
+          return makeStaticString(rp);
+        }
+      }
+    }
+    return path;
+  }();
+
+  auto const cuptr = [&] () -> std::shared_ptr<CachedUnitWithFree> {
+    NonRepoUnitCache::accessor rpathAcc;
+
+    if (!s_nonRepoUnitCache.insert(rpathAcc, rpath)) {
+      if (!isChanged(rpathAcc->second, statInfo)) {
+        return rpathAcc->second.cachedUnit;
+      }
+    }
+
+    /*
+     * NB: the new-unit creation path is here, and is done while holding the tbb
+     * lock on s_nonRepoUnitCache.  This was originally done deliberately to
+     * avoid wasting time in the compiler (during server startup, many requests
+     * hit the same code initial paths that are shared, and would all be
+     * compiling the same files).  It's not 100% clear if this is the best way
+     * to handle that idea, though (tbb locks spin aggressively and are
+     * expected to be low contention).
+     */
+
+    auto const cu = createUnitFromFile(rpath);
+    rpathAcc->second.cachedUnit = std::make_shared<CachedUnitWithFree>(cu);
+#ifdef _MSC_VER
+    rpathAcc->second.mtime      = statInfo.st_mtime;
+#else
+    rpathAcc->second.mtime      = statInfo.st_mtim;
+    rpathAcc->second.ctime      = statInfo.st_ctim;
+#endif
+    rpathAcc->second.ino        = statInfo.st_ino;
+    rpathAcc->second.devId      = statInfo.st_dev;
+
+    return rpathAcc->second.cachedUnit;
+  }();
+
+  if (path != rpath) {
+    NonRepoUnitCache::accessor pathAcc;
+    s_nonRepoUnitCache.insert(pathAcc, path);
+    pathAcc->second.cachedUnit = cuptr;
+#ifdef _MSC_VER
+    pathAcc->second.mtime      = statInfo.st_mtime;
+#else
+    pathAcc->second.mtime      = statInfo.st_mtim;
+    pathAcc->second.ctime      = statInfo.st_ctim;
+#endif
+    pathAcc->second.ino        = statInfo.st_ino;
+    pathAcc->second.devId      = statInfo.st_dev;
+  }
+
+  return cuptr->cu;
+}
+
+CachedUnit lookupUnitNonRepoAuth(StringData* requestedPath,
+                                 const struct stat& statInfo) {
+  // Steady state, its probably already in the cache. Try that first
+  {
+    NonRepoUnitCache::const_accessor acc;
+    if (s_nonRepoUnitCache.find(acc, requestedPath)) {
+      if (!isChanged(acc->second, statInfo)) {
+        return acc->second.cachedUnit->cu;
+      }
     }
   }
-
-  /*
-   * NB: the new-unit creation path is here, and is done while holding the tbb
-   * lock on s_nonRepoUnitCache.  This was originally done deliberately to
-   * avoid wasting time in the compiler (during server startup, many requests
-   * hit the same code initial paths that are shared, and would all be
-   * compiling the same files).  It's not 100% clear if this is the best way to
-   * handle that idea, though (tbb locks spin aggressively and are expected to
-   * be low contention).
-   */
-
-  /*
-   * Don't cache if createNewUnit returns an empty CachedUnit---we'll need to
-   * try again anyway if someone tries to load this path, since it might exist
-   * later.
-   *
-   * If there was a unit for this path already, we need to put it on the
-   * Treadmill for eventual reclaimation.  We can't delete it immediately
-   * because other requests may still be using it.
-   */
-  auto const cu = createUnitFromFile(path, statInfo.st_size);
-  if (auto const oldUnit = acc->second.cachedUnit.unit) {
-    Treadmill::enqueue([oldUnit] { reclaimUnit(oldUnit); });
-  }
-  acc->second.cachedUnit = cu;
-  acc->second.mtime      = statInfo.st_mtim;
-  acc->second.ino        = statInfo.st_ino;
-  acc->second.devId      = statInfo.st_dev;
-  return cu;
+  return loadUnitNonRepoAuth(requestedPath, statInfo);
 }
 
 //////////////////////////////////////////////////////////////////////
 // resolveVmInclude callbacks
 
-const StaticString s_file_url("file://");
-
 struct ResolveIncludeContext {
   String path;    // translated path of the file
   struct stat* s; // stat for the file
@@ -315,7 +357,7 @@ bool findFileWrapper(const String& file, void* ctx) {
   }
 
   // handle file://
-  if (file.substr(0, 7) == s_file_url) {
+  if (StringUtil::IsFileUrl(file)) {
     return findFileWrapper(file.substr(7), ctx);
   }
 
@@ -324,7 +366,7 @@ bool findFileWrapper(const String& file, void* ctx) {
   // TranslatePath() will canonicalize the path and also check
   // whether the file is in an allowed directory.
   String translatedPath = File::TranslatePathKeepRelative(file);
-  if (file[0] != '/') {
+  if (!FileUtil::isAbsolutePath(file.toCppString())) {
     if (findFile(translatedPath.get(), context->s, context->allow_dir)) {
       context->path = translatedPath;
       return true;
@@ -340,8 +382,9 @@ bool findFileWrapper(const String& file, void* ctx) {
   std::string server_root(SourceRootInfo::GetCurrentSourceRoot());
   if (server_root.empty()) {
     server_root = std::string(g_context->getCwd().data());
-    if (server_root.empty() || server_root[server_root.size() - 1] != '/') {
-      server_root += "/";
+    if (server_root.empty() ||
+        FileUtil::isDirSeparator(server_root[server_root.size() - 1])) {
+      server_root += FileUtil::getDirSeparator();
     }
   }
   String rel_path(FileUtil::relativePath(server_root, translatedPath.data()));
@@ -362,19 +405,39 @@ CachedUnit checkoutFile(StringData* path, const struct stat& statInfo) {
 
 //////////////////////////////////////////////////////////////////////
 
-}
+} // end empty namespace
 
 //////////////////////////////////////////////////////////////////////
 
+const std::string mangleUnitPHP7Options() {
+  // As the list of options increases, we may want to do something smarter here?
+  std::string s;
+  s +=
+      (RuntimeOption::PHP7_IntSemantics ? '1' : '0')
+    + (RuntimeOption::PHP7_LTR_assign ? '1' : '0')
+    + (RuntimeOption::PHP7_NoHexNumerics ? '1' : '0')
+    + (RuntimeOption::PHP7_ScalarTypes ? '1' : '0')
+    + (RuntimeOption::PHP7_UVS ? '1' : '0');
+  return s;
+}
+
 std::string mangleUnitMd5(const std::string& fileMd5) {
   std::string t = fileMd5 + '\0'
-    + (RuntimeOption::EnableEmitSwitch ? '1' : '0')
+    + (RuntimeOption::EvalEmitSwitch ? '1' : '0')
+    + (RuntimeOption::EvalEmitNewMInstrs ? '1' : '0')
     + (RuntimeOption::EnableHipHopExperimentalSyntax ? '1' : '0')
     + (RuntimeOption::EnableHipHopSyntax ? '1' : '0')
     + (RuntimeOption::EnableXHP ? '1' : '0')
     + (RuntimeOption::EvalAllowHhas ? '1' : '0')
     + (RuntimeOption::EvalJitEnableRenameFunction ? '1' : '0')
-    + (RuntimeOption::IntsOverflowToInts ? '1' : '0');
+    + (RuntimeOption::IntsOverflowToInts ? '1' : '0')
+    + (RuntimeOption::EvalEnableCallBuiltin ? '1' : '0')
+    + (RuntimeOption::AssertEmitted ? '1' : '0')
+    + RuntimeOption::EvalUseExternalEmitter + '\0'
+    + (RuntimeOption::EvalExternalEmitterFallback ? '1' : '0')
+    + (RuntimeOption::EvalExternalEmitterAllowPartial ? '1' : '0')
+    + (RuntimeOption::AutoprimeGenerators ? '1' : '0')
+    + mangleUnitPHP7Options();
   return string_md5(t.c_str(), t.size());
 }
 
@@ -393,7 +456,7 @@ String resolveVmInclude(StringData* path,
   ctx.s = s;
   ctx.allow_dir = allow_dir;
   void* vpCtx = &ctx;
-  resolve_include(path, currentDir, findFileWrapper, vpCtx);
+  resolve_include(String{path}, currentDir, findFileWrapper, vpCtx);
   // If resolve_include() could not find the file, return NULL
   return ctx.path;
 }
@@ -421,38 +484,12 @@ Unit* lookupUnit(StringData* path, const char* currentDir, bool* initial_opt) {
     return it->second;
   }
 
-  // We didn't find it, so try the realpath.
-  auto const alreadyResolved =
-    RuntimeOption::RepoAuthoritative ||
-    (!RuntimeOption::CheckSymLink && (spath[0] == '/'));
-  bool hasRealpath = false;
-  String rpath;
-  if (!alreadyResolved) {
-    std::string rp = StatCache::realpath(spath.data());
-    if (rp.size() != 0) {
-      rpath = StringData::Make(rp.data(), rp.size(), CopyString);
-      if (!rpath.same(spath)) {
-        hasRealpath = true;
-        it = eContext->m_evaledFiles.find(rpath.get());
-        if (it != eContext->m_evaledFiles.end()) {
-          // We found it! Update the mapping for spath and return the
-          // unit.
-          auto const unit = it->second;
-          spath.get()->incRefCount();
-          eContext->m_evaledFiles[spath.get()] = unit;
-          initial = false;
-          return unit;
-        }
-      }
-    }
-  }
-
   // This file hasn't been included yet, so we need to parse the file
-  auto const cunit = checkoutFile(hasRealpath ? rpath.get() : spath.get(), s);
+  auto const cunit = checkoutFile(spath.get(), s);
   if (cunit.unit && initial_opt) {
     // if initial_opt is not set, this shouldn't be recorded as a
     // per request fetch of the file.
-    if (RDS::testAndSetBit(cunit.rdsBitId)) {
+    if (rds::testAndSetBit(cunit.rdsBitId)) {
       initial = false;
     }
     // if parsing was successful, update the mappings for spath and
@@ -460,9 +497,11 @@ Unit* lookupUnit(StringData* path, const char* currentDir, bool* initial_opt) {
     eContext->m_evaledFilesOrder.push_back(cunit.unit->filepath());
     eContext->m_evaledFiles[spath.get()] = cunit.unit;
     spath.get()->incRefCount();
-    if (hasRealpath) {
-      eContext->m_evaledFiles[rpath.get()] = cunit.unit;
-      rpath.get()->incRefCount();
+    if (!cunit.unit->filepath()->same(spath.get())) {
+      eContext->m_evaledFiles[cunit.unit->filepath()] = cunit.unit;
+    }
+    if (g_system_profiler) {
+      g_system_profiler->fileLoadCallBack(path->toCppString());
     }
     DEBUGGER_ATTACHED_ONLY(phpDebuggerFileLoadHook(cunit.unit));
   }
@@ -472,4 +511,54 @@ Unit* lookupUnit(StringData* path, const char* currentDir, bool* initial_opt) {
 
 //////////////////////////////////////////////////////////////////////
 
+void preloadRepo() {
+  auto& repo = Repo::get();
+  auto units = repo.enumerateUnits(RepoIdLocal, true, false);
+  if (units.size() == 0) {
+    units = repo.enumerateUnits(RepoIdCentral, true, false);
+  }
+  if (!units.size()) return;
+
+  std::vector<std::thread> workers;
+  auto numWorkers = Process::GetCPUCount();
+  // Compute a batch size that causes each thread to process approximately 16
+  // batches.  Even if the batches are somewhat imbalanced in what they contain,
+  // the straggler workers are very unlikey to take more than 10% longer than
+  // the first worker to finish.
+  size_t batchSize{std::max(units.size() / numWorkers / 16, size_t(1))};
+  std::atomic<size_t> index{0};
+  for (auto worker = 0; worker < numWorkers; ++worker) {
+    workers.push_back(std::thread([&] {
+      hphp_thread_init();
+      hphp_session_init();
+      hphp_context_init();
+
+      while (true) {
+        auto begin = index.fetch_add(batchSize);
+        auto end = std::min(begin + batchSize, units.size());
+        if (begin >= end) break;
+        auto unitCount = end - begin;
+        for (auto i = size_t{0}; i < unitCount; ++i) {
+          auto& kv = units[begin + i];
+          try {
+            lookupUnit(String(RuntimeOption::SourceRoot + kv.first).get(),
+                       "", nullptr);
+          } catch (...) {
+            // swallow errors silently
+          }
+        }
+      }
+
+      hphp_context_exit();
+      hphp_session_exit();
+      hphp_thread_exit();
+    }));
+  }
+  for (auto& worker : workers) {
+    worker.join();
+  }
+}
+
+//////////////////////////////////////////////////////////////////////
+
 }
