@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1998-2010 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
@@ -19,8 +19,6 @@
 #include "hphp/runtime/base/intl-convert.h"
 #include "hphp/runtime/base/type-conversions.h"
 #include "hphp/runtime/base/builtin-functions.h"
-#include "hphp/runtime/base/types.h"
-#include "hphp/runtime/base/complex-types.h"
 #include "hphp/runtime/base/runtime-error.h"
 #include "hphp/runtime/base/array-iterator.h"
 #include "hphp/runtime/base/comparisons.h"
@@ -86,7 +84,7 @@ static double collator_u_strtod(const UChar *nptr, UChar **endptr) {
     if (length < (int)sizeof(buf)) {
       numbuf = buf;
     } else {
-      numbuf = (char *) smart_malloc(length + 1);
+      numbuf = (char *) req::malloc(length + 1);
     }
 
     bufpos = numbuf;
@@ -99,7 +97,7 @@ static double collator_u_strtod(const UChar *nptr, UChar **endptr) {
     value = zend_strtod(numbuf, nullptr);
 
     if (numbuf != buf) {
-      smart_free(numbuf);
+      req::free(numbuf);
     }
 
     if (endptr != nullptr) {
@@ -289,7 +287,7 @@ static String intl_convert_str_utf8_to_utf16(const String& utf8_str,
                              utf8_str.data(), utf8_str.length(),
                              status);
   if (U_FAILURE(*status)) {
-    return (const char *)(L"");
+    return empty_string();
   }
   return String((char*)ustr, UBYTES(ustr_len), AttachString);
 }
@@ -554,7 +552,7 @@ static int collator_string_compare_descending(const Variant& v1, const Variant&
 }
 
 static bool collator_sort_internal(bool renumber, Variant &array,
-                                   int sort_flags, bool ascending,
+                                   int sort_flags, bool ascending, bool byKey,
                                    UCollator *coll, Intl::IntlError *errcode) {
   assert(coll);
   errcode->clearError();
@@ -586,7 +584,7 @@ static bool collator_sort_internal(bool renumber, Variant &array,
   }
 
   /* Sort specified array. */
-  temp.sort(cmp_func, false, renumber, coll);
+  temp.sort(cmp_func, byKey, renumber, coll);
 
   /* Convert strings in the specified array back to UTF-8. */
   errcode->clearError();
@@ -603,16 +601,27 @@ static bool collator_sort_internal(bool renumber, Variant &array,
 bool collator_sort(Variant &array, int sort_flags, bool ascending,
                    UCollator *coll, Intl::IntlError *errcode) {
   assert(coll);
-  bool ret = collator_sort_internal(true, array, sort_flags, ascending, coll,
-                                    errcode);
+  bool byKey = false;
+  bool ret = collator_sort_internal(true, array, sort_flags, ascending, byKey,
+                                    coll, errcode);
   return ret;
 }
 
 bool collator_asort(Variant &array, int sort_flags, bool ascending,
                     UCollator *coll, Intl::IntlError *errcode) {
   assert(coll);
-  bool ret = collator_sort_internal(false, array, sort_flags, ascending, coll,
-                                    errcode);
+  bool byKey = false;
+  bool ret = collator_sort_internal(false, array, sort_flags, ascending, byKey,
+                                    coll, errcode);
+  return ret;
+}
+
+bool collator_ksort(Variant &array, int sort_flags, bool ascending,
+                    UCollator *coll, Intl::IntlError *errcode) {
+  assert(coll);
+  bool byKey = true;
+  bool ret = collator_sort_internal(false, array, sort_flags, ascending, byKey,
+                                    coll, errcode);
   return ret;
 }
 
