@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,22 +18,17 @@
 
 namespace HPHP {
 
-ZendCustomElement::ZendCustomElement(
-    void * data_, unsigned int data_size, void ** dest, DtorFunc destructor)
+ZendCustomElement::ZendCustomElement(void* data_, unsigned data_size,
+                                     DtorFunc destructor)
   : m_destructor(destructor)
 {
-  m_data = smart_malloc(data_size);
+  m_data = req::malloc(data_size);
   memcpy(m_data, data_, data_size);
-  if (dest) {
-    *dest = m_data;
-  }
 }
 
 ZendCustomElement::~ZendCustomElement() {
-  if (m_destructor) {
-    m_destructor(data());
-  }
-  smart_free(m_data);
+  if (m_destructor) m_destructor(m_data);
+  req::free(m_data);
 }
 
 }
