@@ -21,6 +21,13 @@ function gd_info(): array;
 function getimagesize(string $filename,
                       mixed &$imageinfo = null): mixed;
 
+/* Identical to getimagesize() except that getimagesizefromstring() accepts
+ * a string instead of a file name as the first parameter.
+ */
+<<__Native>>
+function getimagesizefromstring(string $filename,
+                      mixed &$imageinfo = null): mixed;
+
 /* Returns the extension for the given IMAGETYPE_XXX constant.
  */
 <<__Native>>
@@ -40,6 +47,25 @@ function image2wbmp(resource $image,
                     string $filename = "",
                     int $threshold = -1): bool;
 
+/* Return an image containing the affine tramsformed src image, using
+ * an optional clipping area.
+ */
+<<__Native>>
+function imageaffine(resource $image,
+                     array $affine = [],
+                     array $clip = []): mixed;
+
+/* Concat two matrices.
+ */
+<<__Native>>
+function imageaffinematrixconcat(array $m1, array $m2): mixed;
+
+/* Return an image containing the affine tramsformed src image, using
+ * an optional clipping area.
+ */
+<<__Native>>
+function imageaffinematrixget(int $type, mixed $options = []): mixed;
+
 /* imagealphablending() allows for two different modes of drawing on truecolor
  * images. In blending mode, the alpha channel component of the color supplied
  * to all drawing function, such as imagesetpixel() determines how much of the
@@ -434,6 +460,9 @@ function imagecreatefromjpeg(string $filename): mixed;
 <<__Native>>
 function imagecreatefrompng(string $filename): mixed;
 
+<<__Native>>
+function imagecreatefromwebp(string $filename): mixed;
+
 /* imagecreatefromstring() returns an image identifier representing the image
  * obtained from the given data. These types will be automatically detected if
  * your build of PHP supports them: JPEG, PNG, GIF, WBMP, and GD2.
@@ -486,6 +515,17 @@ function imagecreatefromxpm(string $filename): mixed;
 function imagecreatetruecolor(int $width,
                               int $height): mixed;
 
+/* Crop an image using the given coordinates and size, x, y, width and height.
+ */
+<<__Native>>
+function imagecrop(resource $image, array $rect): mixed;
+
+/* Crop an image automatically using one of the available modes.
+ */
+<<__Native>>
+function imagecropauto(resource $image, int $mode = -1,
+                       float $threshold = 0.5, int $color = -1): mixed;
+
 /* This function is deprecated. Use combination of imagesetstyle() and
  * imageline() instead.
  */
@@ -512,6 +552,11 @@ function imageellipse(resource $image,
                       int $height,
                       int $color): bool;
 
+/* Draws an ellipse centered at the specified coordinates.
+ */
+<<__Native>>
+function imageflip(resource $image, int $mode = -1): bool;
+
 /* Performs a flood fill starting at the given coordinate (top left is 0, 0)
  * with the given color in the image.
  */
@@ -697,6 +742,13 @@ function imagepng(resource $image,
                   int $quality = -1,
                   int $filters = -1): bool;
 
+/* Outputs or saves a webp image from the given image.
+ */
+<<__Native>>
+function imagewebp(resource $image,
+                  string $filename = "",
+                  int $quality = 80): bool;
+
 /* imagepolygon() creates a polygon in the given image.
  */
 <<__Native>>
@@ -736,6 +788,13 @@ function imagerotate(resource $source_image,
 function imagesavealpha(resource $image,
                         bool $saveflag): bool;
 
+/*
+ * imagescale - Scale an image using the given new width and height.
+ */
+<<__Native>>
+function imagescale(resource $img, int $newwidth, int $newheigh = -1,
+                               int $method = IMG_BILINEAR_FIXED): mixed;
+
 /* imagesetbrush() sets the brush image to be used by all line drawing
  * functions (such as imageline() and imagepolygon()) when drawing with the
  * special colors IMG_COLOR_BRUSHED or IMG_COLOR_STYLEDBRUSHED.
@@ -893,3 +952,20 @@ function png2wbmp(string $pngname,
                   int $dest_height,
                   int $dest_width,
                   int $threshold): bool;
+
+/**
+ * imagepalettecopy() copies the palette from the source image
+ * to the destination image.
+ */
+<<__Native>>
+function imagepalettecopy(resource $dst,
+                          resource $src): mixed;
+
+/**
+ * Sets the interpolation method, setting an interpolation method
+ * effects the rendering of various functions in GD,
+ * such as the imagerotate() function.
+ */
+<<__Native>>
+function imagesetinterpolation(resource $img,
+                               int $method = IMG_BILINEAR_FIXED): bool;
