@@ -509,19 +509,6 @@ static int CMYKToRGB(int c, int m, int y, int k, int inverted)
  *
  */
 
-/* Different versions of libjpeg use either 'jboolean' or 'boolean', and
-   some platforms define 'boolean', and so forth. Deal with this
-   madness by typedeffing 'safeboolean' to 'boolean' if HAVE_BOOLEAN
-   is already set, because this is the test that libjpeg uses.
-   Otherwise, typedef it to int, because that's what libjpeg does
-   if HAVE_BOOLEAN is not defined. -TBB */
-
-#ifdef HAVE_BOOLEAN
-typedef boolean safeboolean;
-#else
-typedef int safeboolean;
-#endif /* HAVE_BOOLEAN */
-
 /* Expanded data source object for gdIOCtx input */
 
 typedef struct
@@ -530,7 +517,7 @@ typedef struct
 
 	gdIOCtx *infile;		/* source stream */
 	unsigned char *buffer;	/* start of buffer */
-	safeboolean start_of_file;	/* have we gotten any data yet? */
+	boolean start_of_file;	/* have we gotten any data yet? */
 } my_source_mgr;
 
 typedef my_source_mgr *my_src_ptr;
@@ -589,7 +576,7 @@ void init_source (j_decompress_ptr cinfo)
 
 #define END_JPEG_SEQUENCE "\r\n[*]--:END JPEG:--[*]\r\n"
 
-safeboolean fill_input_buffer (j_decompress_ptr cinfo)
+boolean fill_input_buffer (j_decompress_ptr cinfo)
 {
 	my_src_ptr src = (my_src_ptr) cinfo->src;
 	/* 2.0.12: signed size. Thanks to Geert Jansen */
@@ -780,7 +767,7 @@ void init_destination (j_compress_ptr cinfo)
  * write it out when emptying the buffer externally.
  */
 
-safeboolean empty_output_buffer (j_compress_ptr cinfo)
+boolean empty_output_buffer (j_compress_ptr cinfo)
 {
 	my_dest_ptr dest = (my_dest_ptr) cinfo->dest;
 
