@@ -705,14 +705,14 @@ static WebPResult VPXEncode(const uint8* Y,
     codec_ctl(&enc, VP8E_SET_STATIC_THRESHOLD, 0);
     codec_ctl(&enc, VP8E_SET_TOKEN_PARTITIONS, 2);
 
-    vpx_img_wrap(&img, IMG_FMT_I420,
+    vpx_img_wrap(&img, VPX_IMG_FMT_I420,
                  y_width, y_height, 16, (uint8*)(Y));
-    img.planes[PLANE_Y] = (uint8*)(Y);
-    img.planes[PLANE_U] = (uint8*)(U);
-    img.planes[PLANE_V] = (uint8*)(V);
-    img.stride[PLANE_Y] = y_stride;
-    img.stride[PLANE_U] = uv_stride;
-    img.stride[PLANE_V] = uv_stride;
+    img.planes[VPX_PLANE_Y] = (uint8*)(Y);
+    img.planes[VPX_PLANE_U] = (uint8*)(U);
+    img.planes[VPX_PLANE_V] = (uint8*)(V);
+    img.stride[VPX_PLANE_Y] = y_stride;
+    img.stride[VPX_PLANE_U] = uv_stride;
+    img.stride[VPX_PLANE_V] = uv_stride;
 
     res = vpx_codec_encode(&enc, &img, 0, 1, 0, VPX_DL_BEST_QUALITY);
 
@@ -765,17 +765,17 @@ WebPResult WebPEncode(const uint8* Y,
 	  const int img_size_bytes  = *p_out_size_bytes - kRiffHeaderSize;
 	  const int chunk_size = (img_size_bytes + 1) & ~1;  /* make size even */
 	  const int riff_size = chunk_size + 12;
-	  const uint8_t kRiffHeader[20] = { 'R', 'I', 'F', 'F',
-										(riff_size >>  0) & 255,
-										(riff_size >>  8) & 255,
-										(riff_size >> 16) & 255,
-										(riff_size >> 24) & 255,
+	  const char kRiffHeader[20] = { 'R', 'I', 'F', 'F',
+										char((riff_size >>  0) & 255),
+										char((riff_size >>  8) & 255),
+										char((riff_size >> 16) & 255),
+										char((riff_size >> 24) & 255),
 										'W', 'E', 'B', 'P',
 										'V', 'P', '8', ' ',
-										(chunk_size >>  0) & 255,
-										(chunk_size >>  8) & 255,
-										(chunk_size >> 16) & 255,
-										(chunk_size >> 24) & 255 };
+										char((chunk_size >>  0) & 255),
+										char((chunk_size >>  8) & 255),
+										char((chunk_size >> 16) & 255),
+										char((chunk_size >> 24) & 255) };
 	  memcpy(*p_out, kRiffHeader, kRiffHeaderSize);
 
 	  if (psnr) {
