@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -16,7 +16,7 @@
 */
 #include "hphp/runtime/ext/icu/ext_icu_locale.h"
 #include "hphp/runtime/ext/icu/icu.h"
-#include "hphp/runtime/ext/ext_string.h"
+#include "hphp/runtime/ext/string/ext_string.h"
 
 #include <unicode/ures.h>
 #include <unicode/uloc.h>
@@ -160,7 +160,7 @@ static Variant get_icu_value(const String &locale, LocaleTag tag,
       if (pos == 0) {
         return init_null();
       } else if (pos > 0) {
-        locale_name = f_substr(locale, 0, pos - 1);
+        locale_name = locale.substr(0, pos - 1);
       }
     }
   }
@@ -181,7 +181,7 @@ static Variant get_icu_value(const String &locale, LocaleTag tag,
   do {
     UErrorCode error = U_ZERO_ERROR;
     int32_t len = ulocfunc(locale_name.c_str(),
-                           buf.get()->mutableData(), buf.get()->capacity(),
+                           buf.get()->mutableData(), buf.capacity() + 1,
                            &error);
     if (error != U_BUFFER_OVERFLOW_ERROR &&
         error != U_STRING_NOT_TERMINATED_WARNING) {
@@ -192,7 +192,7 @@ static Variant get_icu_value(const String &locale, LocaleTag tag,
       buf.setSize(len);
       return buf;
     }
-    if (len <= buf.get()->capacity()) {
+    if (len <= buf.capacity() + 1) {
       // Avoid infinite loop
       s_intl_error->setError(U_INTERNAL_PROGRAM_ERROR,
                              "Got invalid response from ICU");
@@ -490,7 +490,7 @@ static Array HHVM_STATIC_METHOD(Locale, getKeywords, const String& locale) {
 tryagain:
     error = U_ZERO_ERROR;
     int val_len = uloc_getKeywordValue(locname.c_str(), key,
-                                       ptr, val.get()->capacity(), &error);
+                                       ptr, val.capacity() + 1, &error);
     if (error == U_BUFFER_OVERFLOW_ERROR) {
       val = String(val_len + 128, ReserveString);
       ptr = val.get()->mutableData();
@@ -523,10 +523,10 @@ static Variant HHVM_STATIC_METHOD(Locale, getScript, const String& locale) {
 static String locale_suffix_strip(const String& locale) {
   for (int i = locale.size(); i >= 0; --i) {
     if (isIDSeparator(locale[i])) {
-      if ((i>=2) && isIDSeparator(locale[i-2])) {
-        return f_substr(locale, 0, i - 2);
+      if ((i >= 2) && isIDSeparator(locale[i - 2])) {
+        return locale.substr(0, i - 2);
       } else {
-        return f_substr(locale, 0, i);
+        return locale.substr(0, i);
       }
     }
   }
@@ -602,12 +602,12 @@ static Variant get_private_subtags(const String& locname) {
         /* loc_name ends with '-x-' */
         return init_null();
       }
-      return f_substr(locale, pos);
+      return locale.substr(pos);
     }
     if ((pos + 1) >= locale.size()) {
       return init_null();
     }
-    locale = f_substr(locale, pos + 1);
+    locale = locale.substr(pos + 1);
   }
   return init_null();
 }
@@ -691,30 +691,22 @@ void IntlExtension::initLocale() {
   HHVM_STATIC_ME(Locale, parseLocale);
   HHVM_STATIC_ME(Locale, setDefault);
 
-#define ULOC_CONST(nm,val) Native::registerClassConstant<KindOfStaticString>\
-                               (s_Locale.get(), s_##nm.get(), s_##val.get())
-
   Native::registerClassConstant<KindOfNull>(s_Locale.get(),
                                             s_DEFAULT_LOCALE.get());
-  ULOC_CONST(LANG_TAG,               LOC_LANG);
-  ULOC_CONST(EXTLANG_TAG,            LOC_EXTLANG);
-  ULOC_CONST(SCRIPT_TAG,             LOC_SCRIPT);
-  ULOC_CONST(REGION_TAG,             LOC_REGION);
-  ULOC_CONST(VARIANT_TAG,            LOC_VARIANT);
-  ULOC_CONST(GRANDFATHERED_LANG_TAG, GRANDFATHERED);
-  ULOC_CONST(PRIVATE_TAG,            LOC_PRIVATE);
+  HHVM_RCC_STR(Locale, LANG_TAG,               s_LOC_LANG);
+  HHVM_RCC_STR(Locale, EXTLANG_TAG,            s_LOC_EXTLANG);
+  HHVM_RCC_STR(Locale, SCRIPT_TAG,             s_LOC_SCRIPT);
+  HHVM_RCC_STR(Locale, REGION_TAG,             s_LOC_REGION);
+  HHVM_RCC_STR(Locale, VARIANT_TAG,            s_LOC_VARIANT);
+  HHVM_RCC_STR(Locale, GRANDFATHERED_LANG_TAG, s_GRANDFATHERED);
+  HHVM_RCC_STR(Locale, PRIVATE_TAG,            s_LOC_PRIVATE);
 
-#undef ULOC_CONST
+  HHVM_RC_INT_SAME(ULOC_ACTUAL_LOCALE);
+  HHVM_RCC_INT(Locale, ACTUAL_LOCALE, ULOC_ACTUAL_LOCALE);
+  HHVM_RC_INT_SAME(ULOC_VALID_LOCALE);
+  HHVM_RCC_INT(Locale, VALID_LOCALE, ULOC_VALID_LOCALE);
 
-#define ULOC_LOCALE_CONST(cns) \
-  Native::registerConstant<KindOfInt64>\
-    (makeStaticString("ULOC_" #cns), ULOC_##cns); \
-  Native::registerClassConstant<KindOfInt64>\
-    (s_Locale.get(), makeStaticString(#cns), ULOC_##cns);
-
-  ULOC_LOCALE_CONST(ACTUAL_LOCALE);
-  ULOC_LOCALE_CONST(VALID_LOCALE);
-#undef ULOC_LOCALE_CONST
+  HHVM_RC_INT(INTL_MAX_LOCALE_LEN, 80);
 
   loadSystemlib("icu_locale");
 }
