@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -15,8 +15,9 @@
    +----------------------------------------------------------------------+
 */
 
-#include "hphp/runtime/base/base-includes.h"
+#include "hphp/runtime/ext/extension.h"
 #include "hphp/runtime/base/runtime-error.h"
+#include "hphp/runtime/ext/std/ext_std_math.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,15 +33,14 @@ namespace HPHP {
 
 class MCrypt : public SweepableResourceData {
 public:
-  explicit MCrypt(MCRYPT td) : m_td(td), m_init(false) {
-  }
+  explicit MCrypt(MCRYPT td) : m_td(td), m_init(false) {}
 
   ~MCrypt() {
     MCrypt::close();
   }
 
-  void sweep() FOLLY_OVERRIDE {
-    close();
+  bool isInvalid() const override {
+    return m_td == MCRYPT_FAILED;
   }
 
   void close() {
@@ -53,12 +53,17 @@ public:
 
   CLASSNAME_IS("mcrypt");
   // overriding ResourceData
-  virtual const String& o_getClassNameHook() const { return classnameof(); }
+  const String& o_getClassNameHook() const override { return classnameof(); }
 
+  DECLARE_RESOURCE_ALLOCATION(MCrypt)
+
+public:
   MCRYPT m_td;
   bool m_init;
 };
 
+IMPLEMENT_RESOURCE_ALLOCATION(MCrypt)
+
 typedef enum {
   RANDOM = 0,
   URANDOM,
@@ -131,15 +136,15 @@ static Variant php_mcrypt_do_crypt(const String& cipher, const String& key,
   if (mcrypt_enc_mode_has_iv(td) == 1) {
     if (!iv.empty()) {
       if (iv_size != iv.size()) {
-        raise_warning("mcrypt_encrypt(): The IV parameter must be as long as "
-                      "the blocksize");
+        raise_warning("%s(): The IV parameter must be as long as "
+                      "the blocksize", name);
       } else {
         iv_s = (char*)malloc(iv_size + 1);
         memcpy(iv_s, iv.data(), iv_size);
       }
     } else {
-      raise_warning("mcrypt_encrypt(): The IV parameter must be as long as "
-                    "the blocksize");
+      raise_warning("%s(): The IV parameter must be as long as "
+                    "the blocksize", name);
       iv_s = (char*)malloc(iv_size + 1);
       memset(iv_s, 0, iv_size + 1);
     }
@@ -154,13 +159,13 @@ static Variant php_mcrypt_do_crypt(const String& cipher, const String& key,
     block_size = mcrypt_enc_get_block_size(td);
     data_size = (((data.size() - 1) / block_size) + 1) * block_size;
     s = String(data_size, ReserveString);
-    data_s = (char*)s.bufferSlice().ptr;
+    data_s = (char*)s.mutableData();
     memset(data_s, 0, data_size);
     memcpy(data_s, data.data(), data.size());
   } else { /* It's not a block algorithm */
     data_size = data.size();
     s = String(data_size, ReserveString);
-    data_s = (char*)s.bufferSlice().ptr;
+    data_s = (char*)s.mutableData();
     memcpy(data_s, data.data(), data.size());
   }
 
@@ -186,10 +191,23 @@ static Variant php_mcrypt_do_crypt(const String& cipher, const String& key,
   return s;
 }
 
+static req::ptr<MCrypt> get_valid_mcrypt_resource(const Resource& td) {
+  auto pm = dyn_cast_or_null<MCrypt>(td);
+
+  if (pm == nullptr || pm->isInvalid()) {
+    raise_warning("supplied argument is not a valid MCrypt resource");
+    return nullptr;
+  }
+
+  return pm;
+}
+
 static Variant mcrypt_generic(const Resource& td, const String& data,
                               bool dencrypt) {
-  MCrypt *pm = td.getTyped<MCrypt>();
-  if (!pm->m_init) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  } else if (!pm->m_init) {
     raise_warning("Operation disallowed prior to mcrypt_generic_init().");
     return false;
   }
@@ -207,13 +225,13 @@ static Variant mcrypt_generic(const Resource& td, const String& data,
     block_size = mcrypt_enc_get_block_size(pm->m_td);
     data_size = (((data.size() - 1) / block_size) + 1) * block_size;
     s = String(data_size, ReserveString);
-    data_s = (unsigned char *)s.bufferSlice().ptr;
+    data_s = (unsigned char *)s.mutableData();
     memset(data_s, 0, data_size);
     memcpy(data_s, data.data(), data.size());
   } else { /* It's not a block algorithm */
     data_size = data.size();
     s = String(data_size, ReserveString);
-    data_s = (unsigned char *)s.bufferSlice().ptr;
+    data_s = (unsigned char *)s.mutableData();
     memcpy(data_s, data.data(), data.size());
   }
 
@@ -244,11 +262,16 @@ Variant HHVM_FUNCTION(mcrypt_module_open, const String& algorithm,
     return false;
   }
 
-  return Resource(new MCrypt(td));
+  return Variant(req::make<MCrypt>(td));
 }
 
 bool HHVM_FUNCTION(mcrypt_module_close, const Resource& td) {
-  td.getTyped<MCrypt>()->close();
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  pm->close();
   return true;
 }
 
@@ -376,7 +399,8 @@ Variant HHVM_FUNCTION(mcrypt_create_iv, int size, int source /* = 0 */) {
   } else {
     n = size;
     while (size) {
-      iv[--size] = (char)(255.0 * rand() / RAND_MAX);
+      // Use userspace rand() function because it handles auto-seeding
+      iv[--size] = (char)HHVM_FN(rand)(0, 255);
     }
   }
   return String(iv, n, AttachString);
@@ -505,36 +529,66 @@ Variant HHVM_FUNCTION(mcrypt_get_key_size, const String& cipher,
   return ret;
 }
 
-String HHVM_FUNCTION(mcrypt_enc_get_algorithms_name, const Resource& td) {
-  char *name = mcrypt_enc_get_algorithms_name(td.getTyped<MCrypt>()->m_td);
+Variant HHVM_FUNCTION(mcrypt_enc_get_algorithms_name, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  char *name = mcrypt_enc_get_algorithms_name(pm->m_td);
   String ret(name, CopyString);
   mcrypt_free(name);
   return ret;
 }
 
-int64_t HHVM_FUNCTION(mcrypt_enc_get_block_size, const Resource& td) {
-  return mcrypt_enc_get_block_size(td.getTyped<MCrypt>()->m_td);
+Variant HHVM_FUNCTION(mcrypt_enc_get_block_size, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_get_block_size(pm->m_td);
 }
 
-int64_t HHVM_FUNCTION(mcrypt_enc_get_iv_size, const Resource& td) {
-  return mcrypt_enc_get_iv_size(td.getTyped<MCrypt>()->m_td);
+Variant HHVM_FUNCTION(mcrypt_enc_get_iv_size, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_get_iv_size(pm->m_td);
 }
 
-int64_t HHVM_FUNCTION(mcrypt_enc_get_key_size, const Resource& td) {
-  return mcrypt_enc_get_key_size(td.getTyped<MCrypt>()->m_td);
+Variant HHVM_FUNCTION(mcrypt_enc_get_key_size, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_get_key_size(pm->m_td);
 }
 
-String HHVM_FUNCTION(mcrypt_enc_get_modes_name, const Resource& td) {
-  char *name = mcrypt_enc_get_modes_name(td.getTyped<MCrypt>()->m_td);
+Variant HHVM_FUNCTION(mcrypt_enc_get_modes_name, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  char *name = mcrypt_enc_get_modes_name(pm->m_td);
   String ret(name, CopyString);
   mcrypt_free(name);
   return ret;
 }
 
-Array HHVM_FUNCTION(mcrypt_enc_get_supported_key_sizes, const Resource& td) {
+Variant HHVM_FUNCTION(mcrypt_enc_get_supported_key_sizes, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
   int count = 0;
   int *key_sizes =
-    mcrypt_enc_get_supported_key_sizes(td.getTyped<MCrypt>()->m_td, &count);
+    mcrypt_enc_get_supported_key_sizes(pm->m_td, &count);
 
   Array ret = Array::Create();
   for (int i = 0; i < count; i++) {
@@ -545,25 +599,49 @@ Array HHVM_FUNCTION(mcrypt_enc_get_supported_key_sizes, const Resource& td) {
 }
 
 bool HHVM_FUNCTION(mcrypt_enc_is_block_algorithm_mode, const Resource& td) {
-  return mcrypt_enc_is_block_algorithm_mode(td.getTyped<MCrypt>()->m_td) == 1;
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_is_block_algorithm_mode(pm->m_td) == 1;
 }
 
 bool HHVM_FUNCTION(mcrypt_enc_is_block_algorithm, const Resource& td) {
-  return mcrypt_enc_is_block_algorithm(td.getTyped<MCrypt>()->m_td) == 1;
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_is_block_algorithm(pm->m_td) == 1;
 }
 
 bool HHVM_FUNCTION(mcrypt_enc_is_block_mode, const Resource& td) {
-  return mcrypt_enc_is_block_mode(td.getTyped<MCrypt>()->m_td) == 1;
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_is_block_mode(pm->m_td) == 1;
 }
 
-int64_t HHVM_FUNCTION(mcrypt_enc_self_test, const Resource& td) {
-  return mcrypt_enc_self_test(td.getTyped<MCrypt>()->m_td);
+Variant HHVM_FUNCTION(mcrypt_enc_self_test, const Resource& td) {
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
+  return mcrypt_enc_self_test(pm->m_td);
 }
 
-int64_t HHVM_FUNCTION(mcrypt_generic_init, const Resource& td,
+Variant HHVM_FUNCTION(mcrypt_generic_init, const Resource& td,
                                            const String& key,
                                            const String& iv) {
-  MCrypt *pm = td.getTyped<MCrypt>();
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
   int max_key_size = mcrypt_enc_get_key_size(pm->m_td);
   int iv_size = mcrypt_enc_get_iv_size(pm->m_td);
 
@@ -591,7 +669,7 @@ int64_t HHVM_FUNCTION(mcrypt_generic_init, const Resource& td,
     raise_warning("Iv size incorrect; supplied length: %d, needed: %d",
                     iv.size(), iv_size);
   }
-  memcpy(iv_s, iv.data(), iv_size);
+  memcpy(iv_s, iv.data(), std::min(iv_size, iv.size()));
 
   mcrypt_generic_deinit(pm->m_td);
   int result = mcrypt_generic_init(pm->m_td, key_s, key_size, iv_s);
@@ -612,8 +690,10 @@ int64_t HHVM_FUNCTION(mcrypt_generic_init, const Resource& td,
       raise_warning("Unknown error");
       break;
     }
+  } else {
+    pm->m_init = true;
   }
-  pm->m_init = true;
+
   free(iv_s);
   free(key_s);
   return result;
@@ -629,7 +709,11 @@ Variant HHVM_FUNCTION(mdecrypt_generic, const Resource& td,
 }
 
 bool HHVM_FUNCTION(mcrypt_generic_deinit, const Resource& td) {
-  MCrypt *pm = td.getTyped<MCrypt>();
+  auto pm = get_valid_mcrypt_resource(td);
+  if (!pm) {
+    return false;
+  }
+
   if (mcrypt_generic_deinit(pm->m_td) < 0) {
     raise_warning("Could not terminate encryption specifier");
     return false;
@@ -644,175 +728,51 @@ bool HHVM_FUNCTION(mcrypt_generic_end, const Resource& td) {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-const StaticString s_MCRYPT_3DES("MCRYPT_3DES");
-const StaticString s_MCRYPT_ARCFOUR("MCRYPT_ARCFOUR");
-const StaticString s_MCRYPT_ARCFOUR_IV("MCRYPT_ARCFOUR_IV");
-const StaticString s_MCRYPT_BLOWFISH("MCRYPT_BLOWFISH");
-const StaticString s_MCRYPT_BLOWFISH_COMPAT("MCRYPT_BLOWFISH_COMPAT");
-const StaticString s_MCRYPT_CAST_128("MCRYPT_CAST_128");
-const StaticString s_MCRYPT_CAST_256("MCRYPT_CAST_256");
-const StaticString s_MCRYPT_CRYPT("MCRYPT_CRYPT");
-const StaticString s_MCRYPT_DECRYPT("MCRYPT_DECRYPT");
-const StaticString s_MCRYPT_DES("MCRYPT_DES");
-const StaticString s_MCRYPT_DEV_RANDOM("MCRYPT_DEV_RANDOM");
-const StaticString s_MCRYPT_DEV_URANDOM("MCRYPT_DEV_URANDOM");
-const StaticString s_MCRYPT_ENCRYPT("MCRYPT_ENCRYPT");
-const StaticString s_MCRYPT_ENIGNA("MCRYPT_ENIGNA");
-const StaticString s_MCRYPT_GOST("MCRYPT_GOST");
-const StaticString s_MCRYPT_IDEA("MCRYPT_IDEA");
-const StaticString s_MCRYPT_LOKI97("MCRYPT_LOKI97");
-const StaticString s_MCRYPT_MARS("MCRYPT_MARS");
-const StaticString s_MCRYPT_MODE_CBC("MCRYPT_MODE_CBC");
-const StaticString s_MCRYPT_MODE_CFB("MCRYPT_MODE_CFB");
-const StaticString s_MCRYPT_MODE_ECB("MCRYPT_MODE_ECB");
-const StaticString s_MCRYPT_MODE_NOFB("MCRYPT_MODE_NOFB");
-const StaticString s_MCRYPT_MODE_OFB("MCRYPT_MODE_OFB");
-const StaticString s_MCRYPT_MODE_STREAM("MCRYPT_MODE_STREAM");
-const StaticString s_MCRYPT_PANAMA("MCRYPT_PANAMA");
-const StaticString s_MCRYPT_RAND("MCRYPT_RAND");
-const StaticString s_MCRYPT_RC2("MCRYPT_RC2");
-const StaticString s_MCRYPT_RC6("MCRYPT_RC6");
-const StaticString s_MCRYPT_RIJNDAEL_128("MCRYPT_RIJNDAEL_128");
-const StaticString s_MCRYPT_RIJNDAEL_192("MCRYPT_RIJNDAEL_192");
-const StaticString s_MCRYPT_RIJNDAEL_256("MCRYPT_RIJNDAEL_256");
-const StaticString s_MCRYPT_SAFER128("MCRYPT_SAFER128");
-const StaticString s_MCRYPT_SAFER64("MCRYPT_SAFER64");
-const StaticString s_MCRYPT_SAFERPLUS("MCRYPT_SAFERPLUS");
-const StaticString s_MCRYPT_SERPENT("MCRYPT_SERPENT");
-const StaticString s_MCRYPT_SKIPJACK("MCRYPT_SKIPJACK");
-const StaticString s_MCRYPT_THREEWAY("MCRYPT_THREEWAY");
-const StaticString s_MCRYPT_TRIPLEDES("MCRYPT_TRIPLEDES");
-const StaticString s_MCRYPT_TWOFISH("MCRYPT_TWOFISH");
-const StaticString s_MCRYPT_WAKE("MCRYPT_WAKE");
-const StaticString s_MCRYPT_XTEA("MCRYPT_XTEA");
-
-class McryptExtension : public Extension {
+class McryptExtension final : public Extension {
  public:
   McryptExtension() : Extension("mcrypt") {}
-  virtual void moduleInit() {
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_3DES.get(), StaticString("tripledes").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_ARCFOUR.get(), StaticString("arcfour").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_ARCFOUR_IV.get(), StaticString("arcfour-iv").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_BLOWFISH.get(), StaticString("blowfish").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_BLOWFISH_COMPAT.get(), StaticString("blowfish-compat").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_CAST_128.get(), StaticString("cast-128").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_CAST_256.get(), StaticString("cast-256").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_CRYPT.get(), StaticString("crypt").get()
-    );
-    Native::registerConstant<KindOfInt64>(
-      s_MCRYPT_DECRYPT.get(), 1
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_DES.get(), StaticString("des").get()
-    );
-    Native::registerConstant<KindOfInt64>(
-      s_MCRYPT_DEV_RANDOM.get(), RANDOM
-    );
-    Native::registerConstant<KindOfInt64>(
-      s_MCRYPT_DEV_URANDOM.get(), URANDOM
-    );
-    Native::registerConstant<KindOfInt64>(
-      s_MCRYPT_ENCRYPT.get(), 0
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_ENIGNA.get(), StaticString("crypt").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_GOST.get(), StaticString("gost").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_IDEA.get(), StaticString("idea").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_LOKI97.get(), StaticString("loki97").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MARS.get(), StaticString("mars").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MODE_CBC.get(), StaticString("cbc").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MODE_CFB.get(), StaticString("cfb").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MODE_ECB.get(), StaticString("ecb").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MODE_NOFB.get(), StaticString("nofb").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MODE_OFB.get(), StaticString("ofb").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_MODE_STREAM.get(), StaticString("stream").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_PANAMA.get(), StaticString("panama").get()
-    );
-    Native::registerConstant<KindOfInt64>(
-      s_MCRYPT_RAND.get(), RAND
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_RC2.get(), StaticString("rc2").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_RC6.get(), StaticString("rc6").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_RIJNDAEL_128.get(), StaticString("rijndael-128").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_RIJNDAEL_192.get(), StaticString("rijndael-192").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_RIJNDAEL_256.get(), StaticString("rijndael-256").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_SAFER128.get(), StaticString("safer-sk128").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_SAFER64.get(), StaticString("safer-sk64").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_SAFERPLUS.get(), StaticString("saferplus").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_SERPENT.get(), StaticString("serpent").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_SKIPJACK.get(), StaticString("skipjack").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_THREEWAY.get(), StaticString("threeway").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_TRIPLEDES.get(), StaticString("tripledes").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_TWOFISH.get(), StaticString("twofish").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_WAKE.get(), StaticString("wake").get()
-    );
-    Native::registerConstant<KindOfStaticString>(
-      s_MCRYPT_XTEA.get(), StaticString("xtea").get()
-    );
+  void moduleInit() override {
+    HHVM_RC_STR(MCRYPT_3DES, "tripledes");
+    HHVM_RC_STR(MCRYPT_ARCFOUR, "arcfour");
+    HHVM_RC_STR(MCRYPT_ARCFOUR_IV, "arcfour-iv");
+    HHVM_RC_STR(MCRYPT_BLOWFISH, "blowfish");
+    HHVM_RC_STR(MCRYPT_BLOWFISH_COMPAT, "blowfish-compat");
+    HHVM_RC_STR(MCRYPT_CAST_128, "cast-128");
+    HHVM_RC_STR(MCRYPT_CAST_256, "cast-256");
+    HHVM_RC_STR(MCRYPT_CRYPT, "crypt");
+    HHVM_RC_INT(MCRYPT_DECRYPT, 1);
+    HHVM_RC_STR(MCRYPT_DES, "des");
+    HHVM_RC_INT(MCRYPT_DEV_RANDOM, RANDOM);
+    HHVM_RC_INT(MCRYPT_DEV_URANDOM, URANDOM);
+    HHVM_RC_INT(MCRYPT_ENCRYPT, 0);
+    HHVM_RC_STR(MCRYPT_ENIGNA, "crypt");
+    HHVM_RC_STR(MCRYPT_GOST, "gost");
+    HHVM_RC_STR(MCRYPT_IDEA, "idea");
+    HHVM_RC_STR(MCRYPT_LOKI97, "loki97");
+    HHVM_RC_STR(MCRYPT_MARS, "mars");
+    HHVM_RC_STR(MCRYPT_MODE_CBC, "cbc");
+    HHVM_RC_STR(MCRYPT_MODE_CFB, "cfb");
+    HHVM_RC_STR(MCRYPT_MODE_ECB, "ecb");
+    HHVM_RC_STR(MCRYPT_MODE_NOFB, "nofb");
+    HHVM_RC_STR(MCRYPT_MODE_OFB, "ofb");
+    HHVM_RC_STR(MCRYPT_MODE_STREAM, "stream");
+    HHVM_RC_STR(MCRYPT_PANAMA, "panama");
+    HHVM_RC_INT(MCRYPT_RAND, RAND);
+    HHVM_RC_STR(MCRYPT_RC2, "rc2");
+    HHVM_RC_STR(MCRYPT_RC6, "rc6");
+    HHVM_RC_STR(MCRYPT_RIJNDAEL_128, "rijndael-128");
+    HHVM_RC_STR(MCRYPT_RIJNDAEL_192, "rijndael-192");
+    HHVM_RC_STR(MCRYPT_RIJNDAEL_256, "rijndael-256");
+    HHVM_RC_STR(MCRYPT_SAFER128, "safer-sk128");
+    HHVM_RC_STR(MCRYPT_SAFER64, "safer-sk64");
+    HHVM_RC_STR(MCRYPT_SAFERPLUS, "saferplus");
+    HHVM_RC_STR(MCRYPT_SERPENT, "serpent");
+    HHVM_RC_STR(MCRYPT_SKIPJACK, "skipjack");
+    HHVM_RC_STR(MCRYPT_THREEWAY, "threeway");
+    HHVM_RC_STR(MCRYPT_TRIPLEDES, "tripledes");
+    HHVM_RC_STR(MCRYPT_TWOFISH, "twofish");
+    HHVM_RC_STR(MCRYPT_WAKE, "wake");
+    HHVM_RC_STR(MCRYPT_XTEA, "xtea");
 
     HHVM_FE(mcrypt_module_open);
     HHVM_FE(mcrypt_module_close);
