@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -15,17 +15,27 @@
    +----------------------------------------------------------------------+
 */
 
-#include "hphp/runtime/base/base-includes.h"
+#include "hphp/runtime/ext/extension.h"
+#include "hphp/runtime/base/builtin-functions.h"
 #include "hphp/runtime/vm/native-data.h"
-#include "hphp/runtime/ext/libmemcached_portability.h"
+#include "hphp/runtime/ext/memcached/libmemcached_portability.h"
+#include "hphp/runtime/ext/sockets/ext_sockets.h"
 #include "hphp/runtime/base/request-local.h"
 #include "hphp/runtime/base/ini-setting.h"
 #include "hphp/runtime/base/request-event-handler.h"
 #include "hphp/runtime/base/zend-string.h"
 #include <vector>
 
-#define MMC_SERIALIZED 1
-#define MMC_COMPRESSED 2
+// MMC values must match pecl-memcache for compatibility
+#define MMC_SERIALIZED  0x0001
+#define MMC_COMPRESSED  0x0002
+
+#define MMC_TYPE_STRING 0x0000
+#define MMC_TYPE_BOOL   0x0100
+#define MMC_TYPE_LONG   0x0300
+#define MMC_TYPE_DOUBLE 0x0700
+
+#define MMC_TYPE_MASK   0x0F00
 
 namespace HPHP {
 
@@ -34,21 +44,12 @@ const int64_t k_MEMCACHE_COMPRESSED = MMC_COMPRESSED;
 static bool ini_on_update_hash_strategy(const std::string& value);
 static bool ini_on_update_hash_function(const std::string& value);
 
-struct MEMCACHEGlobals final : RequestEventHandler {
+struct MEMCACHEGlobals final {
   std::string hash_strategy;
   std::string hash_function;
-
-  MEMCACHEGlobals() {}
-
-  void requestInit() override {
-    hash_strategy = "standard";
-    hash_function = "crc32";
-  }
-
-  void requestShutdown() override {}
 };
 
-IMPLEMENT_STATIC_REQUEST_LOCAL(MEMCACHEGlobals, s_memcache_globals);
+static __thread MEMCACHEGlobals* s_memcache_globals;
 #define MEMCACHEG(name) s_memcache_globals->name
 
 const StaticString s_MemcacheData("MemcacheData");
@@ -96,13 +97,30 @@ static bool ini_on_update_hash_strategy(const std::string& value) {
 
 static bool ini_on_update_hash_function(const std::string& value) {
   if (!strncasecmp(value.data(), "crc32", sizeof("crc32"))) {
-    MEMCACHEG(hash_strategy) = "crc32";
+    MEMCACHEG(hash_function) = "crc32";
   } else if (!strncasecmp(value.data(), "fnv", sizeof("fnv"))) {
-    MEMCACHEG(hash_strategy) = "fnv";
+    MEMCACHEG(hash_function) = "fnv";
   }
   return false;
 }
 
+static bool hasAvailableServers(const MemcacheData* data) {
+  if (memcached_server_count(&data->m_memcache) == 0) {
+    raise_warning("Memcache: No servers added to memcache connection");
+    return false;
+  }
+  return true;
+}
+
+static bool isServerReachable(const String& host, int port /*= 0*/) {
+  auto hostInfo = HHVM_FN(getaddrinfo)(host, port);
+  if (hostInfo.isBoolean() && !hostInfo.toBoolean()) {
+    raise_warning("Memcache: Can't connect to %s:%d", host.c_str(), port);
+    return false;
+  }
+  return true;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // methods
 
@@ -117,12 +135,55 @@ static bool HHVM_METHOD(Memcache, connect, const String& host, int port /*= 0*/,
     const char *socket_path = host.substr(sizeof("unix://") - 1).c_str();
     ret = memcached_server_add_unix_socket(&data->m_memcache, socket_path);
   } else {
+    if (!isServerReachable(host, port)) {
+      return false;
+    }
     ret = memcached_server_add(&data->m_memcache, host.c_str(), port);
   }
 
   return (ret == MEMCACHED_SUCCESS);
 }
 
+static uint32_t memcache_get_flag_for_type(const Variant& var) {
+  switch (var.getType()) {
+    case KindOfBoolean:
+      return MMC_TYPE_BOOL;
+    case KindOfInt64:
+      return MMC_TYPE_LONG;
+    case KindOfDouble:
+      return MMC_TYPE_DOUBLE;
+
+    case KindOfUninit:
+    case KindOfNull:
+    case KindOfStaticString:
+    case KindOfString:
+    case KindOfPersistentArray:
+    case KindOfArray:
+    case KindOfObject:
+    case KindOfResource:
+    case KindOfRef:
+      return MMC_TYPE_STRING;
+
+    case KindOfClass:
+      break;
+  }
+  not_reached();
+}
+
+static void memcache_set_type_from_flag(Variant& var, uint32_t flags) {
+  switch (flags & MMC_TYPE_MASK) {
+  case MMC_TYPE_BOOL:
+    var = var.toBoolean();
+    break;
+  case MMC_TYPE_LONG:
+    var = var.toInt64();
+    break;
+  case MMC_TYPE_DOUBLE:
+    var = var.toDouble();
+    break;
+  }
+}
+
 static std::vector<char> memcache_prepare_for_storage(const MemcacheData* data,
                                                       const Variant& var,
                                                       int &flag) {
@@ -130,6 +191,7 @@ static std::vector<char> memcache_prepare_for_storage(const MemcacheData* data,
   if (var.isString()) {
     v = var.toString();
   } else if (var.isNumeric() || var.isBoolean()) {
+    flag &= ~MMC_COMPRESSED;
     v = var.toString();
   } else {
     flag |= MMC_SERIALIZED;
@@ -138,7 +200,8 @@ static std::vector<char> memcache_prepare_for_storage(const MemcacheData* data,
   std::vector<char> payload;
   size_t value_len = v.length();
 
-  if (data->m_compress_threshold && value_len >= data->m_compress_threshold) {
+  if (!var.isNumeric() && !var.isBoolean() &&
+    data->m_compress_threshold && value_len >= data->m_compress_threshold) {
     flag |= MMC_COMPRESSED;
   }
   if (flag & MMC_COMPRESSED) {
@@ -159,11 +222,14 @@ static std::vector<char> memcache_prepare_for_storage(const MemcacheData* data,
     payload.resize(0);
     payload.insert(payload.end(), v.data(), v.data() + value_len);
    }
+  flag |= memcache_get_flag_for_type(var);
+
   return payload;
 }
 
 static String memcache_prepare_key(const String& var) {
-  auto data = var.get()->mutableData();
+  String var_mutable(var, CopyString);
+  auto data = var_mutable.get()->mutableData();
   for (int i = 0; i < var.length(); i++) {
     // This is a stupid encoding since it causes collisions but it matches php5
     if (data[i] <= ' ') {
@@ -180,7 +246,11 @@ static Variant unserialize_if_serialized(const char *payload,
   if (flags & MMC_SERIALIZED) {
     ret = unserialize_from_buffer(payload, payload_len);
   } else {
-    ret = String(payload, payload_len, CopyString);
+    if (payload_len == 0) {
+      ret = String("");
+    } else {
+      ret = String(payload, payload_len, CopyString);
+    }
   }
   return ret;
  }
@@ -195,25 +265,26 @@ static Variant memcache_fetch_from_storage(const char *payload,
     std::vector<char> buffer;
     size_t buffer_len;
     for (int factor = 1; !done && factor <= 16; ++factor) {
-    if (payload_len >=
-        std::numeric_limits<unsigned long>::max() / (1 << factor)) {
-      break;
+      if (payload_len >=
+          std::numeric_limits<unsigned long>::max() / (1 << factor)) {
+        break;
+      }
+      buffer_len = payload_len * (1 << factor) + 1;
+      buffer.resize(buffer_len);
+      if (uncompress((Bytef*)buffer.data(), &buffer_len,
+                     (const Bytef*)payload, (uLong)payload_len) == Z_OK) {
+        done = true;
+      }
     }
-    buffer_len = payload_len * (1 << factor) + 1;
-    buffer.resize(buffer_len);
-    if (uncompress((Bytef*)buffer.data(), &buffer_len,
-                   (const Bytef*)payload, (uLong)payload_len) == Z_OK) {
-      done = true;
-    }
-  }
-  if (!done) {
-    raise_warning("could not uncompress value");
+    if (!done) {
+      raise_warning("could not uncompress value");
       return init_null();
-  }
-  ret = unserialize_if_serialized(buffer.data(), buffer_len, flags);
+    }
+    ret = unserialize_if_serialized(buffer.data(), buffer_len, flags);
   } else {
     ret = unserialize_if_serialized(payload, payload_len, flags);
   }
+  memcache_set_type_from_flag(ret, flags);
 
   return ret;
 }
@@ -226,6 +297,11 @@ static bool HHVM_METHOD(Memcache, add, const String& key, const Variant& var,
   }
 
   auto data = Native::data<MemcacheData>(this_);
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
   std::vector<char> serialized = memcache_prepare_for_storage(data, var, flag);
 
   String serializedKey = memcache_prepare_key(key);
@@ -247,6 +323,11 @@ static bool HHVM_METHOD(Memcache, set, const String& key, const Variant& var,
   }
 
   auto data = Native::data<MemcacheData>(this_);
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
   String serializedKey = memcache_prepare_key(key);
   std::vector<char> serializedVar =
     memcache_prepare_for_storage(data, var, flag);
@@ -273,6 +354,11 @@ static bool HHVM_METHOD(Memcache, replace, const String& key,
   }
 
   auto data = Native::data<MemcacheData>(this_);
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
   String serializedKey = memcache_prepare_key(key);
   std::vector<char> serialized = memcache_prepare_for_storage(data, var, flag);
 
@@ -288,7 +374,12 @@ static bool HHVM_METHOD(Memcache, replace, const String& key,
 static Variant HHVM_METHOD(Memcache, get, const Variant& key,
                                           VRefParam flags /*= null*/) {
   auto data = Native::data<MemcacheData>(this_);
-  if (key.is(KindOfArray)) {
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
+  if (key.isArray()) {
     std::vector<const char *> real_keys;
     std::vector<size_t> key_len;
     Array keyArr = key.toArray();
@@ -306,10 +397,10 @@ static Variant HHVM_METHOD(Memcache, get, const Variant& key,
     }
 
     if (!real_keys.empty()) {
-      const char *payload = NULL;
+      const char *payload = nullptr;
       size_t payload_len = 0;
       uint32_t flags = 0;
-      const char *res_key = NULL;
+      const char *res_key = nullptr;
       size_t res_key_len = 0;
 
       memcached_result_st result;
@@ -317,7 +408,9 @@ static Variant HHVM_METHOD(Memcache, get, const Variant& key,
       memcached_return_t ret = memcached_mget(&data->m_memcache, &real_keys[0],
                                               &key_len[0], real_keys.size());
       memcached_result_create(&data->m_memcache, &result);
-      Array return_val;
+
+      // To mimic PHP5 should return empty array at failure.
+      Array return_val = Array::Create();
 
       while ((memcached_fetch_result(&data->m_memcache, &result, &ret))
              != nullptr) {
@@ -344,7 +437,7 @@ static Variant HHVM_METHOD(Memcache, get, const Variant& key,
       return return_val;
     }
   } else {
-    char *payload = NULL;
+    char *payload = nullptr;
     size_t payload_len = 0;
     uint32_t flags = 0;
 
@@ -367,6 +460,10 @@ static Variant HHVM_METHOD(Memcache, get, const Variant& key,
       return false;
     }
 
+    if (ret != MEMCACHED_SUCCESS) {
+      return false;
+    }
+
     Variant retval = memcache_fetch_from_storage(payload, payload_len, flags);
     free(payload);
 
@@ -383,6 +480,11 @@ static bool HHVM_METHOD(Memcache, delete, const String& key,
   }
 
   auto data = Native::data<MemcacheData>(this_);
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
   String serializedKey = memcache_prepare_key(key);
   memcached_return_t ret = memcached_delete(&data->m_memcache,
                                             serializedKey.c_str(),
@@ -391,7 +493,7 @@ static bool HHVM_METHOD(Memcache, delete, const String& key,
   return (ret == MEMCACHED_SUCCESS);
 }
 
-static int64_t HHVM_METHOD(Memcache, increment, const String& key,
+static Variant HHVM_METHOD(Memcache, increment, const String& key,
                                                 int offset /*= 1*/) {
   if (key.empty()) {
     raise_warning("Key cannot be empty");
@@ -399,6 +501,11 @@ static int64_t HHVM_METHOD(Memcache, increment, const String& key,
   }
 
   auto data = Native::data<MemcacheData>(this_);
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
   uint64_t value;
   String serializedKey = memcache_prepare_key(key);
   memcached_return_t ret = memcached_increment(&data->m_memcache,
@@ -413,7 +520,7 @@ static int64_t HHVM_METHOD(Memcache, increment, const String& key,
   return false;
 }
 
-static int64_t HHVM_METHOD(Memcache, decrement, const String& key,
+static Variant HHVM_METHOD(Memcache, decrement, const String& key,
                                                 int offset /*= 1*/) {
   if (key.empty()) {
     raise_warning("Key cannot be empty");
@@ -421,6 +528,11 @@ static int64_t HHVM_METHOD(Memcache, decrement, const String& key,
   }
 
   auto data = Native::data<MemcacheData>(this_);
+
+  if (!hasAvailableServers(data)) {
+    return false;
+  }
+
   uint64_t value;
   String serializedKey = memcache_prepare_key(key);
   memcached_return_t ret = memcached_decrement(&data->m_memcache,
@@ -631,11 +743,16 @@ static bool HHVM_METHOD(Memcache, addserver, const String& host,
 
 ///////////////////////////////////////////////////////////////////////////////
 const StaticString s_MEMCACHE_COMPRESSED("MEMCACHE_COMPRESSED");
+const StaticString s_MEMCACHE_HAVE_SESSION("MEMCACHE_HAVE_SESSION");
 
-class MemcacheExtension : public Extension {
+class MemcacheExtension final : public Extension {
   public:
     MemcacheExtension() : Extension("memcache", "3.0.8") {};
     void threadInit() override {
+      // TODO: t5226715 We shouldn't need to check s_defaultLocale here,
+      // but right now this is called for every request.
+      if (s_memcache_globals) return;
+      s_memcache_globals = new MEMCACHEGlobals;
       IniSetting::Bind(this, IniSetting::PHP_INI_ALL,
                        "memcache.hash_strategy", "standard",
                        IniSetting::SetAndGet<std::string>(
@@ -651,11 +768,18 @@ class MemcacheExtension : public Extension {
                        ),
                        &MEMCACHEG(hash_function));
     }
+    void threadShutdown() override {
+      delete s_memcache_globals;
+      s_memcache_globals = nullptr;
+    }
 
-    virtual void moduleInit() {
+    void moduleInit() override {
       Native::registerConstant<KindOfInt64>(
         s_MEMCACHE_COMPRESSED.get(), k_MEMCACHE_COMPRESSED
       );
+      Native::registerConstant<KindOfBoolean>(
+        s_MEMCACHE_HAVE_SESSION.get(), true
+      );
       HHVM_ME(Memcache, connect);
       HHVM_ME(Memcache, add);
       HHVM_ME(Memcache, set);
