@@ -13,68 +13,6 @@ class mysqli {
   <<__Native>>
   private function hh_get_connection(int $state = 0): ?resource;
 
-  public function __get(string $name): mixed {
-    switch ($name) {
-      case 'client_info':
-        return mysqli_get_client_info();
-      case 'client_version':
-        return mysqli_get_client_version();
-      case 'connect_errno':
-        return self::$__connection_errno;
-      case 'connect_error':
-        return self::$__connection_error;
-    }
-
-    // The following properties only work if we are connected
-    $conn = $this->hh_get_connection(1);
-    if (!$conn) {
-      return null;
-    }
-
-    switch ($name) {
-      case 'affected_rows':
-        return mysql_affected_rows($conn);
-      case 'error':
-        return mysql_error($conn);
-      case 'errno':
-        return mysql_errno($conn);
-      case 'field_count':
-        return $this->hh_field_count();
-      case 'host_info':
-        return mysql_get_host_info($conn);
-      case 'info':
-        return mysql_info($conn);
-      case 'insert_id':
-        return mysql_insert_id($conn);
-      case 'protocol_version':
-        return mysql_get_proto_info($conn);
-      case 'server_info':
-        return mysql_get_server_info($conn);
-      case 'server_version':
-        return $this->hh_server_version();
-      case 'sqlstate':
-        return $this->hh_sqlstate();
-      case 'thread_id':
-        return mysql_thread_id($conn);
-      case 'warning_count':
-        return mysql_warning_count($conn);
-      case 'error_list':
-        return $this->__get_error_list();
-    }
-
-    trigger_error('Undefined property: mysqli::$'. $name, E_USER_NOTICE);
-    return null;
-  }
-
-  <<__Native>>
-  private function hh_field_count(): mixed;
-
-  <<__Native>>
-  private function hh_server_version(): mixed;
-
-  <<__Native>>
-  private function hh_sqlstate(): mixed;
-
   /**
    * Open a new connection to the MySQL server
    *
@@ -111,13 +49,29 @@ class mysqli {
       return;
     }
 
+    // If any of the necessary mysqli properties come in as null, then we can
+    // use our default ini options.
+    $host = $this->get_ini_default_if_null($host, "host");
+    $username = $this->get_ini_default_if_null($username, "user");
+    $passwd = $this->get_ini_default_if_null($passwd, "pw");
+    $port = $this->get_ini_default_if_null($port, "port");
+    $socket = $this->get_ini_default_if_null($socket, "socket");
+
     // Connect
     $this->real_connect($host, $username, $passwd, $dbname, $port, $socket);
   }
 
+  private function get_ini_default_if_null(mixed $connect_option,
+                                           string $name) {
+    if ($connect_option === null) {
+      $connect_option = ini_get("mysqli.default_" . $name);
+    }
+    return $connect_option;
+  }
+
   public function __clone(): void {
-    throw new Exception(
-      'Trying to clone an uncloneable object of class mysqli'
+    hphp_throw_fatal_error(
+      'Trying to clone an uncloneable object of class mysqli_result'
     );
   }
 
@@ -302,16 +256,19 @@ class mysqli {
     return $this->real_escape_string($escapestr);
   }
 
-  /**
+ /**
    * Returns a character set object
    *
-   * @return object - The function returns a character set object with
-   *   the following properties:   charset Character set name   collation
-   *   Collation name   dir Directory the charset description was fetched
-   *   from (?) or "" for built-in character sets   min_length Minimum
-   *   character length in bytes   max_length Maximum character length in
-   *   bytes   number Internal character set number   state Character set
-   *   status (?)
+   * @return object - The function returns a character set object with the
+   *   following properties:
+   *   - charset Character set name
+   *   - collation Collation name
+   *   - dir Directory the charset description was fetched from (?) or "" for
+   *         built-in character sets
+   *   - min_length Minimum character length in bytes
+   *   - max_length Maximum character length in bytes
+   *   - number Internal character set number
+   *   - state Character set status (?)
    */
   <<__Native>>
   public function get_charset(): mixed;
@@ -326,19 +283,6 @@ class mysqli {
     return mysqli_get_client_info();
   }
 
-  // The implementation of the getter for $error_list
-  private function __get_error_list(): array {
-    $result = array();
-    if ($this->errno) {
-      $result[] = array(
-        'errno' => $this->errno,
-        'sqlstate' => $this->sqlstate,
-        'error' => $this->error,
-      );
-    }
-    return $result;
-  }
-
   /**
    * Returns statistics about the client connection
    *
@@ -477,7 +421,7 @@ class mysqli {
    *
    * @param array $read - List of connections to check for outstanding
    *   results that can be read.
-   * @param array $error - List of connections on which an error occured,
+   * @param array $error - List of connections on which an error occurred,
    *   for example, query failure or lost connection.
    * @param array $reject - List of connections rejected because no
    *   asynchronous query has been run on for which the function could poll
@@ -560,7 +504,7 @@ class mysqli {
                         int $resultmode = MYSQLI_STORE_RESULT): ?mixed {
     if ($resultmode !== MYSQLI_STORE_RESULT &&
         $resultmode !== MYSQLI_USE_RESULT) {
-      trigger_error("Invalid value for resultmode", E_USER_WARNING);
+      trigger_error("Invalid value for resultmode", E_WARNING);
       return false;
     }
 
@@ -922,47 +866,13 @@ class mysqli_driver {
   private bool $__reconnect = false;
   private int $__report_mode = 0;
 
-  public function __get(string $name): mixed {
-    switch ($name) {
-      case 'client_info':
-        return mysqli_get_client_info();
-      case 'client_version':
-        return mysqli_get_client_version();
-      case 'driver_version':
-        // Lets pretend we are the same version as PHP. Taken from here
-        // http://git.io/wY2WPw
-        return 101009;
-      case 'embedded':
-        return false;
-      case 'reconnect':
-        return $this->__reconnect;
-      case 'report_mode':
-        return $this->__report_mode;
-    }
-
-    trigger_error('Undefined property: mysqli_driver::$'. $name, E_USER_NOTICE);
-    return null;
-  }
-
-  public function __set(string $name, mixed $value): void {
-    switch ($name) {
-      case 'reconnect':
-        $this->__reconnect = (bool)$value;
-        break;
-      case 'report_mode':
-        $this->__report_mode = (int)$value;
-        break;
-      default:
-        trigger_error(
-          'Undefined property: mysqli_driver::$'. $name,
-          E_USER_NOTICE
-        );
-    }
+  public function __construct() {
+    $this->__reconnect = ini_get("mysqli.reconnect") === "1" ? true : false;
   }
 
   public function __clone(): void {
-    throw new Exception(
-      'Trying to clone an uncloneable object of class mysqli_driver'
+    hphp_throw_fatal_error(
+      'Trying to clone an uncloneable object of class mysqli_result'
     );
   }
 }
@@ -974,48 +884,35 @@ class mysqli_driver {
  */
 class mysqli_result {
 
-  private ?resource $__result = null;
+  // Not typing this since we are setting it as a mixed to comply with
+  // GitHub issue 2082. As it is, even with invariants and various ifs
+  // to guarantee the type, HHHBC is not able to infer the property type.
+  // Anyway, the typehint is currently only for optimization purposes at this
+  // point in time. See D1663326
+  private $__result = null;
   private ?int $__resulttype = null;
   private bool $__done = false;
 
-  public function __get(string $name): mixed {
-    if ($this->__result === null) {
-      trigger_error("supplied argument is not a valid MySQL result resource",
-                    E_USER_WARNING);
-      return null;
-    }
-
-    switch ($name) {
-      case 'current_field':
-        return $this->hh_field_tell();
-      case 'field_count':
-        return mysql_num_fields($this->__result);
-      case 'lengths':
-        return mysql_fetch_lengths($this->__result);
-      case 'num_rows':
-        if ($this->__resulttype == MYSQLI_USE_RESULT && !$this->__done) {
-          trigger_error("Function can not be used with MYSQL_USE_RESULT",
-                        E_USER_WARNING);
-          return 0;
-        }
-        return mysql_num_rows($this->__result);
-    }
-
-    trigger_error('Undefined property: mysqli_result::$'. $name, E_USER_NOTICE);
-    return null;
-  }
-
   <<__Native>>
-  private function hh_field_tell(): mixed;
+  private function get_mysqli_conn_resource(mysqli $connection): ?resource;
 
-  public function __construct(resource $result,
+  public function __construct(mixed $result,
                               int $resulttype = MYSQLI_STORE_RESULT) {
-    $this->__result = $result;
+    if (!is_resource($result) && !($result instanceof mysqli)) {
+      $msg = "Argument to mysqli_result::__construct must be of type "
+           . "resource or mysqli";
+      throw new Exception($msg);
+    }
+   if ($result instanceof mysqli) {
+      $this->__result = $this->get_mysqli_conn_resource($result);
+    } else {
+      $this->__result = is_resource($result) ? $result : null;
+    }
     $this->__resulttype = $resulttype;
   }
 
   public function __clone(): void {
-    throw new Exception(
+    hphp_throw_fatal_error(
       'Trying to clone an uncloneable object of class mysqli_result'
     );
   }
@@ -1039,7 +936,7 @@ class mysqli_result {
     }
 
     trigger_error('Mode can be only MYSQLI_NUM, MYSQLI_ASSOC or MYSQLI_BOTH',
-                  E_USER_WARNING);
+                  E_WARNING);
     return null;
   }
 
@@ -1283,28 +1180,6 @@ class mysqli_stmt {
   private ?resource $__stmt = null;
   private ?mysqli $__link = null;
 
-  public function __get(string $name): mixed {
-    switch ($name) {
-      case 'affected_rows':
-        return $this->hh_affected_rows();
-      case 'errno':
-        return $this->hh_errno();
-      case 'error':
-        return $this->hh_error();
-      case 'field_count':
-        return $this->hh_field_count();
-      case 'insert_id':
-        return $this->hh_insert_id();
-      case 'num_rows':
-        return $this->hh_num_rows();
-      case 'param_count':
-        return $this->hh_param_count();
-    }
-
-    trigger_error('Undefined property: mysqli_stmt::$'. $name, E_USER_NOTICE);
-    return null;
-  }
-
   public function __construct(mysqli $link, string $query = null) {
     $this->__link = $link;
     $this->hh_init($link);
@@ -1314,35 +1189,14 @@ class mysqli_stmt {
   }
 
   public function __clone(): void {
-    throw new Exception(
-      'Trying to clone an uncloneable object of class mysqli_stmt'
+    hphp_throw_fatal_error(
+      'Trying to clone an uncloneable object of class mysqli_result'
     );
   }
 
-  <<__Native>>
-  private function hh_affected_rows(): mixed;
-
-  <<__Native>>
-  private function hh_errno(): mixed;
-
-  <<__Native>>
-  private function hh_error(): mixed;
-
-  <<__Native>>
-  private function hh_field_count(): mixed;
-
-  <<__Native>>
-  private function hh_insert_id(): mixed;
-
   <<__Native>>
   private function hh_init(mysqli $connection): void;
 
-  <<__Native>>
-  private function hh_num_rows(): mixed;
-
-  <<__Native>>
-  private function hh_param_count(): mixed;
-
   /**
    * Used to get the current value of a statement attribute
    *
@@ -1555,8 +1409,30 @@ class mysqli_stmt {
    *
    * @return bool -
    */
+  public function store_result(): mixed {
+    // First we need to set the MYSQLI_STMT_ATTR_UPDATE_MAX_LENGTH attribute in
+    // some cases.
+    if ($this->__link->field_count > 0) {
+      $result = $this->result_metadata();
+      if (!is_bool($result)) {
+        $fields = $result->fetch_fields();
+        foreach ($fields as $field) {
+          if ($field->type == MYSQLI_TYPE_BLOB ||
+              $field->type == MYSQLI_TYPE_MEDIUM_BLOB ||
+              $field->type == MYSQLI_TYPE_LONG_BLOB ||
+              $field->type == MYSQLI_TYPE_GEOMETRY) {
+            $this->attr_set(MYSQLI_STMT_ATTR_UPDATE_MAX_LENGTH, 1);
+            break;
+          }
+        }
+      }
+    }
+
+    return $this->hh_store_result();
+  }
+
   <<__Native>>
-  public function store_result(): mixed;
+  public function hh_store_result(): mixed;
 
 }
 
@@ -1648,8 +1524,8 @@ function mysqli_autocommit(mysqli $link, bool $mode): bool {
  * @return bool -
  */
 function mysqli_begin_transaction(mysqli $link,
-                                  int $flags,
-                                  string $name): bool {
+                                  int $flags = 0,
+                                  ?string $name = null): bool {
   return $link->begin_transaction($flags, $name);
 }
 
@@ -1706,9 +1582,8 @@ function mysqli_character_set_name(mysqli $link): string {
  * @return string - A string that represents the MySQL client library
  *   version
  */
-function mysqli_get_client_info(): string {
-  return mysql_get_client_info();
-}
+<<__Native>>
+function mysqli_get_client_info(): string;
 
 /**
  * Returns the MySQL client version as an integer
@@ -1846,14 +1721,17 @@ function mysqli_field_count(mysqli $link): ?int {
  * @param mysqli $link -
  *
  * @return object - The function returns a character set object with the
- *   following properties:   charset Character set name   collation
- *   Collation name   dir Directory the charset description was fetched
- *   from (?) or "" for built-in character sets   min_length Minimum
- *   character length in bytes   max_length Maximum character length in
- *   bytes   number Internal character set number   state Character set
- *   status (?)
+ *   following properties:
+ *   - charset Character set name
+ *   - collation Collation name
+ *   - dir Directory the charset description was fetched from (?) or "" for
+ *         built-in character sets
+ *   - min_length Minimum character length in bytes
+ *   - max_length Maximum character length in bytes
+ *   - number Internal character set number
+ *   - state Character set status (?)
  */
-function mysqli_get_charset(mysqli $link): object {
+function mysqli_get_charset(mysqli $link): mixed {
   return $link->get_charset();
 }
 
@@ -2063,7 +1941,7 @@ function mysqli_ping(mysqli $link): ?bool {
  *
  * @param array $read - List of connections to check for outstanding
  *   results that can be read.
- * @param array $error - List of connections on which an error occured,
+ * @param array $error - List of connections on which an error occurred,
  *   for example, query failure or lost connection.
  * @param array $reject - List of connections rejected because no
  *   asynchronous query has been run on for which the function could poll
