@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -16,14 +16,13 @@
 */
 #include "hphp/runtime/ext/soap/sdl.h"
 
-#include <boost/lexical_cast.hpp>
+#include <folly/Conv.h>
 
 #include "hphp/runtime/ext/soap/soap.h"
 
 namespace HPHP {
 
 using std::string;
-using boost::lexical_cast;
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -364,7 +363,7 @@ static bool schema_simpleType(sdlPtr sdl, xmlAttrPtr tns,
     cur_type->encode->details.sdl_type = newType.get();
     cur_type->encode->to_xml = sdl_guess_convert_xml;
     cur_type->encode->to_zval = sdl_guess_convert_zval;
-    sdl->encoders[boost::lexical_cast<std::string>(sdl->encoders.size())] =
+    sdl->encoders[folly::to<std::string>(sdl->encoders.size())] =
       cur_type->encode;
 
     cur_type = newType;
@@ -451,7 +450,7 @@ static bool schema_list(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr listType,
                           "attribute and subtype");
     }
     sdlTypePtr newType(new sdlType());
-    newType->name = "anonymous" + lexical_cast<string>(sdl->types.size());
+    newType->name = "anonymous" + folly::to<string>(sdl->types.size());
     newType->namens = (char*)tns->children->content;
     cur_type->elements.push_back(newType);
 
@@ -516,7 +515,7 @@ static bool schema_union(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr unionType,
   while (trav) {
     if (node_is_equal(trav,"simpleType")) {
       sdlTypePtr newType(new sdlType());
-      newType->name = "anonymous" + lexical_cast<string>(sdl->types.size());
+      newType->name = "anonymous" + folly::to<string>(sdl->types.size());
       newType->namens = (char*)tns->children->content;
       cur_type->elements.push_back(newType);
       schema_simpleType(sdl, tns, trav, newType);
@@ -1224,7 +1223,7 @@ static bool schema_complexType(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr compType,
     cur_type->encode->details.sdl_type = newType.get();
     cur_type->encode->to_xml = sdl_guess_convert_xml;
     cur_type->encode->to_zval = sdl_guess_convert_zval;
-    sdl->encoders[lexical_cast<string>(sdl->encoders.size())] =
+    sdl->encoders[folly::to<string>(sdl->encoders.size())] =
       cur_type->encode;
 
     cur_type = newType;
@@ -1695,7 +1694,7 @@ static bool schema_attribute(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr attrType,
       }
       sdlTypePtr dummy_type(new sdlType());
       dummy_type->name = string("anonymous") +
-        lexical_cast<string>(sdl->types.size());
+        folly::to<string>(sdl->types.size());
       dummy_type->namens = (char*)tns->children->content;
       schema_simpleType(sdl, tns, trav, dummy_type);
       newAttr->encode = dummy_type->encode;
@@ -1752,7 +1751,7 @@ static bool schema_attributeGroup(sdlPtr sdl, xmlAttrPtr tns,
       }
       key += group_name;
       newAttr->ref = key;
-      cur_type->attributes[lexical_cast<string>(cur_type->attributes.size())] =
+      cur_type->attributes[folly::to<string>(cur_type->attributes.size())] =
         newAttr;
       cur_type = sdlTypePtr();
     }
