@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -25,26 +25,29 @@
 #include <algorithm>
 #include <vector>
 
-#include "folly/ScopeGuard.h"
-#include "folly/String.h"
+#include <folly/ScopeGuard.h>
+#include <folly/String.h>
 
-#include "hphp/runtime/ext/ext_misc.h"
-#include "hphp/runtime/ext/std/ext_std_errorfunc.h"
-#include "hphp/runtime/ext/ext_function.h"
-#include "hphp/runtime/ext/extension.h"
-#include "hphp/runtime/base/runtime-option.h"
-#include "hphp/runtime/base/php-globals.h"
-#include "hphp/runtime/base/unit-cache.h"
+#include "hphp/runtime/base/array-init.h"
+#include "hphp/runtime/base/builtin-functions.h"
 #include "hphp/runtime/base/ini-setting.h"
 #include "hphp/runtime/base/memory-manager.h"
+#include "hphp/runtime/base/php-globals.h"
+#include "hphp/runtime/base/request-event-handler.h"
 #include "hphp/runtime/base/request-local.h"
 #include "hphp/runtime/base/runtime-error.h"
+#include "hphp/runtime/base/runtime-option.h"
+#include "hphp/runtime/base/unit-cache.h"
 #include "hphp/runtime/base/zend-functions.h"
 #include "hphp/runtime/base/zend-string.h"
+#include "hphp/runtime/ext/extension.h"
+#include "hphp/runtime/ext/extension-registry.h"
+#include "hphp/runtime/ext/std/ext_std_errorfunc.h"
+#include "hphp/runtime/ext/std/ext_std_function.h"
+#include "hphp/runtime/ext/std/ext_std_misc.h"
 #include "hphp/runtime/vm/jit/translator-inline.h"
 #include "hphp/system/constants.h"
 #include "hphp/util/process.h"
-#include "hphp/runtime/base/request-event-handler.h"
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -66,13 +69,15 @@ const int64_t k_ASSERT_CALLBACK    = 2;
 const int64_t k_ASSERT_BAIL        = 3;
 const int64_t k_ASSERT_WARNING     = 4;
 const int64_t k_ASSERT_QUIET_EVAL  = 5;
+const int64_t k_ASSERT_EXCEPTION   = 6;
 
 ///////////////////////////////////////////////////////////////////////////////
 
 struct OptionData final : RequestEventHandler {
   void requestInit() override {
-    assertActive = RuntimeOption::AssertActive ? 1 : 0;
-    assertWarning = RuntimeOption::AssertWarning ? 1 : 0;
+    assertActive = 0;
+    assertException = 0;
+    assertWarning = 0;
     assertBail = 0;
     assertQuietEval = false;
   }
@@ -81,7 +86,12 @@ struct OptionData final : RequestEventHandler {
     assertCallback.unset();
   }
 
+  void vscan(IMarker& mark) const override {
+    mark(assertCallback);
+  }
+
   int assertActive;
+  int assertException;
   int assertWarning;
   int assertBail;
   bool assertQuietEval;
@@ -92,6 +102,17 @@ IMPLEMENT_STATIC_REQUEST_LOCAL(OptionData, s_option_data);
 
 /////////////////////////////////////////////////////////////////////////////
 
+void StandardExtension::requestInitOptions() {
+  IniSetting::Bind(IniSetting::CORE, IniSetting::PHP_INI_ALL,
+    "assert.active", "0", &s_option_data->assertActive);
+  IniSetting::Bind(IniSetting::CORE, IniSetting::PHP_INI_ALL,
+    "assert.exception", "0", &s_option_data->assertException);
+  IniSetting::Bind(IniSetting::CORE, IniSetting::PHP_INI_ALL,
+    "assert.warning", "0", &s_option_data->assertWarning);
+  IniSetting::Bind(IniSetting::CORE, IniSetting::PHP_INI_ALL,
+    "assert.bail", "0", &s_option_data->assertBail);
+}
+
 static Variant HHVM_FUNCTION(assert_options,
                              int64_t what, const Variant& value /*=null */) {
   if (what == k_ASSERT_ACTIVE) {
@@ -119,6 +140,11 @@ static Variant HHVM_FUNCTION(assert_options,
     if (!value.isNull()) s_option_data->assertQuietEval = value.toBoolean();
     return Variant(oldValue);
   }
+  if (what == k_ASSERT_EXCEPTION) {
+    int oldValue = s_option_data->assertException;
+    if (!value.isNull()) s_option_data->assertException = value.toBoolean();
+    return Variant(oldValue);
+  }
   throw_invalid_argument("assert option %ld is not supported", (long)what);
   return false;
 }
@@ -140,17 +166,25 @@ static Variant eval_for_assert(ActRec* const curFP, const String& codeStr) {
     return Variant(true);
   }
 
+  if (!(curFP->func()->attrs() & AttrMayUseVV)) {
+    throw_not_supported("assert()",
+                        "assert called from non-varenv function");
+  }
+
+  if (!curFP->hasVarEnv()) {
+    curFP->setVarEnv(VarEnv::createLocal(curFP));
+  }
+  auto varEnv = curFP->getVarEnv();
+
   auto const func = unit->getMain();
   TypedValue retVal;
   g_context->invokeFunc(
     &retVal,
     func,
     init_null_variant,
+    curFP->hasThis() ? curFP->getThis() : nullptr,
     nullptr,
-    nullptr,
-    // Zend appears to share the variable environment with the assert()
-    // builtin, but we deviate by having no shared env here.
-    nullptr /* VarEnv */,
+    varEnv,
     nullptr,
     ExecutionContext::InvokePseudoMain
   );
@@ -158,7 +192,9 @@ static Variant eval_for_assert(ActRec* const curFP, const String& codeStr) {
   return tvAsVariant(&retVal);
 }
 
-static Variant HHVM_FUNCTION(assert, const Variant& assertion) {
+// assert_impl already defined in util/assertions.h
+static Variant impl_assert(const Variant& assertion,
+                           const Variant& message /* = null */) {
   if (!s_option_data->assertActive) return true;
 
   CallerFrame cf;
@@ -170,7 +206,7 @@ static Variant HHVM_FUNCTION(assert, const Variant& assertion) {
       if (RuntimeOption::EvalAuthoritativeMode) {
         // We could support this with compile-time string literals,
         // but it's not yet implemented.
-        throw_not_supported(__func__,
+        throw_not_supported("assert()",
           "assert with strings argument in RepoAuthoritative mode");
       }
       return eval_for_assert(fp, assertion.toString()).toBoolean();
@@ -186,33 +222,54 @@ static Variant HHVM_FUNCTION(assert, const Variant& assertion) {
     ai.append(String(const_cast<StringData*>(unit->filepath())));
     ai.append(Variant(unit->getLineNumber(callerOffset)));
     ai.append(assertion.isString() ? assertion : empty_string_variant_ref);
-    f_call_user_func(1, s_option_data->assertCallback, ai.toArray());
+    HHVM_FN(call_user_func)(s_option_data->assertCallback, ai.toArray());
   }
+  if (s_option_data->assertException) {
+    if (message.isObject()) {
+      Object exn = message.toObject();
+      if (exn.instanceof(SystemLib::s_AssertionErrorClass)) {
+        throw exn;
+      }
+    }
 
+    SystemLib::throwExceptionObject(message.toString());
+  }
   if (s_option_data->assertWarning) {
+    String name(message.isNull() ? "Assertion" : message.toString());
     auto const str = !assertion.isString()
-      ? String("Assertion failed")
-      : concat3("Assertion \"", assertion.toString(), "\" failed");
-    raise_warning("%s", str.data());
+      ? " failed"
+      : concat3(" \"",  assertion.toString(), "\" failed");
+    raise_warning("assert(): %s%s", name.data(),  str.data());
   }
   if (s_option_data->assertBail) {
-    throw ExtendedException("An assertion was raised.");
+    throw ExitException(1);
   }
 
   return init_null();
 }
 
+static Variant HHVM_FUNCTION(SystemLib_assert, const Variant& assertion,
+                             const Variant& message = uninit_null()) {
+  return impl_assert(assertion, message);
+}
+
+static Variant HHVM_FUNCTION(assert, const Variant& assertion,
+                                     const Variant& message /* = null */) {
+  raise_disallowed_dynamic_call("assert should not be called dynamically");
+  return impl_assert(assertion, message);
+}
+
 static int64_t HHVM_FUNCTION(dl, const String& library) {
   return 0;
 }
 
 static bool HHVM_FUNCTION(extension_loaded, const String& name) {
-  return Extension::IsLoaded(name);
+  return ExtensionRegistry::isLoaded(name);
 }
 
 static Array HHVM_FUNCTION(get_loaded_extensions,
                            bool zend_extensions /*=false */) {
-  return Extension::GetLoadedExtensions();
+  return ExtensionRegistry::getLoaded();
 }
 
 static Array HHVM_FUNCTION(get_extension_funcs,
@@ -228,20 +285,24 @@ static Variant HHVM_FUNCTION(get_cfg_var, const String& option) {
 }
 
 static String HHVM_FUNCTION(get_current_user) {
+#ifdef _MSC_VER
+  return Process::GetCurrentUser();
+#else
   int pwbuflen = sysconf(_SC_GETPW_R_SIZE_MAX);
   if (pwbuflen < 1) {
     return empty_string();
   }
-  char *pwbuf = (char*)smart_malloc(pwbuflen);
+  char *pwbuf = (char*)req::malloc(pwbuflen);
   struct passwd pw;
   struct passwd *retpwptr = NULL;
   if (getpwuid_r(getuid(), &pw, pwbuf, pwbuflen, &retpwptr) != 0) {
-    smart_free(pwbuf);
+    req::free(pwbuf);
     return empty_string();
   }
   String ret(pw.pw_name, CopyString);
-  smart_free(pwbuf);
+  req::free(pwbuf);
   return ret;
+#endif
 }
 
 static Array HHVM_FUNCTION(get_defined_constants, bool categorize /*=false */) {
@@ -267,19 +328,11 @@ static String HHVM_FUNCTION(set_include_path, const Variant& new_include_path) {
 static Array HHVM_FUNCTION(get_included_files) {
   PackedArrayInit pai(g_context->m_evaledFilesOrder.size());
   for (auto& file : g_context->m_evaledFilesOrder) {
-    pai.append(const_cast<StringData*>(file));
+    pai.append(Variant{const_cast<StringData*>(file)});
   }
   return pai.toArray();
 }
 
-static int64_t HHVM_FUNCTION(get_magic_quotes_gpc) {
-  return RuntimeOption::EnableMagicQuotesGpc ? 1 : 0;
-}
-
-static int64_t HHVM_FUNCTION(get_magic_quotes_runtime) {
-  return 0;
-}
-
 static Variant HHVM_FUNCTION(getenv, const String& varname) {
   String ret = g_context->getenv(varname);
   if (!ret.isNull()) {
@@ -518,7 +571,7 @@ static int parse_opts(const char * opts, int opts_len, opt_struct **result) {
     }
   }
 
-  opt_struct *paras = (opt_struct *)smart_malloc(sizeof(opt_struct) * count);
+  opt_struct *paras = (opt_struct *)req::malloc(sizeof(opt_struct) * count);
   memset(paras, 0, sizeof(opt_struct) * count);
   *result = paras;
   while ((*opts >= 48 && *opts <= 57) ||  /* 0 - 9 */
@@ -550,7 +603,7 @@ static Array HHVM_FUNCTION(getopt, const String& options,
 
     /* the first <len> slots are filled by the one short ops
      * we now extend our array and jump to the new added structs */
-    opts = (opt_struct *)smart_realloc(
+    opts = (opt_struct *)req::realloc(
       opts, sizeof(opt_struct) * (len + count + 1));
     orig_opts = opts;
     opts += len;
@@ -575,7 +628,7 @@ static Array HHVM_FUNCTION(getopt, const String& options,
       opts++;
     }
   } else {
-    opts = (opt_struct*) smart_realloc(opts, sizeof(opt_struct) * (len + 1));
+    opts = (opt_struct*) req::realloc(opts, sizeof(opt_struct) * (len + 1));
     orig_opts = opts;
     opts += len;
   }
@@ -588,7 +641,7 @@ static Array HHVM_FUNCTION(getopt, const String& options,
   static const StaticString s_argv("argv");
   Array vargv = php_global(s_argv).toArray();
   int argc = vargv.size();
-  char **argv = (char **)smart_malloc((argc+1) * sizeof(char*));
+  char **argv = (char **)req::malloc((argc+1) * sizeof(char*));
   std::vector<String> holders;
   int index = 0;
   for (ArrayIter iter(vargv); iter; ++iter) {
@@ -607,8 +660,8 @@ static Array HHVM_FUNCTION(getopt, const String& options,
 
   SCOPE_EXIT {
     free_longopts(orig_opts);
-    smart_free(orig_opts);
-    smart_free(argv);
+    req::free(orig_opts);
+    req::free(argv);
   };
 
   Array ret = Array::Create();
@@ -709,8 +762,24 @@ const StaticString
 static Array HHVM_FUNCTION(getrusage, int64_t who /* = 0 */) {
   struct rusage usg;
   memset(&usg, 0, sizeof(struct rusage));
+  int actual_who;
+  switch (who) {
+  case 1:
+    actual_who = RUSAGE_CHILDREN;
+    break;
+  case 2:
+#ifdef RUSAGE_THREAD
+    actual_who = RUSAGE_THREAD;
+#else
+    throw_not_supported(__func__, "RUSAGE_THREAD is not defined on this sytem");
+#endif
+    break;
+  default:
+    actual_who = RUSAGE_SELF;
+    break;
+  }
 
-  if (getrusage(who == 1 ? RUSAGE_CHILDREN : RUSAGE_SELF, &usg) == -1) {
+  if (getrusage(actual_who, &usg) == -1) {
     raise_error("getrusage returned %d: %s", errno,
       folly::errnoStr(errno).c_str());
   }
@@ -743,8 +812,8 @@ static bool HHVM_FUNCTION(clock_getres,
 #else
   struct timespec ts;
   int ret = clock_getres(clk_id, &ts);
-  sec = (int64_t)ts.tv_sec;
-  nsec = (int64_t)ts.tv_nsec;
+  sec.assignIfRef((int64_t)ts.tv_sec);
+  nsec.assignIfRef((int64_t)ts.tv_nsec);
   return ret == 0;
 #endif
 }
@@ -753,8 +822,8 @@ static bool HHVM_FUNCTION(clock_gettime,
                           int64_t clk_id, VRefParam sec, VRefParam nsec) {
   struct timespec ts;
   int ret = gettime(clk_id, &ts);
-  sec = (int64_t)ts.tv_sec;
-  nsec = (int64_t)ts.tv_nsec;
+  sec.assignIfRef((int64_t)ts.tv_sec);
+  nsec.assignIfRef((int64_t)ts.tv_nsec);
   return ret == 0;
 }
 
@@ -767,11 +836,25 @@ static String HHVM_FUNCTION(cpu_get_model) {
 }
 
 Variant HHVM_FUNCTION(ini_get, const String& varname) {
-  String value = empty_string();
+  Variant value;
   bool ret = IniSetting::Get(varname, value);
+
+  // ret will be false if varname isn't a valid ini setting
   if (!ret) {
     return false;
   }
+
+  // Anything other than array for ini_get can be converted to
+  // the expected string result for this function call.
+  // If value is null, an empty string is returned, which is good
+  // and expected.
+  if (!value.isArray()) {
+    return value.toString();
+  }
+  // For arrays, this will return an array of values. It will also return
+  // an empty array if the values for a valid collection-like configuration
+  // has not been set. This is similar to returning an empty-string for
+  // standard configuration options.
   return value;
 }
 
@@ -834,10 +917,6 @@ static bool HHVM_FUNCTION(hphp_memory_stop_interval) {
   return MM().stopStatsInterval();
 }
 
-static Variant HHVM_FUNCTION(php_ini_loaded_file) {
-  return false;
-}
-
 String HHVM_FUNCTION(php_sapi_name) {
   return RuntimeOption::ExecutionMode;
 }
@@ -873,11 +952,6 @@ Variant HHVM_FUNCTION(php_uname, const String& mode /*="" */) {
   }
 }
 
-static bool HHVM_FUNCTION(phpinfo, int64_t what /*=0 */) {
-  g_context->write("HipHop\n");
-  return false;
-}
-
 static Variant HHVM_FUNCTION(phpversion, const String& extension /*="" */) {
   Extension *ext;
 
@@ -885,7 +959,7 @@ static Variant HHVM_FUNCTION(phpversion, const String& extension /*="" */) {
     return k_PHP_VERSION;
   }
 
-  if ((ext = Extension::GetExtension(extension)) != nullptr &&
+  if ((ext = ExtensionRegistry::get(extension)) != nullptr &&
       strcmp(ext->getVersion(), NO_EXTENSION_VERSION_YET) != 0) {
     return ext->getVersion();
   }
@@ -905,23 +979,26 @@ static bool HHVM_FUNCTION(putenv, const String& setting) {
   return true;
 }
 
-static bool HHVM_FUNCTION(set_magic_quotes_runtime, bool new_setting) {
-  if (new_setting) {
-    throw_not_supported(__func__, "not using magic quotes");
-  }
-  return true;
-}
-
 static void HHVM_FUNCTION(set_time_limit, int64_t seconds) {
   ThreadInfo *info = ThreadInfo::s_threadInfo.getNoCheck();
   RequestInjectionData &data = info->m_reqInjectionData;
-  data.setTimeout(seconds);
+  if (RuntimeOption::TimeoutsUseWallTime) {
+    data.setTimeout(seconds);
+  } else {
+    data.setCPUTimeout(seconds);
+  }
 }
 
 String HHVM_FUNCTION(sys_get_temp_dir) {
+#ifdef WIN32
+  char buf[PATH_MAX];
+  auto len = GetTempPathA(PATH_MAX, buf);
+  return String(buf, len, CopyString);
+#else
   char *env = getenv("TMPDIR");
   if (env && *env) return String(env, CopyString);
   return s_SLASH_TMP;
+#endif
 }
 
 static String HHVM_FUNCTION(zend_version) {
@@ -1128,35 +1205,12 @@ Variant HHVM_FUNCTION(version_compare,
   return init_null();
 }
 
-static bool HHVM_FUNCTION(gc_enabled) {
-  return false;
-}
-
-static void HHVM_FUNCTION(gc_enable) {
-  if (RuntimeOption::EnableHipHopSyntax) {
-    raise_warning("HipHop currently does not support circular reference "
-                  "collection");
-  }
-}
-
-static void HHVM_FUNCTION(gc_disable) {
-  // we could raise a warning here, but gc_disable can be considered
-  // "successful" in that there's (still) no official GC after it's
-  // called ; and previous callers of gc_enable have already been warned.
-}
-
-static int64_t HHVM_FUNCTION(gc_collect_cycles) {
-  if (RuntimeOption::EnableHipHopSyntax) {
-    raise_warning("HipHop currently does not support circular reference "
-                  "collection");
-  }
-  return 0;
-}
 ///////////////////////////////////////////////////////////////////////////////
 
 void StandardExtension::initOptions() {
   HHVM_FE(assert_options);
   HHVM_FE(assert);
+  HHVM_FALIAS(__SystemLib\\assert, SystemLib_assert);
   HHVM_FE(dl);
   HHVM_FE(extension_loaded);
   HHVM_FE(get_loaded_extensions);
@@ -1168,8 +1222,6 @@ void StandardExtension::initOptions() {
   HHVM_FE(restore_include_path);
   HHVM_FE(set_include_path);
   HHVM_FE(get_included_files);
-  HHVM_FE(get_magic_quotes_gpc);
-  HHVM_FE(get_magic_quotes_runtime);
   HHVM_FE(getenv);
   HHVM_FE(getlastmod);
   HHVM_FE(getmygid);
@@ -1182,6 +1234,7 @@ void StandardExtension::initOptions() {
   HHVM_FE(clock_gettime);
   HHVM_FE(cpu_get_count);
   HHVM_FE(cpu_get_model);
+  HHVM_FALIAS(ini_alter, ini_set);
   HHVM_FE(ini_get);
   HHVM_FE(ini_get_all);
   HHVM_FE(ini_restore);
@@ -1193,21 +1246,14 @@ void StandardExtension::initOptions() {
   HHVM_FE(hphp_memory_get_interval_peak_usage);
   HHVM_FE(hphp_memory_start_interval);
   HHVM_FE(hphp_memory_stop_interval);
-  HHVM_FE(php_ini_loaded_file);
   HHVM_FE(php_sapi_name);
   HHVM_FE(php_uname);
-  HHVM_FE(phpinfo);
   HHVM_FE(phpversion);
   HHVM_FE(putenv);
-  HHVM_FE(set_magic_quotes_runtime);
   HHVM_FE(set_time_limit);
   HHVM_FE(sys_get_temp_dir);
   HHVM_FE(zend_version);
   HHVM_FE(version_compare);
-  HHVM_FE(gc_enabled);
-  HHVM_FE(gc_enable);
-  HHVM_FE(gc_disable);
-  HHVM_FE(gc_collect_cycles);
 
 #define INFO(v) Native::registerConstant<KindOfInt64> \
                   (makeStaticString("INFO_" #v), k_INFO_##v);
@@ -1228,11 +1274,11 @@ void StandardExtension::initOptions() {
   ASSERTCONST(BAIL);
   ASSERTCONST(WARNING);
   ASSERTCONST(QUIET_EVAL);
+  ASSERTCONST(EXCEPTION);
 #undef ASSERTCONST
 
   loadSystemlib("std_options");
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
 }
