@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -16,13 +16,28 @@
 */
 #include <zip.h>
 
-#include "hphp/runtime/base/base-includes.h"
+#include "hphp/runtime/base/array-init.h"
+#include "hphp/runtime/base/preg.h"
 #include "hphp/runtime/base/stream-wrapper-registry.h"
-#include "hphp/runtime/ext/ext_file.h"
+#include "hphp/runtime/ext/extension.h"
 #include "hphp/runtime/ext/pcre/ext_pcre.h"
+#include "hphp/runtime/ext/std/ext_std_file.h"
 
 namespace HPHP {
 
+static String to_full_path(const String& filename) {
+  if (filename.charAt(0) == '/') {
+    return filename;
+  }
+  return f_getcwd().toString() + String::FromChar('/') + filename;
+}
+
+// A wrapper for `zip_open` that prepares a full path
+// file name to consider current working directory.
+static zip* _zip_open(const String& filename, int _flags, int* zep) {
+  return zip_open(to_full_path(filename).c_str(), _flags, zep);
+}
+
 class ZipStream : public File {
  public:
   DECLARE_RESOURCE_ALLOCATION(ZipStream);
@@ -43,9 +58,9 @@ class ZipStream : public File {
 
   virtual ~ZipStream() { close(); }
 
-  virtual bool open(const String&, const String&) { return false; }
+  bool open(const String&, const String&) override { return false; }
 
-  virtual bool close() {
+  bool close() override {
     bool noError = true;
     if (!eof()) {
       if (zip_fclose(m_zipFile) != 0) {
@@ -56,7 +71,7 @@ class ZipStream : public File {
     return noError;
   }
 
-  virtual int64_t readImpl(char *buffer, int64_t length) {
+  int64_t readImpl(char *buffer, int64_t length) override {
     auto n = zip_fread(m_zipFile, buffer, length);
     if (n <= 0) {
       if (n == -1) {
@@ -68,9 +83,9 @@ class ZipStream : public File {
     return n;
   }
 
-  virtual int64_t writeImpl(const char *buffer, int64_t length) { return 0; }
+  int64_t writeImpl(const char *buffer, int64_t length) override { return 0; }
 
-  virtual bool eof() { return m_zipFile == nullptr; }
+  bool eof() override { return m_zipFile == nullptr; }
 
  private:
   zip_file* m_zipFile;
@@ -83,8 +98,10 @@ void ZipStream::sweep() {
 
 class ZipStreamWrapper : public Stream::Wrapper {
  public:
-  virtual File* open(const String& filename, const String& mode,
-                     int options, const Variant& context) {
+  virtual req::ptr<File> open(const String& filename,
+                              const String& mode,
+                              int options,
+                              const req::ptr<StreamContext>& context) {
     std::string url(filename.c_str());
     auto pound = url.find('#');
     if (pound == std::string::npos) {
@@ -100,12 +117,12 @@ class ZipStreamWrapper : public Stream::Wrapper {
     }
 
     int err;
-    auto z = zip_open(path.c_str(), 0, &err);
+    auto z = _zip_open(path, 0, &err);
     if (z == nullptr) {
       return nullptr;
     }
 
-    return NEWOBJ(ZipStream)(z, file);
+    return req::make<ZipStream>(z, file);
   }
 };
 
@@ -115,7 +132,7 @@ class ZipEntry : public SweepableResourceData {
 
   CLASSNAME_IS("ZipEntry");
   // overriding ResourceData
-  const String& o_getClassNameHook() const { return classnameof(); }
+  const String& o_getClassNameHook() const override { return classnameof(); }
 
   ZipEntry(zip* z, int index) : m_zipFile(nullptr) {
     if (zip_stat_index(z, index, 0, &m_zipStat) == 0) {
@@ -143,12 +160,12 @@ class ZipEntry : public SweepableResourceData {
   }
 
   String read(int64_t len) {
-    auto sb  = NEWOBJ(StringBuffer)(len);
-    auto buf = sb->appendCursor(len);
+    StringBuffer sb(len);
+    auto buf = sb.appendCursor(len);
     auto n   = zip_fread(m_zipFile, buf, len);
     if (n > 0) {
-      sb->resize(n);
-      return sb->detach();
+      sb.resize(n);
+      return sb.detach();
     }
     return empty_string();
   }
@@ -195,7 +212,7 @@ class ZipEntry : public SweepableResourceData {
   struct zip_stat m_zipStat;
   zip_file*       m_zipFile;
 };
-IMPLEMENT_OBJECT_ALLOCATION(ZipEntry);
+IMPLEMENT_RESOURCE_ALLOCATION(ZipEntry);
 
 class ZipDirectory: public SweepableResourceData {
  public:
@@ -203,7 +220,7 @@ class ZipDirectory: public SweepableResourceData {
 
   CLASSNAME_IS("ZipDirectory");
   // overriding ResourceData
-  const String& o_getClassNameHook() const { return classnameof(); }
+  const String& o_getClassNameHook() const override { return classnameof(); }
 
   explicit ZipDirectory(zip *z) : m_zip(z),
                                   m_numFiles(zip_get_num_files(z)),
@@ -232,7 +249,7 @@ class ZipDirectory: public SweepableResourceData {
       return false;
     }
 
-    auto zipEntry = NEWOBJ(ZipEntry)(m_zip, m_curIndex);
+    auto zipEntry = req::make<ZipEntry>(m_zip, m_curIndex);
 
     if (!zipEntry->isValid()) {
       return false;
@@ -240,7 +257,7 @@ class ZipDirectory: public SweepableResourceData {
 
     ++m_curIndex;
 
-    return Resource(zipEntry);
+    return Variant(std::move(zipEntry));
   }
 
   zip* getZip() {
@@ -252,133 +269,23 @@ class ZipDirectory: public SweepableResourceData {
   int  m_numFiles;
   int  m_curIndex;
 };
-IMPLEMENT_OBJECT_ALLOCATION(ZipDirectory);
+IMPLEMENT_RESOURCE_ALLOCATION(ZipDirectory);
 
 const StaticString s_ZipArchive("ZipArchive");
-const StaticString s_CREATE("CREATE");
-const int64_t k_CREATE = 1;
-const StaticString s_EXCL("EXCL");
-const int64_t k_EXCL = 2;
-const StaticString s_CHECKCONS("CHECKCONS");
-const int64_t k_CHECKCONS = 4;
-const StaticString s_OVERWRITE("OVERWRITE");
-const int64_t k_OVERWRITE = 8;
-const StaticString s_FL_NOCASE("FL_NOCASE");
-const int64_t k_FL_NOCASE = 1;
-const StaticString s_FL_NODIR("FL_NODIR");
-const int64_t k_FL_NODIR = 2;
-const StaticString s_FL_COMPRESSED("FL_COMPRESSED");
-const int64_t k_FL_COMPRESSED = 4;
-const StaticString s_FL_UNCHANGED("FL_UNCHANGED");
-const int64_t k_FL_UNCHANGED = 8;
-const StaticString s_FL_RECOMPRESS("FL_RECOMPRESS");
-const int64_t k_FL_RECOMPRESS = 16;
-const StaticString s_FL_ENCRYPTED("FL_ENCRYPTED");
-const int64_t k_FL_ENCRYPTED = 32;
-const StaticString s_ER_OK("ER_OK");
-const int64_t k_ER_OK = 0;
-const StaticString s_ER_MULTIDISK("ER_MULTIDISK");
-const int64_t k_ER_MULTIDISK = 1;
-const StaticString s_ER_RENAME("ER_RENAME");
-const int64_t k_ER_RENAME = 2;
-const StaticString s_ER_CLOSE("ER_CLOSE");
-const int64_t k_ER_CLOSE = 3;
-const StaticString s_ER_SEEK("ER_SEEK");
-const int64_t k_ER_SEEK = 4;
-const StaticString s_ER_READ("ER_READ");
-const int64_t k_ER_READ = 5;
-const StaticString s_ER_WRITE("ER_WRITE");
-const int64_t k_ER_WRITE = 6;
-const StaticString s_ER_CRC("ER_CRC");
-const int64_t k_ER_CRC = 7;
-const StaticString s_ER_ZIPCLOSED("ER_ZIPCLOSED");
-const int64_t k_ER_ZIPCLOSED = 8;
-const StaticString s_ER_NOENT("ER_NOENT");
-const int64_t k_ER_NOENT = 9;
-const StaticString s_ER_EXISTS("ER_EXISTS");
-const int64_t k_ER_EXISTS = 10;
-const StaticString s_ER_OPEN("ER_OPEN");
-const int64_t k_ER_OPEN = 11;
-const StaticString s_ER_TMPOPEN("ER_TMPOPEN");
-const int64_t k_ER_TMPOPEN = 12;
-const StaticString s_ER_ZLIB("ER_ZLIB");
-const int64_t k_ER_ZLIB = 13;
-const StaticString s_ER_MEMORY("ER_MEMORY");
-const int64_t k_ER_MEMORY = 14;
-const StaticString s_ER_CHANGED("ER_CHANGED");
-const int64_t k_ER_CHANGED = 15;
-const StaticString s_ER_COMPNOTSUPP("ER_COMPNOTSUPP");
-const int64_t k_ER_COMPNOTSUPP = 16;
-const StaticString s_ER_EOF("ER_EOF");
-const int64_t k_ER_EOF = 17;
-const StaticString s_ER_INVAL("ER_INVAL");
-const int64_t k_ER_INVAL = 18;
-const StaticString s_ER_NOZIP("ER_NOZIP");
-const int64_t k_ER_NOZIP = 19;
-const StaticString s_ER_INTERNAL("ER_INTERNAL");
-const int64_t k_ER_INTERNAL = 20;
-const StaticString s_ER_INCONS("ER_INCONS");
-const int64_t k_ER_INCONS = 21;
-const StaticString s_ER_REMOVE("ER_REMOVE");
-const int64_t k_ER_REMOVE = 22;
-const StaticString s_ER_DELETED("ER_DELETED");
-const int64_t k_ER_DELETED = 23;
-const StaticString s_ER_ENCRNOTSUPP("ER_ENCRNOTSUPP");
-const int64_t k_ER_ENCRNOTSUPP = 24;
-const StaticString s_ER_RDONLY("ER_RDONLY");
-const int64_t k_ER_RDONLY = 25;
-const StaticString s_ER_NOPASSWD("ER_NOPASSWD");
-const int64_t k_ER_NOPASSWD = 26;
-const StaticString s_ER_WRONGPASSWD("ER_WRONGPASSWD");
-const int64_t k_ER_WRONGPASSWD = 27;
-const StaticString s_CM_DEFAULT("CM_DEFAULT");
-const int64_t k_CM_DEFAULT = -1;
-const StaticString s_CM_STORE("CM_STORE");
-const int64_t k_CM_STORE = 0;
-const StaticString s_CM_SHRINK("CM_SHRINK");
-const int64_t k_CM_SHRINK = 1;
-const StaticString s_CM_REDUCE_1("CM_REDUCE_1");
-const int64_t k_CM_REDUCE_1 = 2;
-const StaticString s_CM_REDUCE_2("CM_REDUCE_2");
-const int64_t k_CM_REDUCE_2 = 3;
-const StaticString s_CM_REDUCE_3("CM_REDUCE_3");
-const int64_t k_CM_REDUCE_3 = 4;
-const StaticString s_CM_REDUCE_4("CM_REDUCE_4");
-const int64_t k_CM_REDUCE_4= 5;
-const StaticString s_CM_IMPLODE("CM_IMPLODE");
-const int64_t k_CM_IMPLODE = 6;
-const StaticString s_CM_DEFLATE("CM_DEFLATE");
-const int64_t k_CM_DEFLATE = 8;
-const StaticString s_CM_DEFLATE64("CM_DEFLATE64");
-const int64_t k_CM_DEFLATE64 = 9;
-const StaticString s_CM_PKWARE_IMPLODE("CM_PKWARE_IMPLODE");
-const int64_t k_CM_PKWARE_IMPLODE = 10;
-const StaticString s_CM_BZIP2("CM_BZIP2");
-const int64_t k_CM_BZIP2 = 12;
-const StaticString s_CM_LZMA("CM_LZMA");
-const int64_t k_CM_LZMA = 14;
-const StaticString s_CM_TERSE("CM_TERSE");
-const int64_t k_CM_TERSE = 18;
-const StaticString s_CM_LZ77("CM_LZ77");
-const int64_t k_CM_LZ77 = 19;
-const StaticString s_CM_WAVPACK("CM_WAVPACK");
-const int64_t k_CM_WAVPACK = 97;
-const StaticString s_CM_PPMD("CM_PPMD");
-const int64_t k_CM_PPMD = 98;
 
 template<class T>
 ALWAYS_INLINE
-static T* getResource(ObjectData* obj, const char* varName) {
-  auto var = obj->o_get(varName, true, s_ZipArchive.get());
+static req::ptr<T> getResource(ObjectData* obj, const char* varName) {
+  auto var = obj->o_get(varName, true, s_ZipArchive);
   if (var.getType() == KindOfNull) {
     return nullptr;
   }
-  return var.asCResRef().getTyped<T>();
+  return cast<T>(var);
 }
 
 ALWAYS_INLINE
 static Variant setVariable(const Object& obj, const char* varName, const Variant& varValue) {
-  return obj->o_set(varName, varValue, s_ZipArchive.get());
+  return obj->o_set(varName, varValue, s_ZipArchive);
 }
 
 #define FAIL_IF_EMPTY_STRING(func, str)                     \
@@ -413,14 +320,14 @@ static Variant setVariable(const Object& obj, const char* varName, const Variant
 #define FAIL_IF_INVALID_ZIPDIRECTORY(func, res)                         \
   if (!res->isValid()) {                                                \
     raise_warning(#func "(): %d is not a valid "                        \
-                  "Zip Directory resource", res->o_getId());            \
+                  "Zip Directory resource", res->getId());              \
     return false;                                                       \
   }
 
 #define FAIL_IF_INVALID_ZIPENTRY(func, res)                             \
   if (!res->isValid()) {                                                \
     raise_warning(#func "(): %d is not a valid Zip Entry resource",     \
-                  res->o_getId());                                      \
+                  res->getId());                                        \
     return false;                                                       \
   }
 
@@ -461,7 +368,7 @@ static Variant HHVM_METHOD(ZipArchive, getProperty, int64_t property) {
     }
     case 3:
     {
-      return this_->o_get("filename", true, s_ZipArchive.get()).asCStrRef();
+      return this_->o_get("filename", true, s_ZipArchive).asCStrRef();
     }
     case 4:
     {
@@ -507,7 +414,7 @@ static bool HHVM_METHOD(ZipArchive, addEmptyDir, const String& dirname) {
 
 static bool addFile(zip* zipStruct, const char* source, const char* dest,
                     int64_t start = 0, int64_t length = 0) {
-  if (!f_is_file(source)) {
+  if (!HHVM_FN(is_file)(source)) {
     return false;
   }
 
@@ -608,7 +515,7 @@ static bool addPattern(zip* zipStruct, const String& pattern, const Array& optio
 
   Array files;
   if (glob) {
-    auto match = f_glob(pattern, flags);
+    auto match = HHVM_FN(glob)(pattern, flags);
     if (match.isArray()) {
       files = match.toArrRef();
     } else {
@@ -618,7 +525,7 @@ static bool addPattern(zip* zipStruct, const String& pattern, const Array& optio
     if (path[path.size() - 1] != '/') {
       path.push_back('/');
     }
-    auto allFiles = f_scandir(path);
+    auto allFiles = HHVM_FN(scandir)(path);
     if (allFiles.isArray()) {
       files = allFiles.toArrRef();
     } else {
@@ -636,12 +543,12 @@ static bool addPattern(zip* zipStruct, const String& pattern, const Array& optio
     }
 
     auto source = var.asCStrRef();
-    if (f_is_dir(source)) {
+    if (HHVM_FN(is_dir)(source)) {
       continue;
     }
 
     if (!glob) {
-      auto var = HHVM_FN(preg_match)(pattern, source);
+      auto var = preg_match(pattern, source);
       if (var.isInteger()) {
         if (var.asInt64Val() == 0) {
           continue;
@@ -709,7 +616,7 @@ static bool HHVM_METHOD(ZipArchive, close) {
 
   bool ret = zipDir->close();
 
-  setVariable(this_, "zipDir", null_resource);
+  setVariable(Object{this_}, "zipDir", null_resource);
 
   return ret;
 }
@@ -752,7 +659,7 @@ static bool extractFileTo(zip* zip, const std::string &file, std::string& to,
   auto sep = file.rfind('/');
   if (sep != std::string::npos) {
     auto path = to + file.substr(0, sep);
-    if (!f_is_dir(path) && !f_mkdir(path, 0777, true)) {
+    if (!HHVM_FN(is_dir)(path) && !HHVM_FN(mkdir)(path, 0777, true)) {
       return false;
     }
 
@@ -812,7 +719,7 @@ static bool HHVM_METHOD(ZipArchive, extractTo, const String& destination,
     to.push_back('/');
   }
 
-  if (!f_is_dir(to) && !f_mkdir(to)) {
+  if (!HHVM_FN(is_dir)(to) && !HHVM_FN(mkdir)(to)) {
     return false;
   }
 
@@ -930,12 +837,12 @@ static Variant HHVM_METHOD(ZipArchive, getFromIndex, int64_t index,
     length = zipStat.size;
   }
 
-  auto sb  = NEWOBJ(StringBuffer)(length);
-  auto buf = sb->appendCursor(length);
+  StringBuffer sb(length);
+  auto buf = sb.appendCursor(length);
   auto n   = zip_fread(zipFile, buf, length);
   if (n > 0) {
-    sb->resize(n);
-    return sb->detach();
+    sb.resize(n);
+    return sb.detach();
   }
   return empty_string_variant();
 }
@@ -967,12 +874,12 @@ static Variant HHVM_METHOD(ZipArchive, getFromName, const String& name,
     length = zipStat.size;
   }
 
-  auto sb  = NEWOBJ(StringBuffer)(length);
-  auto buf = sb->appendCursor(length);
+  StringBuffer sb(length);
+  auto buf = sb.appendCursor(length);
   auto n   = zip_fread(zipFile, buf, length);
   if (n > 0) {
-    sb->resize(n);
-    return sb->detach();
+    sb.resize(n);
+    return sb.detach();
   }
   return empty_string_variant();
 }
@@ -1010,13 +917,11 @@ static Variant HHVM_METHOD(ZipArchive, getStream, const String& name) {
   FAIL_IF_INVALID_ZIPARCHIVE(getStream, zipDir);
   FAIL_IF_EMPTY_STRING_ZIPARCHIVE(getStream, name);
 
-  auto zipStream    = NEWOBJ(ZipStream)(zipDir->getZip(), name);
-  auto zipStreamRes = Resource(zipStream);
+  auto zipStream = req::make<ZipStream>(zipDir->getZip(), name);
   if (zipStream->eof()) {
     return false;
   }
-
-  return zipStreamRes;
+  return Variant(std::move(zipStream));
 }
 
 static Variant HHVM_METHOD(ZipArchive, locateName, const String& name,
@@ -1037,15 +942,15 @@ static Variant HHVM_METHOD(ZipArchive, open, const String& filename,
   FAIL_IF_EMPTY_STRING_ZIPARCHIVE(open, filename);
 
   int  err;
-  auto z = zip_open(filename.c_str(), flags, &err);
+  auto z = _zip_open(filename, flags, &err);
   if (z == nullptr) {
     return err;
   }
 
-  auto zipDir = NEWOBJ(ZipDirectory)(z);
+  auto zipDir = req::make<ZipDirectory>(z);
 
-  setVariable(this_, "zipDir", Resource(zipDir));
-  setVariable(this_, "filename", filename);
+  setVariable(Object{this_}, "zipDir", Variant(zipDir));
+  setVariable(Object{this_}, "filename", filename);
 
   zip_error_clear(zipDir->getZip());
   return true;
@@ -1258,7 +1163,7 @@ static bool HHVM_METHOD(ZipArchive, unchangeName, const String& name) {
 // functions
 
 static Variant HHVM_FUNCTION(zip_close, const Resource& zip) {
-  auto zipDir = zip.getTyped<ZipDirectory>();
+  auto zipDir = cast<ZipDirectory>(zip);
 
   FAIL_IF_INVALID_ZIPDIRECTORY(zip_close, zipDir);
 
@@ -1268,7 +1173,7 @@ static Variant HHVM_FUNCTION(zip_close, const Resource& zip) {
 }
 
 static bool HHVM_FUNCTION(zip_entry_close, const Resource& zip_entry) {
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_close, zipEntry);
 
@@ -1276,7 +1181,7 @@ static bool HHVM_FUNCTION(zip_entry_close, const Resource& zip_entry) {
 }
 
 static Variant HHVM_FUNCTION(zip_entry_compressedsize, const Resource& zip_entry) {
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_compressedsize, zipEntry);
 
@@ -1284,7 +1189,7 @@ static Variant HHVM_FUNCTION(zip_entry_compressedsize, const Resource& zip_entry
 }
 
 static Variant HHVM_FUNCTION(zip_entry_compressionmethod, const Resource& zip_entry) {
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_compressionmethod, zipEntry);
 
@@ -1292,7 +1197,7 @@ static Variant HHVM_FUNCTION(zip_entry_compressionmethod, const Resource& zip_en
 }
 
 static Variant HHVM_FUNCTION(zip_entry_filesize, const Resource& zip_entry) {
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_filesize, zipEntry);
 
@@ -1300,7 +1205,7 @@ static Variant HHVM_FUNCTION(zip_entry_filesize, const Resource& zip_entry) {
 }
 
 static Variant HHVM_FUNCTION(zip_entry_name, const Resource& zip_entry) {
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_name, zipEntry);
 
@@ -1309,8 +1214,8 @@ static Variant HHVM_FUNCTION(zip_entry_name, const Resource& zip_entry) {
 
 static bool HHVM_FUNCTION(zip_entry_open, const Resource& zip, const Resource& zip_entry,
                           const String& mode) {
-  auto zipDir   = zip.getTyped<ZipDirectory>();
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipDir   = cast<ZipDirectory>(zip);
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPDIRECTORY(zip_entry_open, zipDir);
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_open, zipEntry);
@@ -1321,7 +1226,7 @@ static bool HHVM_FUNCTION(zip_entry_open, const Resource& zip, const Resource& z
 
 static Variant HHVM_FUNCTION(zip_entry_read, const Resource& zip_entry,
                              int64_t length) {
-  auto zipEntry = zip_entry.getTyped<ZipEntry>();
+  auto zipEntry = cast<ZipEntry>(zip_entry);
 
   FAIL_IF_INVALID_ZIPENTRY(zip_entry_read, zipEntry);
 
@@ -1332,18 +1237,16 @@ static Variant HHVM_FUNCTION(zip_open, const String& filename) {
   FAIL_IF_EMPTY_STRING(zip_open, filename);
 
   int  err;
-  auto z = zip_open(filename.c_str(), 0, &err);
+  auto z = _zip_open(filename, 0, &err);
   if (z == nullptr) {
     return err;
   }
 
-  auto zipDir = NEWOBJ(ZipDirectory)(z);
-
-  return Resource(zipDir);
+  return Variant(req::make<ZipDirectory>(z));
 }
 
 static Variant HHVM_FUNCTION(zip_read, const Resource& zip) {
-  auto zipDir = zip.getTyped<ZipDirectory>();
+  auto zipDir = cast<ZipDirectory>(zip);
 
   FAIL_IF_INVALID_ZIPDIRECTORY(zip_read, zipDir);
 
@@ -1352,10 +1255,10 @@ static Variant HHVM_FUNCTION(zip_read, const Resource& zip) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-class zipExtension : public Extension {
+class zipExtension final : public Extension {
  public:
   zipExtension() : Extension("zip", "1.12.4-dev") {}
-  virtual void moduleInit() {
+  void moduleInit() override {
     HHVM_ME(ZipArchive, getProperty);
     HHVM_ME(ZipArchive, addEmptyDir);
     HHVM_ME(ZipArchive, addFile);
@@ -1387,145 +1290,62 @@ class zipExtension : public Extension {
     HHVM_ME(ZipArchive, unchangeArchive);
     HHVM_ME(ZipArchive, unchangeIndex);
     HHVM_ME(ZipArchive, unchangeName);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CREATE.get(), k_CREATE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_EXCL.get(), k_EXCL);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CHECKCONS.get(), k_CHECKCONS);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_OVERWRITE.get(), k_OVERWRITE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_FL_NOCASE.get(), k_FL_NOCASE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_FL_NODIR.get(), k_FL_NODIR);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_FL_COMPRESSED.get(),
-                                               k_FL_COMPRESSED);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_FL_UNCHANGED.get(),
-                                               k_FL_UNCHANGED);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_FL_RECOMPRESS.get(),
-                                               k_FL_RECOMPRESS);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_FL_ENCRYPTED.get(),
-                                               k_FL_ENCRYPTED);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_OK.get(), k_ER_OK);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_MULTIDISK.get(),
-                                               k_ER_MULTIDISK);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_RENAME.get(), k_ER_RENAME);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_CLOSE.get(), k_ER_CLOSE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_SEEK.get(), k_ER_SEEK);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_READ.get(), k_ER_READ);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_WRITE.get(), k_ER_WRITE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_CRC.get(), k_ER_CRC);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_ZIPCLOSED.get(),
-                                               k_ER_ZIPCLOSED);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_NOENT.get(), k_ER_NOENT);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_EXISTS.get(), k_ER_EXISTS);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_OPEN.get(), k_ER_OPEN);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_TMPOPEN.get(),
-                                               k_ER_TMPOPEN);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_ZLIB.get(), k_ER_ZLIB);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_MEMORY.get(), k_ER_MEMORY);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_CHANGED.get(),
-                                               k_ER_CHANGED);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_COMPNOTSUPP.get(),
-                                               k_ER_COMPNOTSUPP);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_EOF.get(), k_ER_EOF);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_INVAL.get(), k_ER_INVAL);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_NOZIP.get(), k_ER_NOZIP);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_INTERNAL.get(),
-                                               k_ER_INTERNAL);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_INCONS.get(), k_ER_INCONS);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_REMOVE.get(), k_ER_REMOVE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_DELETED.get(),
-                                               k_ER_DELETED);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_ENCRNOTSUPP.get(),
-                                               k_ER_ENCRNOTSUPP);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_RDONLY.get(), k_ER_RDONLY);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_NOPASSWD.get(),
-                                               k_ER_NOPASSWD);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_ER_WRONGPASSWD.get(),
-                                               k_ER_WRONGPASSWD);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_DEFAULT.get(),
-                                               k_CM_DEFAULT);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_STORE.get(), k_CM_STORE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_SHRINK.get(), k_CM_SHRINK);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_REDUCE_1.get(),
-                                               k_CM_REDUCE_1);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_REDUCE_2.get(),
-                                               k_CM_REDUCE_2);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_REDUCE_3.get(),
-                                               k_CM_REDUCE_3);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_REDUCE_4.get(),
-                                               k_CM_REDUCE_4);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_IMPLODE.get(),
-                                               k_CM_IMPLODE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_DEFLATE.get(),
-                                               k_CM_DEFLATE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_DEFLATE64.get(),
-                                               k_CM_DEFLATE64);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_PKWARE_IMPLODE.get(),
-                                               k_CM_PKWARE_IMPLODE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_BZIP2.get(),
-                                               k_CM_BZIP2);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_LZMA.get(),
-                                               k_CM_LZMA);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_TERSE.get(),
-                                               k_CM_TERSE);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_LZ77.get(),
-                                               k_CM_LZ77);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_WAVPACK.get(),
-                                               k_CM_WAVPACK);
-    Native::registerClassConstant<KindOfInt64>(s_ZipArchive.get(),
-                                               s_CM_PPMD.get(),
-                                               k_CM_PPMD);
+
+    HHVM_RCC_INT(ZipArchive, CREATE, ZIP_CREATE);
+    HHVM_RCC_INT(ZipArchive, EXCL, ZIP_EXCL);
+    HHVM_RCC_INT(ZipArchive, CHECKCONS, ZIP_CHECKCONS);
+    HHVM_RCC_INT(ZipArchive, OVERWRITE, ZIP_TRUNCATE);
+    HHVM_RCC_INT(ZipArchive, FL_NOCASE, ZIP_FL_NOCASE);
+    HHVM_RCC_INT(ZipArchive, FL_NODIR, ZIP_FL_NODIR);
+    HHVM_RCC_INT(ZipArchive, FL_COMPRESSED, ZIP_FL_COMPRESSED);
+    HHVM_RCC_INT(ZipArchive, FL_UNCHANGED, ZIP_FL_UNCHANGED);
+    HHVM_RCC_INT(ZipArchive, FL_RECOMPRESS, ZIP_FL_RECOMPRESS);
+    HHVM_RCC_INT(ZipArchive, FL_ENCRYPTED, ZIP_FL_ENCRYPTED);
+    HHVM_RCC_INT(ZipArchive, ER_OK, ZIP_ER_OK);
+    HHVM_RCC_INT(ZipArchive, ER_MULTIDISK, ZIP_ER_MULTIDISK);
+    HHVM_RCC_INT(ZipArchive, ER_RENAME, ZIP_ER_RENAME);
+    HHVM_RCC_INT(ZipArchive, ER_CLOSE, ZIP_ER_CLOSE);
+    HHVM_RCC_INT(ZipArchive, ER_SEEK, ZIP_ER_SEEK);
+    HHVM_RCC_INT(ZipArchive, ER_READ, ZIP_ER_READ);
+    HHVM_RCC_INT(ZipArchive, ER_WRITE, ZIP_ER_WRITE);
+    HHVM_RCC_INT(ZipArchive, ER_CRC, ZIP_ER_CRC);
+    HHVM_RCC_INT(ZipArchive, ER_ZIPCLOSED, ZIP_ER_ZIPCLOSED);
+    HHVM_RCC_INT(ZipArchive, ER_NOENT, ZIP_ER_NOENT);
+    HHVM_RCC_INT(ZipArchive, ER_EXISTS, ZIP_ER_EXISTS);
+    HHVM_RCC_INT(ZipArchive, ER_OPEN, ZIP_ER_OPEN);
+    HHVM_RCC_INT(ZipArchive, ER_TMPOPEN, ZIP_ER_TMPOPEN);
+    HHVM_RCC_INT(ZipArchive, ER_ZLIB, ZIP_ER_ZLIB);
+    HHVM_RCC_INT(ZipArchive, ER_MEMORY, ZIP_ER_MEMORY);
+    HHVM_RCC_INT(ZipArchive, ER_CHANGED, ZIP_ER_CHANGED);
+    HHVM_RCC_INT(ZipArchive, ER_COMPNOTSUPP, ZIP_ER_COMPNOTSUPP);
+    HHVM_RCC_INT(ZipArchive, ER_EOF, ZIP_ER_EOF);
+    HHVM_RCC_INT(ZipArchive, ER_INVAL, ZIP_ER_INVAL);
+    HHVM_RCC_INT(ZipArchive, ER_NOZIP, ZIP_ER_NOZIP);
+    HHVM_RCC_INT(ZipArchive, ER_INTERNAL, ZIP_ER_INTERNAL);
+    HHVM_RCC_INT(ZipArchive, ER_INCONS, ZIP_ER_INCONS);
+    HHVM_RCC_INT(ZipArchive, ER_REMOVE, ZIP_ER_REMOVE);
+    HHVM_RCC_INT(ZipArchive, ER_DELETED, ZIP_ER_DELETED);
+    HHVM_RCC_INT(ZipArchive, ER_ENCRNOTSUPP, ZIP_ER_ENCRNOTSUPP);
+    HHVM_RCC_INT(ZipArchive, ER_RDONLY, ZIP_ER_RDONLY);
+    HHVM_RCC_INT(ZipArchive, ER_NOPASSWD, ZIP_ER_NOPASSWD);
+    HHVM_RCC_INT(ZipArchive, ER_WRONGPASSWD, ZIP_ER_WRONGPASSWD);
+    HHVM_RCC_INT(ZipArchive, CM_DEFAULT, ZIP_CM_DEFAULT);
+    HHVM_RCC_INT(ZipArchive, CM_STORE, ZIP_CM_STORE);
+    HHVM_RCC_INT(ZipArchive, CM_SHRINK, ZIP_CM_SHRINK);
+    HHVM_RCC_INT(ZipArchive, CM_REDUCE_1, ZIP_CM_REDUCE_1);
+    HHVM_RCC_INT(ZipArchive, CM_REDUCE_2, ZIP_CM_REDUCE_2);
+    HHVM_RCC_INT(ZipArchive, CM_REDUCE_3, ZIP_CM_REDUCE_3);
+    HHVM_RCC_INT(ZipArchive, CM_REDUCE_4, ZIP_CM_REDUCE_4);
+    HHVM_RCC_INT(ZipArchive, CM_IMPLODE, ZIP_CM_IMPLODE);
+    HHVM_RCC_INT(ZipArchive, CM_DEFLATE, ZIP_CM_DEFLATE);
+    HHVM_RCC_INT(ZipArchive, CM_DEFLATE64, ZIP_CM_DEFLATE64);
+    HHVM_RCC_INT(ZipArchive, CM_PKWARE_IMPLODE, ZIP_CM_PKWARE_IMPLODE);
+    HHVM_RCC_INT(ZipArchive, CM_BZIP2, ZIP_CM_BZIP2);
+    HHVM_RCC_INT(ZipArchive, CM_LZMA, ZIP_CM_LZMA);
+    HHVM_RCC_INT(ZipArchive, CM_TERSE, ZIP_CM_TERSE);
+    HHVM_RCC_INT(ZipArchive, CM_LZ77, ZIP_CM_LZ77);
+    HHVM_RCC_INT(ZipArchive, CM_WAVPACK, ZIP_CM_WAVPACK);
+    HHVM_RCC_INT(ZipArchive, CM_PPMD, ZIP_CM_PPMD);
 
     HHVM_FE(zip_close);
     HHVM_FE(zip_entry_close);
@@ -1538,8 +1358,9 @@ class zipExtension : public Extension {
     HHVM_FE(zip_open);
     HHVM_FE(zip_read);
 
-    auto wrapper = NEWOBJ(ZipStreamWrapper)();
+    auto wrapper = new ZipStreamWrapper();
     if (wrapper == nullptr || !Stream::registerWrapper("zip", wrapper)) {
+      delete wrapper;
       raise_warning("Couldn't register Zip wrapper");
     }
 
