@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -16,6 +16,7 @@
 */
 
 #include "hphp/runtime/ext/zlib/ext_zlib.h"
+#include "hphp/runtime/base/comparisons.h"
 #include "hphp/runtime/base/file.h"
 #include "hphp/runtime/base/mem-file.h"
 #include "hphp/runtime/ext/zlib/zip-file.h"
@@ -25,7 +26,7 @@
 #include "hphp/runtime/vm/native-data.h"
 #include "hphp/util/compression.h"
 #include "hphp/util/logger.h"
-#include "folly/String.h"
+#include <folly/String.h>
 #ifdef HAVE_SNAPPY
 #include <snappy.h>
 #endif
@@ -46,8 +47,10 @@ namespace {
 
 static class ZlibStreamWrapper : public Stream::Wrapper {
  public:
-  virtual File* open(const String& filename, const String& mode,
-                     int options, const Variant& context) {
+  virtual req::ptr<File> open(const String& filename,
+                              const String& mode,
+                              int options,
+                              const req::ptr<StreamContext>& context) {
     String fname;
     static const char cz[] = "compress.zlib://";
 
@@ -56,13 +59,13 @@ static class ZlibStreamWrapper : public Stream::Wrapper {
     } else if (!strncmp(filename.data(), cz, sizeof(cz) - 1)) {
       fname = filename.substr(sizeof(cz) - 1);
     } else {
-      return NULL;
+      return nullptr;
     }
 
     String translated;
     if (fname.find("://") == -1) {
       translated = File::TranslatePath(fname);
-      if (MemFile* file = FileStreamWrapper::openFromCache(translated, mode)) {
+      if (auto file = FileStreamWrapper::openFromCache(translated, mode)) {
         file->unzip();
         return file;
       }
@@ -70,13 +73,13 @@ static class ZlibStreamWrapper : public Stream::Wrapper {
       translated = fname;
     }
 
-    std::unique_ptr<ZipFile> file(NEWOBJ(ZipFile)());
+    auto file = req::make<ZipFile>();
     bool ret = file->open(translated, mode);
     if (!ret) {
       raise_warning("%s", file->getLastError().c_str());
-      return NULL;
+      return nullptr;
     }
-    return file.release();
+    return file;
   }
 } s_zlib_stream_wrapper;
 
@@ -84,14 +87,14 @@ static class ZlibStreamWrapper : public Stream::Wrapper {
 
 namespace HPHP {
 
-const long k_ZLIB_ENCODING_RAW     = -MAX_WBITS;
-const long k_ZLIB_ENCODING_GZIP    = 0x1f;
-const long k_ZLIB_ENCODING_DEFLATE = 0x0f;
+const int64_t k_ZLIB_ENCODING_RAW     = -MAX_WBITS;
+const int64_t k_ZLIB_ENCODING_GZIP    = 0x1f;
+const int64_t k_ZLIB_ENCODING_DEFLATE = 0x0f;
 
-const long k_ZLIB_ENCODING_ANY     = 0x2f;
+const int64_t k_ZLIB_ENCODING_ANY     = 0x2f;
 
-const long k_FORCE_GZIP            = k_ZLIB_ENCODING_GZIP;
-const long k_FORCE_DEFLATE         = k_ZLIB_ENCODING_DEFLATE;
+const int64_t k_FORCE_GZIP            = k_ZLIB_ENCODING_GZIP;
+const int64_t k_FORCE_DEFLATE         = k_ZLIB_ENCODING_DEFLATE;
 
 ///////////////////////////////////////////////////////////////////////////////
 // zlib functions
@@ -127,11 +130,11 @@ inline size_t hhvm_zlib_buffer_size_guess(size_t inlen) {
 }
 
 static voidpf hhvm_zlib_alloc(voidpf opaque, uInt items, uInt size) {
-  return (voidpf)smart_malloc(items * size);
+  return (voidpf)req::malloc(items * size);
 }
 
 static void hhvm_zlib_free(voidpf opaque, voidpf address) {
-  smart_free((void*)address);
+  req::free((void*)address);
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -167,9 +170,9 @@ static Variant hhvm_zlib_encode(const String& data,
     String ret(outlen, ReserveString);
 
     Z.next_in = (Bytef *) data.c_str();
-    Z.next_out = (Bytef *) ret.bufferSlice().ptr;
+    Z.next_out = (Bytef *) ret.mutableData();
     Z.avail_in = data.size();
-    Z.avail_out = ret.get()->capacity();
+    Z.avail_out = ret.capacity(); // not counting null terminator
 
     if (Z_STREAM_END == (status = deflate(&Z, Z_FINISH))) {
       ret.setSize(Z.total_out);
@@ -223,11 +226,12 @@ static String hhvm_zlib_inflate_rounds(z_stream *Z, int64_t maxlen,
       break;
     }
 
-    char *retbuf = ret.reserve(retsize + 1).ptr;
-    Z->avail_out = (ret.get()->capacity() - retused) - 1;
+    auto const ms = ret.reserve(retsize + 1);
+    auto retbuf = ms.data();
+    Z->avail_out = ms.size() - retused;
     Z->next_out = (Bytef *) (retbuf + retused);
     status = inflate(Z, Z_NO_FLUSH);
-    retused = (ret.get()->capacity() - 1) - Z->avail_out;
+    retused = ms.size() - Z->avail_out;
     ret.setSize(retused);
 
     retsize += (retsize >> 3) + 1;
@@ -309,194 +313,57 @@ String HHVM_FUNCTION(zlib_get_coding_type) {
 
 Variant HHVM_FUNCTION(gzopen, const String& filename, const String& mode,
                               int64_t use_include_path /* = 0 */) {
-  File *file = NEWOBJ(ZipFile)();
-  Resource handle(file);
+  auto file = req::make<ZipFile>();
   bool ret = file->open(File::TranslatePath(filename), mode);
   if (!ret) {
     return false;
   }
-  return handle;
+  return Variant(std::move(file));
 }
 
 bool HHVM_FUNCTION(gzclose, const Resource& zp) {
-  return f_fclose(zp);
+  return HHVM_FN(fclose)(zp);
 }
 Variant HHVM_FUNCTION(gzread, const Resource& zp, int64_t length /* = 0 */) {
-  return f_fread(zp, length);
+  return HHVM_FN(fread)(zp, length);
 }
 Variant HHVM_FUNCTION(gzseek, const Resource& zp, int64_t offset,
                               int64_t whence /* = k_SEEK_SET */) {
-  return f_fseek(zp, offset, whence);
+  return HHVM_FN(fseek)(zp, offset, whence);
 }
 Variant HHVM_FUNCTION(gztell, const Resource& zp) {
-  return f_ftell(zp);
+  return HHVM_FN(ftell)(zp);
 }
 bool HHVM_FUNCTION(gzeof, const Resource& zp) {
-  return f_feof(zp);
+  return HHVM_FN(feof)(zp);
 }
 bool HHVM_FUNCTION(gzrewind, const Resource& zp) {
-  return f_rewind(zp);
+  return HHVM_FN(rewind)(zp);
 }
 Variant HHVM_FUNCTION(gzgetc, const Resource& zp) {
-  return f_fgetc(zp);
+  return HHVM_FN(fgetc)(zp);
 }
 Variant HHVM_FUNCTION(gzgets, const Resource& zp, int64_t length /* = 1024 */) {
-  return f_fgets(zp, length);
+  return HHVM_FN(fgets)(zp, length);
 }
 Variant HHVM_FUNCTION(gzgetss, const Resource& zp, int64_t length /* = 0 */,
                             const String& allowable_tags /* = null_string */) {
-  return f_fgetss(zp, length, allowable_tags);
+  return HHVM_FN(fgetss)(zp, length, allowable_tags);
 }
 Variant HHVM_FUNCTION(gzpassthru, const Resource& zp) {
-  return f_fpassthru(zp);
+  return HHVM_FN(fpassthru)(zp);
 }
 Variant HHVM_FUNCTION(gzwrite, const Resource& zp, const String& str,
                                int64_t length /* = 0 */) {
-  return f_fwrite(zp, str, length);
+  return HHVM_FN(fwrite)(zp, str, length);
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// QuickLZ functions
-
-#ifdef HAVE_QUICKLZ
-
-#define QLZ_MEMORY_SAFE
-
-namespace QuickLZ1 {
-#ifdef QLZ_COMPRESSION_LEVEL
-#undef QLZ_COMPRESSION_LEVEL
-#endif
-#ifdef QLZ_STREAMING_BUFFER
-#undef QLZ_STREAMING_BUFFER
-#endif
-#define QLZ_COMPRESSION_LEVEL 1
-#define QLZ_STREAMING_BUFFER 0
-#include "hphp/runtime/ext/zlib/quicklz.inc"
-}
-
-namespace QuickLZ2 {
-#ifdef QLZ_COMPRESSION_LEVEL
-#undef QLZ_COMPRESSION_LEVEL
-#endif
-#ifdef QLZ_STREAMING_BUFFER
-#undef QLZ_STREAMING_BUFFER
-#endif
-#define QLZ_COMPRESSION_LEVEL 2
-#define QLZ_STREAMING_BUFFER 100000
-#include "hphp/runtime/ext/zlib/quicklz.inc"
-}
-
-namespace QuickLZ3 {
-#ifdef QLZ_COMPRESSION_LEVEL
-#undef QLZ_COMPRESSION_LEVEL
-#endif
-#ifdef QLZ_STREAMING_BUFFER
-#undef QLZ_STREAMING_BUFFER
-#endif
-#define QLZ_COMPRESSION_LEVEL 3
-#define QLZ_STREAMING_BUFFER 1000000
-#include "hphp/runtime/ext/zlib/quicklz.inc"
-}
-
-Variant HHVM_FUNCTION(qlzcompress, const String& data, int level /* = 1 */) {
-  if (level < 1 || level > 3) {
-    throw_invalid_argument("level: %d", level);
-    return false;
-  }
-
-  String str(data.size() + 400, ReserveString);
-  char* compressed = str.bufferSlice().ptr;
-  size_t size = 0;
-
-  switch (level) {
-    case 1: {
-      QuickLZ1::qlz_state_compress state;
-      memset(&state, 0, sizeof(state));
-      size = QuickLZ1::qlz_compress(data.data(), compressed, data.size(),
-                                    &state);
-      break;
-    }
-    case 2: {
-      QuickLZ2::qlz_state_compress state;
-      memset(&state, 0, sizeof(state));
-      size = QuickLZ2::qlz_compress(data.data(), compressed, data.size(),
-                                    &state);
-      break;
-    }
-    case 3:
-      QuickLZ3::qlz_state_compress *state = new QuickLZ3::qlz_state_compress();
-      memset(state, 0, sizeof(*state));
-      size = QuickLZ3::qlz_compress(data.data(), compressed, data.size(),
-                                    state);
-      delete state;
-      break;
-  }
-
-  assert(size <= (size_t)data.size() + 400);
-  str.shrink(size);
-  return str;
-}
-
-Variant HHVM_FUNCTION(qlzuncompress, const String& data, int level /* = 1 */) {
-  if (level < 1 || level > 3) {
-    throw_invalid_argument("level: %d", level);
-    return false;
-  }
-
-  if (data.size() < 9) {
-    raise_notice("passing invalid data to qlzuncompress()");
-    return false;
-  }
-
-  size_t size = QuickLZ1::qlz_size_decompressed(data.data());
-  if ((int64_t)size < 0 ||
-      (RuntimeOption::SerializationSizeLimit > 0 &&
-       (int64_t)size > RuntimeOption::SerializationSizeLimit)) {
-    raise_notice("invalid size in compressed header: %zd", size);
-    return false;
-  }
-
-  String s = String(size, ReserveString);
-  char *decompressed = s.bufferSlice().ptr;
-  size_t dsize = 0;
-
-  switch (level) {
-    case 1: {
-      QuickLZ1::qlz_state_decompress state;
-      memset(&state, 0, sizeof(state));
-      dsize = QuickLZ1::qlz_decompress(data.data(), decompressed, &state);
-      break;
-    }
-    case 2: {
-      QuickLZ2::qlz_state_decompress state;
-      memset(&state, 0, sizeof(state));
-      dsize = QuickLZ2::qlz_decompress(data.data(), decompressed, &state);
-      break;
-    }
-    case 3: {
-      QuickLZ3::qlz_state_decompress *state =
-        new QuickLZ3::qlz_state_decompress();
-      memset(state, 0, sizeof(*state));
-      dsize = QuickLZ3::qlz_decompress(data.data(), decompressed, state);
-      delete state;
-      break;
-    }
-  }
-
-  if (dsize != size) {
-    return false;
-  }
-  s.setSize(dsize);
-  return s;
-}
-#endif // HAVE_QUICKLZ
-
 ///////////////////////////////////////////////////////////////////////////////
 // Snappy functions
 
 #ifdef HAVE_SNAPPY
 
-Variant HHVM_FUNCTION(sncompress, const String& data) {
+Variant HHVM_FUNCTION(snappy_compress, const String& data) {
   size_t size;
   char *compressed =
     (char *)malloc(snappy::MaxCompressedLength(data.size()) + 1);
@@ -507,12 +374,12 @@ Variant HHVM_FUNCTION(sncompress, const String& data) {
   return String(compressed, size, AttachString);
 }
 
-Variant HHVM_FUNCTION(snuncompress, const String& data) {
+Variant HHVM_FUNCTION(snappy_uncompress, const String& data) {
   size_t dsize;
 
   snappy::GetUncompressedLength(data.data(), data.size(), &dsize);
   String s = String(dsize, ReserveString);
-  char *uncompressed = s.bufferSlice().ptr;
+  char *uncompressed = s.mutableData();
 
   if (!snappy::RawUncompress(data.data(), data.size(), uncompressed)) {
     return false;
@@ -537,9 +404,9 @@ typedef struct nzlib_format_s {
 } nzlib_format_t;
 
 Variant HHVM_FUNCTION(nzcompress, const String& uncompressed) {
-  size_t len = compressBound(uncompressed.size());
+  uLong len = compressBound(uncompressed.size());
   String str(sizeof(nzlib_format_t) + len, ReserveString);
-  nzlib_format_t* format = (nzlib_format_t*)str.bufferSlice().ptr;
+  nzlib_format_t* format = (nzlib_format_t*)str.mutableData();
 
   format->magic = htonl(NZLIB_MAGIC);
   format->uncompressed_sz = htonl(uncompressed.size());
@@ -563,13 +430,13 @@ Variant HHVM_FUNCTION(nzuncompress, const String& compressed) {
     return false;
   }
 
-  size_t len = ntohl(format->uncompressed_sz);
+  uLong len = ntohl(format->uncompressed_sz);
   if (len == 0) {
     return empty_string_variant();
   }
 
   String str(len, ReserveString);
-  char* uncompressed = str.bufferSlice().ptr;
+  char* uncompressed = str.mutableData();
   int rc = uncompress((Bytef*)uncompressed, &len, format->buf,
                       compressed.size() - sizeof(*format));
   if (rc != Z_OK) {
@@ -622,7 +489,11 @@ int VarintDecode(const char** src, int max_size) {
   return val;
 }
 
-Variant HHVM_FUNCTION(lz4compress, const String& uncompressed) {
+Variant HHVM_FUNCTION(lz4_compress, const String& uncompressed,
+                                    bool high /* = false */) {
+  if (high) {
+    return f_lz4_hccompress(uncompressed);
+  }
   int bufsize = LZ4_compressBound(uncompressed.size());
   if (bufsize < 0) {
     return false;
@@ -630,7 +501,7 @@ Variant HHVM_FUNCTION(lz4compress, const String& uncompressed) {
   int headerSize = VarintSize(uncompressed.size());
   bufsize += headerSize;  // for the header
   String s = String(bufsize, ReserveString);
-  char *compressed = s.bufferSlice().ptr;
+  char *compressed = s.mutableData();
 
   VarintEncode(uncompressed.size(), &compressed);  // write the header
 
@@ -644,7 +515,7 @@ Variant HHVM_FUNCTION(lz4compress, const String& uncompressed) {
   return s;
 }
 
-Variant HHVM_FUNCTION(lz4hccompress, const String& uncompressed) {
+Variant HHVM_FUNCTION(lz4_hccompress, const String& uncompressed) {
   int bufsize = LZ4_compressBound(uncompressed.size());
   if (bufsize < 0) {
     return false;
@@ -652,7 +523,7 @@ Variant HHVM_FUNCTION(lz4hccompress, const String& uncompressed) {
   int headerSize = VarintSize(uncompressed.size());
   bufsize += headerSize;  // for the header
   String s = String(bufsize, ReserveString);
-  char *compressed = s.bufferSlice().ptr;
+  char *compressed = s.mutableData();
 
   VarintEncode(uncompressed.size(), &compressed);  // write the header
 
@@ -666,20 +537,30 @@ Variant HHVM_FUNCTION(lz4hccompress, const String& uncompressed) {
   return s;
 }
 
-Variant HHVM_FUNCTION(lz4uncompress, const String& compressed) {
+Variant HHVM_FUNCTION(lz4_uncompress, const String& compressed) {
   const char* compressed_ptr = compressed.data();
   int dsize = VarintDecode(&compressed_ptr, compressed.size());
   if (dsize < 0) {
     return false;
   }
 
+  int inSize = compressed.size() - (compressed_ptr - compressed.data());
   String s = String(dsize, ReserveString);
-  char *uncompressed = s.bufferSlice().ptr;
-  int ret = LZ4_uncompress(compressed_ptr, uncompressed, dsize);
+  char *uncompressed = s.mutableData();
+#ifdef LZ4_MAX_INPUT_SIZE
+  int ret = LZ4_decompress_safe(compressed_ptr, uncompressed, inSize, dsize);
 
-  if (ret <= 0) {
+  if (ret != dsize) {
     return false;
   }
+#else
+  int ret = LZ4_decompress_fast(compressed_ptr, uncompressed, dsize);
+
+  if (ret <= 0 || ret > inSize) {
+    return false;
+  }
+#endif
+
   s.setSize(dsize);
   return s;
 }
@@ -722,7 +603,7 @@ class __SystemLib_ChunkedInflator {
     do {
       int buffer_length = chunk.length() * (1 << factor);
       String buffer(buffer_length, ReserveString);
-      char* raw = buffer.bufferSlice().ptr;
+      char* raw = buffer.mutableData();
       m_zstream.next_out = (Bytef*) raw;
       m_zstream.avail_out = buffer_length;
 
@@ -767,22 +648,19 @@ String HHVM_METHOD(__SystemLib_ChunkedInflator,
 
 ///////////////////////////////////////////////////////////////////////////////
 
-class ZlibExtension : public Extension {
+class ZlibExtension final : public Extension {
  public:
   ZlibExtension() : Extension("zlib", "2.0") {}
-  virtual void moduleLoad(const IniSetting::Map& ini, Hdf hdf) {
+  void moduleLoad(const IniSetting::Map& ini, Hdf hdf) override {
     s_zlib_stream_wrapper.registerAs("compress.zlib");
   }
-  virtual void moduleInit() {
-#define X(cns) \
-    Native::registerConstant<KindOfInt64>(makeStaticString(#cns), k_##cns);
-    X(ZLIB_ENCODING_RAW);
-    X(ZLIB_ENCODING_GZIP);
-    X(ZLIB_ENCODING_DEFLATE);
-    X(ZLIB_ENCODING_ANY);
-    X(FORCE_GZIP);
-    X(FORCE_DEFLATE);
-#undef X
+  void moduleInit() override {
+    HHVM_RC_INT(ZLIB_ENCODING_RAW, k_ZLIB_ENCODING_RAW);
+    HHVM_RC_INT(ZLIB_ENCODING_GZIP, k_ZLIB_ENCODING_GZIP);
+    HHVM_RC_INT(ZLIB_ENCODING_DEFLATE, k_ZLIB_ENCODING_DEFLATE);
+    HHVM_RC_INT(ZLIB_ENCODING_ANY, k_ZLIB_ENCODING_ANY);
+    HHVM_RC_INT(FORCE_GZIP, k_FORCE_GZIP);
+    HHVM_RC_INT(FORCE_DEFLATE, k_FORCE_DEFLATE);
 
     HHVM_FE(zlib_encode);
     HHVM_FE(gzdeflate);
@@ -814,14 +692,14 @@ class ZlibExtension : public Extension {
     HHVM_FE(qlzuncompress);
 #endif
 #ifdef HAVE_SNAPPY
-    HHVM_FE(sncompress);
-    HHVM_FE(snuncompress);
+    HHVM_FE(snappy_compress);
+    HHVM_FE(snappy_uncompress);
 #endif
     HHVM_FE(nzcompress);
     HHVM_FE(nzuncompress);
-    HHVM_FE(lz4compress);
-    HHVM_FE(lz4hccompress);
-    HHVM_FE(lz4uncompress);
+    HHVM_FE(lz4_compress);
+    HHVM_FE(lz4_hccompress);
+    HHVM_FE(lz4_uncompress);
 
     HHVM_ME(__SystemLib_ChunkedInflator, eof);
     HHVM_ME(__SystemLib_ChunkedInflator, inflateChunk);
