@@ -178,7 +178,7 @@ PHP_FUNCTION(ezc_set_global)
 }
 /* }}} */
 
-/* {{{ proto mixed ezc_call(mixed function_name [, mixed parmeter] [, mixed ...])
+/* {{{ proto mixed ezc_call(mixed function_name [, mixed parameter] [, mixed ...])
  * Call a function, like call_user_func() */
 PHP_FUNCTION(ezc_call)
 {
@@ -204,7 +204,7 @@ PHP_FUNCTION(ezc_call)
 }
 /* }}} */
 
-/* {{{ proto mixed ezc_try_call(mixed function_name [, mixed parmeter] [, mixed ...])
+/* {{{ proto mixed ezc_try_call(mixed function_name [, mixed parameter] [, mixed ...])
  * Call a function. If it throws an exception, catch it and return the exception
  * object. Otherwise, return the return value. */
 PHP_FUNCTION(ezc_try_call)
@@ -535,6 +535,33 @@ PHP_FUNCTION(ezc_array_set)
 
 /* }}} */
 
+/* {{{ proto mixed ezc_get_error_reporting()
+ * Use the zend construct EG(error_reporting) in rval context.
+ */
+PHP_FUNCTION(ezc_get_error_reporting)
+{
+  int old_error_reporting = EG(error_reporting);
+
+  RETVAL_LONG(old_error_reporting);
+}
+/* }}} */
+
+/* {{{ proto mixed ezc_set_error_reporting(integer $val)
+ * Use the zend construct EG(error_reporting) in rval and then lval context.
+ */
+PHP_FUNCTION(ezc_set_error_reporting)
+{
+  long old_error_reporting = EG(error_reporting);
+  long new_error_reporting = 0;
+
+  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &new_error_reporting) == FAILURE) {
+    RETURN_FALSE;
+  }
+  EG(error_reporting) = new_error_reporting;
+  RETVAL_LONG(old_error_reporting);
+}
+/* }}} */
+
 /* {{{ EzcTestCloneable_create_object */
 static zend_object_value EzcTestCloneable_create_object(zend_class_entry *ce TSRMLS_DC)
 {
@@ -624,6 +651,36 @@ PHP_FUNCTION(ezc_create_cloneable_in_array)
 }
 /* }}} */
 
+/* {{{ proto mixed ezc_atof_toa()
+ * Convert a string to a double and back to a string again */
+PHP_FUNCTION(ezc_atof_toa)
+{
+  char *value;
+  int valuelen;
+  double d;
+  const char *end_value;
+  const int ndigits = 10;
+  int decpt = -1;
+  int sign = -1;
+  char *result;
+  char *resulte;
+  char *composed_result;
+
+  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
+        &value, &valuelen) == FAILURE) {
+    RETURN_FALSE;
+  }
+
+  d = zend_strtod(value, &end_value);
+  (void)end_value;
+
+  result = zend_dtoa(d, 0, ndigits, &decpt, &sign, &resulte);
+  asprintf(&composed_result, "%d %d %s", decpt, sign, result);
+  zend_freedtoa(result);
+  RETVAL_STRING(composed_result, 1);
+}
+/* }}} */
+
 /* {{{ arginfo */
 ZEND_BEGIN_ARG_INFO(arginfo_ezc_fetch_global, 0)
 ZEND_END_ARG_INFO()
@@ -697,6 +754,18 @@ ZEND_BEGIN_ARG_INFO(arginfo_ezc_array_set, 0)
   ZEND_ARG_INFO(0, key)
   ZEND_ARG_INFO(0, value)
 ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_ezc_get_error_reporting, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_ezc_set_error_reporting, 0)
+  ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_ezc_atof_toa, 0)
+  ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO()
+
 /* }}} */
 
 /* {{{ ezc_test_functions[]
@@ -718,6 +787,9 @@ const zend_function_entry ezc_test_functions[] = {
   PHP_FE(ezc_array_val_set, arginfo_ezc_array_val_set)
   PHP_FE(ezc_create_cloneable_in_array, arginfo_ezc_create_cloneable_in_array)
   PHP_FE(ezc_array_set, arginfo_ezc_array_set)
+  PHP_FE(ezc_get_error_reporting, arginfo_ezc_get_error_reporting)
+  PHP_FE(ezc_set_error_reporting, arginfo_ezc_set_error_reporting)
+  PHP_FE(ezc_atof_toa, arginfo_ezc_atof_toa)
   PHP_FE_END
 };
 /* }}} */
@@ -745,4 +817,3 @@ zend_module_entry ezc_test_module_entry = {
 #ifdef COMPILE_DL_EZC_TEST
 ZEND_GET_MODULE(ezc_test)
 #endif
-
