@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -19,6 +19,6 @@
 
 namespace HPHP {
 
-__thread RequestLocal<ZendExceptionStore> ZendExceptionStore::tl_instance;
+IMPLEMENT_REQUEST_LOCAL(ZendExceptionStore, ZendExceptionStore::tl_instance);
 
 }
