@@ -64,11 +64,10 @@ static pthread_mutex_t cursor_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /* Macro to check whether a cursor is dead, and if so, bailout */
 #define MONGO_CURSOR_CHECK_DEAD \
-	if (cursor->dead) { \
-		zend_throw_exception(mongo_ce_ConnectionException, "the connection has been terminated, and this cursor is dead", 12 TSRMLS_CC); \
-		return; \
-	}
-
+  if (cursor->dead) { \
+    zend_throw_exception(mongo_ce_ConnectionException, "the connection has been terminated, and this cursor is dead", 12 TSRMLS_CC); \
+    return; \
+  }
 
 /* externs */
 extern zend_class_entry *mongo_ce_Id, *mongo_ce_MongoClient, *mongo_ce_DB;
@@ -124,7 +123,7 @@ static signed int get_cursor_header(mongo_connection *con, mongo_cursor *cursor,
 		return abs(status);
 	} else if (status < INT_32*4) {
 		*error_message = (char*) malloc(256);
-		snprintf(*error_message, 256, "couldn't get full response header, got %d bytes but expected atleast %d", status, INT_32*4);
+		snprintf(*error_message, 256, "couldn't get full response header, got %d bytes but expected at least %d", status, INT_32*4);
 		return 4;
 	}
 
@@ -1186,7 +1185,7 @@ int mongo_cursor__should_retry(mongo_cursor *cursor)
 }
 
 /* Returns 1 when an error was found and it returns 0 if no error
- * situation has ocurred on the cursor */
+ * situation has occurred on the cursor */
 static int have_error_flags(mongo_cursor *cursor)
 {
 	if (cursor->flag & MONGO_OP_REPLY_ERROR_FLAGS) {
@@ -1197,7 +1196,7 @@ static int have_error_flags(mongo_cursor *cursor)
 }
 
 /* Returns 1 when an error was found and *handled*, and it returns 0 if no error
- * situation has ocurred on the cursor */
+ * situation has occurred on the cursor */
 static int handle_error(mongo_cursor *cursor TSRMLS_DC)
 {
 	zval **err = NULL, **wnote = NULL;
@@ -1749,12 +1748,12 @@ int php_mongo_create_le(mongo_cursor *cursor, char *name TSRMLS_DC)
 		prev->next = new_node;
 		new_node->prev = prev;
 	} else {
-		zend_rsrc_list_entry new_le;
-
-		new_le.ptr = new_node;
-		new_le.type = le_cursor_list;
-		new_le.refcount = 1;
-		zend_hash_add(&EG(persistent_list), name, strlen(name) + 1, &new_le, sizeof(zend_rsrc_list_entry), NULL);
+		auto new_le =
+      HPHP::req::make<zend_rsrc_list_entry>(new_node, le_cursor_list).detach();
+    SCOPE_EXIT { delete new_le; };
+		new_le->refcount = 1;
+		zend_hash_add(&EG(persistent_list), name, strlen(name) + 1, new_le,
+                  sizeof(*new_le), nullptr);
 	}
 
 	UNLOCK(cursor);
