@@ -22,9 +22,10 @@
 
 // has to be before zend_API since that defines getThis()
 #include "zend_API.h"
-#include "hphp/runtime/ext/ext_function.h"
+#include "hphp/runtime/ext/std/ext_std_function.h"
 #include "zend_constants.h"
 
+#include "hphp/runtime/base/builtin-functions.h"
 #include "hphp/runtime/base/zend-printf.h"
 #include "hphp/util/thread-local.h"
 #include "hphp/runtime/ext_zend_compat/hhvm/zend-class-entry.h"
@@ -50,7 +51,6 @@ static int parse_arg_object_to_string(zval **arg, const char **p, int *pl, int t
   HPHP::StringData *sd = tvCastToString((*arg)->tv());
   *p = sd->data();
   *pl = sd->size();
-  zval_ptr_dtor(arg);
   return sd->empty();
 }
 
@@ -418,7 +418,7 @@ static const char *zend_parse_arg_impl(int arg_num, zval **arg, va_list *va, con
         if (check_null && Z_TYPE_PP(arg) == IS_NULL) {
           *p = NULL;
         } else {
-          not_implemented();
+          *p = arg;
         }
       }
       break;
@@ -575,22 +575,14 @@ static int zend_parse_va_args(int num_args, const char *type_spec, va_list *va,
 
         *n_varargs = num_varargs;
 
-        /* Allocate space for the args. Zend already has single pointers
-         * persistently stored, and only needs to allocate space for the double
-         * pointers, but we need to allocate space for both.
-         *
-         * We need to allocate it in such a way that a single efree(varargs)
-         * in the caller will free all relevant memory. So we allocate a single
-         * block and then split it.
+        /* Allocate space for the args. We need to allocate it in such a way
+         * that a single efree(varargs) in the caller will free all relevant
+         * memory.
          */
-        zval *** double_ptrs = (zval***)safe_emalloc(num_varargs * 2,
-                                                     sizeof(void*), 0);
-        *varargs = double_ptrs;
-        zval ** single_ptrs = (zval**)(double_ptrs + num_varargs);
+        *varargs = (zval***)safe_emalloc(num_varargs, sizeof(void*), 0);
 
         for (iv = 0; iv < num_varargs; iv++) {
-          double_ptrs[iv] = &single_ptrs[iv];
-          single_ptrs[iv] = HPHP::ZendExecutionStack::getArg(i + iv);
+          (*varargs)[iv] = HPHP::ZendExecutionStack::getArg(i + iv);
         }
 
         /* adjust how many args we have left and restart loop */
@@ -603,8 +595,7 @@ static int zend_parse_va_args(int num_args, const char *type_spec, va_list *va,
       }
     }
 
-    auto tmp = HPHP::ZendExecutionStack::getArg(i);
-    arg = &tmp;
+    arg = HPHP::ZendExecutionStack::getArg(i);
 
     if (zend_parse_arg(i+1, arg, va, &type_spec, quiet TSRMLS_CC) == FAILURE) {
       /* clean up varargs array if it was used */
@@ -1089,11 +1080,14 @@ ZEND_API int array_set_zval_key(HashTable *ht, zval *key, zval *value) /* {{{ */
 ZEND_API zend_bool zend_is_callable_ex(zval *callable, zval *object_ptr, uint check_flags, char **callable_name, int *callable_name_len, zend_fcall_info_cache *fcc, char **error TSRMLS_DC) /* {{{ */
 {
   HPHP::Variant name;
+  HPHP::RefData* nameRef = nullptr;
   int callable_name_len_local;
   zend_fcall_info_cache fcc_local;
 
   if (callable_name) {
     *callable_name = NULL;
+    tvBox(name.asTypedValue());
+    nameRef = name.asTypedValue()->m_data.pref;
   }
   if (callable_name_len == NULL) {
     callable_name_len = &callable_name_len_local;
@@ -1108,10 +1102,10 @@ ZEND_API zend_bool zend_is_callable_ex(zval *callable, zval *object_ptr, uint ch
   fcc->calling_scope = NULL;
   fcc->object_ptr = NULL;
 
-  bool b = f_is_callable(
-      tvAsVariant(callable->tv()),
-      check_flags & IS_CALLABLE_CHECK_SYNTAX_ONLY,
-      HPHP::ref(name));
+  bool b = is_callable(
+    tvAsVariant(callable->tv()),
+    check_flags & IS_CALLABLE_CHECK_SYNTAX_ONLY,
+    nameRef);
   if (b) {
     if (callable_name) {
       HPHP::StringData *sd = name.getStringData();
@@ -1486,12 +1480,12 @@ ZEND_API int zend_update_static_property(zend_class_entry *scope, const char *na
   }
 
   HPHP::String sname(name, name_length, HPHP::CopyString);
-  bool visible, accessible;
-  auto tv = cls->getSProp(cls, sname.get(), visible, accessible);
-  if (!tv) {
-    return FAILURE;
-  }
-  HPHP::tvSetZval(value, tv);
+
+  auto const lookup = cls->getSProp(cls, sname.get());
+
+  if (!lookup.prop) return FAILURE;
+
+  HPHP::tvSetZval(value, lookup.prop);
   return SUCCESS;
 }
 
@@ -1597,7 +1591,7 @@ ZEND_API zend_class_entry *zend_register_internal_class(zend_class_entry *orig_c
         name += "->";
       }
       name += fe->fname;
-      HPHP::Native::registerBuiltinFunction(name.detach(), fe->handler);
+      HPHP::Native::registerBuiltinZendFunction(name, fe->handler);
       fe++;
     }
   }
@@ -1625,8 +1619,7 @@ ZEND_API int _object_and_properties_init(zval *arg, zend_class_entry *class_type
         "Is the system library not loaded yet?", class_type->name);
     return FAILURE;
   }
-  Z_OBJVAL_P(arg) = HPHP::ObjectData::newInstance(cls);
-  Z_OBJVAL_P(arg)->incRefCount();
+  Z_OBJVAL_P(arg) = HPHP::Object{cls}.detach();
   Z_TYPE_P(arg) = IS_OBJECT;
   return SUCCESS;
 }
@@ -1639,16 +1632,19 @@ ZEND_API zval *zend_read_property(zend_class_entry *scope, zval *object, const c
     ctx = HPHP::Unit::lookupClass(scope_name.get());
   }
 
-  bool visible, accessible, unset;
-  auto ret = Z_OBJVAL_P(object)->zGetProp(ctx, prop_name.get(), visible, accessible, unset);
-  if (!accessible || unset) {
-    return nullptr;
+  auto const lookup = Z_OBJVAL_P(object)->getProp(ctx, prop_name.get());
+  auto const prop = lookup.prop;
+
+  if (!lookup.accessible || prop->m_type == HPHP::KindOfUninit) return nullptr;
+
+  if (prop->m_type != HPHP::KindOfRef) {
+    tvBox(prop);
   }
-  return ret;
+  return prop->m_data.pref;
 }
 
 ZEND_API zval *zend_read_static_property(zend_class_entry *scope, const char *name, int name_length, zend_bool silent TSRMLS_DC) {
-  HPHP::Class * cls = HPHP::zend_hphp_class_entry_to_class(scope);
+  auto const cls = HPHP::zend_hphp_class_entry_to_class(scope);
   if (!cls) {
     // You can't call this function from MINIT, sorry
     HPHP::raise_error("cannot read property of class %s. "
@@ -1656,12 +1652,19 @@ ZEND_API zval *zend_read_static_property(zend_class_entry *scope, const char *na
     return nullptr;
   }
   HPHP::String sname(name, name_length, HPHP::CopyString);
-  bool visible, accessible;
-  auto ret = cls->zGetSProp(cls, sname.get(), visible, accessible);
-  if (!accessible || !visible) {
-    return nullptr;
+
+  auto const lookup = cls->getSProp(cls, sname.get());
+  auto const prop = lookup.prop;
+
+  // Static properties should never be Uninit.
+  assert(!prop || prop->m_type != HPHP::KindOfUninit);
+
+  if (!prop || !lookup.accessible) return nullptr;
+
+  if (prop->m_type != HPHP::KindOfRef) {
+    tvBox(prop);
   }
-  return ret;
+  return prop->m_data.pref;
 }
 
 ZEND_API zend_class_entry *zend_register_internal_class_ex(zend_class_entry *class_entry, zend_class_entry *parent_ce, char *parent_name TSRMLS_DC) {
