@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,18 +16,16 @@
 
 #include "hphp/runtime/server/http-server.h"
 
-#include <boost/lexical_cast.hpp>
-
 #include "hphp/runtime/base/class-info.h"
 #include "hphp/runtime/base/externals.h"
+#include "hphp/runtime/base/file-util.h"
 #include "hphp/runtime/base/http-client.h"
 #include "hphp/runtime/base/memory-manager.h"
 #include "hphp/runtime/base/pprof-server.h"
 #include "hphp/runtime/base/program-functions.h"
 #include "hphp/runtime/base/runtime-option.h"
-#include "hphp/runtime/base/thread-init-fini.h"
+#include "hphp/runtime/base/init-fini-node.h"
 #include "hphp/runtime/debugger/debugger.h"
-#include "hphp/runtime/ext/ext_apc.h"
 #include "hphp/runtime/server/admin-request-handler.h"
 #include "hphp/runtime/server/http-request-handler.h"
 #include "hphp/runtime/server/replay-transport.h"
@@ -35,19 +33,20 @@
 #include "hphp/runtime/server/static-content-cache.h"
 #include "hphp/runtime/server/warmup-request-handler.h"
 #include "hphp/runtime/server/xbox-server.h"
-#include "hphp/util/db-conn.h"
+
+#include "hphp/util/boot_timer.h"
 #include "hphp/util/logger.h"
 #include "hphp/util/process.h"
 #include "hphp/util/ssl-init.h"
-#include "hphp/runtime/base/file-util.h"
+
+#include <folly/Conv.h>
+#include <folly/Format.h>
 
 #include <sys/types.h>
 #include <signal.h>
 
 namespace HPHP {
-extern InitFiniNode *extra_server_init, *extra_server_exit;
 
-using boost::lexical_cast;
 using std::string;
 
 
@@ -98,7 +97,7 @@ HttpServer::HttpServer()
   options.m_serverFD = RuntimeOption::ServerPortFd;
   options.m_sslFD = RuntimeOption::SSLPortFd;
   options.m_takeoverFilename = RuntimeOption::TakeoverFilename;
-  m_pageServer = std::move(serverFactory->createServer(options));
+  m_pageServer = serverFactory->createServer(options);
   m_pageServer->addTakeoverListener(this);
   m_pageServer->addServerEventListener(this);
 
@@ -123,14 +122,14 @@ HttpServer::HttpServer()
   ServerOptions admin_options
     (RuntimeOption::ServerIP, RuntimeOption::AdminServerPort,
      RuntimeOption::AdminThreadCount);
-  m_adminServer = std::move(serverFactory->createServer(admin_options));
+  m_adminServer = serverFactory->createServer(admin_options);
   m_adminServer->setRequestHandlerFactory<AdminRequestHandler>(
     RuntimeOption::RequestTimeoutSeconds);
 
   for (unsigned int i = 0; i < RuntimeOption::SatelliteServerInfos.size();
        i++) {
     auto info = RuntimeOption::SatelliteServerInfos[i];
-    auto satellite(std::move(SatelliteServer::Create(info)));
+    auto satellite(SatelliteServer::Create(info));
     if (satellite) {
       if (info->getType() == SatelliteServer::Type::KindOfDanglingPageServer) {
         m_danglings.push_back(std::move(satellite));
@@ -141,7 +140,7 @@ HttpServer::HttpServer()
   }
 
   if (RuntimeOption::XboxServerPort != 0) {
-    std::shared_ptr<SatelliteServerInfo> xboxInfo(new XboxServerInfo());
+    auto xboxInfo = std::make_shared<XboxServerInfo>();
     auto satellite = SatelliteServer::Create(xboxInfo);
     if (satellite) {
       m_satellites.push_back(std::move(satellite));
@@ -164,7 +163,7 @@ HttpServer::HttpServer()
     ReplayTransport rt;
     rt.replayInput(hdf);
     HttpRequestHandler handler(0);
-    handler.handleRequest(&rt);
+    handler.run(&rt);
     int code = rt.getResponseCode();
     if (code == 200) {
       Logger::Info("StartupDocument %s returned 200 OK: %s",
@@ -177,24 +176,11 @@ HttpServer::HttpServer()
       return;
     }
   }
-
-  for (unsigned int i = 0; i < RuntimeOption::ThreadDocuments.size(); i++) {
-    m_serviceThreads.push_back(
-      std::make_shared<ServiceThread>(RuntimeOption::ThreadDocuments[i]));
-  }
-
-  for (unsigned int i = 0; i < RuntimeOption::ThreadLoopDocuments.size(); i++) {
-    m_serviceThreads.push_back(
-      std::make_shared<ServiceThread>(
-        RuntimeOption::ThreadLoopDocuments[i], true));
-  }
 }
 
 // Synchronously stop satellites and start danglings
 void HttpServer::onServerShutdown() {
-  for (InitFiniNode *in = extra_server_exit; in; in = in->next) {
-    in->func();
-  }
+  InitFiniNode::ServerFini();
 
   Eval::Debugger::Stop();
   if (RuntimeOption::EnableDebuggerServer) {
@@ -241,6 +227,11 @@ void HttpServer::serverStopped(HPHP::Server* server) {
   Logger::Info("Page server stopped");
   assert(server == m_pageServer.get());
   removePid();
+
+  auto sockFile = RuntimeOption::ServerFileSocket;
+  if (!sockFile.empty()) {
+    unlink(sockFile.c_str());
+  }
 }
 
 HttpServer::~HttpServer() {
@@ -251,30 +242,23 @@ HttpServer::~HttpServer() {
   stop();
 }
 
-void HttpServer::startupFailure() {
-  Logger::Error("Shutting down due to failure(s) to bind in "
-                "HttpServer::run");
-  // Logger flushes itself---we don't need to run any atexit handlers
-  // (historically we've mostly just SEGV'd while trying) ...
-  _Exit(1);
-}
-
 void HttpServer::runOrExitProcess() {
   StartTime = time(0);
 
-  m_watchDog.start();
+  auto startupFailure = [] (const std::string& msg) {
+    Logger::Error(msg);
+    Logger::Error("Shutting down due to failure(s) to bind in "
+                  "HttpServer::runAndExitProcess");
+    // Logger flushes itself---we don't need to run any atexit handlers
+    // (historically we've mostly just SEGV'd while trying) ...
+    _Exit(1);
+  };
 
-  for (unsigned int i = 0; i < m_serviceThreads.size(); i++) {
-    m_serviceThreads[i]->start();
-  }
-  for (unsigned int i = 0; i < m_serviceThreads.size(); i++) {
-    m_serviceThreads[i]->waitForStarted();
-  }
+  m_watchDog.start();
 
   if (RuntimeOption::ServerPort) {
     if (!startServer(true)) {
-      Logger::Error("Unable to start page server");
-      startupFailure();
+      startupFailure("Unable to start page server");
       not_reached();
     }
     Logger::Info("page server started");
@@ -282,8 +266,7 @@ void HttpServer::runOrExitProcess() {
 
   if (RuntimeOption::AdminServerPort) {
     if (!startServer(false)) {
-      Logger::Error("Unable to start admin server");
-      startupFailure();
+      startupFailure("Unable to start admin server");
       not_reached();
     }
     Logger::Info("admin server started");
@@ -295,34 +278,39 @@ void HttpServer::runOrExitProcess() {
       m_satellites[i]->start();
       Logger::Info("satellite server %s started", name.c_str());
     } catch (Exception &e) {
-      Logger::Error("Unable to start satellite server %s: %s",
-                    name.c_str(), e.getMessage().c_str());
-      startupFailure();
+      startupFailure(
+        folly::format("Unable to start satellite server {}: {}",
+                      name, e.getMessage()).str()
+      );
       not_reached();
     }
   }
 
   if (RuntimeOption::HHProfServerEnabled) {
-    Logger::Info("Starting up profiling server");
-    HeapProfileServer::Server = std::make_shared<HeapProfileServer>();
+    try {
+      HeapProfileServer::Server = std::make_shared<HeapProfileServer>();
+    } catch (FailedToListenException &e) {
+      startupFailure("Unable to start profiling server");
+      not_reached();
+    }
+    Logger::Info("profiling server started");
   }
 
   if (!Eval::Debugger::StartServer()) {
-    Logger::Error("Unable to start debugger server");
-    startupFailure();
+    startupFailure("Unable to start debugger server");
     not_reached();
   } else if (RuntimeOption::EnableDebuggerServer) {
     Logger::Info("debugger server started");
   }
 
-  for (InitFiniNode *in = extra_server_init; in; in = in->next) {
-    in->func();
-  }
+  InitFiniNode::ServerInit();
 
   {
+    BootTimer::mark("servers started");
     Logger::Info("all servers started");
     createPid();
     Lock lock(this);
+    BootTimer::done();
     // continously running until /stop is received on admin server
     while (!m_stopped) {
       wait();
@@ -357,16 +345,9 @@ void HttpServer::runOrExitProcess() {
                  m_danglings[i]->getName().c_str());
   }
 
-  for (unsigned int i = 0; i < m_serviceThreads.size(); i++) {
-    m_serviceThreads[i]->notifyStopped();
-  }
-  for (unsigned int i = 0; i < m_serviceThreads.size(); i++) {
-    m_serviceThreads[i]->waitForEnd();
-  }
-
   waitForServers();
-  hphp_process_exit();
   m_watchDog.waitForEnd();
+  hphp_process_exit();
   Logger::Info("all servers stopped");
 }
 
@@ -386,10 +367,14 @@ void HttpServer::waitForServers() {
 static void exit_on_timeout(int sig) {
   signal(sig, SIG_DFL);
   kill(getpid(), SIGKILL);
-  exit(0);
+  // we really shouldn't get here, but who knows.
+  // abort so we catch it as a crash.
+  abort();
 }
 
 void HttpServer::stop(const char* stopReason) {
+  if (m_stopped) return;
+
   if (RuntimeOption::ServerGracefulShutdownWait) {
     signal(SIGALRM, exit_on_timeout);
     alarm(RuntimeOption::ServerGracefulShutdownWait);
@@ -401,18 +386,6 @@ void HttpServer::stop(const char* stopReason) {
   notify();
 }
 
-void HttpServer::abortServers() {
-  for (unsigned int i = 0; i < m_satellites.size(); i++) {
-    m_satellites[i]->stop();
-  }
-  if (RuntimeOption::AdminServerPort) {
-    m_adminServer->stop();
-  }
-  if (RuntimeOption::ServerPort) {
-    m_pageServer->stop();
-  }
-}
-
 void HttpServer::stopOnSignal(int sig) {
   // Signal to the main server thread to exit immediately if
   // we want to die on SIGTERM
@@ -437,6 +410,8 @@ void HttpServer::stopOnSignal(int sig) {
   if (m_adminServer) {
     m_adminServer->stop();
   }
+
+  waitForServers();
 }
 
 void HttpServer::createPid() {
@@ -515,9 +490,11 @@ void HttpServer::dropCache() {
 }
 
 void HttpServer::checkMemory() {
-  if (RuntimeOption::MaxRSS > 0 &&
-      Process::GetProcessRSS(Process::GetProcessId()) * 1024 * 1024 >
-      RuntimeOption::MaxRSS) {
+  int64_t used = Process::GetProcessRSS(Process::GetProcessId()) * 1024 * 1024;
+  if (RuntimeOption::MaxRSS > 0 && used > RuntimeOption::MaxRSS) {
+    Logger::Error(
+      "ResourceLimit.MaxRSS %" PRId64 " reached %" PRId64 " used, exiting",
+      RuntimeOption::MaxRSS, used);
     stop();
   }
 }
@@ -534,6 +511,16 @@ void HttpServer::getSatelliteStats(
   }
 }
 
+std::pair<int, int> HttpServer::getSatelliteRequestCount() const {
+  int inflight = 0;
+  int queued = 0;
+  for (const auto& i : m_satellites) {
+    inflight += i->getActiveWorker();
+    queued += i->getQueuedJobs();
+  }
+  return std::make_pair(inflight, queued);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // page server
 
@@ -573,7 +560,7 @@ bool HttpServer::startServer(bool pageServer) {
         RuntimeOption::ServerIP;
       url += serverIp;
       url += ":";
-      url += boost::lexical_cast<std::string>(RuntimeOption::AdminServerPort);
+      url += folly::to<std::string>(RuntimeOption::AdminServerPort);
       url += "/stop";
       std::string auth;
 
@@ -664,7 +651,7 @@ bool HttpServer::startServer(bool pageServer) {
           }
 
           std::string cmd = "lsof -t -i :";
-          cmd += lexical_cast<std::string>(port);
+          cmd += folly::to<std::string>(port);
           cmd += " | xargs kill -9";
           FileUtil::ssystem(cmd.c_str());
         }
