@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,11 +22,11 @@
 #include <list>
 #include <iostream>
 
-#include <boost/lexical_cast.hpp>
-
-#include "folly/json.h"
-#include "folly/Range.h"
-#include "folly/String.h"
+#include <folly/json.h>
+#include <folly/Conv.h>
+#include <folly/FBVector.h>
+#include <folly/Range.h>
+#include <folly/String.h>
 
 #include "hphp/runtime/server/http-server.h"
 #include "hphp/runtime/base/runtime-option.h"
@@ -37,13 +37,12 @@
 #include "hphp/runtime/base/datetime.h"
 #include "hphp/runtime/base/array-init.h"
 #include "hphp/util/compatibility.h"
+#include "hphp/util/hardware-counter.h"
 #include "hphp/util/process.h"
 #include "hphp/util/timer.h"
 #include "hphp/util/text-util.h"
-#include "hphp/runtime/base/hardware-counter.h"
-
+#include "hphp/runtime/server/writer.h"
 namespace HPHP {
-
 //////////////////////////////////////////////////////////////////////
 
 using std::list;
@@ -51,43 +50,30 @@ using std::set;
 using std::map;
 using std::ostream;
 using std::string;
-using boost::lexical_cast;
 
 ///////////////////////////////////////////////////////////////////////////////
 // helpers
 
-static folly::fbstring escape_for_json(const char* s) {
-  auto ret = folly::fbstring{};
-  auto opts = folly::json::serialization_opts{};
-  opts.skip_invalid_utf8 = true;
-  opts.encode_non_ascii = true;
-  // NB: escapeString prepends and appends double quotes
-  folly::json::escapeString(folly::StringPiece{s}, ret, opts);
-  return ret;
-}
-
 void ServerStats::GetLogger() {
   s_logger.getCheck();
 }
 
 void ServerStats::Merge(CounterMap &dest, const CounterMap &src) {
-  for (CounterMap::const_iterator iter = src.begin();
-       iter != src.end(); ++iter) {
-    dest[iter->first] += iter->second;
+  for (auto const& iter : src) {
+    dest[iter.first] += iter.second;
   }
 }
 
 void ServerStats::Merge(PageStatsMap &dest, const PageStatsMap &src) {
-  for (PageStatsMap::const_iterator iter = src.begin();
-       iter != src.end(); ++iter) {
-    const SharedString &key = iter->first;
-    const PageStats &s = iter->second;
+  for (auto const& iter : src) {
+    auto const& key = iter.first;
+    auto const& s = iter.second;
 
-    PageStatsMap::iterator diter = dest.find(key);
+    auto diter = dest.find(key);
     if (diter == dest.end()) {
       dest[key] = s;
     } else {
-      PageStats &d = diter->second;
+      auto& d = diter->second;
       assert(d.m_url == s.m_url);
       assert(d.m_code == s.m_code);
       d.m_hit += s.m_hit;
@@ -97,13 +83,10 @@ void ServerStats::Merge(PageStatsMap &dest, const PageStatsMap &src) {
 }
 
 void ServerStats::Merge(list<TimeSlot*> &dest, const list<TimeSlot*> &src) {
-  list<TimeSlot*>::iterator diter = dest.begin();
-  for (list<TimeSlot*>::const_iterator iter = src.begin();
-       iter != src.end(); ++iter) {
-    TimeSlot *s = *iter;
-
+  auto diter = dest.begin();
+  for (auto const& s : src) {
     for (; diter != dest.end(); ++diter) {
-      TimeSlot *d = *diter;
+      auto d = *diter;
       if (d->m_time > s->m_time) {
         TimeSlot *c = new TimeSlot();
         *c = *s;
@@ -127,14 +110,10 @@ void ServerStats::Merge(list<TimeSlot*> &dest, const list<TimeSlot*> &src) {
 
 void ServerStats::GetAllKeys(std::set<std::string> &allKeys,
                              const std::list<TimeSlot*> &slots) {
-  for (auto iter = slots.begin(); iter != slots.end(); ++iter) {
-    TimeSlot *s = *iter;
-    for (PageStatsMap::const_iterator piter = s->m_pages.begin();
-         piter != s->m_pages.end(); ++piter) {
-      const PageStats &ps = piter->second;
-      for (CounterMap::const_iterator viter =
-             ps.m_values.begin(); viter != ps.m_values.end(); ++viter) {
-        allKeys.insert(viter->first->getString());
+  for (auto& slot : slots) {
+    for (auto const& page : slot->m_pages) {
+      for (auto const& kvpair : page.second.m_values) {
+        allKeys.insert(kvpair.first->getString());
       }
     }
   }
@@ -144,20 +123,21 @@ void ServerStats::GetAllKeys(std::set<std::string> &allKeys,
   allKeys.insert("load");
   allKeys.insert("idle");
   allKeys.insert("queued");
+  allKeys.insert("health_level");
 }
 
 void ServerStats::Filter(list<TimeSlot*> &slots, const std::string &keys,
                          const std::string &url, int code,
                          std::map<std::string, int> &wantedKeys) {
   if (!keys.empty()) {
-    std::vector<std::string> rules0;
+    folly::fbvector<std::string> rules0;
     split(',', keys.c_str(), rules0, true);
     if (!rules0.empty()) {
 
       // prepare rules
       std::map<std::string, int> rules;
       for (unsigned int i = 0; i < rules0.size(); i++) {
-        std::string &rule = rules0[i];
+        auto const& rule = rules0[i];
         assert(!rule.empty());
         int len = rule.length();
         std::string suffix;
@@ -179,20 +159,18 @@ void ServerStats::Filter(list<TimeSlot*> &slots, const std::string &keys,
       GetAllKeys(allKeys, slots);
 
       // prepare wantedKeys
-      for (auto iter = allKeys.begin(); iter != allKeys.end(); ++iter) {
-        const string &key = *iter;
-        for (map<string, int>::const_iterator riter = rules.begin();
-             riter != rules.end(); ++riter) {
-          const string &rule = riter->first;
+      for (auto const& key : allKeys) {
+        for (auto const& riter : rules) {
+          const string &rule = riter.first;
           if (rule[0] == ':') {
             Variant ret = preg_match(String(rule.c_str(), rule.size(),
                   CopyString),
                 String(key.c_str(), key.size(), CopyString));
             if (!same(ret, false) && more(ret, 0)) {
-              wantedKeys[key] |= riter->second;
+              wantedKeys[key] |= riter.second;
             }
           } else if (rule == key) {
-            wantedKeys[key] |= riter->second;
+            wantedKeys[key] |= riter.second;
           }
         }
       }
@@ -201,25 +179,21 @@ void ServerStats::Filter(list<TimeSlot*> &slots, const std::string &keys,
 
   bool urlEmpty = url.empty();
   bool keysEmpty = keys.empty();
-  for (list<TimeSlot*>::const_iterator iter = slots.begin();
-       iter != slots.end(); ++iter) {
-    TimeSlot *s = *iter;
-    for (PageStatsMap::iterator piter = s->m_pages.begin();
-         piter != s->m_pages.end();) {
-      PageStats &ps = piter->second;
+  for (auto const& s : slots) {
+    for (auto piter = s->m_pages.begin(); piter != s->m_pages.end();) {
+      auto &ps = piter->second;
       if ((code && ps.m_code != code) || (!urlEmpty && ps.m_url != url)) {
-        PageStatsMap::iterator piterTemp = piter;
+        auto piterTemp = piter;
         ++piter;
         s->m_pages.erase(piterTemp);
         continue;
       }
 
       if (!keysEmpty) {
-        CounterMap &values = ps.m_values;
-        for (CounterMap::iterator viter =
-               values.begin(); viter != values.end();) {
+        auto &values = ps.m_values;
+        for (auto viter = values.begin(); viter != values.end();) {
           if (wantedKeys.find(viter->first->getString()) == wantedKeys.end()) {
-            CounterMap::iterator iterTemp = viter;
+            auto iterTemp = viter;
             ++viter;
             values.erase(iterTemp);
           } else {
@@ -237,14 +211,11 @@ void ServerStats::Aggregate(list<TimeSlot*> &slots, const std::string &agg,
   int slotCount = slots.size();
 
   if (!agg.empty()) {
-    TimeSlot *ts = new TimeSlot();
+    auto const ts = new TimeSlot();
     ts->m_time = 0;
-    for (list<TimeSlot*>::const_iterator iter = slots.begin();
-         iter != slots.end(); ++iter) {
-      TimeSlot *s = *iter;
-      for (PageStatsMap::const_iterator piter = s->m_pages.begin();
-           piter != s->m_pages.end(); ++piter) {
-        const PageStats &ps = piter->second;
+    for (auto const& s : slots) {
+      for (auto const& page : s->m_pages) {
+        auto const& ps = page.second;
         string url = ps.m_url;
         int code = ps.m_code;
         if (agg != "url") {
@@ -253,7 +224,7 @@ void ServerStats::Aggregate(list<TimeSlot*> &slots, const std::string &agg,
         if (agg != "code") {
           code = 0;
         }
-        PageStats &psDest = ts->m_pages[url + lexical_cast<string>(code)];
+        auto &psDest = ts->m_pages[url + folly::to<string>(code)];
         psDest.m_hit += ps.m_hit;
         psDest.m_url = url;
         psDest.m_code = code;
@@ -265,10 +236,9 @@ void ServerStats::Aggregate(list<TimeSlot*> &slots, const std::string &agg,
   }
 
   std::map<std::string, int> udfKeys;
-  for (std::map<std::string, int>::const_iterator iter = wantedKeys.begin();
-       iter != wantedKeys.end(); ++iter) {
-    if (iter->second != UDF_NONE) {
-      udfKeys[iter->first] = iter->second;
+  for (auto const& iter : wantedKeys) {
+    if (iter.second != UDF_NONE) {
+      udfKeys[iter.first] = iter.second;
     }
   }
 
@@ -276,16 +246,14 @@ void ServerStats::Aggregate(list<TimeSlot*> &slots, const std::string &agg,
   int load = HttpServer::Server->getPageServer()->getActiveWorker();
   int idle = RuntimeOption::ServerThreadCount - load;
   int queued = HttpServer::Server->getPageServer()->getQueuedJobs();
+  int health_level = (int)ServerStats::m_ServerHealthLevel;
 
-  for (list<TimeSlot*>::const_iterator iter = slots.begin();
-       iter != slots.end(); ++iter) {
-    TimeSlot *s = *iter;
+  for (auto const& s : slots) {
     int sec = (s->m_time == 0 ? slotCount : 1) *
       RuntimeOption::StatsSlotDuration;
-    for (PageStatsMap::iterator piter = s->m_pages.begin();
-         piter != s->m_pages.end(); ++piter) {
-      PageStats &ps = piter->second;
-      CounterMap &values = ps.m_values;
+    for (auto &page : s->m_pages) {
+      auto &ps = page.second;
+      auto &values = ps.m_values;
 
       // special keys
       if (wantedKeys.find("hit") != wantedKeys.end()) {
@@ -301,11 +269,14 @@ void ServerStats::Aggregate(list<TimeSlot*> &slots, const std::string &agg,
         values["queued"] = queued;
       }
 
-      for (map<string, int>::const_iterator iter = udfKeys.begin();
-           iter != udfKeys.end(); ++iter) {
-        const string &key = iter->first;
-        int udf = iter->second;
-        CounterMap::iterator viter = values.find(key);
+      if (wantedKeys.find("health_level") != wantedKeys.end()) {
+        values["health_level"] = health_level;
+      }
+
+      for (auto const& iter : udfKeys) {
+        const string &key = iter.first;
+        int udf = iter.second;
+        auto viter = values.find(key);
         if (viter != values.end()) {
           if ((udf & UDF_HIT) && ps.m_hit) {
             values[key + "/hit"] = viter->second * PRECISION / ps.m_hit;
@@ -323,312 +294,19 @@ void ServerStats::Aggregate(list<TimeSlot*> &slots, const std::string &agg,
 }
 
 void ServerStats::FreeSlots(list<TimeSlot*> &slots) {
-  for (list<TimeSlot*>::const_iterator iter = slots.begin();
-       iter != slots.end(); ++iter) {
-    delete *iter;
+  for (auto const& slot : slots) {
+    delete slot;
   }
   slots.clear();
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// writers
-
-class Writer {
-public:
-  explicit Writer(ostream &out) : m_out(out), m_indent(0) {}
-  virtual ~Writer() {}
-
-  virtual void writeFileHeader() = 0;
-  virtual void writeFileFooter() = 0;
-
-  // Begins writing an object which is different than a list in JSON.
-  virtual void beginObject(const char *name) = 0;
-
-  // Begins writing a list (an ordered set of potentially unnamed children)
-  // Defaults to begining an object.
-  virtual void beginList(const char *name) {
-    beginObject(name);
-  }
-
-  // Writes a string value with a given name.
-  virtual void writeEntry(const char *name, const std::string &value) = 0;
-
-  // Writes a string value with a given name.
-  virtual void writeEntry(const char *name, int64_t value) = 0;
-
-  // Ends the writing of an object.
-  virtual void endObject(const char *name) = 0;
-
-  // Ends the writing of a list. Defaults to simply ending an Object.
-  virtual void endList(const char *name) {
-    endObject(name);
-  }
-
-
-protected:
-  ostream &m_out;
-  int m_indent;
-
-  virtual void writeIndent() {
-    for (int i = 0; i < m_indent; i++) {
-      m_out << "  ";
-    }
-  }
-};
-
-class XMLWriter : public Writer {
-public:
-  explicit XMLWriter(ostream &out) : Writer(out) {}
-
-
-  virtual void writeFileHeader() {
-    m_out << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
-    if (!RuntimeOption::StatsXSL.empty()) {
-      m_out << "<?xml-stylesheet type=\"text/xsl\" href=\""
-            << RuntimeOption::StatsXSL << "\"?>\n";
-    } else if (!RuntimeOption::StatsXSLProxy.empty()) {
-      m_out << "<?xml-stylesheet type=\"text/xsl\" href=\"stats.xsl\"?>\n";
-    }
-  }
-
-  virtual void writeFileFooter() {}
-
-
-  /**
-   * In XML/HTML there is no distinction between creating a list, and creating
-   * an object with keyed attributes. (unlike the JSON format).
-   */
-  virtual void beginObject(const char *name) {
-    writeIndent();
-    m_out << '<' << name << ">\n";
-    ++m_indent;
-  }
-
-  virtual void endObject(const char *name) {
-    --m_indent;
-    writeIndent();
-    m_out << "</" << name << ">\n";
-  }
-
-  virtual void writeEntry(const char *name, const string &value) {
-    writeIndent();
-    m_out << "<entry><key>" << Escape(name) << "</key>";
-    m_out << "<value>" << Escape(value.c_str()) << "</value></entry>\n";
-  }
-
-  virtual void writeEntry(const char *name, int64_t value) {
-    writeIndent();
-    m_out << "<entry><key>" << Escape(name) << "</key>";
-    m_out << "<value>" << value << "</value></entry>\n";
-  }
-
-private:
-  static std::string Escape(const char *s) {
-    string ret;
-    for (const char *p = s; *p; p++) {
-      switch (*p) {
-      case '<':  ret += "&lt;";  break;
-      case '&':  ret += "&amp;"; break;
-      default:   ret += *p;      break;
-      }
-    }
-    return ret;
-  }
-};
-
-class JSONWriter : public Writer {
-
-protected:
-  // Whether or not we have skiped a comma for this current indent level. Valid
-  // json may not have trailing commas such as {"a":4, "b":5,} Since we are
-  // writing to a stream, we output *valid* json that has commas preceding all
-  // elements except the first, which is equivalent to outputing commas after
-  // each element except the last. Skip the preceding comma when m_justIndented.
-  bool m_justIndented;
-
-  // Stack that determines whether or not at a given object depth level, we are
-  // to be listing child objects with keyed entries. For example, in Json,
-  // inside an array, entries are not keyed. Also at the json root node, we
-  // begin at a nameless context.
-  std::stack<bool> m_namelessContextStack;
-
-  virtual void increaseIndent() {
-    ++m_indent;
-    m_justIndented = true;
-  }
-
-  /**
-   * It is *important* to set m_justIndented to false here in the event that we
-   * write objects with *no* members, we need to set it to false.
-   */
-  virtual void decreaseIndent() {
-    --m_indent;
-    m_justIndented = false;
-
-    // We should never pop off more than we pushed, but just in case someone
-    // called too many endObject's etc, we don't want a segfault.
-    if (m_namelessContextStack.size() != 0) {
-      m_namelessContextStack.pop();
-    }
-  }
-
-  /**
-   * Begins writing a containing entity (such as a list or object).
-   * See the 'isList' parameter.
-   */
-  virtual void beginContainer(const char *name, bool isList) {
-    char opener = isList ? '[' : '{';
-    beginEntity(name);
-    m_out << opener << '\n';
-
-    // Push on whether or not we're entering a nameless context
-    m_namelessContextStack.push(isList);
-    increaseIndent();
-  }
-
-  virtual void endContainer(bool isList) {
-    char closer = isList ? ']' : '}';
-    decreaseIndent();
-    writeIndent();
-    m_out << closer << '\n';
-  }
-
-  /**
-   * Writes any needed leading commas, and keyed name if appropriate.
-   * Reduces redundancy. Used whenever an entity is a child of another
-   * entity - does all the work of determining if the object should be
-   * written with/without a name and if we need a comma.
-   */
-  virtual void beginEntity(const char *name) {
-    writeIndent();
-    if (!m_justIndented) {
-      m_out << ", ";
-    }
-    if (m_namelessContextStack.size() != 0 && !m_namelessContextStack.top()) {
-      m_out << escape_for_json(name) << ": ";
-    }
-    m_justIndented = false;
-  }
-
-public:
-
-  explicit JSONWriter(ostream &out) : Writer(out),
-      m_justIndented(true) {
-
-    // A valid json object begins in the nameless context. See
-    // json.org for JSON state machine.
-    m_namelessContextStack.push(true);
-  }
-
-  virtual void writeFileHeader() {
-    beginContainer("root", false);
-  }
-
-  virtual void writeFileFooter() {
-    endContainer(false);
-  }
-
-
-  virtual void beginObject(const char *name) {
-    beginContainer(name, false);
-  }
-
-  virtual void beginList(const char *name) {
-    beginContainer(name, true);
-  }
-
-  void endObject(const char *name) {
-    endContainer(false);
-  }
-
-  void endList(const char *name) {
-    endContainer(true);
-  }
-
-  virtual void writeEntry(const char *name, const string &value) {
-    beginEntity(name);
-
-    // Now write the actual value
-    m_out << escape_for_json(value.c_str()) << "\n";
-  }
-
-  virtual void writeEntry(const char *name, int64_t value) {
-    beginEntity(name);
-
-    // Now write the actual value
-    m_out << value << '\n';
-  }
-};
-
-class HTMLWriter : public Writer {
-
-public:
-  explicit HTMLWriter(ostream &out) : Writer(out) {}
-
-  virtual void writeFileHeader() {
-    m_out << "<!doctype html>\n<html>\n<head>\n"
-             "<meta http-equiv=\"content-type\" "
-             "content=\"text/html; charset=UTF-8\">\n"
-             "<style type=\"text/css\"> table {margin-left:20px} "
-             "th {text-align:left}</style>\n"
-             "<title>HPHP Stats</title>\n"
-             "</head>\n<body>\n<table>\n<tbody>\n";
-  }
-
-  virtual void writeFileFooter() {
-    m_out << "</tbody>\n</table>\n</body>\n</html>\n";
-  }
-
-
-  /**
-   * In XML/HTML there is no distinction between creating a list, and creating
-   * an object with keyed attributes. (unlike the JSON format).
-   */
-  virtual void beginObject(const char *name) {
-    writeIndent();
-    m_out << "<tr id='" << name << "'><td colspan=2>"
-          << "<table><tbody><tr><th colspan=2>" << name << "</th></tr>\n";
-    ++m_indent;
-  }
-
-  virtual void endObject(const char *name) {
-    --m_indent;
-    writeIndent();
-    m_out << "</tbody></table></td></tr>\n";
-  }
-
-  virtual void writeEntry(const char *name, const string &value) {
-    writeIndent();
-    m_out << "<tr><td>" << Escape(name) << "</td>";
-    m_out << "<td>" << Escape(value.c_str()) << "</td></tr>\n";
-  }
-
-  virtual void writeEntry(const char *name, int64_t value) {
-    writeIndent();
-    m_out << "<tr><td>" << Escape(name) << "</td>";
-    m_out << "<td>" << value << "</td></tr>\n";
-  }
-
-private:
-  static std::string Escape(const char *s) {
-    string ret;
-    for (const char *p = s; *p; p++) {
-      switch (*p) {
-      case '<':  ret += "&lt;";  break;
-      case '&':  ret += "&amp;"; break;
-      default:   ret += *p;      break;
-      }
-    }
-    return ret;
-  }
-};
-
 ///////////////////////////////////////////////////////////////////////////////
 // static
 
 Mutex ServerStats::s_lock;
 std::vector<ServerStats*> ServerStats::s_loggers;
 bool ServerStats::s_profile_network = false;
+HealthLevel ServerStats::m_ServerHealthLevel = HealthLevel::Bold;
 IMPLEMENT_THREAD_LOCAL_NO_CHECK(ServerStats, ServerStats::s_logger);
 
 void ServerStats::LogPage(const string &url, int code) {
@@ -656,6 +334,10 @@ void ServerStats::StartRequest(const char *url, const char *clientIP,
   }
 }
 
+void ServerStats::SetServerHealthLevel(HealthLevel new_health_level) {
+  ServerStats::m_ServerHealthLevel = new_health_level;
+}
+
 void ServerStats::SetThreadMode(ThreadMode mode) {
   ServerStats::s_logger->setThreadMode(mode);
 }
@@ -709,14 +391,14 @@ void ServerStats::GetKeys(string &out, int64_t from, int64_t to) {
   CollectSlots(slots, from, to);
   set<string> allKeys;
   GetAllKeys(allKeys, slots);
-  for (set<string>::const_iterator iter = allKeys.begin();
-       iter != allKeys.end(); ++iter) {
-    out += *iter;
+  for (auto const& iter : allKeys) {
+    out += iter;
     out += "\n";
   }
 }
 
-void ServerStats::Report(string &out, Format format, int64_t from, int64_t to,
+void ServerStats::Report(string &out, Writer::Format format,
+                         int64_t from, int64_t to,
                          const std::string &agg, const std::string &keys,
                          const std::string &url, int code,
                          const std::string &prefix) {
@@ -729,48 +411,45 @@ void ServerStats::Report(string &out, Format format, int64_t from, int64_t to,
   FreeSlots(slots);
 }
 
-void ServerStats::Report(string &output, Format format,
+void ServerStats::Report(string &output, Writer::Format format,
                          const list<TimeSlot*> &slots,
                          const std::string &prefix) {
   std::ostringstream out;
-  if (format == Format::KVP) {
+  if (format == Writer::Format::KVP) {
     bool first = true;
-    for (list<TimeSlot*>::const_iterator iter = slots.begin();
-         iter != slots.end(); ++iter) {
+    for (auto const& s : slots) {
       if (first) {
         first = false;
       } else {
         out << ",\n";
       }
-      TimeSlot *s = *iter;
       if (s->m_time) {
         out << s->m_time << ": ";
       }
       out << "{";
-      for (PageStatsMap::const_iterator piter = s->m_pages.begin();
-           piter != s->m_pages.end(); ++piter) {
-        const PageStats &ps = piter->second;
+      for (auto const& page : s->m_pages) {
+        auto const& ps = page.second;
         string key = prefix;
         if (!ps.m_url.empty()) {
           key += ps.m_url;
         }
         if (ps.m_code) {
           key += "$";
-          key += lexical_cast<string>(ps.m_code);
+          key += folly::to<string>(ps.m_code);
         }
         if (!key.empty()) {
           key += ".";
         }
         bool firstKey = true;
-        for (CounterMap::const_iterator viter =
-               ps.m_values.begin(); viter != ps.m_values.end(); ++viter) {
+        for (auto const& kvpair : ps.m_values) {
           if (firstKey) {
             firstKey = false;
           } else {
             out << ", ";
           }
-          out << escape_for_json((key + viter->first->getString()).c_str())
-              << ": " << viter->second;
+          out << Writer::escape_for_json(
+                  (key + kvpair.first->getString()).c_str())
+              << ": " << kvpair.second;
         }
       }
       out << "}\n";
@@ -778,47 +457,31 @@ void ServerStats::Report(string &output, Format format,
 
   } else {
     Writer *w;
-    if (format == Format::XML) {
+    if (format == Writer::Format::XML) {
       w = new XMLWriter(out);
-    } else if (format == Format::HTML) {
+    } else if (format == Writer::Format::HTML) {
       w = new HTMLWriter(out);
     } else {
-      assert(format == Format::JSON);
+      assert(format == Writer::Format::JSON);
       w = new JSONWriter(out);
     }
 
     w->writeFileHeader();
     w->beginObject("stats");
-    for (list<TimeSlot*>::const_iterator iter = slots.begin();
-         iter != slots.end(); ++iter) {
-      TimeSlot *s = *iter;
-      if (s->m_time) {
-        w->beginObject("slot");
-        w->writeEntry("time", s->m_time * RuntimeOption::StatsSlotDuration);
-      }
-      w->beginList("pages");
-      for (PageStatsMap::const_iterator piter = s->m_pages.begin();
-           piter != s->m_pages.end(); ++piter) {
-        const PageStats &ps = piter->second;
-        w->beginObject("page");
-        w->writeEntry("url", ps.m_url);
-        w->writeEntry("code", ps.m_code);
-        w->writeEntry("hit", ps.m_hit);
 
-        w->beginObject("details");
-        for (CounterMap::const_iterator viter =
-               ps.m_values.begin(); viter != ps.m_values.end(); ++viter) {
-          w->writeEntry(viter->first->getString().c_str(), viter->second);
-        }
-        w->endObject("details");
-
-        w->endObject("page");
-      }
-      w->endList("pages");
-      if (s->m_time) {
-        w->endObject("slot");
-      }
+    if (format != Writer::Format::JSON) {
+      // In JSON, this would create multiple entries with the same 'slot' key.
+      // This isn't valid, and most implementations can't read it.
+      //   https://github.com/facebook/hhvm/issues/3331
+      ReportSlots(w, slots);
+    } else {
+      assert(format == Writer::Format::JSON);
+      // Create a list instead :)
+      w->beginList("slots");
+      ReportSlots(w, slots);
+      w->endList("slots");
     }
+
     w->endObject("stats");
     w->writeFileFooter();
 
@@ -828,6 +491,35 @@ void ServerStats::Report(string &output, Format format,
   output = out.str();
 }
 
+void ServerStats::ReportSlots(Writer* w, const std::list<TimeSlot*> &slots) {
+  for (auto const& s : slots) {
+    if (s->m_time) {
+      w->beginObject("slot");
+      w->writeEntry("time", s->m_time * RuntimeOption::StatsSlotDuration);
+    }
+    w->beginList("pages");
+    for (auto const& page : s->m_pages) {
+      auto const& ps = page.second;
+      w->beginObject("page");
+      w->writeEntry("url", ps.m_url);
+      w->writeEntry("code", ps.m_code);
+      w->writeEntry("hit", ps.m_hit);
+
+      w->beginObject("details");
+      for (auto const& kvpair : ps.m_values) {
+        w->writeEntry(kvpair.first->getString().c_str(), kvpair.second);
+      }
+      w->endObject("details");
+
+      w->endObject("page");
+    }
+    w->endList("pages");
+    if (s->m_time) {
+      w->endObject("slot");
+    }
+  }
+}
+
 static std::string format_duration(timeval &duration) {
   string ret;
   if (duration.tv_sec > 0 || duration.tv_usec > 0) {
@@ -837,18 +529,15 @@ static std::string format_duration(timeval &duration) {
     int hours = minutes / 60;
     minutes = minutes % 60;
     if (hours) {
-      ret += lexical_cast<string>(hours) + " hour";
+      ret += folly::to<string>(hours) + " hour";
       ret += (hours == 1) ? " " : "s ";
     }
     if (minutes || (hours && seconds)) {
-      ret += lexical_cast<string>(minutes) + " minute";
+      ret += folly::to<string>(minutes) + " minute";
       ret += (minutes == 1) ? " " : "s ";
     }
     if (seconds || minutes || hours) {
-      char buf[7];
-      snprintf(buf, sizeof(buf), "%.3f", seconds);
-      buf[sizeof(buf) - 1] = '\0';
-      ret += lexical_cast<string>(buf) + " second";
+      ret += folly::stringPrintf("%.3f", seconds);
       ret += (seconds == 1) ? "" : "s";
     }
   } else {
@@ -857,15 +546,15 @@ static std::string format_duration(timeval &duration) {
   return ret;
 }
 
-void ServerStats::ReportStatus(std::string &output, Format format) {
+void ServerStats::ReportStatus(std::string &output, Writer::Format format) {
   std::ostringstream out;
   Writer *w;
-  if (format == Format::XML) {
+  if (format == Writer::Format::XML) {
     w = new XMLWriter(out);
-  } else if (format == Format::HTML) {
+  } else if (format == Writer::Format::HTML) {
     w = new HTMLWriter(out);
   } else {
-    assert(format == Format::JSON);
+    assert(format == Writer::Format::JSON);
     w = new JSONWriter(out);
   }
 
@@ -886,19 +575,15 @@ void ServerStats::ReportStatus(std::string &output, Format format) {
   w->writeEntry("debug", "no");
 #endif
 
-#ifdef HOTPROFILER
   w->writeEntry("hotprofiler", "yes");
-#else
-  w->writeEntry("hotprofiler", "no");
-#endif
 
   timeval up;
   up.tv_sec = now - HttpServer::StartTime;
   up.tv_usec = 0;
-  w->writeEntry("now", DateTime(now).
-                toString(DateTime::DateFormatCookie).data());
-  w->writeEntry("start", DateTime(HttpServer::StartTime).
-                toString(DateTime::DateFormatCookie).data());
+  w->writeEntry("now", req::make<DateTime>(now)->
+                       toString(DateTime::DateFormatCookie).data());
+  w->writeEntry("start", req::make<DateTime>(HttpServer::StartTime)->
+                         toString(DateTime::DateFormatCookie).data());
   w->writeEntry("up", format_duration(up));
   w->endObject("process");
 
@@ -932,8 +617,8 @@ void ServerStats::ReportStatus(std::string &output, Format format) {
     w->writeEntry("tid", (int64_t)ts.m_threadPid);
     w->writeEntry("req", ts.m_requestCount);
     w->writeEntry("bytes", ts.m_writeBytes);
-    w->writeEntry("start", DateTime(ts.m_start.tv_sec).
-                  toString(DateTime::DateFormatCookie).data());
+    w->writeEntry("start", req::make<DateTime>(ts.m_start.tv_sec)->
+                           toString(DateTime::DateFormatCookie).data());
     w->writeEntry("duration", format_duration(duration));
     if (ts.m_requestCount > 0) {
       auto const stats = ts.m_mm->getStatsCopy();
@@ -995,11 +680,10 @@ Array ServerStats::EndNetworkProfile() {
     Lock lock(ss->m_lock, false);
 
     IOStatusMap &status = ss->m_ioProfiles;
-    for (IOStatusMap::const_iterator iter = status.begin();
-         iter != status.end(); ++iter) {
-      ret.set(String(iter->first),
-              make_map_array(s_ct, iter->second.count,
-                             s_wt, iter->second.wall_time));
+    for (auto const& iter : status) {
+      ret.set(String(iter.first),
+              make_map_array(s_ct, iter.second.count,
+                             s_wt, iter.second.wall_time));
     }
     status.clear();
   }
@@ -1077,7 +761,7 @@ void ServerStats::logPage(const string &url, int code) {
         break; // we have cleared all slots, good enough
       }
     }
-    TimeSlot &ts = m_slots[slot];
+    auto &ts = m_slots[slot];
     if (ts.m_time != now) {
       if (ts.m_time && m_min <= ts.m_time) {
         m_min = ts.m_time + 1;
@@ -1085,7 +769,7 @@ void ServerStats::logPage(const string &url, int code) {
       ts.m_time = now;
       ts.m_pages.clear();
     }
-    PageStats &ps = ts.m_pages[url + lexical_cast<string>(code)];
+    auto &ps = ts.m_pages[url + folly::to<string>(code)];
     ps.m_url = url;
     ps.m_code = code;
     ps.m_hit++;
@@ -1142,14 +826,14 @@ void ServerStats::logBytes(int64_t bytes) {
 static void safe_copy(char *dest, const char *src, int max) {
   int len = strlen(src) + 1;
   dest[--max] = '\0';
-  strncpy(dest, src, len > max ? max : len);
+  memcpy(dest, src, len > max ? max : len);
 }
 
 void ServerStats::startRequest(const char *url, const char *clientIP,
                                const char *vhost) {
   ++m_threadStatus.m_requestCount;
 
-  m_threadStatus.m_mm = ThreadInfo::s_threadInfo->m_mm;
+  m_threadStatus.m_mm = &MM();
   gettimeofday(&m_threadStatus.m_start, 0);
   memset(&m_threadStatus.m_done, 0, sizeof(m_threadStatus.m_done));
   m_threadStatus.m_mode = ThreadMode::Processing;
@@ -1246,16 +930,15 @@ void ServerStats::setThreadIOStatus(const char *name, const char *addr,
 }
 
 Array ServerStats::getThreadIOStatuses() {
-  Array ret;
   IOStatusMap &status = m_threadStatus.m_ioStatuses;
-  for (IOStatusMap::const_iterator iter = status.begin();
-       iter != status.end(); ++iter) {
-    ret.set(String(iter->first),
-            make_map_array(s_ct, iter->second.count,
-                           s_wt, iter->second.wall_time));
+  ArrayInit ret(status.size(), ArrayInit::Map{});
+  for (auto const& iter : status) {
+    ret.set(String(iter.first),
+            make_map_array(s_ct, iter.second.count,
+                           s_wt, iter.second.wall_time));
   }
   status.clear();
-  return ret;
+  return ret.toArray();
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1327,7 +1010,7 @@ IOStatusHelper::IOStatusHelper(const char *name,
     }
     if (port) {
       msg += ":";
-      msg += boost::lexical_cast<std::string>(port);
+      msg += folly::to<std::string>(port);
     }
     ServerStats::SetThreadIOStatus(name, msg.c_str());
   }
@@ -1361,11 +1044,10 @@ void set_curl_statuses(CURL *cp, const char *url) {
 ///////////////////////////////////////////////////////////////////////////////
 
 void server_stats_log_mutex(const std::string &stack, int64_t elapsed_us) {
-  char buf[128];
-  snprintf(buf, sizeof(buf), "mutex.%s.", stack.c_str());
-  ServerStats::Log(string(buf) + "hit", 1);
-  ServerStats::Log(string(buf) + "time", elapsed_us);
+  auto const prefix = folly::to<string>("mutex.", stack);
+  ServerStats::Log(prefix + ".hit", 1);
+  ServerStats::Log(prefix + ".time", elapsed_us);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-}
+};
