@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,9 +15,13 @@
 */
 
 #include "hphp/runtime/server/transport.h"
+
+#include <boost/algorithm/string.hpp>
+
 #include "hphp/runtime/server/server.h"
 #include "hphp/runtime/server/upload.h"
 #include "hphp/runtime/server/server-stats.h"
+#include "hphp/runtime/base/builtin-functions.h"
 #include "hphp/runtime/base/file.h"
 #include "hphp/runtime/base/string-util.h"
 #include "hphp/runtime/base/datetime.h"
@@ -26,16 +30,17 @@
 #include "hphp/runtime/base/zend-url.h"
 #include "hphp/runtime/base/type-conversions.h"
 #include "hphp/runtime/server/access-log.h"
+#include "hphp/runtime/server/http-protocol.h"
 #include "hphp/runtime/ext/openssl/ext_openssl.h"
-#include "hphp/system/constants.h"
-#include "hphp/util/compression.h"
-#include "hphp/util/text-util.h"
-#include "hphp/util/service-data.h"
-#include "hphp/util/logger.h"
 #include "hphp/util/compatibility.h"
+#include "hphp/util/compression.h"
+#include "hphp/util/hardware-counter.h"
+#include "hphp/util/logger.h"
+#include "hphp/util/service-data.h"
+#include "hphp/util/text-util.h"
 #include "hphp/util/timer.h"
-#include "hphp/runtime/base/hardware-counter.h"
-#include "folly/String.h"
+#include "hphp/runtime/ext/string/ext_string.h"
+#include <folly/String.h>
 
 namespace HPHP {
 ///////////////////////////////////////////////////////////////////////////////
@@ -327,9 +332,7 @@ bool Transport::splitHeader(const String& header, String &name, const char *&val
       if (pos2 == String::npos) pos2 = header.size();
       if (pos2 - pos1 > 1) {
         setResponse(atoi(header.data() + pos1),
-                    getResponseInfo().empty() ? "splitHeader"
-                                              : getResponseInfo().c_str()
-                   );
+                    header.size() - pos2 > 1 ? header.data() + pos2 : nullptr);
         return false;
       }
     }
@@ -364,7 +367,7 @@ void Transport::addHeaderNoLock(const char *name, const char *value) {
       setResponse(302);
     }
     */
-    setResponse(302, "forced.302");
+    setResponse(302);
   }
 }
 
@@ -448,7 +451,7 @@ bool Transport::acceptEncoding(const char *encoding) {
     assert(scTokens.size() > 0);
     // lhs contains the encoding
     // rhs, if it exists, contains the qvalue
-    std::string& lhs = scTokens[0];
+    std::string lhs = boost::trim_copy(scTokens[0]);
     if (strcasecmp(lhs.c_str(), encoding) == 0) {
       return true;
     }
@@ -514,6 +517,11 @@ bool Transport::decideCompression() {
   return false;
 }
 
+void Transport::setResponse(int code, const char *info) {
+  m_responseCode = code;
+  m_responseCodeInfo = info ? info : HttpProtocol::GetReasonString(code);
+}
+
 std::string Transport::getHTTPVersion() const {
   return "1.1";
 }
@@ -533,23 +541,43 @@ String Transport::getMimeType() {
 ///////////////////////////////////////////////////////////////////////////////
 // cookies
 
+namespace {
+
+// Make sure cookie names do not contain any illegal characters.
+// Throw a fatal exception if one does.
+void validateCookieNameString(const String& str) {
+  if (!str.empty() && strpbrk(str.data(), "=,; \t\r\n\013\014")) {
+    raise_error("Cookie names can not contain any of the following "
+                "'=,; \\t\\r\\n\\013\\014'");
+  }
+}
+
+// Make sure a component (path, value, domain) of a cookie does not
+// contain any illegal characters.  Throw a fatal exception if it
+// does.
+void validateCookieString(const String& str, const char* component) {
+  if (!str.empty() && strpbrk(str.data(), ",; \t\r\n\013\014")) {
+    raise_error("Cookie %s can not contain any of the following "
+                "',; \\t\\r\\n\\013\\014'", component);
+  }
+}
+
+}
+
 bool Transport::setCookie(const String& name, const String& value, int64_t expire /* = 0 */,
                           const String& path /* = "" */, const String& domain /* = "" */,
                           bool secure /* = false */,
                           bool httponly /* = false */,
                           bool encode_url /* = true */) {
-  if (!name.empty() && strpbrk(name.data(), "=,; \t\r\n\013\014")) {
-    Logger::Warning("Cookie names can not contain any of the following "
-                    "'=,; \\t\\r\\n\\013\\014'");
-    return false;
+  validateCookieNameString(name);
+
+  if (!encode_url) {
+    validateCookieString(value, "values");
   }
 
-  if (!encode_url &&
-      !value.empty() && strpbrk(value.data(), ",; \t\r\n\013\014")) {
-    Logger::Warning("Cookie values can not contain any of the following "
-                    "',; \\t\\r\\n\\013\\014'");
-    return false;
-  }
+  validateCookieString(path, "paths");
+
+  validateCookieString(domain, "domains");
 
   String encoded_value;
   int len = 0;
@@ -569,7 +597,8 @@ bool Transport::setCookie(const String& name, const String& value, int64_t expir
      * so in order to force cookies to be deleted, even on MSIE, we
      * pick an expiry date in the past
      */
-    String sdt = DateTime(1, true).toString(DateTime::DateFormat::Cookie);
+    String sdt = req::make<DateTime>(1, true)->
+      toString(DateTime::DateFormat::Cookie);
     cookie += name.data();
     cookie += "=deleted; expires=";
     cookie += sdt.data();
@@ -584,8 +613,8 @@ bool Transport::setCookie(const String& name, const String& value, int64_t expir
         return false;
       }
       cookie += "; expires=";
-      String sdt =
-        DateTime(expire, true).toString(DateTime::DateFormat::Cookie);
+      String sdt = req::make<DateTime>(expire, true)->
+        toString(DateTime::DateFormat::Cookie);
       cookie += sdt.data();
       cookie += "; Max-Age=";
       String sdelta = toString( expire - time(0) );
@@ -661,6 +690,19 @@ void Transport::prepareHeaders(bool compressed, bool chunked,
     addHeaderImpl("Set-Cookie", iter->c_str());
   }
 
+  if (RuntimeOption::ServerAddVaryEncoding) {
+    /*
+     * Our response may vary depending on the Accept-Encoding header if
+     *  - we compressed it, and compression was not forced; or
+     *  - we didn't compress it because the client does not accept gzip
+     */
+    if (compressed ?
+        m_compressionDecision != CompressionDecision::HasTo :
+        (isCompressionEnabled() && !acceptEncoding("gzip"))) {
+      addHeaderImpl("Vary", "Accept-Encoding");
+    }
+  }
+
   if (compressed) {
     addHeaderImpl("Content-Encoding", "gzip");
     removeHeaderImpl("Content-Length");
@@ -692,13 +734,15 @@ void Transport::prepareHeaders(bool compressed, bool chunked,
 
   if (m_responseHeaders.find("Content-Type") == m_responseHeaders.end() &&
       m_responseCode != 304) {
-    std::string contentType = "text/html; charset="
-                              + IniSetting::Get("default_charset");
+    std::string contentType = "text/html";
+    if (IniSetting::Get("default_charset") != "") {
+      contentType += "; charset=" + IniSetting::Get("default_charset");
+    }
     addHeaderImpl("Content-Type", contentType.c_str());
   }
 
   if (RuntimeOption::ExposeHPHP) {
-    addHeaderImpl("X-Powered-By", ("HHVM/" + k_HHVM_VERSION).c_str());
+    addHeaderImpl("X-Powered-By", (String("HHVM/") + HHVM_VERSION).c_str());
   }
 
   if ((RuntimeOption::ExposeXFBServer || RuntimeOption::ExposeXFBDebug) &&
@@ -765,16 +809,28 @@ StringHolder Transport::prepareResponse(const void *data, int size,
   }
   if (compressed || !isCompressionEnabled() ||
       m_compressionDecision == CompressionDecision::ShouldNot) {
-    return std::move(response);
+    return response;
   }
 
   // There isn't that much need to gzip response, when it can fit into one
   // Ethernet packet (1500 bytes), unless we are doing chunked encoding,
-  // where we don't really know if next chunk will benefit from compresseion.
+  // where we don't really know if next chunk will benefit from compression.
   if (m_chunkedEncoding || size > 1000 ||
       m_compressionDecision == CompressionDecision::HasTo) {
+    String compression;
+    int compressionLevel = RuntimeOption::GzipCompressionLevel;
+    IniSetting::Get("zlib.output_compression", compression);
+    if (compression.size() == 2 && bstrcaseeq(compression.data(), "on", 2)) {
+      String compressionLevelStr;
+      IniSetting::Get("zlib.output_compression_level", compressionLevelStr);
+      int level = compressionLevelStr.toInt64();
+      if (level > compressionLevel &&
+          level <= RuntimeOption::GzipMaxCompressionLevel) {
+        compressionLevel = level;
+      }
+    }
     if (m_compressor == nullptr) {
-      m_compressor = new StreamCompressor(RuntimeOption::GzipCompressionLevel,
+      m_compressor = new StreamCompressor(compressionLevel,
                                           CODING_GZIP, true);
     }
     int len = size;
@@ -789,11 +845,11 @@ StringHolder Transport::prepareResponse(const void *data, int size,
       }
     } else {
       Logger::Error("Unable to compress response: level=%d len=%d",
-                    RuntimeOption::GzipCompressionLevel, len);
+                    compressionLevel, len);
     }
   }
 
-  return std::move(response);
+  return response;
 }
 
 bool Transport::setHeaderCallback(const Variant& callback) {
@@ -805,11 +861,11 @@ bool Transport::setHeaderCallback(const Variant& callback) {
   return true;
 }
 
-void Transport::sendRawLocked(void *data, int size, int code /* = 200 */,
-                              bool compressed /* = false */,
-                              bool chunked /* = false */,
-                              const char *codeInfo /* = "" */
-                              ) {
+void Transport::sendRaw(void *data, int size, int code /* = 200 */,
+                        bool compressed /* = false */,
+                        bool chunked /* = false */,
+                        const char *codeInfo /* = nullptr */
+                       ) {
   // There are post-send functions that can run. Any output from them should
   // be ignored as it doesn't make sense to try and send data after the
   // request has ended.
@@ -820,6 +876,13 @@ void Transport::sendRawLocked(void *data, int size, int code /* = 200 */,
   if (!compressed && RuntimeOption::ForceChunkedEncoding) {
     chunked = true;
   }
+
+  // I don't think there is any need to send an empty chunk, other than sending
+  // out headers earlier, which seems to be a useless feature.
+  if (size == 0 && (chunked || m_chunkedEncoding)) {
+    return;
+  }
+
   if (m_chunkedEncoding) {
     chunked = true;
     assert(!compressed);
@@ -828,11 +891,16 @@ void Transport::sendRawLocked(void *data, int size, int code /* = 200 */,
     assert(!compressed);
   }
 
-  // I don't think there is any need to send an empty chunk, other than sending
-  // out headers earlier, which seems to be a useless feature.
-  if (chunked && size == 0) {
-    return;
-  }
+  sendRawInternal(data, size, code, compressed, codeInfo);
+}
+
+void Transport::sendRawInternal(const void *data, int size,
+                                int code /* = 200 */,
+                                bool compressed /* = false */,
+                                const char *codeInfo /* = nullptr */
+                               ) {
+
+  bool chunked = m_chunkedEncoding;
 
   if (!m_headerCallbackDone && !cellIsNull(&m_headerCallback)) {
     // We could use m_headerSent here, however it seems we can still
@@ -853,8 +921,7 @@ void Transport::sendRawLocked(void *data, int size, int code /* = 200 */,
   StringHolder response = prepareResponse(data, size, compressed, !chunked);
 
   if (m_responseCode < 0) {
-    m_responseCode = code;
-    m_responseCodeInfo = codeInfo ? codeInfo: "";
+    setResponse(code, codeInfo);
   }
 
   // HTTP header handling
@@ -865,7 +932,7 @@ void Transport::sendRawLocked(void *data, int size, int code /* = 200 */,
 
   m_responseSize += response.size();
   ServerStats::SetThreadMode(ServerStats::ThreadMode::Writing);
-  sendImpl(response.data(), response.size(), m_responseCode, chunked);
+  sendImpl(response.data(), response.size(), m_responseCode, chunked, false);
   ServerStats::SetThreadMode(ServerStats::ThreadMode::Processing);
 
   ServerStats::LogBytes(size);
@@ -875,34 +942,34 @@ void Transport::sendRawLocked(void *data, int size, int code /* = 200 */,
   }
 }
 
-void Transport::sendRaw(void *data, int size, int code /* = 200 */,
-                        bool compressed /* = false */,
-                        bool chunked /* = false */,
-                        const char *codeInfo /* = "" */
-                        ) {
-  sendRawLocked(data, size, code, compressed, chunked, codeInfo);
-}
-
 void Transport::onSendEnd() {
+  bool eomSent = false;
   if (m_compressor && m_chunkedEncoding) {
+    assert(m_headerSent);
     bool compressed = false;
     StringHolder response = prepareResponse("", 0, compressed, true);
-    sendImpl(response.data(), response.size(), m_responseCode, true);
+    sendImpl(response.data(), response.size(), m_responseCode, true, true);
+    eomSent = true;
+  } else if (!m_headerSent) {
+    m_compressionDecision = CompressionDecision::ShouldNot;
+    sendRawInternal("", 0);
   }
   auto httpResponseStats = ServiceData::createTimeseries(
     folly::to<std::string>(HTTP_RESPONSE_STATS_PREFIX, getResponseCode()),
     {ServiceData::StatsType::SUM});
   httpResponseStats->addValue(1);
-  onSendEndImpl();
+  if (!eomSent) {
+    onSendEndImpl();
+  }
   // Record that we have ended the request so any further output is discarded.
   m_sendEnded = true;
 }
 
 void Transport::redirect(const char *location, int code /* = 302 */,
-                         const char *info) {
+                         const char *info /* = nullptr */) {
   addHeaderImpl("Location", location);
   setResponse(code, info);
-  sendStringLocked("Moved", code);
+  sendString("Moved", code);
 }
 
 void Transport::onFlushProgress(int writtenSize, int64_t delayUs) {
