@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -18,15 +18,15 @@
 #include "hphp/runtime/server/upload.h"
 #include "hphp/runtime/base/program-functions.h"
 #include "hphp/runtime/base/runtime-option.h"
-#include "hphp/runtime/base/hphp-system.h"
 #include "hphp/runtime/base/request-local.h"
 #include "hphp/runtime/base/zend-printf.h"
 #include "hphp/runtime/base/php-globals.h"
-#include "hphp/runtime/ext/ext_apc.h"
+#include "hphp/runtime/ext/apc/ext_apc.h"
 #include "hphp/util/logger.h"
 #include "hphp/runtime/base/string-util.h"
 #include "hphp/util/text-util.h"
 #include "hphp/runtime/base/request-event-handler.h"
+#include <folly/FileUtil.h>
 
 using std::set;
 
@@ -51,6 +51,7 @@ struct Rfc1867Data final : RequestEventHandler {
   void requestShutdown() override {
     if (!rfc1867UploadedFiles.empty()) destroy_uploaded_files();
   }
+  void vscan(IMarker&) const override {}
 };
 IMPLEMENT_STATIC_REQUEST_LOCAL(Rfc1867Data, s_rfc1867_data);
 
@@ -73,7 +74,7 @@ static void safe_php_register_variable(char *var, const Variant& val,
 #define MAX_SIZE_ANONNAME 33
 
 /* Errors */
-#define UPLOAD_ERROR_OK   0  /* File upload succesful */
+#define UPLOAD_ERROR_OK   0  /* File upload successful */
 #define UPLOAD_ERROR_A    1  /* Uploaded file exceeded upload_max_filesize */
 #define UPLOAD_ERROR_B    2  /* Uploaded file exceeded MAX_FILE_SIZE */
 #define UPLOAD_ERROR_C    3  /* Partially uploaded */
@@ -425,7 +426,8 @@ static int find_boundary(multipart_buffer *self, char *boundary) {
 static int multipart_buffer_headers(multipart_buffer *self,
                                     header_list &header) {
   char *line;
-  std::pair<std::string, std::string> prev_entry;
+  std::string key;
+  std::string buf_value;
   std::pair<std::string, std::string> entry;
 
   /* didn't find boundary, abort */
@@ -438,8 +440,6 @@ static int multipart_buffer_headers(multipart_buffer *self,
   while( (line = get_line(self)) && strlen(line) > 0 )
   {
     /* add header to table */
-
-    char *key = line;
     char *value = nullptr;
 
     /* space in the beginning means same header */
@@ -448,19 +448,27 @@ static int multipart_buffer_headers(multipart_buffer *self,
     }
 
     if (value) {
-      *value = 0;
+      if (!buf_value.empty() && !key.empty() ) {
+        entry = std::make_pair(key, buf_value);
+        header.push_back(entry);
+        buf_value.erase();
+        key.erase();
+      }
+      *value = '\0';
       do { value++; } while(isspace(*value));
-      entry = std::make_pair(key, value);
-    } else if (!header.empty()) {
+      key.assign(line);
+      buf_value.append(value);
+    } else if (!buf_value.empty() ) {
       /* If no ':' on the line, add to previous line */
-      entry = std::make_pair(prev_entry.first, prev_entry.second + line);
-      header.pop_back();
+      buf_value.append(line);
     } else {
       continue;
     }
+  }
 
+  if (!buf_value.empty() && !key.empty()) {
+    entry = std::make_pair(key, buf_value);
     header.push_back(entry);
-    prev_entry = entry;
   }
 
   return 1;
@@ -711,6 +719,7 @@ void rfc1867PostHandler(Transport* transport,
   int fd=-1;
   void *event_extra_data = nullptr;
   unsigned int llen = 0;
+  int upload_count = RuntimeOption::MaxFileUploads;
 
   /* Initialize the buffer */
   if (!(mbuff = multipart_buffer_new(transport,
@@ -799,7 +808,7 @@ void rfc1867PostHandler(Transport* transport,
         new_val_len = value_len;
         if (php_rfc1867_callback != nullptr) {
           multipart_event_formdata event_formdata;
-          size_t newlength = 0;
+          size_t newlength = new_val_len;
 
           event_formdata.post_bytes_processed = mbuff->read_post_bytes;
           event_formdata.name = param;
@@ -831,6 +840,11 @@ void rfc1867PostHandler(Transport* transport,
       /* If file_uploads=off, skip the file part */
       if (!RuntimeOption::EnableFileUploads) {
         skip_upload = 1;
+      } else if (upload_count <= 0) {
+        Logger::Warning(
+          "Maximum number of allowable file uploads has been exceeded"
+        );
+        skip_upload = 1;
       }
 
       /* Return with an error if the posted data is garbled */
@@ -880,6 +894,7 @@ void rfc1867PostHandler(Transport* transport,
         snprintf(path, sizeof(path), "%s/XXXXXX",
                  RuntimeOption::UploadTmpDir.c_str());
         fd = mkstemp(path);
+        upload_count--;
         if (fd == -1) {
           Logger::Warning("Unable to open temporary file");
           Logger::Warning("File upload error - unable to create a "
@@ -961,8 +976,7 @@ void rfc1867PostHandler(Transport* transport,
           cancel_upload = UPLOAD_ERROR_B;
         } else if (blen > 0) {
 
-          wlen = write(fd, buff, blen);
-
+          wlen = folly::writeFull(fd, buff, blen);
           if (wlen < blen) {
             Logger::Verbose("Only %zd bytes were written, expected to "
                             "write %zd", wlen, blen);
