@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -30,24 +30,28 @@
 #include "hphp/runtime/vm/name-value-table.h"
 #include "hphp/runtime/vm/unit.h"
 
+#include "hphp/runtime/vm/jit/types.h"
+
 #include "hphp/util/arena.h"
 
 #include <type_traits>
 
 namespace HPHP {
+///////////////////////////////////////////////////////////////////////////////
 
-/**
- * These macros allow us to easily change the arguments to iop*() opcode
- * implementations.
- */
-#define IOP_ARGS        PC& pc
-#define IOP_PASS_ARGS   pc
-#define IOP_PASS(pc)    pc
+struct ActRec;
+struct Func;
+struct Resumable;
+
+///////////////////////////////////////////////////////////////////////////////
 
 #define EVAL_FILENAME_SUFFIX ") : eval()'d code"
 
+// perform the set(op) operation on lhs & rhs, leaving the result in lhs.
+// The old value of lhs is decrefed. Caller must call tvToCell() if lhs or
+// rhs might be a ref.
 ALWAYS_INLINE
-void SETOP_BODY_CELL(Cell* lhs, SetOpOp op, Cell* rhs) {
+void setopBody(Cell* lhs, SetOpOp op, Cell* rhs) {
   assert(cellIsPlausible(*lhs));
   assert(cellIsPlausible(*rhs));
 
@@ -73,13 +77,7 @@ void SETOP_BODY_CELL(Cell* lhs, SetOpOp op, Cell* rhs) {
   not_reached();
 }
 
-ALWAYS_INLINE
-void SETOP_BODY(TypedValue* lhs, SetOpOp op, Cell* rhs) {
-  SETOP_BODY_CELL(tvToCell(lhs), op, rhs);
-}
-
-class Func;
-struct ActRec;
+///////////////////////////////////////////////////////////////////////////////
 
 struct ExtraArgs : private boost::noncopyable {
   /*
@@ -114,6 +112,11 @@ struct ExtraArgs : private boost::noncopyable {
    */
   TypedValue* getExtraArg(unsigned argInd) const;
 
+  template<class F> void scan(F& mark, unsigned int nargs) const {
+    for (unsigned i = 0; i < nargs; ++i) {
+      mark(*(m_extraArgs + i));
+    }
+  }
 private:
   ExtraArgs();
   ~ExtraArgs();
@@ -147,7 +150,10 @@ class VarEnv {
   NameValueTable m_nvTable;
   ExtraArgs* m_extraArgs;
   uint16_t m_depth;
-  bool m_global;
+  const bool m_global;
+
+ public:
+  template<class F> void scan(F& mark) const;
 
  public:
   explicit VarEnv();
@@ -155,11 +161,15 @@ class VarEnv {
   explicit VarEnv(const VarEnv* varEnv, ActRec* fp);
   ~VarEnv();
 
+  // Free the VarEnv and locals for the given frame
+  // which must have a VarEnv
+  static void deallocate(ActRec* fp);
+
   // Allocates a local VarEnv and attaches it to the existing FP.
   static VarEnv* createLocal(ActRec* fp);
 
   // Allocate a global VarEnv.  Initially not attached to any frame.
-  static VarEnv* createGlobal();
+  static void createGlobal();
 
   VarEnv* clone(ActRec* fp) const;
 
@@ -184,6 +194,36 @@ class VarEnv {
   TypedValue* getExtraArg(unsigned argInd) const;
 };
 
+/*
+ * Action taken to handle any extra arguments passed for a function call.
+ */
+enum class ExtraArgsAction {
+  None,     // no extra arguments; zero out m_extraArgs
+  Discard,  // discard extra arguments
+  Variadic, // populate `...$args' parameter
+  MayUseVV, // create ExtraArgs
+  VarAndVV, // both of the above
+};
+
+inline ExtraArgsAction extra_args_action(const Func* func, uint32_t argc) {
+  using Action = ExtraArgsAction;
+
+  auto const nparams = func->numNonVariadicParams();
+  if (argc <= nparams) return Action::None;
+
+  if (LIKELY(func->discardExtraArgs())) {
+    return Action::Discard;
+  }
+  if (func->attrs() & AttrMayUseVV) {
+    return func->hasVariadicCaptureParam() ? Action::VarAndVV
+                                           : Action::MayUseVV;
+  }
+  assertx(func->hasVariadicCaptureParam());
+  return Action::Variadic;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
 /*
  * An "ActRec" is a call activation record. The ordering of the fields assumes
  * that stacks grow toward lower addresses.
@@ -211,11 +251,8 @@ struct ActRec {
     // dependency.
     TypedValue _dummyA;
     struct {
-      union {
-        ActRec* m_sfp;         // Previous hardware frame pointer/ActRec.
-        uint64_t m_savedRbp;   // TODO: Remove. Used by debugger macros.
-      };
-      uint64_t m_savedRip;     // In-TC address to return to.
+      ActRec* m_sfp;         // Previous hardware frame pointer/ActRec.
+      uint64_t m_savedRip;   // In-TC address to return to.
     };
   };
   union {
@@ -225,10 +262,8 @@ struct ActRec {
       uint32_t m_soff;         // Saved offset of caller from beginning of
                                //   caller's Func's bytecode.
 
-      // Bits 0-28 are the number of function args.
-      // Bit 29 is whether the locals were already decrefd (used by unwinder)
-      // Bit 30 is whether this ActRec is embedded in a Resumable object.
-      // Bit 31 is whether this ActRec came from FPushCtor*.
+      // Bits 0-27 are the number of function args.  Bits 28-31 are values from
+      // the Flags enum.
       uint32_t m_numArgsAndFlags;
     };
   };
@@ -238,7 +273,7 @@ struct ActRec {
     struct {
       union {
         ObjectData* m_this;    // This.
-        Class* m_cls;      // Late bound class.
+        Class* m_cls;          // Late bound class.
       };
       union {
         VarEnv* m_varEnv;       // Variable environment; only used when the
@@ -256,83 +291,94 @@ struct ActRec {
   ActRec* sfp() const;
 
   void setReturn(ActRec* fp, PC pc, void* retAddr);
+  void setJitReturn(void* addr);
   void setReturnVMExit();
 
   // skip this frame if it is for a builtin function
   bool skipFrame() const;
 
-  /**
-   * Accessors for the packed m_numArgsAndFlags field. We track
-   * whether ActRecs came from FPushCtor* so that during unwinding we
-   * can set the flag not to call destructors for objects whose
-   * constructors exit via an exception.
-   */
+  enum Flags : uint32_t {
+    None          = 0,
 
-  static constexpr int kNumArgsBits = 29;
-  static constexpr int kNumArgsMask = (1 << kNumArgsBits) - 1;
-  static constexpr int kFlagsMask   = ~kNumArgsMask;
+    // In non-HH files the caller can specify whether param type-checking should
+    // be strict or weak.
+    UseWeakTypes = (1u << 28),
 
-  static constexpr int kLocalsDecRefdShift = kNumArgsBits;
-  static constexpr int kResumedShift       = kNumArgsBits + 1;
-  static constexpr int kFPushCtorShift     = kNumArgsBits + 2;
+    // This bit can be independently set on ActRecs with any other flag state.
+    // It's used by the unwinder to know that an ActRec has been partially torn
+    // down (locals freed).
+    LocalsDecRefd = (1u << 29),
 
-  static_assert(kFPushCtorShift <= 8 * sizeof(int32_t) - 1,
-                "Out of bits in ActRec");
+    // Mutually exclusive execution mode states in these 2 bits.
+    InResumed     = (1u << 30),
+    SpareFlag     = (1u << 31),
+    MagicDispatch = InResumed|SpareFlag,
+  };
 
-  static constexpr int kLocalsDecRefdMask = 1 << kLocalsDecRefdShift;
-  static constexpr int kResumedMask       = 1 << kResumedShift;
-  static constexpr int kFPushCtorMask     = 1 << kFPushCtorShift;
+  static constexpr int kNumArgsBits       = 28;
+  static constexpr int kNumArgsMask       = (1 << kNumArgsBits) - 1;
+  static constexpr int kFlagsMask         = ~kNumArgsMask;
+  static constexpr int kExecutionModeMask = ~(LocalsDecRefd | UseWeakTypes);
 
-  int32_t numArgs() const {
-    return m_numArgsAndFlags & kNumArgsMask;
-  }
-
-  bool localsDecRefd() const {
-    return m_numArgsAndFlags & kLocalsDecRefdMask;
+  int32_t numArgs() const { return m_numArgsAndFlags & kNumArgsMask; }
+  Flags flags() const {
+    return static_cast<Flags>(m_numArgsAndFlags & kFlagsMask);
   }
 
+  bool useWeakTypes() const { return flags() & UseWeakTypes; }
+  bool localsDecRefd() const { return flags() & LocalsDecRefd; }
   bool resumed() const {
-    return m_numArgsAndFlags & kResumedMask;
+    return (flags() & kExecutionModeMask) == InResumed;
+  }
+  bool isSpareFlag() const {
+    return (flags() & kExecutionModeMask) == SpareFlag;
+  }
+  bool magicDispatch() const {
+    return (flags() & kExecutionModeMask) == MagicDispatch;
   }
 
-  void setResumed() {
-    m_numArgsAndFlags |= kResumedMask;
-  }
-
-  bool isFromFPushCtor() const {
-    return m_numArgsAndFlags & kFPushCtorMask;
-  }
-
-  static inline uint32_t
-  encodeNumArgs(uint32_t numArgs, bool localsDecRefd, bool resumed,
-                bool isFPushCtor) {
-    assert((numArgs & kFlagsMask) == 0);
-    return numArgs |
-      (localsDecRefd << kLocalsDecRefdShift) |
-      (resumed       << kResumedShift) |
-      (isFPushCtor   << kFPushCtorShift);
+  static uint32_t encodeNumArgsAndFlags(uint32_t numArgs, Flags flags) {
+    assert((numArgs & ~kNumArgsMask) == 0);
+    assert((uint32_t{flags} & ~kFlagsMask) == 0);
+    return numArgs | flags;
   }
 
   void initNumArgs(uint32_t numArgs) {
-    m_numArgsAndFlags = encodeNumArgs(numArgs, false, false, false);
-  }
-
-  void initNumArgsFromResumable(uint32_t numArgs) {
-    m_numArgsAndFlags = encodeNumArgs(numArgs, false, true, false);
-  }
-
-  void initNumArgsFromFPushCtor(uint32_t numArgs) {
-    m_numArgsAndFlags = encodeNumArgs(numArgs, false, false, true);
+    m_numArgsAndFlags = encodeNumArgsAndFlags(numArgs, Flags::None);
   }
 
   void setNumArgs(uint32_t numArgs) {
-    m_numArgsAndFlags = encodeNumArgs(numArgs, localsDecRefd(), resumed(),
-                                      isFromFPushCtor());
+    m_numArgsAndFlags = encodeNumArgsAndFlags(numArgs, flags());
   }
 
+  void setUseWeakTypes() {
+    m_numArgsAndFlags |= UseWeakTypes;
+  }
   void setLocalsDecRefd() {
-    assert(!localsDecRefd());
-    m_numArgsAndFlags |= kLocalsDecRefdMask;
+    m_numArgsAndFlags |= LocalsDecRefd;
+  }
+  void setResumed() {
+    m_numArgsAndFlags |= InResumed;
+  }
+  void setSpareFlag() {
+    m_numArgsAndFlags |= SpareFlag;
+  }
+
+  void setMagicDispatch(StringData* invName) {
+    assertx(!resumed());
+    m_numArgsAndFlags |= MagicDispatch;
+    m_invName = invName;
+  }
+
+  StringData* clearMagicDispatch() {
+    assert(magicDispatch());
+    auto const invName = getInvName();
+    m_numArgsAndFlags = encodeNumArgsAndFlags(
+      numArgs(),
+      static_cast<Flags>(flags() & ~MagicDispatch)
+    );
+    trashVarEnv();
+    return invName;
   }
 
   static void* encodeThis(ObjectData* obj, Class* cls) {
@@ -373,88 +419,94 @@ struct ActRec {
     return m_func;
   }
 
-  /**
-   * To conserve space, we use unions for pairs of mutually exclusive
-   * fields (fields that are not used at the same time). We use unions
-   * for m_this/m_cls and m_varEnv/m_invName.
+  /*
+   * To conserve space, we use unions for pairs of mutually exclusive fields
+   * (fields that are not used at the same time). We use unions for
+   * m_this/m_cls and m_varEnv/m_invName.
    *
-   * The least significant bit is used as a marker for each pair of fields
-   * so that we can distinguish at runtime which field is valid. We define
+   * The least significant bit is used as a marker for each pair of fields so
+   * that we can distinguish at runtime which field is valid. We define
    * accessors below to encapsulate this logic.
-   *
-   * Note that m_invName is only used when the ActRec is pre-live. Thus when
-   * an ActRec is live it is safe to directly access m_varEnv without using
-   * accessors.
    */
 
-  static constexpr int8_t kHasClassBit = 0x1;
-  static constexpr int8_t kClassMask   = ~kHasClassBit;
+  static constexpr uintptr_t kHasClassBit       = 0x1;
+  static constexpr uintptr_t kExtraArgsBit      = 0x1;
+  static constexpr uintptr_t kTrashedVarEnvSlot = 0xfeeefeee000f000f;
 
-  inline bool hasThis() const {
-    return m_this && !(reinterpret_cast<intptr_t>(m_this) & kHasClassBit);
+  bool hasThis() const {
+    return m_this && !(reinterpret_cast<uintptr_t>(m_this) & kHasClassBit);
   }
-  inline ObjectData* getThis() const {
+
+  ObjectData* getThis() const {
     assert(hasThis());
     return m_this;
   }
-  inline void setThis(ObjectData* val) {
+
+  void setThis(ObjectData* val) {
     m_this = val;
   }
-  inline bool hasClass() const {
-    return reinterpret_cast<intptr_t>(m_cls) & kHasClassBit;
+
+  bool hasClass() const {
+    return reinterpret_cast<uintptr_t>(m_cls) & kHasClassBit;
   }
-  inline Class* getClass() const {
+
+  Class* getClass() const {
     assert(hasClass());
     return reinterpret_cast<Class*>(
-      reinterpret_cast<intptr_t>(m_cls) & kClassMask);
+      reinterpret_cast<uintptr_t>(m_cls) & ~kHasClassBit);
   }
-  inline void setClass(Class* val) {
+
+  void setClass(Class* val) {
     m_cls = reinterpret_cast<Class*>(
-      reinterpret_cast<intptr_t>(val) | kHasClassBit);
+      reinterpret_cast<uintptr_t>(val) | kHasClassBit);
   }
 
-  // Note that reordering these is likely to require changes to the translator.
-  static constexpr int8_t kInvNameBit    = 0x1;
-  static constexpr int8_t kInvNameMask   = ~kInvNameBit;
-  static constexpr int8_t kExtraArgsBit  = 0x2;
-  static constexpr int8_t kExtraArgsMask = ~kExtraArgsBit;
+  void trashVarEnv() {
+    if (debug) setVarEnv(reinterpret_cast<VarEnv*>(kTrashedVarEnvSlot));
+  }
 
-  inline bool hasVarEnv() const {
-    return m_varEnv &&
-      !(reinterpret_cast<intptr_t>(m_varEnv) & (kInvNameBit | kExtraArgsBit));
+  bool checkVarEnv() const {
+    always_assert(m_varEnv != reinterpret_cast<VarEnv*>(kTrashedVarEnvSlot));
+    return true;
   }
-  inline bool hasInvName() const {
-    return reinterpret_cast<intptr_t>(m_invName) & kInvNameBit;
+
+  bool hasVarEnv() const {
+    assert(checkVarEnv());
+    assert(!magicDispatch());
+    return m_varEnv && !(reinterpret_cast<uintptr_t>(m_varEnv) & kExtraArgsBit);
   }
-  inline bool hasExtraArgs() const {
-    return reinterpret_cast<intptr_t>(m_extraArgs) & kExtraArgsBit;
+
+  bool hasExtraArgs() const {
+    assert(checkVarEnv());
+    return reinterpret_cast<uintptr_t>(m_extraArgs) & kExtraArgsBit;
   }
-  inline VarEnv* getVarEnv() const {
+
+  VarEnv* getVarEnv() const {
     assert(hasVarEnv());
     return m_varEnv;
   }
-  inline StringData* getInvName() const {
-    assert(hasInvName());
-    return reinterpret_cast<StringData*>(
-      reinterpret_cast<intptr_t>(m_invName) & kInvNameMask);
+
+  StringData* getInvName() const {
+    assert(magicDispatch());
+    assert(checkVarEnv());
+    return m_invName;
   }
-  inline ExtraArgs* getExtraArgs() const {
+
+  ExtraArgs* getExtraArgs() const {
+    if (!hasExtraArgs()) return nullptr;
     return reinterpret_cast<ExtraArgs*>(
-      reinterpret_cast<intptr_t>(m_extraArgs) & kExtraArgsMask);
+      reinterpret_cast<uintptr_t>(m_extraArgs) & ~kExtraArgsBit);
   }
-  inline void setVarEnv(VarEnv* val) {
+
+  void setVarEnv(VarEnv* val) {
     m_varEnv = val;
   }
-  inline void setInvName(StringData* val) {
-    m_invName = reinterpret_cast<StringData*>(
-      reinterpret_cast<intptr_t>(val) | kInvNameBit);
-  }
-  inline void setExtraArgs(ExtraArgs* val) {
+
+  void setExtraArgs(ExtraArgs* val) {
     m_extraArgs = reinterpret_cast<ExtraArgs*>(
-      reinterpret_cast<intptr_t>(val) | kExtraArgsBit);
+      reinterpret_cast<uintptr_t>(val) | kExtraArgsBit);
   }
 
-  // Accessors for extra arg queries.
   TypedValue* getExtraArg(unsigned ind) const {
     assert(hasExtraArgs() || hasVarEnv());
     return hasExtraArgs() ? getExtraArgs()->getExtraArg(ind) :
@@ -466,6 +518,36 @@ struct ActRec {
 static_assert(offsetof(ActRec, m_sfp) == 0,
               "m_sfp should be at offset 0 of ActRec");
 
+///////////////////////////////////////////////////////////////////////////////
+
+/*
+ * Returns true iff ar represents a frame on the VM eval stack or a Resumable
+ * object on the PHP heap.
+ */
+bool isVMFrame(const ActRec* ar);
+
+/*
+ * Returns true iff the given address is one of the special debugger return
+ * helpers.
+ */
+bool isDebuggerReturnHelper(void* addr);
+
+/*
+ * If ar->m_savedRip points somewhere in the TC that is not a return helper,
+ * change it to point to an appropriate return helper. The two different
+ * versions are for the different needs of the C++ unwinder and debugger hooks,
+ * respectively.
+ */
+void unwindPreventReturnToTC(ActRec* ar);
+void debuggerPreventReturnToTC(ActRec* ar);
+
+/*
+ * Call debuggerPreventReturnToTC() on all live VM frames in this thread.
+ */
+void debuggerPreventReturnsToTC();
+
+///////////////////////////////////////////////////////////////////////////////
+
 inline int32_t arOffset(const ActRec* ar, const ActRec* other) {
   return (intptr_t(other) - intptr_t(ar)) / sizeof(TypedValue);
 }
@@ -478,10 +560,25 @@ inline ActRec* arFromSpOffset(const ActRec *sp, int32_t offset) {
   return arAtOffset(sp, offset);
 }
 
-inline TypedValue* arReturn(ActRec* ar, const Variant& value) {
-  ar->m_r = *value.asTypedValue();
-  return &ar->m_r;
-}
+void frame_free_locals_no_hook(ActRec* fp);
+
+#define arReturn(a, x)                          \
+  ([&] {                                        \
+    ActRec* ar_ = (a);                          \
+    TypedValue val_;                            \
+    new (&val_) Variant(x);                     \
+    frame_free_locals_no_hook(ar_);             \
+    ar_->m_r = val_;                            \
+    return &ar_->m_r;                           \
+  }())
+
+#define tvReturn(x)                                                     \
+  ([&] {                                                                \
+    TypedValue val_;                                                    \
+    new (&val_) Variant(x);                                             \
+    assert(val_.m_type != KindOfRef && val_.m_type != KindOfUninit);    \
+    return val_;                                                        \
+  }())
 
 template <bool crossBuiltin> Class* arGetContextClassImpl(const ActRec* ar);
 template <> Class* arGetContextClassImpl<true>(const ActRec* ar);
@@ -493,6 +590,8 @@ inline Class* arGetContextClassFromBuiltin(const ActRec* ar) {
   return arGetContextClassImpl<true>(ar);
 }
 
+///////////////////////////////////////////////////////////////////////////////
+
 // Used by extension functions that take a PHP "callback", since they need to
 // figure out the callback context once and call it multiple times. (e.g.
 // array_map, array_filter, ...)
@@ -506,6 +605,8 @@ struct CallCtx {
 constexpr size_t kNumIterCells = sizeof(Iter) / sizeof(Cell);
 constexpr size_t kNumActRecCells = sizeof(ActRec) / sizeof(Cell);
 
+///////////////////////////////////////////////////////////////////////////////
+
 /*
  * We pad all stack overflow checks by a small amount to allow for three
  * things:
@@ -526,22 +627,17 @@ constexpr int kInvalidRaiseLevel = -1;
 constexpr int kInvalidNesting = -1;
 
 struct Fault {
-  enum class Type : int16_t {
-    UserException,
-    CppException
-  };
-
   explicit Fault()
     : m_raiseNesting(kInvalidNesting),
       m_raiseFrame(nullptr),
       m_raiseOffset(kInvalidOffset),
       m_handledCount(0) {}
 
-  union {
-    ObjectData* m_userException;
-    Exception* m_cppException;
-  };
-  Type m_faultType;
+  template<class F> void scan(F& mark) const {
+    mark(m_userException);
+  }
+
+  ObjectData* m_userException;
 
   // The VM nesting at the moment where the exception was thrown.
   int m_raiseNesting;
@@ -570,6 +666,7 @@ class Stack {
                       // m_elms.
 
 public:
+  bool isAllocated() { return m_elms != nullptr; }
   void* getStackLowAddress() const { return m_elms; }
   void* getStackHighAddress() const { return m_base; }
   bool isValidAddress(uintptr_t v) {
@@ -578,12 +675,6 @@ public:
   void requestInit();
   void requestExit();
 
-private:
-  void toStringFrame(std::ostream& os, const ActRec* fp,
-                     int offset, const TypedValue* ftop,
-                     const std::string& prefix) const;
-
-public:
   static const int sSurprisePageSize;
   static const unsigned sMinStackElms;
   static void ValidateStackSize();
@@ -609,6 +700,7 @@ public:
     return offsetof(Stack, m_top);
   }
 
+  static TypedValue* anyFrameStackBase(const ActRec* fp);
   static TypedValue* frameStackBase(const ActRec* fp);
   static TypedValue* resumableStackBase(const ActRec* fp);
 
@@ -622,7 +714,7 @@ public:
   ALWAYS_INLINE
   void popX() {
     assert(m_top != m_base);
-    assert(!IS_REFCOUNTED_TYPE(m_top->m_type));
+    assert(!isRefcountedType(m_top->m_type));
     tvDebugTrash(m_top);
     m_top++;
   }
@@ -631,7 +723,7 @@ public:
   void popC() {
     assert(m_top != m_base);
     assert(cellIsPlausible(*m_top));
-    tvRefcountedDecRefCell(m_top);
+    tvRefcountedDecRef(m_top);
     tvDebugTrash(m_top);
     m_top++;
   }
@@ -670,11 +762,9 @@ public:
     assert(m_top != m_base);
     ActRec* ar = (ActRec*)m_top;
     if (ar->hasThis()) decRefObj(ar->getThis());
-    if (ar->hasInvName()) decRefStr(ar->getInvName());
-
-    // This should only be used on a pre-live ActRec.
-    assert(!ar->hasVarEnv());
-    assert(!ar->hasExtraArgs());
+    if (ar->magicDispatch()) {
+      decRefStr(ar->getInvName());
+    }
     discardAR();
   }
 
@@ -767,26 +857,14 @@ public:
     m_top->m_type = KindOfUninit;
   }
 
-  #define PUSH_METHOD(name, type, field, value)                               \
-  ALWAYS_INLINE void push##name() {                                           \
-    assert(m_top != m_elms);                                                  \
-    m_top--;                                                                  \
-    m_top->m_data.field = value;                                              \
-    m_top->m_type = type;                                                     \
+  template<DataType t, class T> void pushVal(T v) {
+    assert(m_top != m_elms);
+    m_top--;
+    *m_top = make_tv<t>(v);
   }
-  PUSH_METHOD(True, KindOfBoolean, num, 1)
-  PUSH_METHOD(False, KindOfBoolean, num, 0)
-
-  #define PUSH_METHOD_ARG(name, type, field, argtype, arg)                    \
-  ALWAYS_INLINE void push##name(argtype arg) {                                \
-    assert(m_top != m_elms);                                                  \
-    m_top--;                                                                  \
-    m_top->m_data.field = arg;                                                \
-    m_top->m_type = type;                                                     \
-  }
-  PUSH_METHOD_ARG(Bool, KindOfBoolean, num, bool, b)
-  PUSH_METHOD_ARG(Int, KindOfInt64, num, int64_t, i)
-  PUSH_METHOD_ARG(Double, KindOfDouble, dbl, double, d)
+  ALWAYS_INLINE void pushBool(bool v) { pushVal<KindOfBoolean>(v); }
+  ALWAYS_INLINE void pushInt(int64_t v) { pushVal<KindOfInt64>(v); }
+  ALWAYS_INLINE void pushDouble(double v) { pushVal<KindOfDouble>(v); }
 
   // This should only be called directly when the caller has
   // already adjusted the refcount appropriately
@@ -794,8 +872,7 @@ public:
   void pushStringNoRc(StringData* s) {
     assert(m_top != m_elms);
     m_top--;
-    m_top->m_data.pstr = s;
-    m_top->m_type = KindOfString;
+    *m_top = make_tv<KindOfString>(s);
   }
 
   ALWAYS_INLINE
@@ -803,8 +880,7 @@ public:
     assert(s->isStatic()); // No need to call s->incRefCount().
     assert(m_top != m_elms);
     m_top--;
-    m_top->m_data.pstr = s;
-    m_top->m_type = KindOfStaticString;
+    *m_top = make_tv<KindOfStaticString>(s);
   }
 
   // This should only be called directly when the caller has
@@ -813,8 +889,7 @@ public:
   void pushArrayNoRc(ArrayData* a) {
     assert(m_top != m_elms);
     m_top--;
-    m_top->m_data.parr = a;
-    m_top->m_type = KindOfArray;
+    *m_top = make_tv<KindOfArray>(a);
   }
 
   ALWAYS_INLINE
@@ -827,7 +902,9 @@ public:
   ALWAYS_INLINE
   void pushStaticArray(ArrayData* a) {
     assert(a->isStatic()); // No need to call a->incRefCount().
-    pushArrayNoRc(a);
+    assert(m_top != m_elms);
+    m_top--;
+    *m_top = make_tv<KindOfPersistentArray>(a);
   }
 
   // This should only be called directly when the caller has
@@ -836,8 +913,7 @@ public:
   void pushObjectNoRc(ObjectData* o) {
     assert(m_top != m_elms);
     m_top--;
-    m_top->m_data.pobj = o;
-    m_top->m_type = KindOfObject;
+    *m_top = make_tv<KindOfObject>(o);
   }
 
   ALWAYS_INLINE
@@ -889,10 +965,10 @@ public:
   }
 
   ALWAYS_INLINE
-  void replaceC(const Cell& c) {
+  void replaceC(const Cell c) {
     assert(m_top != m_base);
     assert(m_top->m_type != KindOfRef);
-    tvRefcountedDecRefCell(m_top);
+    tvRefcountedDecRef(m_top);
     *m_top = c;
   }
 
@@ -901,7 +977,7 @@ public:
   void replaceC() {
     assert(m_top != m_base);
     assert(m_top->m_type != KindOfRef);
-    tvRefcountedDecRefCell(m_top);
+    tvRefcountedDecRef(m_top);
     *m_top = make_tv<DT>();
   }
 
@@ -910,7 +986,7 @@ public:
   void replaceC(T value) {
     assert(m_top != m_base);
     assert(m_top->m_type != KindOfRef);
-    tvRefcountedDecRefCell(m_top);
+    tvRefcountedDecRef(m_top);
     *m_top = make_tv<DT>(value);
   }
 
@@ -1010,9 +1086,7 @@ visitStackElems(const ActRec* const fp,
                 Offset const bcOffset,
                 ARFun arFun,
                 TVFun tvFun) {
-  const TypedValue* const base =
-    fp->resumed() ? Stack::resumableStackBase(fp)
-                  : Stack::frameStackBase(fp);
+  const TypedValue* const base = Stack::anyFrameStackBase(fp);
   MaybeConstTVPtr cursor = stackTop;
   assert(cursor <= base);
 
@@ -1036,7 +1110,7 @@ visitStackElems(const ActRec* const fp,
       while (cursor < reinterpret_cast<TypedValue*>(ar)) {
         tvFun(cursor++);
       }
-      arFun(ar);
+      arFun(ar, fe->m_fpushOff);
 
       cursor += kNumActRecCells;
       if (fe->m_parentIndex == -1) break;
@@ -1049,8 +1123,31 @@ visitStackElems(const ActRec* const fp,
   }
 }
 
+void resetCoverageCounters();
+
+// The interpOne*() methods implement individual opcode handlers.
+using InterpOneFunc = jit::TCA (*) (ActRec*, TypedValue*, Offset);
+extern InterpOneFunc interpOneEntryPoints[];
+
+bool doFCallArrayTC(PC pc);
+bool doFCall(ActRec* ar, PC& pc);
+jit::TCA dispatchBB();
+void pushLocalsAndIterators(const Func* func, int nparams = 0);
+Array getDefinedVariables(const ActRec*);
+
 ///////////////////////////////////////////////////////////////////////////////
 
+template<class F> void VarEnv::scan(F& mark) const {
+  mark(m_nvTable);
+  if (m_extraArgs) {
+    if (const auto ar = m_nvTable.getFP()) {
+      const int numExtra =
+        ar->numArgs() - ar->m_func->numNonVariadicParams();
+      m_extraArgs->scan(mark, numExtra);
+    }
+  }
+}
+
 }
 
 #endif
