@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -29,7 +29,7 @@
 #include "hphp/runtime/base/runtime-option.h"
 #include "hphp/runtime/vm/jit/mc-generator.h"
 
-using namespace HPHP::JIT;
+using namespace HPHP::jit;
 
 namespace HPHP {
 namespace Debug {
@@ -95,6 +95,7 @@ void ElfWriter::initStrtab() {
   addSectionString("");
 }
 
+#if defined(LIBDWARF_USE_INIT_C) || defined(FACEBOOK)
 bool ElfWriter::initDwarfProducer() {
   Dwarf_Error error = 0;
   /* m_dwarfProducer is the handle used for interaction for libdwarf */
@@ -116,6 +117,27 @@ bool ElfWriter::initDwarfProducer() {
   }
   return true;
 }
+#else
+bool ElfWriter::initDwarfProducer() {
+  Dwarf_Error error = 0;
+  auto ret = dwarf_producer_init(
+    DW_DLC_WRITE | DW_DLC_SIZE_64 | DW_DLC_SYMBOLIC_RELOCATIONS,
+    g_dwarfCallback,
+    nullptr,
+    nullptr,
+    reinterpret_cast<Dwarf_Ptr>(this),
+    "x86_64",
+    "V2",
+    nullptr,
+    &m_dwarfProducer,
+    &error);
+  if (ret != DW_DLV_OK) {
+    logError("Unable to create dwarf producer");
+    return false;
+  }
+  return true;
+}
+#endif
 
 Dwarf_P_Die ElfWriter::addFunctionInfo(FunctionInfo* f, Dwarf_P_Die type) {
   Dwarf_Error error = 0;
