@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -21,4 +21,4 @@
  * and optimizes away the call. This prevents gdb from trapping updates to
  * the DWARF files emitted by HHVM */
 
-void __attribute__((noinline)) __jit_debug_register_code() { };
+void __attribute__((__noinline__)) __jit_debug_register_code() { };
