@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -21,19 +21,19 @@
 
 #include <boost/variant.hpp>
 
-#include "folly/String.h"
+#include <folly/String.h>
 
-#include "hphp/util/match.h"
-#include "hphp/runtime/vm/as-shared.h"
-#include "hphp/runtime/vm/hhbc.h"
-#include "hphp/runtime/vm/repo-global-data.h"
+#include "hphp/runtime/base/builtin-functions.h" // f_serialize
 #include "hphp/runtime/base/repo-auth-type-array.h"
 #include "hphp/runtime/base/repo-auth-type-codec.h"
-#include "hphp/runtime/base/complex-types.h"
-#include "hphp/runtime/base/builtin-functions.h" // f_serialize
-#include "hphp/runtime/vm/unit.h"
-#include "hphp/runtime/vm/func.h"
+#include "hphp/runtime/vm/as-shared.h"
 #include "hphp/runtime/vm/class.h"
+#include "hphp/runtime/vm/func.h"
+#include "hphp/runtime/vm/hhbc-codec.h"
+#include "hphp/runtime/vm/hhbc.h"
+#include "hphp/runtime/vm/repo-global-data.h"
+#include "hphp/runtime/vm/unit.h"
+#include "hphp/util/match.h"
 
 namespace HPHP {
 
@@ -91,8 +91,7 @@ void indented(Output& out, Func f) {
 
 std::string escaped(const StringData* sd) {
   auto const sl = sd->slice();
-  auto const sp = folly::StringPiece{sl.ptr, sl.ptr + sl.len};
-  return folly::format("\"{}\"", folly::cEscape<std::string>(sp)).str();
+  return folly::format("\"{}\"", folly::cEscape<std::string>(sl)).str();
 }
 
 /*
@@ -160,16 +159,16 @@ FuncInfo find_func_info(const Func* func) {
   };
 
   auto find_jump_targets = [&] {
-    auto it           = func->unit()->at(func->base());
+    auto pc           = func->unit()->at(func->base());
     auto const stop   = func->unit()->at(func->past());
-    auto const bcBase = reinterpret_cast<const Op*>(func->unit()->at(0));
+    auto const bcBase = func->unit()->at(0);
 
-    for (; it != stop; it += instrLen(reinterpret_cast<const Op*>(it))) {
-      auto const pop = reinterpret_cast<const Op*>(it);
-      auto const off = func->unit()->offsetOf(it);
-      if (isSwitch(*pop)) {
-        foreachSwitchTarget(pop, [&] (Offset off) {
-          add_target("L", pop - bcBase + off);
+    for (; pc != stop; pc += instrLen(pc)) {
+      auto const op = peek_op(pc);
+      auto const off = func->unit()->offsetOf(pc);
+      if (isSwitch(op)) {
+        foreachSwitchTarget(pc, [&] (Offset off) {
+          add_target("L", pc - bcBase + off);
         });
         continue;
       }
@@ -229,9 +228,9 @@ template<class T> T decode(PC& pc) {
 std::string loc_name(const FuncInfo& finfo, uint32_t id) {
   auto const sd = finfo.func->localVarName(id);
   if (!sd || sd->empty()) {
-    always_assert(!"unnamed locals need to be supported");
+    return folly::format("_{}", id).str();
   }
-  return sd->data();
+  return folly::format("${}", sd->data()).str();
 }
 
 std::string jmp_label(const FuncInfo& finfo, Offset tgt) {
@@ -257,7 +256,7 @@ void print_instr(Output& out, const FuncInfo& finfo, PC pc) {
     out.fmt(" <{}", locationCodeString(lcode));
     if (numLocationCodeImms(lcode)) {
       always_assert(numLocationCodeImms(lcode) == 1);
-      out.fmt(":${}", loc_name(finfo, decodeVariableSizeImm(&vec)));
+      out.fmt(":{}", loc_name(finfo, decodeVariableSizeImm(&vec)));
     }
 
     while (vec < pc) {
@@ -268,7 +267,7 @@ void print_instr(Output& out, const FuncInfo& finfo, PC pc) {
       case MCodeImm::None:
         break;
       case MCodeImm::Local:
-        out.fmt(":${}", loc_name(finfo, imm()));
+        out.fmt(":{}", loc_name(finfo, imm()));
         break;
       case MCodeImm::String:
         out.fmt(":{}", escaped(finfo.unit->lookupLitstrId(imm())));
@@ -344,7 +343,7 @@ void print_instr(Output& out, const FuncInfo& finfo, PC pc) {
 #define IMM_ILA    print_itertab();
 #define IMM_IVA    out.fmt(" {}", decodeVariableSizeImm(&pc));
 #define IMM_I64A   out.fmt(" {}", decode<int64_t>(pc));
-#define IMM_LA     out.fmt(" ${}", loc_name(finfo, decodeVariableSizeImm(&pc)));
+#define IMM_LA     out.fmt(" {}", loc_name(finfo, decodeVariableSizeImm(&pc)));
 #define IMM_IA     out.fmt(" {}", decodeVariableSizeImm(&pc));
 #define IMM_DA     out.fmt(" {}", decode<double>(pc));
 #define IMM_SA     out.fmt(" {}", \
@@ -369,10 +368,10 @@ void print_instr(Output& out, const FuncInfo& finfo, PC pc) {
     out.fmt("{}", #opcode);                               \
     IMM_##imms                                            \
     break;
-  switch (*reinterpret_cast<const Op*>(pc)) { OPCODES }
+  switch (peek_op(pc)) { OPCODES }
 #undef O
 
-  assert(pc == startPc + instrLen(reinterpret_cast<const Op*>(startPc)));
+  assert(pc == startPc + instrLen(startPc));
 
 #undef IMM_NA
 #undef IMM_ONE
@@ -399,10 +398,19 @@ void print_instr(Output& out, const FuncInfo& finfo, PC pc) {
   out.nl();
 }
 
-void print_func_directives(Output& out, const Func* func) {
+void print_func_directives(Output& out, const FuncInfo& finfo) {
+  const Func* func = finfo.func;
   if (auto const niters = func->numIterators()) {
     out.fmtln(".numiters {};", niters);
   }
+  if (func->numNamedLocals() > func->numParams()) {
+    std::vector<std::string> locals;
+    for (int i = func->numParams(); i < func->numNamedLocals(); i++) {
+      locals.push_back(loc_name(finfo, i));
+    }
+    out.fmtln(".declvars {};", folly::join(" ", locals));
+  }
+
 }
 
 void print_func_body(Output& out, const FuncInfo& finfo) {
@@ -463,10 +471,27 @@ void print_func_body(Output& out, const FuncInfo& finfo) {
 
     print_instr(out, finfo, bcIter);
 
-    bcIter += instrLen(reinterpret_cast<const Op*>(bcIter));
+    bcIter += instrLen(bcIter);
   }
 }
 
+std::string type_constraint(const TypeConstraint &tc) {
+  std::string typeName = tc.typeName() ? escaped(tc.typeName()) : "N";
+  return folly::format("{} {} ",
+                       typeName,
+                       type_flags_to_string(tc.flags())).str();
+}
+std::string opt_type_info(const StringData *userType,
+                          const TypeConstraint &tc) {
+  if (userType || tc.typeName() || tc.flags()) {
+    std::string utype = userType ? escaped(userType) : "N";
+    return folly::format("<{} {}> ",
+                         utype,
+                         type_constraint(tc)).str();
+  }
+  return "";
+}
+
 std::string func_param_list(const FuncInfo& finfo) {
   auto ret = std::string{};
   auto const func = finfo.func;
@@ -474,8 +499,10 @@ std::string func_param_list(const FuncInfo& finfo) {
   for (auto i = uint32_t{0}; i < func->numParams(); ++i) {
     if (i != 0) ret += ", ";
 
+    ret += opt_type_info(func->params()[i].userType,
+                         func->params()[i].typeConstraint);
     if (func->byRef(i)) ret += "&";
-    ret += folly::format("${}", loc_name(finfo, i)).str();
+    ret += folly::format("{}", loc_name(finfo, i)).str();
     if (func->params()[i].hasDefaultValue()) {
       auto const off = func->params()[i].funcletOff;
       ret += folly::format(" = {}", jmp_label(finfo, off)).str();
@@ -502,6 +529,27 @@ std::string func_flag_list(const FuncInfo& finfo) {
   return " ";
 }
 
+std::string user_attrs(const UserAttributeMap* attrsMap) {
+  if (!attrsMap || attrsMap->empty()) return "";
+
+  std::vector<std::string> attrs;
+
+  for (auto& entry : *attrsMap) {
+    attrs.push_back(
+      folly::format("{}({})", escaped(entry.first),
+                    escaped_long(entry.second)).str());
+  }
+  return folly::join(" ", attrs);
+}
+
+std::string opt_attrs(AttrContext ctx, Attr attrs,
+                      const UserAttributeMap* userAttrs = nullptr) {
+  auto str = folly::trimWhitespace(folly::format(
+               "{} {}",
+               attrs_to_string(ctx, attrs), user_attrs(userAttrs)).str()).str();
+  if (!str.empty()) str = folly::format(" [{}]", str).str();
+  return str;
+}
 
 void print_func(Output& out, const Func* func) {
   auto const finfo = find_func_info(func);
@@ -509,23 +557,21 @@ void print_func(Output& out, const Func* func) {
   if (func->isPseudoMain()) {
     out.fmtln(".main {{");
   } else {
-    out.fmtln(".function {}({}){}{{", func->name()->data(),
-      func_param_list(finfo), func_flag_list(finfo));
+    out.fmtln(".function{} {}{}({}){}{{",
+      opt_attrs(AttrContext::Func, func->attrs(), &func->userAttributes()),
+      opt_type_info(func->returnUserType(), func->returnTypeConstraint()),
+      func->name(),
+      func_param_list(finfo),
+      func_flag_list(finfo));
   }
   indented(out, [&] {
-    print_func_directives(out, func);
+    print_func_directives(out, finfo);
     print_func_body(out, finfo);
   });
   out.fmtln("}}");
   out.nl();
 }
 
-std::string opt_attrs(AttrContext ctx, Attr attrs) {
-  auto str = attrs_to_string(ctx, attrs);
-  if (!str.empty()) str = folly::format(" [{}]", str).str();
-  return str;
-}
-
 std::string member_tv_initializer(Cell cell) {
   assert(cellIsPlausible(cell));
   if (cell.m_type == KindOfUninit) return "uninit";
@@ -533,7 +579,9 @@ std::string member_tv_initializer(Cell cell) {
 }
 
 void print_constant(Output& out, const PreClass::Const* cns) {
-  out.fmtln(".const {} = {};", cns->name()->data(),
+  if (cns->isAbstract()) { return; }
+  out.fmtln(".const {}{} = {};", cns->name(),
+    cns->isType() ? " isType" : "",
     member_tv_initializer(cns->val()));
 }
 
@@ -548,12 +596,14 @@ void print_property(Output& out, const PreClass::Prop* prop) {
 
 void print_method(Output& out, const Func* func) {
   auto const finfo = find_func_info(func);
-  out.fmtln(".method{} {}({}) {{",
-    opt_attrs(AttrContext::Func, func->attrs()),
-    func->name()->data(),
-    func_param_list(finfo));
+  out.fmtln(".method{} {}{}({}){}{{",
+    opt_attrs(AttrContext::Func, func->attrs(), &func->userAttributes()),
+    opt_type_info(func->returnUserType(), func->returnTypeConstraint()),
+    func->name(),
+    func_param_list(finfo),
+    func_flag_list(finfo));
   indented(out, [&] {
-    print_func_directives(out, func);
+    print_func_directives(out, finfo);
     print_func_body(out, finfo);
   });
   out.fmtln("}}");
@@ -561,17 +611,23 @@ void print_method(Output& out, const Func* func) {
 
 void print_cls_inheritance_list(Output& out, const PreClass* cls) {
   if (!cls->parent()->empty()) {
-    out.fmt(" extends {}", cls->parent()->data());
+    out.fmt(" extends {}", cls->parent());
   }
   if (!cls->interfaces().empty()) {
     out.fmt(" implements (");
     for (auto i = uint32_t{}; i < cls->interfaces().size(); ++i) {
-      out.fmt("{}{}", i != 0 ? " " : "", cls->interfaces()[i]->data());
+      out.fmt("{}{}", i != 0 ? " " : "", cls->interfaces()[i].get());
     }
     out.fmt(")");
   }
 }
 
+void print_cls_enum_ty(Output& out, const PreClass* cls) {
+  if (cls->attrs() & AttrEnum) {
+    out.fmtln(".enum_ty <{}>;", type_constraint(cls->enumBaseTy()));
+  }
+}
+
 void print_cls_used_traits(Output& out, const PreClass* cls) {
   auto& traits = cls->usedTraits();
   if (traits.empty()) return;
@@ -580,7 +636,7 @@ void print_cls_used_traits(Output& out, const PreClass* cls) {
   out.fmt(".use");
 
   for (auto i = uint32_t{0}; i < traits.size(); ++i) {
-    out.fmt(" {}", traits[i]->data());
+    out.fmt(" {}", traits[i].get());
   }
 
   auto& precRules  = cls->traitPrecRules();
@@ -601,17 +657,18 @@ void print_cls_used_traits(Output& out, const PreClass* cls) {
         [&]() -> std::string {
           auto ret = std::string{};
           for (auto& name : prec.otherTraitNames()) {
-            ret += folly::format(" {}", name->data()).str();
+            ret += folly::format(" {}", name.get()).str();
           }
           return ret;
         }()
       );
     }
+
     for (auto& alias : aliasRules) {
       out.fmtln("{}{} as{}{};",
         alias.traitName()->empty()
           ? std::string{}
-          : folly::format("{}::", alias.traitName()->data()).str(),
+          : folly::format("{}::", alias.traitName()).str(),
         alias.origMethodName()->data(),
         opt_attrs(AttrContext::TraitImport, alias.modifiers()),
         alias.newMethodName() != alias.origMethodName()
@@ -624,6 +681,7 @@ void print_cls_used_traits(Output& out, const PreClass* cls) {
 }
 
 void print_cls_directives(Output& out, const PreClass* cls) {
+  print_cls_enum_ty(out, cls);
   print_cls_used_traits(out, cls);
   for (auto& c : cls->allConstants())  print_constant(out, &c);
   for (auto& p : cls->allProperties()) print_property(out, &p);
@@ -633,8 +691,8 @@ void print_cls_directives(Output& out, const PreClass* cls) {
 void print_cls(Output& out, const PreClass* cls) {
   out.indent();
   out.fmt(".class{} {}",
-    opt_attrs(AttrContext::Class, cls->attrs()),
-    cls->name()->data());
+    opt_attrs(AttrContext::Class, cls->attrs(), &cls->userAttributes()),
+    cls->name());
   print_cls_inheritance_list(out, cls);
   out.fmt(" {{");
   out.nl();
@@ -643,6 +701,17 @@ void print_cls(Output& out, const PreClass* cls) {
   out.nl();
 }
 
+void print_alias(Output& out, const TypeAlias& alias) {
+  auto flags = TypeConstraint::NoFlags;
+  if (alias.nullable) flags = flags | TypeConstraint::Nullable;
+  TypeConstraint constraint(alias.value, flags);
+
+  out.fmtln(".alias{} {} = <{}>;",
+            opt_attrs(AttrContext::Alias, alias.attrs),
+            (const StringData*)alias.name,
+            type_constraint(constraint));
+}
+
 void print_unit_metadata(Output& out, const Unit* unit) {
   out.fmtln(".filepath {};", escaped(unit->filepath()));
   for (auto i = size_t{0}; i < unit->numArrays(); ++i) {
@@ -653,12 +722,13 @@ void print_unit_metadata(Output& out, const Unit* unit) {
 }
 
 void print_unit(Output& out, const Unit* unit) {
-  out.fmtln("# {} starts here", unit->filepath()->data());
+  out.fmtln("# {} starts here", unit->filepath());
   out.nl();
   print_unit_metadata(out, unit);
-  for (auto* func : unit->funcs())       print_func(out, func);
-  for (auto& cls : unit->preclasses())   print_cls(out, cls.get());
-  out.fmtln("# {} ends here", unit->filepath()->data());
+  for (auto* func : unit->funcs())        print_func(out, func);
+  for (auto& cls : unit->preclasses())    print_cls(out, cls.get());
+  for (auto& alias : unit->typeAliases()) print_alias(out, alias);
+  out.fmtln("# {} ends here", unit->filepath());
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -671,14 +741,7 @@ void print_unit(Output& out, const Unit* unit) {
  *
  * - .line/.srcpos directives
  *
- * - need support for $ and :: in identifiers for traits and
- *   generators to work properly
- *
- * - Unnamed locals.
- *
  * - Static locals.
- *
- * - Parameter type hints.
  */
 
 std::string disassemble(const Unit* unit) {
