@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
 
 #include "hphp/parser/parser.h"
 
-#include "hphp/runtime/base/base-includes.h"
+#include "hphp/runtime/ext/extension.h"
 #include "hphp/runtime/base/unit-cache.h"
 #include "hphp/runtime/base/rds.h"
 #include "hphp/runtime/base/strings.h"
@@ -53,15 +53,20 @@ FuncEmitter::FuncEmitter(UnitEmitter& ue, int sn, Id id, const StringData* n)
   , m_id(id)
   , name(n)
   , top(false)
-  , returnType(KindOfInvalid)
+  , returnType(folly::none)
   , retUserType(nullptr)
   , isClosureBody(false)
   , isAsync(false)
   , isGenerator(false)
   , isPairGenerator(false)
+  , isMemoizeImpl(false)
+  , isMemoizeWrapper(false)
+  , hasMemoizeSharedProp(false)
   , containsCalls(false)
   , docComment(nullptr)
   , originalFilename(nullptr)
+  , memoizePropName(nullptr)
+  , memoizeSharedPropIndex(0)
   , m_numLocals(0)
   , m_numUnnamedLocals(0)
   , m_activeUnnamedLocals(0)
@@ -81,15 +86,20 @@ FuncEmitter::FuncEmitter(UnitEmitter& ue, int sn, const StringData* n,
   , m_id(kInvalidId)
   , name(n)
   , top(false)
-  , returnType(KindOfInvalid)
+  , returnType(folly::none)
   , retUserType(nullptr)
   , isClosureBody(false)
   , isAsync(false)
   , isGenerator(false)
   , isPairGenerator(false)
+  , isMemoizeImpl(false)
+  , isMemoizeWrapper(false)
+  , hasMemoizeSharedProp(false)
   , containsCalls(false)
   , docComment(nullptr)
   , originalFilename(nullptr)
+  , memoizePropName(nullptr)
+  , memoizeSharedPropIndex(0)
   , m_numLocals(0)
   , m_numUnnamedLocals(0)
   , m_activeUnnamedLocals(0)
@@ -106,7 +116,7 @@ FuncEmitter::~FuncEmitter() {
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// Initialization and completion.
+// Initialization and execution.
 
 void FuncEmitter::init(int l1, int l2, Offset base_, Attr attrs_, bool top_,
                        const StringData* docComment_) {
@@ -136,13 +146,30 @@ void FuncEmitter::finish(Offset past_, bool load) {
 void FuncEmitter::commit(RepoTxn& txn) const {
   Repo& repo = Repo::get();
   FuncRepoProxy& frp = repo.frp();
-  int repoId = m_ue.repoId();
-  int64_t usn = m_ue.sn();
+  int repoId = m_ue.m_repoId;
+  int64_t usn = m_ue.m_sn;
 
-  frp.insertFunc(repoId)
+  frp.insertFunc[repoId]
      .insert(*this, txn, usn, m_sn, m_pce ? m_pce->id() : -1, name, top);
 }
 
+static std::vector<EHEnt> toFixed(const std::vector<EHEntEmitter>& vec) {
+  std::vector<EHEnt> ret;
+  for (auto const& ehe : vec) {
+    EHEnt e;
+    e.m_type = ehe.m_type;
+    e.m_itRef = ehe.m_itRef;
+    e.m_base = ehe.m_base;
+    e.m_past = ehe.m_past;
+    e.m_iterId = ehe.m_iterId;
+    e.m_parentIndex = ehe.m_parentIndex;
+    e.m_fault = ehe.m_fault;
+    e.m_catches = ehe.m_catches;
+    ret.emplace_back(std::move(e));
+  }
+  return ret;
+}
+
 Func* FuncEmitter::create(Unit& unit, PreClass* preClass /* = NULL */) const {
   bool isGenerated = isdigit(name->data()[0]) ||
     ParserBase::IsClosureName(name->toCppString());
@@ -160,7 +187,11 @@ Func* FuncEmitter::create(Unit& unit, PreClass* preClass /* = NULL */) const {
     }
     attrs = Attr(attrs & ~AttrPersistent);
   }
-  if (RuntimeOption::EvalJitEnableRenameFunction &&
+  if (!RuntimeOption::RepoAuthoritative) {
+    // In non-RepoAuthoritative mode, any function could get a VarEnv because
+    // of evalPHPDebugger.
+    attrs |= AttrMayUseVV;
+  } else if (RuntimeOption::EvalJitEnableRenameFunction &&
       !name->empty() &&
       !Func::isSpecial(name) &&
       !isClosureBody) {
@@ -173,63 +204,120 @@ Func* FuncEmitter::create(Unit& unit, PreClass* preClass /* = NULL */) const {
   if (!containsCalls) { attrs |= AttrPhpLeafFn; }
 
   assert(!m_pce == !preClass);
-  Func* f = m_ue.newFunc(this, unit, preClass, line1, line2, base,
-                         past, name, attrs, top, docComment,
-                         params.size(), isClosureBody);
+  auto f = m_ue.newFunc(this, unit, name, attrs, params.size());
+
+  f->m_isPreFunc = !!preClass;
+
+  bool const needsExtendedSharedData =
+    m_info ||
+    m_builtinFuncPtr ||
+    m_nativeFuncPtr ||
+    (attrs & AttrNative) ||
+    line2 - line1 >= Func::kSmallDeltaLimit ||
+    past - base >= Func::kSmallDeltaLimit;
+
+  f->m_shared.reset(
+    needsExtendedSharedData
+      ? new Func::ExtendedSharedData(preClass, base, past, line1, line2,
+                                     top, docComment)
+      : new Func::SharedData(preClass, base, past,
+                             line1, line2, top, docComment)
+  );
+
+  f->init(params.size());
+
+  if (auto const ex = f->extShared()) {
+    ex->m_hasExtendedSharedData = true;
+    ex->m_builtinFuncPtr = m_builtinFuncPtr;
+    ex->m_nativeFuncPtr = m_nativeFuncPtr;
+    ex->m_info = m_info;
+    ex->m_line2 = line2;
+    ex->m_past = past;
+    ex->m_returnByValue = false;
+  }
 
-  f->shared()->m_info = m_info;
-  f->shared()->m_returnType = returnType;
   std::vector<Func::ParamInfo> fParams;
+  bool usesDoubles = false;
   for (unsigned i = 0; i < params.size(); ++i) {
     Func::ParamInfo pi = params[i];
+    if (pi.builtinType == KindOfDouble) usesDoubles = true;
+    if (pi.isVariadic()) {
+      pi.builtinType = KindOfArray;
+    }
     f->appendParam(params[i].byRef, pi, fParams);
   }
 
+  f->shared()->m_returnType = returnType;
   f->shared()->m_localNames.create(m_localNames);
   f->shared()->m_numLocals = m_numLocals;
   f->shared()->m_numIterators = m_numIterators;
   f->m_maxStackCells = maxStackCells;
   f->shared()->m_staticVars = staticVars;
-  f->shared()->m_ehtab = ehtab;
+  f->shared()->m_ehtab = toFixed(ehtab);
   f->shared()->m_fpitab = fpitab;
   f->shared()->m_isClosureBody = isClosureBody;
   f->shared()->m_isAsync = isAsync;
   f->shared()->m_isGenerator = isGenerator;
   f->shared()->m_isPairGenerator = isPairGenerator;
   f->shared()->m_userAttributes = userAttributes;
-  f->shared()->m_builtinFuncPtr = m_builtinFuncPtr;
-  f->shared()->m_nativeFuncPtr = m_nativeFuncPtr;
   f->shared()->m_retTypeConstraint = retTypeConstraint;
   f->shared()->m_retUserType = retUserType;
   f->shared()->m_originalFilename = originalFilename;
   f->shared()->m_isGenerated = isGenerated;
 
-  f->finishedEmittingParams(fParams);
-
   if (attrs & AttrNative) {
-    auto nif = Native::GetBuiltinFunction(name,
-                                          m_pce ? m_pce->name() : nullptr,
-                                          f->isStatic());
+    auto const ex = f->extShared();
+
+    auto const& info = Native::GetBuiltinFunction(
+      name,
+      m_pce ? m_pce->name() : nullptr,
+      f->isStatic()
+    );
+
+    auto const nif = info.ptr;
     if (nif) {
       Attr dummy = AttrNone;
       int nativeAttrs = parseNativeAttributes(dummy);
       if (nativeAttrs & Native::AttrZendCompat) {
-        f->shared()->m_nativeFuncPtr = nif;
-        f->shared()->m_builtinFuncPtr = zend_wrap_func;
+        ex->m_nativeFuncPtr = nif;
+        ex->m_builtinFuncPtr = zend_wrap_func;
       } else {
         if (parseNativeAttributes(dummy) & Native::AttrActRec) {
-          f->shared()->m_builtinFuncPtr = nif;
-          f->shared()->m_nativeFuncPtr = nullptr;
+          ex->m_builtinFuncPtr = nif;
+          ex->m_nativeFuncPtr = nullptr;
         } else {
-          f->shared()->m_nativeFuncPtr = nif;
-          f->shared()->m_builtinFuncPtr = m_pce ? Native::methodWrapper
-                                                : Native::functionWrapper;
+          ex->m_nativeFuncPtr = nif;
+          ex->m_builtinFuncPtr = Native::getWrapper(m_pce, usesDoubles);
+
+          if (info.sig.ret == Native::NativeSig::Type::MixedTV) {
+            ex->m_returnByValue = true;
+          }
+          int extra =
+            (attrs & AttrNumArgs ? 1 : 0) +
+            (isMethod() ? 1 : 0);
+          assert(info.sig.args.size() == params.size() + extra);
+          for (auto i = params.size(); i--; ) {
+            switch (info.sig.args[extra + i]) {
+              case Native::NativeSig::Type::ObjectArg:
+              case Native::NativeSig::Type::StringArg:
+              case Native::NativeSig::Type::ArrayArg:
+              case Native::NativeSig::Type::ResourceArg:
+              case Native::NativeSig::Type::OutputArg:
+              case Native::NativeSig::Type::MixedTV:
+                fParams[i].nativeArg = true;
+                break;
+              default:
+                break;
+            }
+          }
         }
       }
     } else {
-      f->shared()->m_builtinFuncPtr = Native::unimplementedWrapper;
+      ex->m_builtinFuncPtr = Native::unimplementedWrapper;
     }
   }
+
+  f->finishedEmittingParams(fParams);
   return f;
 }
 
@@ -303,10 +391,10 @@ Id FuncEmitter::allocUnnamedLocal() {
 ///////////////////////////////////////////////////////////////////////////////
 // Unit tables.
 
-EHEnt& FuncEmitter::addEHEnt() {
+EHEntEmitter& FuncEmitter::addEHEnt() {
   assert(!m_ehTabSorted
     || "should only mark the ehtab as sorted after adding all of them");
-  ehtab.push_back(EHEnt());
+  ehtab.push_back(EHEntEmitter());
   ehtab.back().m_parentIndex = 7777;
   return ehtab.back();
 }
@@ -327,10 +415,12 @@ namespace {
  *       e2 is a Fault funclet.
  */
 struct EHEntComp {
-  bool operator()(const EHEnt& e1, const EHEnt& e2) const {
+  bool operator()(const EHEntEmitter& e1, const EHEntEmitter& e2) const {
     if (e1.m_base == e2.m_base) {
       if (e1.m_past == e2.m_past) {
-        return e1.m_type == EHEnt::Type::Catch;
+        static_assert(!static_cast<uint8_t>(EHEnt::Type::Catch),
+            "Catch should be the smallest type");
+        return e1.m_type < e2.m_type;
       }
       return e1.m_past > e2.m_past;
     }
@@ -434,7 +524,9 @@ static const StaticString
   s_nofcallbuiltin("NoFCallBuiltin"),
   s_variadicbyref("VariadicByRef"),
   s_noinjection("NoInjection"),
-  s_zendcompat("ZendCompat");
+  s_zendcompat("ZendCompat"),
+  s_numargs("NumArgs"),
+  s_opcodeimpl("OpCodeImpl");
 
 int FuncEmitter::parseNativeAttributes(Attr& attrs_) const {
   int ret = Native::AttrNone;
@@ -442,7 +534,7 @@ int FuncEmitter::parseNativeAttributes(Attr& attrs_) const {
   auto it = userAttributes.find(s_native.get());
   assert(it != userAttributes.end());
   const TypedValue userAttr = it->second;
-  assert(userAttr.m_type == KindOfArray);
+  assert(isArrayType(userAttr.m_type));
   for (ArrayIter it(userAttr.m_data.parr); it; ++it) {
     Variant userAttrVal = it.second();
     if (userAttrVal.isString()) {
@@ -461,6 +553,10 @@ int FuncEmitter::parseNativeAttributes(Attr& attrs_) const {
         // ZendCompat implies ActRec, no FCallBuiltin
         attrs_ |= AttrMayUseVV | AttrNoFCallBuiltin;
         ret |= Native::AttrActRec;
+      } else if (userAttrStrVal.get()->isame(s_numargs.get())) {
+        attrs_ |= AttrNumArgs;
+      } else if (userAttrStrVal.get()->isame(s_opcodeimpl.get())) {
+        ret |= Native::AttrOpCodeImpl;
       }
     }
   }
@@ -473,8 +569,7 @@ void FuncEmitter::setBuiltinFunc(const ClassInfo::MethodInfo* info,
   assert(info);
   m_info = info;
   Attr attrs_ = AttrBuiltin;
-  if (info->attribute & (ClassInfo::RefVariableArguments |
-                         ClassInfo::MixedVariableArguments)) {
+  if (info->attribute & ClassInfo::RefVariableArguments) {
     attrs_ |= AttrVariadicByRef;
   }
   if (info->attribute & ClassInfo::IsReference) {
@@ -544,18 +639,10 @@ void FuncEmitter::setBuiltinFunc(BuiltinFunction bif, BuiltinFunction nif,
 // FuncRepoProxy.
 
 FuncRepoProxy::FuncRepoProxy(Repo& repo)
-  : RepoProxy(repo)
-#define FRP_OP(c, o) \
-  , m_##o##Local(repo, RepoIdLocal), m_##o##Central(repo, RepoIdCentral)
-    FRP_OPS
-#undef FRP_OP
-{
-#define FRP_OP(c, o) \
-  m_##o[RepoIdLocal] = &m_##o##Local; \
-  m_##o[RepoIdCentral] = &m_##o##Central;
-  FRP_OPS
-#undef FRP_OP
-}
+    : RepoProxy(repo),
+      insertFunc{InsertFuncStmt(repo, 0), InsertFuncStmt(repo, 1)},
+      getFuncs{GetFuncsStmt(repo, 0), GetFuncsStmt(repo, 1)}
+{}
 
 FuncRepoProxy::~FuncRepoProxy() {
 }
@@ -607,7 +694,7 @@ void FuncRepoProxy::GetFuncsStmt
     txn.prepare(*this, ssSelect.str());
   }
   RepoTxnQuery query(txn, *this);
-  query.bindInt64("@unitSn", ue.sn());
+  query.bindInt64("@unitSn", ue.m_sn);
   do {
     query.step();
     if (query.row()) {
