@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,11 +18,9 @@
 
 #include "hphp/runtime/vm/srckey.h"
 
-namespace HPHP {
-namespace JIT { class SrcRec; }
-namespace JIT {
-
-void addDbgGuardImpl(SrcKey sk, JIT::SrcRec* srcRec);
+namespace HPHP { namespace jit {
+class SrcRec;
+void addDbgGuardImpl(SrcKey sk, SrcRec* srcRec);
 
 }}
 
