@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,11 +14,11 @@
    +----------------------------------------------------------------------+
 */
 
+#include "hphp/runtime/vm/jit/prof-data.h"
 #include "hphp/runtime/vm/jit/trans-cfg.h"
 #include "hphp/runtime/vm/jit/translator-inline.h"
 
-namespace HPHP {
-namespace JIT {
+namespace HPHP { namespace jit {
 
 RegionDescPtr selectHotBlock(TransID transId,
                              const ProfData* profData,
@@ -26,4 +26,4 @@ RegionDescPtr selectHotBlock(TransID transId,
   return profData->transRegion(transId);
 }
 
-} }
+}}
