@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -19,12 +19,11 @@
 #include "hphp/runtime/vm/jit/mc-generator.h"
 #include "translator.h"
 
-namespace HPHP { namespace JIT {
+namespace HPHP { namespace jit {
 
 void reportTraceletToVtune(const Unit* unit,
                            const Func* func,
                            const TransRec& transRec);
-void reportTrampolineToVtune(void* begin, size_t size);
 
 }}
 
