@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | HipHop for PHP                                                       |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2010-2014 Facebook, Inc. (http://www.facebook.com)     |
+   | Copyright (c) 2010-2015 Facebook, Inc. (http://www.facebook.com)     |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,7 +15,8 @@
 */
 #include "hphp/runtime/vm/repo.h"
 
-#include "folly/Format.h"
+#include <folly/Format.h>
+#include <folly/Singleton.h>
 
 #include "hphp/util/logger.h"
 #include "hphp/util/trace.h"
@@ -75,11 +76,14 @@ bool Repo::prefork() {
     s_lock.unlock();
     return true;
   }
+  folly::SingletonVault::singleton()->destroyInstances();
   return false;
 }
 
 void Repo::postfork(pid_t pid) {
+  folly::SingletonVault::singleton()->reenableInstances();
   if (pid == 0) {
+    Logger::ResetPid();
     new (&s_lock) SimpleMutex();
   } else {
     s_lock.unlock();
@@ -88,19 +92,13 @@ void Repo::postfork(pid_t pid) {
 
 Repo::Repo()
   : RepoProxy(*this),
-#define RP_OP(c, o) \
-    m_##o##Local(*this, RepoIdLocal), m_##o##Central(*this, RepoIdCentral),
-    RP_OPS
-#undef RP_OP
+    m_insertFileHash{InsertFileHashStmt(*this, 0),
+                     InsertFileHashStmt(*this, 1)},
+    m_getFileHash{GetFileHashStmt(*this, 0), GetFileHashStmt(*this, 1)},
     m_dbc(nullptr), m_localReadable(false), m_localWritable(false),
     m_evalRepoId(-1), m_txDepth(0), m_rollback(false), m_beginStmt(*this),
     m_rollbackStmt(*this), m_commitStmt(*this), m_urp(*this), m_pcrp(*this),
     m_frp(*this), m_lsrp(*this) {
-#define RP_OP(c, o) \
-  m_##o[RepoIdLocal] = &m_##o##Local; \
-  m_##o[RepoIdCentral] = &m_##o##Central;
-  RP_OPS
-#undef RP_OP
   {
     SimpleLock lock(s_lock);
     s_nRepos++;
@@ -123,6 +121,11 @@ void Repo::setCliFile(const std::string& cliFile) {
   s_cliFile = cliFile;
 }
 
+size_t Repo::stringLengthLimit() const {
+  static const size_t limit = sqlite3_limit(m_dbc, SQLITE_LIMIT_LENGTH, -1);
+  return limit;
+}
+
 void Repo::loadGlobalData(bool allowFailure /* = false */) {
   m_lsrp.load();
 
@@ -169,6 +172,14 @@ void Repo::loadGlobalData(bool allowFailure /* = false */) {
       continue;
     }
 
+    // TODO: this should probably read out the other elements of the global data
+    // which control Option or RuntimeOption values -- the others are read out
+    // in an inconsistent and ad-hoc manner. But I don't understand their uses
+    // and interactions well enough to feel comfortable fixing now.
+    RuntimeOption::PHP7_IntSemantics = s_globalData.PHP7_IntSemantics;
+    RuntimeOption::PHP7_ScalarTypes  = s_globalData.PHP7_ScalarTypes;
+    RuntimeOption::AutoprimeGenerators = s_globalData.AutoprimeGenerators;
+
     return;
   }
 
@@ -211,27 +222,34 @@ void Repo::saveGlobalData(GlobalData newData) {
 
   // TODO(#3521039): we could just put the litstr table in the same
   // blob as the above and delete LitstrRepoProxy.
-  LitstrTable::get().insert(txn, UnitOrigin::File);
+  LitstrTable::get().insert(txn);
 
   txn.commit();
 }
 
-Unit* Repo::loadUnit(const std::string& name, const MD5& md5) {
+std::unique_ptr<Unit> Repo::loadUnit(const std::string& name, const MD5& md5) {
   if (m_dbc == nullptr) {
     return nullptr;
   }
   return m_urp.load(name, md5);
 }
 
-std::vector<std::pair<std::string,MD5>> Repo::enumerateUnits() {
+std::vector<std::pair<std::string,MD5>>
+Repo::enumerateUnits(int repoId, bool preloadOnly, bool warn) {
   std::vector<std::pair<std::string,MD5>> ret;
 
   try {
     RepoStmt stmt(*this);
-    stmt.prepare(
-      folly::format(
-        "SELECT path, md5 FROM {};", table(RepoIdCentral, "FileMd5")
-      ).str()
+    stmt.prepare(preloadOnly ?
+                 folly::sformat(
+                   "SELECT path, {0}.md5 FROM {0} "
+                   "LEFT JOIN {1} ON ({0}.md5={1}.md5) WHERE preload != 0 "
+                   "ORDER BY preload DESC;",
+                   table(repoId, "FileMd5"),
+                   table(repoId, "Unit")) :
+                 folly::sformat(
+                   "SELECT path, md5 FROM {};",
+                   table(repoId, "FileMd5"))
     );
     RepoTxn txn(*this);
     RepoTxnQuery query(txn, stmt);
@@ -248,7 +266,9 @@ std::vector<std::pair<std::string,MD5>> Repo::enumerateUnits() {
 
     txn.commit();
   } catch (RepoExc& e) {
-    fprintf(stderr, "failed to enumerate units: %s\n", e.what());
+    if (warn) {
+      fprintf(stderr, "failed to enumerate units: %s\n", e.what());
+    }
   }
 
   return ret;
@@ -303,12 +323,12 @@ bool Repo::findFile(const char *path, const std::string &root, MD5& md5) {
   for (repoId = RepoIdCount - 1; repoId >= 0; --repoId) {
     if (*path == '/' && !root.empty() &&
         !strncmp(root.c_str(), path, root.size()) &&
-        getFileHash(repoId).get(path + root.size(), md5)) {
+        m_getFileHash[repoId].get(path + root.size(), md5)) {
       TRACE(3, "Repo loaded file hash for '%s' from '%s'\n",
                path + root.size(), repoName(repoId).c_str());
       return true;
     }
-    if (getFileHash(repoId).get(path, md5)) {
+    if (m_getFileHash[repoId].get(path, md5)) {
       TRACE(3, "Repo loaded file hash for '%s' from '%s'\n",
                 path, repoName(repoId).c_str());
       return true;
@@ -319,14 +339,14 @@ bool Repo::findFile(const char *path, const std::string &root, MD5& md5) {
 }
 
 bool Repo::insertMd5(UnitOrigin unitOrigin, UnitEmitter* ue, RepoTxn& txn) {
-  const StringData* path = ue->getFilepath();
+  const StringData* path = ue->m_filepath;
   const MD5& md5 = ue->md5();
   int repoId = repoIdForNewUnit(unitOrigin);
   if (repoId == RepoIdInvalid) {
     return true;
   }
   try {
-    insertFileHash(repoId).insert(txn, path, md5);
+    m_insertFileHash[repoId].insert(txn, path, md5);
     return false;
   } catch(RepoExc& re) {
     TRACE(3, "Failed to commit md5 for '%s' to '%s': %s\n",
@@ -346,7 +366,7 @@ void Repo::commitMd5(UnitOrigin unitOrigin, UnitEmitter* ue) {
     int repoId = repoIdForNewUnit(unitOrigin);
     if (repoId != RepoIdInvalid) {
       TRACE(3, "Failed to commit md5 for '%s' to '%s': %s\n",
-               ue->getFilepath()->data(), repoName(repoId).c_str(),
+               ue->m_filepath->data(), repoName(repoId).c_str(),
                re.msg().c_str());
     }
   }
@@ -535,6 +555,7 @@ void Repo::initCentral() {
     }
   }
 
+#ifndef _WIN32
   // Try the equivalent of "$HOME/.hhvm.hhbc", but look up the home directory
   // in the password database.
   {
@@ -542,7 +563,8 @@ void Repo::initCentral() {
     struct passwd* pwbufp;
     long bufsize = sysconf(_SC_GETPW_R_SIZE_MAX);
     if (bufsize != -1) {
-      char buf[size_t(bufsize)];
+      auto buf = new char[bufsize];
+      SCOPE_EXIT { delete[] buf; };
       if (!getpwuid_r(getuid(), &pwbuf, buf, size_t(bufsize), &pwbufp)
           && (HOME == nullptr || strcmp(HOME, pwbufp->pw_dir))) {
         std::string centralPath = pwbufp->pw_dir;
@@ -553,6 +575,20 @@ void Repo::initCentral() {
       }
     }
   }
+#else // _WIN32
+  // Try "$HOMEDRIVE$HOMEPATH/.hhvm.hhbc"
+  char* HOMEDRIVE = getenv("HOMEDRIVE");
+  if (HOMEDRIVE != nullptr) {
+    char* HOMEPATH = getenv("HOMEPATH");
+    if (HOMEPATH != nullptr) {
+      std::string centralPath = HOMEDRIVE;
+      centralPath += HOMEPATH;
+      centralPath += "\\.hhvm.hhbc";
+      if (tryPath(centralPath.c_str()))
+        return;
+    }
+  }
+#endif
 
   error = "Failed to initialize central HHBC repository:\n" + error;
   // Database initialization failed; this is an unrecoverable state.
@@ -695,20 +731,19 @@ void Repo::pragmas(int repoId) {
   // Valid synchronous values: 0 (OFF), 1 (NORMAL), 2 (FULL).
   static const int synchronous = 0;
   setIntPragma(repoId, "synchronous", synchronous);
+  setIntPragma(repoId, "cache_size", 20);
   // Valid journal_mode values: delete, truncate, persist, memory, wal, off.
   setTextPragma(repoId, "journal_mode", RuntimeOption::RepoJournal.c_str());
 }
 
 void Repo::getIntPragma(int repoId, const char* name, int& val) {
-  RepoTxn txn(*this);
   std::stringstream ssPragma;
   ssPragma << "PRAGMA " << dbName(repoId) << "." << name << ";";
   RepoStmt stmt(*this);
   stmt.prepare(ssPragma.str());
-  RepoTxnQuery query(txn, stmt);
+  RepoQuery query(stmt);
   query.step();
   query.getInt(0, val);
-  txn.commit();
 }
 
 void Repo::setIntPragma(int repoId, const char* name, int val) {
@@ -729,24 +764,23 @@ void Repo::setIntPragma(int repoId, const char* name, int val) {
 }
 
 void Repo::getTextPragma(int repoId, const char* name, std::string& val) {
-  RepoTxn txn(*this);
   std::stringstream ssPragma;
   ssPragma << "PRAGMA " << dbName(repoId) << "." << name << ";";
   RepoStmt stmt(*this);
   stmt.prepare(ssPragma.str());
-  RepoTxnQuery query(txn, stmt);
+  RepoQuery query(stmt);
   const char* s;
   query.step();
   query.getText(0, s);
   val = s;
-  txn.commit();
 }
 
 void Repo::setTextPragma(int repoId, const char* name, const char* val) {
   // Pragma writes must be executed outside transactions, since they may change
   // transaction behavior.
   std::stringstream ssPragma;
-  ssPragma << "PRAGMA " << dbName(repoId) << "." << name << " = " << val << ";";
+  ssPragma <<
+    "PRAGMA " << dbName(repoId) << "." << name << " = '" << val << "';";
   exec(ssPragma.str());
   if (debug) {
     // Verify that the pragma had the desired effect.
