@@ -134,14 +134,12 @@ define('IsCppSerializable',              1 << 15);
 define('HipHopSpecific',                 1 << 16);
 define('VariableArguments',              1 << 17);
 define('RefVariableArguments',           1 << 18);
-define('MixedVariableArguments',         1 << 19);
 define('FunctionIsFoldable',             1 << 20);
 define('NoEffect',                       1 << 21);
 define('NoInjection',                    1 << 22);
 define('HasOptFunction',                 1 << 23);
 define('AllowIntercept',                 1 << 24);
 define('NoProfile',                      1 << 25);
-define('ContextSensitive',               1 << 26);
 define('NoDefaultSweep',                 1 << 27);
 define('IsSystem',                       1 << 28);
 define('IsTrait',                        1 << 29);
@@ -149,8 +147,7 @@ define('ParamCoerceModeFalse',             1 << 30);
 define('NoFCallBulitin',                 1 << 31);
 
 // Mask for checking the flags related to variable arguments
-define('VarArgsMask', (VariableArguments | RefVariableArguments |
-                       MixedVariableArguments));
+define('VarArgsMask', (VariableArguments | RefVariableArguments));
 
 function get_flag_names($arr, $name, $global_func) {
   $flag = 0;
@@ -173,14 +170,12 @@ function get_flag_names($arr, $name, $global_func) {
   if ($flag & HipHopSpecific        ) $ret .= ' | HipHopSpecific'        ;
   if ($flag & VariableArguments     ) $ret .= ' | VariableArguments'     ;
   if ($flag & RefVariableArguments  ) $ret .= ' | RefVariableArguments'  ;
-  if ($flag & MixedVariableArguments) $ret .= ' | MixedVariableArguments';
   if ($flag & FunctionIsFoldable    ) $ret .= ' | FunctionIsFoldable'    ;
   if ($flag & NoEffect              ) $ret .= ' | NoEffect'              ;
   if ($flag & NoInjection           ) $ret .= ' | NoInjection'           ;
   if ($flag & HasOptFunction        ) $ret .= ' | HasOptFunction'        ;
   if ($flag & AllowIntercept        ) $ret .= ' | AllowIntercept'        ;
   if ($flag & NoProfile             ) $ret .= ' | NoProfile'             ;
-  if ($flag & ContextSensitive      ) $ret .= ' | ContextSensitive'      ;
   if ($flag & NoDefaultSweep        ) $ret .= ' | NoDefaultSweep'        ;
   if ($flag & IsTrait               ) $ret .= ' | IsTrait'               ;
   if ($flag & NoFCallBuiltin        ) $ret .= ' | NoFCallBuiltin'        ;
