@@ -46,7 +46,6 @@ class Redis {
   /* Type of serialization to use with values stored in redis */
   const SERIALIZER_NONE     = 0;
   const SERIALIZER_PHP      = 1;
-  const SERIALIZER_IGBINARY = 2;
 
   /* Options used by lInsert and similar methods */
   const AFTER  = 'after';
@@ -97,9 +96,8 @@ class Redis {
         return true;
 
       case self::OPT_SERIALIZER:
-        if (($value !== self::SERIALIZER_NONE) AND
-            ($value !== self::SERIALIZER_PHP) AND
-            ($value !== self::SERIALIZER_IGBINARY)) {
+        if (($value !== self::SERIALIZER_NONE) &&
+            ($value !== self::SERIALIZER_PHP)) {
           throw new RedisException("Invalid serializer option: $value");
         }
         $this->serializer = (int)$value;
@@ -186,7 +184,7 @@ class Redis {
     if ($by !== 1) {
       return $this->decrBy($key, $by);
     }
-    $this->processCommand("DECR", $this->prefix($key));
+    $this->processCommand("DECR", $this->_prefix($key));
     return $this->processLongResponse();
   }
 
@@ -194,7 +192,7 @@ class Redis {
     if ($by === 1) {
       return $this->decr($key);
     }
-    $this->processCommand("DECRBY", $this->prefix($key), (int)$by);
+    $this->processCommand("DECRBY", $this->_prefix($key), (int)$by);
     return $this->processLongResponse();
   }
 
@@ -202,7 +200,7 @@ class Redis {
     if ($by !== 1) {
       return $this->incrBy($key, $by);
     }
-    $this->processCommand("INCR", $this->prefix($key));
+    $this->processCommand("INCR", $this->_prefix($key));
     return $this->processLongResponse();
   }
 
@@ -210,25 +208,25 @@ class Redis {
     if ($by === 1) {
       return $this->incr($key);
     }
-    $this->processCommand("INCRBY", $this->prefix($key), (int)$by);
+    $this->processCommand("INCRBY", $this->_prefix($key), (int)$by);
     return $this->processLongResponse();
   }
 
   public function incrByFloat($key, $by) {
-    $this->processCommand("INCRBYFLOAT", $this->prefix($key),
+    $this->processCommand("INCRBYFLOAT", $this->_prefix($key),
                                                (float)$by);
     return $this->processDoubleResponse();
   }
 
   public function set($key, $value, $optionArrayOrExpiration = -1) {
-    $key = $this->prefix($key);
-    $value = $this->serialize($value);
+    $key = $this->_prefix($key);
+    $value = $this->_serialize($value);
     if (is_array($optionArrayOrExpiration) &&
         count($optionArrayOrExpiration) > 0) {
       $ex = array_key_exists('ex', $optionArrayOrExpiration);
       $px = array_key_exists('px', $optionArrayOrExpiration);
-      $nx = in_array('nx', $optionArrayOrExpiration);
-      $xx = in_array('xx', $optionArrayOrExpiration);
+      $nx = in_array('nx', $optionArrayOrExpiration, true);
+      $xx = in_array('xx', $optionArrayOrExpiration, true);
       if ($nx && $xx) {
         throw new RedisException(
           "Invalid set options: nx and xx may not be specified at the same time"
@@ -339,7 +337,7 @@ class Redis {
   }
 
   public function object($info, $key) {
-    $this->processCommand('OBJECT', $info, $this->prefix($key));
+    $this->processCommand('OBJECT', $info, $this->_prefix($key));
     switch ($info) {
       case 'refcount': return $this->processLongResponse();
       case 'encoding': return $this->processStringResponse();
@@ -351,21 +349,35 @@ class Redis {
 
   public function hMGet($key, array $members) {
     $members = array_values($members);
-    $args = array_merge([$this->prefix($key)], $members);
+    $args = array_merge([$this->_prefix($key)], $members);
     $this->processArrayCommand('HMGET', $args);
     return $this->processAssocResponse($members);
   }
 
   public function hMSet($key, array $pairs) {
-    $args = [$this->prefix($key)];
+    $args = [$this->_prefix($key)];
     foreach ($pairs as $k => $v) {
       $args[] = $k;
-      $args[] = $this->serialize($v);
+      $args[] = $this->_serialize($v);
     }
     $this->processArrayCommand('HMSET', $args);
     return $this->processBooleanResponse();
   }
 
+  /* Sets ---------------------------------------------------------------- */
+
+  public function sRandMember($key, $count = null) {
+    $args = [$this->_prefix($key)];
+    if ($count !== null) {
+       $args[] = $count;
+    }
+    $this->processArrayCommand('SRANDMEMBER', $args);
+    if ($count !== null) {
+       return $this->processVectorResponse(true);
+    }
+    return $this->processStringResponse();
+  }
+
   /* zSets --------------------------------------------------------------- */
 
   public function zAdd($key, $score, $value/*, $scoreN, $valueN */) {
@@ -374,10 +386,10 @@ class Redis {
     if (($count - 1) % 2) {
       return false;
     }
-    $args[0] = $this->prefix($args[0]);
+    $args[0] = $this->_prefix($args[0]);
     for ($i = 1; $i < $count; $i += 2) {
       $args[$i  ] = (double)$args[$i];
-      $args[$i+1] = $this->serialize($args[$i+1]);
+      $args[$i+1] = $this->_serialize($args[$i+1]);
     }
     $this->processArrayCommand('ZADD', $args);
     return $this->processLongResponse();
@@ -388,9 +400,9 @@ class Redis {
                                       array $keys,
                                       array $weights = null,
                                       $op = '') {
-    $args = [ $this->prefix($key), count($keys) ];
+    $args = [ $this->_prefix($key), count($keys) ];
     foreach ($keys as $k) {
-      $args[] = $this->prefix($k);
+      $args[] = $this->_prefix($k);
     }
 
     if ($weights) {
@@ -431,7 +443,7 @@ class Redis {
 
   public function zRange($key, $start, $end, $withscores = false) {
     $args = [
-      $this->prefix($key),
+      $this->_prefix($key),
       (int)$start,
       (int)$end,
     ];
@@ -450,7 +462,7 @@ class Redis {
                                        $start,
                                        $end,
                                        array $opts = null) {
-    $args = [$this->prefix($key), (int)$start, (int)$end];
+    $args = [$this->_prefix($key), $start, $end];
     if (isset($opts['limit']) AND
         is_array($opts['limit']) AND
         (count($opts['limit']) == 2)) {
@@ -481,7 +493,7 @@ class Redis {
 
   public function zRevRange($key, $start, $end, $withscores = false) {
     $args = [
-      $this->prefix($key),
+      $this->_prefix($key),
       (int)$start,
       (int)$end,
     ];
@@ -495,6 +507,39 @@ class Redis {
     return $this->processVectorResponse(true);
   }
 
+  /* Scan --------------------------------------------------------------- */
+
+  protected function scanImpl($cmd, $key, $cursor, $pattern, $count) {
+    $args = [];
+    if ($key !== null) {
+      $args[] = $this->_prefix($key);
+    }
+    $args[] = (int)$cursor;
+    if ($pattern !== null) {
+      $args[] = 'MATCH';
+      if ($cmd === 'SCAN') {
+        $args[] = (string)$this->_prefix($pattern);
+      }
+      else {
+        $args[] = (string)$pattern;
+      }
+    }
+    if ($count !== null) {
+      $args[] = 'COUNT';
+      $args[] = (int)$count;
+    }
+    $this->processArrayCommand($cmd, $args);
+    return $this->processVariantResponse();
+  }
+
+  public function scan($cursor, $pattern = null, $count = null) {
+    return $this->scanImpl('SCAN', null, $cursor, $pattern, $count);
+}
+
+  public function sScan($key, $cursor, $pattern = null, $count = null) {
+    return $this->scanImpl('SSCAN', $key, $cursor, $pattern, $count);
+  }
+
   /* Multi --------------------------------------------------------------- */
 
   protected function flushCallbacks($multibulk = true) {
@@ -559,10 +604,15 @@ class Redis {
     return $this;
   }
 
-  public function watch($key, /* ... */) {
-    $args = array_map([$this, 'prefix'], func_get_args());
+  public function watch($key/* ... */) {
+    $args = array_map([$this, '_prefix'], func_get_args());
     $this->processArrayCommand("WATCH", $args);
-    return $this->processBooleanResponse();;
+    return $this->processBooleanResponse();
+  }
+
+  public function unwatch() {
+    $this->processCommand("UNWATCH");
+    return $this->processBooleanResponse();
   }
 
   /* Batch --------------------------------------------------------------- */
@@ -570,8 +620,8 @@ class Redis {
   protected function processMSetCommand($cmd, array $data) {
     $args = [];
     foreach ($data as $key => $val) {
-      $args[] = $this->prefix($key);
-      $args[] = $this->serialize($val);
+      $args[] = $this->_prefix($key);
+      $args[] = $this->_serialize($val);
     }
     $this->processArrayCommand($cmd, $args);
   }
@@ -592,22 +642,31 @@ class Redis {
     $keyCount = $numKeys;
     foreach($args as &$arg) {
       if ($keyCount-- <= 0) break;
-      $arg = $this->prefix($arg);
+      $arg = $this->_prefix($arg);
     }
     array_unshift($args, $numKeys);
     array_unshift($args, $script);
     $this->processArrayCommand($cmd, $args);
-    return $this->processVariantResponse();
+    $response = $this->processVariantResponse();
+    return ($response !== NULL) ? $response : false;
   }
 
   public function evaluate($script, array $args = [], $numKeys = 0) {
     return $this->doEval('EVAL', $script, $args, $numKeys);
   }
 
+  public function eval($script, array $args = [], $numKeys = 0) {
+    return $this->doEval('EVAL', $script, $args, $numKeys);
+  }
+
   public function evaluateSha($sha, array $args = [], $numKeys = 0) {
     return $this->doEval('EVALSHA', $sha, $args, $numKeys);
   }
 
+  public function evalSha($sha, array $args = [], $numKeys = 0) {
+    return $this->doEval('EVALSHA', $sha, $args, $numKeys);
+  }
+
   public function script($subcmd/* ... */) {
     switch (strtolower($subcmd)) {
       case 'flush':
@@ -655,7 +714,7 @@ class Redis {
   }
 
   public function getTimeout() {
-    return $this->getReadTimeout();
+    return $this->timeout_connect;
   }
 
   public function getReadTimeout() {
@@ -807,7 +866,7 @@ class Redis {
     'lpushx' => [ 'format' => 'kl', 'return' => 'Long' ],
     'lrange' => [ 'format' => 'kll', 'return' => 'Vector', 'retargs' => [1] ],
     'lgetrange' => [ 'alias' => 'lrange' ],
-    'lrem' => [ 'format' => 'kvl', 'return' => 'Long' ],
+    'lrem' => [ 'format' => 'kvs', 'return' => 'Long' ],
     'lremove' => [ 'alias' => 'lrem' ],
     'lset' => [ 'format' => 'klv', 'return' => 'Boolean' ],
     'ltrim' => [ 'format' => 'kll', 'return' => 'Boolean' ],
@@ -833,8 +892,6 @@ class Redis {
     'sgetmembers' => [ 'alias' => 'smembers' ],
     'smove' => [ 'format' => 'kkv', 'return' => '1' ],
     'spop' => [ 'format' => 'k', 'return' => 'Serialized' ],
-    'srandmember' => [ 'format' => 'kl', 'return' => 'Serialized',
-                       'default' => [ 1 => 1 ] ],
     'srem' => [ 'vararg' => self::VAR_KEY_FIRST_AND_SERIALIZE,
                 'return' => 'Long' ],
     'sremove' => [ 'alias' => 'srem' ],
@@ -869,19 +926,10 @@ class Redis {
     'unsubscribe' => false,
     'punsubscribe' => false,
 
-    // Introspection
-    '_prefix' => [ 'alias' => 'prefix' ],
-    '_serialize' => [ 'alias' => 'serialize' ],
-    '_unserialize' => [ 'alias' => 'unserialize' ],
-
     // Batch Ops
     'mget' => [ 'vararg' => self::VAR_KEY_ALL,
                 'return' => 'Vector', 'retargs' => [1] ],
     'getmultiple' => [ 'alias' => 'mget' ],
-
-    // Eval
-    'eval' => [ 'alias' => 'evaluate' ],
-    'evalsha' => [ 'alias' => 'evaluateSha' ],
   ];
 
 
@@ -932,7 +980,15 @@ class Redis {
   const TYPE_MULTIBULK = '*';
 
   protected function checkConnection($auto_reconnect = true) {
-    if ($this->connection AND !feof($this->connection)) {
+    if (!$this->connection) {
+        return false;
+    }
+
+    // Check if we have hit the stream timeout
+    if (stream_get_meta_data($this->connection)['timed_out']) {
+      throw new RedisException("read error on connection");
+    }
+    if (!feof($this->connection)) {
       // Connection seems fine
       return true;
     }
@@ -961,24 +1017,15 @@ class Redis {
   }
 
   protected function sockReadLine() {
-    if (!$this->checkConnection()) {
-      return false;
-    }
-    $line = fgets($this->connection);
-    if (substr($line, -2) == "\r\n") {
-      $line = substr($line, 0, -2);
-    } else if (substr($line, -1) == "\r") {
-      $line = substr($line, 0, -1);
-      $lf = fgetc($this->connection);
-      if ($lf === false) {
-        // A response must terminate with both CR and LF. Refuse to guess.
+    $line = '';
+    do {
+      if (!$this->checkConnection()) {
         return false;
-      } else if ($lf !== "\n") {
-        throw new RedisException("Protocol error: CR not followed by LF");
       }
-    }
+      $line .= fgets($this->connection);
+    } while (substr($line, -2) !== "\r\n");
 
-    return $line;
+    return substr($line, 0, -2);
   }
 
   protected function sockReadData(&$type) {
@@ -1056,9 +1103,9 @@ class Redis {
         if (( $first AND ($varkey == self::VAR_KEY_FIRST)) OR
             (!$first AND ($varkey == self::VAR_KEY_NOT_FIRST)) OR
                          ($varkey == self::VAR_KEY_ALL)) {
-          $arg = $this->prefix($arg);
+          $arg = $this->_prefix($arg);
         } else if ($flags & self::VAR_SERIALIZE) {
-          $arg = $this->serialize($arg);
+          $arg = $this->_serialize($arg);
         }
         $first = false;
       }
@@ -1110,31 +1157,29 @@ class Redis {
     return (bool)fwrite($this->connection, $cmd);
   }
 
-  protected function processCommand($cmd, /* ... */) {
+  protected function processCommand($cmd/* ... */) {
     $args = func_get_args();
     array_shift($args);
     return $this->processArrayCommand($cmd, $args);
   }
 
-  protected function serialize($str) {
+  public function _serialize($str) {
     switch ($this->serializer) {
       case self::SERIALIZER_NONE:
         return $str;
       case self::SERIALIZER_PHP:
         return serialize($str);
-      case self::SERIALIZER_IGBINARY:
       default:
         throw new RedisException("Not Implemented");
     }
   }
 
-  protected function unserialize($str) {
+  public function _unserialize($str) {
     switch ($this->serializer) {
       case self::SERIALIZER_NONE:
         return $str;
       case self::SERIALIZER_PHP:
         return unserialize($str);
-      case self::SERIALIZER_IGBINARY:
       default:
         throw new RedisException("Not Implemented");
     }
@@ -1207,8 +1252,11 @@ class Redis {
   protected function processSerializedResponse() {
     if ($this->mode === self::ATOMIC) {
       $resp = $this->sockReadData($type);
+      if ($resp === null) {
+        return false;
+      }
       return (($type === self::TYPE_LINE) OR ($type === self::TYPE_BULK))
-             ? $this->unserialize($resp) : null;
+             ? $this->_unserialize($resp) : false;
     }
     $this->multiHandler[] = [ 'cb' => [$this,'processSerializedResponse'] ];
     if (($this->mode === self::MULTI) && !$this->processQueuedResponse()) {
@@ -1248,7 +1296,7 @@ class Redis {
           ($type === self::TYPE_BULK && is_numeric($resp))) {
         return (float)$resp;
       }
-      return null;
+      return false;
     }
     $this->multiHandler[] = [ 'cb' => [$this,'processDoubleResponse'] ];
     if (($this->mode === self::MULTI) && !$this->processQueuedResponse()) {
@@ -1292,9 +1340,9 @@ class Redis {
       $lineNo++;
       $val = $this->sockReadData($type);
       if ($unser AND (($lineNo % $unser) == 0)) {
-        $val = $this->unserialize($val);
+        $val = $this->_unserialize($val);
       }
-      $ret[] = $val;
+      $ret[] = $val !== null ? $val : false;
     }
     return $ret;
   }
@@ -1319,11 +1367,11 @@ class Redis {
     while($count > 1) {
       $key = $this->sockReadData($type);
       if ($unser_key) {
-        $key = $this->unserialize($key);
+        $key = $this->_unserialize($key);
       }
       $val = $this->sockReadData($type);
       if ($unser_val) {
-        $val = $this->unserialize($val);
+        $val = $this->_unserialize($val);
       }
       $ret[$key] = $val;
       $count -= 2;
@@ -1355,9 +1403,9 @@ class Redis {
       $key = array_shift($keys);
       $val = $this->sockReadData($type);
       if ($unser_val) {
-        $val = $this->unserialize($val);
+        $val = $this->_unserialize($val);
       }
-      $ret[$key] = $val;
+      $ret[$key] = $val !== null ? $val : false;
     }
     return $ret;
   }
@@ -1442,7 +1490,7 @@ class Redis {
     return ($type === self::TYPE_LINE) AND ($resp === 'QUEUED');
   }
 
-  protected function prefix($key) {
+  public function _prefix($key) {
     return $this->prefix . $key;
   }
 
@@ -1454,7 +1502,7 @@ class Redis {
   public function __call($fname, $args) {
     $fname = strtolower($fname);
     if (!isset(self::$map[$fname])) {
-      trigger_error("Call to undefined function Redis::$fname()", E_USER_ERROR);
+      trigger_error("Call to undefined function Redis::$fname()", E_ERROR);
       return null;
     }
     $func = self::$map[$fname];
@@ -1499,20 +1547,20 @@ class Redis {
 
     $flen = strlen($format);
     for ($i = 0; $i < $flen; $i++) {
-      if (!isset($args[$i])) {
+      if (!array_key_exists($i, $args)) {
         if (isset($func['defaults']) AND
             array_key_exists($func['defaults'], $i)) {
           $args[$i] = $func['defaults'][$i];
         } else {
           trigger_error(
             "Redis::$fname requires at least $flen parameters $argc given",
-            E_USER_ERROR);
+            E_ERROR);
           return null;
         }
       }
       switch ($format[$i]) {
-        case 'k': $args[$i] = $this->prefix($args[$i]); break;
-        case 'v': $args[$i] = $this->serialize($args[$i]); break;
+        case 'k': $args[$i] = $this->_prefix($args[$i]); break;
+        case 'v': $args[$i] = $this->_serialize($args[$i]); break;
         case 's': $args[$i] = (string)$args[$i]; break;
         case 'l': $args[$i] = (int)$args[$i]; break;
         case 'd': $args[$i] = (float)$args[$i]; break;
@@ -1521,11 +1569,23 @@ class Redis {
           if (($args[$i] !== self::BEFORE) AND ($args[$i] !== self::AFTER)) {
             trigger_error(
               "Argument $i to Redis::$fname must be 'before' or 'after'",
-              E_USER_ERROR);
+              E_ERROR);
             return null;
           } break;
       }
     }
+    if ($func['cmd'] == "LREM") {
+      //
+      // The PHP interface has arguments in one order:
+      //   https://github.com/nicolasff/phpredis#lrem-lremove
+      // But the server wants them in another:
+      //   http://redis.io/commands/lrem
+      // So just swap them prior to marshalling them out.
+      //
+      $tmp = $args[1];
+      $args[1] = $args[2];
+      $args[2] = $tmp;
+    }
     $this->processArrayCommand($func['cmd'], $args);
     if (empty($func['handler'])) {
       return null;
@@ -1541,9 +1601,7 @@ class Redis {
                                $persistent_id,
                                $retry_interval,
                                $persistent = false) {
-    if (!empty($persistent_id)) {
-      throw new RedisException("Named persistent connections not supported");
-    }
+
 
     if ($port <= 0) {
       if ((strlen($host) > 0) && ($host[0] == '/')) {
@@ -1560,9 +1618,18 @@ class Redis {
     }
 
     if ($persistent) {
-      $conn = pfsockopen($host, $port, $errno, $errstr, $timeout);
+      if (!empty($persistent_id)) {
+        $pid     = array('id' => array('persistent_id' => $persistent_id));
+        $context = stream_context_create($pid);
+        $sok     = $host;
+        if ($port > 0) $sok .= ':' . $port;
+        $conn    = stream_socket_client(
+          $sok, $errno, $errstr, $timeout, 2, $context);
+      } else {
+        $conn = pfsockopen($host, $port, $errno, $errstr, $timeout);
+      }
     } else {
-      $conn = fsockopen($host, $port, $errno, $errstr, $timeout);
+        $conn = fsockopen($host, $port, $errno, $errstr, $timeout);
     }
     $this->last_connect = time();
     $this->host = $host;
@@ -1570,20 +1637,21 @@ class Redis {
     $this->retry_interval = $retry_interval;
     $this->timeout_connect = $timeout;
     $this->persistent = $persistent;
+    $this->persistent_id = $persistent_id;
     $this->connection = $conn;
     $this->dbNumber = 0;
     $this->commands = [];
     $this->multiHandler = [];
     $this->mode = self::ATOMIC;
 
-    if (!$conn) {
+    if (!$this->connection) {
       trigger_error(
         "Failed connecting to redis server at {$host}: {$errstr}",
-        E_USER_WARNING);
+        E_WARNING);
       return false;
     }
-    stream_set_blocking($conn, true);
-    stream_set_timeout($conn, $this->timeout_seconds, $this->timeout_useconds);
+    stream_set_blocking($this->connection, true);
+    $this->setOption(Redis::OPT_READ_TIMEOUT, $timeout);
 
     return true;
   }
