@@ -1,5 +1,4 @@
 <?hh
-// Copyright 2004-present Facebook. All Rights Reserved.
 
 /* While the parameters are of type mixed, going to assume
    they are SplFileInfos for now. This will sort based on
@@ -16,13 +15,15 @@
    where the ending files are not necessarily in order, but the containing
    directories are in order.
 */
-class SortedIterator extends SplHeap {
-  public function __construct(Iterator $iterator) {
+class SortedIterator<T> extends SplHeap<T> {
+  public function __construct(Iterator<T> $iterator) {
+    parent::__construct();
     foreach ($iterator as $item) {
       $this->insert($item);
     }
   }
-  public function compare(mixed $b, mixed $a): int {
+
+  public function compare(T $b, T $a): int {
     assert($a instanceof SplFileInfo);
     assert($b instanceof SplFileInfo);
     return strcmp($a->getPath(), $b->getPath());
