@@ -2,8 +2,8 @@
 require_once __DIR__.'/../Framework.php';
 
 class FacebookPhpSdk extends Framework {
-  protected function install(): void {
-    parent::install();
+  <<Override>>
+  protected function extraPostComposer(): void {
     verbose("Creating a phpunit.xml for running the Facebook PHP SDK tests.\n");
     $phpunit_xml = <<<XML
 <phpunit bootstrap="./tests/bootstrap.php">
@@ -17,6 +17,7 @@ XML;
     file_put_contents($this->getTestPath()."/phpunit.xml", $phpunit_xml);
   }
 
+  <<Override>>
   protected function isInstalled(): bool {
     $extra_files = Set {
       $this->getTestPath()."/phpunit.xml",
