@@ -5,4 +5,4 @@ include __DIR__."/builtin_extensions.inc";
 class A_DOMAttr extends DOMAttr {
   public $___x;
 }
-test("DOMAttr");
+test("DOMAttr", "Hello");
