@@ -5,4 +5,4 @@ include __DIR__."/builtin_extensions.inc";
 class A_DOMEntityReference extends DOMEntityReference {
   public $___x;
 }
-test("DOMEntityReference");
+test("DOMEntityReference", "Foo");
