@@ -5,4 +5,4 @@ include __DIR__."/builtin_extensions.inc";
 class A_DOMProcessingInstruction extends DOMProcessingInstruction {
   public $___x;
 }
-test("DOMProcessingInstruction");
+test("DOMProcessingInstruction", "Foo");
