@@ -5,4 +5,4 @@ include __DIR__."/builtin_extensions.inc";
 class A_DOMXPath extends DOMXPath {
   public $___x;
 }
-test("DOMXPath");
+test("DOMXPath", new DOMNode);
