@@ -5,4 +5,4 @@ include __DIR__."/builtin_extensions.inc";
 class A_SoapClient extends SoapClient {
   public $___x;
 }
-test("SoapClient");
+test("SoapClient", __FILE__ . ".wsdl");
