@@ -5,14 +5,14 @@
 async function gen1($a) {
   error_log('In gen1');
   error_log('Finished in gen1');
-  await RescheduleWaitHandle::Create(1, 1); // simulate blocking I/O
+  await RescheduleWaitHandle::Create(0, 0); // simulate blocking I/O
   return $a + 1;
 }
 
 async function gen2($a) {
   error_log('In gen2');
-  await RescheduleWaitHandle::Create(1, 1); // simulate blocking I/O
-  $x = gen1($a)->join();
+  await RescheduleWaitHandle::Create(0, $a); // simulate blocking I/O
+  $x = HH\Asio\join(gen1($a));
   error_log('Finished in gen2');
   return $x;
 }
@@ -20,7 +20,7 @@ async function gen2($a) {
 async function genBar($a) {
   error_log('In genBar');
   var_dump($a);
-  await RescheduleWaitHandle::Create(1, 1); // simulate blocking I/O
+  await RescheduleWaitHandle::Create(0, $a); // simulate blocking I/O
   error_log('Finished in genBar');
   return $a + 2;
 }
@@ -43,7 +43,7 @@ async function genFoo($a) {
 }
 
 function main($a) {
-  $result = genFoo($a)->join();
+  $result = HH\Asio\join(genFoo($a));
   var_dump($result);
 }
 
