@@ -5,6 +5,15 @@ function f($a=500, $b) {
   var_dump(get_defined_vars());
 }
 
+function g() {
+  $usevar = 0;
+  $closure = function($arg1) use ($usevar) {
+    $local = 0;
+    var_dump(get_defined_vars());
+  };
+  $closure(1);
+}
+
 function modify($arr) {
   $arr['a'] += 200;
 }
@@ -19,4 +28,5 @@ function ref($a) {
 }
 
 @f();
+g();
 ref(20);
