@@ -5,16 +5,16 @@ type Point = shape(
   'y' => int,
 );
 
-type Recur = shape(
-  'shape' => Recur
-);
-
 type Point3 = shape(
   'x' => int,
   'y' => int,
   'z' => int,
 );
 
+type Optional = shape(
+  ?'optional_field' => ?int,
+);
+
 function foo(Point $x) : Point {
   $x['x'] = 42;
   return $x;
