@@ -1,6 +1,11 @@
 <?hh
 
-function block() { return RescheduleWaitHandle::create(1,1); };
+function block() {
+  return RescheduleWaitHandle::create(
+    RescheduleWaitHandle::QUEUE_NO_PENDING_IO,
+    1,
+  );
+};
 
 class F implements Awaitable {
   function __construct(){
@@ -25,4 +30,4 @@ async function foo() {
   var_dump($a);
 }
 
-foo()->join();
+HH\Asio\join(foo());
