@@ -1,6 +1,11 @@
 <?hh
 
-function block() { return RescheduleWaitHandle::create(1,1); };
+function block() {
+  return RescheduleWaitHandle::create(
+    RescheduleWaitHandle::QUEUE_NO_PENDING_IO,
+    1,
+  );
+};
 
 // closure in use param
 
@@ -13,8 +18,8 @@ $y = async function() use ($callback) {
   return await $callback();
 };
 
-var_dump($x($callback)->join());
-var_dump($y($callback)->join());
+var_dump(HH\Asio\join($x($callback)));
+var_dump(HH\Asio\join($y($callback)));
 
 // reference passing
 
@@ -25,12 +30,12 @@ $incB = async function () use ($env) { await block(); return ++$env; };
 $incref = async function () use (&$env) { return ++$env; };
 $increfB = async function () use (&$env) { await block(); return ++$env; };
 
-var_dump($inc()->join());
-var_dump($incB()->join());
+var_dump(HH\Asio\join($inc()));
+var_dump(HH\Asio\join($incB()));
 var_dump($env);
 
-var_dump($incref()->join());
+var_dump(HH\Asio\join($incref()));
 var_dump($env);
 
-var_dump($increfB()->join());
+var_dump(HH\Asio\join($increfB()));
 var_dump($env);
