@@ -10,6 +10,6 @@ class F {
 $async = new F;
 var_dump(F::async);
 var_dump($async::async);
-var_dump(F::foo()->join());
+var_dump(HH\Asio\join(F::foo()));
 
 
