@@ -11,13 +11,6 @@ $obj, $obj}
   $obj = new stdClass();
   $a = Map {
 'a' => $obj, 'b' => $obj}
-;
-  $a = unserialize(serialize($a));
-  $a['a']->prop = 22;
-  var_dump($a['b']->prop);
-  $obj = new stdClass();
-  $a = StableMap {
-'a' => $obj, 'b' => $obj}
 ;
   $a = unserialize(serialize($a));
   $a['a']->prop = 33;
