@@ -22,13 +22,6 @@ foreach ($mp->lazy()->map($mapFn)->filter($filtFn) as $k => $v) {
   var_dump($k, $v);
 }
 echo "------------------------\n";
-$smp = StableMap {
-'a' => 0, 'b' => 3, 'c' => 6, 'd' => 9}
-;
-foreach ($smp->lazy()->map($mapFn)->filter($filtFn) as $k => $v) {
-  var_dump($k, $v);
-}
-echo "------------------------\n";
 $pair = Pair {
 0, 3}
 ;
