@@ -1,5 +1,7 @@
 <?php
 $a = array_fill(5, 6, "banana");
 $b = array_fill(-2, 2, "pear");
+$c = array_fill(0, 0, "apple");
 var_dump($a);
 var_dump($b);
+var_dump($c);
