@@ -6,6 +6,7 @@ var_dump(array_slice($input, 2));
 var_dump(array_slice($input, 2, null));
 var_dump(array_slice($input, -2, 1));
 var_dump(array_slice($input, 0, 3));
+var_dump(array_slice($input, 0, 5));
 
 // note the differences in the array keys
 var_dump(array_slice($input, 2, -1));
@@ -26,3 +27,6 @@ var_dump(array_slice($a, 1, 2, false));
 var_dump(array_slice(array(123, "b", "c"), 1, 2, true));
 
 var_dump(array_slice(array(123, "b", "c"), 1, 2, false));
+
+var_dump(array_slice(array(12 => 1, 6 => 2), 0, 2, true));
+var_dump(array_slice(array(12 => 1, 6 => 2), 0, 2, false));
