@@ -14,6 +14,8 @@ function VUNPACK($fmt, $inp, $exp) {
 
 function test_unpack() {
   $iFF = str_repeat("\xFF", 4);
+  $le64_FF = "\xFF\x00\x00\x00\x00\x00\x00\x00";
+  $be64_FF = "\x00\x00\x00\x00\x00\x00\x00\xFF";
   $le32_FF = "\xFF\x00\x00\x00";
   $be32_FF = "\x00\x00\x00\xFF";
   $le16_FF = "\xFF\x00";
@@ -24,6 +26,16 @@ function test_unpack() {
 
   VUNPACK("i", $iFF, -1);
 
+  // QqJP test 64-bit ints specifically
+  VUNPACK("Q", $iFF.$iFF, -1);
+  VUNPACK("q", $iFF.$iFF, -1);
+
+  VUNPACK("J", $be64_FF, 0xFF);
+  VUNPACK("P", $le64_FF, 0xFF);
+  VUNPACK("P", $be64_FF, -72057594037927936);
+
+  VUNPACK("Q", $le64_FF, 0xFF);
+
   // LlNV test 32-bit ints specifically
   VUNPACK("L", $iFF, 0xFFFFFFFF);
   VUNPACK("l", $iFF, -1);
@@ -45,4 +57,9 @@ function test_unpack() {
   VUNPACK("S", $le16_FF, 0xFF);
 }
 
+function test_unpack_empty() {
+  var_dump(unpack("n*", ''));
+}
+
 test_unpack();
+test_unpack_empty();
