@@ -5,8 +5,8 @@ class Foo implements Serializable {
 }
 $f = new Foo(42);
 $fs = serialize($f);
-$non_existant_bar = preg_replace('/Foo/', 'Bar', $fs);
-var_dump( unserialize($non_existant_bar) );
+$non_existent_bar = preg_replace('/Foo/', 'Bar', $fs);
+var_dump( unserialize($non_existent_bar) );
 
 class Baz {}
 $non_serializable_baz = preg_replace('/Foo/', 'Baz', $fs);
