@@ -6,7 +6,8 @@
  */
 
 // Figure out what handler to use
-$host = getenv('PHP_CURL_HTTP_REMOTE_SERVER');
+include 'server.inc';
+$host = curl_cli_server_start();
 if(!empty($host)) {
 
     // Use the set Environment variable
@@ -32,4 +33,4 @@ curl_exec($ch);
 curl_close($ch);
 is_file($tempfile) and @unlink($tempfile);
 isset($tempname) and is_file($tempname) and @unlink($tempname);
-?>
\ No newline at end of file
+?>
