@@ -1,6 +1,7 @@
 <?php
 
-  $host = getenv('PHP_CURL_HTTP_REMOTE_SERVER');
+  include 'server.inc';
+  $host = curl_cli_server_start();
 
   echo '*** Test curl_copy_handle() after exec() with POST ***' . "\n";
 
@@ -23,4 +24,4 @@
 
   var_dump( $curl_content_copy );
 ?>
-===DONE===
\ No newline at end of file
+===DONE===
