@@ -1,5 +1,6 @@
 <?php
-  $host = getenv('PHP_CURL_HTTP_REMOTE_SERVER');
+  include 'server.inc';
+  $host = curl_cli_server_start();
 
   $url = "{$host}/get.php";
   $ch = curl_init($url);
@@ -12,4 +13,4 @@
   unset($ch);
   echo curl_exec($ch2);
 
-?>
\ No newline at end of file
+?>
