@@ -32,7 +32,8 @@ try {
 
 ?>
 ===DONE===
+<?php error_reporting(0); ?>
 <?php 
-unlink(dirname(__FILE__) . '/files/phar_oo_test.phar.php');
+unlink(dirname(__FILE__) . '/files/phar_oo_001.phar.php');
 __halt_compiler();
 ?>
\ No newline at end of file
