@@ -15,7 +15,8 @@ echo "\n";
 
 ?>
 ===DONE===
+<?php error_reporting(0); ?>
 <?php 
-unlink(dirname(__FILE__) . '/files/phar_oo_test.phar.php');
+unlink(dirname(__FILE__) . '/files/phar_oo_011.phar.php');
 __halt_compiler();
 ?>
\ No newline at end of file
