@@ -23,7 +23,8 @@ catch (BadMethodCallException $e)
 
 ?>
 ===DONE===
+<?php error_reporting(0); ?>
 <?php 
-unlink(dirname(__FILE__) . '/files/phar_oo_test.phar.php');
+unlink(dirname(__FILE__) . '/files/phar_oo_012b.phar.php');
 __halt_compiler();
 ?>
\ No newline at end of file
