@@ -37,7 +37,9 @@ $data = [
 ];
 var_dump($data);
 if (!socket_recvmsg($s, $data, 0)) die("recvmsg");
-print_r($data);?>
+print_r($data);
+?>
+<?php error_reporting(0); ?>
 <?php
 $path = __DIR__ . "/unix_sock";
 @unlink($path);
