@@ -1,5 +1,5 @@
 <?php
-$num = 100004313234244; // notice this exceeds 32 bits
+$num = PHP_INT_MAX; // 32 bits
 $conn = new sqlite3(':memory:');
 $conn->query('CREATE TABLE users (id INTEGER NOT NULL, num INTEGER NOT NULL, PRIMARY KEY(id))');
 
