@@ -1,8 +1,4 @@
 <?php
-ini_set('from', teste@teste.pt);
-
-ini_set('allow_url_fopen', 1);
-
 require 'server.inc';
 
 function do_test() {
@@ -36,4 +32,4 @@ ini_set('from', 'junk@junk.com');
 
 do_test();
 
-?>
\ No newline at end of file
+?>
