@@ -1,8 +1,4 @@
 <?php
-ini_set('allow_url_include', 1);
-
-ini_set('allow_url_fopen', 1);
-
 require 'server.inc';
 
 function do_test() {
@@ -34,3 +30,4 @@ function do_test() {
 }
 
 do_test();
+
