@@ -1,6 +1,4 @@
 <?php
-ini_set('sendmail_path', rubbish 2>/dev/null);
-
 /* Prototype  : int mail(string to, string subject, string message [, string additional_headers [, string additional_parameters]])
  * Description: Send an email message 
  * Source code: ext/standard/mail.c
@@ -10,9 +8,9 @@ ini_set('sendmail_path', rubbish 2>/dev/null);
 echo "*** Testing mail() : variation ***\n";
 
 // Initialise all required variables
-$to = 'user@company.com';
+$to = 'user@example.com';
 $subject = 'Test Subject';
 $message = 'A Message';
 var_dump( mail($to, $subject, $message) );
 ?>
-===DONE===
\ No newline at end of file
+===DONE===
