@@ -1,7 +1,4 @@
 <?php
-ini_set('default_socket_timeout', 2
-);
-
 
 $dir = 'ftp://your:self@localhost/';
 
