@@ -1,6 +1,4 @@
 <?php
-ini_set('open_basedir', .);
-
 require_once "open_basedir.inc";
 $initdir = getcwd();
 
@@ -12,6 +10,7 @@ var_dump(dir($initdir."/test/ok"));
 var_dump(dir($initdir."/test/ok/../ok"));
 
 test_open_basedir_after("dir");?>
+<?php error_reporting(0); ?>
 <?php
 require_once "open_basedir.inc";
 delete_directories();
