@@ -1,9 +1,8 @@
 <?php
-ini_set('open_basedir', .);
-
 require_once "open_basedir.inc";
 test_open_basedir_array("lstat");
 ?>
+<?php error_reporting(0); ?>
 <?php
 require_once "open_basedir.inc";
 delete_directories();
