@@ -1,6 +1,4 @@
 <?php
-ini_set('open_basedir', .);
-
 require_once "open_basedir.inc";
 $initdir = getcwd();
 test_open_basedir_before("opendir");
@@ -11,6 +9,7 @@ var_dump(opendir($initdir."/test/ok"));
 var_dump(opendir($initdir."/test/ok/../ok"));
 
 test_open_basedir_after("opendir");?>
+<?php error_reporting(0); ?>
 <?php
 require_once "open_basedir.inc";
 delete_directories();
