@@ -1,6 +1,4 @@
 <?php
-ini_set('open_basedir', .);
-
 require_once "open_basedir.inc";
 $initdir = getcwd();
 test_open_basedir_before("touch");
@@ -22,6 +20,7 @@ var_dump(touch("../ok/./ok.txt"));
 
 test_open_basedir_after("touch");
 ?>
+<?php error_reporting(0); ?>
 <?php
 require_once "open_basedir.inc";
 delete_directories();
