@@ -1,6 +1,4 @@
 <?php
-ini_set('open_basedir', .);
-
 require_once "open_basedir.inc";
 $initdir = getcwd();
 test_open_basedir_before("unlink");
@@ -13,6 +11,7 @@ var_dump(unlink($initdir."/test/bad/bad.txt"));
 
 test_open_basedir_after("unlink");
 ?>
+<?php error_reporting(0); ?>
 <?php
 require_once "open_basedir.inc";
 delete_directories();
